#!/usr/bin/env python

# test_vatstuff.py  - test the vatstuff program
#
# Version : 0.7.0
# Date : 20100902
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#

"Creates test databases and runs tests with unittest"

import unittest
import os
import os.path
import sys
import datetime

import vatstuff

from vatstuff_packages.vs_utilities import config
from vatstuff_packages.vs_utilities import objects
from vatstuff_packages.vs_utilities import odd_functions
from vatstuff_packages.vs_database import make_new
from vatstuff_packages.vs_database import create_objects

from vatstuff_packages.vs_tests import test_objects
from vatstuff_packages.vs_tests import test_odd_functions
from vatstuff_packages.vs_tests import test_accounts
from vatstuff_packages.vs_tests import test_sales
from vatstuff_packages.vs_tests import test_purchases
from vatstuff_packages.vs_tests import test_sample_databases

def setup_database(scriptdirectory):
    "Set up the default configuration, and a test database"

    # Set default values in the configuration dictionary vs_cfg 
    if not vatstuff.setup_defaults(scriptdirectory):
        return False

    # Use a test database name
    database_filepath=os.path.join(config.vs_cfg["DATABASE_FOLDER"], "testdata.vat")

    # set configuration dictionary to point to this filepath
    odd_functions.set_database_filepath(database_filepath)

    # Delete any old test database
    if os.path.exists(config.vs_cfg["DATABASE_FILEPATH"]):
        os.remove(config.vs_cfg["DATABASE_FILEPATH"])
    # Delete any old test lockfile
    if os.path.exists(config.vs_cfg["LOCK_FILEPATH"]):
        os.remove(config.vs_cfg["LOCK_FILEPATH"])
    # Set write access, should be True if no lockfile exists
    assert odd_functions.check_write_access()

    # Create a new test database
    make_new.new_database()

    # The database needs a company and VAT Scheme    

    # Create a registered address for the company
    address = create_objects.create_Address(line1 = "Test Place",
                                            line2 = "Test Region",
                                            city = "Test City",
                                            county = "Test County",
                                            postcode = "TEST 123",
                                            country = "England",
                                            reg_office = 1,
                                            delivery_point = 1,
                                            invoice_point = 1,
                                            order_point = 1,
                                            payment_point = 1,
                                            edit_new = False,
                                            visible = True,
                                            system = 1)


    # Create a company
    company = objects.Company()
    company.country_registration = "England"
    company.company_number = "12345678"
    company.name = "Test Company Ltd"
    company.registered_address = address

    today_is = datetime.date.today()
    starting_date = datetime.date(today_is.year, today_is.month, 1)

    # Create a VAT Scheme
    create_objects.create_VATScheme(vat_regnumber = "12345678",
                                    vat_country_code = "GB",
                                    start_date = starting_date)
    return True



# Create the unittest class

class TestDatabase(unittest.TestCase):
    """A test class for the initial database"""

    def test_01_python_version(self):
        "version of python"
        self.assertTrue(vatstuff.check_python())

    def test_02_database_exists(self):
        "database exists"
        self.assertTrue(os.path.exists(config.vs_cfg["DATABASE_FILEPATH"]))
        self.assertEqual(config.vs_cfg["DATABASE_FILE"], "testdata.vat")

    def test_03_check(self):
        "odd_functions.check"
        self.assertEqual(odd_functions.check(), (True, None))

    def test_04_database_contents(self):
        "database contents"
        company = objects.Company()
        self.assertEqual(company.country_registration, "England")
        self.assertEqual(company.company_number, "12345678")
        self.assertEqual(company.vat_regnumber, "12345678")
        self.assertEqual(company.name, "Test Company Ltd")
        self.assertEqual(company.vat_country_code, "GB")

        self.assertEqual(company.registered_address.lines[0], "Test Place")
        self.assertEqual(company.registered_address.lines[1], "Test Region")
        self.assertEqual(company.registered_address.lines[2], "Test City")
        self.assertEqual(company.registered_address.lines[3], "Test County")
        self.assertEqual(company.registered_address.lines[4], "TEST 123")
        self.assertEqual(company.registered_address.lines[5], "England")

        vat_scheme = objects.VATScheme()
        self.assertEqual(vat_scheme.method, "STANDARD")
        self.assertEqual(vat_scheme.period, "QUARTERLY")


if __name__ == "__main__":

    print "Creating database testdata.vat ..."

    # get the directory this script is in
    scriptdirectory=os.path.abspath(os.path.dirname(sys.argv[0]))
    # setup the test database
    if not setup_database(scriptdirectory):
        print "Failed to setup the test database"
        sys.exit(1)

    # create test suite from the class TestDatabase
    suite = unittest.TestLoader().loadTestsFromTestCase(TestDatabase)

    # add to suit by importing from other test modules
    suite.addTest(test_objects.suite())
    suite.addTest(test_odd_functions.suite())
    suite.addTest(test_accounts.suite())
    suite.addTest(test_sales.suite())
    suite.addTest(test_purchases.suite())

    # make this the last test - to create sample databases
    suite.addTest(test_sample_databases.suite())


    # and run the test suit
    unittest.TextTestRunner(verbosity=2).run(suite)
    # unittest.TextTestRunner().run(suite)

