#!/usr/bin/env python

####### VATStuff #######
#
# vatstuff.py  - starts the vatstuff program
#
# Version : 0.7.0
# Date : 20100910
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#


""" VATStuff
Provides double entry book keeping and VAT recording for small businesses.

Normal usage:

python vatstuff.py [options] [database-filepath]

vatstuff may also have a '.pyw' rather than a '.py' extension on a Windows
platform - or may be a .exe if the program has been bundled into an
executable.

The initial 'python' may not be required if vatstuff.py has been set to be
executable on a Linux platform, or if the extension '.pyw' has been
associated with python on a Windows platform - in which case the python
interpreter will be called automatically.

The options are:
--version         shows the VATStuff version number
-d or --debug     sets the most verbose logging
-n or --nologs    turn off logging completely
-h or --help      shows these options and exits


If vatstuff is run with the option '--debug' then detailed information
is logged.  Normally only information and error events are logged

If vatstuff is run with the option '--nologs' then no logs are recorded.

After the options, if a database filepath is given, vatstuff will
attempt to open that database. If no database file is given, the GUI
will ask the user for a file on startup (which is the normal case).

Having set the configuration, the gui library wx is imported and
vatstuff_packages.vs_gui.gui.maingui() is then called to start the gui.
"""

import os
import os.path
import sys

from optparse import OptionParser

from vatstuff_packages.vs_utilities import config


# This variable should be set to the filepath of the configuration file
# If it is set to None, the configuration file vatstuff.cfg
# will be searched for at a number of standard locations
# see vatstuff_packages/vs_utilities/config.py for details

_CONFIG_LOCATION = None

# For example:
# _CONFIG_LOCATION = "/etc/vatstuff/vatstuff.cfg"


def check_python():
    """Returns True if this version of Python is greater or equal
       to 2.5 and less than 3.0, returns False otherwise"""
    return sys.version_info[0] == 2 and sys.version_info[1] >= 5


def setup_defaults(scriptdirectory,
                   config_location =_CONFIG_LOCATION,
                   logging = "DEFAULT",
                   debug_mode = "DEFAULT"):
    """Sets the global defaults required by this program in the vs_cfg dictionary.
 
       scriptdirectory should be set to the installation directory of this script,
       config_location is the filepath to a configuration file used to set
       default locations, such as where image, and log files are kept.
       If it is None the configuration file will be looked for at a number
       of standard locations.
       logging and debug_mode are normally DEFAULT, but can also be ON or OFF
       Returns True on success, False on failure."""

    return config.set_defaults(scriptdirectory,
                               config_location,
                               logging,
                               debug_mode)

def get_config_value(key_string):
    "Returns a vs_cfg value given its key"
    # if setup_defaults() has not been called only the version info is available
    # this function may be useful if vatstuff.py is imported - perhaps for testing
    return config.vs_cfg.get(key_string)


def interactive(database_filepath):
    """This function is used for testing from the python interactive interpreter.

       If the interpreter is started in the same directory as vatstuff.py
       the user can import vatstuff, then run this function with a database file.
       From then on, anything that doesn't need the gui should be available and
       will operate on the database - however without error checking !!!
       so this function is strictly for developers, and on test databases only"""
    # check python version
    assert check_python()
    # scriptdirectory is assumed to be os.getcwd(), which is
    # correct if vatsuff is being run from the python interactive interpreter,
    # started in the same directory as vatstuff.py
    scriptdirectory = os.getcwd()
    # set up global defaults
    assert setup_defaults(scriptdirectory)
    # set database file
    assert os.path.isfile(database_filepath)
    import vs_utilities.odd_functions
    vs_utilities.odd_functions.set_database_filepath(database_filepath)
    # test the database
    status, message = vs_utilities.odd_functions.check()
    assert status, message
    # Check we have write access
    assert get_config_value("WRITE_ACCESS")
    # However note - this interactive session does not create a lockfile
    print("VATStuff ready with database %s" % database_filepath)


# This version tuple is available should vatstuff.py be imported as a module
vs_version = (get_config_value("PROGRAM_VERSION"),
              get_config_value("PROGRAM_ID"),
              get_config_value("RELEASE_DATE"))


if __name__ == "__main__":

    # Exit codes used here are zero for normal usage, and 1, 2, 3 etc arbitrarily chosen
    # for different fault conditions.

    # Check the python version
    if not check_python():
        print("Sorry, your python version is not compatable")
        print("This program requires python 2.5, 2.6 or 2.7")
        print("Program exiting")
        sys.exit(1)

    usage = "usage: %prog [options] <database-filepath>"
    vers = "%s %s" % (vs_version[1], vs_version[0])

    parser = OptionParser(usage=usage, version=vers)

    parser.add_option("-d", "--debug", action="store_true", dest="debug", default=False,
                      help="sets the most verbose logging")

    parser.add_option("-n", "--nologs", action="store_true", dest="nologs", default=False,
                      help="turn off logging completely")

    (options, args) = parser.parse_args()
    if options.debug and options.nologs:
        parser.error("Options -d and -n are mutually exclusive")

    if len(args) > 1:
        parser.error("Incorrect number of arguments, either no argument, or the path of a database file is required")

    database_filepath = None
    if args:
        database_filepath = args[0]
        if not os.path.isfile(database_filepath):
            parser.error("The database at the given filepath does not exist")

    debug_mode = "DEFAULT"
    logging = "DEFAULT"
    if options.debug:
        debug_mode = "ON"
    if options.nologs:
        logging = "OFF"

    # get the directory this script is in
    scriptdirectory=os.path.abspath(os.path.dirname(sys.argv[0]))

    # Set up variables into the vs_cfg dictionary
    if not setup_defaults(scriptdirectory, logging = logging, debug_mode = debug_mode):
        print("Sorry, unable to set the program defaults")
        print("Program exiting")
        sys.exit(3)

    # Log the program has started
    config.info_log("%s started" % vers)

    # And continue to the gui part of the program
    try:
        import wx
    except Exception, e:
        config.exception_log(e)
        print("Failed to import wx")
        print("This program needs the wxPython library (version 2.8 or later).")
        print("Program exiting")
        sys.exit(4)

    wx_version = wx.version().split()[0]
    wx_version_digits = wx_version.split(".")
    if int(wx_version_digits[0])<2 or (int(wx_version_digits[0])==2 and int(wx_version_digits[1])<8):
        config.info_log("Invalid wx.version %s" % wx_version )
        print("Sorry, your wxPython version is %s\nthe minimum required is 2.8" % wx_version)
        print("Program exiting")
        sys.exit(5)

    # Now run the program gui
    try:
        from vatstuff_packages.vs_gui.gui import maingui
    except Exception, e:
        config.exception_log(e)
        print("Sorry, an error occurred trying to import vatstuff_packages.vs_gui.gui")
        print str(e)
        print("Program exiting")
        sys.exit(6)

    # maingui() starts the gui
    maingui(database_filepath)

    # Normal exit with a zero
    sys.exit(0)


