####### VATStuff #######
#
# change_transaction_date.py  - Check a proposed transaction date change, and implement it
#
# Version : 0.1
# Date : 20091121
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#

"""Check a proposed transaction date change, and implement it"""

import decimal

from vatstuff_packages.vs_utilities import *



D = decimal.Decimal   # D used to avoid typing


def check_date(transaction, new_date):
    """Returns status, message where status is True or False depending on if ok to
       change the date, and message is a confirmation request text, or an error message"""
    transaction_id      = transaction.id
    transaction_system  = transaction.system

    if not vs_cfg["WRITE_ACCESS"]:
        # Another process has locked the database
        return False, "Another process has locked this database"

    if transaction_system == 12:
        # System 12 - date of delete old data transaction cannot be changed
        return False, "This transaction date cannot be changed"

    if vs_cfg["DELETE_TO_DATE"]:
        if new_date < vs_cfg["DELETE_TO_DATE"]:
            return False, "A date prior to the last database deletion date cannot be accepted"

    if new_date == transaction.datestamp:
        return False, "The transaction already has that date"

    if (transaction_system == 0) or (transaction_system == 1):
        # This is a 'move money' or 'openning balance' transaction
        return True, "Transaction date change: do you wish to proceed?"

    if transaction_system == 2:
        # This is a purchase order transaction, it cannot be set to a date after
        # any of its associated invoices, or after the date it was closed
        order = transaction.get_object()
        invoice_list = order.get_invoices()
        if invoice_list:
            if new_date > invoice_list[0].datestamp:
               return False, "The Order date cannot be set after the first invoice date"
        po_closed_date = order.closed_date
        if po_closed_date:
            if new_date > po_closed_date:
               return False, "The Order date cannot be set after the order closed date"
        return True, "Change purchase order date: do you wish to proceed?"

    if transaction_system == 8:
        # This is a close order transaction
        # it cannot be set before the order date, or before any
        # associated invoice, payment or creditnote
        closedorder = transaction.get_object()
        order = closedorder.purchaseorder
        payments_list = order.get_payments()
        if payments_list:
            if new_date < payments_list[-1].datestamp:
                return False, "The order cannot be closed before a received payment"
        creditnotes_list = order.get_creditnotes()
        if creditnotes_list:
            if new_date < creditnotes_list[-1].datestamp:
                return False, "The order cannot be closed before a received credit note"
        invoices_list = order.get_invoices()
        if invoices_list:
            if new_date < invoices_list[-1].datestamp:
                return False, "The order cannot be closed before a received invoice"
        if new_date < order.datestamp:
            return False, "The order cannot be closed before it is created"
        return True, "Change the order closed date: do you wish to proceed?"

    if transaction_system == 10:
        # This is a VAT return submission transaction
        # It cannot be set to a date before the end of the period it was submitted for
        vatreturn = transaction.get_object()
        if new_date <= vatreturn.to_date:
            return False, "The VAT Return date cannot be before the period it covers"
        return True, "Change the VAT Return submission date: do you wish to proceed?"

    if transaction_system == 11:
        # This is a VAT correction transaction
        return True, "Change the VAT correction date: do you wish to proceed?"

    # remaining transaction dates cannot be changed if they are prior to the
    # current vat start date

    vat_scheme = VATScheme()
    vat_period_start_date = vat_scheme.start_date
    if vat_period_start_date:
        if transaction.datestamp < vat_period_start_date:
            return False, "Transaction locked after VAT return:\nthe date cannot be changed"
        if new_date < vat_period_start_date:
            return False, "A date prior to the current VAT period cannot be accepted"

    if transaction_system == 3:
        # This is a payment to a supplier, it cannot be set to a date
        # before the suppliers invoice
        # If the purchase order has a close/cancel transaction, the payment cannot
        # be set after that 
        makepayment = transaction.get_object()
        invoices = makepayment.get_invoices()
        if new_date < invoices[0].datestamp:
            return False, "The payment date cannot be\nchanged to before the invoice date"
        order_closed = makepayment.purchaseorder.closed_date
        if order_closed:
            if new_date > order_closed:
                return False, "The payment date cannot be changed\nto after the purchase order is closed"
        return True, "Change date of payment to supplier:\ndo you wish to proceed?"

    if transaction_system == 4:
        # This is a suppliers credit note
        # It cannot be set to a date
        # before the suppliers invoice if it is linked to an invoice
        # (creditnotes received with payment are not)
        # If it is linked to a PO: If the purchase order has a close/cancel transaction,
        # the creditnote cannot be set after that
        suppliercreditnote = transaction.get_object()
        if suppliercreditnote.vat_return:
            return False, "The credit note date cannot be changed\nto after it has been applied to a VAT return."
        if suppliercreditnote.status == 1:
            return True, "Change date of supplier creditnote and\nreceived payment: do you wish to proceed?"
        if new_date < suppliercreditnote.supplierinvoice.datestamp:
            return False, "The creditnote date cannot\nbe set before the invoice date"
        order_closed = suppliercreditnote.purchaseorder.closed_date
        if order_closed:
            if new_date > order_closed:
                return False, "The creditnote date cannot be\nchanged to after the purchase order was closed"
        return True, "Change date of supplier creditnote:\ndo you wish to proceed?"

    if transaction_system == 5:
        # This is an invoice to  customer
        # If it is linked to a VAT return, it cannot be changed
        # it cannot be set to a date
        # after a received payment
        # or after a creditnote has been sent
        invoice = transaction.get_object()
        if invoice.vat_return:
            return False, "The invoice date cannot be changed\nafter it has been applied to a VAT return."
        # payments received
        payments = invoice.get_payments()
        if payments:
            first_payment = payments[0]
            if new_date > first_payment.datestamp:
                return False, "The Invoice date cannot be\nchanged to after the first payment date"
        # creditnotes sent
        creditnotes = invoice.get_creditnotes()
        if creditnotes:
            if new_date > creditnotes[0].datestamp:
                return False, "The Invoice date cannot be\nchanged to after a sent credit note"
        return True, "Change date of invoice:\ndo you wish to proceed?"

    if transaction_system == 6:
        # This is a payment from a customer
        # it cannot be set to a date
        # before an associated invoice (the payment may be for several invoices)
        customer_payment = transaction.get_object()
        invoices = customer_payment.get_invoices()
        if invoices:
            first_invoice = invoices[0]
            if new_date < first_invoice.datestamp:
                return False, "The payment date cannot be\nchanged to before an associated invoice"
        return True, "Change date of customer payment:\ndo you wish to proceed?"

    if transaction_system == 7:
        # This is a creditnote you have issued to a customer
        # If it is linked to a VAT return, it cannot be changed
        # it cannot be set to a date
        # before an associated invoice (if it is associated with an invoice - it may be a payment note)
        creditnote = transaction.get_object()
        if creditnote.vat_return:
            return False, "The credit note date cannot be changed\nafter it has been applied to a VAT return."
        invoice = creditnote.invoice
        if invoice:
            if new_date < invoice.datestamp:
                return False, "The credit note date cannot be\nset before the associated invoice date"
        return True, "Change date of issued credit note:\ndo you wish to proceed?"

    if transaction_system == 9:
        # This is a suppliers invoice received transaction,
        # If it is linked to a VAT return, it cannot be changed
        # It cannot be set to a date
        # before its associated purchase order,
        # or after a payment has been sent
        # or after a received creditnote
        # or after the order has closed
        supplierinvoice = transaction.get_object()
        if supplierinvoice.vat_return:
            return False, "The invoice date cannot be changed\nafter it has been applied to a VAT return."
        order = supplierinvoice.purchaseorder
        if new_date < order.datestamp:
            return False, "The Supplier Invoice date cannot be\nset before the purchase order date"
        # payments sent
        payments = supplierinvoice.get_payments_with_costvat()
        if payments:
            first_makepayment = payments[0]["PAYMENT"]
            if new_date > first_makepayment.datestamp:
                return False, "The Supplier Invoice date cannot\nbe set after the payment date"
        # received creditnotes
        creditnotes = supplierinvoice.get_creditnotes()
        if creditnotes:
            if new_date > creditnotes[0].datestamp:
                return False, "The Supplier Invoice date cannot be changed\nto after a received credit note"
        order_closed = order.closed_date
        if order_closed:
            if new_date > order_closed:
                return False, "The invoice date cannot be changed\nto after the purchase order was closed"
        return True, "Change supplier invoice date:\ndo you wish to proceed?"

    return False, "This transaction date cannot be changed" 


def change_date(transaction, new_date):
    """Set the given transaction with the new date, and return status

       on success: status is True
       on failure: status is False"""

    transaction_system  = transaction.system

    # Get all properties, to save them being accessed from the database every time they are needed
    t_dict = {"transaction" : transaction,
              "id"          : transaction.id,
              "system"      : transaction_system,
              "datestamp"   : transaction.datestamp,
              "gridrow"     : transaction.gridrow,
              "trans_list"  : transaction.trans_list  }

    # check date is ok - note this may have already been done by the gui
    # but there's no harm doing it twice!
    status, message = check_date(transaction, new_date)
    if not status:
        return False

    conn = None
    try:
        conn, cursor = vs_connect()

        # Change the transaction date
        _transaction_change_date(cursor, t_dict, new_date)

        # Some date changes require a gridrow change

        if transaction_system == 2:
            # This is a purchase order transaction
            _order_change_gridrow(cursor, t_dict, new_date)

        if transaction_system == 5:
            # This is an invoice to a customer
            _invoice_change_gridrow(cursor, t_dict, new_date)

        if transaction_system == 7:
            # This is a creditnote you have issued to a customer
            _creditnote_change_gridrow(cursor, t_dict, new_date)

        conn.commit()
    finally:
        if conn:
            conn.close()

    from_date = t_dict["datestamp"].isoformat()
    to_date = new_date.isoformat()
    info_log("Transaction %s date changed from %s to %s" % (t_dict["id"], from_date, to_date))

    return True


def _transaction_change_date(cursor, t_dict, new_date):
    "Change the date of the transaction and update gridrow"
    # create transaction_id as its easier to read
    transaction_id = t_dict["id"]

    # If there is no change in date, just return
    if new_date == t_dict["datestamp"]:
        return

    # Get new gridrow
    cursor.execute("SELECT COUNT(*) FROM TRANSACTIONS WHERE DATESTAMP < ? AND TRANSACTION_ID <> ?",
                  (new_date, transaction_id))
    result=cursor.fetchone()
    earlier_items = result[0]

    cursor.execute("SELECT COUNT(*) FROM TRANSACTIONS WHERE DATESTAMP = ? AND TRANSACTION_ID < ?",
                  (new_date, transaction_id))
    result=cursor.fetchone()
    new_gridrow = result[0] + earlier_items

    if new_gridrow != t_dict["gridrow"]:
        # For every gridrow greater than the current one, reduce value by one
        cursor.execute("UPDATE TRANSACTIONS SET GRIDROW = GRIDROW-1 WHERE GRIDROW > ?", (t_dict["gridrow"],))
        # Give the transaction its new gridrow and date
        cursor.execute("""UPDATE TRANSACTIONS SET GRIDROW = ?, DATESTAMP = ?
                          WHERE TRANSACTION_ID = ?""", (new_gridrow, new_date, transaction_id))
        # For every gridrow greater or equal to this new value, increase by one
        cursor.execute("""UPDATE TRANSACTIONS SET GRIDROW = GRIDROW+1
                          WHERE GRIDROW >= ?
                          AND TRANSACTION_ID <> ?""", (new_gridrow, transaction_id))
    else:
        # gridrow is not changing, but the date has, so change the date
        cursor.execute("""UPDATE TRANSACTIONS SET DATESTAMP = ?
                          WHERE TRANSACTION_ID = ?""", (new_date, transaction_id))

    # For each ledger in trans_list, change the gridrow and balance
    ledger_list = []
    for trans in t_dict["trans_list"]:
        account = trans[0]
        ledger_table_name = "LEDGER_%i" % account.id
        # Has this ledger already been done?
        if ledger_table_name in ledger_list:
            continue
        else:
            ledger_list.append(ledger_table_name)
        # Get the ledger_id for each row in this ledger - note there may be multiple entries
        # in this ledger with this transaction id, we have to sort them all
        sql_string1 = "SELECT LEDGER_ID FROM %s WHERE TRANSACTION_ID = ?" % ledger_table_name
        cursor.execute(sql_string1, (transaction_id,))
        result = cursor.fetchall()
        # get a list of ledger id's (not a tuple of tuples!)
        ledger_id_list = [ item[0] for item in result ]
        if not ledger_id_list:
            continue
        # The sorting mechanism requires the id list to be sorted in ascending
        # order for changing to an earlier date, (so the first id is done first)
        # or descending order for a later date, (so the last id is done first)
        # - don't ask why!!  Its due to when multiple id's are in one transaction
        # if not done in order one change can affect the position of previous changes
        ledger_id_list.sort()
        if new_date > t_dict["datestamp"]:
            ledger_id_list.reverse()
        # keep a note of the minimum gridrow affected by these changes
        # inititally set this at the max value
        sql_string = "SELECT MAX(GRIDROW) FROM %s" % ledger_table_name
        cursor.execute(sql_string)
        result = cursor.fetchone()
        min_gridrow = result[0]
        # if this maximum value is 0, there is only one entry, which will not change
        if not min_gridrow:
            continue
        for ledger_id in ledger_id_list:
            # get the gridrow of this entry
            sql_string = "SELECT GRIDROW FROM %s WHERE LEDGER_ID = ?" % ledger_table_name
            cursor.execute(sql_string, (ledger_id,))
            result = cursor.fetchone()
            gridrow = result[0]
            # Get new gridrow
            sql_string = """SELECT COUNT(*) FROM %s, TRANSACTIONS
                            WHERE DATESTAMP < ?
               AND %s.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID""" % (ledger_table_name, ledger_table_name)
            cursor.execute(sql_string, (new_date,))
            result=cursor.fetchone()
            earlier_items = result[0]
            sql_string = """SELECT COUNT(*) FROM %s, TRANSACTIONS
                            WHERE DATESTAMP = ?
                            AND LEDGER_ID < ?
               AND %s.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID""" % (ledger_table_name, ledger_table_name)
            cursor.execute(sql_string, (new_date, ledger_id))
            result=cursor.fetchone()
            new_gridrow = result[0] + earlier_items
            # Do the gridrow changes
            if new_gridrow != gridrow:
                # For every gridrow greater than the current one, reduce value by one
                sql_string = "UPDATE %s SET GRIDROW = GRIDROW-1 WHERE GRIDROW > ?" % ledger_table_name
                cursor.execute(sql_string, (gridrow,))
                # Give the ledger entry its new gridrow
                sql_string = "UPDATE %s SET GRIDROW = ? WHERE LEDGER_ID = ?" % ledger_table_name
                cursor.execute(sql_string, (new_gridrow, ledger_id))
                # For every gridrow greater or equal to this new value, increase by one
                sql_string = """UPDATE %s SET GRIDROW = GRIDROW+1
                                  WHERE GRIDROW >= ?
                                  AND LEDGER_ID <> ?""" % ledger_table_name
                cursor.execute(sql_string, (new_gridrow, ledger_id))
                if gridrow <= min_gridrow:
                    min_gridrow = gridrow - 1
                if new_gridrow <= min_gridrow:
                    min_gridrow = new_gridrow - 1

        # All this ledger gridrows updated, but now balance is all wrong
        # min_gridrow is the minimum gridrow where the balance is still ok
        # Get that last correct balance
        if min_gridrow < 0:
            balance = D("0.00")
        else:
            sql_string = "SELECT BALANCE FROM %s WHERE GRIDROW = ?" % ledger_table_name
            cursor.execute(sql_string, (min_gridrow,))
            result = cursor.fetchone()
            balance = D(result[0])

        # so update the balance from that row upwards
        sql_string1 = """SELECT DEBIT, CREDIT, LEDGER_ID
                             FROM %s
                             WHERE GRIDROW > ?
                             ORDER BY GRIDROW""" % ledger_table_name

        sql_string2 = "UPDATE %s SET BALANCE = ? WHERE LEDGER_ID = ?" % ledger_table_name

        cursor.execute(sql_string1, (min_gridrow,))
        for ledger_entry in cursor.fetchall():
                # ledger_entry[0] is the debit
                # ledger_entry[1] is the credit
                # ledger_entry[2] is the ledger_id

                balance = balance + D(ledger_entry[0]) - D(ledger_entry[1])
                # insert balance
                cursor.execute(sql_string2, (str(balance), ledger_entry[2]))


def _order_change_gridrow(cursor, t_dict, new_date):
    "Change gridrow of the purchase order"

    # If there is no change in date, just return
    if new_date == t_dict["datestamp"]:
        return

    order = t_dict["transaction"].get_object()
    order_id = order.id
    gridrow = order.gridrow

    # Get new gridrow
    cursor.execute("""SELECT COUNT(*) FROM PURCHASEORDERS, TRANSACTIONS
                      WHERE DATESTAMP < ?
                      AND PURCHASEORDER_ID <> ?
                      AND PURCHASEORDERS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID""", (new_date, order_id))
    result=cursor.fetchone()
    earlier_items = result[0]
    cursor.execute("""SELECT COUNT(*) FROM PURCHASEORDERS, TRANSACTIONS
                      WHERE DATESTAMP = ?
                      AND PURCHASEORDER_ID < ?
                      AND PURCHASEORDERS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID""", (new_date, order_id))
    result=cursor.fetchone()
    new_gridrow = result[0] + earlier_items

    if new_gridrow != gridrow:
        # For every gridrow greater than the current one, reduce value by one
        cursor.execute("UPDATE PURCHASEORDERS SET GRIDROW = GRIDROW-1 WHERE GRIDROW > ?", (gridrow,))
        # Give the purchase order its new gridrow
        cursor.execute("""UPDATE PURCHASEORDERS SET GRIDROW = ?
                          WHERE PURCHASEORDER_ID = ?""", (new_gridrow, order_id))
        # For every gridrow greater or equal to this new value, increase by one
        cursor.execute("""UPDATE PURCHASEORDERS SET GRIDROW = GRIDROW+1
                          WHERE GRIDROW >= ?
                          AND PURCHASEORDER_ID <> ?""", (new_gridrow, order_id))


def _invoice_change_gridrow(cursor, t_dict, new_date):
    "Change gridrow of the invoice"

    # If there is no change in date, just return
    if new_date == t_dict["datestamp"]:
        return

    invoice = t_dict["transaction"].get_object()
    invoice_id = invoice.id
    gridrow = invoice.gridrow

    # The taxpoint date must be earlier or equal to the invoice issue date
    if invoice.date_of_taxpoint > new_date:
        cursor.execute("UPDATE INVOICES SET DATE_OF_TAXPOINT = ? WHERE INVOICE_ID = ?", (new_date, invoice_id))

    # Get new gridrow
    cursor.execute("""SELECT COUNT(*) FROM INVOICES, TRANSACTIONS
                      WHERE DATESTAMP < ?
                      AND INVOICE_ID <> ?
                      AND INVOICES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID""", (new_date, invoice_id))
    result=cursor.fetchone()
    earlier_items = result[0]
    cursor.execute("""SELECT COUNT(*) FROM INVOICES, TRANSACTIONS
                      WHERE DATESTAMP = ?
                      AND INVOICE_ID < ?
                      AND INVOICES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID""", (new_date, invoice_id))
    result=cursor.fetchone()
    new_gridrow = result[0] + earlier_items

    if new_gridrow != gridrow:
        # For every gridrow greater than the current one, reduce value by one
        cursor.execute("UPDATE INVOICES SET GRIDROW = GRIDROW-1 WHERE GRIDROW > ?", (gridrow,))
        # Give the invoice its new gridrow
        cursor.execute("""UPDATE INVOICES SET GRIDROW = ?
                          WHERE INVOICE_ID = ?""", (new_gridrow, invoice_id))
        # For every gridrow greater or equal to this new value, increase by one
        cursor.execute("""UPDATE INVOICES SET GRIDROW = GRIDROW+1
                          WHERE GRIDROW >= ?
                          AND INVOICE_ID <> ?""", (new_gridrow, invoice_id))


def _creditnote_change_gridrow(cursor, t_dict, new_date):
    "Change gridrow of the creditnote"

    # If there is no change in date, just return
    if new_date == t_dict["datestamp"]:
        return

    creditnote = t_dict["transaction"].get_object()
    creditnote_id = creditnote.id
    gridrow = creditnote.gridrow

    # Get new gridrow
    cursor.execute("""SELECT COUNT(*) FROM CREDITNOTES, TRANSACTIONS
                      WHERE DATESTAMP < ?
                      AND CREDITNOTE_ID <> ?
                      AND CREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID""", (new_date, creditnote_id))
    result=cursor.fetchone()
    earlier_items = result[0]
    cursor.execute("""SELECT COUNT(*) FROM CREDITNOTES, TRANSACTIONS
                      WHERE DATESTAMP = ?
                      AND CREDITNOTE_ID < ?
                      AND CREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID""", (new_date, creditnote_id))
    result=cursor.fetchone()
    new_gridrow = result[0] + earlier_items

    if new_gridrow != gridrow:
        # For every gridrow greater than the current one, reduce value by one
        cursor.execute("UPDATE CREDITNOTES SET GRIDROW = GRIDROW-1 WHERE GRIDROW > ?", (gridrow,))
        # Give the creditnote its new gridrow
        cursor.execute("""UPDATE CREDITNOTES SET GRIDROW = ?
                          WHERE CREDITNOTE_ID = ?""", (new_gridrow, creditnote_id))
        # For every gridrow greater or equal to this new value, increase by one
        cursor.execute("""UPDATE CREDITNOTES SET GRIDROW = GRIDROW+1
                          WHERE GRIDROW >= ?
                          AND CREDITNOTE_ID <> ?""", (new_gridrow, creditnote_id))

