####### VATStuff #######
#
# create_objects.py  - functions to create database objects
#
# Version : 0.1
# Date : 20090128
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#


"Functions to create database objects"

# Do not import wx - so these database functions can be used by the testing
# process without the graphical wx being present

import decimal

from vatstuff_packages.vs_utilities import *




D = decimal.Decimal

def create_Folder(name, description = "", system = 0, expanded = 0):
    """Create a new folder on the database and return a Folder object, return None on failure

       system flag
       0 - This folder has no accounts, and can be deleted
       1 - This folder does have accounts, cannot be deleted
       2 - System folder - holds system accounts

       expanded flag
       0 - show as a closed folder on the gui
       1 - show as an open folder on the gui"""

    # A name must be given
    if not name:
        return None

    new_folder=(None, name, description, system, expanded)

    # insert the new folder
    folder_id = insert("""INSERT INTO FOLDERS 
                          (FOLDER_ID, NAME, DESCRIPTION, SYSTEM, EXPANDED) VALUES (?, ?, ?, ?, ?)""",
                          new_folder)
    info_log("New folder created, id: %s" % folder_id)
    return Folder(folder_id)


def create_AccountType(name, description = None):
    "Create a new account type on the database, return None on failure"
    # A name must be given, check it doesn't already exist, if it does, return None
    if not name:
        return None
    number = select("SELECT COUNT(*) FROM ACCOUNTTYPES WHERE NAME = ?", (name,))
    if number:
        # number is not equal to zero, therefore this name is used
        return None

    # insert the new AccountType
    accounttype_id = insert("""INSERT INTO ACCOUNTTYPES
                          (ACCOUNTTYPE_ID, NAME, DESCRIPTION) VALUES (?, ?, ?)""",(None, name, description))
    return AccountType(accounttype_id)


def create_Account(folder,                # The Folder object holding this account
                   name,                  # The name of the account
                   description = None,
                   system = 0,
                   acc_type = None):
    """Create a new account and ledger on the database and return an Account object, return None on failure"""

    if not folder:
        return None

    if not name:
        return None

    if not acc_type:
        return None

    # Note the "0.00" is the balance, always zero for a new account             
    new_account=(None, folder.id, name, description, "0.00", system, acc_type.id)
        
    # System flags
    #
    # 0 - If this account is new, and can have an openning balance put in via the edit window
    # 1 - Set to 1 after the first transaction, the openning balance option is no longer shown
    #
    # All numbers above 1 are for system accounts, and identify the usage of the account
    #
    # 2 - System account - the openning balance account
    # 3 - Customer accounts
    # 4 - Supplier accounts
    # 5 - VAT balance account
    # 6 - Sales account
    # 7 - Purchases account
    # 8 - VAT income from purchases (FLAT Scheme)
    # 9 - Input VAT not claimed back

    # Create an account
    # as a ledger needs to be created, do this all as a single transaction
    # to ensure it is committed together

    # Make the connection
    conn = None
    try:
        conn, c = vs_connect()

        # insert the account
        c.execute("""INSERT INTO ACCOUNTS 
                         (ACCOUNT_ID,
                          FOLDER_ID,
                          NAME,
                          DESCRIPTION,
                          BALANCE,
                          SYSTEM,
                          ACCOUNTTYPE_ID) VALUES (?, ?, ?, ?, ?, ?, ?)""", new_account)

        c.execute("SELECT last_insert_rowid()")
        result     = c.fetchone()
        account_id    = result[0]

        if not account_id:
            return None
        ledger = "LEDGER_%i" % account_id
        
        # create a ledger table
        # Create a separate string for the sql query, since it
        # seems ? parameters are not allowed here
      
        sql_string = """CREATE TABLE %s
                             (
                             LEDGER_ID INTEGER PRIMARY KEY,
                             TRANSACTION_ID INTEGER REFERENCES TRANSACTIONS(TRANSACTION_ID),
                             GRIDROW INTEGER NOT NULL DEFAULT -1,
                             DEBIT TEXT,
                             CREDIT TEXT,
                             BALANCE TEXT)""" % ledger

        c.execute(sql_string)
               
        conn.commit()
    finally:
        if conn: 
            conn.close()
    info_log("New account created, id: %s" % account_id)
    return Account(account_id)


def create_Label(name = "", description = "", button = None):
    """Create a new label on the database and return a Label object, return None on failure"""

    # If button is None, get the max button value and increment by one
    if button == None:
        max_button = select("SELECT MAX(BUTTON) FROM LABELS")
        button = max_button+1
        
    # Create a new label
    new_label=(None, name, description, button)

    label_id = insert("""INSERT INTO LABELS 
                         (LABEL_ID, NAME, DESCRIPTION, BUTTON) VALUES (?, ?, ?, ?)""",
                         new_label)
    return Label(label_id)


def create_Definition(label,
                      account = None,
                      action = 0,
                      value = "0.00",
                      operator = 0,
                      operand = "0.00"):

    """Create a new definition on the database and return a Definition object, return None on failure"""
        
    # action is an integer 0 = debit, 1 = credit
    # value is a string - as well as a number it could be something like 'Result1'
    # operator is an integer, 0 = None, 1 = +, 2 = -, 3 = x, 4 = /
    # operand is a string

    if not label:
        return None

    # Create a new definition, note account.id could be None
    if account == None:
        new_definition=(None, label.id, None, action, value, operator, operand)
    else:
        new_definition=(None, label.id, account.id, action, value, operator, operand)

    # insert the new definition

    definition_id = insert("""INSERT INTO DEFINITIONS 
                            (DEFINITION_ID,
                             LABEL_ID,
                             ACCOUNT_ID,
                             ACTION,
                             VALUE,
                             OPERATOR,
                             OPERAND) VALUES (?, ?, ?, ?, ?, ?, ?)""",
                             new_definition)

    return Definition(definition_id)


def create_VATRate(name = "", symbol = "", rate = D("0.0"), description = ""):
    """Create a new VAT Rate on the database and return a VATRate object, return None on failure"""

    if not symbol:
        return None

    # Note the rate is passed to and from this object as a Decimal value, however
    # it is stored on the database as a text string

    # Create a new rate
    new_vat_rate=(None, name, description, symbol, str(rate))

    # insert the new vat rate
    vat_rate_id = insert("""INSERT INTO VAT_RATES 
                            (VAT_RATE_ID, NAME, DESCRIPTION, SYMBOL, RATE) VALUES (?, ?, ?, ?, ?)""",
                            new_vat_rate)

    return VATRate(vat_rate_id)



def create_VATScheme(vat_regnumber, vat_country_code, start_date, method = "STANDARD", period = "QUARTERLY",
                     percent="10.0"):
    """Create a new VATScheme on the database and update the vat_regnumber and
       vat_country_code in the Company. return a VATScheme object, return None on failure"""

    if not vs_cfg["WRITE_ACCESS"]:
        # Another process has locked the database
        raise VS_ANOTHER_PROCESS, "Database change cancelled - another process has locked this database"

    company = Company()
    if company.registered_address:
        company_registered_address_id = company.registered_address.id
    else:
        company_registered_address_id = None

    company_vat_regnumber    = company.vat_regnumber
    company_vat_country_code = company.vat_country_code
    if company_vat_regnumber != vat_regnumber or company_vat_country_code != vat_country_code:
        new_company = (None,
                       company_registered_address_id,
                       vat_country_code,
                       vat_regnumber,
                       company.company_number,
                       company.country_registration,
                       company.name,
                       company.sortcode,
                       company.bank_account,
                       company.iban,
                       company.bic)

    # Create a new scheme
    new_vat_scheme=[None, start_date, method, period, percent]

    # compare if changes are needed
    current_vat_scheme = VATScheme()

    vat_scheme_id = None

    if (current_vat_scheme.start_date == start_date and
        current_vat_scheme.method == method and
        current_vat_scheme.period == period and
        str(current_vat_scheme.flat_percent) == percent):
        # all equal, so do not create another vat scheme
        vat_scheme_id = current_vat_scheme.id

    conn = None
    try:
        conn, c = vs_connect()

        if not vat_scheme_id:
            # get the last transaction_id
            c.execute("SELECT MAX(TRANSACTION_ID) FROM TRANSACTIONS")
            result = c.fetchone()
            if not result:
                transaction_id = None
            else:
                transaction_id = result[0]
            new_vat_scheme.append(transaction_id)
            c.execute("""INSERT INTO VAT_SCHEMES
                             (VAT_SCHEME_ID, STARTDATE, METHOD, PERIOD, FLAT_PERCENT, TRANSACTION_ID)
                              VALUES (?, ?, ?, ?, ?, ?)""",
                             new_vat_scheme)
            c.execute("SELECT last_insert_rowid()")
            result = c.fetchone()
            vat_scheme_id = result[0]

        if company_vat_regnumber != vat_regnumber or company_vat_country_code != vat_country_code:
            # Create a new company row
            c.execute("""INSERT INTO COMPANIES
                             ( COMPANY_ID,
                               ADDRESS_ID,
                               VAT_COUNTRY_CODE,
                               VAT_REGNUMBER,
                               COMPANY_NUMBER,
                               COUNTRY_REGISTRATION,
                               COMPANY_NAME,
                               SORTCODE,
                               BANK_ACCOUNT,
                               IBAN,
                               BIC ) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)""",
                             new_company)

        conn.commit()
        # Note, there is no except clause, so if an exception occurs, it is propagated upwards
    finally:
        if conn:
            conn.close()

    # Make the default country code for suppliers and customers, the same code
    # as the company country code
    vs_cfg["VAT_COUNTRY_CODE"] = vat_country_code

    return VATScheme(vat_scheme_id)


def create_Customer(customer):
    """customer is a CustomerForm object, use it to create a new customer on the database
       and return a Customer object, return None on failure"""

    if not customer.name:
        return None

    # get next free customer number
    number = select("SELECT MAX(NUMBER) FROM CUSTOMERS")
    if not number:
        number = 1
    else:
        number += 1

    if customer.ec_country:
        ec_country = 1
    else:
        ec_country = 0

    if customer.vat_registered:
        vat_registered = 1
    else:
        vat_registered = 0

    if customer.ec_country and not customer.vat_country_code:
        # Set default VAT country code
        customer.vat_country_code = vs_cfg["VAT_COUNTRY_CODE"]

    discount = str(customer.discount)

    # EDIT_NEW - if zero (False) the name, ec_country, vat_country_code and vat_regnumber can be edited
    #          - if one (True), if the above are edited, a new item is created, leaving
    #          - the old one untouched

    # VISIBLE - If zero (false) the item is not available for editing
    #         - if one (true) the item can be edited

    # As this is a new customer, edit_new is False, and visible is True
    edit_new = 0
    visible = 1

    # These bank details do not yet exist
    sortcode = ""
    bank_account = ""
    iban = ""
    bic = ""

    # First, get the folder for customer accounts
    folder_id = select("SELECT FOLDER_ID FROM FOLDERS WHERE SYSTEM = 3")
    customer_folder = Folder(folder_id)

    # Create the customers account in this folder
    # Initially give the account the same name as the customer
    account_name = customer.name 

    # Check for name uniqueness within the folder
    accounts_in_folder = customer_folder.get_accounts()
    if accounts_in_folder != []:
        name_unique = False
        index = 1
        while not name_unique:
            for acc in accounts_in_folder:
                name_unique = True
                if account_name == acc.name:
                    name_unique = False
                    index += 1
                    account_name = "%s_%i" % (customer.name, index)
                    break

    customer_account_type_id = select("SELECT CFG_ITEM FROM CONFIGURATION WHERE CFG_KEY = ?", ("CUSTOMER_ACCOUNT_TYPE_ID",))

    # Note the "0.00" is the balance, always zero for a new account
    # and system is 3 - Customer accounts            
    new_account=(None,
                 folder_id,
                 account_name,
                 "Customer account for %s" % customer.name,
                 "0.00",
                 3,
                 int(customer_account_type_id))

    # Make the connection
    conn = None
    try:
        conn, c = vs_connect()

        # insert the account
        c.execute("""INSERT INTO ACCOUNTS 
                         (ACCOUNT_ID,
                          FOLDER_ID,
                          NAME,
                          DESCRIPTION,
                          BALANCE,
                          SYSTEM,
                          ACCOUNTTYPE_ID) VALUES (?, ?, ?, ?, ?, ?, ?)""", new_account)

        c.execute("SELECT last_insert_rowid()")
        result     = c.fetchone()
        account_id    = result[0]

        if not account_id:
            conn.close()
            return None
        ledger = "LEDGER_%i" % account_id
        
        # create a ledger table
        # Create a separate string for the sql query, since it
        # seems ? parameters are not allowed here
      
        sql_string = """CREATE TABLE %s
                             (
                             LEDGER_ID INTEGER PRIMARY KEY,
                             TRANSACTION_ID INTEGER REFERENCES TRANSACTIONS(TRANSACTION_ID),
                             GRIDROW INTEGER NOT NULL DEFAULT -1,
                             DEBIT TEXT,
                             CREDIT TEXT,
                             BALANCE TEXT)""" % ledger

        c.execute(sql_string)
               
        # create the customer
        new_customer=(None, customer.name, ec_country, customer.vat_country_code,
                      customer.vat_regnumber, vat_registered, discount, account_id,
                      number, sortcode, bank_account, iban, bic, edit_new, visible)

        # insert the new customer
        c.execute("""INSERT INTO CUSTOMERS 
                             (CUSTOMER_ID,
                              NAME,
                              EC_COUNTRY,
                              VAT_COUNTRY_CODE,
                              VAT_REGNUMBER,
                              VAT_REGISTERED,
                              DISCOUNT,
                              ACCOUNT_ID,
                              NUMBER,
                              SORTCODE,
                              BANK_ACCOUNT,
                              IBAN,
                              BIC,
                              EDIT_NEW,
                              VISIBLE
                              ) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)""", new_customer)

        c.execute("SELECT last_insert_rowid()")
        result      = c.fetchone()
        customer_id = result[0]

        if not customer_id:
            conn.close()
            return None

        conn.commit()
    finally:
        if conn: 
            conn.close()

    return Customer(customer_id)


def create_Supplier(supplier):
    """supplier is a SupplierForm object, use it to create a new supplier on the database
       and return a Supplier object, return None on failure"""

    if not supplier.name:
        return None

    # get next free supplier number
    number = select("SELECT MAX(NUMBER) FROM SUPPLIERS")
    if not number:
        number = 1
    else:
        number += 1

    if supplier.ec_country:
        ec_country = 1
    else:
        ec_country = 0

    if supplier.vat_registered:
        vat_registered = 1
    else:
        vat_registered = 0

    if supplier.ec_country and not supplier.vat_country_code:
        supplier.vat_country_code = vs_cfg["VAT_COUNTRY_CODE"]

    discount = str(supplier.discount)

    # EDIT_NEW - if zero (False) the name, ec_country, vat_country_code and vat_regnumber can be edited
    #          - if one (True), if the above are edited, a new item is created, leaving
    #          - the old one untouched

    # VISIBLE - If zero (false) the item is not available for editing
    #         - if one (true) the item can be edited

    # As this is a new supplier, edit_new is False, and visible is True
    edit_new = 0
    visible = 1

    # These bank details do not yet exist
    sortcode = ""
    bank_account = ""
    iban = ""
    bic = ""

    # folder which holds supplier accounts
    folder_id = select("SELECT FOLDER_ID FROM FOLDERS WHERE SYSTEM = 4")
    supplier_folder = Folder(folder_id)

    # Create an account for this supplier

    # Initially give the account the same name as the supplier
    account_name = supplier.name 

    # Check for name uniqueness within the folder
    accounts_in_folder = supplier_folder.get_accounts()
    if accounts_in_folder != []:
        name_unique = False
        index = 1
        while not name_unique:
            for acc in accounts_in_folder:
                name_unique = True
                if account_name == acc.name:
                    name_unique = False
                    index += 1
                    account_name = "%s_%i" % (supplier.name, index)
                    break

    supplier_account_type_id = select("SELECT CFG_ITEM FROM CONFIGURATION WHERE CFG_KEY = ?", ("SUPPLIER_ACCOUNT_TYPE_ID",))

    # Note the "0.00" is the balance, always zero for a new account
    # and system is 4 - Supplier accounts            
    new_account=(None,
                 folder_id,
                 account_name,
                 "Supplier account for %s" % supplier.name,
                 "0.00",
                 4,
                 int(supplier_account_type_id))

    # Make the connection
    conn = None
    try:
        conn, c = vs_connect()

        # insert the account
        c.execute("""INSERT INTO ACCOUNTS 
                         (ACCOUNT_ID,
                          FOLDER_ID,
                          NAME,
                          DESCRIPTION,
                          BALANCE,
                          SYSTEM,
                          ACCOUNTTYPE_ID) VALUES (?, ?, ?, ?, ?, ?, ?)""", new_account)

        c.execute("SELECT last_insert_rowid()")
        result     = c.fetchone()
        account_id    = result[0]

        if not account_id:
            conn.close()
            return None
        ledger = "LEDGER_%i" % account_id
        
        # create a ledger table
        # Create a separate string for the sql query, since it
        # seems ? parameters are not allowed here
      
        sql_string = """CREATE TABLE %s
                             (
                             LEDGER_ID INTEGER PRIMARY KEY,
                             TRANSACTION_ID INTEGER REFERENCES TRANSACTIONS(TRANSACTION_ID),
                             GRIDROW INTEGER NOT NULL DEFAULT -1,
                             DEBIT TEXT,
                             CREDIT TEXT,
                             BALANCE TEXT)""" % ledger

        c.execute(sql_string)

        new_supplier=(None, supplier.name, ec_country, supplier.vat_country_code,
                      supplier.vat_regnumber, vat_registered, discount, account_id,
                      number, sortcode, bank_account, iban, bic, edit_new, visible)


        # insert the new supplier
        c.execute("""INSERT INTO SUPPLIERS 
                            (SUPPLIER_ID,
                             NAME,
                             EC_COUNTRY,
                             VAT_COUNTRY_CODE,
                             VAT_REGNUMBER,
                             VAT_REGISTERED,
                             DISCOUNT,
                             ACCOUNT_ID,
                             NUMBER,
                             SORTCODE,
                             BANK_ACCOUNT,
                             IBAN,
                             BIC,
                             EDIT_NEW,
                             VISIBLE
                              ) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)""", new_supplier)

        c.execute("SELECT last_insert_rowid()")
        result      = c.fetchone()
        supplier_id = result[0]

        if not supplier_id:
            conn.close()
            return None

        conn.commit()
    finally:
        if conn: 
            conn.close()

    return Supplier(supplier_id)


def create_Address(customer=None,
                   supplier=None,
                   employee=None,
                   contractor=None,
                   line1 = "",
                   line2 = "",
                   city = "",
                   county = "",
                   postcode = "",
                   country = "",
                   reg_office = 0,
                   delivery_point = 0,
                   invoice_point = 0,
                   order_point = 0,
                   payment_point = 0,
                   edit_new = False,
                   visible = True,
                   system = 0):
    """Create a new address on the database and return an Address object, return None on failure"""

    # Create a new address
    if customer:
        customer_id = customer.id
    else:
        customer_id = None

    if supplier:
        supplier_id = supplier.id
    else:
        supplier_id = None

    # Employee and Contractor classes have not yet been defined
    employee_id = None
    contractor_id = None

    if edit_new:
        edit_new = 1
    else:
        edit_new = 0

    if visible:
        visible = 1
    else:
        visible = 0

    new_address=(None,
                 customer_id,
                 supplier_id,
                 employee_id,
                 contractor_id,
                 line1,
                 line2,
                 city,
                 county,
                 postcode,
                 country,
                 reg_office,
                 delivery_point,
                 invoice_point,
                 order_point,
                 payment_point,
                 edit_new,
                 visible,
                 system)

    # SYSTEM NUMBER

    # 0 supplier or customer address
    # 1 your address - which appears in choices where your company address is to be picked

    # insert the new address
    address_id=insert("""INSERT INTO ADDRESSES 
                          (
                          ADDRESS_ID,
                          CUSTOMER_ID,
                          SUPPLIER_ID,
                          EMPLOYEE_ID,
                          CONTRACTOR_ID, 
                          LINE1,
                          LINE2,
                          CITY,
                          COUNTY,
                          POSTCODE,
                          COUNTRY,
                          REG_OFFICE,
                          DELIVERY_POINT,
                          INVOICE_POINT,
                          ORDER_POINT,
                          PAYMENT_POINT,
                          EDIT_NEW,
                          VISIBLE,
                          SYSTEM) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)""", new_address)

    return Address(address_id)


def create_Contact(customer = None,
                   supplier = None,
                   employee = None,
                   contractor = None,
                   name = "",
                   email = "",
                   phone = "",
                   mobile = "",
                   notes = "",
                   edit_new = False,
                   visible = True,
                   system = 0 ):
    """Create a new contact on the database and return a Contact object, return None on failure"""

    # SYSTEM NUMBER
    # 0 : names which should not appear on orders and invoices as points of contact
    #     - ie your customer or suppliers contacts
    # 1 : names which should appear as enquiry contacts on orders and invoices
    #     - ie your own comapany contacts, these appear in thr drop down list
    #          when you create an invoice, purchase order or credit note

    # EDIT_NEW - if zero (False) The item name, email, phone, mobile can be edited
    #          - if one (True), if the above are edited, a new item is created, leaving
    #          - the old one untouched

    # VISIBLE - If zero (false) the item is not available for editing
    #         - if one (true) the item can be edited

    if customer:
        customer_id = customer.id
    else:
        customer_id = None

    if supplier:
        supplier_id = supplier.id
    else:
        supplier_id = None

    # Currently the employee and contractor classes have not been defined
    employee_id = None
    contractor_id = None

    if edit_new:
        edit_new = 1
    else:
        edit_new = 0

    if visible:
        visible = 1
    else:
        visible = 0
   
    # Create a new contact
    new_contact=(None, customer_id, 
                       supplier_id, 
                       employee_id, 
                       contractor_id, 
                       name, email, phone, mobile, notes,
                       edit_new,
                       visible,
                       system)

    # insert the new contact
    contact_id=insert("""INSERT INTO CONTACTS 
                        (CONTACT_ID,
                         CUSTOMER_ID,
                         SUPPLIER_ID,
                         EMPLOYEE_ID,
                         CONTRACTOR_ID,
                         NAME,
                         EMAIL,
                         PHONE,
                         MOBILE,
                         NOTES,
                         EDIT_NEW,
                         VISIBLE,
                         SYSTEM) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)""",
                         new_contact)

    return Contact(contact_id)


def create_ItemDescription(description):
    """This object is slightly different from most, in that calling it with a description will
       normally create a new table row - but if a table row already has that description, it will
       return an object with the id of the existing row, rather than create a duplicate.

       Returns None on failure

       Therefore x = create_ItemDescription(description="hello")
       will make x an ItemDescription object, by either referring to an existing database row
       or creating a new one if one does not already exist."""

    # check description must exist
    if not description:
        return None

    description = description.strip()

    # check description must still exist
    if not description:
        return None

    # First check an item with this description doesn't already exist
    number = select("SELECT COUNT(*) FROM ITEMDESCRIPTIONS WHERE DESCRIPTION = ?", (description,))
    if number > 0:
        itemdescription_id = select("SELECT ITEMDESCRIPTION_ID FROM ITEMDESCRIPTIONS WHERE DESCRIPTION = ?",
                                     (description,))
        return ItemDescription(itemdescription_id)

    # Description does not exist, so insert a new value
    itemdescription_id = insert("""INSERT INTO ITEMDESCRIPTIONS 
                                   (ITEMDESCRIPTION_ID, DESCRIPTION) VALUES (?, ?)""", (None, description))
    return ItemDescription(itemdescription_id)


def create_SellItem(description, cost = D("0.00"), vatrate = None, notes = ""):
    "Create a new sellitem on the database and return a SellItem object, return None on failure"

    if not description:
        return None

    itemdescription = create_ItemDescription(description)

    if itemdescription:
        itemdescription_id = itemdescription.id
    else:
        return None

    if isinstance(vatrate, VATRate):
        vatrate_id = vatrate.id
    else:
        return None

    # Create a new sellitem
    sellitem_id=insert("""INSERT INTO SELLITEMS 
                        (SELLITEM_ID,
                         ITEMDESCRIPTION_ID,
                         COST,
                         VAT_RATE_ID,
                         NOTES) VALUES (?, ?, ?, ?, ?)""",
                           (None, itemdescription_id, str(cost), vatrate_id, notes))
    return SellItem(sellitem_id)


def create_PurchaseItem(description, cost = D("0.00"), vatrate = None, notes = ""):
    "Create a new purchaseitem on the database and return a PurchaseItem object, return None on failure"

    if not description:
        return None

    itemdescription = create_ItemDescription(description)

    if itemdescription:
        itemdescription_id = itemdescription.id
    else:
        return None

    if isinstance(vatrate, VATRate):
        vatrate_id = vatrate.id
    else:
        return None
   
    # Create a new purchaseitem
    purchaseitem_id=insert("""INSERT INTO PURCHASEITEMS 
                           (PURCHASEITEM_ID,
                            ITEMDESCRIPTION_ID,
                            COST,
                            VAT_RATE_ID,
                            NOTES)
                         VALUES (?, ?, ?, ?, ?)""", (None, itemdescription_id, str(cost), vatrate_id, notes))
    return PurchaseItem(purchaseitem_id)


def create_Project(number, description="", contact = None):
    "Create a new project on the database and return a Project object, return None on failure"

    if not number:
        return None

    if contact:
        contact_id = contact.id
    else:
        contact_id = None

    conn = None
    try:
        conn, c = vs_connect()

        # First check a project with this project number doesn't already exist
        c.execute("SELECT COUNT(*) FROM PROJECTS WHERE PROJECT_NUMBER = ?", (number,))
        result = c.fetchone()
        if result[0] > 0:
            return None

        # Create a new project
        c.execute("""INSERT INTO PROJECTS 
                           (PROJECT_ID,
                            PROJECT_NUMBER,
                            DESCRIPTION,
                            CONTACT_ID)
                         VALUES (?, ?, ?, ?)""", (None, number, description, contact_id))

        c.execute("SELECT last_insert_rowid()")
        result = c.fetchone()
        project_id = result[0]

        conn.commit()
    finally:
        if conn:
            conn.close()

    return Project(project_id)



