####### VATStuff #######
#
# database_versions.py  - version conversion functions
#
# Version : 0.7.0
# Date : 20100912
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#


"""version conversion functions"""

# Do not import wx - so these database functions can be done and used by the testing
# process without the graphical wx being present

import decimal
import os

from vatstuff_packages.vs_utilities import *
from vatstuff_packages.vs_database.create_objects import create_Account


D = decimal.Decimal   # D used to avoid typing


def versioncheck():
    """Called if check on the database failed, calls functions to
       determine and convert older versions of the database, returns
       True on successfull conversion, False otherwise"""

    debug_log("running versioncheck")

    # Check database file version,
    try:
        read_database_version = get_database_version()
        info_log("Database has version %s" % read_database_version)
        if read_database_version == vs_cfg["DATABASE_VERSION"]:
            # Database is current, no conversion required
            return True

        # So the database is not current - call conversion functions
        # If there is any error, conversions raise an exception
        # which causes this function to return a False

        if read_database_version == "0.0":
            ### CONVERT DATABASE 0.0
            _convert_from_0_0()
        # database versions 0.1 and 0.2 cannot be converted
        elif read_database_version == "0.3":
            ### CONVERT DATABASE 0.3
            _convert_from_0_3()
        # Note - No version 0.4 database was produced
        elif read_database_version == "0.5":
            ### CONVERT DATABASE 0.5
            _convert_from_0_5()
        elif read_database_version == "0.6":
            ### CONVERT DATABASE 0.6
            _convert_from_0_6()
        else:
            # other database versions cannot be converted so return False
            info_log("Database cannot be converted")
            return False

    except Exception, e:
        exception_log(e)
        info_log("Conversion failed") 
        return False

    info_log("Conversion completed")
    return True


def _convert_from_0_0():
    "test conversion - this simply changes the database version number to the current version"
    # this function is used for test purposes
    info_log("Converting database from 0.0 to %s" % vs_cfg["DATABASE_VERSION"])
    conn = None
    try:
        conn, c = vs_connect()
        c.execute("UPDATE CONFIGURATION SET CFG_ITEM = ? WHERE CFG_KEY = ?", (vs_cfg["DATABASE_VERSION"], "DATABASE_VERSION"))
        conn.commit()
    finally:
        if conn:
            conn.close()


def _convert_from_0_3():
    "This changes the database version 0.3 to 0.7"
    # From 0.5 invoicepayments may have a NULL in the invoice_id field
    # From 0.7 account names may be unicode
    # 0.7 introduced a system account "Unclaimed VAT" (new system number 9)
    info_log("Converting database from 0.3 to 0.7")
    # Check - does a system 9 account already exist?
    account = get_account_by_system(9)
    account_details = None
    if not account:
        # get a tuple to insert into the database
        account_details = unclaimed_VAT_account_details()
        if not account_details:
            raise VS_ERROR, "Unable to create unclaimed VAT account details"
    conn = None
    try:
        conn, c = vs_connect()
        c.execute("UPDATE CONFIGURATION SET CFG_ITEM = ? WHERE CFG_KEY = ?", ("0.7", "DATABASE_VERSION"))
        # Add Unclaimed VAT account
        insert_account(c, account_details)
        # add the CLAIM_VAT columns
        add_claim_vat_to_purchases(c)
        add_claim_vat_to_supplierinvoices(c)
        add_claim_vat_to_suppliercreditnotes(c)
        add_claim_vat_to_supplierinvoicepayments(c)
        conn.commit()
    finally:
        if conn:
            conn.close()


def _convert_from_0_5():
    "This changes the database version 0.5 to 0.7"
    # From 0.7 account names may be unicode
    # 0.7 introduced a system account "Unclaimed VAT" (new system number 9)
    info_log("Converting database from 0.5 to 0.7")
    # Check - does a system 9 account already exist?
    account = get_account_by_system(9)
    account_details = None
    if not account:
        # get a tuple to insert into the database
        account_details = unclaimed_VAT_account_details()
        if not account_details:
            raise VS_ERROR, "Unable to create unclaimed VAT account details"
    conn = None
    try:
        conn, c = vs_connect()
        c.execute("UPDATE CONFIGURATION SET CFG_ITEM = ? WHERE CFG_KEY = ?", ("0.7", "DATABASE_VERSION"))
        # Add Unclaimed VAT account
        insert_account(c, account_details)
        # add the CLAIM_VAT columns
        add_claim_vat_to_purchases(c)
        add_claim_vat_to_supplierinvoices(c)
        add_claim_vat_to_suppliercreditnotes(c)
        add_claim_vat_to_supplierinvoicepayments(c)
        conn.commit()
    finally:
        if conn:
            conn.close()


def _convert_from_0_6():
    "This changes the database version 0.6 to 0.7"
    # From 0.7 account names may be unicode
    # 0.7 introduced a system account "Unclaimed VAT" (new system number 9)
    info_log("Converting database from 0.6 to 0.7")
    # Check - does a system 9 account already exist?
    account = get_account_by_system(9)
    account_details = None
    if not account:
        # get a tuple to insert into the database
        account_details = unclaimed_VAT_account_details()
        if not account_details:
            raise VS_ERROR, "Unable to create unclaimed VAT account details"
    conn = None
    try:
        conn, c = vs_connect()
        c.execute("UPDATE CONFIGURATION SET CFG_ITEM = ? WHERE CFG_KEY = ?", ("0.7", "DATABASE_VERSION"))
        # Add Unclaimed VAT account
        insert_account(c, account_details)
        # add the CLAIM_VAT columns
        add_claim_vat_to_purchases(c)
        add_claim_vat_to_supplierinvoices(c)
        add_claim_vat_to_suppliercreditnotes(c)
        add_claim_vat_to_supplierinvoicepayments(c)
        conn.commit()
    finally:
        if conn:
            conn.close()


########### utility functions

def unclaimed_VAT_account_details():
    """From version 0.7 a system account, unclaimed VAT was added
       this returns a tuple of account details, for the new account
       returns None on failure"""

    # get the system folder
    all_folders = Folder.get_all()
    system_folder = None
    for folder in all_folders:
        if folder.system == 2:
            system_folder = folder
            break

    if not system_folder:
        return None

    # Ensure the account name is unique
    name = "Unclaimed VAT"
    acc = system_folder.get_account_with_name(name)
    inc = 1
    while acc:
        inc += 1
        name = "Unclaimed VAT %s" % inc
        acc = system_folder.get_account_with_name(name)
        # this while breaks when no acc with the given name is found

    # Get the account type, which should be an Expense
    acc_type = AccountType.from_name("Expense")
    # however, if the user has changed the name, use the same account type
    # as the purchases account
    if not acc_type:
        purchases_account = get_account_by_system(7)
        acc_type = purchases_account.acc_type

    # and return a tuple of account details
    # Note the "0.00" is the balance, always zero for a new account             
    return (None, system_folder.id, name, "Input VAT not claimed back", "0.00", 9, acc_type.id)


def insert_account(c, details):
    """creates an account, given the database cursor, and the account details
       does not commit the transaction"""
    if not details:
        return
    # insert the account
    c.execute("""INSERT INTO ACCOUNTS 
                        (ACCOUNT_ID,
                        FOLDER_ID,
                        NAME,
                        DESCRIPTION,
                        BALANCE,
                        SYSTEM,
                        ACCOUNTTYPE_ID) VALUES (?, ?, ?, ?, ?, ?, ?)""", details)

    c.execute("SELECT last_insert_rowid()")
    result     = c.fetchone()
    account_id    = result[0]

    ledger = "LEDGER_%i" % account_id
    
    # create a ledger table
    # Create a separate string for the sql query, since it
    # seems ? parameters are not allowed here
    
    sql_string = """CREATE TABLE %s
                            (
                            LEDGER_ID INTEGER PRIMARY KEY,
                            TRANSACTION_ID INTEGER REFERENCES TRANSACTIONS(TRANSACTION_ID),
                            GRIDROW INTEGER NOT NULL DEFAULT -1,
                            DEBIT TEXT,
                            CREDIT TEXT,
                            BALANCE TEXT)""" % ledger
    c.execute(sql_string)


def add_claim_vat_to_purchases(c):
    """Adds the CLAIM_VAT column to the PURCHASEORDERS table"""
    # check the column isn't already there
    c.execute("SELECT * FROM PURCHASEORDERS")
    col_name_list = [item[0] for item in c.description]
    if "CLAIM_VAT" in col_name_list:
        return
    c.execute("""ALTER TABLE PURCHASEORDERS
                 ADD CLAIM_VAT INTEGER DEFAULT 1""")

def add_claim_vat_to_supplierinvoices(c):
    """Adds the CLAIM_VAT column to the SUPPLIERINVOICES table"""
    # check the column isn't already there
    c.execute("SELECT * FROM SUPPLIERINVOICES")
    col_name_list = [item[0] for item in c.description]
    if "CLAIM_VAT" in col_name_list:
        return
    c.execute("""ALTER TABLE SUPPLIERINVOICES
                 ADD CLAIM_VAT INTEGER DEFAULT 1""")

def add_claim_vat_to_suppliercreditnotes(c):
    """Adds the CLAIM_VAT column to the SUPPLIERCREDITNOTES table"""
    # check the column isn't already there
    c.execute("SELECT * FROM SUPPLIERCREDITNOTES")
    col_name_list = [item[0] for item in c.description]
    if "CLAIM_VAT" in col_name_list:
        return
    c.execute("""ALTER TABLE SUPPLIERCREDITNOTES
                 ADD CLAIM_VAT INTEGER DEFAULT 1""")

def add_claim_vat_to_supplierinvoicepayments(c):
    """Adds the CLAIM_VAT column to the SUPPLIERINVOICEPAYMENTS table"""
    # check the column isn't already there
    c.execute("SELECT * FROM SUPPLIERINVOICEPAYMENTS")
    col_name_list = [item[0] for item in c.description]
    if "CLAIM_VAT" in col_name_list:
        return
    c.execute("""ALTER TABLE SUPPLIERINVOICEPAYMENTS
                 ADD CLAIM_VAT INTEGER DEFAULT 1""")


