####### VATStuff #######
#
# delete_old_data.py  - deletes old data from the database
#
# Version : 0.2
# Date : 20091113
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#

"Deletes old data from the database"

import decimal
import datetime

from vatstuff_packages.vs_utilities import *

D = decimal.Decimal   # D used to avoid typing


def update_progress(dlg, percent):
    "Updates the progress dialog by percent amount, returns True if OK, False if an Abort is requested"
    if not dlg:
        return True
    status, skip = dlg.Update(percent)
    return status


def do_delete(to_date, dlg):
    """This deletes database data prior to to_date, returns (status, message)
       status = True if data deleted, False otherwise, message is a text string
       to be displayed"""

    if not vs_cfg["WRITE_ACCESS"]:
        # Another process has locked the database
        return False, "Deletion failed - another process has locked this database"

    if not update_progress(dlg, 1):
        return False, "Delete Aborted"

    # Get a list of all accounts
    all_accounts = Account.get_all()
    if not update_progress(dlg, 3):
        return False, "Delete Aborted"

    conn = None
    try:
        conn, cursor = vs_connect()

        # for every account, get the balance on the day to be deleted
        all_balances = _get_balances(cursor, to_date, all_accounts)

        if not update_progress(dlg, 5):
            return False, "Delete Aborted"

        # Now do deletions
        # Note, the order of deletions is important so tables that reference other tables
        # are deleted such that the one doing the referencing is deleted before the 
        # the table which is the destination reference

        ### SALES
        # Delete customer payments
        _delete_customer_payments(cursor, to_date)
        if not update_progress(dlg, 10):
            return False, "Delete Aborted"

        # Delete Creditnotes to customers
        _delete_creditnotes(cursor, to_date)
        if not update_progress(dlg, 15):
            return False, "Delete Aborted"

        # Delete invoices to customers
        status, message = _delete_invoices(cursor, to_date)
        if not status:
            return False, message
        if not update_progress(dlg, 20):
            return False, "Delete Aborted"

        ### Purchases
        # Delete payments made to suppliers
        _delete_makepayments(cursor, to_date)
        if not update_progress(dlg, 25):
            return False, "Delete Aborted"

        # Delete Creditnotes from suppliers
        _delete_suppliercreditnotes(cursor, to_date)
        if not update_progress(dlg, 30):
            return False, "Delete Aborted"

        # Delete Invoices from suppliers
        status, message = _delete_supplierinvoices(cursor, to_date)
        if not status:
            return False, message
        if not update_progress(dlg, 35):
            return False, "Delete Aborted"

        # Delete Purchase Orders
        status, message = _delete_purchaseorders(cursor, to_date)
        if not status:
            return False, message
        if not update_progress(dlg, 40):
            return False, "Delete Aborted"

        ### ITEM DESCRIPTIONS
        # delete any item descriptions not referenced by anything
        _delete_itemdescriptions(cursor)
        if not update_progress(dlg, 45):
            return False, "Delete Aborted"

        ### CONTACTS
        # delete any contacts not referenced by anything
        _delete_contacts(cursor)
        if not update_progress(dlg, 50):
            return False, "Delete Aborted"

        ### VAT RETURNS
        # delete vat corrections
        _delete_vatcorrections(cursor, to_date)
        if not update_progress(dlg, 55):
            return False, "Delete Aborted"

        # delete vat returns - these are referenced in invoices etc, so have to be done after
        # invoices are deleted
        _delete_vatreturns(cursor, to_date)
        if not update_progress(dlg, 60):
            return False, "Delete Aborted"

        ### Delete Companies
        # delete past records of this company not referenced by anything
        _delete_companies(cursor)
        if not update_progress(dlg, 65):
            return False, "Delete Aborted"

        ### Delete Addresses
        # delete addresses not referenced by anything
        _delete_addresses(cursor)
        if not update_progress(dlg, 70):
            return False, "Delete Aborted"

        ### Delete Customers
        # delete past records of customers not referenced by anything
        _delete_customers(cursor)
        if not update_progress(dlg, 75):
            return False, "Delete Aborted"

        ### Delete Suppliers
        # delete past records of suppliers not referenced by anything
        _delete_suppliers(cursor)
        if not update_progress(dlg, 80):
            return False, "Delete Aborted"

        ###LEDGER ENTRIES AND TRANSACTIONS
        # delete all ledger entries prior to the to_date
        _delete_ledger_entries(cursor, to_date, all_accounts)
        if not update_progress(dlg, 85):
            return False, "Delete Aborted"

        # delete transactions
        _delete_transactions(cursor, to_date)
        if not update_progress(dlg, 90):
            return False, "Delete Aborted"

        # create carried forwards transaction
        _carried_forwards(cursor, to_date, all_accounts, all_balances)
        if not update_progress(dlg, 95):
            return False, "Delete Aborted"

        # set the delete_to_date in the configuration
        cursor.execute("UPDATE CONFIGURATION SET CFG_ITEM = ? WHERE CFG_KEY = ?",
                        (to_date.isoformat(),"DELETE_TO_DATE"))
        if not update_progress(dlg, 98):
            return False, "Delete Aborted"

        conn.commit()
    finally:
        if conn:
            conn.close()

    vs_cfg["DELETE_TO_DATE"] = to_date
    return True, "Old Data deleted"


def _get_balances(cursor, to_date, all_accounts):
    "Return a list of all balances for each account prior to to_date"
    balance_list = []
    for acc in all_accounts:
        ledger = "LEDGER_%i" % acc.id

        sql_string = """SELECT MAX(%s.GRIDROW) FROM %s, TRANSACTIONS
                        WHERE
                        DATESTAMP < ?
                        AND %s.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID""" % (ledger, ledger, ledger)

        cursor.execute(sql_string, (to_date,))
        result = cursor.fetchone()
        if not result:
            balance_list.append(D("0.00"))
            continue
        gridrow = result[0]
        sql_string = "SELECT BALANCE FROM %s WHERE GRIDROW = ?" % ledger
        cursor.execute(sql_string, (gridrow,))
        result = cursor.fetchone()
        if not result:
            balance_list.append(D("0.00"))
        else:
            balance_list.append(D(result[0]))

    return balance_list


def _delete_customer_payments(cursor, to_date):
    "Deletes customer payments older than to_date"
    # Deletes invoice payments that reference customer payments to be deleted
    cursor.execute("""DELETE FROM INVOICEPAYMENTS WHERE CUSTOMERPAYMENT_ID IN
                      (SELECT CUSTOMERPAYMENT_ID FROM CUSTOMERPAYMENTS, TRANSACTIONS
                       WHERE
                       DATESTAMP < ?
                       AND CUSTOMERPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID)""", (to_date,))

    # Delete the customer payments
    cursor.execute("""DELETE FROM CUSTOMERPAYMENTS WHERE CUSTOMERPAYMENT_ID IN
                      (SELECT CUSTOMERPAYMENT_ID FROM CUSTOMERPAYMENTS, TRANSACTIONS
                       WHERE
                       DATESTAMP < ?
                       AND CUSTOMERPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID)""", (to_date,))


def _delete_creditnotes(cursor, to_date):
    "Deletes creditnotes to customers older than to_date"
    # Delete the creditnoteitems which reference creditnotes to be deleted
    cursor.execute("""DELETE FROM CREDITNOTEITEMS WHERE CREDITNOTE_ID IN
                      (SELECT CREDITNOTE_ID FROM CREDITNOTES, TRANSACTIONS
                       WHERE
                       DATESTAMP < ?
                       AND CREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID)""", (to_date,))

    # Delete the creditnotes
    cursor.execute("""DELETE FROM CREDITNOTES WHERE CREDITNOTE_ID IN
                      (SELECT CREDITNOTE_ID FROM CREDITNOTES, TRANSACTIONS
                       WHERE
                       DATESTAMP < ?
                       AND CREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID)""", (to_date,))

    # And update the gridrow value for the remaining creditnotes
    cursor.execute("SELECT MIN(GRIDROW) FROM CREDITNOTES")
    result = cursor.fetchone()
    # gridrow start at zero
    if not result:
        return
    if result[0]:
        # result[0] must be greater than 0
        sql_string = "UPDATE CREDITNOTES SET GRIDROW = GRIDROW-%s" % result[0]
        cursor.execute(sql_string)


def _delete_invoices(cursor, to_date):
    "Deletes invoices to customers older than to_date, returns status, message"

    # Invoices with transaction datestamp prior to to_date are to be deleted

    # Check - are any invoices to be deleted still open?
    cursor.execute("""SELECT COUNT(*) FROM INVOICES, TRANSACTIONS
                       WHERE
                       DATESTAMP < ?
                       AND OPEN = ?
                       AND INVOICES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID""", (to_date, 1))
    result = cursor.fetchone()
    if result[0]:
        # There must be no open invoices to be deleted  
        return False, "Invoices cannot be deleted if they are still open"

    # Update invoice_id to None in those invoicepayments referring to these invoices
    cursor.execute("""UPDATE INVOICEPAYMENTS SET INVOICE_ID = ? WHERE INVOICE_ID IN
                      (SELECT INVOICE_ID FROM INVOICES, TRANSACTIONS
                       WHERE
                       DATESTAMP < ?
                       AND INVOICES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID)""", (None, to_date))

    # Update invoice_id to None in those creditnotes referring to these invoices
    cursor.execute("""UPDATE CREDITNOTES SET INVOICE_ID = ? WHERE INVOICE_ID IN
                      (SELECT INVOICE_ID FROM INVOICES, TRANSACTIONS
                       WHERE
                       DATESTAMP < ?
                       AND INVOICES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID)""", (None, to_date))

    # Delete the solditems which reference invoices to be deleted
    cursor.execute("""DELETE FROM SOLDITEMS WHERE INVOICE_ID IN
                      (SELECT INVOICE_ID FROM INVOICES, TRANSACTIONS
                       WHERE
                       DATESTAMP < ?
                       AND INVOICES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID)""", (to_date,))

    # Delete the invoices
    cursor.execute("""DELETE FROM INVOICES WHERE INVOICE_ID IN
                      (SELECT INVOICE_ID FROM INVOICES, TRANSACTIONS
                       WHERE
                       DATESTAMP < ?
                       AND INVOICES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID)""", (to_date,))

    # And update the gridrow value for the remaining invoices
    cursor.execute("SELECT MIN(GRIDROW) FROM INVOICES")
    result = cursor.fetchone()
    # gridrow start at zero
    if not result:
        return
    if result[0]:
        # result[0] must be greater than 0
        sql_string = "UPDATE INVOICES SET GRIDROW = GRIDROW-%s" % result[0]
        cursor.execute(sql_string)
    return True, None


def _delete_makepayments(cursor, to_date):
    "Deletes payments made to suppliers older than to_date"
    # Deletes supplierinvoice payments that reference supplier payments to be deleted
    cursor.execute("""DELETE FROM SUPPLIERINVOICEPAYMENTS WHERE MAKEPAYMENT_ID IN
                      (SELECT MAKEPAYMENT_ID FROM MAKEPAYMENTS, TRANSACTIONS
                       WHERE
                       DATESTAMP < ?
                       AND MAKEPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID)""", (to_date,))

    # Delete the payments
    cursor.execute("""DELETE FROM MAKEPAYMENTS WHERE MAKEPAYMENT_ID IN
                      (SELECT MAKEPAYMENT_ID FROM MAKEPAYMENTS, TRANSACTIONS
                       WHERE
                       DATESTAMP < ?
                       AND MAKEPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID)""", (to_date,))


def _delete_suppliercreditnotes(cursor, to_date):
    "Deletes creditnotes from suppliers older than to_date"
    # Delete the creditnotes that reference transactions to be deleted
    cursor.execute("""DELETE FROM SUPPLIERCREDITNOTES WHERE SUPPLIERCREDITNOTE_ID IN
                      (SELECT SUPPLIERCREDITNOTE_ID FROM SUPPLIERCREDITNOTES, TRANSACTIONS
                       WHERE
                       DATESTAMP < ?
                       AND SUPPLIERCREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID)""", (to_date,))


def _delete_supplierinvoices(cursor, to_date):
    "Deletes invoices from suppliers older than to_date, returns status, message"

    # Check - are any supplierinvoices to be deleted still open?
    cursor.execute("""SELECT COUNT(*) FROM SUPPLIERINVOICES, TRANSACTIONS
                       WHERE
                       DATESTAMP < ?
                       AND OPEN = ?
                       AND SUPPLIERINVOICES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID""", (to_date, 1))
    result = cursor.fetchone()
    if result[0]:
        # There must be no open invoices to be deleted  
        return False, "Supplier Invoices cannot be deleted if they are still open"

    # Update supplierinvoice_id to None in those supplierinvoicepayments referring to these supplierinvoices
    # which are to be deleted.
    cursor.execute("""UPDATE SUPPLIERINVOICEPAYMENTS SET SUPPLIERINVOICE_ID = ? WHERE SUPPLIERINVOICE_ID IN
                      (SELECT SUPPLIERINVOICE_ID FROM SUPPLIERINVOICES, TRANSACTIONS
                       WHERE
                       DATESTAMP < ?
                       AND SUPPLIERINVOICES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID)""", (None, to_date))

    # Update supplierinvoice_id to None in those suppliercreditnotes referring to these supplierinvoices
    # which are to be deleted.
    cursor.execute("""UPDATE SUPPLIERCREDITNOTES SET SUPPLIERINVOICE_ID = ? WHERE SUPPLIERINVOICE_ID IN
                      (SELECT SUPPLIERINVOICE_ID FROM SUPPLIERINVOICES, TRANSACTIONS
                       WHERE
                       DATESTAMP < ?
                       AND SUPPLIERINVOICES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID)""", (None, to_date))

    # Delete the supplier invoices that reference transactions to be deleted
    cursor.execute("""DELETE FROM SUPPLIERINVOICES WHERE SUPPLIERINVOICE_ID IN
                      (SELECT SUPPLIERINVOICE_ID FROM SUPPLIERINVOICES, TRANSACTIONS
                       WHERE
                       DATESTAMP < ?
                       AND SUPPLIERINVOICES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID)""", (to_date,))

    return True, None


def _delete_purchaseorders(cursor, to_date):
    "Deletes purchase orders older than to_date, returns status, message"

    # Check - are any purchase orders to be deleted still open?
    # open orders have status 0 or 1, closed orders have status 2 and above
    cursor.execute("""SELECT COUNT(*) FROM PURCHASEORDERS, TRANSACTIONS
                       WHERE
                       DATESTAMP < ?
                       AND STATUS < ?
                       AND PURCHASEORDERS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID""", (to_date, 2))
    result = cursor.fetchone()
    if result[0]:
        # There must be no open purchase orders to be deleted  
        return False, "Purchase orders cannot be deleted if they are still open"

    # Update purchaseorder_id to None in those makepayments referring to these orders
    # which are to be deleted.
    cursor.execute("""UPDATE MAKEPAYMENTS SET PURCHASEORDER_ID = ? WHERE PURCHASEORDER_ID IN
                      (SELECT PURCHASEORDER_ID FROM PURCHASEORDERS, TRANSACTIONS
                       WHERE
                       DATESTAMP < ?
                       AND PURCHASEORDERS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID)""", (None, to_date))

    # Update purchaseorder_id to None in those supplier creditnotes referring to these orders
    # which are to be deleted.
    cursor.execute("""UPDATE SUPPLIERCREDITNOTES SET PURCHASEORDER_ID = ? WHERE PURCHASEORDER_ID IN
                      (SELECT PURCHASEORDER_ID FROM PURCHASEORDERS, TRANSACTIONS
                       WHERE
                       DATESTAMP < ?
                       AND PURCHASEORDERS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID)""", (None, to_date))

    # Update purchaseorder_id to None in those supplier invoices referring to these orders
    # which are to be deleted.
    cursor.execute("""UPDATE SUPPLIERINVOICES SET PURCHASEORDER_ID = ? WHERE PURCHASEORDER_ID IN
                      (SELECT PURCHASEORDER_ID FROM PURCHASEORDERS, TRANSACTIONS
                       WHERE
                       DATESTAMP < ?
                       AND PURCHASEORDERS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID)""", (None, to_date))

    # Delete the ordereditems which reference purchase orders to be deleted
    cursor.execute("""DELETE FROM ORDEREDITEMS WHERE PURCHASEORDER_ID IN
                      (SELECT PURCHASEORDER_ID FROM PURCHASEORDERS, TRANSACTIONS
                       WHERE
                       DATESTAMP < ?
                       AND PURCHASEORDERS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID)""", (to_date,))

    # Delete the purchase orders that reference transactions to be deleted
    cursor.execute("""DELETE FROM PURCHASEORDERS WHERE PURCHASEORDER_ID IN
                      (SELECT PURCHASEORDER_ID FROM PURCHASEORDERS, TRANSACTIONS
                       WHERE
                       DATESTAMP < ?
                       AND PURCHASEORDERS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID)""", (to_date,))

    # Delete the closed orders that reference transactions to be deleted
    cursor.execute("""DELETE FROM CLOSEDORDERS WHERE CLOSEDORDER_ID IN
                      (SELECT CLOSEDORDER_ID FROM CLOSEDORDERS, TRANSACTIONS
                       WHERE
                       DATESTAMP < ?
                       AND CLOSEDORDERS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID)""", (to_date,))

    # And update the gridrow value for the remaining purchase orders
    cursor.execute("SELECT MIN(GRIDROW) FROM PURCHASEORDERS")
    result = cursor.fetchone()
    # gridrow start at zero
    if not result:
        return
    if result[0]:
        # result[0] must be greater than 0
        sql_string = "UPDATE PURCHASEORDERS SET GRIDROW = GRIDROW-%s" % result[0]
        cursor.execute(sql_string)
    return True, None


def _delete_itemdescriptions(cursor):
    "Delete any item description no longer referenced by anything"
    cursor.execute("""DELETE FROM ITEMDESCRIPTIONS WHERE ITEMDESCRIPTION_ID NOT IN
                     (SELECT ITEMDESCRIPTION_ID FROM ITEMDESCRIPTIONS
                      WHERE ITEMDESCRIPTION_ID IN
                             (SELECT ITEMDESCRIPTION_ID FROM PURCHASEITEMS)
                      OR ITEMDESCRIPTION_ID IN
                             (SELECT ITEMDESCRIPTION_ID FROM SELLITEMS)
                      OR ITEMDESCRIPTION_ID IN
                             (SELECT ITEMDESCRIPTION_ID FROM ORDEREDITEMS)
                      OR ITEMDESCRIPTION_ID IN
                             (SELECT ITEMDESCRIPTION_ID FROM SOLDITEMS)
                      OR ITEMDESCRIPTION_ID IN
                             (SELECT ITEMDESCRIPTION_ID FROM CREDITNOTEITEMS))""")


def _delete_contacts(cursor):
    "Delete any contacts no longer referenced by anything"
    cursor.execute("""DELETE FROM CONTACTS WHERE
                       VISIBLE = 0
                       AND CONTACT_ID NOT IN
                     (SELECT CONTACT_ID FROM CONTACTS
                      WHERE CONTACT_ID IN
                             (SELECT CONTACT_ID FROM PURCHASEORDERS)
                      OR CONTACT_ID IN
                             (SELECT CONTACT_ID FROM INVOICES)
                      OR CONTACT_ID IN
                             (SELECT CONTACT_ID FROM CREDITNOTES)
                      OR CONTACT_ID IN
                             (SELECT CONTACT_ID FROM PROJECTS))""")


def _delete_vatcorrections(cursor, to_date):
    "Deletes vat corrections older than to_date"
    # Delete the vat corrections
    cursor.execute("""DELETE FROM VATCORRECTIONS WHERE VATCORRECTION_ID IN
                      (SELECT VATCORRECTION_ID FROM VATCORRECTIONS, TRANSACTIONS
                       WHERE
                       DATESTAMP < ?
                       AND VATCORRECTIONS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID)""", (to_date,))


def _delete_vatreturns(cursor, to_date):
    "Deletes vat returns older than to_date"

    # Get VAT return created after the date - this will not be deleted
    cursor.execute("""SELECT MIN(VAT_RETURN_ID) FROM VAT_RETURNS, TRANSACTIONS
                       WHERE
                       DATESTAMP >= ?
                       AND VAT_RETURNS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID""", (to_date,))

    result = cursor.fetchone()
    if not result:
        vat_return_id = None
    else:
        vat_return_id = result[0]

    # If this is None, then no vat returns will remain, and there
    # will be no config value for "LAST_VAT_RETURN_ID"

    if not vat_return_id:
        cursor.execute("UPDATE CONFIGURATION SET CFG_ITEM = ? WHERE CFG_KEY = ?",
                            ("0", "LAST_VAT_RETURN_ID"))
    else:
        # There is a vat return after the date, set its previous value to None
        cursor.execute("""UPDATE VAT_RETURNS SET PREVIOUS_VAT_RETURN_ID = ?
                          WHERE VAT_RETURN_ID =?""", (None, vat_return_id))

    # Update vat_return_id to None in those invoicepayments referring to these vat returns
    # which have been deleted.
    cursor.execute("""UPDATE INVOICEPAYMENTS SET VAT_RETURN_ID = ? WHERE VAT_RETURN_ID IN
                      (SELECT VAT_RETURN_ID FROM VAT_RETURNS, TRANSACTIONS
                       WHERE
                       DATESTAMP < ?
                       AND VAT_RETURNS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID)""", (None, to_date))

    # Update vat_return_id to None in those creditnotes referring to these vat returns
    # which have been deleted.
    cursor.execute("""UPDATE CREDITNOTES SET VAT_RETURN_ID = ? WHERE VAT_RETURN_ID IN
                      (SELECT VAT_RETURN_ID FROM VAT_RETURNS, TRANSACTIONS
                       WHERE
                       DATESTAMP < ?
                       AND VAT_RETURNS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID)""", (None, to_date))

    # Update vat_return_id to None in those invoices referring to these vat returns
    # which have been deleted.
    cursor.execute("""UPDATE INVOICES SET VAT_RETURN_ID = ? WHERE VAT_RETURN_ID IN
                      (SELECT VAT_RETURN_ID FROM VAT_RETURNS, TRANSACTIONS
                       WHERE
                       DATESTAMP < ?
                       AND VAT_RETURNS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID)""", (None, to_date))

    # Update vat_return_id to None in those supplierinvoicepayments referring to these vat returns
    # which have been deleted.
    cursor.execute("""UPDATE SUPPLIERINVOICEPAYMENTS SET VAT_RETURN_ID = ? WHERE VAT_RETURN_ID IN
                      (SELECT VAT_RETURN_ID FROM VAT_RETURNS, TRANSACTIONS
                       WHERE
                       DATESTAMP < ?
                       AND VAT_RETURNS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID)""", (None, to_date))

    # Update vat_return_id to None in those supplier creditnotes referring to these vat returns
    # which have been deleted.
    cursor.execute("""UPDATE SUPPLIERCREDITNOTES SET VAT_RETURN_ID = ? WHERE VAT_RETURN_ID IN
                      (SELECT VAT_RETURN_ID FROM VAT_RETURNS, TRANSACTIONS
                       WHERE
                       DATESTAMP < ?
                       AND VAT_RETURNS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID)""", (None, to_date))

    # Update vat_return_id to None in those supplierinvoices referring to these vat returns
    # which have been deleted.
    cursor.execute("""UPDATE SUPPLIERINVOICES SET VAT_RETURN_ID = ? WHERE VAT_RETURN_ID IN
                      (SELECT VAT_RETURN_ID FROM VAT_RETURNS, TRANSACTIONS
                       WHERE
                       DATESTAMP < ?
                       AND VAT_RETURNS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID)""", (None, to_date))

    # Delete the vat returns
    cursor.execute("""DELETE FROM VAT_RETURNS WHERE VAT_RETURN_ID IN
                      (SELECT VAT_RETURN_ID FROM VAT_RETURNS, TRANSACTIONS
                       WHERE
                       DATESTAMP < ?
                       AND VAT_RETURNS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID)""", (to_date,))


def _delete_companies(cursor):
    "Delete past records of this company not referenced by anything"
    # Get current company id - this must not be deleted
    cursor.execute("SELECT CFG_ITEM FROM CONFIGURATION WHERE CFG_KEY = ?", ("COMPANY_ID",))
    result = cursor.fetchone()
    company_id = int(result[0])

    cursor.execute("""DELETE FROM COMPANIES WHERE
                       COMPANY_ID <> ?
                       AND COMPANY_ID NOT IN
                     (SELECT COMPANY_ID FROM COMPANIES
                      WHERE COMPANY_ID IN
                             (SELECT COMPANY_ID FROM VAT_RETURNS)
                      OR COMPANY_ID IN
                             (SELECT COMPANY_ID FROM PURCHASEORDERS)
                      OR COMPANY_ID IN
                             (SELECT COMPANY_ID FROM MAKEPAYMENTS)
                      OR COMPANY_ID IN
                             (SELECT COMPANY_ID FROM INVOICES)
                      OR COMPANY_ID IN
                             (SELECT COMPANY_ID FROM CUSTOMERPAYMENTS)
                      OR COMPANY_ID IN
                             (SELECT COMPANY_ID FROM CREDITNOTES))""", (company_id,))


def _delete_addresses(cursor):
    "delete addresses not referenced by anything"
    cursor.execute("""DELETE FROM ADDRESSES WHERE
                       VISIBLE = 0
                       AND ADDRESS_ID NOT IN
                     (SELECT ADDRESS_ID FROM ADDRESSES
                      WHERE ADDRESS_ID IN
                             (SELECT SUPPLIER_ADDRESS_ID FROM PURCHASEORDERS)
                      OR ADDRESS_ID IN
                             (SELECT INVOICE_ADDRESS_ID FROM PURCHASEORDERS)
                      OR ADDRESS_ID IN
                             (SELECT DELIVERY_ADDRESS_ID FROM PURCHASEORDERS)
                      OR ADDRESS_ID IN
                             (SELECT SUPPLIER_ADDRESS_ID FROM MAKEPAYMENTS)
                      OR ADDRESS_ID IN
                             (SELECT CUSTOMER_ADDRESS_ID FROM INVOICES)
                      OR ADDRESS_ID IN
                             (SELECT PAYMENT_ADDRESS_ID FROM INVOICES)
                      OR ADDRESS_ID IN
                             (SELECT CUSTOMER_ADDRESS_ID FROM CUSTOMERPAYMENTS)
                      OR ADDRESS_ID IN
                             (SELECT CUSTOMER_ADDRESS_ID FROM CREDITNOTES)
                      OR ADDRESS_ID IN
                             (SELECT ADDRESS_ID FROM COMPANIES))""")


def _delete_customers(cursor):
    "delete past records of customers not referenced by anything"
    cursor.execute("""DELETE FROM CUSTOMERS WHERE
                       VISIBLE = 0
                       AND CUSTOMER_ID NOT IN
                     (SELECT CUSTOMER_ID FROM CUSTOMERS
                      WHERE CUSTOMER_ID IN
                             (SELECT CUSTOMER_ID FROM CONTACTS)
                      OR CUSTOMER_ID IN
                             (SELECT CUSTOMER_ID FROM ADDRESSES)
                      OR CUSTOMER_ID IN
                             (SELECT CUSTOMER_ID FROM INVOICES)
                      OR CUSTOMER_ID IN
                             (SELECT CUSTOMER_ID FROM CUSTOMERPAYMENTS)
                      OR CUSTOMER_ID IN
                             (SELECT CUSTOMER_ID FROM CREDITNOTES))""")


def _delete_suppliers(cursor):
    "delete past records of suppliers not referenced by anything"
    cursor.execute("""DELETE FROM SUPPLIERS WHERE
                       VISIBLE = 0
                       AND SUPPLIER_ID NOT IN
                     (SELECT SUPPLIER_ID FROM SUPPLIERS
                      WHERE SUPPLIER_ID IN
                             (SELECT SUPPLIER_ID FROM CONTACTS)
                      OR SUPPLIER_ID IN
                             (SELECT SUPPLIER_ID FROM ADDRESSES)
                      OR SUPPLIER_ID IN
                             (SELECT SUPPLIER_ID FROM PURCHASEORDERS)
                      OR SUPPLIER_ID IN
                             (SELECT SUPPLIER_ID FROM SUPPLIERINVOICES)
                      OR SUPPLIER_ID IN
                             (SELECT SUPPLIER_ID FROM MAKEPAYMENTS)
                      OR SUPPLIER_ID IN
                             (SELECT SUPPLIER_ID FROM SUPPLIERCREDITNOTES))""")


def _delete_ledger_entries(cursor, to_date, all_accounts):
    "Delete ledger entries older than to_date"
    for acc in all_accounts:
        ledger = "LEDGER_%i" % acc.id

        sql_string = """DELETE FROM %s WHERE LEDGER_ID IN
                        (SELECT LEDGER_ID FROM %s, TRANSACTIONS
                         WHERE
                         DATESTAMP < ?
                         AND %s.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID)""" % (ledger, ledger, ledger)

        cursor.execute(sql_string, (to_date,))


        # And update the gridrow value for the remaining entries
        sql_string = "SELECT MIN(GRIDROW) FROM %s" % ledger
        cursor.execute(sql_string)
        result = cursor.fetchone()
        # gridrow start at zero - at this point they start from 1 as we are going to
        # insert the carry forward entry at point 0
        if not result:
            continue
        if result[0] == None:
            continue
        elif result[0] == 0:
            sql_string = "UPDATE %s SET GRIDROW = GRIDROW+1" % ledger
        elif result[0] == 1:
            continue
        else:
            # result[0] must be greater than 1
            sql_string = "UPDATE %s SET GRIDROW = GRIDROW-%s" % (ledger, result[0] - 1)
        cursor.execute(sql_string)


def _delete_transactions(cursor, to_date):
    "Delete all transactions older than to_date"

    cursor.execute("""DELETE FROM TRANSACTIONACCOUNTS
                      WHERE TRANSACTIONACCOUNT_ID IN
                      (SELECT TRANSACTIONACCOUNT_ID FROM TRANSACTIONACCOUNTS, TRANSACTIONS
                       WHERE
                       DATESTAMP < ?
                       AND TRANSACTIONACCOUNTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID)""", (to_date,))

    cursor.execute("""DELETE FROM TRANSACTIONS WHERE DATESTAMP < ?""", (to_date,))

    # And update the gridrow value for the remaining transactions
    cursor.execute("SELECT MIN(GRIDROW) FROM TRANSACTIONS")
    result = cursor.fetchone()
    # gridrow start at zero - at this point they start from 1 as we are going to
    # insert the carry forward entry at point 0
    if not result:
        return
    if result[0] == None:
        return
    elif result[0] == 0:
        sql_string = "UPDATE TRANSACTIONS SET GRIDROW = GRIDROW+1"
    elif result[0] == 1:
        return
    else:
        # result[0] must be greater than 1
        sql_string = "UPDATE TRANSACTIONS SET GRIDROW = GRIDROW-%s" % (result[0] - 1)
    cursor.execute(sql_string)


def _carried_forwards(cursor, to_date, all_accounts, all_balances):
    "Carry forwards from openning balance to each account"
    # Create a new transaction

    one_day = datetime.timedelta(days=1)
    datestamp = to_date - one_day

    new_transaction = [ None,                 # transaction_id
                        datestamp,            # datestamp day before the to_date
                        "Carried forward",    # Description
                        12,                   # System 12 - a carry forward transaction
                        0 ]                   # Gridrow set to zero (first entry)

    cursor.execute("""INSERT INTO TRANSACTIONS
                     (TRANSACTION_ID, DATESTAMP, DESCRIPTION, SYSTEM, GRIDROW)
                     VALUES (?, ?, ?, ?, ?)""", new_transaction)
    cursor.execute("SELECT last_insert_rowid()")
    result = cursor.fetchone()
    transaction_id = result[0]

    # For each Account/Ledger insert a pointer with a link to this transaction id
    # and with gridrow 0

    for key,acc in enumerate(all_accounts):
        if all_balances[key] >= D("0.00"):
            debit = all_balances[key]
            credit = D("0.00")
        else:
            debit = D("0.00")
            credit = abs(all_balances[key])

        ledger = "LEDGER_%i" % acc.id
        # Create the ledger entry
        ledger_entry = [
                      None,                   # Each ledger id will be automatically incremented
                      transaction_id,         # references the transaction id found above
                      0,                      # gridrow set to 0 (first entry)
                      str(debit),             # the debit
                      str(credit),            # the credit
                      str(all_balances[key])  # balance
                      ]

        sql_string = """INSERT INTO %s
                            (LEDGER_ID,
                             TRANSACTION_ID,
                             GRIDROW,
                             DEBIT,
                             CREDIT,
                             BALANCE)
                              VALUES (?, ?, ?, ?, ?, ?)""" % ledger

        cursor.execute(sql_string, ledger_entry)
        # Insert every account into transaction accounts
        cursor.execute("""INSERT INTO TRANSACTIONACCOUNTS
                             (TRANSACTIONACCOUNT_ID, TRANSACTION_ID, ACCOUNT_ID)
                              VALUES (?, ?, ?)""", (None, transaction_id, acc.id))

