####### VATStuff #######
#
# delete_transaction.py  - functions to delete a transaction
#
# Version : 0.6
# Date : 20100716
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009, 2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#

"functions to delete a transaction - used by the Undo button"

import decimal

from vatstuff_packages.vs_utilities import *



from vatstuff_packages.vs_database.transactions import transaction_sort

D = decimal.Decimal   # D used to avoid typing


def delete_vat_scheme():
    "Deletes the current vat scheme"
    
    if not vs_cfg["WRITE_ACCESS"]:
        # Another process has locked the database
        raise VS_ANOTHER_PROCESS, "Deletion cancelled - another process has locked this database"

    vatscheme = VATScheme()
    vatscheme_id = vatscheme.id

    conn = None
    try:
        conn, cursor = vs_connect()

        # Delete the vatscheme
        cursor.execute("DELETE FROM VAT_SCHEMES WHERE VAT_SCHEME_ID = ?", (vatscheme_id,))

        # now find new current vat scheme
        cursor.execute("SELECT MAX(VAT_SCHEME_ID) FROM VAT_SCHEMES")
        result = cursor.fetchone()
        if not result:
            previous_vatscheme_id = None
        else:
            previous_vatscheme_id = result[0]

        cursor.execute("UPDATE CONFIGURATION SET CFG_ITEM = ? WHERE CFG_KEY = ?",
                                            (previous_vatscheme_id, "VAT_SCHEME_ID"))
        conn.commit()
    finally:
        if conn:
            conn.close()


def delete_last_transaction(transaction):
    """Deletes the last transaction and associated items, such as invoice etc.,
       returns the transaction id of the transaction deleted"""

    transaction_id      = transaction.id
    transaction_system  = transaction.system
    trans_list          = transaction.trans_list

    if not vs_cfg["WRITE_ACCESS"]:
        # Another process has locked the database
        raise VS_ANOTHER_PROCESS, "Deletion cancelled - another process has locked this database" 

    conn = None
    try:
        conn, cursor = vs_connect()

        cursor.execute("SELECT MAX(TRANSACTION_ID) FROM TRANSACTIONS")
        result = cursor.fetchone()

        # This ensures the correct transaction is being deleted
        assert transaction_id == result[0]

        if (transaction_system == 0) or (transaction_system == 1):
            # This is a 'move money' or 'openning balance' transaction
            pass
        elif transaction_system == 2:
            # This is a purchase order transaction
            _delete_purchaseorder(cursor, transaction_id)
        elif transaction_system == 3:
            # This is a payment to a supplier
            _delete_makepayment(cursor, transaction_id)
        elif transaction_system == 4:
            # This is a suppliers credit note
            _delete_suppliercreditnote(cursor, transaction_id)
        elif transaction_system == 5:
            # This is an invoice
            _delete_invoice(cursor, transaction_id)
        elif transaction_system == 6:
            # This is a payment from a customer
            _delete_payment(cursor, transaction_id)
        elif transaction_system == 7:
            # This is an issued creditnote
            _delete_creditnote(cursor, transaction_id)
        elif transaction_system == 8:
            # This is a close order transaction
            _delete_closeorder(cursor, transaction_id)
        elif transaction_system == 9:
            # This is a suppliers invoice received transaction
            _delete_supplierinvoice(cursor, transaction_id)
        elif transaction_system == 10:
            # This is a VAT submission transaction
            _delete_vat(cursor, transaction_id)
        elif transaction_system == 11:
            # This is a VAT correction transaction
            _delete_vatcorrection(cursor, transaction_id)
        else:
            raise VS_ERROR, "Transaction does not have a recognised system value"

        # and delete the transactions in the tran_list
        _delete_transaction(cursor, transaction_id, trans_list)
            
        conn.commit()
    finally:
        if conn:
            conn.close()
    info_log("Transaction: %i deleted" % transaction_id)
    return transaction_id


def delete_quick_purchase(makepayment):
    "This deletes a payment, supplier invoice and purchase order in one go"

    if not vs_cfg["WRITE_ACCESS"]:
        # Another process has locked the database
        raise VS_ANOTHER_PROCESS, "Deletion cancelled - another process has locked this database"

    last_transaction = Transaction.get_last()
    assert last_transaction == makepayment.transaction

    # Get the transaction id's for the purchase order, invoice and payment

    makepayment_transaction_id = last_transaction.id
    makepayment_trans_list     = last_transaction.trans_list

    purchaseorder                = makepayment.purchaseorder
    purchaseorder_transaction_id = purchaseorder.transaction.id
    purchaseorder_trans_list     = purchaseorder.transaction.trans_list

    supplier_invoice_list = purchaseorder.get_invoices()
    assert len(supplier_invoice_list) == 1

    supplierinvoice                = supplier_invoice_list[0]
    supplierinvoice_transaction_id = supplierinvoice.transaction.id
    supplierinvoice_trans_list     = supplierinvoice.transaction.trans_list

    # assert the three transactions are consecutive
    assert makepayment_transaction_id == supplierinvoice_transaction_id + 1
    assert supplierinvoice_transaction_id == purchaseorder_transaction_id + 1

    # So this is a valid quick purchase, delete it
    conn = None
    try:
        conn, cursor = vs_connect()

        # delete the payment
        _delete_makepayment(cursor, makepayment_transaction_id)
        _delete_transaction(cursor, makepayment_transaction_id, makepayment_trans_list)

        # delete the invoice
        _delete_supplierinvoice(cursor, supplierinvoice_transaction_id)
        _delete_transaction(cursor, supplierinvoice_transaction_id, supplierinvoice_trans_list)

        # delete the purchase order
        _delete_purchaseorder(cursor, purchaseorder_transaction_id)
        _delete_transaction(cursor, purchaseorder_transaction_id, purchaseorder_trans_list)
        conn.commit()
    finally:
        if conn:
            conn.close()


def _delete_transaction(cursor, transaction_id, trans_list):
    "Deletes the transaction from the database and from the accounts"

    # For each account in trans_list, delete the transaction
    ledger_list = []
    for trans in trans_list:
        account = trans[0]
        ledger_table_name = "LEDGER_%i" % account.id
        # Has this ledger already been done?
        if ledger_table_name in ledger_list:
            continue
        else:
            ledger_list.append(ledger_table_name)
        # Get the ledger_id for each row in this ledger - note there may be multiple entries
        # in this ledger with this transaction id, we have to delete them all
        sql_string1 = "SELECT LEDGER_ID FROM %s WHERE TRANSACTION_ID = ?" % ledger_table_name
        cursor.execute(sql_string1, (transaction_id,))
        result = cursor.fetchall()
        # get a list of ledger id's (not a tuple of tuples!)
        ledger_id_list = [ item[0] for item in result ]
        if not ledger_id_list:
            continue
        for ledger_id in ledger_id_list:
            # get the gridrow of this entry
            sql_string2 = "SELECT GRIDROW, BALANCE, DEBIT, CREDIT FROM %s WHERE LEDGER_ID = ?" % ledger_table_name
            cursor.execute(sql_string2, (ledger_id,))
            result = cursor.fetchone()
            gridrow = result[0]
            balance = D(result[1])
            amount = D(result[2]) - D(result[3])
            # Delete the entry
            sql_string3 = "DELETE FROM %s WHERE LEDGER_ID = ?" % ledger_table_name
            cursor.execute(sql_string3, (ledger_id,))
            # update the balance of all later entries

            # Sort the ledger
            # Decrement the gridrow of all ledger entries with later gridrows
            sql_string4 = "UPDATE %s SET GRIDROW = GRIDROW-1 WHERE GRIDROW > ?" % ledger_table_name
            cursor.execute(sql_string4, (gridrow,))

            sql_string5 = """SELECT BALANCE, LEDGER_ID
                             FROM %s
                             WHERE GRIDROW >= ?
                             ORDER BY GRIDROW""" % ledger_table_name

            sql_string6 = "UPDATE %s SET BALANCE = ? WHERE LEDGER_ID = ?" % ledger_table_name

            cursor.execute(sql_string5, (gridrow,))
            # new_balance is the balance minus the amount - i.e. the balance prior to this ledger transaction
            new_balance = balance-amount
            for ledger_entry in cursor.fetchall():
                # subtract the deleted amount from each balance
                new_balance = D(ledger_entry[0]) - amount
                # insert balance
                cursor.execute(sql_string6, (str(new_balance), ledger_entry[1]))

            # and update the account
            cursor.execute("""UPDATE ACCOUNTS
                                  SET BALANCE = ?
                                  WHERE ACCOUNT_ID = ?""", (str(new_balance), account.id))

        # If the account has system 1, and its ledger has no entries, the system flag can change to zero
        cursor.execute("SELECT SYSTEM FROM ACCOUNTS WHERE ACCOUNT_ID = ?", (account.id,))
        result = cursor.fetchone()
        if result[0] == 1:
            cursor.execute("SELECT COUNT(*) FROM %s" % ledger_table_name)
            count_of_transactions = cursor.fetchone()
            if count_of_transactions[0] == 0:
                cursor.execute("UPDATE ACCOUNTS SET SYSTEM = 0 WHERE ACCOUNT_ID = ?", (account.id,))

    # Remove reference to the transaction from TRANSACTIONACCOUNTS
    cursor.execute("DELETE FROM TRANSACTIONACCOUNTS WHERE TRANSACTION_ID = ?", (transaction_id,))

    # Get the gridrow of the transaction
    cursor.execute("SELECT GRIDROW FROM TRANSACTIONS WHERE TRANSACTION_ID = ?", (transaction_id,))
    result = cursor.fetchone()
    gridrow = result[0]

    # Now delete the transaction
    cursor.execute("DELETE FROM TRANSACTIONS WHERE TRANSACTION_ID = ?", (transaction_id,))

    # Sort the transactions
    # Decrement the gridrow of all transactions with later gridrows
    cursor.execute("UPDATE TRANSACTIONS SET GRIDROW = GRIDROW-1 WHERE GRIDROW > ?", (gridrow,))


def _delete_purchaseorder(cursor, transaction_id):
    "Deletes the purchase order from the database"

    cursor.execute("SELECT PURCHASEORDER_ID, GRIDROW FROM PURCHASEORDERS WHERE TRANSACTION_ID = ?", (transaction_id,))
    result = cursor.fetchone()
    purchaseorder_id = result[0]
    gridrow = result[1]

    # Delete from the table of ordered items
    cursor.execute("DELETE FROM ORDEREDITEMS WHERE PURCHASEORDER_ID = ?", (purchaseorder_id,))

    # Now delete the purchase order
    cursor.execute("DELETE FROM PURCHASEORDERS WHERE PURCHASEORDER_ID = ?", (purchaseorder_id,))

    # Sort the purchase orders
    # Decrement the gridrow of all purchase orders with later gridrows
    cursor.execute("UPDATE PURCHASEORDERS SET GRIDROW = GRIDROW-1 WHERE GRIDROW > ?", (gridrow,))

    # Get the maximum purchase order number
    cursor.execute("SELECT MAX(ORDER_NUMBER) FROM PURCHASEORDERS")
    result = cursor.fetchone()
    if not result[0]:
        max_order_number = "0"
    else:
        max_order_number = str(result[0])
    # Set this value in "LAST_ORDER_NUMBER" of the configuration table

    cursor.execute("UPDATE CONFIGURATION SET CFG_ITEM = ? WHERE CFG_KEY = ?", (max_order_number, "LAST_ORDER_NUMBER"))
        


def _delete_makepayment(cursor, transaction_id):
    "Deletes the payment to the supplier"

    cursor.execute("SELECT MAKEPAYMENT_ID, PURCHASEORDER_ID, AMOUNT FROM MAKEPAYMENTS WHERE TRANSACTION_ID = ?",
                   (transaction_id,))
    result = cursor.fetchone()
    makepayment_id   = result[0]
    purchaseorder_id = result[1]
    amount           = D(result[2])

    # Get the currently set 'paid' on the purchase order

    cursor.execute("SELECT PAID, STATUS FROM PURCHASEORDERS WHERE PURCHASEORDER_ID = ?",
                   (purchaseorder_id,))
    result = cursor.fetchone()
    paid            = D(result[0])
    status          = result[1]

    # status should be 1 or 2
    assert (status == 1) or (status == 2)

    # subtract this payment
    new_paid = paid - amount 

    # insert the new paid amount, and set status to 1
    cursor.execute("UPDATE PURCHASEORDERS SET PAID = ?, STATUS = ? WHERE PURCHASEORDER_ID = ?",
                   (str(new_paid), 1, purchaseorder_id))


    # This make payment could be for one or more supplierinvoices, get the list of invoices
    # and the cost and vat paid to that invoice by this payment
    cursor.execute("SELECT SUPPLIERINVOICE_ID, COST, VAT FROM SUPPLIERINVOICEPAYMENTS WHERE MAKEPAYMENT_ID = ?",
                   (makepayment_id,))
    result = cursor.fetchall()
    # result is a list of lists: [supplierinvoice_id, cost, vat]

    for supplierinvoice_id, cost, vat in result:
        # Get the currently set 'paid' on the invoice

        cursor.execute("SELECT PAID FROM SUPPLIERINVOICES WHERE SUPPLIERINVOICE_ID = ?", (supplierinvoice_id,))
        paidresult = cursor.fetchone()
        paid            = D(paidresult[0])

        # subtract this payment
        new_paid = paid - D(cost) - D(vat) 

        # insert the new paid amount, and set OPEN to 1, to indicate the supplierinvoice is open
        cursor.execute("UPDATE SUPPLIERINVOICES SET PAID = ?, OPEN = ? WHERE SUPPLIERINVOICE_ID = ?",
                       (str(new_paid), 1, supplierinvoice_id))

    # delete the links in SUPPLIERINVOICEPAYMENTS
    cursor.execute("DELETE FROM SUPPLIERINVOICEPAYMENTS WHERE MAKEPAYMENT_ID = ?", (makepayment_id,))

    # Now delete the payment
    cursor.execute("DELETE FROM MAKEPAYMENTS WHERE MAKEPAYMENT_ID = ?", (makepayment_id,))



def _delete_suppliercreditnote(cursor, transaction_id):
    "Deletes the suppliers credit note"

    cursor.execute("""SELECT SUPPLIERCREDITNOTE_ID,
                             PURCHASEORDER_ID,
                             SUPPLIERINVOICE_ID,
                             COST,
                             VAT,
                             STATUS
                       FROM SUPPLIERCREDITNOTES WHERE TRANSACTION_ID = ?""", (transaction_id,))
    result = cursor.fetchone()
    suppliercreditnote_id   = result[0]
    purchaseorder_id        = result[1]
    supplierinvoice_id      = result[2]
    cost                    = D(result[3])
    vat                     = D(result[4])
    status                  = result[5]   # 0 if against a purchase order, 1 if it comes with a payment

    # If the cost and vat on this suppliercreditnote have reduced the invoice_cost and invoice_vat on
    # a purchaseorder - undo the change

    if not status :
        cursor.execute("""SELECT INVOICE_VAT,
                                 INVOICE_COST
                       FROM PURCHASEORDERS WHERE PURCHASEORDER_ID = ?""", (purchaseorder_id,))
        result = cursor.fetchone()
        invoice_vat   = D(result[0])
        invoice_cost  = D(result[1])

        invoice_vat  += vat
        invoice_cost += cost

        # and insert these new totals into the purchase order
        cursor.execute("""UPDATE PURCHASEORDERS
                          SET INVOICE_VAT = ?, INVOICE_COST = ?
                          WHERE PURCHASEORDER_ID = ?""",
                          (str(invoice_vat), str(invoice_cost), purchaseorder_id))

        # Also update the supplier invoice paid value, and set its status to open (value 1)
        cursor.execute("""SELECT PAID
                       FROM SUPPLIERINVOICES WHERE SUPPLIERINVOICE_ID = ?""", (supplierinvoice_id,))
        result = cursor.fetchone()
        paid   = D(result[0])
        new_paid = paid - cost - vat

        # and insert these new values into the supplierinvoice
        cursor.execute("""UPDATE SUPPLIERINVOICES
                          SET PAID = ?, OPEN = ?
                          WHERE SUPPLIERINVOICE_ID = ?""",
                          (str(new_paid), 1, supplierinvoice_id))


    # Delete the supplier credit note
    cursor.execute("DELETE FROM SUPPLIERCREDITNOTES WHERE SUPPLIERCREDITNOTE_ID = ?", (suppliercreditnote_id,))


def _delete_invoice(cursor, transaction_id):
    "Deletes the invoice from the database"

    cursor.execute("SELECT INVOICE_ID, GRIDROW FROM INVOICES WHERE TRANSACTION_ID = ?", (transaction_id,))
    result = cursor.fetchone()
    invoice_id = result[0]
    gridrow = result[1]

    # Delete from the table of sold items
    cursor.execute("DELETE FROM SOLDITEMS WHERE INVOICE_ID = ?", (invoice_id,))

    # Now delete the invoice
    cursor.execute("DELETE FROM INVOICES WHERE INVOICE_ID = ?", (invoice_id,))

    # Sort the invoices
    # Decrement the gridrow of all invoices with later gridrows
    cursor.execute("UPDATE INVOICES SET GRIDROW = GRIDROW-1 WHERE GRIDROW > ?", (gridrow,))

    # Get the maximum invoice number
    cursor.execute("SELECT MAX(INVOICE_NUMBER) FROM INVOICES")
    result = cursor.fetchone()
    if not result[0]:
        max_invoice_number = "0"
    else:
        max_invoice_number = str(result[0])
    # Set this value in "LAST_INVOICE_NUMBER" of the configuration table

    cursor.execute("UPDATE CONFIGURATION SET CFG_ITEM = ? WHERE CFG_KEY = ?",
                                         (max_invoice_number, "LAST_INVOICE_NUMBER"))
        

def _delete_payment(cursor, transaction_id):
    "Deletes the customer payment from the database"

    cursor.execute("SELECT CUSTOMERPAYMENT_ID, AMOUNT FROM CUSTOMERPAYMENTS WHERE TRANSACTION_ID = ?",
                   (transaction_id,))
    result = cursor.fetchone()
    payment_id = result[0]
    amount     = D(result[1])

    # This customer payment could be for one or more invoices, get the list of invoices
    # and the cost and vat paid to that invoice by this payment
    cursor.execute("SELECT INVOICE_ID, COST, VAT FROM INVOICEPAYMENTS WHERE CUSTOMERPAYMENT_ID = ?",
                   (payment_id,))
    result = cursor.fetchall()
    # result is a list of lists: [invoice_id, cost, vat]
    for invoice_id, cost, vat in result:
        # Get the currently set 'paid' on the invoice

        cursor.execute("SELECT PAID FROM INVOICES WHERE INVOICE_ID = ?", (invoice_id,))
        paidresult = cursor.fetchone()
        paid       = D(paidresult[0])

        # subtract this payment
        new_paid = paid - D(cost) - D(vat) 

        # insert the new paid amount, and set OPEN to 1, to indicate the invoice is open
        cursor.execute("UPDATE INVOICES SET PAID = ?, OPEN = ? WHERE INVOICE_ID = ?",
                       (str(new_paid), 1, invoice_id))

    # delete the links in INVOICEPAYMENTS
    cursor.execute("DELETE FROM INVOICEPAYMENTS WHERE CUSTOMERPAYMENT_ID = ?", (payment_id,))

    # Now delete the payment
    cursor.execute("DELETE FROM CUSTOMERPAYMENTS WHERE CUSTOMERPAYMENT_ID = ?", (payment_id,))


def _delete_creditnote(cursor, transaction_id):
    "deletes the creditnote from the database"

    cursor.execute("""SELECT CREDITNOTE_ID,
                             INVOICE_ID,
                             GRIDROW,
                             VAT,
                             COST,
                             STATUS FROM CREDITNOTES WHERE TRANSACTION_ID = ?""", (transaction_id,))
    result = cursor.fetchone()
    creditnote_id = result[0]
    invoice_id = result[1]
    gridrow = result[2]
    vat = D(result[3])
    cost = D(result[4])
    status = result[5]

    if not status:   # note - if status == 0 this is a payment only creditnote, there will be no invoice
        # Get the currently set 'paid' on the invoice
        cursor.execute("SELECT PAID FROM INVOICES WHERE INVOICE_ID = ?", (invoice_id,))
        result = cursor.fetchone()
        paid   = D(result[0])

        # Take off the vat and cost of this creditnote from the paid value in the invoice
        new_paid = paid - vat - cost

        # and also mark the invoice as open and not cancelled
  
        cursor.execute("UPDATE INVOICES SET PAID = ?, OPEN = ?, CANCELLED = ? WHERE INVOICE_ID = ?",
                       (str(new_paid), 1, 0, invoice_id))

    # Delete from the table of sold items
    cursor.execute("DELETE FROM CREDITNOTEITEMS WHERE CREDITNOTE_ID = ?", (creditnote_id,))

    # Now delete the credit note
    cursor.execute("DELETE FROM CREDITNOTES WHERE CREDITNOTE_ID = ?", (creditnote_id,))

    # Sort the credit notes
    # Decrement the gridrow of all invoices with later gridrows
    cursor.execute("UPDATE CREDITNOTES SET GRIDROW = GRIDROW-1 WHERE GRIDROW > ?", (gridrow,))

    # Get the maximum credit note number
    cursor.execute("SELECT MAX(CREDITNOTE_NUMBER) FROM CREDITNOTES")
    result = cursor.fetchone()
    if not result[0]:
        max_creditnote_number = "0"
    else:
        max_creditnote_number = str(result[0])
    # Set this value in "LAST_CREDITNOTE_NUMBER" of the configuration table

    cursor.execute("UPDATE CONFIGURATION SET CFG_ITEM = ? WHERE CFG_KEY = ?",
                                         (max_creditnote_number, "LAST_CREDITNOTE_NUMBER"))
        

def _delete_closeorder(cursor, transaction_id):
    "This deletes a 'close order'"

    # Get the closedorder this transaction refers to
    cursor.execute("""SELECT CLOSEDORDER_ID FROM CLOSEDORDERS WHERE TRANSACTION_ID = ?""", (transaction_id,))
    result = cursor.fetchone()
    closedorder_id = result[0]

    # get the order
    cursor.execute("SELECT PURCHASEORDER_ID FROM PURCHASEORDERS WHERE CLOSEDORDER_ID = ?",
                   (closedorder_id,))
    result = cursor.fetchone()
    if result:
        purchaseorder_id = result[0]
    else:
        purchaseorder_id = None

    if purchaseorder_id:
        status = 0  # if no invoices received on order

        cursor.execute("SELECT COUNT(*) FROM SUPPLIERINVOICES WHERE PURCHASEORDER_ID = ?",
                       (purchaseorder_id,))
        result = cursor.fetchone()
        if result[0] > 0:
            status = 1     # invoice have been received on order

        cursor.execute("""UPDATE PURCHASEORDERS SET 
                             STATUS = ?,
                             CLOSEDORDER_ID = ?
                             WHERE PURCHASEORDER_ID = ?""", (status, None, purchaseorder_id))

    # Now delete the closedorder
    cursor.execute("DELETE FROM CLOSEDORDERS WHERE CLOSEDORDER_ID = ?", (closedorder_id,))


def _delete_supplierinvoice(cursor, transaction_id):
    "This deletes a suppliers invoice received"

    cursor.execute("""SELECT SUPPLIERINVOICE_ID,
                             PURCHASEORDER_ID,
                             VAT,
                             COST
                       FROM SUPPLIERINVOICES WHERE TRANSACTION_ID = ?""", (transaction_id,))
    result = cursor.fetchone()
    supplierinvoice_id = result[0]
    purchaseorder_id   = result[1]
    vat                = D(result[2])
    cost               = D(result[3])

    # Get the invoice_cost and invoice_vat from the purchase order

    cursor.execute("""SELECT INVOICE_VAT,
                             INVOICE_COST
                      FROM PURCHASEORDERS WHERE PURCHASEORDER_ID = ?""", (purchaseorder_id,))
    result = cursor.fetchone()
    invoice_vat   = D(result[0])
    invoice_cost  = D(result[1])

    invoice_vat  -= vat
    invoice_cost -= cost

    # Delete the supplier invoice
    cursor.execute("DELETE FROM SUPPLIERINVOICES WHERE SUPPLIERINVOICE_ID = ?", (supplierinvoice_id,))

    # Count have any other supplier invoices been made on the order?

    status = 0  # if no invoices received on order

    cursor.execute("SELECT COUNT(*) FROM SUPPLIERINVOICES WHERE PURCHASEORDER_ID = ?",
                   (purchaseorder_id,))
    result = cursor.fetchone()
    if result[0] > 0:
        status = 1     # invoice have been received on order

    # update status, invoice cost and invoice vat on the purchase order
    cursor.execute("""UPDATE PURCHASEORDERS SET 
                         STATUS = ?,
                         INVOICE_VAT = ?,
                         INVOICE_COST = ?
                          WHERE PURCHASEORDER_ID = ?""",
                          (status, str(invoice_vat), str(invoice_cost), purchaseorder_id))


def _delete_vat(cursor, transaction_id):
    "Delete VAT submission"

    cursor.execute("""SELECT VAT_RETURN_ID,
                             VAT_SCHEME_ID,
                             PREVIOUS_VAT_RETURN_ID,
                             FROM_DATE,
                             TO_DATE
                       FROM VAT_RETURNS WHERE TRANSACTION_ID = ?""", (transaction_id,))
    result = cursor.fetchone()
    vatreturn_id, vatscheme_id, previous_vatreturn_id, from_date, to_date = result

    cursor.execute("""SELECT METHOD FROM VAT_SCHEMES WHERE VAT_SCHEME_ID = ?""", (vatscheme_id,))
    result = cursor.fetchone()
    method = result[0]

    # set the vat scheme date back to the from date

    cursor.execute("UPDATE VAT_SCHEMES SET STARTDATE = ? WHERE VAT_SCHEME_ID = ?",
                     (from_date, vatscheme_id))

    # and unset the VAT_RETURN_ID's
    cursor.execute("""UPDATE INVOICES SET VAT_RETURN_ID = ?
                         WHERE VAT_RETURN_ID = ?""", (None, vatreturn_id))

    cursor.execute("""UPDATE SUPPLIERINVOICES SET VAT_RETURN_ID = ?
                         WHERE VAT_RETURN_ID = ?""", (None, vatreturn_id))

    cursor.execute("""UPDATE CREDITNOTES SET VAT_RETURN_ID = ?
                         WHERE VAT_RETURN_ID = ?""", (None, vatreturn_id))

    cursor.execute("""UPDATE SUPPLIERCREDITNOTES SET VAT_RETURN_ID = ?
                         WHERE VAT_RETURN_ID = ?""", (None, vatreturn_id))

    cursor.execute("""UPDATE SUPPLIERINVOICEPAYMENTS SET VAT_RETURN_ID = ?
                         WHERE VAT_RETURN_ID = ?""", (None, vatreturn_id))

    cursor.execute("""UPDATE INVOICEPAYMENTS SET VAT_RETURN_ID = ?
                         WHERE VAT_RETURN_ID = ?""", (None, vatreturn_id))


    # and delete the vat return
    cursor.execute("DELETE FROM VAT_RETURNS WHERE VAT_RETURN_ID = ?", (vatreturn_id,))

    if not previous_vatreturn_id:
        previous_vatreturn_id = "0"
    else:
        previous_vatreturn_id = str(previous_vatreturn_id)
    # Set this value in "LAST_VAT_RETURN_ID" of the configuration table

    cursor.execute("UPDATE CONFIGURATION SET CFG_ITEM = ? WHERE CFG_KEY = ?",
                            (previous_vatreturn_id, "LAST_VAT_RETURN_ID"))


def _delete_vatcorrection(cursor, transaction_id):
    "This deletes a vat correction"

    # Delete the vat correction
    cursor.execute("DELETE FROM VATCORRECTIONS WHERE TRANSACTION_ID  = ?", (transaction_id,))

