####### VATStuff #######
#
# grid_objects.py  - Provides classes inherited from wx.grid.PyGridTableBase used to create grids
#
# Version : 0.4
# Date : 20100225
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#

"""Provides classes inherited from wx.grid.PyGridTableBase used to create grid information. 
   These are used to generate cell contents dynamically as the table is being displayed,
   and hence the whole database does not have to be read and held in memory."""

import datetime
import decimal
import wx.grid

from vatstuff_packages.vs_utilities import *

D = decimal.Decimal   # D used to avoid typing


# --------- This class is used to create an Account list table - in effect to display the account ledger

class AccountGrid(wx.grid.PyGridTableBase):

    def __init__(self, account):
        wx.grid.PyGridTableBase.__init__(self)

        # Get the table name which stores these transactions
        self.ledger_table_name = account.ledger

        self.col_labels = ["Date", "Description", "Debit", "Credit", "Balance"]
        self.col_items  = ["DATESTAMP", "DESCRIPTION", "IC(DEBIT)", "IC(CREDIT)", "IC(BALANCE)"]

        # Make database connection
        self.conn, self.c = vs_connect()
        self.conn.create_function("IC", 1, self.insert_currency)

        # Flag the database is open
        self.database_open = True


    def insert_currency(self, value):
        """value is a string currency - read from the database,
           this inserts a currency symbol thousand separators and point"""
        # Change the decimal point to CP, and convert to currency string
        return to_currency_string(value.replace(".", vs_cfg["CP"]))


    def CloseConnection(self):
            self.database_open = False
            self.conn.close()

    def GetNumberCols(self):
        return len(self.col_labels)

    def GetNumberRows(self):
        if not self.database_open:
            return 0
        self.c.execute("""SELECT COUNT(*) FROM %s""" % self.ledger_table_name)
        result = self.c.fetchone()
        return result[0]

    def IsEmptyCell(self, row, col):
        return False

    def SetValue(self, row, col, value):
        pass

    def GetValue(self, row, col):
        "Get data from the database to go into row, col"
        if not self.database_open:
            return ""
        sql_string = """SELECT %s
                        FROM TRANSACTIONS, %s
                        WHERE %s.GRIDROW = %i
                        AND %s.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                        """ % (self.col_items[col],
                               self.ledger_table_name,
                               self.ledger_table_name, row,
                               self.ledger_table_name)
        self.c.execute(sql_string)
        result = self.c.fetchone()
        if result == None or result == ():
            return ""
        if col == 0:
            return result[0].strftime("%d %b %Y")
        return result[0]

    def GetColLabelValue(self, col):
        return self.col_labels[col]


    def GetRowLabelValue(self, row):
        "Returns the transaction number as the row label"
        if not self.database_open:
            return ""
        sql_string = "SELECT TRANSACTION_ID FROM %s WHERE GRIDROW = %i" % (self.ledger_table_name, row)
        self.c.execute(sql_string)
        result = self.c.fetchone()
        if result == None or result == ():
            return ""
        return str(result[0])

    def AppendRows(self, *kwds):
        print kwds
        return True


# --------- This class is used to create a transaction list table - to display all transactions

class TransactionGrid(wx.grid.PyGridTableBase):

    def __init__(self):
        wx.grid.PyGridTableBase.__init__(self)

        self.col_labels = ["Date", "Description"]
        self.col_items  = ["DATESTAMP", "DESCRIPTION"]

        # Make database connection
        self.conn, self.c = vs_connect()

        # Flag the database is open
        self.database_open = True

    def CloseConnection(self):
            self.database_open = False
            self.conn.close()

    def GetNumberCols(self):
        return len(self.col_labels)

    def GetNumberRows(self):
        if not self.database_open:
            return 0
        self.c.execute("""SELECT COUNT(*) FROM TRANSACTIONS""")
        result = self.c.fetchone()
        return result[0]

    def IsEmptyCell(self, row, col):
        return False

    def SetValue(self, row, col, value):
        pass

    def GetValue(self, row, col):
        "Get data from the database to go into row, col"
        if not self.database_open:
            return ""
        self.c.execute("""SELECT DATESTAMP, DESCRIPTION
                        FROM TRANSACTIONS
                        WHERE GRIDROW = ?""", (row,))
        result = self.c.fetchone()
        if not result:
            return ""
        if col == 0:
            return result[0].strftime("%d %b %Y")
        return result[1]

    def GetColLabelValue(self, col):
        return self.col_labels[col]

    def GetRowLabelValue(self, row):
        "Returns the transaction number as the row label"
        if not self.database_open:
            return ""
        self.c.execute("""SELECT TRANSACTION_ID
                        FROM TRANSACTIONS
                        WHERE GRIDROW = ?""", (row,))
        result = self.c.fetchone()
        if not result:
            return ""
        return str(result[0])


# --------- This class is used to create a list of purchase Orders

class OrdersGrid(wx.grid.PyGridTableBase):

    def __init__(self):
        wx.grid.PyGridTableBase.__init__(self)


        self.col_labels = ["Date", "Supplier", "Description", "Total", "Paid"]

        self.attr = wx.grid.GridCellAttr()

        self.right = wx.grid.GridCellAttr()
        self.right.SetAlignment(wx.ALIGN_RIGHT, wx.ALIGN_CENTER)

        self.cancelled = wx.grid.GridCellAttr()
        self.cancelled.SetBackgroundColour("light grey")

        self.cancelled_right = wx.grid.GridCellAttr()
        self.cancelled_right.SetBackgroundColour("light grey")
        self.cancelled_right.SetAlignment(wx.ALIGN_RIGHT, wx.ALIGN_CENTER)

        self.closed = wx.grid.GridCellAttr()
        self.closed.SetBackgroundColour("#AAFFAA")

        self.closed_right = wx.grid.GridCellAttr()
        self.closed_right.SetBackgroundColour("#AAFFAA")
        self.closed_right.SetAlignment(wx.ALIGN_RIGHT, wx.ALIGN_CENTER)


        # Make database connection
        self.conn, self.c = vs_connect()

        # Flag the database is open
        self.database_open = True


    def CloseConnection(self):
            self.database_open = False
            self.conn.close()

    def GetNumberCols(self):
        return len(self.col_labels)

    def GetNumberRows(self):
        if not self.database_open:
            return 0
        self.c.execute("""SELECT COUNT(*) FROM PURCHASEORDERS""")
        result = self.c.fetchone()
        return result[0]

    def IsEmptyCell(self, row, col):
        return False

    def SetValue(self, row, col, value):
        pass

    def GetValue(self, row, col):
        "Get data from the database to go into row, col"
        if not self.database_open:
            return ""
        if col == 0:
            self.c.execute("""SELECT DATESTAMP
                              FROM TRANSACTIONS, PURCHASEORDERS
                              WHERE PURCHASEORDERS.GRIDROW = ?
                              AND PURCHASEORDERS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID""", (row,))
            result = self.c.fetchone()
            if not result:
                return ""
            return result[0].strftime("%d %b %Y")
        elif col == 1:
            self.c.execute("""SELECT NAME
                              FROM SUPPLIERS, PURCHASEORDERS
                              WHERE PURCHASEORDERS.GRIDROW = ?
                              AND PURCHASEORDERS.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID""", (row,))
            result = self.c.fetchone()
            if not result:
                return ""
            return result[0]
        elif col == 2:
            self.c.execute("""SELECT DESCRIPTION
                              FROM TRANSACTIONS, PURCHASEORDERS
                              WHERE PURCHASEORDERS.GRIDROW = ?
                              AND PURCHASEORDERS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID""", (row,))
            result = self.c.fetchone()
            if not result:
                return ""
            return result[0]
        elif col == 3:
            "Calculates total amount from cost+vat"
            self.c.execute("""SELECT COST,VAT
                              FROM PURCHASEORDERS
                              WHERE GRIDROW = ?""", (row,))
            result = self.c.fetchone()
            if not result:
                return ""
            return to_currency_string(D(result[0]) + D(result[1]))
        elif col == 4:
            "Gets the amount paid"
            self.c.execute("""SELECT PAID
                              FROM PURCHASEORDERS
                              WHERE GRIDROW = ?""", (row,))
            result = self.c.fetchone()
            if not result:
                return ""
            return to_currency_string(D(result[0]))
        else:
            return ""


    def GetColLabelValue(self, col):
        return self.col_labels[col]


    def GetRowLabelValue(self, row):
        "Returns the order number as the row label"
        if not self.database_open:
            return ""
        self.c.execute("SELECT ORDER_NUMBER FROM PURCHASEORDERS WHERE GRIDROW = ?", (row,))
        result = self.c.fetchone()
        if not result:
            return ""
        return str(result[0])

    def GetAttr(self, row, col, kind):
        attr = self.attr
        if col == 3 or col == 4:
            attr = self.right
        if not self.database_open:
            attr.IncRef()
            return attr
        self.c.execute("SELECT STATUS FROM PURCHASEORDERS WHERE GRIDROW = ?", (row,))
        result = self.c.fetchone()
        if result:
            if result[0]<2:
                # Order open
                attr.IncRef()
                return attr
            if result[0]==5:
                # Order cancelled
                if col == 3 or col == 4:
                    attr = self.cancelled_right
                    attr.IncRef()
                    return attr
                else:
                    attr = self.cancelled
                    attr.IncRef()
                    return attr
            else:
                # Order closed
                if col == 3 or col == 4:
                    attr = self.closed_right
                    attr.IncRef()
                    return attr
                else:
                    attr = self.closed
                    attr.IncRef()
                    return attr
        attr.IncRef()
        return attr
        

# --------- This class is used to create a list of Invoices

class InvoicesGrid(wx.grid.PyGridTableBase):

    def __init__(self):
        wx.grid.PyGridTableBase.__init__(self)


        self.col_labels = ["Date", "Customer", "Description", "Total", "Outstanding"]

        self.attr = wx.grid.GridCellAttr()

        self.right = wx.grid.GridCellAttr()
        self.right.SetAlignment(wx.ALIGN_RIGHT, wx.ALIGN_CENTER)

        self.cancelled = wx.grid.GridCellAttr()
        self.cancelled.SetBackgroundColour("light grey")

        self.cancelled_right = wx.grid.GridCellAttr()
        self.cancelled_right.SetBackgroundColour("light grey")
        self.cancelled_right.SetAlignment(wx.ALIGN_RIGHT, wx.ALIGN_CENTER)

        self.closed = wx.grid.GridCellAttr()
        self.closed.SetBackgroundColour("#AAFFAA")

        self.closed_right = wx.grid.GridCellAttr()
        self.closed_right.SetBackgroundColour("#AAFFAA")
        self.closed_right.SetAlignment(wx.ALIGN_RIGHT, wx.ALIGN_CENTER)

        # Make database connection
        self.conn, self.c = vs_connect()

        # Flag the database is open
        self.database_open = True


    def CloseConnection(self):
            self.database_open = False
            self.conn.close()

    def GetNumberCols(self):
        return len(self.col_labels)

    def GetNumberRows(self):
        if not self.database_open:
            return 0
        self.c.execute("""SELECT COUNT(*) FROM INVOICES""")
        result = self.c.fetchone()
        return result[0]

    def IsEmptyCell(self, row, col):
        return False

    def SetValue(self, row, col, value):
        pass

    def GetValue(self, row, col):
        "Get data from the database to go into row, col"
        if not self.database_open:
            return ""
        if col == 0:
            # Get datestamp
            self.c.execute("""SELECT DATESTAMP
                              FROM TRANSACTIONS, INVOICES
                              WHERE INVOICES.GRIDROW = ?
                              AND INVOICES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                              """, (row,))
            result = self.c.fetchone()
            if not result:
                return ""
            return result[0].strftime("%d %b %Y")
        elif col == 1:
            # Get Customer
            self.c.execute("""SELECT NAME
                              FROM INVOICES, CUSTOMERS
                              WHERE INVOICES.GRIDROW = ?
                              AND INVOICES.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID
                              """, (row,))
            result = self.c.fetchone()
            if not result:
                return ""
            return result[0]
        elif col == 2:
            # Get Description
            self.c.execute("""SELECT DESCRIPTION
                              FROM TRANSACTIONS, INVOICES
                              WHERE INVOICES.GRIDROW = ?
                              AND INVOICES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                              """, (row,))
            result = self.c.fetchone()
            if not result:
                return ""
            return result[0]
        elif col == 3:
            # Get Total
            self.c.execute("""SELECT COST, VAT
                              FROM INVOICES
                              WHERE GRIDROW = ?""", (row,))
            result = self.c.fetchone()
            if not result:
                return ""
            return to_currency_string(D(result[0]) + D(result[1]))
        elif col == 4:
            # Gets the amount outstanding
            self.c.execute("""SELECT COST, VAT, PAID
                              FROM INVOICES
                              WHERE GRIDROW = ?""", (row,))
            result = self.c.fetchone()
            if not result:
                return ""
            return to_currency_string(D(result[0]) + D(result[1]) - D(result[2]))
        else:
            return ""

    def GetColLabelValue(self, col):
        return self.col_labels[col]


    def GetRowLabelValue(self, row):
        "Returns the invoice number as the row label"
        if not self.database_open:
            return ""
        self.c.execute("SELECT INVOICE_NUMBER FROM INVOICES WHERE GRIDROW = ?", (row,))
        result = self.c.fetchone()
        if not result:
            return ""
        return str(result[0])

    def GetAttr(self, row, col, kind):
        attr = self.attr
        if col == 3 or col == 4:
            attr = self.right
        if not self.database_open:
            attr.IncRef()
            return attr
        self.c.execute("SELECT CANCELLED FROM INVOICES WHERE GRIDROW = ?", (row,))
        result = self.c.fetchone()
        if result:
            if result[0] == 1:
                if col == 3 or col == 4:
                    attr = self.cancelled_right
                    attr.IncRef()
                    return attr
                else:
                    attr = self.cancelled
                    attr.IncRef()
                    return attr
        self.c.execute("SELECT OPEN FROM INVOICES WHERE GRIDROW = ?", (row,))
        result = self.c.fetchone()
        if result:
            if result[0] == 0:
                if col == 3 or col == 4:
                    attr = self.closed_right
                    attr.IncRef()
                    return attr
                else:
                    attr = self.closed
                    attr.IncRef()
                    return attr
        attr.IncRef()
        return attr


# --------- This class is used to create a list of Credit Notes

class CreditnotesGrid(wx.grid.PyGridTableBase):

    def __init__(self):
        wx.grid.PyGridTableBase.__init__(self)


        self.col_labels = ["Date", "Customer", "Description", "Cost", "VAT"]

        self.attr = wx.grid.GridCellAttr()

        self.right = wx.grid.GridCellAttr()
        self.right.SetAlignment(wx.ALIGN_RIGHT, wx.ALIGN_CENTER)

        # Make database connection
        self.conn, self.c = vs_connect()

        # Flag the database is open
        self.database_open = True

    def CloseConnection(self):
            self.database_open = False
            self.conn.close()

    def GetNumberCols(self):
        return len(self.col_labels)

    def GetNumberRows(self):
        if not self.database_open:
            return 0
        self.c.execute("""SELECT COUNT(*) FROM CREDITNOTES""")
        result = self.c.fetchone()
        return result[0]

    def IsEmptyCell(self, row, col):
        return False

    def SetValue(self, row, col, value):
        pass

    def GetValue(self, row, col):
        "Get data from the database to go into row, col"
        if not self.database_open:
            return ""
        if col == 0:
            # Get datestamp
            self.c.execute("""SELECT DATESTAMP
                                FROM TRANSACTIONS, CREDITNOTES
                               WHERE CREDITNOTES.GRIDROW = ?
                                 AND CREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                              """, (row,))
            result = self.c.fetchone()
            if not result:
                return ""
            return result[0].strftime("%d %b %Y")
        elif col == 1:
            # Get Customer
            self.c.execute("""SELECT NAME
                              FROM CREDITNOTES, CUSTOMERS
                              WHERE CREDITNOTES.GRIDROW = ?
                              AND CREDITNOTES.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID
                              """, (row,))
            result = self.c.fetchone()
            if not result:
                return ""
            return result[0]
        elif col == 2:
            # Get Description
            self.c.execute("""SELECT DESCRIPTION
                                FROM TRANSACTIONS, CREDITNOTES
                               WHERE CREDITNOTES.GRIDROW = ?
                                 AND CREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                              """, (row,))
            result = self.c.fetchone()
            if not result:
                return ""
            return result[0]
        elif col == 3:
            # Get cost
            self.c.execute("""SELECT COST
                              FROM CREDITNOTES
                              WHERE GRIDROW = ?""", (row,))
            result = self.c.fetchone()
            if not result:
                return ""
            return to_currency_string(D(result[0]))
        elif col == 4:
            # Get VAT
            self.c.execute("""SELECT VAT
                              FROM CREDITNOTES
                              WHERE GRIDROW = ?""", (row,))
            result = self.c.fetchone()
            if not result:
                return ""
            return to_currency_string(D(result[0]))
        else:
            return ""

    def GetColLabelValue(self, col):
        return self.col_labels[col]

    def GetRowLabelValue(self, row):
        "Returns the credit note number as the row label"
        if not self.database_open:
            return ""
        self.c.execute("SELECT CREDITNOTE_NUMBER FROM CREDITNOTES WHERE GRIDROW = ?", (row,))
        result = self.c.fetchone()
        if not result:
            return ""
        return str(result[0])

    def GetAttr(self, row, col, kind):
        attr = self.attr
        if col == 3 or col == 4:
            attr = self.right
        attr.IncRef()
        return attr

