####### VATStuff #######
#
# make_new.py  - Creates a new database and the tables within it
#
# Version : 0.2
# Date : 20091126
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#

"""Makes a new database, run as part of vatstuff"""

import os
import datetime
import decimal

from vatstuff_packages.vs_utilities import *

D = decimal.Decimal   # D used to avoid typing

def new_database(prog_func=None):
    """Creates a new database with default accounts
       prog_func is an optional function which, if given, is called at intervals during
       the database creation this function can be used to display progress
       by incrementing a progress bar"""

    database_filepath = vs_cfg["DATABASE_FILEPATH"]

    # As this creates a new database, raise an error if one already exists
    if os.path.exists(database_filepath):
        debug_log("Asked to create database, but %s already exists" % database_filepath)
        raise VS_ERROR, "Database %s already exists" % database_filepath

    info_log("Creating new database %s" % database_filepath)

    # Make the connection
    conn = None
    try:
        conn, cur = vs_connect()        # connect to the database

        create_configuration(cur)       # A table for holding configuration data
                                        # including version number, unicode currency sign

        create_companies(cur)           # Create a table to hold company details

        if prog_func: prog_func()

        create_vat_schemes(cur)         # A table to hold VAT scheme details

        create_vat_returns(cur)         # A table to hold VAT returns

        create_employees(cur)           # A table to hold employees

        if prog_func: prog_func()

        create_contractors(cur)         # A table to hold contractors

        create_suppliers(cur)           # Create a table of suppliers

        create_customers(cur)           # Create a table of customers

        if prog_func: prog_func()

        create_contacts(cur)            # Create a table of contacts

        create_addresses(cur)           # Create a table of company addresses

        create_folders(cur)             # Each folder holds a number of accounts
                                        # account names are unique within each folder

        if prog_func: prog_func()

        create_account_types(cur)       # account types - asset, expense etc

        create_vat_rates(cur)           # Create a tax table of vat rates

        create_accounts(cur)            # Account names, description total balance
                                        # as shown on the tree list of folders and accounts

        if prog_func: prog_func()

        create_transactions(cur)        # Holds a transaction number, datestamp and
                                        # description of each transaction

        create_transactionaccounts(cur) # holds transaction id' agains each account
                                        # id affected by that transaction

        create_vat_corrections(cur)     # holds vat correction descriptions against transaction id

        if prog_func: prog_func()

        create_labels(cur)              # Holds labels of transaction definitions
                                        # so one label(button) can reference a list of transactions

        create_definitions_table(cur)   # transaction definitions - lists sets of
                                        # account names and debits, credits which can be
                                        # called via a button, edited and implemented

        create_projects(cur)            # A table of proects, used to group purchases

        if prog_func: prog_func()

        create_itemdescriptions(cur)    # Descriptions of items yoou buy or sell

        create_sellitems(cur)           # A table of items to sell

        create_purchaseitems(cur)       # A table of items to purchase

        if prog_func: prog_func()

        create_closedorders(cur)        # A table of closed purchase orders linking to a transaction

        create_purchaseorders(cur)      # A table holding Purchase Order records

        create_ordereditems(cur)        # A table linking Purchase Orders and purchaseitems

        if prog_func: prog_func()

        create_makepayments(cur)        # A table of payments made to suppliers

        create_supplierinvoices(cur)    # A table of suppliers invoices received

        create_supplierinvoicepayments(cur)     # A table linking supplier invoices and payments made

        if prog_func: prog_func()

        create_suppliercreditnotes(cur) # A table of suppliers credit notes received

        create_invoices(cur)            # A table holding Invoice records

        create_solditems(cur)           # A table linking Invoices and sellitems

        if prog_func: prog_func()

        create_customerpayments(cur)    # A table of customer payments received

        create_invoicepayments(cur)     # A table linking customer payments and invoices

        create_creditnotes(cur)         # A table of credit notes created

        if prog_func: prog_func()

        create_creditnoteitems(cur)     # A table linking credit notes and sellitems

        conn.commit()                   # Commit all the changes

    finally:
        if conn:
            conn.close()

    if prog_func: prog_func()


def create_configuration(cur):
    """A table for holding configuration data."""

    # For a new table, these configuration values are read from the vs_cfg array
    # When an existing table is read, the program can take the values from this
    # database file - not from vs_cfg

    default_configuration = [
                       (None, "PROGRAM_ID", vs_cfg["PROGRAM_ID"]),
                       (None, "PROGRAM_VERSION", vs_cfg["PROGRAM_VERSION"]),
                       (None, "DATABASE_VERSION", vs_cfg["DATABASE_VERSION"]),                       
                       (None, "CURRENCY_SYMBOL", vs_cfg["CURRENCY_SYMBOL"]),
                       (None, "CURRENCY_POINT", vs_cfg["CURRENCY_POINT"]),
                       (None, "CURRENCY_THOUSANDS_SEPARATOR", vs_cfg["CURRENCY_THOUSANDS_SEPARATOR"]),
                       (None, "COMPANY_ID", "1"),
                       (None, "VAT_SCHEME_ID", "1"),
                       (None, "LAST_VAT_RETURN_ID", "0"),
                       (None, "LAST_ORDER_NUMBER", "0"),
                       (None, "LAST_INVOICE_NUMBER", "0"),
                       (None, "LAST_CREDITNOTE_NUMBER", "0"),
                       (None, "LAST_CASH_ACCOUNT_ID", "0"),
                       (None, "LAST_STOCK_ACCOUNT_ID", "0"),
                       (None, "CUSTOMER_ACCOUNT_TYPE_ID", "0"),
                       (None, "SUPPLIER_ACCOUNT_TYPE_ID", "0"),
                       (None, "LOGO_FILE", vs_cfg["LOGO_FILE"]),
                       (None, "DELETE_TO_DATE", None),
                      ]


    # Create CONFIGURATION table

    cur.execute("""CREATE TABLE CONFIGURATION
                      (
                      CONFIGURATION_ID INTEGER PRIMARY KEY AUTOINCREMENT,
                      CFG_KEY TEXT UNIQUE NOT NULL,
                      CFG_ITEM TEXT
                      )""")

    # Fill the table
    cur.executemany("""INSERT INTO CONFIGURATION 
                         (CONFIGURATION_ID, CFG_KEY, CFG_ITEM) VALUES (?, ?, ?)""",
                         default_configuration)
    return


def create_companies(cur):
    # Create COMPANIES table
    # Note there is only one company, but multiple company details are stored - each one being this
    # companies details, as they have changed - for example if the registered office changed

    cur.execute("""CREATE TABLE COMPANIES
                       (
                        COMPANY_ID INTEGER PRIMARY KEY AUTOINCREMENT,
                        ADDRESS_ID INTEGER REFERENCES ADDRESSES(ADDRESS_ID),
                        VAT_COUNTRY_CODE TEXT,
                        VAT_REGNUMBER TEXT DEFAULT "",
                        COMPANY_NUMBER TEXT DEFAULT "",
                        COUNTRY_REGISTRATION TEXT,
                        COMPANY_NAME TEXT DEFAULT "",
                        SORTCODE TEXT DEFAULT "",
                        BANK_ACCOUNT TEXT DEFAULT "",
                        IBAN TEXT DEFAULT "",
                        BIC TEXT DEFAULT ""
                        )""")

    # Create Default Company
    cur.execute("""INSERT INTO COMPANIES (COMPANY_ID, VAT_COUNTRY_CODE, COUNTRY_REGISTRATION) VALUES (?, ?, ?)""",
                   (None, vs_cfg["VAT_COUNTRY_CODE"], vs_cfg["COUNTRY"]))

    # Whenever a new company id is created, the id is stored in the configuration
    # table with CFG_KEY 'COMPANY_ID'

    cur.execute("""CREATE TRIGGER new_company AFTER INSERT ON COMPANIES
                   BEGIN
                       UPDATE CONFIGURATION SET CFG_ITEM = NEW.COMPANY_ID WHERE CFG_KEY = "COMPANY_ID";
                   END""")

    return


def create_vat_schemes(cur):
    # Create VAT_SCHEMES table
    # Note there is only one current VAT scheme, but multiple schemes are stored - each one being
    # past schemes if they have changed, the current id is stored in the configuration table

    # autoincrement keyword is used in the vat_scheme_id decleration
    # this ensures old deleted vat_scheme_id's are never re-used

    # The startdate stored is the start of the current VAT period, this
    # item can be updated without creating a new vat_scheme

    # creating a vat scheme does not create a transaction, however the current transaction id is stored
    # here, so the undo button can test if the vat scheme has changed since the transaction to be deleted
    # was implemented, and therefore delete the vat scheme change rather than the transaction

    # Note this transaction_id does not 'REFERENCES TRANSACTIONS(TRANSACTION_ID)' as the transaction may
    # eventually be deleted - and this transaction does not directly refer to this scheme change

    cur.execute("""CREATE TABLE VAT_SCHEMES
                       (
                        VAT_SCHEME_ID INTEGER PRIMARY KEY AUTOINCREMENT,
                        TRANSACTION_ID INTEGER,
                        METHOD TEXT DEFAULT "STANDARD",
                        PERIOD TEXT DEFAULT "QUARTERLY",
                        STARTDATE DATE,
                        FLAT_PERCENT TEXT
                        )""")

    start_of_current_month = datetime.date.today().replace(day=1)

    # Create Default Scheme
    cur.execute("""INSERT INTO VAT_SCHEMES (VAT_SCHEME_ID, TRANSACTION_ID, METHOD, PERIOD, STARTDATE, FLAT_PERCENT)
                   VALUES (?, ?, ?, ?, ?, ?)""",
                   (None, None, "STANDARD", "QUARTERLY", start_of_current_month, "10.0"))

    # Whenever a new vat scheme id is created, the id is stored in the configuration
    # table with CFG_KEY 'VAT_SCHEME_ID'

    cur.execute("""CREATE TRIGGER new_vat_scheme AFTER INSERT ON VAT_SCHEMES
                   BEGIN
                       UPDATE CONFIGURATION SET CFG_ITEM = NEW.VAT_SCHEME_ID WHERE CFG_KEY = "VAT_SCHEME_ID";
                   END""")

    return

def create_vat_returns(cur):
    # Create VAT_RETURNS table
    cur.execute("""CREATE TABLE VAT_RETURNS
                       (
                        VAT_RETURN_ID INTEGER PRIMARY KEY AUTOINCREMENT,
                        VAT_SCHEME_ID INTEGER REFERENCES VAT_SCHEMES(VAT_SCHEME_ID),
                        PREVIOUS_VAT_RETURN_ID INTEGER REFERENCES VAT_RETURNS(VAT_RETURN_ID),
                        STANDARD_RATE TEXT,
                        FROM_DATE DATE,
                        TO_DATE DATE,
                        COMPANY_ID INTEGER REFERENCES COMPANIES(COMPANY_ID),
                        TRANSACTION_ID INTEGER REFERENCES TRANSACTIONS(TRANSACTION_ID),
                        ACCOUNT_ID REFERENCES ACCOUNTS(ACCOUNT_ID),
                        BOX_ONE TEXT,
                        BOX_TWO TEXT,
                        BOX_THREE TEXT,
                        BOX_FOUR TEXT,
                        BOX_FIVE TEXT,
                        BOX_SIX TEXT,
                        BOX_SEVEN TEXT,
                        BOX_EIGHT TEXT,
                        BOX_NINE TEXT
                        )""")

   # The ACCOUNT_ID refers to the cash account - money goes from there to the
   # vat balance account, so after a payment, record
   # the last cash account used - so it can be chosen on drop down lists

    cur.execute("""CREATE TRIGGER new_vatreturn AFTER INSERT ON VAT_RETURNS
                   BEGIN
                       UPDATE CONFIGURATION SET CFG_ITEM = NEW.ACCOUNT_ID WHERE CFG_KEY = "LAST_CASH_ACCOUNT_ID";
                       UPDATE CONFIGURATION SET CFG_ITEM = NEW.VAT_RETURN_ID WHERE CFG_KEY = "LAST_VAT_RETURN_ID";
                   END""")
    return


def create_employees(cur):
    # Create EMPLOYEES table - currently unused but available for future expansion
    # Note the CONTACTS and ADDRESSES tables have columns referencing this table

    cur.execute("""CREATE TABLE EMPLOYEES
                       (
                        EMPLOYEE_ID INTEGER PRIMARY KEY AUTOINCREMENT,
                        NUMBER INTEGER UNIQUE,
                        DIRECTOR INTEGER NOT NULL DEFAULT 0,
                        DEPARTMENT TEXT,
                        TITLE TEXT,
                        FORENAME1 TEXT,
                        FORENAME2 TEXT,
                        SURNAME TEXT,
                        GENDER TEXT,
                        NI_NUMBER TEXT,
                        TAX_CODE TEXT,
                        DATE_JOINED DATE,
                        DATE_LEFT DATE,
                        DATE_OF_BIRTH DATE,
                        SORTCODE TEXT DEFAULT "",
                        BANK_ACCOUNT TEXT DEFAULT "",
                        ACCOUNT_ID INTEGER REFERENCES ACCOUNTS(ACCOUNT_ID)
                        )""")
    return


def create_contractors(cur):
    # Create CONTRACTORS table - currently unused but available for future expansion
    # Note the CONTACTS and ADDRESSES tables have columns referencing this table

    cur.execute("""CREATE TABLE CONTRACTORS
                       (
                        CONTRACTOR_ID INTEGER PRIMARY KEY AUTOINCREMENT,
                        NUMBER INTEGER UNIQUE,
                        ACCOUNT_ID INTEGER REFERENCES ACCOUNTS(ACCOUNT_ID)
                        )""")
    return


def create_suppliers(cur):
    # Create SUPPLIERS table

    cur.execute("""CREATE TABLE SUPPLIERS
                       (
                        SUPPLIER_ID INTEGER PRIMARY KEY AUTOINCREMENT,
                        NAME TEXT,
                        EC_COUNTRY INTEGER NOT NULL DEFAULT 1,
                        VAT_COUNTRY_CODE TEXT,
                        VAT_REGNUMBER TEXT,
                        VAT_REGISTERED INTEGER DEFAULT 1,
                        DISCOUNT TEXT DEFAULT "0.0",
                        ACCOUNT_ID INTEGER REFERENCES ACCOUNTS(ACCOUNT_ID),
                        NUMBER INTEGER NOT NULL,
                        SORTCODE TEXT DEFAULT "",
                        BANK_ACCOUNT TEXT DEFAULT "",
                        IBAN TEXT DEFAULT "",
                        BIC TEXT DEFAULT "",
                        EDIT_NEW INTEGER NOT NULL DEFAULT 0,
                        VISIBLE INTEGER NOT NULL DEFAULT 1
                        )""")

    # NUMBER - internal identification for this supplier, as the supplier_id can
    #          change as the supplier is edited

    # EDIT_NEW - if zero (False) the name, ec_country, vat_country_code and vat_regnumber can be edited
    #          - if one (True), if the above are edited, a new item is created, leaving
    #          - the old one untouched

    # VISIBLE - If zero (false) the item is not available for editing
    #         - if one (true) the item can be edited
    return


def create_customers(cur):
    # Create CUSTOMERS table

    cur.execute("""CREATE TABLE CUSTOMERS
                       (
                        CUSTOMER_ID INTEGER PRIMARY KEY AUTOINCREMENT,
                        NAME TEXT,
                        EC_COUNTRY INTEGER NOT NULL DEFAULT 1,
                        VAT_COUNTRY_CODE TEXT,
                        VAT_REGNUMBER TEXT,
                        VAT_REGISTERED INTEGER DEFAULT 1,
                        DISCOUNT TEXT DEFAULT "0.0",
                        ACCOUNT_ID INTEGER REFERENCES ACCOUNTS(ACCOUNT_ID),
                        NUMBER INTEGER NOT NULL,
                        SORTCODE TEXT DEFAULT "",
                        BANK_ACCOUNT TEXT DEFAULT "",
                        IBAN TEXT DEFAULT "",
                        BIC TEXT DEFAULT "",
                        EDIT_NEW INTEGER NOT NULL DEFAULT 0,
                        VISIBLE INTEGER NOT NULL DEFAULT 1
                        )""")

    # NUMBER - internal identification for this customer, as the customer_id can
    #          change as the customer is edited

    # EDIT_NEW - if zero (False) the name, ec_country, vat_country_code and vat_regnumber can be edited
    #          - if one (True), if the above are edited, a new item is created, leaving
    #          - the old one untouched

    # VISIBLE - If zero (false) the item is not available for editing
    #         - if one (true) the item can be edited
    return


def create_contacts(cur):
    """Contacts - lists of names, email, phone, mobile, notes"""

    # SYSTEM NUMBER

    # 0 names which should not appear on orders and invoices - ie your customer or suppliers contacts
    # 1 names which should appear as enquiry contacts on orders and invoices - ie your own company contacts

    # Create CONTACTS table

    cur.execute("""CREATE TABLE CONTACTS
                      (
                      CONTACT_ID INTEGER PRIMARY KEY AUTOINCREMENT,
                      CUSTOMER_ID INTEGER REFERENCES CUSTOMERS(CUSTOMER_ID),
                      SUPPLIER_ID INTEGER REFERENCES SUPPLIERS(SUPPLIER_ID),
                      EMPLOYEE_ID INTEGER REFERENCES EMPLOYEES(EMPLOYEE_ID),
                      CONTRACTOR_ID INTEGER REFERENCES CONTRACTORS(CONTRACTOR_ID),
                      NAME TEXT NOT NULL,
                      EMAIL TEXT,
                      PHONE TEXT,
                      MOBILE TEXT,
                      NOTES TEXT,
                      EDIT_NEW INTEGER NOT NULL DEFAULT 0,
                      VISIBLE INTEGER NOT NULL DEFAULT 1,
                      SYSTEM INTEGER NOT NULL DEFAULT 0
                      )""")

    # EDIT_NEW - if zero (False) The item name, email, phone, mobile can be edited
    #          - if one (True), if the above are edited, a new item is created, leaving
    #          - the old one untouched

    # VISIBLE - If zero (false) the item is not available for editing
    #         - if one (true) the item can be edited
    return


def create_addresses(cur):
    """Each company, supplier, customer, employee, contractor address is held here"""

    # SYSTEM NUMBER

    # 0 supplier or customer address
    # 1 your address - which appears in choices where your company address is to be picked

    
    # EDIT_NEW - if zero (False) Line1, Line2, City, County, Postcode, country can be edited
    #          - if one (True), if the above are edited, a new item is created, leaving
    #          - the old one untouched

    # VISIBLE - If zero (false) the item is not available for editing
    #         - if one (true) the item can be edited

    # Create ADDRESSES table

    cur.execute("""CREATE TABLE ADDRESSES
                      (
                      ADDRESS_ID INTEGER PRIMARY KEY AUTOINCREMENT,
                      CUSTOMER_ID INTEGER REFERENCES CUSTOMERS(CUSTOMER_ID),
                      SUPPLIER_ID INTEGER REFERENCES SUPPLIERS(SUPPLIER_ID),
                      EMPLOYEE_ID INTEGER REFERENCES EMPLOYEES(EMPLOYEE_ID),
                      CONTRACTOR_ID INTEGER REFERENCES CONTRACTORS(CONTRACTOR_ID),
                      LINE1 TEXT,
                      LINE2 TEXT,
                      CITY TEXT,
                      COUNTY TEXT,
                      POSTCODE TEXT,
                      COUNTRY TEXT,
                      REG_OFFICE INTEGER DEFAULT 0,
                      DELIVERY_POINT INTEGER DEFAULT 0,
                      INVOICE_POINT INTEGER DEFAULT 0,
                      ORDER_POINT INTEGER DEFAULT 0,
                      PAYMENT_POINT INTEGER DEFAULT 0,
                      EDIT_NEW INTEGER NOT NULL DEFAULT 0,
                      VISIBLE INTEGER NOT NULL DEFAULT 1,
                      SYSTEM INTEGER NOT NULL DEFAULT 0
                      )""")
    return


def create_vat_rates(cur):
    """A table for holding VAT rates."""

    default_rates = [
                       (None, "Standard", "Standard rate for most items", "S", vs_cfg["DEFAULT_VAT_STANDARD_RATE"]),
                       (None, "Reduced", "Reduced rate", "R", vs_cfg["DEFAULT_VAT_REDUCED_RATE"]),
                       (None, "Zero", "Zero rate", "Z", "0.0"),
                       (None, "Excempt", "See manual if selling VAT Excempt goods.", "E", "0.0"),
                       (None, "Not Registered", "Supplier or customer not VAT registered", "N", "0.0"),
                      ]


    # Create VAT_RATES table

    cur.execute("""CREATE TABLE VAT_RATES
                      (
                      VAT_RATE_ID INTEGER PRIMARY KEY AUTOINCREMENT,
                      NAME TEXT,
                      DESCRIPTION TEXT,
                      SYMBOL TEXT UNIQUE NOT NULL,
                      RATE TEXT NOT NULL
                      )""")

    # Fill the table
    cur.executemany("""INSERT INTO VAT_RATES 
                         (VAT_RATE_ID, NAME, DESCRIPTION, SYMBOL, RATE) VALUES (?, ?, ?, ?, ?)""",
                         default_rates)
    return


def create_folders(cur):
    """Each account is under an folder value, for example - Expenses,
       this function creates the FOLDERS table and populates it with default folders"""

    # Create FOLDERS table

    cur.execute("""CREATE TABLE FOLDERS
                      (
                      FOLDER_ID INTEGER PRIMARY KEY AUTOINCREMENT,
                      NAME TEXT UNIQUE NOT NULL,
                      DESCRIPTION TEXT,
                      SYSTEM INTEGER NOT NULL DEFAULT 0,
                      EXPANDED INTEGER NOT NULL DEFAULT 0
                      )""")

    # Create Default folders
    # Note, putting None into folder_id is the equivalent of auto-incrementing
    
    default_folders = [
                       (None, "Bank", "Bank Accounts", 1, 1),
                       (None, "Income", "Accounts tracking income", 1, 1),
                       (None, "Expenses", "Accounts tracking expenses", 1, 1),
                       (None, "Stock", "Assetts purchased", 1, 1),
                       (None, "System", "vatstuff system accounts", 2, 0),
                       (None, "Customers", "Customer accounts", 3, 0),
                       (None, "Suppliers", "Supplier accounts", 4, 0),
                      ]

    # System flags
    #
    # 0 - This folder has no accounts, and can be deleted
    # 1 - This folder does have accounts, cannot be deleted
    #
    # All numbers above 1 are for system folders, and identify the usage of this folder
    #
    # 2 - System folder - holds system accounts
    # 3 - Customers folder - holds customer accounts
    # 4 - Suppliers folder - holds supplier accounts

    # expanded
    # 0 the folder is displayed as closed
    # 1 the folder is displayed as open

    # Fill the table
    cur.executemany("""INSERT INTO FOLDERS
                        (
                        FOLDER_ID, 
                        NAME,
                        DESCRIPTION,
                        SYSTEM,
                        EXPANDED
                        ) VALUES (?, ?, ?, ?, ?)""",
                         default_folders)


def create_account_types(cur):
    """Account types such as asset, expense etc
       this function creates the ACCOUNTTYPES table and populates it with default types"""

    # Create FOLDERS table

    cur.execute("""CREATE TABLE ACCOUNTTYPES
                      (
                      ACCOUNTTYPE_ID INTEGER PRIMARY KEY AUTOINCREMENT,
                      NAME TEXT,
                      DESCRIPTION TEXT
                      )""")

    # Create Default types
    
    # one of these type names MUST be set in vs_cfg["CUSTOMER_ACCOUNT_TYPE_NAME"]
    # one of these type names MUST be set in vs_cfg["SUPPLIER_ACCOUNT_TYPE_NAME"]
    # As default, they are set in config.py as:
    # vs_cfg["CUSTOMER_ACCOUNT_TYPE_NAME"] = "Asset" 
    # vs_cfg["SUPPLIER_ACCOUNT_TYPE_NAME"] = "Liability"
    
    default_accounttypes = [
                       (None, "Asset", "Accounts holding assets such as cash accounts"),
                       (None, "Liability", "Accounts recording money you owe"),
                       (None, "Income", "Accounts recording income"),
                       (None, "Expense", "Accounts recording expenses, such as electricity expenses"),
                       (None, "Capital", "Accounts holding capital amounts - such as the opening balance account")
                      ]

 
    # Fill the table
    cur.executemany("""INSERT INTO ACCOUNTTYPES
                        (
                        ACCOUNTTYPE_ID,
                        NAME,
                        DESCRIPTION
                        ) VALUES (?, ?, ?)""",
                         default_accounttypes)

    # Set the configuration values
    cur.execute("SELECT ACCOUNTTYPE_ID FROM ACCOUNTTYPES WHERE NAME = ?", (vs_cfg["CUSTOMER_ACCOUNT_TYPE_NAME"],))
    result = cur.fetchone()
    if result[0]:
        customer_account_type_id = str(result[0])
    else:
        # on failure, just pick the first accounttype 'Asset'
        customer_account_type_id = "1"
    cur.execute("UPDATE CONFIGURATION SET CFG_ITEM = ? WHERE CFG_KEY = ?", 
                                          (customer_account_type_id, "CUSTOMER_ACCOUNT_TYPE_ID"))


    cur.execute("SELECT ACCOUNTTYPE_ID FROM ACCOUNTTYPES WHERE NAME = ?", (vs_cfg["SUPPLIER_ACCOUNT_TYPE_NAME"],))
    result = cur.fetchone()
    if result[0]:
        supplier_account_type_id = str(result[0])
    else:
        # on failure, just pick the second accounttype 'Liability'
        supplier_account_type_id = "2"
    cur.execute("UPDATE CONFIGURATION SET CFG_ITEM = ? WHERE CFG_KEY = ?", 
                                          (supplier_account_type_id, "SUPPLIER_ACCOUNT_TYPE_ID"))


def create_accounts(cur):
    """Create a table of accounts, and for each account create a Ledger table"""

    # Note, the BALANCE is held as a text string - being a representative of a Decimal object

    # Create ACCOUNTS table
    cur.execute("""CREATE TABLE ACCOUNTS
                     (
                     ACCOUNT_ID INTEGER PRIMARY KEY AUTOINCREMENT,
                     FOLDER_ID INTEGER REFERENCES FOLDERS(FOLDER_ID),
                     NAME TEXT NOT NULL,
                     DESCRIPTION TEXT,
                     BALANCE TEXT,
                     SYSTEM INTEGER NOT NULL DEFAULT 0,
                     ACCOUNTTYPE_ID INTEGER REFERENCES ACCOUNTTYPES(ACCOUNTTYPE_ID))""")

    # Default accounts
    # Initially create a list with folder names instead of folder id's
    # and account type names instead of account type id's - then swap them out
    
    default_accounts = [
       [None, "Bank", "Current Account", "My current account", "0.00", 0, "Asset"],
       [None, "Bank", "Reserve Account", "Reserve account", "0.00", 0, "Asset"],
       [None, "Income", "Bank interest", "Bank interest income", "0.00", 0, "Income"],
       [None, "Expenses", "Mileage", "Mileage expenses", "0.00", 0, "Expense"],
       [None, "Expenses", "Accountants fees", "Accountants cost", "0.00", 0, "Expense"],
       [None, "Expenses", "Bank charges", "Bank charges", "0.00", 0, "Expense"],
       [None, "Stock", "Computer equipment", "Computer assets", "0.00", 0, "Asset"],
       [None, "Stock", "Miscellaneous", "Miscellaneous items", "0.00", 0, "Asset"],
       [None, "System", "Sales", "Sale transactions", "0.00", 6, "Income"],
       [None, "System", "Purchases", "Orders waiting for invoices", "0.00", 7, "Expense"],
       [None, "System", "Opening balance", "Holds the opening balance of your accounts", "0.00", 2, "Capital"],
       [None, "System", "VAT Balance", "Input-Output", "0.00", 5, "Liability"],
       [None, "System", "VAT Income", "VAT income from sales (FLAT Scheme)", "0.00", 8, "Income"],
       [None, "System", "Unclaimed VAT", "Input VAT not claimed back", "0.00", 9, "Expense"]
                       ]
    # Type
    #
    # Asset
    # Fixed Asset
    # Income
    # Liability
    # Expense
    # Capital

    # System flags
    #
    # 0 - If this account is new, and can have an openning balance put in via the edit window
    # 1 - Set to 1 after the first transaction, the openning balance option is no longer shown
    #
    # All numbers above 1 are for system accounts, and identify the usage of the account
    #
    # 2 - System account - the openning balance account
    # 3 - Customer accounts
    # 4 - Supplier accounts
    # 5 - VAT balance account
    # 6 - Sales account
    # 7 - Purchases account
    # 8 - VAT income from purchases (FLAT Scheme)
    # 9 - Input VAT not claimed back

    # If system flag is zero, it changes automatically to 1 when an account balance is changed
    cur.execute("""CREATE TRIGGER acc_sys_flag AFTER UPDATE OF BALANCE ON ACCOUNTS
                   BEGIN
                       UPDATE ACCOUNTS SET SYSTEM = 1 WHERE ACCOUNT_ID = OLD.ACCOUNT_ID AND SYSTEM = 0;
                   END""")

    # Create trigger to update folder system flag to 1 (if currently zero) if an account is inserted
    cur.execute("""CREATE TRIGGER acc_ins_to_folder AFTER INSERT ON ACCOUNTS
                   BEGIN
                       UPDATE FOLDERS SET SYSTEM = 1 WHERE FOLDER_ID = NEW.FOLDER_ID AND SYSTEM = 0;
                   END""")

    # Create trigger to update folder flag to 0 (if currently 1) if an account is deleted and
    # no other accounts exist in that folder, note 'BEFORE' clause is used so the account
    # being deleted is counted, to void any confusion
    cur.execute("""CREATE TRIGGER acc_del_from_folder BEFORE DELETE ON ACCOUNTS
                   BEGIN
                       UPDATE FOLDERS SET SYSTEM = 0 WHERE
                          FOLDER_ID = OLD.FOLDER_ID AND SYSTEM = 1
                          AND (SELECT COUNT(*) FROM ACCOUNTS WHERE FOLDER_ID = OLD.FOLDER_ID) = 1;
                   END""")

    # Create trigger to update folder system flag to 1 (if currently zero) if an account has
    # its folder_id changed to the new folder
    cur.execute("""CREATE TRIGGER acc_move_to_folder AFTER UPDATE OF FOLDER_ID ON ACCOUNTS
                   BEGIN
                       UPDATE FOLDERS SET SYSTEM = 1 WHERE FOLDER_ID = NEW.FOLDER_ID AND SYSTEM = 0;
                   END""")

    # Create trigger to update folder flag to 0 (if currently 1) if an account is moved from a folder
    # and no other accounts exist in that folder, note 'BEFORE' clause is used so the account
    # being moved is counted, to void any confusion
    cur.execute("""CREATE TRIGGER acc_move_from_folder BEFORE UPDATE OF FOLDER_ID ON ACCOUNTS
                   BEGIN
                       UPDATE FOLDERS SET SYSTEM = 0 WHERE
                          FOLDER_ID = OLD.FOLDER_ID AND SYSTEM = 1
                          AND (SELECT COUNT(*) FROM ACCOUNTS WHERE FOLDER_ID = OLD.FOLDER_ID) = 1;
                   END""")

    # Now change folder names to folder id's
    # and account type names to account type id's in the default_accounts list
    for key, acc in enumerate(default_accounts):      # key,acc is (0, default_accounts[0]) etc...
        # Get the particular folder_id from folders
        cur.execute("SELECT FOLDER_ID FROM FOLDERS WHERE NAME = ?", (acc[1],))
        result = cur.fetchone()
        default_accounts[key][1] = result[0]
        cur.execute("SELECT ACCOUNTTYPE_ID FROM ACCOUNTTYPES WHERE NAME = ?", (acc[6],))
        result = cur.fetchone()
        default_accounts[key][6] = result[0]

    # Fill the table
    cur.executemany("""INSERT INTO ACCOUNTS
                        (
                        ACCOUNT_ID, 
                        FOLDER_ID,
                        NAME,
                        DESCRIPTION,
                        BALANCE,
                        SYSTEM,
                        ACCOUNTTYPE_ID 
                        ) VALUES (?, ?, ?, ?, ?, ?, ?)""",
                         default_accounts)

    # For each Account, need to create a Ledger table, so first, get all account ID's
    cur.execute("SELECT ACCOUNT_ID FROM ACCOUNTS")
    result = cur.fetchall()
    for account_id in result:
        # Create the Ledger names using account id's, i.e LEDGER_1 for account with id of 1 etc
        ledger = "LEDGER_%i" % account_id
        
        # create a ledger table
        # Create a separate string for the sql query, since it
        # seems ? parameters are not allowed here
        
        sql_string = """CREATE TABLE %s
                         (
                         LEDGER_ID INTEGER PRIMARY KEY AUTOINCREMENT,
                         TRANSACTION_ID INTEGER REFERENCES TRANSACTIONS(TRANSACTION_ID),
                         GRIDROW INTEGER NOT NULL DEFAULT -1,
                         DEBIT TEXT,
                         CREDIT TEXT,
                         BALANCE TEXT)""" % ledger

        cur.execute(sql_string)

    return


def create_transactions(cur):
    """Stores the id, timestamp, description, system flag and gridrow of a transaction

       The gridrow is the row number used to calculate the position on a grid when
       transactions are displayed, the id cannot be used since transactions may be input out of date order

       System flags
    
        0 - User defined transaction - using the move buttons (The default)
        1 - An opening balance transaction
        2 - A purchase order transaction
        3 - Pay a supplier invoice transaction
        4 - Supplier credit note transaction
        5 - An invoice transaction
        6 - Customer payment transaction
        7 - Create a credit note transaction
        8 - An order close transaction
        9 - Supplier invoice received transaction
       10 - A VAT return transaction
       11 - A VAT correction transaction
       12 - A 'Carry Forward' transaction after a delete old data action 
       """

    # Note, putting None into transaction_id is the equivalent of auto-incrementing

    # Create TRANSACTIONS table, the autoincrement keyword is used as the transaction_id
    # is used to uniquely identify the transaction, and this ensures old deleted transaction
    # id's are never re-used

    cur.execute("""CREATE TABLE TRANSACTIONS
                      (
                      TRANSACTION_ID INTEGER PRIMARY KEY AUTOINCREMENT,
                      DATESTAMP DATE NOT NULL,
                      DESCRIPTION TEXT,
                      SYSTEM INTEGER NOT NULL DEFAULT 0,
                      GRIDROW INTEGER NOT NULL DEFAULT -1
                      )""")


def create_transactionaccounts(cur):
    """A table linking Transactions and Accounts"""

    # Create TRANSACTIONACCOUNTS table

    cur.execute("""CREATE TABLE TRANSACTIONACCOUNTS
                      (
                      TRANSACTIONACCOUNT_ID INTEGER PRIMARY KEY AUTOINCREMENT,
                      TRANSACTION_ID REFERENCES TRANSACTIONS(TRANSACTION_ID),
                      ACCOUNT_ID REFERENCES ACCOUNTS(ACCOUNT_ID)
                      )""")


def create_vat_corrections(cur):
    """A table linking Transactions and VAT Correction descriptions"""

    # Create VATCORRECTIONS table

    cur.execute("""CREATE TABLE VATCORRECTIONS
                      (
                      VATCORRECTION_ID INTEGER PRIMARY KEY AUTOINCREMENT,
                      TRANSACTION_ID REFERENCES TRANSACTIONS(TRANSACTION_ID),
                      LONGDESCRIPTION TEXT,
                      AMOUNT TEXT
                      )""")


def create_labels(cur):
    """Stores button labels and descriptions"""

    default_labels = [ [None, "Undefined_%02d" % (number+1), "Undefined", number] for number in range(0, vs_cfg["MAX_BUTTONS"]) ]

    cur.execute("""CREATE TABLE LABELS
                      (
                      LABEL_ID INTEGER PRIMARY KEY AUTOINCREMENT,
                      NAME TEXT,
                      DESCRIPTION TEXT,
                      BUTTON INTEGER UNIQUE
                      )""")
    # Fill the table
    cur.executemany("""INSERT INTO LABELS
                        (
                        LABEL_ID, 
                        NAME,
                        DESCRIPTION,
                        BUTTON
                        ) VALUES (?, ?, ?, ?)""",
                         default_labels)


def create_definitions_table(cur):
    """Stores the definition of a series of transactions"""

    # columns: definition_id, label_id, account_id, action, value, operator, operand
    
    # action is an integer 0 = debit, 1 = credit
    # value is a string
    # operator is an integer, 0 = None, 1 = +, 2 = -, 3 = x, 4 = /
    # operand is a string

    cur.execute("""CREATE TABLE DEFINITIONS
                      (
                      DEFINITION_ID INTEGER PRIMARY KEY AUTOINCREMENT,
                      LABEL_ID INTEGER REFERENCES LABELS(LABEL_ID),
                      ACCOUNT_ID INTEGER REFERENCES ACCOUNTS(ACCOUNT_ID),
                      ACTION INTEGER,
                      VALUE TEXT,
                      OPERATOR INTEGER,
                      OPERAND TEXT
                      )""")
    return


def create_projects(cur):
    """Table projects - lists project numbers, description and contact name
       referenced from purchaseorders - and so can be used to group purchaseorders
       into projects - enabling purchase orders to be listed by project in reports"""

    # Create PROJECTS table

    cur.execute("""CREATE TABLE PROJECTS
                      (
                      PROJECT_ID INTEGER PRIMARY KEY AUTOINCREMENT,
                      PROJECT_NUMBER TEXT UNIQUE NOT NULL,
                      DESCRIPTION TEXT,
                      CONTACT_ID REFERENCES CONTACTS(CONTACT_ID)
                      )""")

    cur.execute("""CREATE TRIGGER new_project AFTER INSERT ON PROJECTS
                   BEGIN
                       UPDATE CONTACTS SET EDIT_NEW = 1 WHERE CONTACT_ID = NEW.CONTACT_ID;
                   END""")



def create_itemdescriptions(cur):
    """Table itemdescriptions - lists item descriptions you buy and sell
       Whenever an item description is 'edited', a new item description is
       created - so old descriptions linked from invoices and purchase
       orders, remain in place"""

    # Create ITEMDESCRIPTIONS table

    cur.execute("""CREATE TABLE ITEMDESCRIPTIONS
                      (
                      ITEMDESCRIPTION_ID INTEGER PRIMARY KEY AUTOINCREMENT,
                      DESCRIPTION TEXT UNIQUE NOT NULL
                      )""")


def create_sellitems(cur):
    """SellItems- list of item cost and vat rates of items you frequently sell"""

    # Create SELLITEMS table

    cur.execute("""CREATE TABLE SELLITEMS
                      (
                      SELLITEM_ID INTEGER PRIMARY KEY AUTOINCREMENT,
                      ITEMDESCRIPTION_ID INTEGER REFERENCES ITEMDESCRIPTIONS(ITEMDESCRIPTION_ID),
                      COST TEXT,
                      VAT_RATE_ID INTEGER REFERENCES VAT_RATES(VAT_RATE_ID),
                      NOTES TEXT
                      )""")

    # This table is displayed to the user as frequently sold items and is shown as a drop
    # down list when the user is creating an invoice or credit note


def create_purchaseitems(cur):
    """PurchaseItems- lists of frequently purchased items, cost and vat rates of items you purchase"""

    # Create PURCHASEITEMS table

    cur.execute("""CREATE TABLE PURCHASEITEMS
                      (
                      PURCHASEITEM_ID INTEGER PRIMARY KEY AUTOINCREMENT,
                      ITEMDESCRIPTION_ID INTEGER REFERENCES ITEMDESCRIPTIONS(ITEMDESCRIPTION_ID),
                      COST TEXT DEFAULT NULL,
                      VAT_RATE_ID INTEGER REFERENCES VAT_RATES(VAT_RATE_ID),
                      NOTES TEXT DEFAULT NULL
                      )""")

    # This table is displayed to the user as frequently purchased items and is shown as a drop
    # down list when the user is creating a purchase order


def create_closedorders(cur):
    """Closedorders - closed or cancelled purchase orders requiring a transaction"""

    # Create CLOSEDORDERS table
    cur.execute("""CREATE TABLE CLOSEDORDERS
                      (
                      CLOSEDORDER_ID INTEGER PRIMARY KEY AUTOINCREMENT,
                      TRANSACTION_ID REFERENCES TRANSACTIONS(TRANSACTION_ID),
                      REASON TEXT
                      )""")


def create_purchaseorders(cur):
    """PurchaseOrders - list of purchase orders"""

    # Create PURCHASEORDERS table

    cur.execute("""CREATE TABLE PURCHASEORDERS
                      (
                      PURCHASEORDER_ID INTEGER PRIMARY KEY AUTOINCREMENT,
                      COMPANY_ID REFERENCES COMPANIES(COMPANY_ID),
                      SUPPLIER_ID REFERENCES SUPPLIERS(SUPPLIER_ID),
                      SUPPLIER_ADDRESS_ID REFERENCES ADDRESSES(ADDRESS_ID),
                      INVOICE_ADDRESS_ID REFERENCES ADDRESSES(ADDRESS_ID),
                      DELIVERY_ADDRESS_ID REFERENCES ADDRESSES(ADDRESS_ID),
                      PROJECT_ID REFERENCES PROJECTS(PROJECT_ID),
                      ORDER_NUMBER INTEGER UNIQUE,
                      CONTACT_ID REFERENCES CONTACTS(CONTACT_ID),
                      TRANSACTION_ID REFERENCES TRANSACTIONS(TRANSACTION_ID),
                      DISCOUNT TEXT,
                      VAT TEXT,
                      COST TEXT,
                      INVOICE_VAT TEXT DEFAULT "0.00",
                      INVOICE_COST TEXT DEFAULT "0.00",
                      SUPPLIER_REFERENCE TEXT,
                      PRINT_CHECK INTEGER,
                      PDF_WIDTH INTEGER DEFAULT %s,
                      PDF_HEIGHT INTERGER DEFAULT %s,
                      PAID TEXT DEFAULT "0.00",
                      STATUS INTEGER DEFAULT 0,
                      CLOSEDORDER_ID REFERENCES CLOSEDORDERS(CLOSEDORDER_ID),
                      GRIDROW INTEGER NOT NULL DEFAULT -1,
                      SHOW_VAT INTEGER DEFAULT 1,
                      CLAIM_VAT INTEGER DEFAULT 1
                      )""" % vs_cfg["A4"])

    # STATUS
    # 0 if this Purchase Order is still open, no invoices received
    # 1 order open, and one or more invoices have been received
    # 2 if normally closed and order fully paid, i.e total == paid
    # 3 if closed, with order total less than that paid, i.e. total<paid
    # 4 if closed, with order total greater than that paid, i.e. total>paid
    # 5 if closed by the order being cancelled with no invoice recieved or payment made

    # INVOICE_COST and INVOICE_VAT are running totals being the sum of all invoices (minus creditnotes)
    # received on this order - and are therefore the actual amounts to be paid. They are normally
    # equal to the order VAT and COST, but could be different - for example if the supplier
    # substitutes a different but equivalent product at a slightly different price

    # CLOSEDORDER_ID is Null if the order is open or normally closed
    # or references an id in the CLOSEDORDERS table if closed with a reason and
    # a transaction - such as order not fully paid or cancelled

    # gridrow default is -1 to initially give it a non valid gridrow number

    # CLAIM_VAT is 1 if VAT is to be claimed back as input tax - the normal situation,
    # but in some cases, where VAT cannot be claimed, such as entertainment expenses
    # then this is set to zero

    # Whenever a new purchaseorder is created, the order_number is stored in the configuration
    # table as LAST_ORDER_NUMBER, and items referred to on the order, such as contact and
    # addresses must now be fixed - so if they are edited in future, new copies are made
    # and this purchase order is not changed


    cur.execute("""CREATE TRIGGER new_purchase_order AFTER INSERT ON PURCHASEORDERS
                   BEGIN
                       UPDATE CONFIGURATION SET CFG_ITEM = NEW.ORDER_NUMBER WHERE CFG_KEY = "LAST_ORDER_NUMBER";
                       UPDATE CONTACTS SET EDIT_NEW = 1 WHERE CONTACT_ID = NEW.CONTACT_ID;
                       UPDATE SUPPLIERS SET EDIT_NEW = 1 WHERE SUPPLIER_ID = NEW.SUPPLIER_ID;
                       UPDATE ADDRESSES SET EDIT_NEW = 1 WHERE ADDRESS_ID = NEW.SUPPLIER_ADDRESS_ID;
                       UPDATE ADDRESSES SET EDIT_NEW = 1 WHERE ADDRESS_ID = NEW.INVOICE_ADDRESS_ID;
                       UPDATE ADDRESSES SET EDIT_NEW = 1 WHERE ADDRESS_ID = NEW.DELIVERY_ADDRESS_ID;

                       UPDATE ADDRESSES SET EDIT_NEW = 1 WHERE ADDRESS_ID = 
                          (SELECT ADDRESS_ID FROM COMPANIES WHERE COMPANIES.COMPANY_ID = NEW.COMPANY_ID);

                   END""")


def create_ordereditems(cur):
    """A table linking Purchase Orders and item descriptions"""

    # Create ORDEREDITEMS table

    # DISCOUNT is 0 for no discount applied, 1 for discount applied

    cur.execute("""CREATE TABLE ORDEREDITEMS
                      (
                      ORDEREDITEM_ID INTEGER PRIMARY KEY AUTOINCREMENT,
                      PURCHASEORDER_ID REFERENCES PURCHASEORDERS(PURCHASEORDER_ID),
                      ITEMDESCRIPTION_ID INTEGER REFERENCES ITEMDESCRIPTIONS(ITEMDESCRIPTION_ID),
                      QUANTITY TEXT,
                      UNIT_COST TEXT,
                      DISCOUNT INTEGER,
                      VAT_PERCENT TEXT
                      )""")
    return

def create_supplierinvoices(cur):
    """SupplierInvoices - Table of suppliers invoices received"""

    # Create SUPPLIERINVOICES table

    cur.execute("""CREATE TABLE SUPPLIERINVOICES
                      (
                      SUPPLIERINVOICE_ID INTEGER PRIMARY KEY AUTOINCREMENT,
                      SUPPLIER_ID REFERENCES SUPPLIERS(SUPPLIER_ID),
                      PURCHASEORDER_ID REFERENCES PURCHASEORDERS(PURCHASEORDER_ID),
                      TRANSACTION_ID REFERENCES TRANSACTIONS(TRANSACTION_ID),
                      NUMBER TEXT,
                      OPEN INTEGER DEFAULT 1,
                      PAID TEXT DEFAULT "0.00",
                      COST TEXT,
                      VAT TEXT,
                      VAT_RETURN_ID REFERENCES VAT_RETURNS(VAT_RETURN_ID),
                      CLAIM_VAT INTEGER DEFAULT 1
                      )""")

    # OPEN = 1 (True) if this invoice is still active
    #      = 0 (False) if closed - fully paid

    # When a new supplierinvoice is created from a supplier, set the
    # suppliers EDIT_NEW flag, so the Supplier is permanently stored
    # and set the purchase order STATUS to 1 to show an invoice has been received

    cur.execute("""CREATE TRIGGER new_supplierinvoice AFTER INSERT ON SUPPLIERINVOICES
                   BEGIN
                       UPDATE SUPPLIERS SET EDIT_NEW = 1 WHERE SUPPLIER_ID = NEW.SUPPLIER_ID;
                       UPDATE PURCHASEORDERS SET STATUS = 1 WHERE PURCHASEORDER_ID = NEW.PURCHASEORDER_ID;
                   END""")
    return


def create_supplierinvoicepayments(cur):
    """Links supplier invoices with makepayments, as one payment made could be for several invoices
       or several payments could be for one invoice"""
    # Create SUPPLIERINVOICEPAYMENTS table

    cur.execute("""CREATE TABLE SUPPLIERINVOICEPAYMENTS
                      (
                      SUPPLIERINVOICEPAYMENT_ID INTEGER PRIMARY KEY AUTOINCREMENT,
                      SUPPLIERINVOICE_ID REFERENCES SUPPLIERINVOICES(SUPPLIERINVOICE_ID),
                      MAKEPAYMENT_ID REFERENCES MAKEPAYMENTS(MAKEPAYMENT_ID),
                      COST TEXT,
                      VAT TEXT,
                      VAT_RETURN_ID REFERENCES VAT_RETURNS(VAT_RETURN_ID),
                      CLAIM_VAT INTEGER DEFAULT 1
                      )""")


def create_suppliercreditnotes(cur):
    """SupplierCreditNotes - Table of suppliers credit notes received"""

    # Create SUPPLIERCREDITNOTES table

    cur.execute("""CREATE TABLE SUPPLIERCREDITNOTES
                      (
                      SUPPLIERCREDITNOTE_ID INTEGER PRIMARY KEY AUTOINCREMENT,
                      SUPPLIER_ID REFERENCES SUPPLIERS(SUPPLIER_ID),
                      SUPPLIERINVOICE_ID REFERENCES SUPPLIERINVOICES(SUPPLIERINVOICE_ID),
                      PURCHASEORDER_ID REFERENCES PURCHASEORDERS(PURCHASEORDER_ID),
                      TRANSACTION_ID REFERENCES TRANSACTIONS(TRANSACTION_ID),
                      ACCOUNT_ID REFERENCES ACCOUNTS(ACCOUNT_ID),
                      NUMBER TEXT,
                      COST TEXT,
                      VAT TEXT,
                      STATUS INTEGER DEFAULT 0,
                      VAT_RETURN_ID REFERENCES VAT_RETURNS(VAT_RETURN_ID),
                      CLAIM_VAT INTEGER DEFAULT 1
                      )""")

    # STATUS - normally 0 when a creditnote is received against an invoice
    # STATUS - 1 when a creditnote is received with a payment

    # Account_id is stored to reference the stock/expence account credited since
    # this information is not available for creditnotes which have been received with
    # a payment

    # When a new creditnote is created from a supplier, set the
    # suppliers EDIT_NEW flag, so the Supplier is permanently stored

    cur.execute("""CREATE TRIGGER new_suppliercreditnote AFTER INSERT ON SUPPLIERCREDITNOTES
                   BEGIN
                       UPDATE SUPPLIERS SET EDIT_NEW = 1 WHERE SUPPLIER_ID = NEW.SUPPLIER_ID;
                   END""")
    return


def create_makepayments(cur):
    """MakePayments - Table of payments made to suppliers"""

    # Create MAKEPAYMENTS table

    cur.execute("""CREATE TABLE MAKEPAYMENTS
                      (
                      MAKEPAYMENT_ID INTEGER PRIMARY KEY AUTOINCREMENT,
                      COMPANY_ID REFERENCES COMPANIES(COMPANY_ID),
                      SUPPLIER_ID REFERENCES SUPPLIERS(SUPPLIER_ID),
                      SUPPLIER_ADDRESS_ID REFERENCES ADDRESSES(ADDRESS_ID),
                      PURCHASEORDER_ID REFERENCES PURCHASEORDERS(PURCHASEORDER_ID),
                      ACCOUNT_ID REFERENCES ACCOUNTS(ACCOUNT_ID),
                      TRANSACTION_ID REFERENCES TRANSACTIONS(TRANSACTION_ID),
                      STATUS INTEGER DEFAULT 0,
                      AMOUNT TEXT
                      )""")

    # STATUS
    # 0 if this payment is a normal payment
    # 1 if this is a payment for a 'quick purchase' order

    # When a new payment is made to a supplier, set the
    # suppliers EDIT_NEW flag, so the Supplier is permanently stored
    # and also set the EDIT_NEW flag in the supplier address

    # The ACCOUNT_ID refers to the cash account - money goes from there to the
    # supplier account and the vat balance account, so after a payment, record
    # the last cash account used - so it can be chosen on drop down lists

    cur.execute("""CREATE TRIGGER new_makepayment AFTER INSERT ON MAKEPAYMENTS
                   BEGIN
                       UPDATE SUPPLIERS SET EDIT_NEW = 1 WHERE SUPPLIER_ID = NEW.SUPPLIER_ID;
                       UPDATE ADDRESSES SET EDIT_NEW = 1 WHERE ADDRESS_ID = NEW.SUPPLIER_ADDRESS_ID;
                       UPDATE CONFIGURATION SET CFG_ITEM = NEW.ACCOUNT_ID WHERE CFG_KEY = "LAST_CASH_ACCOUNT_ID";

                       UPDATE ADDRESSES SET EDIT_NEW = 1 WHERE ADDRESS_ID = 
                          (SELECT ADDRESS_ID FROM COMPANIES WHERE COMPANIES.COMPANY_ID = NEW.COMPANY_ID);
                   END""")
    return


def create_invoices(cur):
    """Invoices - list of invoices"""

    # Create INVOICES table

    cur.execute("""CREATE TABLE INVOICES
                      (
                      INVOICE_ID INTEGER PRIMARY KEY AUTOINCREMENT,
                      COMPANY_ID REFERENCES COMPANIES(COMPANY_ID),
                      CUSTOMER_ID REFERENCES CUSTOMERS(CUSTOMER_ID),
                      CUSTOMER_ADDRESS_ID REFERENCES ADDRESSES(ADDRESS_ID),
                      PAYMENT_ADDRESS_ID REFERENCES ADDRESSES(ADDRESS_ID),
                      INVOICE_NUMBER INTEGER UNIQUE,
                      CONTACT_ID REFERENCES CONTACTS(CONTACT_ID),
                      TRANSACTION_ID REFERENCES TRANSACTIONS(TRANSACTION_ID),
                      DATE_OF_SUPPLY DATE NOT NULL,
                      DATE_OF_TAXPOINT DATE NOT NULL,
                      DISCOUNT TEXT,
                      VAT TEXT,
                      COST TEXT,
                      PAYMENT_INSTRUCTIONS TEXT,
                      CUSTOMER_ORDER_NUMBER TEXT,
                      PRINT_CHECK INTEGER,
                      PDF_WIDTH INTEGER DEFAULT %s,
                      PDF_HEIGHT INTERGER DEFAULT %s,
                      PAID TEXT DEFAULT "0.00",
                      OPEN INTEGER DEFAULT 1,
                      CANCELLED INTEGER DEFAULT 0,
                      VAT_RETURN_ID REFERENCES VAT_RETURNS(VAT_RETURN_ID),
                      GRIDROW INTEGER NOT NULL DEFAULT -1,
                      SHOW_VAT INTEGER DEFAULT 1
                      )""" % vs_cfg["A4"])

    # OPEN = 1 (True) if this invoice is still active
    #      = 0 (False) if closed - fully paid or cancelled
    # CANCELLED = 0 (False) if the invoice is not cancelled
    #           = 1 (True) if the invoice has been cancelled

    # Whenever a new invoice is created, the INVOICE_NUMBER is stored in the configuration
    # table as LAST_INVOICE_NUMBER, and items referred to on the invoice, such as contact and
    # addresses must now be fixed - so if they are edited in future, new copies are made
    # and this invoice is not changed

    cur.execute("""CREATE TRIGGER new_invoice AFTER INSERT ON INVOICES
                   BEGIN
                       UPDATE CONFIGURATION SET CFG_ITEM = NEW.INVOICE_NUMBER WHERE CFG_KEY = "LAST_INVOICE_NUMBER";
                       UPDATE CONTACTS SET EDIT_NEW = 1 WHERE CONTACT_ID = NEW.CONTACT_ID;
                       UPDATE CUSTOMERS SET EDIT_NEW = 1 WHERE CUSTOMER_ID = NEW.CUSTOMER_ID;
                       UPDATE ADDRESSES SET EDIT_NEW = 1 WHERE ADDRESS_ID = NEW.CUSTOMER_ADDRESS_ID;
                       UPDATE ADDRESSES SET EDIT_NEW = 1 WHERE ADDRESS_ID = NEW.PAYMENT_ADDRESS_ID;

                       UPDATE ADDRESSES SET EDIT_NEW = 1 WHERE ADDRESS_ID = 
                          (SELECT ADDRESS_ID FROM COMPANIES WHERE COMPANIES.COMPANY_ID = NEW.COMPANY_ID);

                   END""")

    return


def create_solditems(cur):
    """A table linking Invoices and itemdescriptions"""

    # Create SOLDITEMS table

    # DISCOUNT is 0 for no discount applied, 1 for discount applied

    cur.execute("""CREATE TABLE SOLDITEMS
                      (
                      SOLDITEM_ID INTEGER PRIMARY KEY AUTOINCREMENT,
                      INVOICE_ID REFERENCES INVOICES(INVOICE_ID),
                      ITEMDESCRIPTION_ID INTEGER REFERENCES ITEMDESCRIPTIONS(ITEMDESCRIPTION_ID),
                      QUANTITY TEXT,
                      UNIT_COST TEXT,
                      DISCOUNT INTEGER,
                      VAT_PERCENT TEXT
                      )""")
    return


def create_invoicepayments(cur):
    """Links invoices with customerpayments, as one customer payment could be for several invoices
       or could just cover part of an invoice"""
    # Create INVOICEPAYMENTS table

    cur.execute("""CREATE TABLE INVOICEPAYMENTS
                      (
                      INVOICEPAYMENT_ID INTEGER PRIMARY KEY AUTOINCREMENT,
                      INVOICE_ID REFERENCES INVOICES(INVOICE_ID),
                      CUSTOMERPAYMENT_ID REFERENCES CUSTOMERPAYMENTS(CUSTOMERPAYMENT_ID),
                      COST TEXT,
                      VAT TEXT,
                      VAT_RETURN_ID REFERENCES VAT_RETURNS(VAT_RETURN_ID)
                      )""")


def create_customerpayments(cur):
    """CUSTOMERPAYMENTS - Table of customer payments received"""

    # Create CUSTOMERPAYMENTS table

    cur.execute("""CREATE TABLE CUSTOMERPAYMENTS
                      (
                      CUSTOMERPAYMENT_ID INTEGER PRIMARY KEY AUTOINCREMENT,
                      COMPANY_ID REFERENCES COMPANIES(COMPANY_ID),
                      CUSTOMER_ID REFERENCES CUSTOMERS(CUSTOMER_ID),
                      CUSTOMER_REFERENCE TEXT,
                      CUSTOMER_ADDRESS_ID REFERENCES ADDRESSES(ADDRESS_ID),
                      ACCOUNT_ID REFERENCES ACCOUNTS(ACCOUNT_ID),
                      TRANSACTION_ID REFERENCES TRANSACTIONS(TRANSACTION_ID),
                      AMOUNT TEXT
                      )""")

   # Set the customer edit_new flag so any change to the customer requires a new customer creating

   # The ACCOUNT_ID refers to the cash account - money goes from the
   # customer account and the vat balance account to this cash account, so after
   # a received payment, record the last cash account used - so it can be chosen on drop down lists

    cur.execute("""CREATE TRIGGER new_customerpayment AFTER INSERT ON CUSTOMERPAYMENTS
                   BEGIN
                       UPDATE CUSTOMERS SET EDIT_NEW = 1 WHERE CUSTOMER_ID = NEW.CUSTOMER_ID;
                       UPDATE ADDRESSES SET EDIT_NEW = 1 WHERE ADDRESS_ID = NEW.CUSTOMER_ADDRESS_ID;
                       UPDATE CONFIGURATION SET CFG_ITEM = NEW.ACCOUNT_ID WHERE CFG_KEY = "LAST_CASH_ACCOUNT_ID";

                       UPDATE ADDRESSES SET EDIT_NEW = 1 WHERE ADDRESS_ID = 
                          (SELECT ADDRESS_ID FROM COMPANIES WHERE COMPANIES.COMPANY_ID = NEW.COMPANY_ID);
                   END""")
    return


def create_creditnotes(cur):
    """CreditNotes - Table of credit notes created"""

    # Create CREDITNOTES table

    cur.execute("""CREATE TABLE CREDITNOTES
                      (
                      CREDITNOTE_ID INTEGER PRIMARY KEY AUTOINCREMENT,
                      COMPANY_ID REFERENCES COMPANIES(COMPANY_ID),
                      CUSTOMER_ID REFERENCES CUSTOMERS(CUSTOMER_ID),
                      PRINT_CHECK INTEGER,
                      PDF_WIDTH INTEGER DEFAULT %s,
                      PDF_HEIGHT INTERGER DEFAULT %s,
                      CUSTOMER_ADDRESS_ID REFERENCES ADDRESSES(ADDRESS_ID),
                      INVOICE_ID REFERENCES INVOICES(INVOICE_ID),
                      REASON TEXT,
                      CREDITNOTE_NUMBER INTEGER UNIQUE,
                      CONTACT_ID REFERENCES CONTACTS(CONTACT_ID),
                      TRANSACTION_ID REFERENCES TRANSACTIONS(TRANSACTION_ID),
                      DISCOUNT TEXT,
                      VAT TEXT,
                      COST TEXT,
                      GRIDROW INTEGER NOT NULL DEFAULT -1,
                      STATUS INTEGER DEFAULT 0,
                      VAT_RETURN_ID REFERENCES VAT_RETURNS(VAT_RETURN_ID),
                      SHOW_VAT INTEGER DEFAULT 1
                      )""" % vs_cfg["A4"])

    # STATUS - normally 0 when a creditnote is sent against an invoice
    # STATUS - 1 when a creditnote is sent with a payment

    # Whenever a new credit note is created, the CREDITNOTE_NUMBER is stored in the configuration
    # table as LAST_CREDITNOTE_NUMBER, and items referred to on the creditnote, such as contact and
    # addresses must now be fixed - so if they are edited in future, new copies are made
    # and this creditnote is not changed

    cur.execute("""CREATE TRIGGER new_credit_note AFTER INSERT ON CREDITNOTES
                   BEGIN
                   UPDATE CONFIGURATION SET CFG_ITEM = NEW.CREDITNOTE_NUMBER WHERE CFG_KEY = "LAST_CREDITNOTE_NUMBER";
                   UPDATE CONTACTS SET EDIT_NEW = 1 WHERE CONTACT_ID = NEW.CONTACT_ID;
                   UPDATE CUSTOMERS SET EDIT_NEW = 1 WHERE CUSTOMER_ID = NEW.CUSTOMER_ID;
                   UPDATE ADDRESSES SET EDIT_NEW = 1 WHERE ADDRESS_ID = NEW.CUSTOMER_ADDRESS_ID;

                   UPDATE ADDRESSES SET EDIT_NEW = 1 WHERE ADDRESS_ID = 
                          (SELECT ADDRESS_ID FROM COMPANIES WHERE COMPANIES.COMPANY_ID = NEW.COMPANY_ID);

                   END""")

    return


def create_creditnoteitems(cur):
    """A table linking credit notes and itemdescriptions"""

    # Create CREDITNOTEITEMS table

    # DISCOUNT is 0 for no discount applied, 1 for discount applied

    cur.execute("""CREATE TABLE CREDITNOTEITEMS
                      (
                      CREDITNOTEITEM_ID INTEGER PRIMARY KEY AUTOINCREMENT,
                      CREDITNOTE_ID REFERENCES CREDITNOTES(CREDITNOTE_ID),
                      ITEMDESCRIPTION_ID INTEGER REFERENCES ITEMDESCRIPTIONS(ITEMDESCRIPTION_ID),
                      QUANTITY TEXT,
                      UNIT_COST TEXT,
                      DISCOUNT INTEGER,
                      VAT_PERCENT TEXT
                      )""")
    return



