####### VATStuff #######
#
# save_closeorder.py  - function to close a purchase order
#
# Version : 0.7.0
# Date : 20100928
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#


"""Provides a function to close a purchase order"""


import datetime
import decimal

from vatstuff_packages.vs_utilities import *

from vatstuff_packages.vs_database.transactions import *

D = decimal.Decimal   # D used to avoid typing


def save(order, status, closed_reason=None, transaction_form=None):
    "Implements the transaction and CLOSEDORDERS table entry"

    if status != 2:
        assert transaction_form.system == 8

    # STATUS
    # 0 if this Purchase Order is still open, no invoices received
    # 1 order open, and one or more invoices have been received
    # 2 if normally closed and order fully paid, i.e total == paid
    # 3 if closed, with order total less than that paid, i.e. total<paid
    # 4 if closed, with order total greater than that paid, i.e. total>paid
    # 5 if closed by the order being cancelled with no invoice recieved or payment made

    # if status is 2, then this order is simply closed without any reason or transaction being created
    # however if status is 3 to 5 then a CLOSEDORDERS table entry is made with a reason and a transaction
    # is created

    assert status > 1
    # Function is to close an order, status must be 2 or above

    assert order.is_open
    # The current status must be open

    # Open database,  Note - all these transactions are done, and only committed at the end.

    conn = None
    try:
        conn, c = vs_connect()
    
        if status == 2:
            # Normal close
            c.execute("""UPDATE PURCHASEORDERS SET 
                         STATUS = ?
                         WHERE PURCHASEORDER_ID = ?""", (2, order.id))
            # And commit the change and return
            conn.commit()
            return None

        # So status is 3, 4 or 5
    
        # Do the transaction    
        transaction_id = implement(c, transaction_form)
    
        # Create a new closedorders table entry
    
        c.execute("""INSERT INTO CLOSEDORDERS
                          (CLOSEDORDER_ID, TRANSACTION_ID, REASON)
                           VALUES (?, ?, ?)""", (None, transaction_id, closed_reason))
    
        c.execute("SELECT last_insert_rowid()")
        result = c.fetchone()
        closedorder_id = result[0]
    
        # Update the purchase order
        
        c.execute("""UPDATE PURCHASEORDERS SET 
                        CLOSEDORDER_ID = ?,
                        STATUS = ?
                        WHERE PURCHASEORDER_ID = ?""", (closedorder_id, status, order.id))

        # And commit the changes
        conn.commit()
    finally:
        if conn:
            conn.close()

    info_log("Closed order number: %s : id: %s : closeorder id: %s" % (order.number, order.id, closedorder_id))
    return ClosedOrder(closedorder_id)


