####### VATStuff #######
#
# save_creditnote.py  - functions to save a credit note
#
# Version : 0.7.0
# Date : 20100926
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#


"""Provides functions to transact a credit note"""


import decimal

from vatstuff_packages.vs_utilities import *

from vatstuff_packages.vs_database.transactions import *


D = decimal.Decimal   # D used to avoid typing


def save(creditnote, transaction_form):
    """Implements the transaction and saves a CREDITNOTE as defined by CreditNoteForm object creditnote
     this being a creditnote you are issuing to a custumer"""

    # Creditnote has a status value
    #  - normally 0 when a creditnote is sent against an invoice
    #  - 1 when a creditnote is sent with a payment - no invoice

    # Note - this is a transaction system type 7
    assert transaction_form.system == 7

    assert creditnote.cost+creditnote.vat > D(0)

    if creditnote.status:
         # creditnote is sent with a payment - no invoice
         invoice_id = None
    else:
        # The creditnote is raised against an invoice
        invoice = creditnote.invoice
        invoice_id = invoice.id
        assert invoice.is_open

        total_on_note = creditnote.cost+creditnote.vat
        already_paid = invoice.paid

        assert total_on_note <= invoice.cost + invoice.vat - already_paid

        invoice_open = 1
        if total_on_note == invoice.cost + invoice.vat - already_paid:
            # invoice will be fully paid, close it
            invoice_open = 0

    if creditnote.print_check:
        print_check = 1
    else:
        print_check = 0

    if creditnote.show_vat:
        show_vat = 1
    else:
        show_vat = 0

    if isinstance(creditnote.customer_address, Address):
        customer_address_id = creditnote.customer_address.id
    else:
        customer_address_id = None

    if isinstance(creditnote.contact, Contact):
        contact_id = creditnote.contact.id
    else:
        contact_id = None

    # Also get a local copy of the item list
    note_list = creditnote.item_list

    # Open database,  Note - all these transactions are done, and only committed at the end.
    conn = None
    try:
        conn, c = vs_connect()

        # Do the transaction
        transaction_id = implement(c, transaction_form)

        # Create a new creditnote

        new_creditnote = (None,
                          creditnote.company.id,
                          creditnote.customer.id,
                          print_check,
                          creditnote.pdf_width,
                          creditnote.pdf_height,
                          customer_address_id,
                          invoice_id,
                          creditnote.reason,
                          creditnote.number,
                          contact_id,  
                          transaction_id,
                          str(creditnote.discount),
                          str(creditnote.vat),
                          str(creditnote.cost),
                          -1,                            # temp gridrow object
                          creditnote.status,
                          show_vat,                      # Show the VAT column
                          None                           # The VAT Return
                               )
    
        c.execute("""INSERT INTO CREDITNOTES
                          (CREDITNOTE_ID,
                           COMPANY_ID,
                           CUSTOMER_ID,
                           PRINT_CHECK,
                           PDF_WIDTH,
                           PDF_HEIGHT,
                           CUSTOMER_ADDRESS_ID,
                           INVOICE_ID,
                           REASON,
                           CREDITNOTE_NUMBER,
                           CONTACT_ID,
                           TRANSACTION_ID,
                           DISCOUNT,
                           VAT,
                           COST,
                           GRIDROW,
                           STATUS,
                           SHOW_VAT,
                           VAT_RETURN_ID
                           ) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)""",
                            new_creditnote)
    
        c.execute("SELECT last_insert_rowid()")
        result = c.fetchone()
        note_id = result[0]
    
        # the gridrow number has to be found, and any creditnote dated after this
        # one requires its gridrow number to be incremented

        # Get the gridrow number of the creditnote, being the count of the previous
        # rows (as gridrow starts at zero)

        c.execute("""SELECT COUNT(*) FROM CREDITNOTES, TRANSACTIONS
                     WHERE DATESTAMP <= ? AND
                     CREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                     AND CREDITNOTES.CREDITNOTE_ID != ?""",
                     (transaction_form.datestamp, note_id))
        result=c.fetchone()
        gridrow = result[0]
    
        # Increment the gridrow of all creditnotes with later or equal gridrows
        c.execute("""UPDATE CREDITNOTES SET GRIDROW = GRIDROW+1
                     WHERE GRIDROW >= ?""", (gridrow, ))
    
        # Set the creditnote to have this gridrow
        c.execute("""UPDATE CREDITNOTES SET GRIDROW = ?
                     WHERE CREDITNOTE_ID = ?""", (gridrow, note_id))
    
        if not creditnote.status:
            # the creditnote is associated with an invoice - update the invoice
            new_total_paid = total_on_note + already_paid
    
            c.execute("UPDATE INVOICES SET PAID = ?, OPEN = ? WHERE INVOICE_ID = ?",
                       (str(new_total_paid), invoice_open, invoice_id))


        # For each item in note_list, compare its description to see if its a ItemDescription object
        # and create a list of item descriptions

        itemdesc_list = []
        for item in note_list:
            description = item["Description"]
            # Check if this description exists
            c.execute("""SELECT COUNT(*) FROM ITEMDESCRIPTIONS
                                           WHERE DESCRIPTION = ?""", (description,))
            result = c.fetchone()
            description_count = result[0]
            if description_count > 0:
                # Get the itemdescription which already exists in ITEMDESCRIPTIONS
                c.execute("""SELECT ITEMDESCRIPTION_ID FROM ITEMDESCRIPTIONS
                                           WHERE DESCRIPTION = ?""", (description,))
                result = c.fetchone()
                itemdescription_id = result[0]
            else:
                # itemdescription does not exist, so create one
                c.execute("""INSERT INTO ITEMDESCRIPTIONS 
                            (ITEMDESCRIPTION_ID,
                             DESCRIPTION
                             ) VALUES (?, ?)""", (None, description))
                c.execute("SELECT last_insert_rowid()")
                result = c.fetchone()
                itemdescription_id = result[0]
            itemdesc_list.append(itemdescription_id)
    
        # Create the CREDITNOTEITEMS table entry
        for index, item in enumerate(note_list):
            if item["Discount"]:
                discount = 1
            else:
                discount = 0
            
            new_creditnoteitem = [None,
                                  note_id,
                                  itemdesc_list[index],
                                  str(item["Qty"]),
                                  str(item["Unit Cost"]),
                                  discount,
                                  str(item["VAT"])]

            c.execute("""INSERT INTO CREDITNOTEITEMS 
                             (
                             CREDITNOTEITEM_ID,
                             CREDITNOTE_ID,
                             ITEMDESCRIPTION_ID,
                             QUANTITY,
                             UNIT_COST,
                             DISCOUNT,
                             VAT_PERCENT
                             ) VALUES (?, ?, ?, ?, ?, ?, ?)""",
                             new_creditnoteitem)


        # And commit the changes
        conn.commit()
    finally:
        if conn:
            conn.close()

    info_log("Credit note saved number: %s : id: %s" % (creditnote.number, note_id))
    return CreditNote(note_id)

################################################################################################




