####### VATStuff #######
#
# save_invoice.py  - functions to save an invoice
#
# Version : 0.7.0
# Date : 20100925
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#


"""Provides functions to save an invoice"""

import decimal

from vatstuff_packages.vs_utilities import *

from vatstuff_packages.vs_database.transactions import *


D = decimal.Decimal   # D used to avoid typing


def save(invoice, transaction_form):
    "Implements the transaction and saves an invoice as defined by InvoiceForm object invoice"

    # Note - this is a transaction system type 5
    assert transaction_form.system == 5

    if invoice.print_check:
        print_check = 1
    else:
        print_check = 0

    if invoice.show_vat:
        show_vat = 1
    else:
        show_vat = 0

    if isinstance(invoice.customer_address, Address):
        customer_address_id = invoice.customer_address.id
    else:
        customer_address_id = None

    if isinstance(invoice.payment_address, Address):
        payment_address_id = invoice.payment_address.id
    else:
        payment_address_id = None

    if isinstance(invoice.contact, Contact):
        contact_id = invoice.contact.id
    else:
        contact_id = None

    # Get a local copy of the item list
    invoice_list = invoice.item_list

    # Open database,  Note - all these transactions are done, and only committed at the end.

    conn = None
    try:
        conn, c = vs_connect()

        # Do the transaction
        transaction_id = implement(c, transaction_form)

        # Create a new invoice
        new_invoice=(None,
                   invoice.company.id,             # This company id
                   invoice.customer.id,            # The customer id to whom the invoice is being sent
                   customer_address_id,            # The customer address id where the order is sent
                   payment_address_id,             # address id where payments should be sent
                   invoice.number,                 # The invoice number
                   contact_id,                     # The contact_id for enquiries
                   transaction_id,                 # The transaction id associated with this invoice
                   invoice.date_of_supply,         # the date of goods supply, as given on the form
                   invoice.date_of_taxpoint,       # The date when VAT is calculated
                   str(invoice.discount),          # The discount associated with this invoice
                   str(invoice.vat),               # Total VAT
                   str(invoice.cost),              # Total cost (net - exclusive of VAT)
                   invoice.payment_instructions,   # The payment instructions text
                   invoice.cus_order_number,       # The customers order number
                   print_check,                    # 1 if country to be show in addresses
                   invoice.pdf_width,              # pdf form width
                   invoice.pdf_height,             # pdf form height
                   "0.00",                         # Amount paid, initially zero
                   1,                              # 1 for True, the order is open
                   0,                              # 0 for False, the order is not cancelled
                   -1,                             # temp gridrow object
                   show_vat,                       # Show the VAT column
                   None                            # The vat return id
                   )

        # insert the new invoice

        c.execute("""INSERT INTO INVOICES
                          (
                          INVOICE_ID,
                          COMPANY_ID,
                          CUSTOMER_ID,
                          CUSTOMER_ADDRESS_ID,
                          PAYMENT_ADDRESS_ID,
                          INVOICE_NUMBER,
                          CONTACT_ID,
                          TRANSACTION_ID,
                          DATE_OF_SUPPLY,
                          DATE_OF_TAXPOINT,
                          DISCOUNT,
                          VAT,
                          COST,
                          PAYMENT_INSTRUCTIONS,
                          CUSTOMER_ORDER_NUMBER,
                          PRINT_CHECK,
                          PDF_WIDTH,
                          PDF_HEIGHT,
                          PAID,
                          OPEN,
                          CANCELLED,
                          GRIDROW,
                          SHOW_VAT,
                          VAT_RETURN_ID
                          ) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)""",
                                  new_invoice)
    
        c.execute("SELECT last_insert_rowid()")
        result = c.fetchone()
        invoice_id = result[0]

        # the gridrow number has to be found, and any invoice dated after this
        # one requires its gridrow number to be incremented

        # Get the gridrow number of the invoice, being the count of the previous
        # rows (as gridrow starts at zero)

        c.execute("""SELECT COUNT(*) FROM INVOICES, TRANSACTIONS
                     WHERE DATESTAMP <= ? AND
                     INVOICES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                     AND INVOICES.INVOICE_ID != ?""",
                     (transaction_form.datestamp, invoice_id))
        result=c.fetchone()
        gridrow = result[0]

        # Increment the gridrow of all invoices with later or equal gridrows
        c.execute("""UPDATE INVOICES SET GRIDROW = GRIDROW+1
                     WHERE GRIDROW >= ?""", (gridrow, ))
    
        # Set the invoice to have this gridrow
        c.execute("""UPDATE INVOICES SET GRIDROW = ?
                     WHERE INVOICE_ID = ?""", (gridrow, invoice_id))

        # invoice_list
        # Each invoice_list element being a dictionary of
        # {"Qty":Decimal, "Description":string, "Unit Cost":Decimal, "Discount":boolean "VAT":Decimal (The % value)}
        # Qty is a Decimal value
        # Description is a string
        # Unit Cost is a Decimal value
        # VAT is a Decimal value - being the percentage

        # For each item in invoice_list, compare its description to see if its a ItemDescription object
        # and create a list of item descriptions

        itemdesc_list = []
        for item in invoice_list:
            description = item["Description"]
            # Check if this description exists
            c.execute("""SELECT COUNT(*) FROM ITEMDESCRIPTIONS
                                       WHERE DESCRIPTION = ?""", (description,))
            result = c.fetchone()
            description_count = result[0]
            if description_count > 0:
                # Get the itemdescription which already exists in ITEMDESCRIPTIONS
                c.execute("""SELECT ITEMDESCRIPTION_ID FROM ITEMDESCRIPTIONS
                                       WHERE DESCRIPTION = ?""", (description,))
                result = c.fetchone()
                itemdescription_id = result[0]
            else:
                # itemdescription does not exist, so create one
                c.execute("""INSERT INTO ITEMDESCRIPTIONS 
                            (ITEMDESCRIPTION_ID,
                             DESCRIPTION
                             ) VALUES (?, ?)""", (None, description))
                c.execute("SELECT last_insert_rowid()")
                result = c.fetchone()
                itemdescription_id = result[0]
            itemdesc_list.append(itemdescription_id)

        # Create the SOLDITEMS table entry
        for index, item in enumerate(invoice_list):
            if item["Discount"]:
                discount = 1
            else:
                discount = 0
            
            new_solditem = [None,
                            invoice_id,
                            itemdesc_list[index],
                            str(item["Qty"]),
                            str(item["Unit Cost"]),
                            discount,
                            str(item["VAT"])]

            c.execute("""INSERT INTO SOLDITEMS 
                             (
                             SOLDITEM_ID,
                             INVOICE_ID,
                             ITEMDESCRIPTION_ID,
                             QUANTITY,
                             UNIT_COST,
                             DISCOUNT,
                             VAT_PERCENT
                             ) VALUES (?, ?, ?, ?, ?, ?, ?)""",
                             new_solditem)
    
        # And commit the changes
        conn.commit()
    finally:
        if conn:
            conn.close()

    info_log("Invoice saved number: %s : id: %s" % (invoice.number, invoice_id))
    return Invoice(invoice_id)

