####### VATStuff #######
#
# save_makepayment.py  - function to create the payment to a supplier
#
# Version : 0.7.0
# Date : 20100918
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#


"""Function to create the payment to a supplier"""


import datetime
import decimal

from vatstuff_packages.vs_utilities import *

from vatstuff_packages.vs_database.transactions import *

D = decimal.Decimal   # D used to avoid typing


def save(payment,
         transaction_date,
         transaction_text,
         transaction_list):
    """Implements the transaction and saves a MakePayment as defined by MakePaymentForm object payment
       Returns (True, MakePayment) on success, (False, None) on failure"""

    if not payment: return False, None
    amount = payment.amount
    if amount <= D(0): return False, None

    if isinstance(payment.supplier_address, Address):
        supplier_address_id = payment.supplier_address.id
    else:
        supplier_address_id = None

    purchaseorder = payment.purchaseorder
    if not purchaseorder.open_and_invoiced: return False, None

    # invoice_amount is the sum of all invoice amounts less all credit note amounts
    invoice_amount = purchaseorder.invoice_cost + purchaseorder.invoice_vat

    # The new paid amount on the purchaseorder will be
    new_total_paid = payment.amount + purchaseorder.paid

    # If new_total_paid == invoice_amount == total on order
    # Then all invoices paid, and the order is paid, so can safely close the order

    # If new_total_paid == invoice_amount but not equal to total on order then
    # the invoices are settled, but not the order, and the user will have to be asked
    # if he wants to close - this to be done back in the gui

    po_close = False
    amount_on_order = purchaseorder.cost + purchaseorder.vat
    if new_total_paid == amount_on_order and new_total_paid == invoice_amount:
        po_close = True


    # For each invoice, covered by this payment, create a list of lists,
    # [invoice_id, new_paid, invoice_open, cost, vat, vat_return_id]
    # invoice_open is 1 if the invoice is still open, or 0 if now to be fully paid
 
    invoice_items = []

    for inv in payment.supplierinvoice_list:
        vat_return = inv["INVOICE"].vat_return
        if vat_return:
            vat_return_id = vat_return.id
        else:
            vat_return_id = None
        if inv["INVOICE"].claim_vat:
            claim_vat = 1
        else:
            claim_vat = 0
        new_paid = inv["INVOICE"].paid + inv["COST"] + inv["VAT"]
        if new_paid > inv["INVOICE"].total:
            return False, None
        if new_paid == inv["INVOICE"].total:
            # invoice is to be fully paid
            invoice_items.append( [inv["INVOICE"].id, new_paid, 0, inv["COST"], inv["VAT"], vat_return_id, claim_vat] )
        else:
            # invoice is still open
            invoice_items.append( [inv["INVOICE"].id, new_paid, 1, inv["COST"], inv["VAT"], vat_return_id, claim_vat] )


    # Create a new MakePayment - note list object, not tuple, as an append is done

    new_makepayment = [None,
                       payment.company.id,
                       payment.supplier.id,
                       supplier_address_id,
                       purchaseorder.id,
                       payment.account.id,
                       payment.status,
                       str(payment.amount)
                       ]


    # Note - this is a transaction system type 3
    #
    #   0 - User defined transaction - using the move buttons
    #   1 - This was an opening balance transaction
    #   2 - Create a purchase order transaction
    #   3 - Pay a supplier transaction
    #   4 - Payment from a supplier - creditnote transaction
    #   5 - Create an invoice transaction
    #   6 - Customer payment transaction
    #   7 - Create a credit note transaction
    #   8 - An order close transaction

    # Open database,  Note - all these transactions are done, and only committed at the end. 

    conn = None
    try:
        conn, c = vs_connect()

        transaction_id = implement(c, transaction_date, transaction_text, transaction_list, 3)

        new_makepayment.append(transaction_id)


        c.execute("""INSERT INTO MAKEPAYMENTS
                          (MAKEPAYMENT_ID,
                           COMPANY_ID,
                           SUPPLIER_ID,
                           SUPPLIER_ADDRESS_ID,
                           PURCHASEORDER_ID,
                           ACCOUNT_ID,
                           STATUS,
                           AMOUNT,
                           TRANSACTION_ID
                           ) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)""",
                            new_makepayment)

        c.execute("SELECT last_insert_rowid()")
        result = c.fetchone()
        makepayment_id = result[0]

        c.execute("UPDATE PURCHASEORDERS SET PAID = ? WHERE PURCHASEORDER_ID = ?",
                   (str(new_total_paid), purchaseorder.id))

        if po_close:
            # purchase order status is set to 2, which is the normal closed order status
            c.execute("UPDATE PURCHASEORDERS SET STATUS = 2 WHERE PURCHASEORDER_ID = ?", (purchaseorder.id,))


        for inv in invoice_items:
            #  inv = [supplierinvoice_id, new_paid, invoice_open, cost, vat, vat_return_id]
            c.execute("UPDATE SUPPLIERINVOICES SET PAID = ?, OPEN = ? WHERE SUPPLIERINVOICE_ID = ?",
                       (str(inv[1]), inv[2], inv[0]))

            # If the supplierinvoice has already been submitted in a VAT return - so have
            # these payments so this is recorded in the vat_return_id
            new_supplierinvoicepayment = (None, inv[0], makepayment_id, str(inv[3]), str(inv[4]), inv[5], inv[6])

            # save the makepayment - supplierinvoice link in SUPPLIERINVOICEPAYMENTS
            c.execute("""INSERT INTO SUPPLIERINVOICEPAYMENTS
                                (SUPPLIERINVOICEPAYMENT_ID,
                                 SUPPLIERINVOICE_ID,
                                 MAKEPAYMENT_ID,
                                 COST,
                                 VAT,
                                 VAT_RETURN_ID,
                                 CLAIM_VAT
                                 ) VALUES (?, ?, ?, ?, ?, ?, ?)""", new_supplierinvoicepayment)

        # And commit the changes
        conn.commit()
    finally:
        if conn:
            conn.close()

    info_log("Payment made id: %s" % makepayment_id)

    return True, MakePayment(makepayment_id)

################################################################################################




