####### VATStuff #######
#
# save_payment.py  - Provides functions to receive a customers payment
#
# Version : 0.7.0
# Date : 20100926
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#

"""Provides functions to receive a customers payment"""


import datetime
import decimal

from vatstuff_packages.vs_utilities import *

from vatstuff_packages.vs_database.transactions import *

D = decimal.Decimal   # D used to avoid typing


def save(pay, transaction_form):
    """Implements the transaction and saves a CustomerPayment as defined by
       PaymentForm object pay. Returns CustomerPayment"""

    # Note - this is a transaction system type 6
    assert transaction_form.system == 6

    amount = pay.amount
    assert amount > D(0)

    # For each invoice, covered by this payment, create a list of lists,
    # [invoice_id, new_paid, invoice_open, cost, vat, vat_return_id]
    # invoice_open is 1 if the invoice is still open, or 0 if now to be fully paid
 
    invoice_items = []

    for inv in pay.invoice_list:
        vat_return = inv["INVOICE"].vat_return
        if vat_return:
            vat_return_id = vat_return.id
        else:
            vat_return_id = None
        new_paid = inv["INVOICE"].paid + inv["COST"] + inv["VAT"]
        if new_paid > inv["INVOICE"].total:
            return False, None
        if new_paid == inv["INVOICE"].total:
            # invoice is to be fully paid
            invoice_items.append( [inv["INVOICE"].id, new_paid, 0, inv["COST"], inv["VAT"], vat_return_id] )
        else:
            # invoice is still open
            invoice_items.append( [inv["INVOICE"].id, new_paid, 1, inv["COST"], inv["VAT"], vat_return_id] )

    customer_address_id = None
    if pay.customer_address:
        customer_address_id = pay.customer_address.id


    new_customerpayment = [None,
                           pay.company.id,
                           pay.customer.id,
                           pay.customer_reference,
                           customer_address_id,
                           pay.account.id,
                           str(pay.amount)
                           ]

    # Open database,  Note - all these transactions are done, and only committed at the end. 

    conn = None
    try:
        conn, c = vs_connect()

        # Do the transaction
        transaction_id = implement(c, transaction_form)

        # Create a new CUSTOMERPAYMENT

        new_customerpayment.append(transaction_id)

        c.execute("""INSERT INTO CUSTOMERPAYMENTS
                          (CUSTOMERPAYMENT_ID,
                           COMPANY_ID,
                           CUSTOMER_ID,
                           CUSTOMER_REFERENCE,
                           CUSTOMER_ADDRESS_ID,
                           ACCOUNT_ID,
                           AMOUNT,
                           TRANSACTION_ID
                           ) VALUES (?, ?, ?, ?, ?, ?, ?, ?)""",
                            new_customerpayment)

        c.execute("SELECT last_insert_rowid()")
        result = c.fetchone()
        customerpayment_id = result[0]


        for inv in invoice_items:
            #  inv = [invoice_id, new_paid, invoice_open, cost, vat, vat_return_id]
            c.execute("UPDATE INVOICES SET PAID = ?, OPEN = ? WHERE INVOICE_ID = ?",
                       (str(inv[1]), inv[2], inv[0]))
    
            new_invoicepayment = (None, inv[0], customerpayment_id, str(inv[3]), str(inv[4]), inv[5])

            # save the customer payment - invoice link in INVOICEPAYMENTS
            c.execute("""INSERT INTO INVOICEPAYMENTS
                                (INVOICEPAYMENT_ID,
                                 INVOICE_ID,
                                 CUSTOMERPAYMENT_ID,
                                 COST,
                                 VAT,
                                 VAT_RETURN_ID
                                 ) VALUES (?, ?, ?, ?, ?, ?)""", new_invoicepayment)

        # And commit the changes
        conn.commit()
    finally:
        if conn:
            conn.close()

    info_log("Payment received id: %s" % customerpayment_id)

    return CustomerPayment(customerpayment_id)


