####### VATStuff #######
#
# save_purchaseorder.py  - Creates a purchaseorder on the database with its transaction
#
# Version : 0.7.0
# Date : 20100912
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#

"""Creates a purchaseorder on the database with its transaction"""


import decimal

from vatstuff_packages.vs_utilities import *
from vatstuff_packages.vs_database.transactions import *

D = decimal.Decimal   # D used to avoid typing


def save(order, transaction_form):
    "Implements the transaction and saves a purchase order as defined by OrderForm object order"

    # Note - this is a transaction system type 2
    assert transaction_form.system == 2

    if order.print_check:
        print_check = 1
    else:
        print_check = 0

    if order.show_vat:
        show_vat = 1
    else:
        show_vat = 0

    if order.claim_vat:
        claim_vat = 1
    else:
        claim_vat = 0

    if isinstance(order.supplier_address, Address):
        supplier_address_id = order.supplier_address.id
    else:
        supplier_address_id = None

    if isinstance(order.invoice_address, Address):
        invoice_address_id = order.invoice_address.id
    else:
        invoice_address_id = None

    if isinstance(order.delivery_address, Address):
        delivery_address_id = order.delivery_address.id
    else:
        delivery_address_id = None

    if isinstance(order.contact, Contact):
        contact_id = order.contact.id
    else:
        contact_id = None

    # datestamp used several times, so get local copy
    order_datestamp = order.datestamp

    # Also get a local copy of the item list
    order_list = order.item_list

    # Create a new order
    new_order=[None,
               order.company.id,             # This company id
               order.supplier.id,            # The supplier id to whom the order is being sent
               supplier_address_id,          # The suppliers address id where the order is sent
               invoice_address_id,           # address id where invoices should be sent
               delivery_address_id,          # address id where deliveries should be sent
               None,                         # project id, initially None
               order.number,                 # The purchase order number
               contact_id,                   # The contact_id for enquiries
               str(order.discount),          # The discount associated with this order
               str(order.vat),               # Total VAT
               str(order.cost),              # Total cost (net - exclusive of VAT)
               order.sup_ref_field,          # The suppliers reference text
               print_check,                  # 1 if country to be show in addresses
               order.pdf_width,              # pdf form width
               order.pdf_height,             # pdf form height
               "0.00",                       # Invoice VAT, initially zero
               "0.00",                       # Invoice cost, initially zero
               "0.00",                       # Amount paid, initially zero
               0,                            # Status 0, the order is open
               None,                         # None, the order is not cancelled or closed
               -1,                           # temp gridrow object
               show_vat,                     # Show the VAT column
               claim_vat                     # True if this VAT is to be claimed - relevent to STANDARD and CASH methods
               ]                             # the FLAT and non registered do not use this flag at all, as none of
                                             # them claim Input VAT back.


    # Open database,  Note - all these transactions are done, and only committed at the end. 
    conn = None
    try:
        conn, c = vs_connect()

        # Do the transaction

        transaction_id = implement(c, transaction_form)
        new_order.append(transaction_id)

        # insert the new order
        c.execute("""INSERT INTO PURCHASEORDERS 
                                (PURCHASEORDER_ID,
                                 COMPANY_ID,
                                 SUPPLIER_ID,
                                 SUPPLIER_ADDRESS_ID,
                                 INVOICE_ADDRESS_ID,
                                 DELIVERY_ADDRESS_ID,
                                 PROJECT_ID,
                                 ORDER_NUMBER,
                                 CONTACT_ID,
                                 DISCOUNT,
                                 VAT,
                                 COST,
                                 SUPPLIER_REFERENCE,
                                 PRINT_CHECK,
                                 PDF_WIDTH,
                                 PDF_HEIGHT,
                                 INVOICE_VAT,
                                 INVOICE_COST,
                                 PAID,
                                 STATUS,
                                 CLOSEDORDER_ID,
                                 GRIDROW,
                                 SHOW_VAT,
                                 CLAIM_VAT,
                                 TRANSACTION_ID
                                ) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)""",
                                  new_order)

        c.execute("SELECT last_insert_rowid()")
        result = c.fetchone()
        purchaseorder_id = result[0]

        # the gridrow number has to be found, and any purchaseorder dated after this
        # one requires its gridrow number to be incremented

        # Get the gridrow number of the purchaseorder, being the count of the previous
        # rows (as gridrow starts at zero)

        c.execute("""SELECT COUNT(*) FROM PURCHASEORDERS, TRANSACTIONS
                     WHERE DATESTAMP <= ? AND
                     PURCHASEORDERS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                     AND PURCHASEORDERS.PURCHASEORDER_ID != ?""",
                     (transaction_form.datestamp, purchaseorder_id))
        result=c.fetchone()
        gridrow = result[0]

        # Increment the gridrow of all purchaseorders with later or equal gridrows
        c.execute("""UPDATE PURCHASEORDERS SET GRIDROW = GRIDROW+1
                     WHERE GRIDROW >= ?""", (gridrow, ))

        # Set the purchaseorder to have this gridrow
        c.execute("""UPDATE PURCHASEORDERS SET GRIDROW = ?
                     WHERE PURCHASEORDER_ID = ?""", (gridrow, purchaseorder_id))


        # order_list
        # Each item_list element being a dictionary of
        # {"Qty":Decimal, "Description":string, "Unit Cost":Decimal, "Discount":boolean "VAT":Decimal (The % value)}
        # Qty is a Decimal value
        # Description is a string
        # Unit Cost is a Decimal value
        # VAT is a Decimal value - being the percentage

        # For each item in order_list, compare its description to see if its a ItemDescription object
        # and create a list of item descriptions

        itemdesc_list = []
        for item in order_list:
            description = item["Description"]
            # Check if this description exists
            c.execute("""SELECT COUNT(*) FROM ITEMDESCRIPTIONS
                                           WHERE DESCRIPTION = ?""", (description,))
            result = c.fetchone()
            description_count = result[0]
            if description_count > 0:
                # Get the itemdescription which already exists in ITEMDESCRIPTIONS
                c.execute("""SELECT ITEMDESCRIPTION_ID FROM ITEMDESCRIPTIONS
                                           WHERE DESCRIPTION = ?""", (description,))
                result = c.fetchone()
                itemdescription_id = result[0]
            else:
                # itemdescription does not exist, so create one
                c.execute("""INSERT INTO ITEMDESCRIPTIONS 
                            (ITEMDESCRIPTION_ID,
                             DESCRIPTION
                             ) VALUES (?, ?)""", (None, description))
                c.execute("SELECT last_insert_rowid()")
                result = c.fetchone()
                itemdescription_id = result[0]
            itemdesc_list.append(itemdescription_id)

        # Create the ORDEREDITEMS table entry
        for index, item in enumerate(order_list):
            if item["Discount"]:
                discount = 1
            else:
                discount = 0
            
            new_ordereditem = [None,
                               purchaseorder_id,
                               itemdesc_list[index],
                               str(item["Qty"]),
                               str(item["Unit Cost"]),
                               discount,
                               str(item["VAT"])]

            c.execute("""INSERT INTO ORDEREDITEMS 
                             (
                             ORDEREDITEM_ID,
                             PURCHASEORDER_ID,
                             ITEMDESCRIPTION_ID,
                             QUANTITY,
                             UNIT_COST,
                             DISCOUNT,
                             VAT_PERCENT
                             ) VALUES (?, ?, ?, ?, ?, ?, ?)""",
                             new_ordereditem)

        # And commit the changes
        conn.commit()
    finally:
        if conn:
            conn.close()

    info_log("Purchase order saved number: %s : id: %s" % (order.number, purchaseorder_id))
    return PurchaseOrder(purchaseorder_id)


