####### VATStuff #######
#
# save_suppliercreditnote.py  - Provides function to create a suppliers credit note
#
# Version : 0.7.0
# Date : 20100914
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#


"""Provides function to create a suppliers credit note"""

from vatstuff_packages.vs_database.transactions import *
from vatstuff_packages.vs_utilities import *


def save(suppliercreditnote,
         transaction_date = None,
         transaction_text = None,
         transaction_list = None):
    """Saves a SupplierCreditNote as defined by SupplierCreditNoteForm object suppliercreditnote
       this being a creditnote you have received from a supplier"""

    # Creditnote has a status value
    #  - normally 0 when a creditnote is received against an invoice and purchase order
    #  - 1 when a creditnote is received with a payment - no invoice

    # basic checks
    if not suppliercreditnote: return False
    if suppliercreditnote.cost+suppliercreditnote.vat <= D(0): return False


    # Its possible no supplier invoice or purchaseorder is associated with the note so have
    # to deal with the case where there is no supplierinvoice.id value
    if suppliercreditnote.status:
        supplierinvoice_id = None
        purchaseorder_id = None
    else:
        # Get the supplierinvoice this suppliercreditnote is associated with
        supplierinvoice = suppliercreditnote.supplierinvoice
        supplierinvoice_id = supplierinvoice.id
        if not supplierinvoice.is_open:
            return False

        total_on_note = suppliercreditnote.cost+suppliercreditnote.vat
        already_paid = supplierinvoice.paid

        if total_on_note > supplierinvoice.cost + supplierinvoice.vat - already_paid:
            return False

        invoice_open = 1
        if total_on_note == supplierinvoice.cost + supplierinvoice.vat - already_paid:
            # invoice will be fully paid, close it
            invoice_open = 0

        # Get the purchaseorder this suppliercreditnote is associated with
        purchaseorder = suppliercreditnote.purchaseorder
        purchaseorder_id = purchaseorder.id
        if not purchaseorder.is_open:
            return False

        # Get the running totals of invoice cost and vat from the purchaseorder
        total_invoices_cost = purchaseorder.invoice_cost
        total_invoices_vat = purchaseorder.invoice_vat

        # And as this is a suppliercreditnote rather than an invoice, reduce them
        total_invoices_cost -= suppliercreditnote.cost
        total_invoices_vat -= suppliercreditnote.vat

    # Get the stock/expense account this suppliercreditnote is associated with
    account = suppliercreditnote.account
    if not account:
        return False
    account_id = account.id

    if suppliercreditnote.claim_vat:
        claim_vat = 1
    else:
        claim_vat = 0



    # Create a new SupplierCreditNote

    new_suppliercreditnote = [None,
                           suppliercreditnote.supplier.id,
                           supplierinvoice_id,
                           purchaseorder_id,
                           account_id,
                           suppliercreditnote.number,
                           str(suppliercreditnote.cost),
                           str(suppliercreditnote.vat),
                           suppliercreditnote.status,
                           None,
                           claim_vat
                           ]

    # Note - this is a transaction system type 4
    #
    #   0 - User defined transaction - using the move buttons
    #   1 - This was an opening balance transaction
    #   2 - Create a purchase order transaction
    #   3 - Pay a supplier transaction
    #   4 - Payment from a supplier - creditnote transaction
    #   5 - Create an invoice transaction
    #   6 - Customer payment transaction
    #   7 - Create a credit note transaction
    #   8 - An order close transaction


    # Open database,  Note - all these transactions are done, and only committed at the end.
    conn = None
    try:
        conn, c = vs_connect()

        if transaction_date:
            transaction_id = implement(c, transaction_date, transaction_text, transaction_list, 4)
        else:
            transaction_id = None

        new_suppliercreditnote.append(transaction_id)

        c.execute("""INSERT INTO SUPPLIERCREDITNOTES
                          (SUPPLIERCREDITNOTE_ID,
                           SUPPLIER_ID,
                           SUPPLIERINVOICE_ID,
                           PURCHASEORDER_ID,
                           ACCOUNT_ID,
                           NUMBER,
                           COST,
                           VAT,
                           STATUS,
                           VAT_RETURN_ID,
                           CLAIM_VAT,
                           TRANSACTION_ID
                           ) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)""",
                            new_suppliercreditnote)

        if not suppliercreditnote.status:
            # the suppliercreditnote is associated with a supplierinvoice and purchase order
            # update the supplierinvoice and purchase order
            new_total_paid = total_on_note + already_paid
    
            c.execute("UPDATE SUPPLIERINVOICES SET PAID = ?, OPEN = ? WHERE SUPPLIERINVOICE_ID = ?",
                       (str(new_total_paid), invoice_open, supplierinvoice_id))

            # and insert running totals into the purchase order
            c.execute("""UPDATE PURCHASEORDERS
                         SET INVOICE_VAT = ?, INVOICE_COST = ?
                         WHERE PURCHASEORDER_ID = ?""",
                        (str(total_invoices_vat), str(total_invoices_cost), purchaseorder_id))

        # And commit the changes
        conn.commit()
    finally:
        if conn:
            conn.close()

    info_log("Supplier credit note saved: %s, transaction id: %s" % (suppliercreditnote.number, transaction_id))
    return True


################################################################################################




