####### VATStuff #######
#
# save_vatreturn.py  - save a vat return and submit VAT payment
#
# Version : 0.5
# Date : 20100705
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#


"""Save a vat return and submit VAT payment"""


import datetime
import decimal

from vatstuff_packages.vs_utilities import *


from vatstuff_packages.vs_database.transactions import *



D = decimal.Decimal   # D used to avoid typing


def save(vat_return,
         transaction_date,
         transaction_text,
         transaction_list):
    "Implements the transaction and saves a vat return as defined by VATReturnForm object vat_return"

    if not vat_return: return

    # Create a new vat return

    vat_scheme = VATScheme()
    vat_scheme_id = vat_scheme.id
    method = vat_scheme.method
    previous_vatreturn_id = int(get_config_item("LAST_VAT_RETURN_ID"))
    if previous_vatreturn_id == 0:
        previous_vatreturn_id = None

    from_date = vat_return.from_date
    to_date = vat_return.to_date

    new_vat_return = [None,
                       vat_scheme_id,
                       previous_vatreturn_id,
                       str(vs_cfg["VAT_STANDARD_RATE"]),
                       from_date,
                       to_date,
                       vat_return.company.id,
                       vat_return.account.id,
                       str(vat_return.box_one),
                       str(vat_return.box_two),
                       str(vat_return.box_three),
                       str(vat_return.box_four),
                       str(vat_return.box_five),
                       str(vat_return.box_six),
                       str(vat_return.box_seven),
                       str(vat_return.box_eight),
                       str(vat_return.box_nine),
                       ]


    # As the VAT return is submitted, set the new current start date in the vat scheme
    one_day = datetime.timedelta(days=1)
    new_start_date = to_date + one_day


    # Note - this is a transaction system type 10
    #
    #   0 - User defined transaction - using the move buttons
    #   1 - This was an opening balance transaction
    #   2 - Create a purchase order transaction
    #   3 - Pay a supplier transaction
    #   4 - Payment from a supplier - creditnote transaction
    #   5 - Create an invoice transaction
    #   6 - Customer payment transaction
    #   7 - Create a credit note transaction
    #   8 - An order close transaction
    #   9 - Supplier invoice received transaction
    #  10 - A VAT return transaction

    # Open database,  Note - all these transactions are done, and only committed at the end.
    conn = None
    try:
        conn, c = vs_connect()

        transaction_id = implement(c, transaction_date, transaction_text, transaction_list, 10)

        new_vat_return.append(transaction_id)


        c.execute("""INSERT INTO VAT_RETURNS
                          ( VAT_RETURN_ID,
                            VAT_SCHEME_ID,
                            PREVIOUS_VAT_RETURN_ID,
                            STANDARD_RATE,
                            FROM_DATE,
                            TO_DATE,
                            COMPANY_ID,
                            ACCOUNT_ID,
                            BOX_ONE,
                            BOX_TWO,
                            BOX_THREE,
                            BOX_FOUR,
                            BOX_FIVE,
                            BOX_SIX,
                            BOX_SEVEN,
                            BOX_EIGHT,
                            BOX_NINE,
                            TRANSACTION_ID
                           ) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)""",
                            new_vat_return)

        c.execute("SELECT last_insert_rowid()")
        result = c.fetchone()
        vat_return_id = result[0]

        c.execute("UPDATE VAT_SCHEMES SET STARTDATE = ? WHERE VAT_SCHEME_ID = ?", (new_start_date, vat_scheme_id))

        # If the method is STANDARD, then for every invoice and creditnote, set the VAT_RETURN_ID
        # Foreign key
        if method == "STANDARD":

            c.execute("""UPDATE INVOICES SET VAT_RETURN_ID = ?
                         WHERE DATE_OF_TAXPOINT BETWEEN ? AND ?""", (vat_return_id, from_date, to_date))

            c.execute("""UPDATE CREDITNOTES SET VAT_RETURN_ID = ?
                         WHERE CREDITNOTE_ID IN
                           (SELECT CREDITNOTE_ID FROM CREDITNOTES, TRANSACTIONS
                            WHERE DATESTAMP BETWEEN ? AND ?
                            AND CREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID)""",
                             (vat_return_id, from_date, to_date))

            c.execute("""UPDATE SUPPLIERINVOICES SET VAT_RETURN_ID = ?
                         WHERE SUPPLIERINVOICE_ID IN
                           (SELECT SUPPLIERINVOICE_ID FROM SUPPLIERINVOICES, TRANSACTIONS
                            WHERE DATESTAMP BETWEEN ? AND ?
                            AND SUPPLIERINVOICES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID)""",
                             (vat_return_id, from_date, to_date))

            c.execute("""UPDATE SUPPLIERCREDITNOTES SET VAT_RETURN_ID = ?
                         WHERE SUPPLIERCREDITNOTE_ID IN
                           (SELECT SUPPLIERCREDITNOTE_ID FROM SUPPLIERCREDITNOTES, TRANSACTIONS
                            WHERE DATESTAMP BETWEEN ? AND ?
                            AND SUPPLIERCREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID)""",
                             (vat_return_id, from_date, to_date))

            # And for every payment associated with invoices
            # also set the VAT_RETURN_ID in the linking tables

            # payments to suppliers
            c.execute("""UPDATE SUPPLIERINVOICEPAYMENTS SET VAT_RETURN_ID = ?
                         WHERE SUPPLIERINVOICEPAYMENT_ID IN
                           (SELECT SUPPLIERINVOICEPAYMENT_ID FROM SUPPLIERINVOICEPAYMENTS, SUPPLIERINVOICES
                            WHERE SUPPLIERINVOICES.VAT_RETURN_ID = ?
                            AND SUPPLIERINVOICEPAYMENTS.SUPPLIERINVOICE_ID = SUPPLIERINVOICES.SUPPLIERINVOICE_ID)""",
                          (vat_return_id, vat_return_id))

            # payments from customers
            c.execute("""UPDATE INVOICEPAYMENTS SET VAT_RETURN_ID = ?
                         WHERE INVOICEPAYMENT_ID IN
                           (SELECT INVOICEPAYMENT_ID FROM INVOICEPAYMENTS, INVOICES
                            WHERE INVOICES.VAT_RETURN_ID = ?
                            AND INVOICEPAYMENTS.INVOICE_ID = INVOICES.INVOICE_ID)""",
                          (vat_return_id, vat_return_id))


        # If the method is CASH, then for every payment, and creditnote-with-payment,
        # set the VAT_RETURN_ID Foreign key

        elif method == "CASH":
            # If the method is CASH, then for every creditnote with payment,
            # set the VAT_RETURN_ID foreign key

            c.execute("""UPDATE CREDITNOTES SET VAT_RETURN_ID = ?
                         WHERE CREDITNOTE_ID IN
                           (SELECT CREDITNOTE_ID FROM CREDITNOTES, TRANSACTIONS
                            WHERE DATESTAMP BETWEEN ? AND ?
                            AND STATUS = 1
                            AND CREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID)""",
                          (vat_return_id, from_date, to_date))

            c.execute("""UPDATE SUPPLIERCREDITNOTES SET VAT_RETURN_ID = ?
                         WHERE SUPPLIERCREDITNOTE_ID IN
                           (SELECT SUPPLIERCREDITNOTE_ID FROM SUPPLIERCREDITNOTES, TRANSACTIONS
                            WHERE TRANSACTIONS.DATESTAMP BETWEEN ? AND ?
                            AND STATUS = 1
                            AND SUPPLIERCREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID)""",
                          (vat_return_id, from_date, to_date))

            # And for every payment associated with invoices
            # also set the VAT_RETURN_ID in the linking tables
            # if the current VAT_RETURN_ID is not set

            # payments to suppliers
            c.execute("""UPDATE SUPPLIERINVOICEPAYMENTS SET VAT_RETURN_ID = ?
                         WHERE SUPPLIERINVOICEPAYMENT_ID IN
                           (SELECT SUPPLIERINVOICEPAYMENT_ID FROM SUPPLIERINVOICEPAYMENTS, MAKEPAYMENTS
                            WHERE
                                SUPPLIERINVOICEPAYMENTS.VAT_RETURN_ID IS NULL
                                AND SUPPLIERINVOICEPAYMENTS.MAKEPAYMENT_ID = MAKEPAYMENTS.MAKEPAYMENT_ID
                                AND MAKEPAYMENTS.MAKEPAYMENT_ID IN
                               (SELECT MAKEPAYMENTS.MAKEPAYMENT_ID FROM MAKEPAYMENTS, TRANSACTIONS
                                WHERE DATESTAMP BETWEEN ? AND ?
                                AND MAKEPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID))""",
                          (vat_return_id, from_date, to_date))

            # payments from customers
            c.execute("""UPDATE INVOICEPAYMENTS SET VAT_RETURN_ID = ?
                         WHERE INVOICEPAYMENT_ID IN
                           (SELECT INVOICEPAYMENT_ID FROM INVOICEPAYMENTS, CUSTOMERPAYMENTS
                            WHERE
                                INVOICEPAYMENTS.VAT_RETURN_ID IS NULL
                                AND INVOICEPAYMENTS.CUSTOMERPAYMENT_ID = CUSTOMERPAYMENTS.CUSTOMERPAYMENT_ID
                                AND CUSTOMERPAYMENTS.CUSTOMERPAYMENT_ID IN
                           (SELECT CUSTOMERPAYMENTS.CUSTOMERPAYMENT_ID FROM CUSTOMERPAYMENTS, TRANSACTIONS
                            WHERE DATESTAMP BETWEEN ? AND ?
                            AND CUSTOMERPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID))""",
                          (vat_return_id, from_date, to_date))


        # If the method is FLAT-BASIC, then for every invoice and creditnote,
        # set the VAT_RETURN_ID foreign key
        # but only on sales - not purchases (apart from EC AQUISITIONS)
        elif method == "FLAT-BASIC":
            c.execute("""UPDATE INVOICES SET VAT_RETURN_ID = ?
                         WHERE DATE_OF_TAXPOINT BETWEEN ? AND ?""", (vat_return_id, from_date, to_date))

            c.execute("""UPDATE CREDITNOTES SET VAT_RETURN_ID = ?
                         WHERE CREDITNOTE_ID IN
                           (SELECT CREDITNOTE_ID FROM CREDITNOTES, TRANSACTIONS
                            WHERE DATESTAMP BETWEEN ? AND ?
                            AND CREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID)""",
                             (vat_return_id, from_date, to_date))

            # VAT is paid on EC aquisitions

            c.execute("""UPDATE SUPPLIERINVOICES SET VAT_RETURN_ID = ?
                         WHERE SUPPLIERINVOICE_ID IN
                           (SELECT SUPPLIERINVOICE_ID FROM SUPPLIERINVOICES, SUPPLIERS, TRANSACTIONS
                             WHERE DATESTAMP BETWEEN ? AND ?
                             AND EC_COUNTRY = ?
                             AND VAT_COUNTRY_CODE <> ?
                             AND SUPPLIERINVOICES.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID
                             AND SUPPLIERINVOICES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID)""",
                             (vat_return_id, from_date, to_date, 1, vs_cfg["VAT_COUNTRY_CODE"]))


            c.execute("""UPDATE SUPPLIERCREDITNOTES SET VAT_RETURN_ID = ?
                         WHERE SUPPLIERCREDITNOTE_ID IN
                           (SELECT SUPPLIERCREDITNOTE_ID FROM SUPPLIERCREDITNOTES, SUPPLIERS, TRANSACTIONS
                             WHERE DATESTAMP BETWEEN ? AND ?
                             AND EC_COUNTRY = ?
                             AND VAT_COUNTRY_CODE <> ?
                             AND SUPPLIERCREDITNOTES.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID
                             AND SUPPLIERCREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID)""",
                             (vat_return_id, from_date, to_date, 1, vs_cfg["VAT_COUNTRY_CODE"]))


            # And for every payment associated with invoices
            # also set the VAT_RETURN_ID in the linking tables

            # payments to EC suppliers
            c.execute("""UPDATE SUPPLIERINVOICEPAYMENTS SET VAT_RETURN_ID = ?
                         WHERE SUPPLIERINVOICEPAYMENT_ID IN
                           (SELECT SUPPLIERINVOICEPAYMENT_ID FROM SUPPLIERINVOICEPAYMENTS, SUPPLIERINVOICES
                            WHERE SUPPLIERINVOICES.VAT_RETURN_ID = ?
                            AND SUPPLIERINVOICEPAYMENTS.SUPPLIERINVOICE_ID = SUPPLIERINVOICES.SUPPLIERINVOICE_ID)""",
                          (vat_return_id, vat_return_id))

            # payments from customers
            c.execute("""UPDATE INVOICEPAYMENTS SET VAT_RETURN_ID = ?
                         WHERE INVOICEPAYMENT_ID IN
                           (SELECT INVOICEPAYMENT_ID FROM INVOICEPAYMENTS, INVOICES
                            WHERE INVOICES.VAT_RETURN_ID = ?
                            AND INVOICEPAYMENTS.INVOICE_ID = INVOICES.INVOICE_ID)""",
                          (vat_return_id, vat_return_id))

        # If the method is FLAT-CASH then for every creditnote payment,
        # set the VAT_RETURN_ID foreign key
        # but only on sales - not purchases (apart from EC AQUISITIONS)
        elif method == "FLAT-CASH":

            c.execute("""UPDATE CREDITNOTES SET VAT_RETURN_ID = ?
                         WHERE CREDITNOTE_ID IN
                           (SELECT CREDITNOTE_ID FROM CREDITNOTES, TRANSACTIONS
                            WHERE DATESTAMP BETWEEN ? AND ?
                            AND STATUS = 1
                            AND CREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID)""",
                          (vat_return_id, from_date, to_date))

            # VAT is paid on EC aquisitions
            c.execute("""UPDATE SUPPLIERCREDITNOTES SET VAT_RETURN_ID = ?
                         WHERE SUPPLIERCREDITNOTE_ID IN
                           (SELECT SUPPLIERCREDITNOTE_ID
                      FROM SUPPLIERCREDITNOTES, TRANSACTIONS, SUPPLIERS
                     WHERE TRANSACTIONS.DATESTAMP BETWEEN ? AND ?
                       AND STATUS = 1
                       AND EC_COUNTRY = 1
                       AND VAT_COUNTRY_CODE <> ?
                       AND SUPPLIERCREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                       AND SUPPLIERCREDITNOTES.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID)""",
                      (vat_return_id, from_date, to_date, vs_cfg["VAT_COUNTRY_CODE"]))


            # also set the VAT_RETURN_ID in the linking tables
            # if the VAT_RETURN_ID is not set

            # payments to EC suppliers
            c.execute("""UPDATE SUPPLIERINVOICEPAYMENTS SET VAT_RETURN_ID = ?
                         WHERE SUPPLIERINVOICEPAYMENT_ID IN
                           (SELECT SUPPLIERINVOICEPAYMENT_ID FROM SUPPLIERINVOICEPAYMENTS, MAKEPAYMENTS
                            WHERE
                              SUPPLIERINVOICEPAYMENTS.VAT_RETURN_ID IS NULL
                              AND SUPPLIERINVOICEPAYMENTS.MAKEPAYMENT_ID = MAKEPAYMENTS.MAKEPAYMENT_ID
                              AND MAKEPAYMENTS.MAKEPAYMENT_ID IN
                                (SELECT MAKEPAYMENTS.MAKEPAYMENT_ID
                                 FROM SUPPLIERINVOICEPAYMENTS, MAKEPAYMENTS, TRANSACTIONS, SUPPLIERS
                                  WHERE DATESTAMP BETWEEN ? AND ?
                                  AND EC_COUNTRY = 1
                                  AND VAT_COUNTRY_CODE <> ?
                                  AND SUPPLIERINVOICEPAYMENTS.MAKEPAYMENT_ID = MAKEPAYMENTS.MAKEPAYMENT_ID
                                  AND MAKEPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                                  AND MAKEPAYMENTS.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID))""",
                          (vat_return_id, from_date, to_date, vs_cfg["VAT_COUNTRY_CODE"]))

            # payments from customers
            c.execute("""UPDATE INVOICEPAYMENTS SET VAT_RETURN_ID = ?
                         WHERE INVOICEPAYMENT_ID IN
                           (SELECT INVOICEPAYMENT_ID FROM INVOICEPAYMENTS, CUSTOMERPAYMENTS
                            WHERE 
                              INVOICEPAYMENTS.VAT_RETURN_ID IS NULL
                              AND INVOICEPAYMENTS.CUSTOMERPAYMENT_ID = CUSTOMERPAYMENTS.CUSTOMERPAYMENT_ID
                              AND CUSTOMERPAYMENTS.CUSTOMERPAYMENT_ID IN
                                (SELECT CUSTOMERPAYMENTS.CUSTOMERPAYMENT_ID FROM CUSTOMERPAYMENTS, TRANSACTIONS
                                 WHERE DATESTAMP BETWEEN ? AND ?
                                 AND CUSTOMERPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID))""",
                          (vat_return_id, from_date, to_date))

        # And commit the changes
        conn.commit()
    finally:
        if conn:
            conn.close()

################################################################################################




