####### VATStuff #######
#
# transactions.py  - database functions that defines and implements transactions
#
# Version : 0.7
# Date : 20100925
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#

"""Provides functions to implement and store a money transaction"""


import decimal

from vatstuff_packages.vs_utilities import *

D = decimal.Decimal   # D used to avoid typing


# ------------- IMPLEMENT A TRANSACTION SET

def implement(cursor, transaction_form):
    """Stores a transaction, returns the transaction id created - NOT a Transaction object

    cursor           = database cursor, if it is None, a new database connection is created
    transaction_form = A TransactionForm object, holding the transaction details.
       
    The advantage of calling this function with a cursor is so that several transactions or other
    database changes can be done together - but not committed, until all are properly complete.
    If no cursor is given, a connection is created, the transaction saved, and the connection
    committed and closed.
"""

    if not vs_cfg["WRITE_ACCESS"]:
        # Another process has locked the database
        raise VS_ANOTHER_PROCESS, "Database change cancelled - another process has locked this database" 

    assert transaction_form.trans_list

    # Make local copy since datestamp used a lot
    datestamp = transaction_form.datestamp

    # The TRANSACTIONS table holds each transaction id, datestamp, description and system flag
    # The ledger tables hold the transaction id as a foreign key, and hold the actual debit and credit
    # amounts - so there may be several account changes linked to one transaction.

    # Create the transaction, id is None so that sqlite generates the next id number
    transaction = [None,
                   datestamp,
                   transaction_form.description,
                   transaction_form.system]

    conn = None
    try:
        # Make the connection
        if not cursor:
            conn, c = vs_connect()
        else:
            c = cursor

        c.execute("""INSERT INTO TRANSACTIONS
                     (TRANSACTION_ID, DATESTAMP, DESCRIPTION, SYSTEM)
                     VALUES (?, ?, ?, ?)""", transaction)
        c.execute("SELECT last_insert_rowid()")
        result = c.fetchone()
        transaction_id = result[0]

        # sort the transactions database, updating the gridrow value in datetime order
        transaction_sort(c, transaction_id, datestamp)

        # Must now create the ledger entries
        # record each account id in a list as they are made
        store_account_id = []

        # for each [account, debit, credit] in trans_list
        for account, debit, credit in transaction_form.trans_list:

            # Record each transaction in its account ledger, account.ledger is an attribute of an
            # Account object and is the ledger table name, LEDGER_ID - where ID is the Account.id
            ledger_table_name = account.ledger

            # Create the ledger entry
            ledger = [
                      None,                   # Each ledger id will be automatically incremented
                      transaction_id,         # taken from the transaction set
                      -1,                     # gridrow set to -1 (to be calculated)
                      str(debit),             # the debit
                      str(credit),            # the credit
                      "0.00"                  # balance set to zero (to be calculated)
                      ]

            sql_string = """INSERT INTO %s
                            (LEDGER_ID,
                             TRANSACTION_ID,
                             GRIDROW,
                             DEBIT,
                             CREDIT,
                             BALANCE)
                              VALUES (?, ?, ?, ?, ?, ?)""" % ledger_table_name

            c.execute(sql_string, ledger)
            c.execute("SELECT last_insert_rowid()")
            result = c.fetchone()
            ledger_id = result[0]

            # And sort the ledger - this inserts gridrow and balance values, and returns final balance
            balance = ledger_sort(c, ledger_table_name, ledger_id, datestamp)

            # and update the account
            c.execute("""UPDATE ACCOUNTS
                         SET BALANCE = ?
                         WHERE ACCOUNT_ID = ?""", (str(balance), account.id))

            # Insert each account into TRANSACTIONACCOUNTS - this is used to find out which accounts
            # each transaction affected, rather than have to search all the ledgers
            # Avoid a line being duplicated - ensure the same account_id does not occur twice
            if not account.id in store_account_id:
                store_account_id.append(account.id)
                c.execute("""INSERT INTO TRANSACTIONACCOUNTS
                             (TRANSACTIONACCOUNT_ID, TRANSACTION_ID, ACCOUNT_ID)
                              VALUES (?, ?, ?)""", (None, transaction_id, account.id))
        if not cursor:
            conn.commit()
    finally:
        if conn:
            conn.close()

    info_log("Transaction created: %s : system: %s" % (transaction_id, transaction_form.system))

    # NOTE: returns transaction_id, not a Transaction object, this is because the item
    # may not be on the database yet if the connection has not been committed, and as
    # the object reads the database, the Transaction object may not yet be available
    return transaction_id


def transaction_sort(cursor, transaction_id, datestamp):
    """Sorts the transaction table, updating gridrow for each transaction"""

    conn = None
    try:
        # Make the connection
        if not cursor:
            conn, c = vs_connect()
        else:
            c = cursor

        # Get the gridrow number of the transaction, being the count of the previous
        # rows (as gridrow starts at zero)

        c.execute("SELECT COUNT(*) FROM TRANSACTIONS WHERE DATESTAMP <= ? AND TRANSACTION_ID != ?",
                  (datestamp,transaction_id))
        result=c.fetchone()
        gridrow = result[0]

        # Set the transaction to have this gridrow
        c.execute("""UPDATE TRANSACTIONS SET GRIDROW = ? WHERE TRANSACTION_ID = ?""", (gridrow, transaction_id))

        # Increment the gridrow of all transactions with later datestamp
        c.execute("""UPDATE TRANSACTIONS SET GRIDROW = GRIDROW+1 WHERE DATESTAMP > ?""", (datestamp,))

        if not cursor:
            conn.commit()
    finally:
        if conn:
            conn.close()
    return


def ledger_sort(cursor, ledger_table_name, ledger_id, datestamp):
    """Sorts the ledger updating balance and gridrow, returns the final balance"""

    conn = None
    try:
        # Make the connection
        if not cursor:
            conn, c = vs_connect()
        else:
            c = cursor

        #   LEDGER TABLE

        #   LEDGER_ID INTEGER PRIMARY KEY,
        #   TRANSACTION_ID INTEGER REFERENCES TRANSACTIONS(TRANSACTION_ID),
        #   GRIDROW INTEGER UNIQUE,
        #   DEBIT TEXT,
        #   CREDIT TEXT,
        #   BALANCE TEXT


        # Get the gridrow number of the ledger entry, being the count of the previous
        # rows (as gridrow starts at zero)
        sql_string1 = """SELECT COUNT(*) FROM %s, TRANSACTIONS
                         WHERE DATESTAMP <= ? AND
                         %s.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                         AND LEDGER_ID != ?""" % (ledger_table_name, ledger_table_name)

        c.execute(sql_string1, (datestamp, ledger_id))
        result=c.fetchone()
        gridrow = result[0]


        # Increment the gridrow of all ledger entries with later or equal gridrows
        sql_string2 = "UPDATE %s SET GRIDROW = GRIDROW+1 WHERE GRIDROW >= ?" % ledger_table_name

        c.execute(sql_string2, (gridrow,))

        # Set the new ledger entry to have this gridrow
        sql_string3 = "UPDATE %s SET GRIDROW = ? WHERE LEDGER_ID = ?" % ledger_table_name

        c.execute(sql_string3, (gridrow, ledger_id))

        # So ledger entry is now inserted with the correct gridrow, and all entries have
        # their gridrow values set in correct incrementing date order,
        # must now set the balance for each entry

        # Get the balance of the entry immediately previous to this new entry
        balance = D("0.00")

        if gridrow > 0:
            sql_string4 = "SELECT BALANCE FROM %s WHERE GRIDROW = ?" % ledger_table_name
            c.execute(sql_string4, (gridrow-1,))
            result=c.fetchone()
            balance = D(result[0])


        sql_string5 = """SELECT LEDGER_ID, DEBIT, CREDIT
                         FROM %s
                         WHERE GRIDROW >= ?
                         ORDER BY GRIDROW""" % ledger_table_name


        sql_string6 = """UPDATE %s SET BALANCE = ?
                         WHERE LEDGER_ID = ?""" % ledger_table_name


        c.execute(sql_string5, (gridrow,))
        for ledger_entry in c.fetchall():
            # ledger_entry is each ledger line ordered by gridrow
            # create a running balance total of the debits minus the credits
            balance = balance + D(ledger_entry[1]) - D(ledger_entry[2])
            # insert balance
            c.execute(sql_string6, (str(balance), ledger_entry[0]))

        if not cursor:
            conn.commit()
    finally:
        if conn:
            conn.close()
    return balance


