####### VATStuff #######
#
# vat_cash.py  - generate VAT reports/returns for the Cash method
#
# Version : 0.7.0
# Date : 20100918
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#

"""Generate VAT reports and VAT returns for the cash method

contains:
   cash_report() - returns a ReportForm object for the cash method
   get_box_1() etc - for each box of a VAT return, gives the decimal result
   cash_output() - these calculate the output - input which has been put
   cash_input()     in the VAT Balance account - so differences can be
                    moved to the VAT income, and unclaimed VAT acounts
   cash_detail() - returns a detailed ReportForm object for the cash method
"""

import decimal

from vatstuff_packages.vs_utilities import *

from vatstuff_packages.vs_verify.form_objects import ReportForm

D = decimal.Decimal   # D used to avoid typing



def cash_report(from_date, to_date, vat_return, vat_scheme):
    """Create a basic report for the cash vat method
       returns a ReportForm object"""

    report = ReportForm("VAT report")

    report.header = ["For the period:",
                     "From: %s" % from_date.strftime("%d %b %Y"),
                     "To: %s" % to_date.strftime("%d %b %Y"),
                     "",
                     "Method: Cash"]

    if vs_cfg["DELETE_TO_DATE"]:
        if from_date < vs_cfg["DELETE_TO_DATE"]:
            report.header.append("!!WARNING!! - this report may be innaccurate")
            report.header.append(
            "Data prior to %s has been deleted from the database" % vs_cfg["DELETE_TO_DATE"].strftime("%d %b %Y"))

    # tables are described by a list of list,
    # each list being a row, consisting of cell tuple
    # each cell tuple containing two strings; 
    # The first string is formed from "XXXN",
    # and the second string is the text to be placed in the cell

    # N is the collspan number, as used in a table cell tag attribute
    # such as 'colspan="4"' - in which case N is 4

    # so normally N is '1'

    # XXX is a formatting string :
    # HBL  - Header Bold Left Justified (R for Right instead of L
    # HNL  - Header Normal Left Justified  etc
    # DNL  - Data Cell Normal Left Justified  etc
    # XXC  - Right Justified currency, as pulled from the database

    # Set the running totals
    input_VAT = D("0.00")
    input_VAT_correction = D("0.00")
    output_VAT = D("0.00")
    output_VAT_correction = D("0.00")
    unclaimed_VAT = D("0.00")

    conn=None
    try:
        # Make database connection
        conn, c = vs_connect()

        ############## Output VAT

        # Payments from customers headings
        report.table = [[ ('HNL6', 'Output VAT from payments received from customers') ],
                        [ ('HBL1', 'Date'),
                          ('HBL1', 'Customer'),
                          ('HBL1', 'Invoice number'),
                          ('HBL1', 'Payment transaction'),
                          ('HBR1', 'Net Cost'),
                          ('HBR1', 'VAT') ]]

        # Get the payments from the database
        # UNION select needed as invoices may be NULL if a delete old data has occurred
        if vat_return:
            # This is for a previous vat return, in which a vat_return_id exists
            # so search for payments with this vat_return_id
            c.execute("""SELECT DATESTAMP,
                                NAME,
                                INVOICES.INVOICE_NUMBER,
                                CUSTOMERPAYMENTS.TRANSACTION_ID,
                                INVOICEPAYMENTS.COST,
                                INVOICEPAYMENTS.VAT
                           FROM INVOICEPAYMENTS, CUSTOMERPAYMENTS, TRANSACTIONS, CUSTOMERS, INVOICES
                          WHERE INVOICEPAYMENTS.VAT_RETURN_ID = ?
                            AND INVOICEPAYMENTS.CUSTOMERPAYMENT_ID = CUSTOMERPAYMENTS.CUSTOMERPAYMENT_ID
                            AND INVOICEPAYMENTS.INVOICE_ID = INVOICES.INVOICE_ID
                            AND CUSTOMERPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                            AND CUSTOMERPAYMENTS.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID
                       UNION
                         SELECT DATESTAMP,
                                NAME,
                                NULL,
                                CUSTOMERPAYMENTS.TRANSACTION_ID,
                                INVOICEPAYMENTS.COST,
                                INVOICEPAYMENTS.VAT
                           FROM INVOICEPAYMENTS, CUSTOMERPAYMENTS, TRANSACTIONS, CUSTOMERS
                          WHERE INVOICEPAYMENTS.VAT_RETURN_ID = ?
                            AND INVOICEPAYMENTS.CUSTOMERPAYMENT_ID = CUSTOMERPAYMENTS.CUSTOMERPAYMENT_ID
                            AND INVOICEPAYMENTS.INVOICE_ID IS NULL
                            AND CUSTOMERPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                            AND CUSTOMERPAYMENTS.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID

                       ORDER BY DATESTAMP""", (vat_return.id, vat_return.id))
        else:
            # This is for the current period, in which no vat_return_id exists
            # so search for payments within the dates, but with a null vat_return_id
            # so any payment which already has vat payed on it is not included
            # (if the previous scheme was standard foR example)
            c.execute("""SELECT DATESTAMP,
                                NAME,
                                INVOICES.INVOICE_NUMBER,
                                CUSTOMERPAYMENTS.TRANSACTION_ID,
                                INVOICEPAYMENTS.COST,
                                INVOICEPAYMENTS.VAT
                           FROM INVOICEPAYMENTS, CUSTOMERPAYMENTS, TRANSACTIONS, CUSTOMERS, INVOICES
                          WHERE DATESTAMP BETWEEN ? AND ?
                            AND INVOICEPAYMENTS.VAT_RETURN_ID IS NULL
                            AND INVOICEPAYMENTS.CUSTOMERPAYMENT_ID = CUSTOMERPAYMENTS.CUSTOMERPAYMENT_ID
                            AND INVOICEPAYMENTS.INVOICE_ID = INVOICES.INVOICE_ID
                            AND CUSTOMERPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                            AND CUSTOMERPAYMENTS.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID
                       UNION
                         SELECT DATESTAMP,
                                NAME,
                                NULL,
                                CUSTOMERPAYMENTS.TRANSACTION_ID,
                                INVOICEPAYMENTS.COST,
                                INVOICEPAYMENTS.VAT
                           FROM INVOICEPAYMENTS, CUSTOMERPAYMENTS, TRANSACTIONS, CUSTOMERS
                          WHERE DATESTAMP BETWEEN ? AND ?
                            AND INVOICEPAYMENTS.VAT_RETURN_ID IS NULL
                            AND INVOICEPAYMENTS.CUSTOMERPAYMENT_ID = CUSTOMERPAYMENTS.CUSTOMERPAYMENT_ID
                            AND INVOICEPAYMENTS.INVOICE_ID IS NULL
                            AND CUSTOMERPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                            AND CUSTOMERPAYMENTS.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID
                       ORDER BY DATESTAMP""", (from_date, to_date, from_date, to_date))

        for info in c:
            # Fill table with payments from customers data

            # It looks as if the UNION function returns a unicode value of the datestamp, rather
            # than a datetime.date object
            if isinstance(info[0], datetime.date):
                datestamp = info[0].strftime("%d %b %Y")
            else:
                int_dates = map(int, info[0].split("-"))
                datestamp = datetime.date(*int_dates).strftime("%d %b %Y")

            # The invoice number could be None if no invoice exists
            # in which case display a '?' string 
            if info[2]:
                invoicenumber = str(info[2])
            else:
                invoicenumber = "?"

            report.add_row_to_table( ('DNL1', datestamp),
                                     ('DNL1', info[1]),
                                     ('DNL1', invoicenumber),
                                     ('DNL1', str(info[3])),
                                     ('DNC1', info[4]),
                                     ('DNC1', info[5]) )

            output_VAT += D(info[5])

        report.add_row_to_table( ('HBR5', "Output VAT from customer payments:"),
                                 ('HBC1', str(output_VAT)) )


        # Now list creditnotes with payments issued by you - Output VAT corrections 

        # Check if there are any VAT corrections from creditnotes issued with a payment
        c.execute("""SELECT COUNT(*) FROM CREDITNOTES, TRANSACTIONS
                     WHERE DATESTAMP BETWEEN ? AND ?
                       AND STATUS = 1
                       AND CREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID""",
                      (from_date, to_date))
        result = c.fetchone()

        if result[0]:
            report.add_row_to_table( ('HNL6', 'Output VAT correction from credit notes issued with a payment') )

            report.add_row_to_table( ('HBL1', 'Date'),
                                     ('HBL1', 'Customer'),
                                     ('HBL1', 'Credit Note'),
                                     ('HBL1', 'Payment transaction'),
                                     ('HBR1', 'Net Cost'),
                                     ('HBR1', 'VAT') )

            c.execute("""SELECT DATESTAMP,
                                CUSTOMERS.NAME,
                                CREDITNOTE_NUMBER,
                                CREDITNOTES.TRANSACTION_ID,
                                COST,
                                VAT
                           FROM CREDITNOTES, CUSTOMERS, TRANSACTIONS
                          WHERE DATESTAMP BETWEEN ? AND ?
                            AND STATUS = 1
                            AND CREDITNOTES.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID
                            AND CREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                       ORDER BY DATESTAMP""", (from_date, to_date))

            for info in c:
                report.add_row_to_table( ('DNL1', info[0].strftime("%d %b %Y")),
                                         ('DNL1', info[1]),
                                         ('DNL1', info[2]),
                                         ('DNL1', info[3]),
                                         ('DNC1', info[4]),
                                         ('DNC1', info[5]) )

                output_VAT_correction += D(info[5])

            report.add_row_to_table( ('HBR5', "Correction from payments issued with credit notes:"),
                                     ('HBC1', str(output_VAT_correction)) )

        output_VAT = output_VAT - output_VAT_correction
        if output_VAT_correction:
            report.add_row_to_table( ('HBR5', "Total output VAT:"),
                                     ('HBC1', str(output_VAT)) )

        ############## Input VAT

        report.add_row_to_table( ('HNL6', 'Input VAT from payments made to suppliers') )
        report.add_row_to_table( ('HBL1', 'Date'),
                                 ('HBL1', 'Supplier'),
                                 ('HBL1', 'Order number'),
                                 ('HBL1', 'Payment transaction'),
                                 ('HBR1', 'Net Cost'),
                                 ('HBR1', 'VAT') )

        # Get VAT from payments to supplier
        # UNION select needed as purchaseorders may be NULL if a delete old data has occurred

        if vat_return:
            # This is for a previous vat return
            c.execute("""SELECT DATESTAMP,
                                SUPPLIERS.NAME,
                                PURCHASEORDERS.ORDER_NUMBER,
                                MAKEPAYMENTS.TRANSACTION_ID,
                                SUPPLIERINVOICEPAYMENTS.COST,
                                SUPPLIERINVOICEPAYMENTS.VAT
                           FROM SUPPLIERINVOICEPAYMENTS, MAKEPAYMENTS, TRANSACTIONS, SUPPLIERS, PURCHASEORDERS
                          WHERE SUPPLIERINVOICEPAYMENTS.VAT_RETURN_ID = ?
                            AND SUPPLIERINVOICEPAYMENTS.CLAIM_VAT = 1
                            AND SUPPLIERINVOICEPAYMENTS.MAKEPAYMENT_ID = MAKEPAYMENTS.MAKEPAYMENT_ID
                            AND MAKEPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                            AND MAKEPAYMENTS.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID
                            AND MAKEPAYMENTS.PURCHASEORDER_ID = PURCHASEORDERS.PURCHASEORDER_ID
                    UNION
                         SELECT DATESTAMP,
                                SUPPLIERS.NAME,
                                NULL,
                                MAKEPAYMENTS.TRANSACTION_ID,
                                SUPPLIERINVOICEPAYMENTS.COST,
                                SUPPLIERINVOICEPAYMENTS.VAT
                           FROM SUPPLIERINVOICEPAYMENTS, MAKEPAYMENTS, TRANSACTIONS, SUPPLIERS
                          WHERE SUPPLIERINVOICEPAYMENTS.VAT_RETURN_ID = ?
                            AND SUPPLIERINVOICEPAYMENTS.CLAIM_VAT = 1
                            AND SUPPLIERINVOICEPAYMENTS.MAKEPAYMENT_ID = MAKEPAYMENTS.MAKEPAYMENT_ID
                            AND MAKEPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                            AND MAKEPAYMENTS.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID
                            AND MAKEPAYMENTS.PURCHASEORDER_ID IS NULL

                       ORDER BY TRANSACTIONS.DATESTAMP""", (vat_return.id, vat_return.id))
        else:
            # This is for the current period
            c.execute("""SELECT DATESTAMP,
                                SUPPLIERS.NAME,
                                PURCHASEORDERS.ORDER_NUMBER,
                                MAKEPAYMENTS.TRANSACTION_ID,
                                SUPPLIERINVOICEPAYMENTS.COST,
                                SUPPLIERINVOICEPAYMENTS.VAT
                           FROM SUPPLIERINVOICEPAYMENTS, MAKEPAYMENTS, TRANSACTIONS, SUPPLIERS, PURCHASEORDERS
                          WHERE DATESTAMP BETWEEN ? AND ?
                            AND SUPPLIERINVOICEPAYMENTS.VAT_RETURN_ID IS NULL
                            AND SUPPLIERINVOICEPAYMENTS.CLAIM_VAT = 1
                            AND SUPPLIERINVOICEPAYMENTS.MAKEPAYMENT_ID = MAKEPAYMENTS.MAKEPAYMENT_ID
                            AND MAKEPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                            AND MAKEPAYMENTS.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID
                            AND MAKEPAYMENTS.PURCHASEORDER_ID = PURCHASEORDERS.PURCHASEORDER_ID
                    UNION
                         SELECT DATESTAMP,
                                SUPPLIERS.NAME,
                                NULL,
                                MAKEPAYMENTS.TRANSACTION_ID,
                                SUPPLIERINVOICEPAYMENTS.COST,
                                SUPPLIERINVOICEPAYMENTS.VAT
                           FROM SUPPLIERINVOICEPAYMENTS, MAKEPAYMENTS, TRANSACTIONS, SUPPLIERS
                          WHERE DATESTAMP BETWEEN ? AND ?
                            AND SUPPLIERINVOICEPAYMENTS.CLAIM_VAT = 1
                            AND SUPPLIERINVOICEPAYMENTS.VAT_RETURN_ID IS NULL
                            AND SUPPLIERINVOICEPAYMENTS.MAKEPAYMENT_ID = MAKEPAYMENTS.MAKEPAYMENT_ID
                            AND MAKEPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                            AND MAKEPAYMENTS.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID
                            AND MAKEPAYMENTS.PURCHASEORDER_ID IS NULL

                       ORDER BY TRANSACTIONS.DATESTAMP""", (from_date, to_date, from_date, to_date))
        
        for info in c:
            # Input VAT from payments to suppliers

            # It looks as if the UNION function returns a unicode value of the datestamp, rather
            # than a datetime.date object
            if isinstance(info[0], datetime.date):
                datestamp = info[0].strftime("%d %b %Y")
            else:
                int_dates = map(int, info[0].split("-"))
                datestamp = datetime.date(*int_dates).strftime("%d %b %Y")

            # The purchaseorder number could be None if no order exists
            # in which case display a '?' string 
            if info[2]:
                ordernumber = str(info[2])
            else:
                ordernumber = "?"

            report.add_row_to_table( ('DNL1', datestamp),
                                     ('DNL1', info[1]),
                                     ('DNL1', ordernumber),
                                     ('DNL1', info[3]),
                                     ('DNC1', info[4]),
                                     ('DNC1', info[5]) )

            input_VAT += D(info[5])

        report.add_row_to_table( ('HBR5', "Input VAT from payments to suppliers:"),
                                 ('HBC1', str(input_VAT)) )

        # Check if there are any VAT corrections from supplier creditnotes received with a payment
        c.execute("""SELECT COUNT(*) FROM SUPPLIERCREDITNOTES, TRANSACTIONS
                      WHERE TRANSACTIONS.DATESTAMP BETWEEN ? AND ?
                        AND STATUS = 1
                        AND SUPPLIERCREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                       """, (from_date, to_date))

        result = c.fetchone()

        if result[0]:
            report.add_row_to_table( ('HNL6', 'Input VAT correction from supplier credit notes received with a payment') )
            report.add_row_to_table( ('HBL1', 'Date'),
                                     ('HBL1', 'Supplier'),
                                     ('HBL1', 'Supplier Credit Note'),
                                     ('HBL1', 'Payment transaction'),
                                     ('HBR1', 'Net Cost'),
                                     ('HBR1', 'VAT') )

            c.execute("""SELECT DATESTAMP,
                                SUPPLIERS.NAME,
                                SUPPLIERCREDITNOTES.NUMBER,
                                SUPPLIERCREDITNOTES.TRANSACTION_ID,
                                COST,
                                VAT
                           FROM SUPPLIERCREDITNOTES, SUPPLIERS, TRANSACTIONS
                          WHERE DATESTAMP BETWEEN ? AND ?
                            AND STATUS = 1
                            AND SUPPLIERCREDITNOTES.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID
                            AND SUPPLIERCREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                       ORDER BY DATESTAMP""", (from_date, to_date))

            for info in c:
                report.add_row_to_table( ('DNL1', info[0].strftime("%d %b %Y")),
                                         ('DNL1', info[1]),
                                         ('DNL1', info[2]),
                                         ('DNL1', info[3]),
                                         ('DNC1', info[4]),
                                         ('DNC1', info[5]) )

                input_VAT_correction += D(info[5])

            report.add_row_to_table( ('HBR5', "Correction from received credit notes:"),
                                     ('HBC1', str(input_VAT_correction)) )

            input_VAT = input_VAT - input_VAT_correction

            if input_VAT_correction:
                report.add_row_to_table( ('HBR5', "Total input VAT:"),
                                         ('HBC1', str(input_VAT)) )

        # And final VAT owed
        report.add_row_to_table( ('HBR5', "VAT owed (output - input):"),
                                 ('HBC1', str(output_VAT - input_VAT)) )


        # List payments made with unclaimed VAT
        if vat_return:
            # This is for a previous vat return
            c.execute("""SELECT COUNT(*) FROM SUPPLIERINVOICEPAYMENTS
                         WHERE SUPPLIERINVOICEPAYMENTS.VAT_RETURN_ID = ?
                           AND SUPPLIERINVOICEPAYMENTS.CLAIM_VAT = 0""", (vat_return.id,))
            result = c.fetchone()
        else:
            c.execute("""SELECT COUNT(*) FROM SUPPLIERINVOICEPAYMENTS, TRANSACTIONS, MAKEPAYMENTS
                          WHERE DATESTAMP BETWEEN ? AND ?
                            AND SUPPLIERINVOICEPAYMENTS.VAT_RETURN_ID IS NULL
                            AND SUPPLIERINVOICEPAYMENTS.CLAIM_VAT = 0
                            AND SUPPLIERINVOICEPAYMENTS.MAKEPAYMENT_ID = MAKEPAYMENTS.MAKEPAYMENT_ID
                            AND MAKEPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID""", (from_date, to_date))
            result = c.fetchone()
        if result[0]:
            report.add_row_to_table( ('HNL6', 'Payments made, set as unclaimed VAT and not included in input VAT') )
            report.add_row_to_table( ('HBL1', 'Date'),
                                     ('HBL1', 'Supplier'),
                                     ('HBL1', 'Order number'),
                                     ('HBL1', 'Payment transaction'),
                                     ('HBR1', 'Net Cost'),
                                     ('HBR1', 'VAT') )

            if vat_return:
                # This is for a previous vat return
                c.execute("""SELECT DATESTAMP,
                                    SUPPLIERS.NAME,
                                    PURCHASEORDERS.ORDER_NUMBER,
                                    MAKEPAYMENTS.TRANSACTION_ID,
                                    SUPPLIERINVOICEPAYMENTS.COST,
                                    SUPPLIERINVOICEPAYMENTS.VAT
                               FROM SUPPLIERINVOICEPAYMENTS, MAKEPAYMENTS, TRANSACTIONS, SUPPLIERS, PURCHASEORDERS
                              WHERE SUPPLIERINVOICEPAYMENTS.VAT_RETURN_ID = ?
                                AND SUPPLIERINVOICEPAYMENTS.CLAIM_VAT = 0
                                AND SUPPLIERINVOICEPAYMENTS.MAKEPAYMENT_ID = MAKEPAYMENTS.MAKEPAYMENT_ID
                                AND MAKEPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                                AND MAKEPAYMENTS.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID
                                AND MAKEPAYMENTS.PURCHASEORDER_ID = PURCHASEORDERS.PURCHASEORDER_ID
                        UNION
                             SELECT DATESTAMP,
                                    SUPPLIERS.NAME,
                                    NULL,
                                    MAKEPAYMENTS.TRANSACTION_ID,
                                    SUPPLIERINVOICEPAYMENTS.COST,
                                    SUPPLIERINVOICEPAYMENTS.VAT
                               FROM SUPPLIERINVOICEPAYMENTS, MAKEPAYMENTS, TRANSACTIONS, SUPPLIERS
                              WHERE SUPPLIERINVOICEPAYMENTS.VAT_RETURN_ID = ?
                                AND SUPPLIERINVOICEPAYMENTS.CLAIM_VAT = 0
                                AND SUPPLIERINVOICEPAYMENTS.MAKEPAYMENT_ID = MAKEPAYMENTS.MAKEPAYMENT_ID
                                AND MAKEPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                                AND MAKEPAYMENTS.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID
                                AND MAKEPAYMENTS.PURCHASEORDER_ID IS NULL

                           ORDER BY TRANSACTIONS.DATESTAMP""", (vat_return.id, vat_return.id))
            else:
                # This is for the current period
                c.execute("""SELECT DATESTAMP,
                                    SUPPLIERS.NAME,
                                    PURCHASEORDERS.ORDER_NUMBER,
                                    MAKEPAYMENTS.TRANSACTION_ID,
                                    SUPPLIERINVOICEPAYMENTS.COST,
                                    SUPPLIERINVOICEPAYMENTS.VAT
                               FROM SUPPLIERINVOICEPAYMENTS, MAKEPAYMENTS, TRANSACTIONS, SUPPLIERS, PURCHASEORDERS
                              WHERE DATESTAMP BETWEEN ? AND ?
                                AND SUPPLIERINVOICEPAYMENTS.VAT_RETURN_ID IS NULL
                                AND SUPPLIERINVOICEPAYMENTS.CLAIM_VAT = 0
                                AND SUPPLIERINVOICEPAYMENTS.MAKEPAYMENT_ID = MAKEPAYMENTS.MAKEPAYMENT_ID
                                AND MAKEPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                                AND MAKEPAYMENTS.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID
                                AND MAKEPAYMENTS.PURCHASEORDER_ID = PURCHASEORDERS.PURCHASEORDER_ID
                        UNION
                             SELECT DATESTAMP,
                                    SUPPLIERS.NAME,
                                    NULL,
                                    MAKEPAYMENTS.TRANSACTION_ID,
                                    SUPPLIERINVOICEPAYMENTS.COST,
                                    SUPPLIERINVOICEPAYMENTS.VAT
                               FROM SUPPLIERINVOICEPAYMENTS, MAKEPAYMENTS, TRANSACTIONS, SUPPLIERS
                              WHERE DATESTAMP BETWEEN ? AND ?
                                AND SUPPLIERINVOICEPAYMENTS.CLAIM_VAT = 0
                                AND SUPPLIERINVOICEPAYMENTS.VAT_RETURN_ID IS NULL
                                AND SUPPLIERINVOICEPAYMENTS.MAKEPAYMENT_ID = MAKEPAYMENTS.MAKEPAYMENT_ID
                                AND MAKEPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                                AND MAKEPAYMENTS.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID
                                AND MAKEPAYMENTS.PURCHASEORDER_ID IS NULL

                           ORDER BY TRANSACTIONS.DATESTAMP""", (from_date, to_date, from_date, to_date))
            
            for info in c:
                # It looks as if the UNION function returns a unicode value of the datestamp, rather
                # than a datetime.date object
                if isinstance(info[0], datetime.date):
                    datestamp = info[0].strftime("%d %b %Y")
                else:
                    int_dates = map(int, info[0].split("-"))
                    datestamp = datetime.date(*int_dates).strftime("%d %b %Y")

                # The purchaseorder number could be None if no order exists
                # in which case display a '?' string 
                if info[2]:
                    ordernumber = str(info[2])
                else:
                    ordernumber = "?"

                report.add_row_to_table( ('DNL1', datestamp),
                                         ('DNL1', info[1]),
                                         ('DNL1', ordernumber),
                                         ('DNL1', info[3]),
                                         ('DNC1', info[4]),
                                         ('DNC1', info[5]) )

                unclaimed_VAT += D(info[5])

            if result[0]>1:
                report.add_row_to_table( ('DNR5', "Unclaimed VAT from payments to suppliers:"),
                                         ('DNC1', str(unclaimed_VAT)) )
    finally:
        if conn:
            conn.close()


    previous_vatreturn, previous_vatscheme, previous_method = get_previous_vat_return_scheme(vat_return)

    if previous_vatscheme:
        if previous_method != "CASH":
            # There has been a change of scheme
            report.add_row_to_table( ('HBR6', "----------------------") )
            report.add_row_to_table( ('HBL6', "Note: change of VAT scheme may require additional payments") )

    return report


############# CASH VAT RETURN ####################################

def get_box_1(from_date, to_date):
    """Form text : VAT due in this period on sales and other outputs.

    My interpretation : The VAT from all payments you have received with
    dates within the period (and which have not already been included in a
    previous vat return) are added, then the vat from all credit notes
    you have issued together with a payment within the period are subtracted.
    The result is the value in Box 1."""

    output_VAT = D("0.00")

    conn = None
    try:
        conn, c = vs_connect()

        # Get VAT from customer payments
        c.execute("""SELECT VAT
                       FROM INVOICEPAYMENTS, CUSTOMERPAYMENTS, TRANSACTIONS
                      WHERE DATESTAMP BETWEEN ? AND ?
                        AND INVOICEPAYMENTS.VAT_RETURN_ID IS NULL
                        AND INVOICEPAYMENTS.CUSTOMERPAYMENT_ID = CUSTOMERPAYMENTS.CUSTOMERPAYMENT_ID
                        AND CUSTOMERPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                        """, (from_date, to_date))

        for result in c:
            output_VAT += D(result[0])

        # Subtract VAT from any creditnotes sent to the customer with a payment
        # that is : creditnotes with status 1
        c.execute("""SELECT VAT
                      FROM CREDITNOTES, TRANSACTIONS
                     WHERE DATESTAMP BETWEEN ? AND ?
                       AND STATUS = 1
                       AND CREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID""",
                      (from_date, to_date))

        for result in c:
            output_VAT -= D(result[0])

    finally:
        if conn:
            conn.close()
    return output_VAT


def get_box_2(from_date, to_date):
    """Form text : VAT due in this period on acquisitions from other EC member States

    My interpretation : The value from Box 9 is taken, and the standard rate of VAT
    is applied.  (That is - the vat rate with symbol S set in the vat rates table).
    The subsequent result is set in Box 2."""

    EC_cost = get_box_9(from_date, to_date)
    if EC_cost == D(0):
        return D("0.00")
    result = (EC_cost * vs_cfg["VAT_STANDARD_RATE"]) / D("100.0000")
    EC_purchases_VAT = result.quantize(D('.01'), rounding=decimal.ROUND_HALF_EVEN)
    return EC_purchases_VAT


# Box 3 - Form text : Total VAT due (the sum of boxes 1 and 2)


def get_box_4(from_date, to_date):
    """Form text : VAT reclaimed in this period on purchase and other inputs
                   (including acquisitions from the EC)

    My interpretation : The VAT from all payments to suppliers made in the period
    (and which have not already been included in a previous vat return),
    with the VAT from any money received with a suppliers credit note subtracted.
    Those invoices and creditnotes flagged with VAT unclaimable are not included
    The value from Box 2 is then added."""

    input_VAT = D("0.00")

    conn = None
    try:
        conn, c = vs_connect()

        # Get VAT from payments to supplier
        c.execute("""SELECT VAT
                       FROM SUPPLIERINVOICEPAYMENTS, MAKEPAYMENTS, TRANSACTIONS
                      WHERE DATESTAMP BETWEEN ? AND ?
                        AND SUPPLIERINVOICEPAYMENTS.VAT_RETURN_ID IS NULL
                        AND SUPPLIERINVOICEPAYMENTS.CLAIM_VAT = 1
                        AND SUPPLIERINVOICEPAYMENTS.MAKEPAYMENT_ID = MAKEPAYMENTS.MAKEPAYMENT_ID
                        AND MAKEPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                        """, (from_date, to_date))

        for result in c:
            input_VAT += D(result[0])

        # Subtract VAT from any supplier creditnotes received with a payment
        c.execute("""SELECT VAT
                      FROM SUPPLIERCREDITNOTES, TRANSACTIONS
                     WHERE TRANSACTIONS.DATESTAMP BETWEEN ? AND ?
                       AND STATUS = 1
                       AND SUPPLIERCREDITNOTES.CLAIM_VAT = 1
                       AND SUPPLIERCREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                       """, (from_date, to_date))
        for result in c:
            input_VAT -= D(result[0])

    finally:
        if conn:
            conn.close()

    # Also include the input VAT applied from Box 2 
    return input_VAT + get_box_2(from_date, to_date)


# Box 5 - Form text : Net VAT to be paid to HMRC or reclaimed by you
#                     (Difference between boxes 3 and 4)


def get_box_6(from_date, to_date):
    """Form text : Total value of sales and all other outputs excluding
                   any VAT. Include your box 8 figure

    My interpretation : The net cost (amount without the VAT) on all payments
    received from customers within the period are added,
    (and which have not already been included in a previous vat return),
    then subtract the net cost (again without VAT) of all payments you have
    issued to customers with credit notes.
    The result is the value in Box 6."""

    output_cost = D("0.00")

    conn = None
    try:
        conn, c = vs_connect()

        # Get cost of payments received
        c.execute("""SELECT COST
                       FROM INVOICEPAYMENTS, CUSTOMERPAYMENTS, TRANSACTIONS
                      WHERE DATESTAMP BETWEEN ? AND ?
                        AND INVOICEPAYMENTS.VAT_RETURN_ID IS NULL
                        AND INVOICEPAYMENTS.CUSTOMERPAYMENT_ID = CUSTOMERPAYMENTS.CUSTOMERPAYMENT_ID
                        AND CUSTOMERPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID""",
                         (from_date, to_date))

        for result in c:
            output_cost += D(result[0])

        # Subtract cost from any creditnotes issued together with a payment
        # that is - creditnotes with status value 1
        c.execute("""SELECT COST
                      FROM CREDITNOTES, TRANSACTIONS
                     WHERE DATESTAMP BETWEEN ? AND ?
                       AND STATUS = 1
                       AND CREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID""",
                      (from_date, to_date))

        for result in c:
            output_cost -= D(result[0])

    finally:
        if conn:
            conn.close()
    return output_cost


def get_box_7(from_date, to_date):
    """Form text : Total value of purchases and all other inputs excluding
                   any VAT. Include your box 9 figure

    My interpretation : The net cost (amount without VAT) of all payments
    made to suppliers in the period,
    (and which have not already been included in a previous vat return),
    with the net costs (again without VAT) from any payment received
    with a suppliers credit notes subtracted."""

    input_cost = D("0.00")

    conn = None
    try:
        conn, c = vs_connect()

        # Get cost from payments made
        c.execute("""SELECT COST
                       FROM SUPPLIERINVOICEPAYMENTS, MAKEPAYMENTS, TRANSACTIONS
                      WHERE DATESTAMP BETWEEN ? AND ?
                        AND SUPPLIERINVOICEPAYMENTS.VAT_RETURN_ID IS NULL
                        AND SUPPLIERINVOICEPAYMENTS.MAKEPAYMENT_ID = MAKEPAYMENTS.MAKEPAYMENT_ID
                        AND MAKEPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID""",
                        (from_date, to_date))

        for result in c:
            input_cost += D(result[0])

        # Subtract cost of any payment received with supplier creditnotes
        c.execute("""SELECT COST
                      FROM SUPPLIERCREDITNOTES, TRANSACTIONS
                     WHERE TRANSACTIONS.DATESTAMP BETWEEN ? AND ?
                       AND STATUS = 1
                       AND SUPPLIERCREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID""", (from_date, to_date))
        for result in c:
            input_cost -= D(result[0])

    finally:
        if conn:
            conn.close()
    return input_cost


def get_box_8(from_date, to_date):
    """Form text : Total value of all supplies of goods and related costs,
                   excluding any VAT, to other EC Member States

    My interpretation : The net cost (amount without VAT) from all payments received
    from EC customers within the period,
    (and which have not already been included in a previous vat return),
    minus the costs (again without VAT) of payments
    sent to EC customers with credit notes. An 'EC customer' being a customer
    who is in the EC, who does not have GB set in their country code, and who is VAT registered."""

    output_cost = D("0.00")

    conn = None
    try:
        conn, c = vs_connect()

        # Get Cost from payments from EC registered customers
        c.execute("""SELECT COST
                     FROM INVOICEPAYMENTS, CUSTOMERPAYMENTS, TRANSACTIONS, CUSTOMERS
                     WHERE DATESTAMP BETWEEN ? AND ?
                        AND EC_COUNTRY = 1
                        AND VAT_COUNTRY_CODE <> ?
                        AND VAT_REGISTERED = ?
                        AND INVOICEPAYMENTS.VAT_RETURN_ID IS NULL
                        AND INVOICEPAYMENTS.CUSTOMERPAYMENT_ID = CUSTOMERPAYMENTS.CUSTOMERPAYMENT_ID
                        AND CUSTOMERPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                        AND CUSTOMERPAYMENTS.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID""",
                      (from_date, to_date, vs_cfg["VAT_COUNTRY_CODE"], 1))

        for result in c:
            output_cost += D(result[0])

        # Subtract any creditnotes
        c.execute("""SELECT COST
                      FROM CREDITNOTES, TRANSACTIONS, CUSTOMERS
                     WHERE DATESTAMP BETWEEN ? AND ?
                       AND STATUS = 1
                       AND EC_COUNTRY = 1
                       AND VAT_COUNTRY_CODE <> ?
                       AND VAT_REGISTERED = ?
                       AND CREDITNOTES.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID
                       AND CREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID""",
                      (from_date, to_date, vs_cfg["VAT_COUNTRY_CODE"], 1))

        for result in c:
            output_cost -= D(result[0])

    finally:
        if conn:
            conn.close()
    return output_cost


def get_box_9(from_date, to_date):
    """Form text : Total value of all acquisitions of goods and related costs,
                   excluding any VAT, from other EC Member States.

    My interpretation : The net cost (amount without VAT) of payments made to
    all EC suppliers in the period,
    (and which have not already been included in a previous vat return),
    with the costs (again without VAT) of any
    money received from EC suppliers credit notes subtracted. An 'EC supplier'
    being a supplier who is in the EC, who does not have GB set in their country code."""

    input_cost = D("0.00")

    conn = None
    try:
        conn, c = vs_connect()

        # Get Cost from supplier invoices from EC suppliers
        c.execute("""SELECT COST
                       FROM SUPPLIERINVOICEPAYMENTS, MAKEPAYMENTS, TRANSACTIONS, SUPPLIERS
                      WHERE DATESTAMP BETWEEN ? AND ?
                        AND EC_COUNTRY = 1
                        AND VAT_COUNTRY_CODE <> ?
                        AND SUPPLIERINVOICEPAYMENTS.VAT_RETURN_ID IS NULL
                        AND SUPPLIERINVOICEPAYMENTS.MAKEPAYMENT_ID = MAKEPAYMENTS.MAKEPAYMENT_ID
                        AND MAKEPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                        AND MAKEPAYMENTS.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID""",
                      (from_date, to_date, vs_cfg["VAT_COUNTRY_CODE"]))

        for result in c:
            input_cost += D(result[0])

        # Subtract any suppliercreditnotes
        c.execute("""SELECT COST
                      FROM SUPPLIERCREDITNOTES, TRANSACTIONS, SUPPLIERS
                     WHERE TRANSACTIONS.DATESTAMP BETWEEN ? AND ?
                       AND STATUS = 1
                       AND EC_COUNTRY = 1
                       AND VAT_COUNTRY_CODE <> ?
                       AND SUPPLIERCREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                       AND SUPPLIERCREDITNOTES.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID""",
                      (from_date, to_date, vs_cfg["VAT_COUNTRY_CODE"]))

        for result in c:
            input_cost -= D(result[0])

    finally:
        if conn:
            conn.close()
    return input_cost


############### cash output and input amounts

# These are used to calculate the output-input which has gone into the VAT
# Balance account - differences from this (due to VAT on certain purchases
# not being claimed) are transerred to 'unclaimed VAT' as an expense.

# These are needed so that when submitting a return under the Flat scheme, the
# difference between the VAT paid on the flat scheme and the VAT paid on the
# cash scheme can be recorded as VAT income.


def cash_output(from_date, to_date):
    """The VAT from all payments you have received with
    dates within the period (and which have not already been included in a
    previous vat return) are added, then the vat from all credit notes
    you have issued together with a payment within the period are subtracted."""

    output_VAT = D("0.00")

    conn = None
    try:
        conn, c = vs_connect()

        # Get VAT from customer payments
        c.execute("""SELECT VAT
                       FROM INVOICEPAYMENTS, CUSTOMERPAYMENTS, TRANSACTIONS
                      WHERE DATESTAMP BETWEEN ? AND ?
                        AND INVOICEPAYMENTS.VAT_RETURN_ID IS NULL
                        AND INVOICEPAYMENTS.CUSTOMERPAYMENT_ID = CUSTOMERPAYMENTS.CUSTOMERPAYMENT_ID
                        AND CUSTOMERPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                        """, (from_date, to_date))

        for result in c:
            output_VAT += D(result[0])

        # Subtract VAT from any creditnotes sent to the customer with a payment
        # that is : creditnotes with status 1
        c.execute("""SELECT VAT
                      FROM CREDITNOTES, TRANSACTIONS
                     WHERE DATESTAMP BETWEEN ? AND ?
                       AND STATUS = 1
                       AND CREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID""",
                      (from_date, to_date))

        for result in c:
            output_VAT -= D(result[0])

    finally:
        if conn:
            conn.close()
    return output_VAT


def cash_input(from_date, to_date):
    """The VAT from all payments to suppliers made in the period
    (and which have not already been included in a previous vat return),
    with the VAT from any money received with a suppliers credit note subtracted."""

    input_VAT = D("0.00")

    conn = None
    try:
        conn, c = vs_connect()

        # Get VAT from payments to supplier
        c.execute("""SELECT VAT
                       FROM SUPPLIERINVOICEPAYMENTS, MAKEPAYMENTS, TRANSACTIONS
                      WHERE DATESTAMP BETWEEN ? AND ?
                        AND SUPPLIERINVOICEPAYMENTS.VAT_RETURN_ID IS NULL
                        AND SUPPLIERINVOICEPAYMENTS.MAKEPAYMENT_ID = MAKEPAYMENTS.MAKEPAYMENT_ID
                        AND MAKEPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                        """, (from_date, to_date))

        for result in c:
            input_VAT += D(result[0])

        # Subtract VAT from any supplier creditnotes received with a payment
        c.execute("""SELECT VAT
                      FROM SUPPLIERCREDITNOTES, TRANSACTIONS
                     WHERE TRANSACTIONS.DATESTAMP BETWEEN ? AND ?
                       AND STATUS = 1
                       AND SUPPLIERCREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                       """, (from_date, to_date))
        for result in c:
            input_VAT -= D(result[0])

    finally:
        if conn:
            conn.close()

    return input_VAT


################### the detail report ############################

def cash_detail(from_date, to_date, vat_return, vat_scheme):
    """Create a detailed report for the cash vat method
       returns a ReportForm object"""

    report = ReportForm("VAT detail report")

    report.header = ["For the period:",
                     "From: %s" % from_date.strftime("%d %b %Y"),
                     "To: %s" % to_date.strftime("%d %b %Y"),
                     "",
                     "Method: Cash"]

    if vs_cfg["DELETE_TO_DATE"]:
        if from_date < vs_cfg["DELETE_TO_DATE"]:
            report.header.append("!!WARNING!! - this report may be innaccurate")
            report.header.append(
            "Data prior to %s has been deleted from the database" % vs_cfg["DELETE_TO_DATE"].strftime("%d %b %Y"))

    # tables are described by a list of list,
    # each list being a row, consisting of cell tuple
    # each cell tuple containing two strings; 
    # The first string is formed from "XXXN",
    # and the second string is the text to be placed in the cell

    # N is the collspan number, as used in a table cell tag attribute
    # such as 'colspan="4"' - in which case N is 4

    # so normally N is '1'

    # XXX is a formatting string :
    # HBL  - Header Bold Left Justified (R for Right instead of L
    # HNL  - Header Normal Left Justified  etc
    # DNL  - Data Cell Normal Left Justified  etc
    # XXC  - Right Justified currency, as pulled from the database

    # Set the running totals
    input_VAT = D("0.00")
    input_VAT_correction = D("0.00")
    output_VAT = D("0.00")
    output_VAT_correction = D("0.00")
    output_sales = D("0.00")
    output_sales_correction = D("0.00")
    input_purchases = D("0.00")
    input_purchases_correction = D("0.00")
    EC_output_sales = D("0.00")
    EC_output_sales_correction = D("0.00")
    EC_input_purchases = D("0.00")
    EC_input_purchases_correction = D("0.00")
    unclaimed_cost = D("0.00")
    unclaimed_VAT = D("0.00")


    conn=None
    try:
        # Make database connection
        conn, c = vs_connect()

        ##### BOX 1
        # Payments to customer headings
        report.table = [ [('HBL6', 'BOX 1 --------')],
                        [ ('HNL6', 'Output VAT from payments received from customers') ],
                        [ ('HBL1', 'Date'),
                          ('HBL1', 'Customer'),
                          ('HBL1', 'Invoice number'),
                          ('HBL1', 'Payment transaction'),
                          ('HBR1', 'Net Cost'),
                          ('HBR1', 'VAT') ]]

        # Get the payments from the database
        # UNION select needed as invoices may be NULL if a delete old data has occurred
        if vat_return:
            # This is for a previous vat return, in which a vat_return_id exists
            # so search for payments with this vat_return_id
            c.execute("""SELECT DATESTAMP,
                                NAME,
                                INVOICES.INVOICE_NUMBER,
                                CUSTOMERPAYMENTS.TRANSACTION_ID,
                                INVOICEPAYMENTS.COST,
                                INVOICEPAYMENTS.VAT
                           FROM INVOICEPAYMENTS, CUSTOMERPAYMENTS, TRANSACTIONS, CUSTOMERS, INVOICES
                          WHERE INVOICEPAYMENTS.VAT_RETURN_ID = ?
                            AND INVOICEPAYMENTS.CUSTOMERPAYMENT_ID = CUSTOMERPAYMENTS.CUSTOMERPAYMENT_ID
                            AND INVOICEPAYMENTS.INVOICE_ID = INVOICES.INVOICE_ID
                            AND CUSTOMERPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                            AND CUSTOMERPAYMENTS.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID
                      UNION
                         SELECT DATESTAMP,
                                NAME,
                                NULL,
                                CUSTOMERPAYMENTS.TRANSACTION_ID,
                                INVOICEPAYMENTS.COST,
                                INVOICEPAYMENTS.VAT
                           FROM INVOICEPAYMENTS, CUSTOMERPAYMENTS, TRANSACTIONS, CUSTOMERS
                          WHERE INVOICEPAYMENTS.VAT_RETURN_ID = ?
                            AND INVOICEPAYMENTS.CUSTOMERPAYMENT_ID = CUSTOMERPAYMENTS.CUSTOMERPAYMENT_ID
                            AND INVOICEPAYMENTS.INVOICE_ID IS NULL
                            AND CUSTOMERPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                            AND CUSTOMERPAYMENTS.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID

                       ORDER BY DATESTAMP""", (vat_return.id, vat_return.id))
        else:
            # This is for the current period, in which no vat_return_id exists
            # so search for payments within the dates, but with a null vat_return_id
            # so any payment which already has vat payed on it is not included
            # (if the previous scheme was standard fro example)
            c.execute("""SELECT DATESTAMP,
                                NAME,
                                INVOICES.INVOICE_NUMBER,
                                CUSTOMERPAYMENTS.TRANSACTION_ID,
                                INVOICEPAYMENTS.COST,
                                INVOICEPAYMENTS.VAT
                           FROM INVOICEPAYMENTS, CUSTOMERPAYMENTS, TRANSACTIONS, CUSTOMERS, INVOICES
                          WHERE DATESTAMP BETWEEN ? AND ?
                            AND INVOICEPAYMENTS.VAT_RETURN_ID IS NULL
                            AND INVOICEPAYMENTS.CUSTOMERPAYMENT_ID = CUSTOMERPAYMENTS.CUSTOMERPAYMENT_ID
                            AND INVOICEPAYMENTS.INVOICE_ID = INVOICES.INVOICE_ID
                            AND CUSTOMERPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                            AND CUSTOMERPAYMENTS.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID
                      UNION
                         SELECT DATESTAMP,
                                NAME,
                                NULL,
                                CUSTOMERPAYMENTS.TRANSACTION_ID,
                                INVOICEPAYMENTS.COST,
                                INVOICEPAYMENTS.VAT
                           FROM INVOICEPAYMENTS, CUSTOMERPAYMENTS, TRANSACTIONS, CUSTOMERS
                          WHERE DATESTAMP BETWEEN ? AND ?
                            AND INVOICEPAYMENTS.VAT_RETURN_ID IS NULL
                            AND INVOICEPAYMENTS.CUSTOMERPAYMENT_ID = CUSTOMERPAYMENTS.CUSTOMERPAYMENT_ID
                            AND INVOICEPAYMENTS.INVOICE_ID IS NULL
                            AND CUSTOMERPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                            AND CUSTOMERPAYMENTS.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID

                       ORDER BY DATESTAMP""", (from_date, to_date, from_date, to_date))

        for info in c:
            # Fill table with payments from customers data

            # It looks as if the UNION function returns a unicode value of the datestamp, rather
            # than a datetime.date object
            if isinstance(info[0], datetime.date):
                datestamp = info[0].strftime("%d %b %Y")
            else:
                int_dates = map(int, info[0].split("-"))
                datestamp = datetime.date(*int_dates).strftime("%d %b %Y")

            # The invoice number could be None if no invoice exists
            # in which case display a '?' string 
            if info[2]:
                invoicenumber = str(info[2])
            else:
                invoicenumber = "?"

            report.add_row_to_table( ('DNL1', datestamp),
                                     ('DNL1', info[1]),
                                     ('DNL1', invoicenumber),
                                     ('DNL1', str(info[3])),
                                     ('DNC1', info[4]),
                                     ('DNC1', info[5]) )

            output_sales +=D(info[4])
            output_VAT += D(info[5])

        report.add_row_to_table( ('DNR4', "Totals:"),
                                 ('DNC1', str(output_sales)),
                                 ('DNC1', str(output_VAT)) )

        # Now list creditnotes with payments issued by you - Output VAT corrections 

        # Check if there are any VAT corrections from creditnotes issued with a payment
        c.execute("""SELECT COUNT(*) FROM CREDITNOTES, TRANSACTIONS
                     WHERE DATESTAMP BETWEEN ? AND ?
                       AND STATUS = 1
                       AND CREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID""",
                      (from_date, to_date))
        result = c.fetchone()

        if result[0]:
            report.add_row_to_table( ('HNL6', 'Output VAT correction from credit notes issued with a payment') )

            report.add_row_to_table( ('HBL1', 'Date'),
                                     ('HBL1', 'Customer'),
                                     ('HBL1', 'Credit Note'),
                                     ('HBL1', 'Payment transaction'),
                                     ('HBR1', 'Net Cost'),
                                     ('HBR1', 'VAT') )

            c.execute("""SELECT DATESTAMP,
                                CUSTOMERS.NAME,
                                CREDITNOTE_NUMBER,
                                CREDITNOTES.TRANSACTION_ID,
                                COST,
                                VAT
                           FROM CREDITNOTES, CUSTOMERS, TRANSACTIONS
                          WHERE DATESTAMP BETWEEN ? AND ?
                            AND STATUS = 1
                            AND CREDITNOTES.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID
                            AND CREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                       ORDER BY DATESTAMP""", (from_date, to_date))

            for info in c:
                report.add_row_to_table( ('DNL1', info[0].strftime("%d %b %Y")),
                                         ('DNL1', info[1]),
                                         ('DNL1', info[2]),
                                         ('DNL1', info[3]),
                                         ('DNC1', info[4]),
                                         ('DNC1', info[5]) )

                output_sales_correction += D(info[4])
                output_VAT_correction += D(info[5])

            report.add_row_to_table( ('DNR4', "Totals:"),
                                     ('DNC1', str(output_sales_correction)),
                                     ('DNC1', str(output_VAT_correction)) )

        if output_VAT_correction:
            output_VAT = output_VAT - output_VAT_correction
            report.add_row_to_table( ('HBL4', "Output VAT corrected from payments sent with credit notes"),
                                     ('HBR1', "BOX 1:"),
                                     ('HBC1', str(output_VAT)) )
        else:
            report.add_row_to_table( ('HBR5', "BOX 1:"),
                                     ('HBC1', str(output_VAT)) )

        ##### BOX 2
        report.add_row_to_table( ('HBL6', "BOX 2 --------") )
        if vat_return:
            # get the stored values
            EC_cost = vat_return.box_nine
            standard_rate = vat_return.standard_rate
            box_2 = vat_return.box_two
        else:
            # calculate using current values
            EC_cost = get_box_9(from_date, to_date)
            standard_rate = vs_cfg["VAT_STANDARD_RATE"]
            if EC_cost == D(0):
                box_2 = D("0.00")
            else:
                result = (EC_cost * standard_rate) / D("100.0000")
                box_2 = result.quantize(D('.01'), rounding=decimal.ROUND_HALF_EVEN)

        report.add_row_to_table( ('DNR5', "Value from BOX 9:"), ('DNC1', str(EC_cost)) )
        report.add_row_to_table( ('DNR5', "At the standard VAT rate of:"), ('DNR1', str(standard_rate).replace(".", vs_cfg["CP"])) )
        report.add_row_to_table( ('HBR5', "BOX 2:"), ('HBC1', str(box_2)) )


        ##### BOX 3
        report.add_row_to_table( ('HBL6', "BOX 3 --------") )
        report.add_row_to_table( ('HBL4', "Total VAT due (the sum of boxes 1 and 2)"),
                                 ('HBR1', "BOX 3:"),
                                 ('HBC1', str(box_2 + output_VAT)) )

        ##### BOX 4
        report.add_row_to_table( ('HBL6', "BOX 4 --------") )
        report.add_row_to_table( ('HNL6', 'Input VAT from payments made to suppliers') )
        report.add_row_to_table( ('HBL1', 'Date'),
                                  ('HBL1', 'Supplier'),
                                  ('HBL1', 'Order number'),
                                  ('HBL1', 'Payment transaction'),
                                  ('HBR1', 'Net Cost'),
                                  ('HBR1', 'VAT') )

        # Get VAT from payments to supplier
        # UNION select needed as purchaseorders may be NULL if a delete old data has occurred

        if vat_return:
            # This is for a previous vat return
            c.execute("""SELECT DATESTAMP,
                                SUPPLIERS.NAME,
                                PURCHASEORDERS.ORDER_NUMBER,
                                MAKEPAYMENTS.TRANSACTION_ID,
                                SUPPLIERINVOICEPAYMENTS.COST,
                                SUPPLIERINVOICEPAYMENTS.VAT
                           FROM SUPPLIERINVOICEPAYMENTS, MAKEPAYMENTS, TRANSACTIONS, SUPPLIERS, PURCHASEORDERS
                          WHERE SUPPLIERINVOICEPAYMENTS.VAT_RETURN_ID = ?
                            AND SUPPLIERINVOICEPAYMENTS.CLAIM_VAT = 1
                            AND SUPPLIERINVOICEPAYMENTS.MAKEPAYMENT_ID = MAKEPAYMENTS.MAKEPAYMENT_ID
                            AND MAKEPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                            AND MAKEPAYMENTS.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID
                            AND MAKEPAYMENTS.PURCHASEORDER_ID = PURCHASEORDERS.PURCHASEORDER_ID
                    UNION
                         SELECT DATESTAMP,
                                SUPPLIERS.NAME,
                                NULL,
                                MAKEPAYMENTS.TRANSACTION_ID,
                                SUPPLIERINVOICEPAYMENTS.COST,
                                SUPPLIERINVOICEPAYMENTS.VAT
                           FROM SUPPLIERINVOICEPAYMENTS, MAKEPAYMENTS, TRANSACTIONS, SUPPLIERS
                          WHERE SUPPLIERINVOICEPAYMENTS.VAT_RETURN_ID = ?
                            AND SUPPLIERINVOICEPAYMENTS.CLAIM_VAT = 1
                            AND SUPPLIERINVOICEPAYMENTS.MAKEPAYMENT_ID = MAKEPAYMENTS.MAKEPAYMENT_ID
                            AND MAKEPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                            AND MAKEPAYMENTS.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID
                            AND MAKEPAYMENTS.PURCHASEORDER_ID IS NULL

                       ORDER BY TRANSACTIONS.DATESTAMP""", (vat_return.id, vat_return.id))
        else:
            # This is for the current period
            c.execute("""SELECT DATESTAMP,
                                SUPPLIERS.NAME,
                                PURCHASEORDERS.ORDER_NUMBER,
                                MAKEPAYMENTS.TRANSACTION_ID,
                                SUPPLIERINVOICEPAYMENTS.COST,
                                SUPPLIERINVOICEPAYMENTS.VAT
                           FROM SUPPLIERINVOICEPAYMENTS, MAKEPAYMENTS, TRANSACTIONS, SUPPLIERS, PURCHASEORDERS
                          WHERE DATESTAMP BETWEEN ? AND ?
                            AND SUPPLIERINVOICEPAYMENTS.VAT_RETURN_ID IS NULL
                            AND SUPPLIERINVOICEPAYMENTS.CLAIM_VAT = 1
                            AND SUPPLIERINVOICEPAYMENTS.MAKEPAYMENT_ID = MAKEPAYMENTS.MAKEPAYMENT_ID
                            AND MAKEPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                            AND MAKEPAYMENTS.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID
                            AND MAKEPAYMENTS.PURCHASEORDER_ID = PURCHASEORDERS.PURCHASEORDER_ID
                    UNION
                         SELECT DATESTAMP,
                                SUPPLIERS.NAME,
                                NULL,
                                MAKEPAYMENTS.TRANSACTION_ID,
                                SUPPLIERINVOICEPAYMENTS.COST,
                                SUPPLIERINVOICEPAYMENTS.VAT
                           FROM SUPPLIERINVOICEPAYMENTS, MAKEPAYMENTS, TRANSACTIONS, SUPPLIERS
                          WHERE DATESTAMP BETWEEN ? AND ?
                            AND SUPPLIERINVOICEPAYMENTS.VAT_RETURN_ID IS NULL
                            AND SUPPLIERINVOICEPAYMENTS.CLAIM_VAT = 1
                            AND SUPPLIERINVOICEPAYMENTS.MAKEPAYMENT_ID = MAKEPAYMENTS.MAKEPAYMENT_ID
                            AND MAKEPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                            AND MAKEPAYMENTS.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID
                            AND MAKEPAYMENTS.PURCHASEORDER_ID IS NULL

                       ORDER BY TRANSACTIONS.DATESTAMP""", (from_date, to_date, from_date, to_date))
        
        for info in c:
            # Input VAT from payments to suppliers

            # It looks as if the UNION function returns a unicode value of the datestamp, rather
            # than a datetime.date object
            if isinstance(info[0], datetime.date):
                datestamp = info[0].strftime("%d %b %Y")
            else:
                int_dates = map(int, info[0].split("-"))
                datestamp = datetime.date(*int_dates).strftime("%d %b %Y")

            # The purchaseorder number could be None if no order exists
            # in which case display a '?' string 
            if info[2]:
                ordernumber = str(info[2])
            else:
                ordernumber = "?"

            report.add_row_to_table( ('DNL1', datestamp),
                                     ('DNL1', info[1]),
                                     ('DNL1', ordernumber),
                                     ('DNL1', info[3]),
                                     ('DNC1', info[4]),
                                     ('DNC1', info[5]) )

            input_purchases += D(info[4])
            input_VAT += D(info[5])

        report.add_row_to_table( ('DNL3', "(This Net cost is included in Box 7)"),
                                 ('DNR1', "Totals:"),
                                 ('DNC1', str(input_purchases)),
                                 ('DNC1', str(input_VAT)) )

        # Check if there are any VAT corrections from supplier creditnotes received with a payment
        c.execute("""SELECT COUNT(*) FROM SUPPLIERCREDITNOTES, TRANSACTIONS
                      WHERE TRANSACTIONS.DATESTAMP BETWEEN ? AND ?
                        AND STATUS = 1
                        AND SUPPLIERCREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                       """, (from_date, to_date))

        result = c.fetchone()

        if result[0]:
            report.add_row_to_table( ('HNL6', 'Input VAT correction from payments received with a supplier credit note') )
            report.add_row_to_table( ('HBL1', 'Date'),
                                     ('HBL1', 'Supplier'),
                                     ('HBL1', 'Supplier Credit Note'),
                                     ('HBL1', 'Payment transaction'),
                                     ('HBR1', 'Net Cost'),
                                     ('HBR1', 'VAT') )

            c.execute("""SELECT DATESTAMP,
                                SUPPLIERS.NAME,
                                SUPPLIERCREDITNOTES.NUMBER,
                                SUPPLIERCREDITNOTES.TRANSACTION_ID,
                                COST,
                                VAT
                           FROM SUPPLIERCREDITNOTES, SUPPLIERS, TRANSACTIONS
                          WHERE DATESTAMP BETWEEN ? AND ?
                            AND STATUS = 1
                            AND SUPPLIERCREDITNOTES.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID
                            AND SUPPLIERCREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                       ORDER BY DATESTAMP""", (from_date, to_date))

            for info in c:
                report.add_row_to_table( ('DNL1', info[0].strftime("%d %b %Y")),
                                         ('DNL1', info[1]),
                                         ('DNL1', info[2]),
                                         ('DNL1', info[3]),
                                         ('DNC1', info[4]),
                                         ('DNC1', info[5]) )

                input_purchases_correction += D(info[4])
                input_VAT_correction += D(info[5])

            report.add_row_to_table( ('DNL3', "(This Net cost is subtracted in Box 7)"),
                                     ('DNR1', "Totals:"),
                                     ('DNC1', str(input_purchases_correction)),
                                     ('DNC1', str(input_VAT_correction)) )

            input_VAT = input_VAT - input_VAT_correction

            report.add_row_to_table( ('DNR5', "Input VAT corrected from credit notes received:"),
                                     ('DNC1', str(input_VAT)) )

        report.add_row_to_table( ('DNR5', "Value from BOX 2:"), ('DNC1', str(box_2)) )

        report.add_row_to_table( ('HBL4', "Input VAT plus BOX 2"),
                                 ('HBR1', "BOX 4:"),
                                 ('HBC1', str(box_2 + input_VAT)) )

        ##### BOX 5
        report.add_row_to_table( ('HBL6', "BOX 5 --------") )
        report.add_row_to_table( ('HBL4', "Net VAT (Difference between boxes 3 and 4)"),
                                 ('HBR1', "BOX 5:"),
                                 ('HBC1', str(output_VAT - input_VAT)) )

        ##### BOX 6
        report.add_row_to_table( ('HBL6', "BOX 6 --------") )
        if not output_sales_correction:
            report.add_row_to_table( ('HBL4', "Value of sales from above payments received from customers"),
                                     ('HBR1', "BOX 6:"),
                                     ('HBC1', str(output_sales)) )
        else:
           report.add_row_to_table( ('DNR5', "Value of sales from above payments received from customers"), ("DNC1", str(output_sales)) )
           report.add_row_to_table( ('DNR5', "Minus correction from payments issued with credit notes"), ("DNC1", str(output_sales_correction)) )
           report.add_row_to_table( ('HBR5', "BOX 6:"), ('HBC1', str(output_sales - output_sales_correction)) )

        ##### BOX 7
        report.add_row_to_table( ('HBL6', "BOX 7 --------") )

        # List payments made with unclaimed VAT
        if vat_return:
            # This is for a previous vat return
            c.execute("""SELECT COUNT(*) FROM SUPPLIERINVOICEPAYMENTS
                         WHERE SUPPLIERINVOICEPAYMENTS.VAT_RETURN_ID = ?
                           AND SUPPLIERINVOICEPAYMENTS.CLAIM_VAT = 0""", (vat_return.id,))
            result = c.fetchone()
        else:
            c.execute("""SELECT COUNT(*) FROM SUPPLIERINVOICEPAYMENTS, TRANSACTIONS, MAKEPAYMENTS
                          WHERE DATESTAMP BETWEEN ? AND ?
                            AND SUPPLIERINVOICEPAYMENTS.VAT_RETURN_ID IS NULL
                            AND SUPPLIERINVOICEPAYMENTS.CLAIM_VAT = 0
                            AND SUPPLIERINVOICEPAYMENTS.MAKEPAYMENT_ID = MAKEPAYMENTS.MAKEPAYMENT_ID
                            AND MAKEPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID""", (from_date, to_date))
            result = c.fetchone()

        if not result[0]:
            if not input_purchases_correction:
                report.add_row_to_table( ('HBL3', "Value of purchases from above payments to suppliers"),
                                         ('HBR1', "BOX 7:"),
                                         ('HBC1', str(input_purchases)),
                                         ('DNL1', "") )
            else:
               report.add_row_to_table( ('DNR4', "Value of purchases from above payments to suppliers"),
                                        ("DNC1", str(input_purchases)),
                                        ('DNL1', "") )
               report.add_row_to_table( ('DNR4', "Minus correction from payments received with credit notes"),
                                        ("DNC1", str(input_purchases_correction)),
                                        ('DNL1', "") )
               report.add_row_to_table( ('HBR4', "BOX 7:"),
                                        ('HBC1', str(input_purchases - input_purchases_correction)),
                                        ('DNL1', "") )
        else:
            report.add_row_to_table( ('HNL6', 'Payments made, set as unclaimed VAT and not included in input VAT') )
            report.add_row_to_table( ('HBL1', 'Date'),
                                     ('HBL1', 'Supplier'),
                                     ('HBL1', 'Order number'),
                                     ('HBL1', 'Payment transaction'),
                                     ('HBR1', 'Net Cost'),
                                     ('HBR1', 'VAT') )

            if vat_return:
                # This is for a previous vat return
                c.execute("""SELECT DATESTAMP,
                                    SUPPLIERS.NAME,
                                    PURCHASEORDERS.ORDER_NUMBER,
                                    MAKEPAYMENTS.TRANSACTION_ID,
                                    SUPPLIERINVOICEPAYMENTS.COST,
                                    SUPPLIERINVOICEPAYMENTS.VAT
                               FROM SUPPLIERINVOICEPAYMENTS, MAKEPAYMENTS, TRANSACTIONS, SUPPLIERS, PURCHASEORDERS
                              WHERE SUPPLIERINVOICEPAYMENTS.VAT_RETURN_ID = ?
                                AND SUPPLIERINVOICEPAYMENTS.CLAIM_VAT = 0
                                AND SUPPLIERINVOICEPAYMENTS.MAKEPAYMENT_ID = MAKEPAYMENTS.MAKEPAYMENT_ID
                                AND MAKEPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                                AND MAKEPAYMENTS.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID
                                AND MAKEPAYMENTS.PURCHASEORDER_ID = PURCHASEORDERS.PURCHASEORDER_ID
                        UNION
                             SELECT DATESTAMP,
                                    SUPPLIERS.NAME,
                                    NULL,
                                    MAKEPAYMENTS.TRANSACTION_ID,
                                    SUPPLIERINVOICEPAYMENTS.COST,
                                    SUPPLIERINVOICEPAYMENTS.VAT
                               FROM SUPPLIERINVOICEPAYMENTS, MAKEPAYMENTS, TRANSACTIONS, SUPPLIERS
                              WHERE SUPPLIERINVOICEPAYMENTS.VAT_RETURN_ID = ?
                                AND SUPPLIERINVOICEPAYMENTS.CLAIM_VAT = 0
                                AND SUPPLIERINVOICEPAYMENTS.MAKEPAYMENT_ID = MAKEPAYMENTS.MAKEPAYMENT_ID
                                AND MAKEPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                                AND MAKEPAYMENTS.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID
                                AND MAKEPAYMENTS.PURCHASEORDER_ID IS NULL

                           ORDER BY TRANSACTIONS.DATESTAMP""", (vat_return.id, vat_return.id))
            else:
                # This is for the current period
                c.execute("""SELECT DATESTAMP,
                                    SUPPLIERS.NAME,
                                    PURCHASEORDERS.ORDER_NUMBER,
                                    MAKEPAYMENTS.TRANSACTION_ID,
                                    SUPPLIERINVOICEPAYMENTS.COST,
                                    SUPPLIERINVOICEPAYMENTS.VAT
                               FROM SUPPLIERINVOICEPAYMENTS, MAKEPAYMENTS, TRANSACTIONS, SUPPLIERS, PURCHASEORDERS
                              WHERE DATESTAMP BETWEEN ? AND ?
                                AND SUPPLIERINVOICEPAYMENTS.VAT_RETURN_ID IS NULL
                                AND SUPPLIERINVOICEPAYMENTS.CLAIM_VAT = 0
                                AND SUPPLIERINVOICEPAYMENTS.MAKEPAYMENT_ID = MAKEPAYMENTS.MAKEPAYMENT_ID
                                AND MAKEPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                                AND MAKEPAYMENTS.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID
                                AND MAKEPAYMENTS.PURCHASEORDER_ID = PURCHASEORDERS.PURCHASEORDER_ID
                        UNION
                             SELECT DATESTAMP,
                                    SUPPLIERS.NAME,
                                    NULL,
                                    MAKEPAYMENTS.TRANSACTION_ID,
                                    SUPPLIERINVOICEPAYMENTS.COST,
                                    SUPPLIERINVOICEPAYMENTS.VAT
                               FROM SUPPLIERINVOICEPAYMENTS, MAKEPAYMENTS, TRANSACTIONS, SUPPLIERS
                              WHERE DATESTAMP BETWEEN ? AND ?
                                AND SUPPLIERINVOICEPAYMENTS.CLAIM_VAT = 0
                                AND SUPPLIERINVOICEPAYMENTS.VAT_RETURN_ID IS NULL
                                AND SUPPLIERINVOICEPAYMENTS.MAKEPAYMENT_ID = MAKEPAYMENTS.MAKEPAYMENT_ID
                                AND MAKEPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                                AND MAKEPAYMENTS.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID
                                AND MAKEPAYMENTS.PURCHASEORDER_ID IS NULL

                           ORDER BY TRANSACTIONS.DATESTAMP""", (from_date, to_date, from_date, to_date))
            
            for info in c:
                # It looks as if the UNION function returns a unicode value of the datestamp, rather
                # than a datetime.date object
                if isinstance(info[0], datetime.date):
                    datestamp = info[0].strftime("%d %b %Y")
                else:
                    int_dates = map(int, info[0].split("-"))
                    datestamp = datetime.date(*int_dates).strftime("%d %b %Y")

                # The purchaseorder number could be None if no order exists
                # in which case display a '?' string 
                if info[2]:
                    ordernumber = str(info[2])
                else:
                    ordernumber = "?"

                report.add_row_to_table( ('DNL1', datestamp),
                                         ('DNL1', info[1]),
                                         ('DNL1', ordernumber),
                                         ('DNL1', info[3]),
                                         ('DNC1', info[4]),
                                         ('DNC1', info[5]) )

                unclaimed_cost += D(info[4])
                unclaimed_VAT += D(info[5])

            if result[0]>1:
                # only do a totals if there's more than one row
                report.add_row_to_table( ('DNR4', "Unclaimed:"),
                                         ('DNC1', str(unclaimed_cost)),
                                         ('DNC1', str(unclaimed_VAT)) )

            if not input_purchases_correction:
                # No credit nodes
                report.add_row_to_table( ('HBL3', "Value of all purchases"),
                                         ('HBR1', "BOX 7:"),
                                         ('HBC1', str(input_purchases+unclaimed_cost)),
                                         ('DNL1', "") )
            else:
               report.add_row_to_table( ('DNR4', "Value of all purchases"),
                                        ("DNC1", str(input_purchases+unclaimed_cost)),
                                        ('DNL1', "") )
               report.add_row_to_table( ('DNR4', "Minus correction from credit notes"),
                                        ("DNC1", str(input_purchases_correction)),
                                        ('DNL1', "") )
               report.add_row_to_table( ('HBR4', "BOX 7:"),
                   ('HBC1', str(input_purchases+unclaimed_cost-input_purchases_correction)),
                   ('DNL1', "") )

        ##### BOX 8
        report.add_row_to_table( ('HBL6', "BOX 8 --------") )
        report.add_row_to_table( ('HNL6', 'Payments received from customers in other EC Member States') )
        report.add_row_to_table( ('HBL1', 'Date'),
                                 ('HBL1', 'Customer'),
                                 ('HBL1', 'Country code'),
                                 ('HBL1', 'Invoice number'),
                                 ('HBL1', 'Payment transaction'),
                                 ('HBR1', 'Net Cost') )

        # Get the payments from the database
        if vat_return:
            # This is for a previous vat return, in which a vat_return_id exists
            # so search for payments with this vat_return_id
            c.execute("""SELECT DATESTAMP,
                                NAME,
                                VAT_COUNTRY_CODE,
                                INVOICES.INVOICE_NUMBER,
                                CUSTOMERPAYMENTS.TRANSACTION_ID,
                                INVOICEPAYMENTS.COST
                           FROM INVOICEPAYMENTS, CUSTOMERPAYMENTS, TRANSACTIONS, CUSTOMERS, INVOICES
                          WHERE INVOICEPAYMENTS.VAT_RETURN_ID = ?
                            AND EC_COUNTRY = 1
                            AND VAT_COUNTRY_CODE <> ?
                            AND VAT_REGISTERED = 1
                            AND INVOICEPAYMENTS.CUSTOMERPAYMENT_ID = CUSTOMERPAYMENTS.CUSTOMERPAYMENT_ID
                            AND INVOICEPAYMENTS.INVOICE_ID = INVOICES.INVOICE_ID
                            AND CUSTOMERPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                            AND CUSTOMERPAYMENTS.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID
                       UNION
                         SELECT DATESTAMP,
                                NAME,
                                VAT_COUNTRY_CODE,
                                NULL,
                                CUSTOMERPAYMENTS.TRANSACTION_ID,
                                INVOICEPAYMENTS.COST
                           FROM INVOICEPAYMENTS, CUSTOMERPAYMENTS, TRANSACTIONS, CUSTOMERS
                          WHERE INVOICEPAYMENTS.VAT_RETURN_ID = ?
                            AND EC_COUNTRY = 1
                            AND VAT_COUNTRY_CODE <> ?
                            AND VAT_REGISTERED = 1
                            AND INVOICEPAYMENTS.CUSTOMERPAYMENT_ID = CUSTOMERPAYMENTS.CUSTOMERPAYMENT_ID
                            AND INVOICEPAYMENTS.INVOICE_ID IS NULL
                            AND CUSTOMERPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                            AND CUSTOMERPAYMENTS.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID

                       ORDER BY DATESTAMP""", (vat_return.id, vs_cfg["VAT_COUNTRY_CODE"], vat_return.id, vs_cfg["VAT_COUNTRY_CODE"]))
        else:
            # This is for the current period, in which no vat_return_id exists
            # so search for payments within the dates, but with a null vat_return_id
            # so any payment which already has vat payed on it is not included
            # (if the previous scheme was standard for example)
            c.execute("""SELECT DATESTAMP,
                                NAME,
                                VAT_COUNTRY_CODE,
                                INVOICES.INVOICE_NUMBER,
                                CUSTOMERPAYMENTS.TRANSACTION_ID,
                                INVOICEPAYMENTS.COST
                           FROM INVOICEPAYMENTS, CUSTOMERPAYMENTS, TRANSACTIONS, CUSTOMERS, INVOICES
                          WHERE DATESTAMP BETWEEN ? AND ?
                            AND EC_COUNTRY = 1
                            AND VAT_COUNTRY_CODE <> ?
                            AND VAT_REGISTERED = 1
                            AND INVOICEPAYMENTS.VAT_RETURN_ID IS NULL
                            AND INVOICEPAYMENTS.CUSTOMERPAYMENT_ID = CUSTOMERPAYMENTS.CUSTOMERPAYMENT_ID
                            AND INVOICEPAYMENTS.INVOICE_ID = INVOICES.INVOICE_ID
                            AND CUSTOMERPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                            AND CUSTOMERPAYMENTS.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID
                       UNION
                         SELECT DATESTAMP,
                                NAME,
                                VAT_COUNTRY_CODE,
                                NULL,
                                CUSTOMERPAYMENTS.TRANSACTION_ID,
                                INVOICEPAYMENTS.COST
                           FROM INVOICEPAYMENTS, CUSTOMERPAYMENTS, TRANSACTIONS, CUSTOMERS
                          WHERE DATESTAMP BETWEEN ? AND ?
                            AND EC_COUNTRY = 1
                            AND VAT_COUNTRY_CODE <> ?
                            AND VAT_REGISTERED = 1
                            AND INVOICEPAYMENTS.VAT_RETURN_ID IS NULL
                            AND INVOICEPAYMENTS.CUSTOMERPAYMENT_ID = CUSTOMERPAYMENTS.CUSTOMERPAYMENT_ID
                            AND INVOICEPAYMENTS.INVOICE_ID IS NULL
                            AND CUSTOMERPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                            AND CUSTOMERPAYMENTS.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID

                       ORDER BY DATESTAMP""", (from_date, to_date, vs_cfg["VAT_COUNTRY_CODE"],
                                               from_date, to_date, vs_cfg["VAT_COUNTRY_CODE"]))

        for info in c:
            # Fill table with payments from customers data

            # It looks as if the UNION function returns a unicode value of the datestamp, rather
            # than a datetime.date object
            if isinstance(info[0], datetime.date):
                datestamp = info[0].strftime("%d %b %Y")
            else:
                int_dates = map(int, info[0].split("-"))
                datestamp = datetime.date(*int_dates).strftime("%d %b %Y")

            # The invoice number could be None if no invoice exists
            # in which case display a '?' string 
            if info[3]:
                invoicenumber = str(info[3])
            else:
                invoicenumber = "?"


            report.add_row_to_table( ('DNL1', datestamp),
                                     ('DNL1', info[1]),
                                     ('DNL1', info[2]),
                                     ('DNL1', invoicenumber),
                                     ('DNL1', str(info[4])),
                                     ('DNC1', info[5]) )

            EC_output_sales +=D(info[5])

        report.add_row_to_table( ('DNR5', "Total:"),
                                 ('DNC1', str(EC_output_sales)) )


        # Now list creditnotes with payments issued by you - Output VAT corrections 

        # Check if there are any VAT corrections from creditnotes issued with a payment
        c.execute("""SELECT COUNT(*) FROM CREDITNOTES, TRANSACTIONS, CUSTOMERS
                     WHERE DATESTAMP BETWEEN ? AND ?
                       AND STATUS = 1
                       AND EC_COUNTRY = 1
                       AND VAT_COUNTRY_CODE <> ?
                       AND VAT_REGISTERED = 1
                       AND CREDITNOTES.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID
                       AND CREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID""",
                      (from_date, to_date, vs_cfg["VAT_COUNTRY_CODE"]))

        result = c.fetchone()

        if result[0]:
            report.add_row_to_table( ('HNL6', 'Correction from payments issued with credit notes') )

            report.add_row_to_table( ('HBL1', 'Date'),
                                     ('HBL1', 'Customer'),
                                     ('HBL1', 'Country code'),
                                     ('HBL1', 'Credit Note'),
                                     ('HBL1', 'Payment transaction'),
                                     ('HBR1', 'Net Cost') )

            c.execute("""SELECT DATESTAMP,
                                CUSTOMERS.NAME,
                                VAT_COUNTRY_CODE,
                                CREDITNOTE_NUMBER,
                                CREDITNOTES.TRANSACTION_ID,
                                COST
                           FROM CREDITNOTES, CUSTOMERS, TRANSACTIONS
                          WHERE DATESTAMP BETWEEN ? AND ?
                            AND STATUS = 1
                            AND EC_COUNTRY = 1
                            AND VAT_COUNTRY_CODE <> ?
                            AND VAT_REGISTERED = 1
                            AND CREDITNOTES.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID
                            AND CREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                       ORDER BY DATESTAMP""", (from_date, to_date, vs_cfg["VAT_COUNTRY_CODE"]))

            for info in c:
                report.add_row_to_table( ('DNL1', info[0].strftime("%d %b %Y")),
                                         ('DNL1', info[1]),
                                         ('DNL1', info[2]),
                                         ('DNL1', info[3]),
                                         ('DNL1', str(info[4])),
                                         ('DNC1', info[5]) )

                EC_output_sales_correction += D(info[5])

            report.add_row_to_table( ('DNR5', "Total:"),
                                     ('DNC1', str(EC_output_sales_correction)) )

        if EC_output_sales_correction:
            EC_output_sales = EC_output_sales - EC_output_sales_correction
            report.add_row_to_table( ('HBL4', "Sales corrected from credit notes issued"),
                                     ('HBR1', "BOX 8:"),
                                     ('HBC1', str(EC_output_sales)) )
        else:
            report.add_row_to_table( ('HBR5', "BOX 8:"),
                                     ('HBC1', str(EC_output_sales)) )

        ##### BOX 9
        report.add_row_to_table( ('HBL6', "BOX 9 --------") )
        report.add_row_to_table( ('HNL6', 'Payments made to companies in other EC Member States') )
        report.add_row_to_table( ('HBL1', 'Date'),
                                 ('HBL1', 'Supplier'),
                                 ('HBL1', 'Country code'),
                                 ('HBL1', 'Order number'),
                                 ('HBL1', 'Payment transaction'),
                                 ('HBR1', 'Net Cost') )

        # Get payments to EC suppliers
        # UNION select needed as purchaseorders may be NULL if a delete old data has occurred

        if vat_return:
            # This is for a previous vat return
            c.execute("""SELECT DATESTAMP,
                                SUPPLIERS.NAME,
                                VAT_COUNTRY_CODE,
                                PURCHASEORDERS.ORDER_NUMBER,
                                MAKEPAYMENTS.TRANSACTION_ID,
                                SUPPLIERINVOICEPAYMENTS.COST
                           FROM SUPPLIERINVOICEPAYMENTS, MAKEPAYMENTS, TRANSACTIONS, SUPPLIERS, PURCHASEORDERS
                          WHERE SUPPLIERINVOICEPAYMENTS.VAT_RETURN_ID = ?
                            AND EC_COUNTRY = 1
                            AND VAT_COUNTRY_CODE <> ?
                            AND SUPPLIERINVOICEPAYMENTS.MAKEPAYMENT_ID = MAKEPAYMENTS.MAKEPAYMENT_ID
                            AND MAKEPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                            AND MAKEPAYMENTS.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID
                            AND MAKEPAYMENTS.PURCHASEORDER_ID = PURCHASEORDERS.PURCHASEORDER_ID
                    UNION
                         SELECT DATESTAMP,
                                SUPPLIERS.NAME,
                                VAT_COUNTRY_CODE,
                                NULL,
                                MAKEPAYMENTS.TRANSACTION_ID,
                                SUPPLIERINVOICEPAYMENTS.COST
                           FROM SUPPLIERINVOICEPAYMENTS, MAKEPAYMENTS, TRANSACTIONS, SUPPLIERS
                          WHERE SUPPLIERINVOICEPAYMENTS.VAT_RETURN_ID = ?
                            AND EC_COUNTRY = 1
                            AND VAT_COUNTRY_CODE <> ?
                            AND SUPPLIERINVOICEPAYMENTS.MAKEPAYMENT_ID = MAKEPAYMENTS.MAKEPAYMENT_ID
                            AND MAKEPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                            AND MAKEPAYMENTS.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID
                            AND MAKEPAYMENTS.PURCHASEORDER_ID IS NULL

                       ORDER BY TRANSACTIONS.DATESTAMP""", (vat_return.id, vs_cfg["VAT_COUNTRY_CODE"], vat_return.id, vs_cfg["VAT_COUNTRY_CODE"]))
        else:
            # This is for the current period
            c.execute("""SELECT DATESTAMP,
                                SUPPLIERS.NAME,
                                VAT_COUNTRY_CODE,
                                PURCHASEORDERS.ORDER_NUMBER,
                                MAKEPAYMENTS.TRANSACTION_ID,
                                SUPPLIERINVOICEPAYMENTS.COST
                           FROM SUPPLIERINVOICEPAYMENTS, MAKEPAYMENTS, TRANSACTIONS, SUPPLIERS, PURCHASEORDERS
                          WHERE DATESTAMP BETWEEN ? AND ?
                            AND SUPPLIERINVOICEPAYMENTS.VAT_RETURN_ID IS NULL
                            AND EC_COUNTRY = 1
                            AND VAT_COUNTRY_CODE <> ?
                            AND SUPPLIERINVOICEPAYMENTS.MAKEPAYMENT_ID = MAKEPAYMENTS.MAKEPAYMENT_ID
                            AND MAKEPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                            AND MAKEPAYMENTS.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID
                            AND MAKEPAYMENTS.PURCHASEORDER_ID = PURCHASEORDERS.PURCHASEORDER_ID
                    UNION
                         SELECT DATESTAMP,
                                SUPPLIERS.NAME,
                                VAT_COUNTRY_CODE,
                                NULL,
                                MAKEPAYMENTS.TRANSACTION_ID,
                                SUPPLIERINVOICEPAYMENTS.COST
                           FROM SUPPLIERINVOICEPAYMENTS, MAKEPAYMENTS, TRANSACTIONS, SUPPLIERS
                          WHERE DATESTAMP BETWEEN ? AND ?
                            AND SUPPLIERINVOICEPAYMENTS.VAT_RETURN_ID IS NULL
                            AND EC_COUNTRY = 1
                            AND VAT_COUNTRY_CODE <> ?
                            AND SUPPLIERINVOICEPAYMENTS.MAKEPAYMENT_ID = MAKEPAYMENTS.MAKEPAYMENT_ID
                            AND MAKEPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                            AND MAKEPAYMENTS.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID
                            AND MAKEPAYMENTS.PURCHASEORDER_ID IS NULL

                       ORDER BY TRANSACTIONS.DATESTAMP""", (from_date, to_date, vs_cfg["VAT_COUNTRY_CODE"],
                                                            from_date, to_date, vs_cfg["VAT_COUNTRY_CODE"]))
        
        for info in c:
            # Input VAT from payments to suppliers

            # It looks as if the UNION function returns a unicode value of the datestamp, rather
            # than a datetime.date object
            if isinstance(info[0], datetime.date):
                datestamp = info[0].strftime("%d %b %Y")
            else:
                int_dates = map(int, info[0].split("-"))
                datestamp = datetime.date(*int_dates).strftime("%d %b %Y")

            # The purchaseorder number could be None if no order exists
            # in which case display a '?' string 
            if info[3]:
                ordernumber = str(info[3])
            else:
                ordernumber = "?"

            report.add_row_to_table( ('DNL1', datestamp),
                                     ('DNL1', info[1]),
                                     ('DNL1', info[2]),
                                     ('DNL1', ordernumber),
                                     ('DNL1', str(info[4])),
                                     ('DNC1', info[5]) )

            EC_input_purchases +=D(info[5])

        report.add_row_to_table( ('DNR5', "Total:"),
                                 ('DNC1', str(EC_input_purchases)) )

        # Check if there are any VAT corrections from supplier creditnotes received with a payment
        c.execute("""SELECT COUNT(*) FROM SUPPLIERCREDITNOTES, TRANSACTIONS, SUPPLIERS
                     WHERE TRANSACTIONS.DATESTAMP BETWEEN ? AND ?
                       AND STATUS = 1
                       AND EC_COUNTRY = 1
                       AND VAT_COUNTRY_CODE <> ?
                       AND SUPPLIERCREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                       AND SUPPLIERCREDITNOTES.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID""",
                      (from_date, to_date, vs_cfg["VAT_COUNTRY_CODE"]))

        result = c.fetchone()

        if result[0]:
            report.add_row_to_table( ('HNL6', 'Payments received with credit notes from suppliers in other EC Member States') )
            report.add_row_to_table( ('HBL1', 'Date'),
                                     ('HBL1', 'Supplier'),
                                     ('HBL1', 'Country code'),
                                     ('HBL1', 'Supplier Credit Note'),
                                     ('HBL1', 'Payment transaction'),
                                     ('HBR1', 'Net Cost') )

            c.execute("""SELECT DATESTAMP,
                                SUPPLIERS.NAME,
                                VAT_COUNTRY_CODE,
                                SUPPLIERCREDITNOTES.NUMBER,
                                SUPPLIERCREDITNOTES.TRANSACTION_ID,
                                COST
                           FROM SUPPLIERCREDITNOTES, SUPPLIERS, TRANSACTIONS
                          WHERE DATESTAMP BETWEEN ? AND ?
                            AND STATUS = 1
                            AND EC_COUNTRY = 1
                            AND VAT_COUNTRY_CODE <> ?
                            AND SUPPLIERCREDITNOTES.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID
                            AND SUPPLIERCREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                       ORDER BY DATESTAMP""", (from_date, to_date))

            for info in c:
                report.add_row_to_table( ('DNL1', info[0].strftime("%d %b %Y")),
                                         ('DNL1', info[1]),
                                         ('DNL1', info[2]),
                                         ('DNL1', info[3]),
                                         ('DNL1', str(info[4])),
                                         ('DNC1', info[5]) )

                EC_input_purchases_correction += D(info[5])

            report.add_row_to_table( ('DNR5', "Total:"),
                                     ('DNC1', str(EC_input_purchases_correction)) )

        if EC_input_purchases_correction:
            EC_input_purchases = EC_input_purchases - EC_input_purchases_correction
            report.add_row_to_table( ('HBL4', "EC Purchases corrected from credit notes received"),
                                     ('HBR1', "BOX 9:"),
                                     ('HBC1', str(EC_input_purchases)) )
        else:
            report.add_row_to_table( ('HBR5', "BOX 9:"),
                                     ('HBC1', str(EC_input_purchases)) )
    finally:
        if conn:
            conn.close()

    previous_vatreturn, previous_vatscheme, previous_method = get_previous_vat_return_scheme(vat_return)

    if previous_vatscheme:
        if previous_method != "CASH":
            # There has been a change of scheme
            report.add_row_to_table( ('HBR6', "----------------------") )
            report.add_row_to_table( ('HBL6', "Note: change of VAT scheme may require additional payments") )

    return report



