####### VATStuff #######
#
# vat_flat_basic.py  - generate VAT reports/returns for the Flat (basic) method
#
# Version : 0.7.0
# Date : 20100914
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#

"""Generate VAT reports and VAT returns for the Flat (basic) method

contains:
   flat_basic_report() - returns a ReportForm object for the Flat (basic) method
   get_box_1() etc - for each box of a VAT return, gives the decimal result
   flat_basic_detail() - returns a detailed ReportForm object for the Flat (basic) method
"""

import decimal

from vatstuff_packages.vs_utilities import *

from vatstuff_packages.vs_verify.form_objects import ReportForm

D = decimal.Decimal   # D used to avoid typing


def flat_basic_report(from_date, to_date, vat_return, vat_scheme):
    "Create a FLAT-BASIC report returns a ReportForm object"

    # get the standard rate at the time of the vat return, or if
    # this is for the current period, get the current standard rate
    if vat_return:
        standard_rate = vat_return.standard_rate
    else:
        standard_rate = vs_cfg["VAT_STANDARD_RATE"]

    # get the flat rate percentage from the applicable vat_scheme
    percent = vat_scheme.flat_percent

    report = ReportForm("VAT report")

    report.header = ["For the period:",
                     "From: %s" % from_date.strftime("%d %b %Y"),
                     "To: %s" % to_date.strftime("%d %b %Y"),
                     "",
                     "Method: Flat (Basic), at a flat rate of %s%%" % str(percent).replace(".", vs_cfg["CP"])]

    if vs_cfg["DELETE_TO_DATE"]:
        if from_date < vs_cfg["DELETE_TO_DATE"]:
            report.header.append("!!WARNING!! - this report may be innaccurate")
            report.header.append(
            "Data prior to %s has been deleted from the database" % vs_cfg["DELETE_TO_DATE"].strftime("%d %b %Y"))

    # tables are described by a list of list,
    # each list being a row, consisting of cell tuple
    # each cell tuple containing two strings; 
    # The first string is formed from "XXXN",
    # and the second string is the text to be placed in the cell

    # N is the collspan number, as used in a table cell tag attribute
    # such as 'colspan="4"' - in which case N is 4

    # so normally N is '1'

    # XXX is a formatting string :
    # HBL  - Header Bold Left Justified (R for Right instead of L
    # HNL  - Header Normal Left Justified  etc
    # DNL  - Data Cell Normal Left Justified  etc
    # XXC  - Right Justified currency, as pulled from the database

    # Set the running totals
    output_COST = D("0.00")
    output_COST_correction = D("0.00")
    output_VAT = D("0.00")
    output_VAT_correction = D("0.00")
    output_sales = D("0.00")
    EC_purchase = D("0.00")
    EC_purchase_correction = D("0.00")

    conn=None
    try:
        # Make database connection
        conn, c = vs_connect()

        ############## Output VAT

        # Invoices to customer headings
        report.table = [ [('HNL5', 'Sales from invoices to customers')],
                         [('HBL1', 'Tax Point Date'),
                          ('HBL1', 'Customer'),
                          ('HBL1', 'Invoice number'),
                          ('HBR1', 'Net Cost'),
                          ('HBR1', 'VAT')] ]

        # Get the invoices from the database
        c.execute("""SELECT
                        DATE_OF_TAXPOINT,
                        NAME,
                        INVOICE_NUMBER,
                        COST,
                        VAT
                      FROM INVOICES, CUSTOMERS
                     WHERE DATE_OF_TAXPOINT BETWEEN ? AND ?
                       AND INVOICES.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID
                     ORDER BY DATE_OF_TAXPOINT""", (from_date, to_date))

        for info in c:
            # Fill table with invoices to customers data

            report.add_row_to_table( ('DNL1', info[0].strftime("%d %b %Y")),
                                     ('DNL1', info[1]),
                                     ('DNL1', info[2]),
                                     ('DNC1', info[3]),
                                     ('DNC1', info[4]) )

            output_COST += D(info[3])
            output_VAT  += D(info[4])

        report.add_row_to_table( ('DNR3', 'Totals:'),
                                 ('DNC1', str(output_COST)),
                                 ('DNC1', str(output_VAT)) )

        output_sales = output_COST + output_VAT

        report.add_row_to_table( ('HBR4', "Total sales (including VAT) from issued invoices:"),
                                 ('HBC1', str(output_sales)) )


        # Now list creditnotes issued by you - Output VAT corrections 

        # Check if there are any corrections from issued creditnotes
        c.execute("""SELECT COUNT(*) FROM CREDITNOTES, TRANSACTIONS
                     WHERE CREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                       AND DATESTAMP BETWEEN ? AND ?""", (from_date, to_date))
        result = c.fetchone()

        if result[0]:
            report.add_row_to_table( ('HNL5', 'Correction from credit notes issued') )

            report.add_row_to_table( ('HBL1', 'Date'),
                                     ('HBL1', 'Customer'),
                                     ('HBL1', 'Credit Note'),
                                     ('HBR1', 'Net Cost'),
                                     ('HBR1', 'VAT') )

            c.execute("""SELECT DATESTAMP,
                                CUSTOMERS.NAME,
                                CREDITNOTE_NUMBER,
                                COST,
                                VAT
                           FROM CREDITNOTES, CUSTOMERS, TRANSACTIONS
                          WHERE DATESTAMP BETWEEN ? AND ?
                            AND CREDITNOTES.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID
                            AND CREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                       ORDER BY DATESTAMP""", (from_date, to_date))

            for info in c:
                report.add_row_to_table( ('DNL1', info[0].strftime("%d %b %Y")),
                                         ('DNL1', info[1]),
                                         ('DNL1', info[2]),
                                         ('DNC1', info[3]),
                                         ('DNC1', info[4]) )


                output_COST_correction += D(info[3])
                output_VAT_correction += D(info[4])

            report.add_row_to_table( ('DNR3', 'Totals:'),
                                     ('DNC1', str(output_COST_correction)),
                                     ('DNC1', str(output_VAT_correction)) )


            report.add_row_to_table( ('HBR4', "Correction from issued credit notes:"),
                                     ('HBC1', str(output_VAT_correction+output_COST_correction)) )

            output_sales = output_sales-output_VAT_correction-output_COST_correction

            report.add_row_to_table( ('HBR4', "Total sales (VAT inclusive):"),
                                     ('HBC1', str(output_sales)) )

        # VAT is the percentage of total sales
        result = (output_sales * percent) / D("100.0000")
        flat_vat = result.quantize(D('.01'), rounding=decimal.ROUND_HALF_EVEN)

        # And total VAT owed as a percentage of sales
        report.add_row_to_table( ('HBR4', "VAT owed is %s%% of sales:" % str(percent).replace(".", vs_cfg["CP"])),
                                 ('HBC1', str(flat_vat)) )


        # If there are any aquisitions from EC member states, then VAT has to be paid on these purchases
        c.execute("""SELECT COUNT(*) FROM SUPPLIERINVOICES, SUPPLIERS, TRANSACTIONS
                       WHERE SUPPLIERINVOICES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                       AND DATESTAMP BETWEEN ? AND ?
                       AND EC_COUNTRY = 1
                       AND VAT_COUNTRY_CODE <> ?
                       AND SUPPLIERINVOICES.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID
                   """, (from_date, to_date, vs_cfg["VAT_COUNTRY_CODE"]))

        result = c.fetchone()

        if result[0]:
            report.add_row_to_table( ('HNL5', 'Aquisitions from EC member states') )

            report.add_row_to_table( ('HBL1', 'Date'),
                                     ('HBL1', 'Supplier'),
                                     ('HBL1', 'Country code'),
                                     ('HBL1', 'Supplier Invoice'),
                                     ('HBR1', 'Net Cost') )

            c.execute("""SELECT DATESTAMP,
                                SUPPLIERS.NAME,
                                VAT_COUNTRY_CODE,
                                SUPPLIERINVOICES.NUMBER,
                                COST
                           FROM SUPPLIERINVOICES, SUPPLIERS, TRANSACTIONS
                          WHERE SUPPLIERINVOICES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                            AND DATESTAMP BETWEEN ? AND ?
                            AND EC_COUNTRY = 1
                            AND VAT_COUNTRY_CODE <> ?
                            AND SUPPLIERINVOICES.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID
                     ORDER BY DATESTAMP""", (from_date, to_date, vs_cfg["VAT_COUNTRY_CODE"]))

            for info in c:
                report.add_row_to_table( ('DNL1', info[0].strftime("%d %b %Y")),
                                         ('DNL1', info[1]),
                                         ('DNL1', info[2]),
                                         ('DNL1', info[3]),
                                         ('DNC1', info[4]) )

            EC_purchase += D(info[4])

            report.add_row_to_table( ('HBR4', "EC Purchases:"),
                                     ('HBC1', str(EC_purchase)) )


        # Now list creditnotes received from EC suppliers
        c.execute("""SELECT COUNT(*) FROM SUPPLIERCREDITNOTES, SUPPLIERS, TRANSACTIONS
                      WHERE SUPPLIERCREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                       AND DATESTAMP BETWEEN ? AND ?
                       AND EC_COUNTRY = 1
                       AND VAT_COUNTRY_CODE <> ?
                       AND SUPPLIERCREDITNOTES.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID""",
                      (from_date, to_date, vs_cfg["VAT_COUNTRY_CODE"]))

        result = c.fetchone()

        if result[0]:
            report.add_row_to_table( ('HNL5', 'Correction from EC credit notes received') )

            report.add_row_to_table( ('HBL1', 'Date'),
                                     ('HBL1', 'Supplier'),
                                     ('HBL1', 'Country code'),
                                     ('HBL1', 'Supplier Credit Note'),
                                     ('HBR1', 'Net Cost') )

            c.execute("""SELECT DATESTAMP,
                                SUPPLIERS.NAME,
                                VAT_COUNTRY_CODE,
                                SUPPLIERCREDITNOTES.NUMBER,
                                COST
                           FROM SUPPLIERCREDITNOTES, SUPPLIERS, TRANSACTIONS
                          WHERE SUPPLIERCREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                            AND DATESTAMP BETWEEN ? AND ?
                            AND EC_COUNTRY = 1
                            AND VAT_COUNTRY_CODE <> ?
                            AND SUPPLIERCREDITNOTES.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID
                     ORDER BY DATESTAMP""", (from_date, to_date, vs_cfg["VAT_COUNTRY_CODE"]))

            for info in c:
                report.add_row_to_table( ('DNL1', info[0].strftime("%d %b %Y")),
                                         ('DNL1', info[1]),
                                         ('DNL1', info[2]),
                                         ('DNL1', info[3]),
                                         ('DNC1', info[4]) )

                EC_purchase_correction += D(info[4])

            report.add_row_to_table( ('HBR4', "Correction from received credit notes:"),
                                     ('HBC1', str(EC_purchase_correction)) )

            EC_purchase = EC_purchase-EC_purchase_correction

            report.add_row_to_table( ('HBR4', "Total EC Purchases:"),
                                     ('HBC1', str(EC_purchase)) )

    finally:
        if conn:
            conn.close()

    if EC_purchase:
        result = (EC_purchase * standard_rate) / D("100.0000")
        EC_purchases_VAT = result.quantize(D('.01'), rounding=decimal.ROUND_HALF_EVEN)

        report.add_row_to_table(
         ('HBR4', "VAT owed is %s%% of EC purchases:" % str(standard_rate).replace(".", vs_cfg["CP"])),
         ('HBC1', str(EC_purchases_VAT)) )

        report.add_row_to_table( ('HBR4', "TOTAL VAT OWED:"), ('HBC1', str(EC_purchases_VAT+flat_vat)) )   


    previous_vatreturn, previous_vatscheme, previous_method = get_previous_vat_return_scheme(vat_return)

    if previous_vatscheme:
        if previous_method != "FLAT-BASIC":
            # There has been a change of scheme
            report.add_row_to_table( ('HBR5', "----------------------") )
            report.add_row_to_table( ('HBL5', "Note: change of VAT scheme may require additional payments") )

    return report


############# FLAT-BASIC RATE VAT RETURN ####################################

def get_box_1(from_date, to_date):
    """Form text : VAT due in this period on sales and other outputs.

    My interpretation : The flat rate percentage of the gross (vat inclusive)
    sales from invoices, as given as the value in box 6"""

    vat_scheme = VATScheme()
    percent = vat_scheme.flat_percent
    gross_sales = get_box_6(from_date, to_date)

    result = (gross_sales * percent) / D("100.0000")
    output_vat = result.quantize(D('.01'), rounding=decimal.ROUND_HALF_EVEN)
    return output_vat


def get_box_2(from_date, to_date):
    """Form text : VAT due in this period on acquisitions from other EC member States

    My interpretation : The value from Box 9 is taken, and the standard rate of VAT
    is applied.  (That is - the vat rate with symbol S set in the vat rates table).
    The subsequent result is set in Box 2."""

    EC_cost = get_box_9(from_date, to_date)
    if EC_cost == D(0):
        return D("0.00")
    result = (EC_cost * vs_cfg["VAT_STANDARD_RATE"]) / D("100.0000")
    EC_purchases_VAT = result.quantize(D('.01'), rounding=decimal.ROUND_HALF_EVEN)
    return EC_purchases_VAT


# Box 3 - Form text : Total VAT due (the sum of boxes 1 and 2)


def get_box_4(from_date, to_date):
    """Form text : VAT reclaimed in this period on purchase and other inputs
                   (including acquisitions from the EC)

    My interpretation : Zero - no VAT is reclaimed.
    In fact VAT can be reclaimed for purchases over
    2000.00, but that is not dealt with yet"""

    return D("0.00")


# Box 5 - Form text : Net VAT to be paid to HMRC or reclaimed by you
#                     (Difference between boxes 3 and 4)


def get_box_6(from_date, to_date):
    """Form text : Total value of sales and all other outputs excluding
                   any VAT. Include your box 8 figure

    My interpretation : The gross cost of all invoices, including VAT.  The result is the value in Box 6."""

    output_cost = D("0.00")

    conn = None
    try:
        conn, c = vs_connect()

        # Get cost from invoices
        c.execute("""SELECT COST, VAT
                       FROM INVOICES
                      WHERE DATE_OF_TAXPOINT BETWEEN ? AND ?""", (from_date, to_date))

        for result in c:
            output_cost += (D(result[0]) + D(result[1]))

        # Subtract cost from any creditnotes
        c.execute("""SELECT COST, VAT
                      FROM CREDITNOTES, TRANSACTIONS
                     WHERE DATESTAMP BETWEEN ? AND ?
                       AND CREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID""",
                      (from_date, to_date))

        for result in c:
            output_cost -= (D(result[0]) + D(result[1]))

    finally:
        if conn:
            conn.close()
    return output_cost


def get_box_7(from_date, to_date):
    """Form text : Total value of purchases and all other inputs excluding
                   any VAT. Include your box 9 figure

    My interpretation : Zero"""

    return D("0.00")


def get_box_8(from_date, to_date):
    """Form text : Total value of all supplies of goods and related costs,
                   excluding any VAT, to other EC Member States

    My interpretation : The net cost (amount without VAT) from all invoices issued
    to EC customers with taxpoint dates within the period, with the costs (again without VAT)
    from credit notes issued to EC customers subtracted. An 'EC customer' being a customer
    who is in the EC, who does not have GB set in their country code, and who is VAT Registered"""

    output_cost = D("0.00")

    conn = None
    try:
        conn, c = vs_connect()

        # Get Cost from invoices to EC registered customers
        c.execute("""SELECT COST
                      FROM INVOICES, CUSTOMERS
                     WHERE DATE_OF_TAXPOINT BETWEEN ? AND ?
                       AND EC_COUNTRY = 1
                       AND VAT_COUNTRY_CODE <> ?
                       AND VAT_REGISTERED = 1
                       AND INVOICES.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID""",
                      (from_date, to_date, vs_cfg["VAT_COUNTRY_CODE"]))

        for result in c:
            output_cost += D(result[0])

        # Subtract any creditnotes
        c.execute("""SELECT COST
                      FROM CREDITNOTES, CUSTOMERS, TRANSACTIONS
                     WHERE DATESTAMP BETWEEN ? AND ?
                       AND EC_COUNTRY = 1
                       AND VAT_COUNTRY_CODE <> ?
                       AND VAT_REGISTERED = 1
                       AND CREDITNOTES.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID
                       AND CREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID""",
                      (from_date, to_date, vs_cfg["VAT_COUNTRY_CODE"]))

        for result in c:
            output_cost -= D(result[0])

    finally:
        if conn:
            conn.close()
    return output_cost


def get_box_9(from_date, to_date):
    """Form text : Total value of all acquisitions of goods and related costs,
                   excluding any VAT, from other EC Member States.

    My interpretation : The net cost (amount without VAT) from all EC supplier
    invoices received in the period, with the costs (again without VAT) from
    EC suppliers credit notes subtracted. An 'EC supplier' being a supplier
    who is in the EC, who does not have GB set in their country code."""

    input_cost = D("0.00")

    conn = None
    try:
        conn, c = vs_connect()

        # Get Cost from supplier invoices from EC suppliers
        c.execute("""SELECT COST
                      FROM SUPPLIERINVOICES, SUPPLIERS, TRANSACTIONS
                     WHERE DATESTAMP BETWEEN ? AND ?
                       AND EC_COUNTRY = 1
                       AND VAT_COUNTRY_CODE <> ?
                       AND SUPPLIERINVOICES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                       AND SUPPLIERINVOICES.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID""",
                      (from_date, to_date, vs_cfg["VAT_COUNTRY_CODE"]))

        for result in c:
            input_cost += D(result[0])

        # Subtract any suppliercreditnotes
        c.execute("""SELECT COST
                      FROM SUPPLIERCREDITNOTES, SUPPLIERS, TRANSACTIONS
                     WHERE DATESTAMP BETWEEN ? AND ?
                       AND EC_COUNTRY = 1
                       AND VAT_COUNTRY_CODE <> ?
                       AND SUPPLIERCREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                       AND SUPPLIERCREDITNOTES.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID""",
                      (from_date, to_date, vs_cfg["VAT_COUNTRY_CODE"]))

        for result in c:
            input_cost -= D(result[0])

    finally:
        if conn:
            conn.close()
    return input_cost


################### the detail report ############################

def flat_basic_detail(from_date, to_date, vat_return, vat_scheme):
    """Create a detailed report for the Flat (basic) vat method
       returns a ReportForm object"""

    report = ReportForm("VAT detail report")

    # get the flat rate percentage from the applicable vat_scheme
    percent = vat_scheme.flat_percent

    report.header = ["For the period:",
                     "From: %s" % from_date.strftime("%d %b %Y"),
                     "To: %s" % to_date.strftime("%d %b %Y"),
                     "",
                     "Method: Flat (Basic), at a flat rate of %s%%" % str(percent).replace(".", vs_cfg["CP"])]

    if vs_cfg["DELETE_TO_DATE"]:
        if from_date < vs_cfg["DELETE_TO_DATE"]:
            report.header.append("!!WARNING!! - this report may be innaccurate")
            report.header.append(
            "Data prior to %s has been deleted from the database" % vs_cfg["DELETE_TO_DATE"].strftime("%d %b %Y"))

    # tables are described by a list of list,
    # each list being a row, consisting of cell tuple
    # each cell tuple containing two strings; 
    # The first string is formed from "XXXN",
    # and the second string is the text to be placed in the cell

    # N is the collspan number, as used in a table cell tag attribute
    # such as 'colspan="4"' - in which case N is 4

    # so normally N is '1'

    # XXX is a formatting string :
    # HBL  - Header Bold Left Justified (R for Right instead of L
    # HNL  - Header Normal Left Justified  etc
    # DNL  - Data Cell Normal Left Justified  etc
    # XXC  - Right Justified currency, as pulled from the database

    # Set the running totals
    output_VAT = D("0.00")
    output_VAT_correction = D("0.00")
    output_COST = D("0.00")
    output_COST_correction = D("0.00")
    EC_output_sales = D("0.00")
    EC_output_sales_correction = D("0.00")
    EC_input_purchases = D("0.00")
    EC_input_purchases_correction = D("0.00")

    conn=None
    try:
        # Make database connection
        conn, c = vs_connect()

        ##### BOX 1
        # Invoices to customer headings
        report.table = [ [('HBL5', 'BOX 1 --------')],
                         [('HNL5', 'Sales from invoices to customers')],
                         [('HBL1', 'Tax Point Date'),
                          ('HBL1', 'Customer'),
                          ('HBL1', 'Invoice number'),
                          ('HBR1', 'Net Cost'),
                          ('HBR1', 'VAT')] ]

        # Get the invoices from the database
        c.execute("""SELECT
                        DATE_OF_TAXPOINT,
                        NAME,
                        INVOICE_NUMBER,
                        COST,
                        VAT
                      FROM INVOICES, CUSTOMERS
                     WHERE DATE_OF_TAXPOINT BETWEEN ? AND ?
                       AND INVOICES.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID
                     ORDER BY DATE_OF_TAXPOINT""", (from_date, to_date))

        for info in c:
            # Fill table with invoices to customers data

            report.add_row_to_table( ('DNL1', info[0].strftime("%d %b %Y")),
                                     ('DNL1', info[1]),
                                     ('DNL1', info[2]),
                                     ('DNC1', info[3]),
                                     ('DNC1', info[4]) )

            output_COST += D(info[3])
            output_VAT  += D(info[4])

        report.add_row_to_table( ('DNR3', "Totals:"),
                                 ('DNC1', str(output_COST)),
                                 ('DNC1', str(output_VAT)) )

        output_sales = output_COST + output_VAT


        # Now list creditnotes issued by you - Output VAT corrections 

        # Check if there are any corrections from issued creditnotes
        c.execute("""SELECT COUNT(*) FROM CREDITNOTES, TRANSACTIONS
                     WHERE CREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                       AND DATESTAMP BETWEEN ? AND ?""", (from_date, to_date))
        result = c.fetchone()

        if result[0]:
            report.add_row_to_table( ('HNL5', 'Correction from credit notes issued') )

            report.add_row_to_table( ('HBL1', 'Date'),
                                     ('HBL1', 'Customer'),
                                     ('HBL1', 'Credit Note'),
                                     ('HBR1', 'Net Cost'),
                                     ('HBR1', 'VAT') )

            c.execute("""SELECT DATESTAMP,
                                CUSTOMERS.NAME,
                                CREDITNOTE_NUMBER,
                                COST,
                                VAT
                           FROM CREDITNOTES, CUSTOMERS, TRANSACTIONS
                          WHERE DATESTAMP BETWEEN ? AND ?
                            AND CREDITNOTES.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID
                            AND CREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                       ORDER BY DATESTAMP""", (from_date, to_date))

            for info in c:
                report.add_row_to_table( ('DNL1', info[0].strftime("%d %b %Y")),
                                         ('DNL1', info[1]),
                                         ('DNL1', info[2]),
                                         ('DNC1', info[3]),
                                         ('DNC1', info[4]) )


                output_COST_correction += D(info[3])
                output_VAT_correction += D(info[4])

            report.add_row_to_table( ('DNR3', "Totals:"),
                                     ('DNC1', str(output_COST_correction)),
                                     ('DNC1', str(output_VAT_correction)) )

            output_sales = output_sales-output_VAT_correction-output_COST_correction

        report.add_row_to_table( ('HBR4', "Total sales (VAT inclusive):"),
                                 ('HBC1', str(output_sales)) )

        # VAT is the percentage of total sales
        result = (output_sales * percent) / D("100.0000")
        flat_vat = result.quantize(D('.01'), rounding=decimal.ROUND_HALF_EVEN)

        # And total VAT owed as a percentage of sales
        report.add_row_to_table( ('HBL3', "VAT owed is %s%% of sales:" % str(percent).replace(".", vs_cfg["CP"])),
                                 ('HBR1', "BOX 1:"),
                                 ('HBC1', str(flat_vat)) )

        ##### BOX 2
        report.add_row_to_table( ('HBL5', "BOX 2 --------") )
        if vat_return:
            # get the stored values
            EC_cost = vat_return.box_nine
            standard_rate = vat_return.standard_rate
            box_2 = vat_return.box_two
        else:
            # calculate using current values
            EC_cost = get_box_9(from_date, to_date)
            standard_rate = vs_cfg["VAT_STANDARD_RATE"]
            if EC_cost == D(0):
                box_2 = D("0.00")
            else:
                result = (EC_cost * standard_rate) / D("100.0000")
                box_2 = result.quantize(D('.01'), rounding=decimal.ROUND_HALF_EVEN)

        report.add_row_to_table( ('DNR4', "Value from BOX 9:"), ('DNC1', str(EC_cost)) )
        report.add_row_to_table( ('DNR4', "At the standard VAT rate of:"), ('DNR1', str(standard_rate).replace(".", vs_cfg["CP"])) )
        report.add_row_to_table( ('HBR4', "BOX 2:"), ('HBC1', str(box_2)) )


        ##### BOX 3
        report.add_row_to_table( ('HBL5', "BOX 3 --------") )
        report.add_row_to_table( ('HBL3', "Total VAT due (the sum of boxes 1 and 2)"),
                                 ('HBR1', "BOX 3:"),
                                 ('HBC1', str(box_2 + flat_vat)) )

        ##### BOX 4
        report.add_row_to_table( ('HBL5', "BOX 4 --------") )

        report.add_row_to_table( ('HBL3', "No VAT is reclaimed"),
                                 ('HBR1', "BOX 4:"),
                                 ('HBC1', "0.00") )

        ##### BOX 5
        report.add_row_to_table( ('HBL5', "BOX 5 --------") )
        report.add_row_to_table( ('HBL3', "Net VAT (Difference between boxes 3 and 4)"),
                                 ('HBR1', "BOX 5:"),
                                 ('HBC1', str(box_2 + flat_vat)) )

        ##### BOX 6
        report.add_row_to_table( ('HBL5', "BOX 6 --------") )
        report.add_row_to_table( ('HBL3', "Total sales (VAT inclusive) as calculated above"),
                                 ('HBR1', "BOX 6:"),
                                 ('HBC1', str(output_sales)) )

        ##### BOX 7
        report.add_row_to_table( ('HBL5', "BOX 7 --------") )
        report.add_row_to_table( ('HBL3', "Value of purchases is not applicable to the FLAT method"),
                                 ('HBR1', "BOX 7:"),
                                 ('HBC1', "0.00") )

        ##### BOX 8
        report.add_row_to_table( ('HBL5', "BOX 8 --------") )
        report.add_row_to_table( ('HNL5', 'Invoices to VAT registered companies in other EC Member States') )
        report.add_row_to_table( ('HBL1', 'Tax Point Date'),
                                 ('HBL1', 'Customer'),
                                 ('HBL1', 'Country code'),
                                 ('HBL1', 'Invoice number'),
                                 ('HBR1', 'Net Cost') )

        c.execute("""SELECT
                        DATE_OF_TAXPOINT,
                        NAME,
                        VAT_COUNTRY_CODE,
                        INVOICE_NUMBER,
                        COST
                   FROM INVOICES, CUSTOMERS
                  WHERE DATE_OF_TAXPOINT BETWEEN ? AND ?
                    AND EC_COUNTRY = 1
                    AND VAT_COUNTRY_CODE <> ?
                    AND VAT_REGISTERED = 1
                    AND INVOICES.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID""",
                      (from_date, to_date, vs_cfg["VAT_COUNTRY_CODE"]))

        for info in c:
            # Invoices to customers in other EC Member States

            report.add_row_to_table( ('DNL1', info[0].strftime("%d %b %Y")),
                                     ('DNL1', info[1]),
                                     ('DNL1', info[2]),
                                     ('DNL1', info[3]),
                                     ('DNC1', info[4]) )

            EC_output_sales +=D(info[4])

        report.add_row_to_table( ('DNR4', "Total:"),
                                 ('DNC1', str(EC_output_sales)) )

        # Now list creditnotes issued by you to customers in other EC Member States

        # Check if there are any issued creditnotes
        c.execute("""SELECT COUNT(*) FROM CREDITNOTES, CUSTOMERS, TRANSACTIONS
                     WHERE DATESTAMP BETWEEN ? AND ?
                       AND EC_COUNTRY = 1
                       AND VAT_COUNTRY_CODE <> ?
                       AND VAT_REGISTERED = 1
                       AND CREDITNOTES.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID
                       AND CREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID""",
                      (from_date, to_date, vs_cfg["VAT_COUNTRY_CODE"]))
        result = c.fetchone()

        if result[0]:
            report.add_row_to_table( ('HNL5', 'Correction from credit notes issued') )

            report.add_row_to_table( ('HBL1', 'Date'),
                                     ('HBL1', 'Customer'),
                                     ('HBL1', 'Country code'),
                                     ('HBL1', 'Credit Note'),
                                     ('HBR1', 'Net Cost') )

            c.execute("""SELECT DATESTAMP,
                                CUSTOMERS.NAME,
                                VAT_COUNTRY_CODE,
                                CREDITNOTE_NUMBER,
                                COST
                           FROM CREDITNOTES, CUSTOMERS, TRANSACTIONS
                          WHERE DATESTAMP BETWEEN ? AND ?
                            AND EC_COUNTRY = 1
                            AND VAT_COUNTRY_CODE <> ?
                            AND VAT_REGISTERED = 1
                            AND CREDITNOTES.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID
                            AND CREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID""",
                      (from_date, to_date, vs_cfg["VAT_COUNTRY_CODE"]))

            for info in c:
                report.add_row_to_table( ('DNL1', info[0].strftime("%d %b %Y")),
                                         ('DNL1', info[1]),
                                         ('DNL1', info[2]),
                                         ('DNL1', info[3]),
                                         ('DNC1', info[4]) )

                EC_output_sales_correction += D(info[4])

            report.add_row_to_table( ('DNR4', "Total:"),
                                     ('DNC1', str(EC_output_sales_correction)) )

        if EC_output_sales_correction:
            EC_output_sales = EC_output_sales - EC_output_sales_correction
            report.add_row_to_table( ('HBL3', "Sales corrected from credit notes issued"),
                                     ('HBR1', "BOX 8:"),
                                     ('HBC1', str(EC_output_sales)) )
        else:
            report.add_row_to_table( ('HBR4', "BOX 8:"),
                                     ('HBC1', str(EC_output_sales)) )


        ##### BOX 9
        report.add_row_to_table( ('HBL5', "BOX 9 --------") )
        report.add_row_to_table( ('HNL5', 'Invoices received from companies in other EC Member States') )
        report.add_row_to_table( ('HBL1', 'Date'),
                                 ('HBL1', 'Supplier'),
                                 ('HBL1', 'Country code'),
                                 ('HBL1', 'Supplier Invoice'),
                                 ('HBR1', 'Net Cost') )

        c.execute("""SELECT DATESTAMP,
                            SUPPLIERS.NAME,
                            VAT_COUNTRY_CODE,
                            SUPPLIERINVOICES.NUMBER,
                            COST
                       FROM SUPPLIERINVOICES, SUPPLIERS, TRANSACTIONS
                      WHERE DATESTAMP BETWEEN ? AND ?
                        AND EC_COUNTRY = 1
                        AND VAT_COUNTRY_CODE <> ?
                        AND SUPPLIERINVOICES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                        AND SUPPLIERINVOICES.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID""",
                            (from_date, to_date, vs_cfg["VAT_COUNTRY_CODE"]))
        for info in c:
            # Invoices from suppliers in other EC Member States

            report.add_row_to_table( ('DNL1', info[0].strftime("%d %b %Y")),
                                     ('DNL1', info[1]),
                                     ('DNL1', info[2]),
                                     ('DNL1', info[3]),
                                     ('DNC1', info[4]) )

            EC_input_purchases +=D(info[4])

        report.add_row_to_table( ('DNR4', "Total:"),
                                 ('DNC1', str(EC_input_purchases)) )

        # Now list creditnotes received from companies in other Member States

        # Check if there are any received creditnotes
        c.execute("""SELECT COUNT(*) FROM SUPPLIERCREDITNOTES, SUPPLIERS, TRANSACTIONS
                     WHERE DATESTAMP BETWEEN ? AND ?
                       AND EC_COUNTRY = 1
                       AND VAT_COUNTRY_CODE <> ?
                       AND SUPPLIERCREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                       AND SUPPLIERCREDITNOTES.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID""",
                      (from_date, to_date, vs_cfg["VAT_COUNTRY_CODE"]))
        result = c.fetchone()

        if result[0]:
            report.add_row_to_table( ('HNL5', 'Correction from credit notes received') )

            report.add_row_to_table( ('HBL1', 'Date'),
                                     ('HBL1', 'Supplier'),
                                     ('HBL1', 'Country code'),
                                     ('HBL1', 'Supplier Credit Note'),
                                     ('HBR1', 'Net Cost') )

            c.execute("""SELECT DATESTAMP,
                                SUPPLIERS.NAME,
                                VAT_COUNTRY_CODE,
                                SUPPLIERCREDITNOTES.NUMBER,
                                COST
                           FROM SUPPLIERCREDITNOTES, SUPPLIERS, TRANSACTIONS
                          WHERE DATESTAMP BETWEEN ? AND ?
                            AND EC_COUNTRY = 1
                            AND VAT_COUNTRY_CODE <> ?
                            AND SUPPLIERCREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                            AND SUPPLIERCREDITNOTES.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID""",
                      (from_date, to_date, vs_cfg["VAT_COUNTRY_CODE"]))

            for info in c:
                report.add_row_to_table( ('DNL1', info[0].strftime("%d %b %Y")),
                                         ('DNL1', info[1]),
                                         ('DNL1', info[2]),
                                         ('DNL1', info[3]),
                                         ('DNC1', info[4]) )

                EC_input_purchases_correction += D(info[4])

            report.add_row_to_table( ('DNR4', "Total:"),
                                     ('DNC1', str(EC_input_purchases_correction)) )

        if EC_input_purchases_correction:
            EC_input_purchases = EC_input_purchases - EC_input_purchases_correction
            report.add_row_to_table( ('HBL3', "Purchases corrected from credit notes received"),
                                     ('HBR1', "BOX 9:"),
                                     ('HBC1', str(EC_input_purchases)) )
        else:
            report.add_row_to_table( ('HBR4', "BOX 9:"),
                                     ('HBC1', str(EC_input_purchases)) )

    finally:
        if conn:
            conn.close()

    previous_vatreturn, previous_vatscheme, previous_method = get_previous_vat_return_scheme(vat_return)

    if previous_vatscheme:
        if previous_method != "STANDARD":
            # There has been a change of scheme
            report.add_row_to_table( ('HBR5', "----------------------") )
            report.add_row_to_table( ('HBL5', "Note: change of VAT scheme may require additional payments") )

    return report


