####### VATStuff #######
#
# vat_flat_cash.py  - generate VAT reports/returns for the Flat (Cash) method
#
# Version : 0.7.0
# Date : 20100914
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#

"""Generate VAT reports and VAT returns for the Flat (cash) method

contains:
   flat_cash_report() - returns a ReportForm object for the Flat (cash) method
   get_box_1() etc - for each box of a VAT return, gives the decimal result
   flat_cash_detail() - returns a detailed ReportForm object for the Flat (cash) method
"""


import decimal

from vatstuff_packages.vs_utilities import *

from vatstuff_packages.vs_verify.form_objects import ReportForm

D = decimal.Decimal   # D used to avoid typing



def flat_cash_report(from_date, to_date, vat_return, vat_scheme):
    """Create a basic report for the flat (cash) vat method
       returns a ReportForm object"""

    # get the standard rate at the time of the vat return, or if
    # this is for the current period, get the current standard rate
    if vat_return:
        standard_rate = vat_return.standard_rate
    else:
        standard_rate = vs_cfg["VAT_STANDARD_RATE"]

    # get the flat rate percentage from the applicable vat_scheme
    percent = vat_scheme.flat_percent

    report = ReportForm("VAT report")

    report.header = ["For the period:",
                     "From: %s" % from_date.strftime("%d %b %Y"),
                     "To: %s" % to_date.strftime("%d %b %Y"),
                     "",
                     "Method: Flat (Cash), at a flat rate of %s%%" % str(percent).replace(".", vs_cfg["CP"])]

    if vs_cfg["DELETE_TO_DATE"]:
        if from_date < vs_cfg["DELETE_TO_DATE"]:
            report.header.append("!!WARNING!! - this report may be innaccurate")
            report.header.append(
            "Data prior to %s has been deleted from the database" % vs_cfg["DELETE_TO_DATE"].strftime("%d %b %Y"))

    # tables are described by a list of list,
    # each list being a row, consisting of cell tuple
    # each cell tuple containing two strings; 
    # The first string is formed from "XXXN",
    # and the second string is the text to be placed in the cell

    # N is the collspan number, as used in a table cell tag attribute
    # such as 'colspan="4"' - in which case N is 4

    # so normally N is '1'

    # XXX is a formatting string :
    # HBL  - Header Bold Left Justified (R for Right instead of L
    # HNL  - Header Normal Left Justified  etc
    # DNL  - Data Cell Normal Left Justified  etc
    # XXC  - Right Justified currency, as pulled from the database

    # Set the running totals
    output_COST = D("0.00")
    output_COST_correction = D("0.00")
    output_VAT = D("0.00")
    output_VAT_correction = D("0.00")
    output_sales = D("0.00")
    EC_purchase = D("0.00")
    EC_purchase_correction = D("0.00")

    conn=None
    try:
        # Make database connection
        conn, c = vs_connect()

        # Payments from customers headings
        report.table = [[ ('HNL6', 'Output VAT from payments received from customers') ],
                        [ ('HBL1', 'Date'),
                          ('HBL1', 'Customer'),
                          ('HBL1', 'Invoice number'),
                          ('HBL1', 'Payment transaction'),
                          ('HBR1', 'Net Cost'),
                          ('HBR1', 'VAT') ]]

        # Get the payments from the database
        # UNION select needed as invoices may be NULL if a delete old data has occurred
        if vat_return:
            # This is for a previous vat return, in which a vat_return_id exists
            # so search for payments with this vat_return_id
            c.execute("""SELECT DATESTAMP,
                                NAME,
                                INVOICES.INVOICE_NUMBER,
                                CUSTOMERPAYMENTS.TRANSACTION_ID,
                                INVOICEPAYMENTS.COST,
                                INVOICEPAYMENTS.VAT
                           FROM INVOICEPAYMENTS, CUSTOMERPAYMENTS, TRANSACTIONS, CUSTOMERS, INVOICES
                          WHERE INVOICEPAYMENTS.VAT_RETURN_ID = ?
                            AND INVOICEPAYMENTS.CUSTOMERPAYMENT_ID = CUSTOMERPAYMENTS.CUSTOMERPAYMENT_ID
                            AND INVOICEPAYMENTS.INVOICE_ID = INVOICES.INVOICE_ID
                            AND CUSTOMERPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                            AND CUSTOMERPAYMENTS.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID
                       UNION
                         SELECT DATESTAMP,
                                NAME,
                                NULL,
                                CUSTOMERPAYMENTS.TRANSACTION_ID,
                                INVOICEPAYMENTS.COST,
                                INVOICEPAYMENTS.VAT
                           FROM INVOICEPAYMENTS, CUSTOMERPAYMENTS, TRANSACTIONS, CUSTOMERS
                          WHERE INVOICEPAYMENTS.VAT_RETURN_ID = ?
                            AND INVOICEPAYMENTS.CUSTOMERPAYMENT_ID = CUSTOMERPAYMENTS.CUSTOMERPAYMENT_ID
                            AND INVOICEPAYMENTS.INVOICE_ID IS NULL
                            AND CUSTOMERPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                            AND CUSTOMERPAYMENTS.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID

                       ORDER BY DATESTAMP""", (vat_return.id, vat_return.id))
        else:
            # This is for the current period, in which no vat_return_id exists
            # so search for payments within the dates, but with a null vat_return_id
            # so any payment which already has vat payed on it is not included
            # (if the previous scheme was standard for example)
            c.execute("""SELECT DATESTAMP,
                                NAME,
                                INVOICES.INVOICE_NUMBER,
                                CUSTOMERPAYMENTS.TRANSACTION_ID,
                                INVOICEPAYMENTS.COST,
                                INVOICEPAYMENTS.VAT
                           FROM INVOICEPAYMENTS, CUSTOMERPAYMENTS, TRANSACTIONS, CUSTOMERS, INVOICES
                          WHERE DATESTAMP BETWEEN ? AND ?
                            AND INVOICEPAYMENTS.VAT_RETURN_ID IS NULL
                            AND INVOICEPAYMENTS.CUSTOMERPAYMENT_ID = CUSTOMERPAYMENTS.CUSTOMERPAYMENT_ID
                            AND INVOICEPAYMENTS.INVOICE_ID = INVOICES.INVOICE_ID
                            AND CUSTOMERPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                            AND CUSTOMERPAYMENTS.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID
                       UNION
                         SELECT DATESTAMP,
                                NAME,
                                NULL,
                                CUSTOMERPAYMENTS.TRANSACTION_ID,
                                INVOICEPAYMENTS.COST,
                                INVOICEPAYMENTS.VAT
                           FROM INVOICEPAYMENTS, CUSTOMERPAYMENTS, TRANSACTIONS, CUSTOMERS
                          WHERE DATESTAMP BETWEEN ? AND ?
                            AND INVOICEPAYMENTS.VAT_RETURN_ID IS NULL
                            AND INVOICEPAYMENTS.CUSTOMERPAYMENT_ID = CUSTOMERPAYMENTS.CUSTOMERPAYMENT_ID
                            AND INVOICEPAYMENTS.INVOICE_ID IS NULL
                            AND CUSTOMERPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                            AND CUSTOMERPAYMENTS.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID
                       ORDER BY DATESTAMP""", (from_date, to_date, from_date, to_date))

        for info in c:
            # Fill table with payments from customers data

            # It looks as if the UNION function returns a unicode value of the datestamp, rather
            # than a datetime.date object
            if isinstance(info[0], datetime.date):
                datestamp = info[0].strftime("%d %b %Y")
            else:
                int_dates = map(int, info[0].split("-"))
                datestamp = datetime.date(*int_dates).strftime("%d %b %Y")

            # The invoice number could be None if no invoice exists
            # in which case display a '?' string 
            if info[2]:
                invoicenumber = str(info[2])
            else:
                invoicenumber = "?"

            report.add_row_to_table( ('DNL1', datestamp),
                                     ('DNL1', info[1]),
                                     ('DNL1', invoicenumber),
                                     ('DNL1', str(info[3])),
                                     ('DNC1', info[4]),
                                     ('DNC1', info[5]) )

            output_COST += D(info[4])
            output_VAT += D(info[5])

        report.add_row_to_table( ('DNR4', 'Totals:'),
                                 ('DNC1', str(output_COST)),
                                 ('DNC1', str(output_VAT)) )

        output_sales = output_VAT+output_COST

        report.add_row_to_table( ('HBR5', 'Payments received from customers :'),
                                 ('HBC1', str(output_sales)) )

        # Now list creditnotes with payments issued by you - Output VAT corrections 

        # Check if there are any VAT corrections from creditnotes issued with a payment
        c.execute("""SELECT COUNT(*) FROM CREDITNOTES, TRANSACTIONS
                     WHERE DATESTAMP BETWEEN ? AND ?
                       AND STATUS = 1
                       AND CREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID""",
                      (from_date, to_date))
        result = c.fetchone()

        if result[0]:
            report.add_row_to_table( ('HNL6', 'Output VAT correction from credit notes issued with a payment') )

            report.add_row_to_table( ('HBL1', 'Date'),
                                     ('HBL1', 'Customer'),
                                     ('HBL1', 'Credit Note'),
                                     ('HBL1', 'Payment transaction'),
                                     ('HBR1', 'Net Cost'),
                                     ('HBR1', 'VAT') )

            c.execute("""SELECT DATESTAMP,
                                CUSTOMERS.NAME,
                                CREDITNOTE_NUMBER,
                                CREDITNOTES.TRANSACTION_ID,
                                COST,
                                VAT
                           FROM CREDITNOTES, CUSTOMERS, TRANSACTIONS
                          WHERE DATESTAMP BETWEEN ? AND ?
                            AND STATUS = 1
                            AND CREDITNOTES.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID
                            AND CREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                       ORDER BY DATESTAMP""", (from_date, to_date))

            for info in c:
                report.add_row_to_table( ('DNL1', info[0].strftime("%d %b %Y")),
                                         ('DNL1', info[1]),
                                         ('DNL1', str(info[2])),
                                         ('DNL1', str(info[3])),
                                         ('DNC1', info[4]),
                                         ('DNC1', info[5]) )

                output_COST_correction += D(info[4])
                output_VAT_correction += D(info[5])

            report.add_row_to_table( ('DNR4', 'Totals:'),
                                     ('DNC1', str(output_COST_correction)),
                                     ('DNC1', str(output_VAT_correction)) )


            output_sales_correction = output_VAT_correction+output_COST_correction

            report.add_row_to_table( ('HBR5', "Correction from payments sent with credit notes:"),
                                     ('HBC1', str(output_sales_correction)) )

            output_sales = output_sales - output_sales_correction

            report.add_row_to_table( ('HBR5', "Total Sales:"),
                                     ('HBC1', str(output_sales)) )


        # VAT is the percentage of total sales
        result = (output_sales * percent) / D("100.0000")
        flat_vat = result.quantize(D('.01'), rounding=decimal.ROUND_HALF_EVEN)

        # And total VAT owed as a percentage of sales
        report.add_row_to_table( ('HBR5', "VAT owed is %s%% of sales:" % str(percent).replace(".", vs_cfg["CP"])),
                                 ('HBC1', str(flat_vat)) )


        # If there are any aquisitions from EC member states, then VAT has to be paid on these purchases
        if vat_return:
            # This is for a previous vat return, in which a vat_return_id exists
            # so search for payments with this vat_return_id
            c.execute("""SELECT COUNT(*)
                           FROM MAKEPAYMENTS, SUPPLIERINVOICEPAYMENTS, SUPPLIERS
                          WHERE SUPPLIERINVOICEPAYMENTS.VAT_RETURN_ID = ?
                            AND EC_COUNTRY = 1
                            AND VAT_COUNTRY_CODE <> ?
                            AND SUPPLIERINVOICEPAYMENTS.MAKEPAYMENT_ID = MAKEPAYMENTS.MAKEPAYMENT_ID
                            AND MAKEPAYMENTS.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID
                                   """, (vat_return.id, vs_cfg["VAT_COUNTRY_CODE"]))
        else:
            # This is for the current period, in which no vat_return_id exists
            # so search for payments made within the dates, but with a null vat_return_id
            # so any payment which already has vat payed on it is not included
            # (if the previous scheme was standard for example)
            c.execute("""SELECT COUNT(*)
                           FROM MAKEPAYMENTS, TRANSACTIONS, SUPPLIERINVOICEPAYMENTS, SUPPLIERS
                          WHERE DATESTAMP BETWEEN ? AND ?
                            AND EC_COUNTRY = 1
                            AND VAT_COUNTRY_CODE <> ?
                            AND SUPPLIERINVOICEPAYMENTS.VAT_RETURN_ID IS NULL
                            AND SUPPLIERINVOICEPAYMENTS.MAKEPAYMENT_ID = MAKEPAYMENTS.MAKEPAYMENT_ID
                            AND MAKEPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                            AND MAKEPAYMENTS.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID
                                  """, (from_date, to_date, vs_cfg["VAT_COUNTRY_CODE"]))

        result = c.fetchone()

        if result[0]:
            report.add_row_to_table( ('HNL6', 'Aquisitions from EC member states') )

            report.add_row_to_table( ('HBL1', 'Date'),
                                     ('HBL1', 'Supplier'),
                                     ('HBL1', 'Country code'),
                                     ('HBL1', 'Supplier Invoice'),
                                     ('HBL1', 'Payment transaction'),
                                     ('HBR1', 'Net Cost') )

            if vat_return:
                # This is for a previous vat return, in which a vat_return_id exists
                # so search for payments with this vat_return_id
                c.execute("""SELECT DATESTAMP,
                                SUPPLIERS.NAME,
                                VAT_COUNTRY_CODE,
                                SUPPLIERINVOICES.NUMBER,
                                MAKEPAYMENTS.TRANSACTION_ID,
                                SUPPLIERINVOICEPAYMENTS.COST
                           FROM MAKEPAYMENTS, TRANSACTIONS, SUPPLIERINVOICEPAYMENTS, SUPPLIERS, SUPPLIERINVOICES
                          WHERE SUPPLIERINVOICEPAYMENTS.VAT_RETURN_ID = ?
                            AND EC_COUNTRY = 1
                            AND VAT_COUNTRY_CODE <> ?
                            AND SUPPLIERINVOICEPAYMENTS.SUPPLIERINVOICE_ID = SUPPLIERINVOICES.SUPPLIERINVOICE_ID
                            AND SUPPLIERINVOICEPAYMENTS.MAKEPAYMENT_ID = MAKEPAYMENTS.MAKEPAYMENT_ID
                            AND MAKEPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                            AND MAKEPAYMENTS.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID
                       UNION
                         SELECT DATESTAMP,
                                SUPPLIERS.NAME,
                                VAT_COUNTRY_CODE,
                                NULL,
                                MAKEPAYMENTS.TRANSACTION_ID,
                                SUPPLIERINVOICEPAYMENTS.COST
                           FROM MAKEPAYMENTS, TRANSACTIONS, SUPPLIERINVOICEPAYMENTS, SUPPLIERS
                          WHERE SUPPLIERINVOICEPAYMENTS.VAT_RETURN_ID = ?
                            AND EC_COUNTRY = 1
                            AND VAT_COUNTRY_CODE <> ?
                            AND SUPPLIERINVOICEPAYMENTS.SUPPLIERINVOICE_ID IS NULL
                            AND SUPPLIERINVOICEPAYMENTS.MAKEPAYMENT_ID = MAKEPAYMENTS.MAKEPAYMENT_ID
                            AND MAKEPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                            AND MAKEPAYMENTS.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID

                       ORDER BY DATESTAMP """, (vat_return.id, vs_cfg["VAT_COUNTRY_CODE"], vat_return.id, vs_cfg["VAT_COUNTRY_CODE"]))
            else:
                # This is for the current period, in which no vat_return_id exists
                # so search for payments made within the dates, but with a null vat_return_id
                # so any payment which already has vat payed on it is not included
                # (if the previous scheme was standard for example)
                c.execute("""SELECT DATESTAMP,
                                SUPPLIERS.NAME,
                                VAT_COUNTRY_CODE,
                                SUPPLIERINVOICES.NUMBER,
                                MAKEPAYMENTS.TRANSACTION_ID,
                                SUPPLIERINVOICEPAYMENTS.COST
                           FROM MAKEPAYMENTS, TRANSACTIONS, SUPPLIERINVOICEPAYMENTS, SUPPLIERS, SUPPLIERINVOICES
                          WHERE DATESTAMP BETWEEN ? AND ?
                            AND EC_COUNTRY = 1
                            AND VAT_COUNTRY_CODE <> ?
                            AND SUPPLIERINVOICEPAYMENTS.VAT_RETURN_ID IS NULL
                            AND SUPPLIERINVOICEPAYMENTS.SUPPLIERINVOICE_ID = SUPPLIERINVOICES.SUPPLIERINVOICE_ID
                            AND SUPPLIERINVOICEPAYMENTS.MAKEPAYMENT_ID = MAKEPAYMENTS.MAKEPAYMENT_ID
                            AND MAKEPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                            AND MAKEPAYMENTS.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID
                       UNION
                         SELECT DATESTAMP,
                                SUPPLIERS.NAME,
                                VAT_COUNTRY_CODE,
                                NULL,
                                MAKEPAYMENTS.TRANSACTION_ID,
                                SUPPLIERINVOICEPAYMENTS.COST
                           FROM MAKEPAYMENTS, TRANSACTIONS, SUPPLIERINVOICEPAYMENTS, SUPPLIERS
                          WHERE DATESTAMP BETWEEN ? AND ?
                            AND EC_COUNTRY = 1
                            AND VAT_COUNTRY_CODE <> ?
                            AND SUPPLIERINVOICEPAYMENTS.VAT_RETURN_ID IS NULL
                            AND SUPPLIERINVOICEPAYMENTS.SUPPLIERINVOICE_ID IS NULL
                            AND SUPPLIERINVOICEPAYMENTS.MAKEPAYMENT_ID = MAKEPAYMENTS.MAKEPAYMENT_ID
                            AND MAKEPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                            AND MAKEPAYMENTS.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID

                       ORDER BY DATESTAMP    """, (from_date, to_date, vs_cfg["VAT_COUNTRY_CODE"],
                                                   from_date, to_date, vs_cfg["VAT_COUNTRY_CODE"]))


            for info in c:
                # It looks as if the UNION function returns a unicode value of the datestamp, rather
                # than a datetime.date object
                if isinstance(info[0], datetime.date):
                    datestamp = info[0].strftime("%d %b %Y")
                else:
                    int_dates = map(int, info[0].split("-"))
                    datestamp = datetime.date(*int_dates).strftime("%d %b %Y")

                # The supplier invoice number could be None if no invoice exists
                # in which case display a '?' string 
                if info[3]:
                    supplierinvoicenumber = str(info[3])
                else:
                    supplierinvoicenumber = "?"

                report.add_row_to_table( ('DNL1', datestamp),
                                         ('DNL1', info[1]),
                                         ('DNL1', info[2]),
                                         ('DNL1', supplierinvoicenumber),
                                         ('DNL1', str(info[4])),
                                         ('DNC1', info[5]) )

                EC_purchase += D(info[5])
 
            report.table.append( ('HBR5', 'Payments sent to EC suppliers :'),
                                 ('HBC1', str(EC_purchase)) )

        # Check if there are any VAT corrections from EC supplier creditnotes received with a payment
        c.execute("""SELECT COUNT(*) FROM SUPPLIERCREDITNOTES, TRANSACTIONS, SUPPLIERS
                      WHERE TRANSACTIONS.DATESTAMP BETWEEN ? AND ?
                        AND SUPPLIERCREDITNOTES.STATUS = 1
                        AND EC_COUNTRY = 1
                        AND VAT_COUNTRY_CODE <> ?
                        AND SUPPLIERCREDITNOTES.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID
                        AND SUPPLIERCREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                       """, (from_date, to_date, vs_cfg["VAT_COUNTRY_CODE"]))

        result = c.fetchone()

        if result[0]:
            report.add_row_to_table( ('HNL6', 'EC supplier credit notes received with payments') )

            report.add_row_to_table( ('HBL1', 'Date'),
                                     ('HBL1', 'Supplier'),
                                     ('HBL1', 'Country code'),
                                     ('HBL1', 'Supplier Credit Note'),
                                     ('HBL1', 'Payment transaction'),
                                     ('HBR1', 'Net Cost') )

            c.execute("""SELECT DATESTAMP,
                                SUPPLIERS.NAME,
                                VAT_COUNTRY_CODE,
                                SUPPLIERCREDITNOTES.NUMBER,
                                SUPPLIERCREDITNOTES.TRANSACTION_ID,
                                COST
                           FROM SUPPLIERCREDITNOTES, SUPPLIERS, TRANSACTIONS
                          WHERE SUPPLIERCREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                            AND SUPPLIERCREDITNOTES.STATUS = 1
                            AND DATESTAMP BETWEEN ? AND ?
                            AND EC_COUNTRY = 1
                            AND VAT_COUNTRY_CODE <> ?
                            AND SUPPLIERCREDITNOTES.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID
                     ORDER BY DATESTAMP""", (from_date, to_date, vs_cfg["VAT_COUNTRY_CODE"]))

            for info in c:
                report.add_row_to_table( ('DNL1', info[0].strftime("%d %b %Y")),
                                         ('DNL1', info[1]),
                                         ('DNL1', info[2]),
                                         ('DNL1', info[3]),
                                         ('DNL1', str(info[4])),
                                         ('DNC1', info[5]) )

                EC_purchase_correction += D(info[5])

            report.add_row_to_table( ('HBR6', "Correction from payments received with credit notes:"),
                                     ('HBR1', str(EC_purchase_correction)) )

            EC_purchase = EC_purchase - EC_purchase_correction

            report.add_row_to_table( ('HBR5', "Total EC purchases:"),
                                     ('HBC1', str(EC_purchase)) )

    finally:
        if conn:
            conn.close()

    if EC_purchase:
        result = (EC_purchase * standard_rate) / D("100.0000")
        EC_purchases_VAT = result.quantize(D('.01'), rounding=decimal.ROUND_HALF_EVEN)

        report.add_row_to_table( 
              ('HBR5', "VAT owed is %s%% of EC purchases:" % str(standard_rate).replace(".", vs_cfg["CP"])),
              ('HBC1', str(EC_purchases_VAT)) )

        report.add_row_to_table( ('HBR5', "TOTAL VAT OWED:"),
                                 ('HBC1', str(EC_purchases_VAT+flat_vat)) )

    previous_vatreturn, previous_vatscheme, previous_method = get_previous_vat_return_scheme(vat_return)

    if previous_vatscheme:
        if previous_method != "FLAT-CASH":
            # There has been a change of scheme
            report.add_row_to_table( ('HBR5', "----------------------") )
            report.add_row_to_table( ('HBL5', "Note: change of VAT scheme may require additional payments") )

    return report


############# FLAT-CASH RATE VAT RETURN ####################################

def get_box_1(from_date, to_date):
    """Form text : VAT due in this period on sales and other outputs.

    My interpretation : The flat rate percentage of the gross (vat inclusive)
    sales from payments, as given as the value in box 6"""

    vat_scheme = VATScheme()
    percent = vat_scheme.flat_percent
    gross_sales = get_box_6(from_date, to_date)

    result = (gross_sales * percent) / D("100.0000")
    output_vat = result.quantize(D('.01'), rounding=decimal.ROUND_HALF_EVEN)
    return output_vat


def get_box_2(from_date, to_date):
    """Form text : VAT due in this period on acquisitions from other EC member States

    My interpretation : The value from Box 9 is taken, and the standard rate of VAT
    is applied.  (That is - the vat rate with symbol S set in the vat rates table).
    The subsequent result is set in Box 2."""

    EC_cost = get_box_9(from_date, to_date)
    if EC_cost == D(0):
        return D("0.00")
    result = (EC_cost * vs_cfg["VAT_STANDARD_RATE"]) / D("100.0000")
    EC_purchases_VAT = result.quantize(D('.01'), rounding=decimal.ROUND_HALF_EVEN)
    return EC_purchases_VAT


# Box 3 - Form text : Total VAT due (the sum of boxes 1 and 2)


def get_box_4(from_date, to_date):
    """Form text : VAT reclaimed in this period on purchase and other inputs
                   (including acquisitions from the EC)

    My interpretation : Zero - no VAT is reclaimed.
    In fact VAT can be reclaimed for purchases over
    2000.00, but that is not dealt with yet"""

    return D("0.00")


# Box 5 - Form text : Net VAT to be paid to HMRC or reclaimed by you
#                     (Difference between boxes 3 and 4)


def get_box_6(from_date, to_date):
    """Form text : Total value of sales and all other outputs excluding
                   any VAT. Include your box 8 figure

    My interpretation : The gross cost of all payments, including VAT,
    (and which have not already been included in a previous vat return).
    The result is the value in Box 6."""

    output_cost = D("0.00")

    conn = None
    try:
        conn, c = vs_connect()

        # Get gross cost+vat of payments received
        c.execute("""SELECT COST, VAT
                       FROM INVOICEPAYMENTS, CUSTOMERPAYMENTS, TRANSACTIONS
                      WHERE DATESTAMP BETWEEN ? AND ?
                        AND INVOICEPAYMENTS.VAT_RETURN_ID IS NULL
                        AND INVOICEPAYMENTS.CUSTOMERPAYMENT_ID = CUSTOMERPAYMENTS.CUSTOMERPAYMENT_ID
                        AND CUSTOMERPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID""",
                         (from_date, to_date))

        for result in c:
            output_cost += (D(result[0]) + D(result[1]))

        # Subtract gross cost+vat from any creditnotes issued together with a payment
        # that is - creditnotes with status value 1
        c.execute("""SELECT COST, VAT
                      FROM CREDITNOTES, TRANSACTIONS
                     WHERE DATESTAMP BETWEEN ? AND ?
                       AND STATUS = 1
                       AND CREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID""",
                      (from_date, to_date))

        for result in c:
            output_cost -= (D(result[0])+D(result[1]))


    finally:
        if conn:
            conn.close()
    return output_cost


def get_box_7(from_date, to_date):
    """Form text : Total value of purchases and all other inputs excluding
                   any VAT. Include your box 9 figure

    My interpretation : Zero"""

    return D("0.00")


def get_box_8(from_date, to_date):
    """Form text : Total value of all supplies of goods and related costs,
                   excluding any VAT, to other EC Member States

    My interpretation : The net cost (amount without VAT) from all payments received
    from EC customers within the period,
    (and which have not already been included in a previous vat return),
    minus the costs (again without VAT) of payments
    sent to EC customers with credit notes. An 'EC customer' being a customer
    who is in the EC, who does not have GB set in their country code, and who is vat registered."""

    output_cost = D("0.00")

    conn = None
    try:
        conn, c = vs_connect()

        # Get Cost from payments received from EC registered customers
        c.execute("""SELECT COST
                     FROM INVOICEPAYMENTS, CUSTOMERPAYMENTS, TRANSACTIONS, CUSTOMERS
                     WHERE DATESTAMP BETWEEN ? AND ?
                        AND EC_COUNTRY = 1
                        AND VAT_COUNTRY_CODE <> ?
                        AND VAT_REGISTERED = 1
                        AND INVOICEPAYMENTS.VAT_RETURN_ID IS NULL
                        AND INVOICEPAYMENTS.CUSTOMERPAYMENT_ID = CUSTOMERPAYMENTS.CUSTOMERPAYMENT_ID
                        AND CUSTOMERPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                        AND CUSTOMERPAYMENTS.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID""",
                      (from_date, to_date, vs_cfg["VAT_COUNTRY_CODE"]))

        for result in c:
            output_cost += D(result[0])

        # Subtract any creditnotes
        c.execute("""SELECT COST
                      FROM CREDITNOTES, TRANSACTIONS, CUSTOMERS
                     WHERE DATESTAMP BETWEEN ? AND ?
                       AND STATUS = 1
                       AND EC_COUNTRY = 1
                       AND VAT_COUNTRY_CODE <> ?
                       AND VAT_REGISTERED = 1
                       AND CREDITNOTES.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID
                       AND CREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID""",
                      (from_date, to_date, vs_cfg["VAT_COUNTRY_CODE"]))

        for result in c:
            output_cost -= D(result[0])

    finally:
        if conn:
            conn.close()
    return output_cost


def get_box_9(from_date, to_date):
    """Form text : Total value of all acquisitions of goods and related costs,
                   excluding any VAT, from other EC Member States.

    My interpretation : The net cost (amount without VAT) of payments made to
    all EC suppliers in the period,
    (and which have not already been included in a previous vat return),
    with the costs (again without VAT) of any
    money received from EC suppliers credit notes subtracted. An 'EC supplier'
    being a supplier who is in the EC, who does not have GB set in their country code."""

    input_cost = D("0.00")

    conn = None
    try:
        conn, c = vs_connect()

        # Get Cost from supplier invoices from EC suppliers
        c.execute("""SELECT COST
                       FROM SUPPLIERINVOICEPAYMENTS, MAKEPAYMENTS, TRANSACTIONS, SUPPLIERS
                      WHERE DATESTAMP BETWEEN ? AND ?
                        AND EC_COUNTRY = 1
                        AND VAT_COUNTRY_CODE <> ?
                        AND SUPPLIERINVOICEPAYMENTS.VAT_RETURN_ID IS NULL
                        AND SUPPLIERINVOICEPAYMENTS.MAKEPAYMENT_ID = MAKEPAYMENTS.MAKEPAYMENT_ID
                        AND MAKEPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                        AND MAKEPAYMENTS.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID""",
                      (from_date, to_date, vs_cfg["VAT_COUNTRY_CODE"]))

        for result in c:
            input_cost += D(result[0])

        # Subtract any suppliercreditnotes
        c.execute("""SELECT COST
                      FROM SUPPLIERCREDITNOTES, TRANSACTIONS, SUPPLIERS
                     WHERE TRANSACTIONS.DATESTAMP BETWEEN ? AND ?
                       AND STATUS = 1
                       AND EC_COUNTRY = 1
                       AND VAT_COUNTRY_CODE <> ?
                       AND SUPPLIERCREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                       AND SUPPLIERCREDITNOTES.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID""",
                      (from_date, to_date, vs_cfg["VAT_COUNTRY_CODE"]))

        for result in c:
            input_cost -= D(result[0])

    finally:
        if conn:
            conn.close()
    return input_cost


################### the detail report ############################

def flat_cash_detail(from_date, to_date, vat_return, vat_scheme):
    """Create a detailed report for the flat (cash) vat method
       returns a ReportForm object"""

    # get the flat rate percentage from the applicable vat_scheme
    percent = vat_scheme.flat_percent

    report = ReportForm("VAT detail report")

    report.header = ["For the period:",
                     "From: %s" % from_date.strftime("%d %b %Y"),
                     "To: %s" % to_date.strftime("%d %b %Y"),
                     "",
                     "Method: Flat (Cash), at a flat rate of %s%%" % str(percent).replace(".", vs_cfg["CP"])]

    if vs_cfg["DELETE_TO_DATE"]:
        if from_date < vs_cfg["DELETE_TO_DATE"]:
            report.header.append("!!WARNING!! - this report may be innaccurate")
            report.header.append(
            "Data prior to %s has been deleted from the database" % vs_cfg["DELETE_TO_DATE"].strftime("%d %b %Y"))

    # tables are described by a list of list,
    # each list being a row, consisting of cell tuple
    # each cell tuple containing two strings; 
    # The first string is formed from "XXXN",
    # and the second string is the text to be placed in the cell

    # N is the collspan number, as used in a table cell tag attribute
    # such as 'colspan="4"' - in which case N is 4

    # so normally N is '1'

    # XXX is a formatting string :
    # HBL  - Header Bold Left Justified (R for Right instead of L
    # HNL  - Header Normal Left Justified  etc
    # DNL  - Data Cell Normal Left Justified  etc
    # XXC  - Right Justified currency, as pulled from the database

    # Set the running totals
    output_COST = D("0.00")
    output_COST_correction = D("0.00")
    output_VAT = D("0.00")
    output_VAT_correction = D("0.00")
    output_sales = D("0.00")
    EC_purchase = D("0.00")
    EC_purchase_correction = D("0.00")
    EC_output_sales = D("0.00")
    EC_output_sales_correction = D("0.00")

    conn=None
    try:
        # Make database connection
        conn, c = vs_connect()

        ##### BOX 1
        # Payments from customers headings
        report.table = [[ ('HBL6', "BOX 1 --------") ],
                        [ ('HNL6', 'Payments received from customers') ],
                        [ ('HBL1', 'Date'),
                          ('HBL1', 'Customer'),
                          ('HBL1', 'Invoice number'),
                          ('HBL1', 'Payment transaction'),
                          ('HBR1', 'Net Cost'),
                          ('HBR1', 'VAT') ]]

        # Get the payments from the database
        # UNION select needed as invoices may be NULL if a delete old data has occurred
        if vat_return:
            # This is for a previous vat return, in which a vat_return_id exists
            # so search for payments with this vat_return_id
            c.execute("""SELECT DATESTAMP,
                                NAME,
                                INVOICES.INVOICE_NUMBER,
                                CUSTOMERPAYMENTS.TRANSACTION_ID,
                                INVOICEPAYMENTS.COST,
                                INVOICEPAYMENTS.VAT
                           FROM INVOICEPAYMENTS, CUSTOMERPAYMENTS, TRANSACTIONS, CUSTOMERS, INVOICES
                          WHERE INVOICEPAYMENTS.VAT_RETURN_ID = ?
                            AND INVOICEPAYMENTS.CUSTOMERPAYMENT_ID = CUSTOMERPAYMENTS.CUSTOMERPAYMENT_ID
                            AND INVOICEPAYMENTS.INVOICE_ID = INVOICES.INVOICE_ID
                            AND CUSTOMERPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                            AND CUSTOMERPAYMENTS.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID
                       UNION
                         SELECT DATESTAMP,
                                NAME,
                                NULL,
                                CUSTOMERPAYMENTS.TRANSACTION_ID,
                                INVOICEPAYMENTS.COST,
                                INVOICEPAYMENTS.VAT
                           FROM INVOICEPAYMENTS, CUSTOMERPAYMENTS, TRANSACTIONS, CUSTOMERS
                          WHERE INVOICEPAYMENTS.VAT_RETURN_ID = ?
                            AND INVOICEPAYMENTS.CUSTOMERPAYMENT_ID = CUSTOMERPAYMENTS.CUSTOMERPAYMENT_ID
                            AND INVOICEPAYMENTS.INVOICE_ID IS NULL
                            AND CUSTOMERPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                            AND CUSTOMERPAYMENTS.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID

                       ORDER BY DATESTAMP""", (vat_return.id, vat_return.id))
        else:
            # This is for the current period, in which no vat_return_id exists
            # so search for payments within the dates, but with a null vat_return_id
            # so any payment which already has vat payed on it is not included
            # (if the previous scheme was standard for example)
            c.execute("""SELECT DATESTAMP,
                                NAME,
                                INVOICES.INVOICE_NUMBER,
                                CUSTOMERPAYMENTS.TRANSACTION_ID,
                                INVOICEPAYMENTS.COST,
                                INVOICEPAYMENTS.VAT
                           FROM INVOICEPAYMENTS, CUSTOMERPAYMENTS, TRANSACTIONS, CUSTOMERS, INVOICES
                          WHERE DATESTAMP BETWEEN ? AND ?
                            AND INVOICEPAYMENTS.VAT_RETURN_ID IS NULL
                            AND INVOICEPAYMENTS.CUSTOMERPAYMENT_ID = CUSTOMERPAYMENTS.CUSTOMERPAYMENT_ID
                            AND INVOICEPAYMENTS.INVOICE_ID = INVOICES.INVOICE_ID
                            AND CUSTOMERPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                            AND CUSTOMERPAYMENTS.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID
                       UNION
                         SELECT DATESTAMP,
                                NAME,
                                NULL,
                                CUSTOMERPAYMENTS.TRANSACTION_ID,
                                INVOICEPAYMENTS.COST,
                                INVOICEPAYMENTS.VAT
                           FROM INVOICEPAYMENTS, CUSTOMERPAYMENTS, TRANSACTIONS, CUSTOMERS
                          WHERE DATESTAMP BETWEEN ? AND ?
                            AND INVOICEPAYMENTS.VAT_RETURN_ID IS NULL
                            AND INVOICEPAYMENTS.CUSTOMERPAYMENT_ID = CUSTOMERPAYMENTS.CUSTOMERPAYMENT_ID
                            AND INVOICEPAYMENTS.INVOICE_ID IS NULL
                            AND CUSTOMERPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                            AND CUSTOMERPAYMENTS.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID
                       ORDER BY DATESTAMP""", (from_date, to_date, from_date, to_date))

        for info in c:
            # Fill table with payments from customers data

            # It looks as if the UNION function returns a unicode value of the datestamp, rather
            # than a datetime.date object
            if isinstance(info[0], datetime.date):
                datestamp = info[0].strftime("%d %b %Y")
            else:
                int_dates = map(int, info[0].split("-"))
                datestamp = datetime.date(*int_dates).strftime("%d %b %Y")

            # The invoice number could be None if no invoice exists
            # in which case display a '?' string 
            if info[2]:
                invoicenumber = str(info[2])
            else:
                invoicenumber = "?"

            report.add_row_to_table( ('DNL1', datestamp),
                                     ('DNL1', info[1]),
                                     ('DNL1', invoicenumber),
                                     ('DNL1', str(info[3])),
                                     ('DNC1', info[4]),
                                     ('DNC1', info[5]) )

            output_COST += D(info[4])
            output_VAT += D(info[5])

        report.add_row_to_table( ('DNR4', 'Totals:'),
                                 ('DNC1', str(output_COST)),
                                 ('DNC1', str(output_VAT)) )

        output_sales = output_COST + output_VAT

        # Now list creditnotes with payments issued by you - Output VAT corrections 

        # Check if there are any VAT corrections from creditnotes issued with a payment
        c.execute("""SELECT COUNT(*) FROM CREDITNOTES, TRANSACTIONS
                     WHERE DATESTAMP BETWEEN ? AND ?
                       AND STATUS = 1
                       AND CREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID""",
                      (from_date, to_date))
        result = c.fetchone()

        if result[0]:
            report.add_row_to_table( ('HNL6', 'Correction from credit notes issued with a payment') )

            report.add_row_to_table( ('HBL1', 'Date'),
                                     ('HBL1', 'Customer'),
                                     ('HBL1', 'Credit Note'),
                                     ('HBL1', 'Payment transaction'),
                                     ('HBR1', 'Net Cost'),
                                     ('HBR1', 'VAT') )

            c.execute("""SELECT DATESTAMP,
                                CUSTOMERS.NAME,
                                CREDITNOTE_NUMBER,
                                CREDITNOTES.TRANSACTION_ID,
                                COST,
                                VAT
                           FROM CREDITNOTES, CUSTOMERS, TRANSACTIONS
                          WHERE DATESTAMP BETWEEN ? AND ?
                            AND STATUS = 1
                            AND CREDITNOTES.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID
                            AND CREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                       ORDER BY DATESTAMP""", (from_date, to_date))

            for info in c:
                report.add_row_to_table( ('DNL1', info[0].strftime("%d %b %Y")),
                                         ('DNL1', info[1]),
                                         ('DNL1', info[2]),
                                         ('DNL1', info[3]),
                                         ('DNC1', info[4]),
                                         ('DNC1', info[5]) )

                output_COST_correction += D(info[4])
                output_VAT_correction += D(info[5])

            report.add_row_to_table( ('DNR4', 'Totals:'),
                                     ('DNC1', str(output_COST_correction)),
                                     ('DNC1', str(output_VAT_correction)) )


            output_sales = output_sales - output_VAT_correction - output_COST_correction

        report.add_row_to_table( ('HBR5', "Total payments received (VAT inclusive):"),
                                 ('HBC1', str(output_sales)) )


        # VAT is the percentage of total sales
        result = (output_sales * percent) / D("100.0000")
        flat_vat = result.quantize(D('.01'), rounding=decimal.ROUND_HALF_EVEN)

        # And total VAT owed as a percentage of sales
        report.add_row_to_table( ('HBL4', "VAT owed is %s%% of sales:" % str(percent).replace(".", vs_cfg["CP"])),
                                 ('HBR1', "BOX 1:"),
                                 ('HBC1', str(flat_vat)) )


        ##### BOX 2
        report.add_row_to_table( ('HBL6', "BOX 2 --------") )
        if vat_return:
            # get the stored values
            EC_cost = vat_return.box_nine
            standard_rate = vat_return.standard_rate
            box_2 = vat_return.box_two
        else:
            # calculate using current values
            EC_cost = get_box_9(from_date, to_date)
            standard_rate = vs_cfg["VAT_STANDARD_RATE"]
            if EC_cost == D(0):
                box_2 = D("0.00")
            else:
                result = (EC_cost * standard_rate) / D("100.0000")
                box_2 = result.quantize(D('.01'), rounding=decimal.ROUND_HALF_EVEN)

        report.add_row_to_table( ('DNR5', "Value from BOX 9:"), ('DNC1', str(EC_cost)) )
        report.add_row_to_table( ('DNR5', "At the standard VAT rate of:"), ('DNR1', str(standard_rate).replace(".", vs_cfg["CP"])) )
        report.add_row_to_table( ('HBR5', "BOX 2:"), ('HBC1', str(box_2)) )


        ##### BOX 3
        report.add_row_to_table( ('HBL6', "BOX 3 --------") )
        report.add_row_to_table( ('HBL4', "Total VAT due (the sum of boxes 1 and 2)"),
                                 ('HBR1', "BOX 3:"),
                                 ('HBC1', str(box_2 + flat_vat)) )

        ##### BOX 4
        report.add_row_to_table( ('HBL6', "BOX 4 --------") )
        report.add_row_to_table( ('HBL4', "No VAT is reclaimed"),
                                 ('HBR1', "BOX 4:"),
                                 ('HBC1', "0.00") )

        ##### BOX 5
        report.add_row_to_table( ('HBL6', "BOX 5 --------") )
        report.add_row_to_table( ('HBL4', "Net VAT (Difference between boxes 3 and 4)"),
                                 ('HBR1', "BOX 5:"),
                                 ('HBC1', str(box_2 + flat_vat)) )

        ##### BOX 6
        report.add_row_to_table( ('HBL6', "BOX 6 --------") )
        report.add_row_to_table( ('HBL4', "Total payments received (VAT inclusive) as calculated above"),
                                 ('HBR1', "BOX 6:"),
                                 ('HBC1', str(output_sales)) )

        ##### BOX 7
        report.add_row_to_table( ('HBL6', "BOX 7 --------") )
        report.add_row_to_table( ('HBL4', "Value of purchases is not applicable to the FLAT method"),
                                 ('HBR1', "BOX 7:"),
                                 ('HBC1', "0.00") )

        ##### BOX 8
        report.add_row_to_table( ('HBL6', "BOX 8 --------") )
        report.add_row_to_table( ('HNL6', 'Payments received from VAT registered companies in other EC Member States') )
        report.add_row_to_table( ('HBL1', 'Date'),
                                 ('HBL1', 'Customer'),
                                 ('HBL1', 'Country code'),
                                 ('HBL1', 'Invoice number'),
                                 ('HBL1', 'Payment transaction'),
                                 ('HBR1', 'Net Cost') )

        # Get the payments from the database
        # UNION select needed as invoices may be NULL if a delete old data has occurred
        if vat_return:
            # This is for a previous vat return, in which a vat_return_id exists
            # so search for payments with this vat_return_id
            c.execute("""SELECT DATESTAMP,
                                NAME,
                                VAT_COUNTRY_CODE,
                                INVOICES.INVOICE_NUMBER,
                                CUSTOMERPAYMENTS.TRANSACTION_ID,
                                INVOICEPAYMENTS.COST
                           FROM INVOICEPAYMENTS, CUSTOMERPAYMENTS, TRANSACTIONS, CUSTOMERS, INVOICES
                          WHERE INVOICEPAYMENTS.VAT_RETURN_ID = ?
                            AND EC_COUNTRY = 1
                            AND VAT_COUNTRY_CODE <> ?
                            AND VAT_REGISTERED = 1
                            AND INVOICEPAYMENTS.CUSTOMERPAYMENT_ID = CUSTOMERPAYMENTS.CUSTOMERPAYMENT_ID
                            AND INVOICEPAYMENTS.INVOICE_ID = INVOICES.INVOICE_ID
                            AND CUSTOMERPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                            AND CUSTOMERPAYMENTS.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID
                       UNION
                         SELECT DATESTAMP,
                                NAME,
                                VAT_COUNTRY_CODE,
                                NULL,
                                CUSTOMERPAYMENTS.TRANSACTION_ID,
                                INVOICEPAYMENTS.COST
                           FROM INVOICEPAYMENTS, CUSTOMERPAYMENTS, TRANSACTIONS, CUSTOMERS
                          WHERE INVOICEPAYMENTS.VAT_RETURN_ID = ?
                            AND EC_COUNTRY = 1
                            AND VAT_COUNTRY_CODE <> ?
                            AND VAT_REGISTERED = 1
                            AND INVOICEPAYMENTS.CUSTOMERPAYMENT_ID = CUSTOMERPAYMENTS.CUSTOMERPAYMENT_ID
                            AND INVOICEPAYMENTS.INVOICE_ID IS NULL
                            AND CUSTOMERPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                            AND CUSTOMERPAYMENTS.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID

                       ORDER BY DATESTAMP""", (vat_return.id, vs_cfg["VAT_COUNTRY_CODE"], vat_return.id, vs_cfg["VAT_COUNTRY_CODE"]))
        else:
            # This is for the current period, in which no vat_return_id exists
            # so search for payments within the dates, but with a null vat_return_id
            # so any payment which already has vat payed on it is not included
            # (if the previous scheme was standard for example)
            c.execute("""SELECT DATESTAMP,
                                NAME,
                                VAT_COUNTRY_CODE,
                                INVOICES.INVOICE_NUMBER,
                                CUSTOMERPAYMENTS.TRANSACTION_ID,
                                INVOICEPAYMENTS.COST
                           FROM INVOICEPAYMENTS, CUSTOMERPAYMENTS, TRANSACTIONS, CUSTOMERS, INVOICES
                          WHERE DATESTAMP BETWEEN ? AND ?
                            AND EC_COUNTRY = 1
                            AND VAT_COUNTRY_CODE <> ?
                            AND VAT_REGISTERED = 1
                            AND INVOICEPAYMENTS.VAT_RETURN_ID IS NULL
                            AND INVOICEPAYMENTS.CUSTOMERPAYMENT_ID = CUSTOMERPAYMENTS.CUSTOMERPAYMENT_ID
                            AND INVOICEPAYMENTS.INVOICE_ID = INVOICES.INVOICE_ID
                            AND CUSTOMERPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                            AND CUSTOMERPAYMENTS.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID
                       UNION
                         SELECT DATESTAMP,
                                NAME,
                                VAT_COUNTRY_CODE,
                                NULL,
                                CUSTOMERPAYMENTS.TRANSACTION_ID,
                                INVOICEPAYMENTS.COST
                           FROM INVOICEPAYMENTS, CUSTOMERPAYMENTS, TRANSACTIONS, CUSTOMERS
                          WHERE DATESTAMP BETWEEN ? AND ?
                            AND EC_COUNTRY = 1
                            AND VAT_COUNTRY_CODE <> ?
                            AND VAT_REGISTERED = 1
                            AND INVOICEPAYMENTS.VAT_RETURN_ID IS NULL
                            AND INVOICEPAYMENTS.CUSTOMERPAYMENT_ID = CUSTOMERPAYMENTS.CUSTOMERPAYMENT_ID
                            AND INVOICEPAYMENTS.INVOICE_ID IS NULL
                            AND CUSTOMERPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                            AND CUSTOMERPAYMENTS.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID
                       ORDER BY DATESTAMP""", (from_date, to_date, vs_cfg["VAT_COUNTRY_CODE"], from_date, to_date, vs_cfg["VAT_COUNTRY_CODE"]))

        for info in c:
            # Fill table with payments from EC customers data

            # It looks as if the UNION function returns a unicode value of the datestamp, rather
            # than a datetime.date object
            if isinstance(info[0], datetime.date):
                datestamp = info[0].strftime("%d %b %Y")
            else:
                int_dates = map(int, info[0].split("-"))
                datestamp = datetime.date(*int_dates).strftime("%d %b %Y")

            # The invoice number could be None if no invoice exists
            # in which case display a '?' string 
            if info[3]:
                invoicenumber = str(info[3])
            else:
                invoicenumber = "?"

            report.add_row_to_table( ('DNL1', datestamp),
                                     ('DNL1', info[1]),
                                     ('DNL1', info[2]),
                                     ('DNL1', invoicenumber),
                                     ('DNL1', str(info[4])),
                                     ('DNC1', info[5]) )

            EC_output_sales +=D(info[5])

        report.add_row_to_table( ('DNR5', "Total:"),
                                 ('DNC1', str(EC_output_sales)) )

        # Now list creditnotes with payments issued by you - Output VAT corrections 

        # Check if there are any VAT corrections from creditnotes issued with a payment
        c.execute("""SELECT COUNT(*) FROM CREDITNOTES, TRANSACTIONS, CUSTOMERS
                     WHERE DATESTAMP BETWEEN ? AND ?
                       AND STATUS = 1
                       AND EC_COUNTRY = 1
                       AND VAT_COUNTRY_CODE <> ?
                       AND VAT_REGISTERED = 1
                       AND CREDITNOTES.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID
                       AND CREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID""",
                      (from_date, to_date, vs_cfg["VAT_COUNTRY_CODE"]))
        result = c.fetchone()

        if result[0]:
            report.add_row_to_table( ('HNL6', 'Correction from credit notes issued with a payment') )

            report.add_row_to_table( ('HBL1', 'Date'),
                                     ('HBL1', 'Customer'),
                                     ('HBL1', 'Country code'),
                                     ('HBL1', 'Credit Note'),
                                     ('HBL1', 'Payment transaction'),
                                     ('HBR1', 'Net Cost') )

            c.execute("""SELECT DATESTAMP,
                                CUSTOMERS.NAME,
                                VAT_COUNTRY_CODE,
                                CREDITNOTE_NUMBER,
                                CREDITNOTES.TRANSACTION_ID,
                                COST
                           FROM CREDITNOTES, CUSTOMERS, TRANSACTIONS
                          WHERE DATESTAMP BETWEEN ? AND ?
                            AND STATUS = 1
                            AND CREDITNOTES.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID
                            AND CREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                       ORDER BY DATESTAMP""", (from_date, to_date))

            for info in c:
                report.add_row_to_table( ('DNL1', info[0].strftime("%d %b %Y")),
                                         ('DNL1', info[1]),
                                         ('DNL1', info[2]),
                                         ('DNL1', str(info[3])),
                                         ('DNL1', str(info[4])),
                                         ('DNC1', info[5]) )

                EC_output_sales_correction += D(info[5])

            report.add_row_to_table( ('DNR5', "Total:"),
                                     ('DNC1', str(EC_output_sales_correction)) )


        if EC_output_sales_correction:
            EC_output_sales = EC_output_sales - EC_output_sales_correction
            report.add_row_to_table( ('HBL4', "Payments received corrected from credit notes issued"),
                                     ('HBR1', "BOX 8:"),
                                     ('HBC1', str(EC_output_sales)) )
        else:
            report.add_row_to_table( ('HBR5', "BOX 8:"),
                                     ('HBC1', str(EC_output_sales)) )


        ##### BOX 9
        report.add_row_to_table( ('HBL6', "BOX 9 --------") )
        report.add_row_to_table( ('HNL6', 'Payments made to companies in other EC Member States') )

        report.add_row_to_table( ('HBL1', 'Date'),
                                 ('HBL1', 'Supplier'),
                                 ('HBL1', 'Country code'),
                                 ('HBL1', 'Supplier Invoice'),
                                 ('HBL1', 'Payment transaction'),
                                 ('HBR1', 'Net Cost') )

        if vat_return:
            # This is for a previous vat return, in which a vat_return_id exists
            # so search for payments with this vat_return_id
            c.execute("""SELECT DATESTAMP,
                                SUPPLIERS.NAME,
                                VAT_COUNTRY_CODE,
                                SUPPLIERINVOICES.NUMBER,
                                MAKEPAYMENTS.TRANSACTION_ID,
                                SUPPLIERINVOICEPAYMENTS.COST
                           FROM MAKEPAYMENTS, TRANSACTIONS, SUPPLIERINVOICEPAYMENTS, SUPPLIERS, SUPPLIERINVOICES
                          WHERE SUPPLIERINVOICEPAYMENTS.VAT_RETURN_ID = ?
                            AND EC_COUNTRY = 1
                            AND VAT_COUNTRY_CODE <> ?
                            AND SUPPLIERINVOICEPAYMENTS.SUPPLIERINVOICE_ID = SUPPLIERINVOICES.SUPPLIERINVOICE_ID
                            AND SUPPLIERINVOICEPAYMENTS.MAKEPAYMENT_ID = MAKEPAYMENTS.MAKEPAYMENT_ID
                            AND MAKEPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                            AND MAKEPAYMENTS.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID
                       UNION
                         SELECT DATESTAMP,
                                SUPPLIERS.NAME,
                                VAT_COUNTRY_CODE,
                                NULL,
                                MAKEPAYMENTS.TRANSACTION_ID,
                                SUPPLIERINVOICEPAYMENTS.COST
                           FROM MAKEPAYMENTS, TRANSACTIONS, SUPPLIERINVOICEPAYMENTS, SUPPLIERS
                          WHERE SUPPLIERINVOICEPAYMENTS.VAT_RETURN_ID = ?
                            AND EC_COUNTRY = 1
                            AND VAT_COUNTRY_CODE <> ?
                            AND SUPPLIERINVOICEPAYMENTS.SUPPLIERINVOICE_ID IS NULL
                            AND SUPPLIERINVOICEPAYMENTS.MAKEPAYMENT_ID = MAKEPAYMENTS.MAKEPAYMENT_ID
                            AND MAKEPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                            AND MAKEPAYMENTS.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID

                       ORDER BY DATESTAMP """, (vat_return.id, vs_cfg["VAT_COUNTRY_CODE"], vat_return.id, vs_cfg["VAT_COUNTRY_CODE"]))
        else:
            # This is for the current period, in which no vat_return_id exists
            # so search for payments made within the dates, but with a null vat_return_id
            # so any payment which already has vat payed on it is not included
            # (if the previous scheme was standard for example)
            c.execute("""SELECT DATESTAMP,
                                SUPPLIERS.NAME,
                                VAT_COUNTRY_CODE,
                                SUPPLIERINVOICES.NUMBER,
                                MAKEPAYMENTS.TRANSACTION_ID,
                                SUPPLIERINVOICEPAYMENTS.COST
                           FROM MAKEPAYMENTS, TRANSACTIONS, SUPPLIERINVOICEPAYMENTS, SUPPLIERS, SUPPLIERINVOICES
                          WHERE DATESTAMP BETWEEN ? AND ?
                            AND EC_COUNTRY = 1
                            AND VAT_COUNTRY_CODE <> ?
                            AND SUPPLIERINVOICEPAYMENTS.VAT_RETURN_ID IS NULL
                            AND SUPPLIERINVOICEPAYMENTS.SUPPLIERINVOICE_ID = SUPPLIERINVOICES.SUPPLIERINVOICE_ID
                            AND SUPPLIERINVOICEPAYMENTS.MAKEPAYMENT_ID = MAKEPAYMENTS.MAKEPAYMENT_ID
                            AND MAKEPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                            AND MAKEPAYMENTS.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID
                       UNION
                         SELECT DATESTAMP,
                                SUPPLIERS.NAME,
                                VAT_COUNTRY_CODE,
                                NULL,
                                MAKEPAYMENTS.TRANSACTION_ID,
                                SUPPLIERINVOICEPAYMENTS.COST
                           FROM MAKEPAYMENTS, TRANSACTIONS, SUPPLIERINVOICEPAYMENTS, SUPPLIERS
                          WHERE DATESTAMP BETWEEN ? AND ?
                            AND EC_COUNTRY = 1
                            AND VAT_COUNTRY_CODE <> ?
                            AND SUPPLIERINVOICEPAYMENTS.VAT_RETURN_ID IS NULL
                            AND SUPPLIERINVOICEPAYMENTS.SUPPLIERINVOICE_ID IS NULL
                            AND SUPPLIERINVOICEPAYMENTS.MAKEPAYMENT_ID = MAKEPAYMENTS.MAKEPAYMENT_ID
                            AND MAKEPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                            AND MAKEPAYMENTS.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID

                       ORDER BY DATESTAMP    """, (from_date, to_date, vs_cfg["VAT_COUNTRY_CODE"],
                                                   from_date, to_date, vs_cfg["VAT_COUNTRY_CODE"]))

        for info in c:
            # It looks as if the UNION function returns a unicode value of the datestamp, rather
            # than a datetime.date object
            if isinstance(info[0], datetime.date):
                datestamp = info[0].strftime("%d %b %Y")
            else:
                int_dates = map(int, info[0].split("-"))
                datestamp = datetime.date(*int_dates).strftime("%d %b %Y")

            # The supplier invoice number could be None if no invoice exists
            # in which case display a '?' string 
            if info[3]:
                supplierinvoicenumber = str(info[3])
            else:
                supplierinvoicenumber = "?"

            report.add_row_to_table( ('DNL1', datestamp),
                                     ('DNL1', info[1]),
                                     ('DNL1', info[2]),
                                     ('DNL1', supplierinvoicenumber),
                                     ('DNL1', str(info[4])),
                                     ('DNC1', info[5]) )

            EC_purchase += D(info[5])
 
        report.add_row_to_table( ('DNR5', "Total:"),
                                 ('DNC1', str(EC_purchase)) )

        # Check if there are any VAT corrections from EC supplier creditnotes received with a payment
        c.execute("""SELECT COUNT(*) FROM SUPPLIERCREDITNOTES, TRANSACTIONS, SUPPLIERS
                      WHERE TRANSACTIONS.DATESTAMP BETWEEN ? AND ?
                        AND SUPPLIERCREDITNOTES.STATUS = 1
                        AND EC_COUNTRY = 1
                        AND VAT_COUNTRY_CODE <> ?
                        AND SUPPLIERCREDITNOTES.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID
                        AND SUPPLIERCREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                       """, (from_date, to_date, vs_cfg["VAT_COUNTRY_CODE"]))

        result = c.fetchone()

        if result[0]:
            report.add_row_to_table( ('HNL6', 'EC supplier credit notes received with payments') )

            report.add_row_to_table( ('HBL1', 'Date'),
                                     ('HBL1', 'Supplier'),
                                     ('HBL1', 'Country code'),
                                     ('HBL1', 'Supplier Credit Note'),
                                     ('HBL1', 'Payment transaction'),
                                     ('HBR1', 'Net Cost') )

            c.execute("""SELECT DATESTAMP,
                                SUPPLIERS.NAME,
                                VAT_COUNTRY_CODE,
                                SUPPLIERCREDITNOTES.NUMBER,
                                SUPPLIERCREDITNOTES.TRANSACTION_ID,
                                COST
                           FROM SUPPLIERCREDITNOTES, SUPPLIERS, TRANSACTIONS
                          WHERE SUPPLIERCREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                            AND SUPPLIERCREDITNOTES.STATUS = 1
                            AND DATESTAMP BETWEEN ? AND ?
                            AND EC_COUNTRY = 1
                            AND VAT_COUNTRY_CODE <> ?
                            AND SUPPLIERCREDITNOTES.SUPPLIER_ID = SUPPLIERS.SUPPLIER_ID
                     ORDER BY DATESTAMP""", (from_date, to_date, vs_cfg["VAT_COUNTRY_CODE"]))

            for info in c:
                report.add_row_to_table( ('DNL1', info[0].strftime("%d %b %Y")),
                                         ('DNL1', info[1]),
                                         ('DNL1', info[2]),
                                         ('DNL1', info[3]),
                                         ('DNL1', str(info[4])),
                                         ('DNC1', info[5]) )

                EC_purchase_correction += D(info[5])

            report.add_row_to_table( ('DNR5', "Total:"),
                                     ('DNC1', str(EC_purchase_correction)) )

        if EC_purchase_correction:
            EC_purchase = EC_purchase - EC_purchase_correction
            report.add_row_to_table( ('HBL4', "Payments made corrected from credit notes received"),
                                     ('HBR1', "BOX 9:"),
                                     ('HBC1', str(EC_purchase)) )
        else:
            report.add_row_to_table( ('HBR5', "BOX 9:"),
                                     ('HBC1', str(EC_purchase)) )

    finally:
        if conn:
            conn.close()

    previous_vatreturn, previous_vatscheme, previous_method = get_previous_vat_return_scheme(vat_return)

    if previous_vatscheme:
        if previous_method != "FLAT-CASH":
            # There has been a change of scheme
            report.add_row_to_table( ('HBR6', "----------------------") )
            report.add_row_to_table( ('HBL6', "Note: change of VAT scheme may require additional payments") )

    return report

