####### VATStuff #######
#
# account_types.py  - defines the TypesEditDialog to edit account types
#
# Version : 0.6
# Date : 20100718
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#

"""Defines the TypesEditDialog to edit account types such as Asset and Expense"""


import wx
import wx.grid



from vatstuff_packages.vs_database.create_objects import create_AccountType



from vatstuff_packages.vs_gui.gui_functions import *

from vatstuff_packages.vs_utilities import *

from vatstuff_packages.vs_verify.accounts import account_type_table_check


def types_edit_dialog(parent):
    """Creates a Types Edit Dialog"""
    types_edit_dialog = TypesEditDialog(parent)

    # If the cursor is busy, unbusy it - as
    # now waiting for a button press
    busy_cursor_off()

    types_edit_dialog.ShowModal()
    types_edit_dialog.Destroy()


class TypesEditDialog(vsDialog):
    "The TypesEditDialog"

    def __init__(self, parent):
        """Create the Dialog"""
        vsDialog.__init__(self, parent, "Edit Account Types")

        # Create the sizer
        sizer=wx.BoxSizer(wx.VERTICAL)

        # help text
        text = wx.StaticText(self, -1, "Edit the account types which can be applied to Accounts")
        sizer.Add(text, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # Create the input labels and fields in a grid sizer
        fields_sizer=wx.FlexGridSizer(rows=2, cols=2, hgap=20, vgap=20)

        # -------------------The new customer and new supplier account type

        customer_account_type = AccountType(int(get_config_item("CUSTOMER_ACCOUNT_TYPE_ID")))
        supplier_account_type = AccountType(int(get_config_item("SUPPLIER_ACCOUNT_TYPE_ID")))

        # customer
        customer_label=wx.StaticText(self, -1, "Default type for new customer accounts :")
        fields_sizer.Add(customer_label, 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.customer_field = wx.TextCtrl(self, -1, customer_account_type.name, size=(200, vs_cfg["BUTTON_HEIGHT"]))
        self.customer_field.SetToolTipString("Set the account type applied to new customer accounts")
        self.customer_field.SetMaxLength(50)
        fields_sizer.Add(self.customer_field, 0, wx.ALIGN_LEFT)

        # supplier
        supplier_label=wx.StaticText(self, -1, "Default type for new supplier accounts :")
        fields_sizer.Add(supplier_label, 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.supplier_field = wx.TextCtrl(self, -1, supplier_account_type.name, size=(200, vs_cfg["BUTTON_HEIGHT"]))
        self.supplier_field.SetToolTipString("Set the account type applied to new supplier accounts")
        self.supplier_field.SetMaxLength(50)
        fields_sizer.Add(self.supplier_field, 0, wx.ALIGN_LEFT)

        # add the fields_sizer to the main vertical sizer
        sizer.Add(fields_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # ---------------------- Create the rates table

        self.all_accounttypes = AccountType.get_all()
        self.types_table = TypesTable(self, self.all_accounttypes)
        sizer.Add(self.types_table, 0, wx.ALL | wx.ALIGN_CENTER, 40)
 
        # --------------------- Apply - Cancel -- buttons

        button_size = (100, vs_cfg["BUTTON_HEIGHT"])

        # Create a horizontal button_sizer to hold the buttons
        button_sizer=wx.BoxSizer(wx.HORIZONTAL)

        # Apply button
        self.apply_button = wx.Button(self, wx.ID_APPLY, size=button_size)
        self.apply_button.SetToolTipString("Click to apply changes.")
        button_sizer.Add(self.apply_button, 0, wx.RIGHT | wx.ALIGN_RIGHT, 20)
        # Start off with apply button disabled - enabled by any changes
        self.apply_button.Enable(False)

        # Close button
        self.close_button = wx.Button(self, wx.ID_CLOSE, size=button_size)
        self.close_button.SetToolTipString("Click to close")
        button_sizer.Add(self.close_button, 0, wx.LEFT | wx.ALIGN_LEFT, 20)

        # Add button_sizer to the main sizer
        sizer.Add(button_sizer, 0, wx.BOTTOM | wx.ALIGN_CENTER, 40)

        # Set the size to the panel
        self.SetSizer(sizer)
        sizer.SetSizeHints(self)
        
        # ---------- Bind buttons to handler functions
        
        self.Bind(wx.EVT_BUTTON, self.apply_button_pressed, self.apply_button)
        self.Bind(wx.EVT_BUTTON, self.close_button_pressed, self.close_button)

        self.Bind(wx.EVT_TEXT, self.enable_apply, self.customer_field)
        self.Bind(wx.EVT_TEXT, self.enable_apply, self.supplier_field)


    # ----------------- Handler functions

    def enable_apply(self, event):
        "Sets apply button true should one of the text fields change"
        self.apply_button.Enable(True)

    @writecheck
    @button_wrap
    def apply_button_pressed(self, event):
        """Apply button - check and apply all fields"""

        customer_type_name=self.customer_field.GetValue().strip()
        supplier_type_name=self.supplier_field.GetValue().strip()

        # get values from the types_table - a list of lists (name, description)
        types_list = self.types_table.get_fields()

        # Check values
        status, message = account_type_table_check(customer_type_name, supplier_type_name, types_list, self.all_accounttypes)

        if not status:
            vs_message(self, message, "ERROR")
            return

        # Implement the changes

        number = len(self.all_accounttypes)
        for index, acc_type in enumerate(self.all_accounttypes):
            # change or delete existing account types
            if types_list[index][0]:
                acc_type.name = types_list[index][0]
                acc_type.description = types_list[index][1]
            else:
                acc_type.delete()

        for index, acc_type in enumerate(types_list):
            if index < number:
                continue
            # So there follows new account types to create
            if acc_type[0]:
                # create new account type
                create_AccountType(acc_type[0], acc_type[1])

        # get all account types, to set the new customer and supplier types
        self.all_accounttypes = AccountType.get_all()
        for acc_type in self.all_accounttypes:
            if customer_type_name == acc_type.name:
                customer_type = acc_type
            if supplier_type_name == acc_type.name:
                supplier_type = acc_type

        set_config_item("CUSTOMER_ACCOUNT_TYPE_ID", customer_type.id)
        set_config_item("SUPPLIER_ACCOUNT_TYPE_ID", supplier_type.id)
            
        top_refresh_accounts()
        self.types_table.refresh(self.all_accounttypes)
        self.apply_button.Enable(False)
        vs_message(self, "Changes saved", "DONE")


    def close_button_pressed(self, event):
        """Close button"""
        if not self.apply_button.IsEnabled():
            self.EndModal(wx.ID_OK)
            return
        if vs_yesno(self, """\
Closing account types: are you sure?
Any changes will be lost unless you press Apply""", "Closing list"):
            self.EndModal(wx.ID_OK)


class TypesTable(wx.grid.Grid):

    def __init__(self, parent, all_accounttypes):
        # dialog has a fixed size grid
        wx.grid.Grid.__init__(self, parent, size=(750,200), style=wx.WANTS_CHARS)

        self.all_accounttypes = all_accounttypes
        self.parent = parent
        self.acctype_names = [acc_type.name for acc_type in self.all_accounttypes ]

        num_rows = len(self.all_accounttypes) + 5   # Make 5 'blank' rows
        num_cols = 2

        self.CreateGrid(num_rows, num_cols)

        # create column labels
        self.SetColLabelValue(0, "Name")
        self.SetColLabelValue(1, "Description")
        self.SetDefaultRowSize(vs_cfg["BUTTON_HEIGHT"], False)
        self.SetDefaultCellAlignment(wx.ALIGN_LEFT, wx.ALIGN_CENTER)

        for key, acc_type in enumerate(self.all_accounttypes):
            self.SetCellValue(key, 0, acc_type.name)
            self.SetCellValue(key, 1, acc_type.description)

        self.SetColSize(0, 150)
        self.SetColSize(1, 500)

        self.SetColMinimalAcceptableWidth(100)
        self.SetRowLabelSize(50)

        # When an item is edited, add a further row to ensure there are always blank rows
        # displayed:

        self.Bind(wx.grid.EVT_GRID_CELL_CHANGE, self.addarow)


    def refresh(self, all_accounttypes):
        "Get all contacts again, and re-fill the table"

        self.all_accounttypes = all_accounttypes

        number_of_types = len(self.all_accounttypes)
        current_number_of_rows = self.GetNumberRows()

        # If everything is working ok, this must be true
        assert number_of_types < current_number_of_rows
        
        # clear the grid
        self.ClearGrid()

        for key, acc_type in enumerate(self.all_accounttypes):
            self.SetCellValue(key, 0, acc_type.name)
            self.SetCellValue(key, 1, acc_type.description)


    def addarow(self, evt):
        """Keep some blank rows at the bottom available for editing
           if the user starts inserting data in the last four rows - 
           add a further row"""
        try:
            # as an item has been edited, enable the apply button
            self.parent.apply_button.Enable(True)

            current_number_of_rows = self.GetNumberRows()
            if evt.GetRow()+4 > current_number_of_rows:
                self.AppendRows(numRows=1)
        except Exception, e:
            error_message("Failure in editing an account type row", e)


    def get_fields(self):
        """Reads the items on the table"""

        table = []
        field_number = self.GetNumberRows()
        for row in range(0, field_number):
            acc_type = [self.GetCellValue(row, 0).strip(), self.GetCellValue(row, 1).strip()] 
            table.append(acc_type)

        return table

