####### VATStuff #######
#
# accounts_tab.py  - defines the panel of the accounts tab
#
# Version : 0.6
# Date : 20100718
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#


""" This is a module in the vatstuff_packages.vs_gui package, and defines the accounts panel

It is run as part of the vatstuff gui interface.
"""


import wx

from vatstuff_packages.vs_gui.accounts import tree
from vatstuff_packages.vs_gui.accounts import new_folder
from vatstuff_packages.vs_gui.accounts import new_account
from vatstuff_packages.vs_gui.accounts import edit_folder
from vatstuff_packages.vs_gui.accounts import edit_account
from vatstuff_packages.vs_gui.accounts import list_account
from vatstuff_packages.vs_gui.accounts import account_types

from vatstuff_packages.vs_gui.gui_functions import *






from vatstuff_packages.vs_utilities import *

from vatstuff_packages.vs_verify.accounts import delete_folder_check, delete_account_check


class AccountsPanel(wx.Panel):
    """The Accounts panel"""

    def __init__(self, parent):
        wx.Panel.__init__(self, parent)
        # Log the panel creation
        debug_log("Creating panel: %s" % self.__class__.__name__)
                     
        self.sizer=wx.BoxSizer(wx.VERTICAL)

        # Create the account_tree
        self.account_tree = tree.AccountsTree(self)
      
        # Create the list of top buttons, pass the account tree to this panel
        # as button bindings refer to tree events
        self.button_box=ButtonBox(self, self.account_tree)

        # Add button_box to the main vertical sizer
        self.sizer.Add(self.button_box, 0,  wx.TOP | wx.EXPAND, 10)

        # Add account_tree to the main vertical sizer
        self.sizer.Add(self.account_tree, 1, wx.EXPAND)

        self.sizer.SetItemMinSize(self.account_tree, (700,400))

        self.SetSizer(self.sizer)
        self.Fit()

        
    def refresh(self, item=None, current_item=True):
        """Deletes current tree, re-assigns folders and accounts to tree items,
           and select the item,
           if no item given, and current item is False, selects first folder
           if no item given, and current item is True, leaves current selection"""
        self.account_tree.refresh(item, current_item)


class ButtonBox(wx.Panel):
    """The Buttons on the top of the tree"""

    def __init__(self, parent, account_tree):
        """Create the panel"""
        wx.Panel.__init__(self, parent)

        self.parent = parent
        self.account_tree = account_tree

        # Set the button size
        button_size = (100, vs_cfg["BUTTON_HEIGHT"])
        # Create a horizontal button_box_sizer to hold the buttons
        button_box_sizer=wx.BoxSizer(wx.HORIZONTAL)

        self.new_folder   = wx.Button(self, label="New Folder", size=button_size)
        self.new_account  = wx.Button(self, label="New Account", size=button_size)
        self.edit         = wx.Button(self, label="Edit", size=button_size)
        self.delete       = wx.Button(self, label="Delete", size=button_size)
        self.acc_types    = wx.Button(self, label="Types", size=button_size)

        button_box_sizer.Add((20,40), 1)
        button_box_sizer.Add(self.new_folder, 0)

        button_box_sizer.Add((20,40), 1)
        button_box_sizer.Add(self.new_account, 0)

        button_box_sizer.Add((20,40), 1)
        button_box_sizer.Add(self.edit, 0)

        button_box_sizer.Add((20,40), 1)
        button_box_sizer.Add(self.delete, 0)

        button_box_sizer.Add((20,40), 1)
        button_box_sizer.Add(self.acc_types, 0)

        button_box_sizer.Add((20,40), 1)

        # Set the button bindings
        self.Bind(wx.EVT_BUTTON, self.new_folder_pressed, self.new_folder)
        self.Bind(wx.EVT_BUTTON, self.new_account_pressed, self.new_account)
        self.Bind(wx.EVT_BUTTON, self.edit_pressed, self.edit)
        self.Bind(wx.EVT_BUTTON, self.delete_pressed, self.delete)
        self.Bind(wx.EVT_BUTTON, self.acc_types_pressed, self.acc_types)
        
        # Set the sizer to the panel
        self.SetSizer(button_box_sizer)
        self.Fit()


    # button handler functions 

    @button_wrap
    def new_folder_pressed(self, evt):
        "Show the new folder dialog"
        new_folder.new_folder_dialog(self)


    @button_wrap
    def new_account_pressed(self, evt):
        "Show the new account dialog"
        # Tell the new account dialog what is currently selected
        selected_item=self.account_tree.GetSelection()
        # Get the Python object associated with that selection
        item=self.account_tree.GetItemPyData(selected_item)
        # Get the parent folder if the item selected is an Account
        if isinstance(item, Folder):
            folder = item
        elif isinstance(item, Account) :
            folder = item.folder
        else:
            raise VS_ERROR, "Cannot find Folder to add the new account to"

        # Create new_account_dialog
        new_account.new_account_dialog(self, folder)


    @button_wrap
    def edit_pressed(self, evt):
        "Edit a folder or an account"
        # Tell the edit dialog what is currently selected
        selected_item=self.account_tree.GetSelection()
        # Get the Python object associated with that selection
        item=self.account_tree.GetItemPyData(selected_item)
        
        # The item is either a Folder or an Account
        if not item:
            # Nothing is selected, just return
            return
        # Set up the folder edit dialog
        if isinstance(item, Folder):
            # Create edit_folder_dialog
            edit_folder.edit_folder_dialog(self, item)
        elif isinstance(item, Account):
            # Create an EditAccountDialog if this is not a system account
            if item.system < 2:
                edit_account.edit_account_dialog(self, item)
            else:
                # Its a system account, so create an EditSystemAccountDialog
                edit_account.edit_system_account_dialog(self, item)
        else:
            raise VS_ERROR, "item not recognised"


    @writecheck
    @button_wrap
    def delete_pressed(self, evt):
        "Call delete functions"
        # Check what is selected
        selected_item=self.account_tree.GetSelection()
        # Get the Python object associated with that selection
        item=self.account_tree.GetItemPyData(selected_item)
        
        # The item selected is either a Folder or an Account
        if not item:
            # Nothing is selected, just return
            return

        result = False
        if isinstance(item, Folder):
            result = self.delete_folder(item)
        elif isinstance(item, Account):
            result = self.delete_account(item)
        else:
            raise VS_ERROR, "item not recognised"
        if result:
            # refresh accounts
            top_refresh_accounts(item=None, current_item=False)


    @button_wrap
    def acc_types_pressed(self, evt):
        "Create the edit account types dialog"
        account_types.types_edit_dialog(self)


    @writecheck
    def delete_folder(self, folder):
        "Checks if a folder can be deleted, if it can, then does so and return True."

        # Check folder can be deleted
        status, message = delete_folder_check(folder)
        if not status:
            vs_message(self, message, "ERROR")
            return False

        retCode=vs_yesno(self,
                         "Are you sure you wish to\ndelete \"%s\"" % folder.name,
                         "Delete Folder")

        if not retCode: return False
        busy_cursor_on()
        wx.Yield()
        # delete from the database
        status = folder.delete()
        wx.Yield()
        if not status:
            vs_message(self, "Unable to delete the folder", "ERROR")
            return False

        return True


    @writecheck
    def delete_account(self, account):
        "Checks if an account can be deleted, if it can, then does so and return True."

        # Check account can be deleted
        status, message = delete_account_check(account)
        if not status:
            vs_message(self, message, "ERROR")
            return False

        retCode=vs_yesno(self,
                         "Are you sure you wish to\ndelete \"%s\"" % account.name,
                         "Delete Account")

        if not retCode : return False

        # if this account has a window open, close it
        for frm in vs_cfg["TRANSACTION_REFRESH"]:
            if isinstance(frm, list_account.ListAccountFrame):
                if frm.account == account:
                    frm.Close()

        busy_cursor_on()
        wx.Yield()
        # delete from the database
        status = account.delete()
        wx.Yield()
        if not status:
            vs_message(self, "Unable to delete the account", "ERROR")
            return False

        return True



