####### VATStuff #######
#
# edit_account.py  - defines the EditAccount dialogs
#
# Version : 0.7.0
# Date : 20100925
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#

""" This is a module in the vatstuff_packages.vs_gui package, and defines the EditAccountDialog
and the EditSystemAccountDialog

It is run as part of the vatstuff gui interface.
"""

import datetime
import decimal

import wx

from vatstuff_packages.vs_database import transactions

from vatstuff_packages.vs_gui.gui_functions import *

from vatstuff_packages.vs_utilities import *

from vatstuff_packages.vs_verify.accounts import edit_account_check
from vatstuff_packages.vs_verify import form_objects

D = decimal.Decimal   # D used to avoid typing


def edit_account_dialog(parent, account):
    """Creates an EditAccountDialog"""
    edit_account_dialog = EditAccountDialog(parent, account)

    # If the cursor is busy, unbusy it - as
    # now waiting for a button press
    busy_cursor_off()

    edit_account_dialog.ShowModal()
    edit_account_dialog.Destroy()


class EditAccountDialog(vsDialog):
    "The edit account dialog"

    def __init__(self, parent, account):
        """Create the Dialog"""
        vsDialog.__init__(self, parent, "Edit Account")
        
        # Store the account in this instance
        self.account=account

        # --------------------- The Folder Choices

        # Get the list of non-system folders, to be used in the folders Choice option
        self.folders=Folder.get_non_system()

        # If no non-system folders exist
        if not self.folders:
            raise VS_ERROR, "No non-system folders exist"

        try:
            folder_selection = self.folders.index(account.folder)
        except Exception:
            folder_selection = 0

        # --------------------- PANEL FIELDS

        # Create the sizer for the panel
        sizer=wx.BoxSizer(wx.VERTICAL)

        # Create the input labels and fields in a grid sizer
        fields_sizer=wx.FlexGridSizer(rows=5, cols=2, hgap=20, vgap=20)

        # Account Type
        # Create an account type list, with a selection - to be used by the accounts type Choice box

        self.all_account_types = AccountType.get_all()

        try:
            acc_type_selection = self.all_account_types.index(self.account.acc_type)
        except Exception:
            acc_type_selection = 0

        self.type_text=wx.StaticText(self, -1, "Account type:")
        fields_sizer.Add(self.type_text, 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.acc_type_field = wx.Choice(self, -1, size=(200, vs_cfg["BUTTON_HEIGHT"]),
                                        choices=[acc_type.name for acc_type in self.all_account_types ])
        self.acc_type_field.SetToolTipString("Set the account type")
        self.acc_type_field.SetSelection(acc_type_selection)
        fields_sizer.Add(self.acc_type_field, 0, wx.ALIGN_LEFT)

        # The Folder Choice box
        self.folder_text=wx.StaticText(self, -1, "Account is a member of folder")
        fields_sizer.Add(self.folder_text, 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.folder_field = wx.Choice(self, -1, size=(200, vs_cfg["BUTTON_HEIGHT"]),
                                      choices=[ f.name for f in self.folders ])
        self.folder_field.SetToolTipString("Set folder, which this account will be shown under")
        self.folder_field.SetSelection(folder_selection)
        fields_sizer.Add(self.folder_field, 0, wx.ALIGN_LEFT)

        # The Account name label and field
        name_text=wx.StaticText(self, -1, "Account name")
        fields_sizer.Add(name_text, 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.name_field = wx.TextCtrl(self, -1, self.account.name, size=(200, vs_cfg["BUTTON_HEIGHT"]))
        self.name_field.SetToolTipString("The name of this account (20 characters maximum)")
        self.name_field.SetMaxLength(20)

        fields_sizer.Add(self.name_field, 0, wx.ALIGN_LEFT)

        # The account description label and field
        description_text=wx.StaticText(self, -1, "Account description")
        fields_sizer.Add(description_text, 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.description_field = wx.TextCtrl(self, -1, self.account.description, size=(300, vs_cfg["BUTTON_HEIGHT"]))
        self.description_field.SetToolTipString("The description of this account (50 characters maximum)")
        self.description_field.SetMaxLength(50)

        fields_sizer.Add(self.description_field, 0, wx.ALIGN_LEFT)

        # Set the opening balance and date (only shown if system flag is zero)

        # opening Balance Text
        self.balance_text=wx.StaticText(self, -1, "opening balance")
        fields_sizer.Add(self.balance_text, 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        # create a horizontal box sizer for the two fields
        balance_sizer=wx.BoxSizer(wx.HORIZONTAL)

        # opening Balance field
        self.balance_field = wx.TextCtrl(self, -1, vs_cfg["SYMBOL_ZERO_STRING"], size=(100, vs_cfg["BUTTON_HEIGHT"]))
        self.balance_field.SetToolTipString("Set the initial amount currently in this account")
        self.balance_field.SetMaxLength(10)
        balance_sizer.Add(self.balance_field, 0, wx.ALIGN_LEFT)

        # opening Balance date
        self.balance_date_field = wx.DatePickerCtrl(self,
                                                    size=(150, vs_cfg["BUTTON_HEIGHT"]),
                                                    style=wx.DP_DROPDOWN | wx.DP_SHOWCENTURY)
        self.balance_date_field.SetToolTipString("Set the date of the opening balance")
        balance_sizer.Add(self.balance_date_field, 0, wx.ALIGN_LEFT | wx.LEFT, 20)

        fields_sizer.Add(balance_sizer, 0, wx.ALIGN_LEFT)

        sizer.Add(fields_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # ------------ display opening balance only if system is zero

        if self.account.system > 0:              # Hide opening balance if system flag is not zero
            self.balance_text.Hide()
            self.balance_field.Hide()
            self.balance_date_field.Hide()

        # -- Apply - Cancel -- buttons
        button_size = (100, vs_cfg["BUTTON_HEIGHT"])

        # Create a horizontal button_sizer to hold the buttons
        button_sizer=wx.BoxSizer(wx.HORIZONTAL)

        # Apply button
        self.apply_button = wx.Button(self, wx.ID_APPLY, size=button_size)
        self.apply_button.SetToolTipString("Click to apply changes.")
        button_sizer.Add(self.apply_button, 0, wx.RIGHT, 40)
        
        # Cancel button
        self.cancel_button = wx.Button(self, wx.ID_CANCEL, size=button_size)
        self.cancel_button.SetToolTipString("Click to cancel changes.")
        button_sizer.Add(self.cancel_button, 0, wx.LEFT)

        # Add button_sizer to the main sizer
        sizer.Add(button_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 40)

        # Set the size to the panel
        self.SetSizer(sizer)
        sizer.SetSizeHints(self)
        
        # ---------- Bind buttons to handler functions
        
        self.Bind(wx.EVT_BUTTON, self.apply_button_pressed, self.apply_button)
        self.Bind(wx.EVT_BUTTON, self.cancel_button_pressed, self.cancel_button)

    # ----------------- Handler functions


    @writecheck
    @button_wrap
    def apply_button_pressed(self, event):
        """Apply button - check and apply all fields"""
        name=self.name_field.GetValue().strip()
        description=self.description_field.GetValue().strip()

        # Get the folder selected
        folder_selected = self.folders[self.folder_field.GetCurrentSelection()]
                
        # opening balance field is only available if this is a new account
        # and the system flag is set to zero
        if self.account.system:
            # system flag is not zero
            # set a zero temporary value, to flag no change
            balance = D("0.00")
            datestamp = None
        else:
            # system flag is zero
            # Get the new balance
            status, balance = string_to_decimal(self.balance_field.GetValue())
            if not status:
                vs_message(self, "Invalid balance", "ERROR")
                return
            # Get the datestamp
            datestamp = wxdate_to_date(self.balance_date_field.GetValue())

        # Get the account type selected
        acc_type = self.all_account_types[self.acc_type_field.GetCurrentSelection()]

        # Check input details are ok
        status, message = edit_account_check(self.account, name, folder_selected, balance, datestamp)
        if not status:
            vs_message(self, message, "ERROR")
            return

        # Write out the changes
        wx.Yield()
        if acc_type != self.account.acc_type:
            self.account.acc_type = acc_type

        if name != self.account.name:
            self.account.name = name

        if description != self.account.description:
            self.account.description = description

        if folder_selected != self.account.folder:
            self.account.folder = folder_selected

        # Create the opening balance transaction
        if balance:
            status, message = self.set_opening_balance(balance, datestamp)
            if not status:
                top_refresh_accounts(self.account)
                vs_message(self, "Failed to implement opening balance\n%s" % message, "DONE")
                self.EndModal(wx.ID_OK)
        
        top_refresh_accounts(self.account)
        self.EndModal(wx.ID_OK)


    def cancel_button_pressed(self, event):
        """ Cancel button - Close parent"""
        self.EndModal(wx.ID_CANCEL)


    def set_opening_balance(self, balance, datestamp):
        """Creates a transaction to set the opening balance"""

        # Store the opening transaction in a TransactionForm object
        transaction = form_objects.TransactionForm()

        transaction.description = "%s opening balance" % self.account.fullname
        transaction.datestamp = datestamp
        # This is an opening balance transaction (system type 1)
        transaction.system = 1

        # Get the system opening balances account - this being the account with system = 2
        op_account = get_account_by_system(2)

        # If balance is greater than zero, then debit the account by the
        # balance, otherwise credit it
        if balance > D(0):
            transaction.add(self.account, debit=balance)
            transaction.add(op_account, credit=balance)
        else:
            balance = abs(balance)
            transaction.add(self.account, credit=balance)
            transaction.add(op_account, debit=balance)

        status, message = transaction.check()
        if not status:
            return False, message

        # In this case do not confirm the transaction - as doing so does not add any value

        # Must now implement the opening balance transaction
        transactions.implement(None, transaction)

        info_log("opening balance set")
        return True, None


################### edit a system account : name and description only

def edit_system_account_dialog(parent, account):
    """Creates an EditSystemAccountDialog"""
    edit_system_account_dialog = EditSystemAccountDialog(parent, account)

    # If the cursor is busy, unbusy it - as
    # now waiting for a button press
    busy_cursor_off()

    edit_system_account_dialog.ShowModal()
    edit_system_account_dialog.Destroy()


class EditSystemAccountDialog(vsDialog):
    "The edit system account dialog"

    def __init__(self, parent, account):
        """Create the Dialog"""
        vsDialog.__init__(self, parent, "Edit Account")
        
        # Store the account in this instance
        self.account=account  

        # Create the sizer
        sizer=wx.BoxSizer(wx.VERTICAL)

        help_text="""This is a system account, used by the program for internal calculations.
You are not given the option to move it to other folders."""
        
        self.top_text=wx.StaticText(self, -1, help_text)
        sizer.Add(self.top_text, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # Create the input labels and fields in a grid sizer
        fields_sizer=wx.FlexGridSizer(rows=3, cols=2, hgap=20, vgap=20)

        # Account Type
        # Create an account type list, with a selection - to be used by the accounts type Choice box

        self.all_account_types = AccountType.get_all()

        try:
            acc_type_selection = self.all_account_types.index(self.account.acc_type)
        except Exception:
            acc_type_selection = 0

        self.type_text=wx.StaticText(self, -1, "Account type:")
        fields_sizer.Add(self.type_text, 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.acc_type_field = wx.Choice(self, -1, size=(200, vs_cfg["BUTTON_HEIGHT"]),
                                         choices=[acc_type.name for acc_type in self.all_account_types ])
        self.acc_type_field.SetToolTipString("Set the account type")
        self.acc_type_field.SetSelection(acc_type_selection)
        fields_sizer.Add(self.acc_type_field, 0, wx.ALIGN_LEFT)

        # The Account name label and field
        name_text=wx.StaticText(self, -1, "Account name")
        fields_sizer.Add(name_text, 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.name_field = wx.TextCtrl(self, -1, self.account.name, size=(200, vs_cfg["BUTTON_HEIGHT"]))
        self.name_field.SetToolTipString("The name of this account (20 characters maximum)")
        self.name_field.SetMaxLength(20)
        fields_sizer.Add(self.name_field, 0, wx.ALIGN_LEFT)

        # The account description label and field
        description_text=wx.StaticText(self, -1, "Account description")
        fields_sizer.Add(description_text, 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.description_field = wx.TextCtrl(self, -1, self.account.description, size=(300, vs_cfg["BUTTON_HEIGHT"]))
        self.description_field.SetToolTipString("The description of this account (50 characters maximum)")
        self.description_field.SetMaxLength(50)
        fields_sizer.Add(self.description_field, 0, wx.ALIGN_LEFT)

        sizer.Add(fields_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # -- Apply - Cancel -- buttons
        button_size = (100, vs_cfg["BUTTON_HEIGHT"])

        # Create a horizontal button_sizer to hold the buttons
        button_sizer=wx.BoxSizer(wx.HORIZONTAL)

        # Apply button
        self.apply_button = wx.Button(self, wx.ID_APPLY, size=button_size)
        self.apply_button.SetToolTipString("Click to apply changes.")
        button_sizer.Add(self.apply_button, 0, wx.RIGHT, 20)
        
        # Cancel button
        self.cancel_button = wx.Button(self, wx.ID_CANCEL, size=button_size)
        self.cancel_button.SetToolTipString("Click to cancel changes.")
        button_sizer.Add(self.cancel_button, 0, wx.LEFT, 20)

        # Add button_sizer to the main sizer
        sizer.Add(button_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 40)

        # Set the size to the panel
        self.SetSizer(sizer)
        sizer.SetSizeHints(self)
        
        # ---------- Bind buttons to handler functions
        
        self.Bind(wx.EVT_BUTTON, self.apply_button_pressed, self.apply_button)
        self.Bind(wx.EVT_BUTTON, self.cancel_button_pressed, self.cancel_button)

    # ----------------- Handler functions

    @writecheck
    @button_wrap
    def apply_button_pressed(self, event):
        """Apply button - check and apply all fields"""

        name=self.name_field.GetValue().strip()

        if not name:
            vs_message(self, "A name must be input", "ERROR")
            return

        # Check for name uniqueness within the folder if the name has changed
        if name != self.account.name:
            if self.account.folder.get_account_with_name(name):
                vs_message(self, "This name is already used in this folder", "ERROR")
                return

            # The name is not already in the folder = ok, so set the name to the new name
            self.account.name = name

        description=self.description_field.GetValue().strip()
        if description != self.account.description:
            self.account.description = description

        # Get the account type selected
        acc_type = self.all_account_types[self.acc_type_field.GetCurrentSelection()]

        if acc_type != self.account.acc_type:
            self.account.acc_type = acc_type

        top_refresh_accounts(self.account)
        self.EndModal(wx.ID_OK)


    def cancel_button_pressed(self, event):
        """ Cancel button - Close parent"""
        self.EndModal(wx.ID_CANCEL)

