####### VATStuff #######
#
# list_account.py  - lists account transactions
#
# Version : 0.7.0
# Date : 20100910
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009, 2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#

""" This is a module in the vatstuff_packages.vs_gui package, and Lists the Account statement

It is run as part of the vatstuff gui interface.
"""

import datetime
import decimal

import wx
import wx.grid

from vatstuff_packages.vs_database.grid_objects import *

from vatstuff_packages.vs_utilities import *

from vatstuff_packages.vs_gui.gui_functions import *
from vatstuff_packages.vs_gui.reports import make_report
from vatstuff_packages.vs_gui.reports.view_transactions import show_transaction_with_view_button


D = decimal.Decimal   # D used to avoid typing



def list_account_frame(account):
    """Show the ListAccountFrame which creates a statement list for the given account"""
    # Check the frame is not already open
    for frm in vs_cfg["TRANSACTION_REFRESH"]:
        if isinstance(frm, ListAccountFrame):
            if frm.account == account:
                frm.Raise()
                return

    # top_frame() is the top frame, which will be the parent
    frm = ListAccountFrame(top_frame(), account)
    frm.Show()
    frm.display_last()


class ListAccountFrame(vsFrame):
    """The ListAccount Frame"""

    def __init__(self, parent, account):
        """Create the frame to show an account list"""

        folder_account_string = account.fullname
        vsFrame.__init__(self, parent, title=folder_account_string)

        # Take a note of the parent
        self.parent = parent
        self.account = account
        
        self.account_grid = AccountGrid(self.account)
        # class defined in vatstuff_packages.vs_database.grid_objects
        # Create a database object of type wx.grid.PyGridTableBase
        # it opens database connections, and gets information for
        # the grid object

        try:
            self.account_grid_rows = self.account_grid.GetNumberRows()
            # Create the ListAccountPanel
            self.list_account_panel = ListAccountPanel(self, self.account, self.account_grid)
        except Exception, e:
            # close database connection, and re-raise the exception
            self.account_grid.CloseConnection()
            raise

        # Get the panel size
        panel_size = self.list_account_panel.GetMinSize()

        # Set minimum equal to the panel
        self.SetMinSize(panel_size)
        self.SetClientSize(panel_size)

        # update the list of windows which need a refresh if
        # a transaction occurs, by adding self
        vs_cfg["TRANSACTION_REFRESH"].append(self)

        self.Bind(wx.EVT_CLOSE, self.onclose)


    def refresh(self):
        "Refreshes the grid"
        new_num_rows = self.account_grid.GetNumberRows()
        if new_num_rows > self.account_grid_rows:
            # More rows are in the database, insert new rows on the grid
            msg = wx.grid.GridTableMessage(self.account_grid,          # The table
                  wx.grid.GRIDTABLE_NOTIFY_ROWS_APPENDED,              # has extra rows
                  new_num_rows - self.account_grid_rows                # the number of rows
                  ) 
            self.list_account_panel.grid.ProcessTableMessage(msg)      # The grid associated with the table
            self.account_grid_rows = new_num_rows                      # record the new number of rows
        elif new_num_rows < self.account_grid_rows:
            # Less rows are in the database, delete rows on the grid
            msg = wx.grid.GridTableMessage(self.account_grid,          # The table
                  wx.grid.GRIDTABLE_NOTIFY_ROWS_DELETED,               # has fewer rows
                  new_num_rows,                                        # delete from row
                  self.account_grid_rows - new_num_rows                # delete the number of rows
                  ) 
            self.list_account_panel.grid.ProcessTableMessage(msg)      # The grid associated with the table
            self.account_grid_rows = new_num_rows                      # record the new number of rows

        # grid associated with table can also be obtained with
        # self.account_grid.GetView().ProcessTableMessage(msg)

        self.list_account_panel.grid.ForceRefresh()


    def display_last(self):
        "Calls the display_last function in the list_account panel"
        self.list_account_panel.display_last()


    def onclose(self, evt):
        "Shutdown the database transaction"
        self.account_grid.CloseConnection()
        # Decrement the refresh list
        vs_cfg["TRANSACTION_REFRESH"].remove(self)
        # and destroy the frame
        self.Destroy()


# ----------------------List an account panel


class ListAccountPanel(wx.Panel):
    """List an account panel"""

    def __init__(self, parent, account, account_grid):
        """Create the panel"""
        wx.Panel.__init__(self, parent)

        # Take a note of the variables
        self.parent = parent
        self.account = account
        self.account_grid = account_grid

        button_size = (100, vs_cfg["BUTTON_HEIGHT"])

        # Create the sizer for the panel
        self.sizer=wx.BoxSizer(wx.VERTICAL)

        self.description_text=wx.StaticText(self, -1, "%s - %s" % (account.fullname, account.description))
        self.sizer.Add(self.description_text, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        help_text=wx.StaticText(self, -1, "Double click on any line to view the related transactions")
        self.sizer.Add(help_text, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # Create a grid
        self.grid = wx.grid.Grid(self, size=(850,200))

        self.grid.SetDefaultColSize(100, True)
        self.grid.SetDefaultRowSize(vs_cfg["BUTTON_HEIGHT"], False)
        self.grid.SetColMinimalAcceptableWidth(100)
        self.grid.EnableEditing(False)

        self.grid.SetDefaultCellAlignment(wx.ALIGN_LEFT, wx.ALIGN_CENTER)

        # Assign the account_grid to the grid
        self.grid.SetTable(self.account_grid, True)

        # Set the description column size
        self.grid.SetColSize(1, 300)

        # Set the colums showing money as right aligned
        attr = wx.grid.GridCellAttr()
        attr.SetAlignment(wx.ALIGN_RIGHT, wx.ALIGN_CENTER)

        self.grid.SetColAttr(2, attr)
        self.grid.SetColAttr(3, attr)
        self.grid.SetColAttr(4, attr)

        self.sizer.Add(self.grid, 1, wx.ALL | wx.ALIGN_CENTER | wx.EXPAND, 20)

        # buttons
        button_sizer=wx.BoxSizer(wx.HORIZONTAL)

        # ------------------- Close button
        self.close_button = wx.Button(self, wx.ID_CLOSE, size=button_size)
        self.close_button.SetToolTipString("Click to close this window")
        button_sizer.Add(self.close_button, 0, wx.RIGHT, 20)

        # ------------------- Report button
        self.report_button = wx.Button(self, label="Report", size=button_size)
        self.report_button.SetToolTipString("Click to generate a report between two dates")
        button_sizer.Add(self.report_button, 0)

        self.sizer.Add(button_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # Set the size to the panel
        self.SetSizer(self.sizer)
        self.sizer.SetSizeHints(self)

        # Bind left mouse double - click
        self.Bind(wx.grid.EVT_GRID_CELL_LEFT_DCLICK, self.show_transaction, self.grid)

        # Bind the buttons
        self.Bind(wx.EVT_BUTTON, self.close_button_pressed, self.close_button)
        self.Bind(wx.EVT_BUTTON, self.report_button_pressed, self.report_button)


    @button_wrap
    def report_button_pressed(self, event):
        """Get the two dates between which the report is generated
           create the statement report
           and then display it in a report frame"""

        help_text = """\
A statement will be generated showing
all account entries between the two dates:"""

        status, from_date, to_date = two_dates(self, "Account Statement", help_text)
        if not status:
            return
        busy_cursor_on()
        wx.Yield()

        filename = proper_filename(self.account.name, "account_report", to_date)
        make_report.make_report_and_frame("statement",
                                          "Statement",
                                          filename,
                                          from_date,
                                          to_date,
                                          account=self.account)


    @button_wrap
    def show_transaction(self, event):
        "On being double clicked, obtain transaction and show it"
        row = event.GetRow()
        # get the transaction for this row
        transaction = Transaction(int(self.account_grid.GetRowLabelValue(row)))
        # Show the transaction in a dialog
        # if this is a vat correction, use the dialog in gui_functions
        if transaction.system == 11:
            show_transaction_dialog(self, transaction)
        else:
            # Show the transaction in a dialog (from vatstuff_packages.vs_gui.reports.view_transactions)
            show_transaction_with_view_button(self, transaction)


    def display_last(self):
        """ Makes the last row visible """
        last_row = self.account_grid.GetNumberRows() - 1
        self.grid.SetGridCursor(last_row, 0)
        self.grid.MakeCellVisible(last_row, 0)


    def close_button_pressed(self, event):
        """Close parent"""
        self.parent.Close()

