####### VATStuff #######
#
# new_account.py  - defines the NewAccount frame
#
# Version : 0.7.0
# Date : 20100925
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#

""" This is a module in the vatstuff_packages.vs_gui package, and defines the NewAccount frame

It is run as part of the vatstuff gui interface.
"""


import datetime
import decimal

import wx

from vatstuff_packages.vs_database.create_objects import create_Account
from vatstuff_packages.vs_database import transactions

from vatstuff_packages.vs_gui.gui_functions import *

from vatstuff_packages.vs_utilities import *

from vatstuff_packages.vs_verify.accounts import new_account_check
from vatstuff_packages.vs_verify import form_objects

D = decimal.Decimal   # D used to avoid typing


def new_account_dialog(parent, folder):
    """Creates a NewAccountDialog"""

    new_account_dialog = NewAccountDialog(parent, folder)

    # If the cursor is busy, unbusy it - as
    # now waiting for a button press
    busy_cursor_off()

    new_account_dialog.ShowModal()
    new_account_dialog.Destroy()


class NewAccountDialog(vsDialog):
    "Add a new account dialog, called with the folder in which to add the account"

    def __init__(self, parent, folder):
        """Create the Dialog"""
        vsDialog.__init__(self, parent, "Add new Account")

        # Create the sizer
        sizer=wx.BoxSizer(wx.VERTICAL)

        # Create the input labels and fields in a grid sizer
        fields_sizer=wx.FlexGridSizer(rows=5, cols=2, hgap=20, vgap=20)

        # -------------------The Account Type Choice box

        # Create the list of account types
        self.all_account_types = AccountType.get_all()

        # Account Type choice box
        self.type_text=wx.StaticText(self, -1, "Account Type")
        fields_sizer.Add(self.type_text, 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.type_field = wx.Choice(self, -1, size=(200, vs_cfg["BUTTON_HEIGHT"]),
                                    choices=[acc_type.name for acc_type in self.all_account_types ])
        self.type_field.SetToolTipString("Set the account as one of these types")
        self.type_field.SetSelection(0)
        fields_sizer.Add(self.type_field, 0, wx.ALIGN_LEFT)

        # --------------------- The Folder Choice box

        # Get the list of non-system folders, to be used in the folders Choice option
        self.folders=Folder.get_non_system()

        # If all folders are system, then this is an error
        if not self.folders:
            raise VS_ERROR, "Cannot create account - no user folders are available"

        # Set the selected item in the choice as the folder given
        try:
            folder_selection = self.folders.index(folder)
        except Exception:
            folder_selection = 0

        # Folder choice box
        self.folder_text=wx.StaticText(self, -1, "Account is a member of folder")
        fields_sizer.Add(self.folder_text, 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.folder_field = wx.Choice(self, -1, size=(200, vs_cfg["BUTTON_HEIGHT"]),
                                      choices=[ f.name for f in self.folders ])
        self.folder_field.SetToolTipString("Set folder, which this account will be shown under")
        self.folder_field.SetSelection(folder_selection)
        fields_sizer.Add(self.folder_field, 0, wx.ALIGN_LEFT)


        # --------------------- The Account name label and field

        # Account Name
        name_text=wx.StaticText(self, -1, "Account name")
        fields_sizer.Add(name_text, 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.name_field = wx.TextCtrl(self, -1, "Account name", size=(200, vs_cfg["BUTTON_HEIGHT"]))
        self.name_field.SetToolTipString("The name of this account (20 characters maximum)")
        self.name_field.SetMaxLength(20)

        fields_sizer.Add(self.name_field, 0, wx.ALIGN_LEFT)

        # --------------------- The account description label and field

        # Description
        description_text=wx.StaticText(self, -1, "Account description")
        fields_sizer.Add(description_text, 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.description_field = wx.TextCtrl(self, -1, "Account description", size=(300, vs_cfg["BUTTON_HEIGHT"]))
        self.description_field.SetToolTipString("The description of this account (50 characters maximum)")
        self.description_field.SetMaxLength(50)

        fields_sizer.Add(self.description_field, 0, wx.ALIGN_LEFT)

        # --------------------- Set the opening balance and date

        # opening Balance text
        self.balance_text=wx.StaticText(self, -1, "opening balance")
        fields_sizer.Add(self.balance_text, 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        # create a horizontal box sizer for the two fields
        balance_sizer=wx.BoxSizer(wx.HORIZONTAL)

        # opening Balance field
        self.balance_field = wx.TextCtrl(self, -1, vs_cfg["SYMBOL_ZERO_STRING"], size=(100, vs_cfg["BUTTON_HEIGHT"]))
        self.balance_field.SetToolTipString("Set the initial amount currently in this account")
        self.balance_field.SetMaxLength(10)
        balance_sizer.Add(self.balance_field, 0, wx.ALIGN_LEFT)

        # opening Balance date
        self.balance_date_field = wx.DatePickerCtrl(self,
                                                    size=(150, vs_cfg["BUTTON_HEIGHT"]),
                                                    style=wx.DP_DROPDOWN | wx.DP_SHOWCENTURY)
        self.balance_date_field.SetToolTipString("Set the date of the opening balance")
        balance_sizer.Add(self.balance_date_field, 0, wx.ALIGN_LEFT | wx.LEFT, 20)

        fields_sizer.Add(balance_sizer, 0, wx.ALIGN_LEFT)

        sizer.Add(fields_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # --------------------- Apply - Cancel -- buttons

        button_size = (100, vs_cfg["BUTTON_HEIGHT"])

        # Create a horizontal button_sizer to hold the buttons
        button_sizer=wx.BoxSizer(wx.HORIZONTAL)

        # Apply button
        self.apply_button = wx.Button(self, wx.ID_APPLY, size=button_size)
        self.apply_button.SetToolTipString("Click to apply changes.")
        button_sizer.Add(self.apply_button, 0, wx.RIGHT, 40)

        # Cancel button
        self.cancel_button = wx.Button(self, wx.ID_CANCEL, size=button_size)
        self.cancel_button.SetToolTipString("Click to cancel changes.")
        button_sizer.Add(self.cancel_button, 0, wx.LEFT)

        # Add button_sizer to the main sizer
        sizer.Add(button_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 40)

        # Set the size to the panel
        self.SetSizer(sizer)
        sizer.SetSizeHints(self)
        
        # ---------- Bind buttons to handler functions
        
        self.Bind(wx.EVT_BUTTON, self.apply_button_pressed, self.apply_button)
        self.Bind(wx.EVT_BUTTON, self.cancel_button_pressed, self.cancel_button)


    # ----------------- Handler functions

    @writecheck
    @button_wrap
    def apply_button_pressed(self, event):
        """Apply button - check and apply all fields"""

        name=self.name_field.GetValue()
        name=name.strip()

        description=self.description_field.GetValue()
        description=description.strip()
    
        # Get the account type selected
        account_type_selected = self.all_account_types[self.type_field.GetCurrentSelection()]
            
        # Get the folder selected
        folder_selected = self.folders[self.folder_field.GetCurrentSelection()]

        # Get the datestamp
        datestamp = wxdate_to_date(self.balance_date_field.GetValue())

        # Get the new balance
        status, balance = string_to_decimal(self.balance_field.GetValue())
        if not status:
            vs_message(self, "Invalid balance", "ERROR")
            return

        # Check input details are ok
        status, message = new_account_check(name, folder_selected, balance, datestamp)
        if not status:
            vs_message(self, message, "ERROR")
            return

        # Create the account
        self.account = create_Account(folder_selected, name, description, 0, account_type_selected)
        if not self.account:
            vs_message(self, "Unable to create the account", "ERROR")
            return

        # Create the opening balance transaction
        if balance:
            status, message = self.set_opening_balance(balance, datestamp)
            if not status:
                top_refresh_accounts(self.account)
                vs_message(self, "Failed to implement opening balance\n%s" % message, "DONE")
                self.EndModal(wx.ID_OK)

        top_refresh_accounts(self.account)
        vs_message(self, "New account created", "DONE")
        self.EndModal(wx.ID_OK)


    def cancel_button_pressed(self, event):
        """ Cancel button - Close the dialog"""
        self.EndModal(wx.ID_CANCEL)


    def set_opening_balance(self, balance, datestamp):
        """Creates a transaction to set the opening balance"""

        # Store the opening transaction in a TransactionForm object
        transaction = form_objects.TransactionForm()

        transaction.description = "%s opening balance" % self.account.fullname
        transaction.datestamp = datestamp
        # This is an opening balance transaction (system type 1)
        transaction.system = 1

        # Get the system opening balances account - this being the account with system = 2
        op_account = get_account_by_system(2)

        # If balance is greater than zero, then debit the account by the
        # balance, otherwise credit it
        if balance > D(0):
            transaction.add(self.account, debit=balance)
            transaction.add(op_account, credit=balance)
        else:
            balance = abs(balance)
            transaction.add(self.account, credit=balance)
            transaction.add(op_account, debit=balance)

        status, message = transaction.check()
        if not status:
            return False, message

        # In this case do not confirm the transaction - as doing so does not add any value

        # Must now implement the opening balance transaction
        transactions.implement(None, transaction)

        info_log("opening balance set")
        return True, None



