####### VATStuff #######
#
# tree.py  - defines the tree of accounts
#
# Version : 0.6
# Date : 20100810
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#

""" This is a module in the vatstuff_packages.vs_gui package, and defines the accounts tree

It is run as part of the vatstuff gui interface.
"""

import decimal

import wx
import wx.gizmos






from vatstuff_packages.vs_utilities import *

from vatstuff_packages.vs_gui.gui_functions import *
from vatstuff_packages.vs_gui.accounts import list_account

D = decimal.Decimal   # D used to avoid typing


class AccountsTree(wx.gizmos.TreeListCtrl):
    "The tree list showing folders, accounts, their names, descriptions and balance"
    
    def __init__(self, parent):
        
        wx.gizmos.TreeListCtrl.__init__(self, parent, style =
                                        #wx.TR_DEFAULT_STYLE
                                        #| wx.TR_HAS_BUTTONS
                                        #| wx.TR_TWIST_BUTTONS
                                        #| wx.TR_ROW_LINES
                                         wx.TR_COLUMN_LINES
                                        #| wx.TR_NO_LINES 
                                        | wx.TR_FULL_ROW_HIGHLIGHT
                                        | wx.TR_HIDE_ROOT
                                       )

        self.parent = parent

        isz = (16,16)
        il = wx.ImageList(isz[0], isz[1])
        self.fldridx     = il.Add(wx.ArtProvider_GetBitmap(wx.ART_FOLDER,      wx.ART_OTHER, isz))
        self.fldropenidx = il.Add(wx.ArtProvider_GetBitmap(wx.ART_FILE_OPEN,   wx.ART_OTHER, isz))
        self.fileidx     = il.Add(wx.ArtProvider_GetBitmap(wx.ART_NORMAL_FILE, wx.ART_OTHER, isz))

        self.SetImageList(il)
        self.il = il

        # create some columns
        self.AddColumn("Accounts")
        self.AddColumn("Description")
        self.AddColumn("Balance", flag = wx.ALIGN_RIGHT)
        self.AddColumn("Type", flag = wx.ALIGN_RIGHT)
        self.SetMainColumn(0) # the one with the tree in it...
        self.SetColumnWidth(0, 200)
        self.SetColumnWidth(1, 300)  # Description

        # Create the root
        self.root = self.AddRoot("")
        self.SetItemImage(self.root, self.fldridx, which = wx.TreeItemIcon_Normal)
        self.SetItemImage(self.root, self.fldropenidx, which = wx.TreeItemIcon_Expanded)
        self.SetItemImage(self.root, self.fldropenidx, which = wx.TreeItemIcon_SelectedExpanded)

        # Create the tree by calling refresh, which builds a new tree
        # current_item is False, so selects the first folder
        self.refresh(item=None, current_item=False)

        # Set the folder open and close event bindings, this records
        # the folder expanded flag in the database, so the folder open,closed
        # status can be retained.
        self.Bind(wx.EVT_TREE_ITEM_COLLAPSED, self.item_collapsed, self)
        self.Bind(wx.EVT_TREE_ITEM_EXPANDED, self.item_expanded, self)

        # If an account is double clicked in the tree, get it to be listed
        self.Bind(wx.EVT_TREE_ITEM_ACTIVATED, self.list_pressed, self)


    def item_collapsed(self, evt):
        "Sets folder expanded flag to zero when the folder is collapsed"
        try:
            if not vs_cfg["WRITE_ACCESS"]:
                return
            item=self.GetItemPyData(evt.GetItem())
            # The item is either a Folder or an Account
            if isinstance(item, Folder):
                # Set the folder collapsed status into the database
                item.expanded = 0
        except Exception, e:
            error_message("Failure recording tree state in the database", e)


    def item_expanded(self, evt):
        "Sets folder expanded flag to one when the folder is expanded"
        try:
            if not vs_cfg["WRITE_ACCESS"]:
                return
            item=self.GetItemPyData(evt.GetItem())
            # The item is either a Folder or an Account
            if isinstance(item, Folder):
                # Set the folder expanded status into the database
                item.expanded = 1
        except Exception, e:
            error_message("Failure recording tree state in the database", e)


    @button_wrap
    def list_pressed(self, evt):
        "List an account - called by a double key press on an account"
        item=self.GetItemPyData(evt.GetItem())
        
        # The item is either a Folder or an Account
        if not item:
            # Nothing is selected, just return
            return
        # If its a folder, simply make sure its expanded
        if isinstance(item, Folder):
            # Toggle the folder
            if item.expanded:
                self.Collapse(evt.GetItem())
            else:
                self.Expand(evt.GetItem())
        elif isinstance(item, Account):
            # show the account statement
            list_account.list_account_frame(item)
        else:
            raise VS_ERROR, "item not recognised"


    def refresh(self, item=None, current_item=True):
        """Deletes current tree, re-assigns folders and accounts to tree items,
           and select the item,
           if no item given, and current item is False, selects first folder
           if no item given, and current item is True, leaves current selection"""

        # item is either a Folder or an Account
        # and defines the item to be shown expanded and selected

        if not item and current_item:
            try:
                item=self.GetItemPyData(self.GetSelection())
            except Exception:
                item = None

        if not item:
            # Initially, get the first folder, to be shown selected on the tree
            item = Folder.get_first()
            folder_selected = True
        else:
            if isinstance(item, Folder):
                folder_selected = True
            else:
                folder_selected = False

        selected_item = None
        item_id = item.id

        # delete all children beneath the root
        self.DeleteChildren(self.root)

        # Read the database
        all_folders=Folder.get_list()
        # A list of all the folders as a list of lists
        # [name, description, expanded, id]

        # Create the folders
        for folder in all_folders:
            fdr = self.AppendItem(self.root, folder[0])          # folder name
            self.SetItemText(fdr, folder[1], 1)           # folder description
            self.SetItemImage(fdr, self.fldridx, which = wx.TreeItemIcon_Normal)
            self.SetItemImage(fdr, self.fldropenidx, which = wx.TreeItemIcon_Expanded)
            self.SetItemImage(fdr, self.fldropenidx, which = wx.TreeItemIcon_SelectedExpanded)
            # set this item with the folder
            folder_object = Folder(folder[3])
            self.SetItemPyData(fdr, folder_object)
            if (not selected_item) and folder_selected and (item_id == folder[3]):
                selected_item = fdr

            # for each folder, get its accounts information as a list of lists
            # of [name, description, balance, accountype name, id]
            account_list = folder_object.get_accounts_list()
            
            for account in account_list:
                acc = self.AppendItem(fdr,  account[0])                     # Account name
                self.SetItemText(acc, account[1], 1)                        # Account description
                self.SetItemText(acc, to_currency_string(account[2]), 2)    # Account balance in pounds
                self.SetItemText(acc, account[3],  3)                       # Account type name
                self.SetItemImage(acc, self.fileidx, which = wx.TreeItemIcon_Normal)
                # set this item with the account object
                self.SetItemPyData(acc, Account(account[4]))
                if (not selected_item) and (not folder_selected) and (item_id == account[4]):
                    selected_item = acc

            # Is this folder expanded
            if folder[2]:
                self.Expand(fdr)

        self.Expand(self.root)

        if selected_item != None:
            parent = self.GetItemParent(selected_item)
            self.Expand(parent)
            self.SelectItem(selected_item)


