####### VATStuff #######
#
# add_item.py  - Creates a table to view and edit items for purchase or sale
#
# Version : 0.6
# Date : 20100728
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#

"""Creates a table to view and edit items for purchase or sale

It is run as part of the vatstuff gui interface.
"""

import sys
import decimal
import datetime
import csv
import os.path

import wx
import wx.grid





from vatstuff_packages.vs_database.create_objects import create_PurchaseItem, create_SellItem
from vatstuff_packages.vs_database import report_functions

from vatstuff_packages.vs_utilities import *

from vatstuff_packages.vs_gui.gui_functions import *
from vatstuff_packages.vs_gui.reports import make_report

D = decimal.Decimal   # D used to avoid typing

if vs_cfg["REPORTLABS_AVAILABLE"]:
    import vatstuff_packages.vs_pdf.page_setup
    import vatstuff_packages.vs_pdf.create_report



def add_item_dialog(parent, item_type):
    "Called when the add purchase or sale item button is pressed"
    add_item_dialog = AddItemDialog(parent, item_type)

    # If the cursor is busy, unbusy it - as
    # now waiting for a button press
    busy_cursor_off()

    add_item_dialog.ShowModal()
    add_item_dialog.Destroy()


class AddItemDialog(vsDialog):
    "Called when the add purchase or sale item button is pressed"

    def __init__(self, parent, item_type):
        """Create the Dialog, item_type is either purchase or sale"""

        if item_type == "purchase":
            frame_title = "Items for purchase"
        else:
            frame_title = "Items for sale"

        vsDialog.__init__(self, parent, frame_title)

        self.item_type = item_type

        if self.item_type == "purchase":
            self.all_items = PurchaseItem.get_all()
        else:
            self.all_items = SellItem.get_all()

        # create a report object which can be exported to pdf or csv files
        self.report = report_functions.create_report( "items", None, None, item_type = self.item_type, all_items = self.all_items)

        button_size = (100, vs_cfg["BUTTON_HEIGHT"])

        # Create a vertical sizer to hold everything
        sizer=wx.BoxSizer(wx.VERTICAL)

        if item_type == "purchase":
            help_text="""This table holds items you frequently purchase, click on any field to edit it.
These items will be available as a drop down choice when you create purchase orders"""
        else:
            help_text="""This table holds items you frequently sell, click on any field to edit it.
These items will be available as a drop down choice when you create invoices"""

        sizer.Add(wx.StaticText(self, -1, help_text), 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # ---------------------- Create the items table

        self.items_table = ItemsTable(self, self.item_type, self.all_items)
        sizer.Add(self.items_table, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # ---------------------- Apply - Cancel - Report -- buttons

        # Create a horizontal button_sizer to hold the buttons
        button_sizer=wx.BoxSizer(wx.HORIZONTAL)

        # Apply button
        self.apply_button = wx.Button(self, wx.ID_APPLY, size=button_size)
        self.apply_button.SetToolTipString("Click to apply changes.")
        button_sizer.Add(self.apply_button, 0, wx.RIGHT, 40)
        # Start off with apply button disabled - enabled by any changes
        self.apply_button.Enable(False)

        # Close button
        self.close_button = wx.Button(self, wx.ID_CLOSE, size=button_size)
        self.close_button.SetToolTipString("Click to close.")
        button_sizer.Add(self.close_button, 0, wx.RIGHT, 40)

        # page setup button
        self.page_button = wx.Button(self, label="Page setup", size=button_size)
        self.page_button.SetToolTipString("Click to set up the pdf page size and orientation")   
        button_sizer.Add(self.page_button, 0, wx.RIGHT, 40)

        # pdf button
        self.pdf_button = wx.Button(self, label="Create pdf", size=button_size)
        self.pdf_button.SetToolTipString("Click to create a pdf file of the item list.")
        button_sizer.Add(self.pdf_button, 0, wx.RIGHT, 40)

        # csv button
        self.csv_button = wx.Button(self, label="CSV File", size=button_size)
        self.csv_button.SetToolTipString("Click to export the item list to a csv file.")
        button_sizer.Add(self.csv_button, 0)


        # Add button_sizer to the main sizer
        sizer.Add(button_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # Set the size to the panel
        self.SetSizer(sizer)
        sizer.SetSizeHints(self)

        # ---------- Bind buttons to handler functions
        
        self.Bind(wx.EVT_BUTTON, self.apply_button_pressed, self.apply_button)
        self.Bind(wx.EVT_BUTTON, self.close_button_pressed, self.close_button)
        self.Bind(wx.EVT_BUTTON, self.page_button_pressed, self.page_button)
        self.Bind(wx.EVT_BUTTON, self.pdf_button_pressed, self.pdf_button)
        self.Bind(wx.EVT_BUTTON, self.csv_button_pressed, self.csv_button)



    # ----------------- Handler functions

    @writecheck
    @button_wrap
    def apply_button_pressed(self, event):
        """Apply button - check and apply all fields"""
        status, message = self.items_table.check_fields()
        if not status:
            vs_message(self, message, "ERROR")
            return
        if self.item_type == "purchase":
            self.all_items = PurchaseItem.get_all()
        else:
            self.all_items = SellItem.get_all()
        self.report = report_functions.create_report( "items", None, None, item_type = self.item_type, all_items = self.all_items)
        self.items_table.refresh(self.all_items)
        self.pdf_button.Enable(True)
        self.csv_button.Enable(True)
        self.apply_button.Enable(False)
        vs_message(self, message, "INFORMATION")


    def close_button_pressed(self, event):
        """Close button - Close down"""
        if not self.apply_button.IsEnabled():
            self.EndModal(wx.ID_OK)
            return
        if vs_yesno(self, """\
Closing item list: are you sure?
Any changes will be lost unless you press Apply""", "Closing list"):
            self.EndModal(wx.ID_OK)


    @button_wrap
    def page_button_pressed(self, event):
        "page button pressed, setup the pdf page size and orientation"
        vatstuff_packages.vs_pdf.page_setup.page_setup_dialog(self)


    @button_wrap
    def pdf_button_pressed(self, event):
        "Create a pdf file from the report"

        if not vs_cfg["REPORTLABS_AVAILABLE"]:
            # no pdf capability
            vs_message(self, "Function unavailable - the reportlab module is not installed",
                             "ERROR")
            return

        if self.item_type == "purchase":
            filename = "purchase_items_list%s.pdf" % datetime.date.today().isoformat()
        else:
            filename = "product_list%s.pdf" % datetime.date.today().isoformat()

        if not vatstuff_packages.vs_pdf.create_report.create_pdf(self, filename, self.report):
            vs_message(self, "Unable to create the pdf file", "ERROR")


    @button_wrap
    def csv_button_pressed(self, event):
        """Export the report to a csv file"""

        if self.item_type == "purchase":
            filename = "purchase_items_list%s.csv" % datetime.date.today().isoformat()
        else:
            filename = "product_list%s.csv" % datetime.date.today().isoformat()

        fp=None
        try:
            # Create the save file dialog.
            status, path = save_file_as(self, 'csv', vs_cfg["EXPORT_FOLDER"], filename)
            if not status:
                return

            # open the csv file
            fp=open(path,"wb")
            # save file in csv format
            writer = csv.writer(fp, dialect="excel")
            for row in self.report.render_to_csv():
                writer.writerow(row)
        finally:
            if fp: fp.close()

        # save the directory to the database and to vs_cfg["EXPORT_FOLDER"]
        directory = os.path.dirname(path)
        if directory != vs_cfg["EXPORT_FOLDER"]:
            set_directory("EXPORT_FOLDER", directory)
        info_log("CSV file created: %s" % path)
        vs_message(self, "Your file has been saved", "Saved")


class ItemsTable(wx.grid.Grid):

    def __init__(self, parent, item_type, all_items=[]):
        wx.grid.Grid.__init__(self, parent, size=(900,200), style=wx.WANTS_CHARS)

        self.parent = parent
        self.item_type = item_type
        self.all_items = all_items

        vat_scheme = VATScheme()
        if vat_scheme.method == "NOT_REGISTERED" and self.item_type == "sale":
            # if selling items, but not vat registered, do not show vat
            self.show_vat = False
        else:
            self.show_vat = True
            
        num_rows = len(self.all_items) + 5   # Make 5 'blank' rows
        if self.show_vat:
            num_cols = 4
        else:
            num_cols = 3

        # VAT is a choice of VAT symbols

        vat_rates = VATRate.get_all()

        symbol_selection = []
        for rate in vat_rates:
            symbol_selection.append(rate.symbol + " - " + rate.name)

        self.CreateGrid(num_rows, num_cols)

        # create column labels
        self.SetColLabelValue(0, "Description")
        self.SetColLabelValue(1, "Cost")
        if self.show_vat:
            self.SetColLabelValue(2, "VAT")
            self.SetColLabelValue(3, "Notes")
            self.vat_editor = wx.grid.GridCellChoiceEditor(choices=symbol_selection, allowOthers=False)
        else:
            self.SetColLabelValue(2, "Notes")

        self.SetDefaultRowSize(vs_cfg["BUTTON_HEIGHT"], False)
        self.SetDefaultCellAlignment(wx.ALIGN_LEFT, wx.ALIGN_CENTER)

        # Set the colums showing money as right aligned
        attr = wx.grid.GridCellAttr()
        attr.SetAlignment(wx.ALIGN_RIGHT, wx.ALIGN_CENTER)

        self.SetColAttr(1, attr)

        for key, data in enumerate(self.all_items):
            self.SetCellValue(key, 0, data.description)
            self.SetCellValue(key, 1, to_currency_string(data.cost))
            if self.show_vat:
                if data.vatrate == None:
                    self.SetCellValue(key, 2, "")
                else: 
                    self.SetCellValue(key, 2, data.vatsymbol + " - " + data.vatrate.name)
                self.SetCellValue(key, 3, data.notes)
            else:
                self.SetCellValue(key, 2, data.notes)

        if self.show_vat:
            for row in range(num_rows):
                self.SetCellEditor(row, 2, self.vat_editor)

        self.SetColSize(0, 350)
        self.SetColSize(1, 150)
        self.SetColSize(2, 150)
        if self.show_vat:
            self.SetColSize(3, 150)

        self.SetColMinimalAcceptableWidth(100)
        self.SetRowLabelSize(50)

        # When an item is edited, add a further row to ensure there are always blank rows
        # displayed:

        self.Bind(wx.grid.EVT_GRID_CELL_CHANGE, self.addarow)


    def refresh(self, all_items):
        "Get all items again, and re-fill the table"

        self.all_items = all_items

        number_of_items = len(self.all_items)
        current_number_of_rows = self.GetNumberRows()

        # If everything is working ok, this must be true
        assert number_of_items < current_number_of_rows
        
        # clear the grid
        self.ClearGrid()

        for key, data in enumerate(self.all_items):
            self.SetCellValue(key, 0, data.description)
            self.SetCellValue(key, 1, to_currency_string(data.cost))
            if self.show_vat:
                if data.vatrate == None:
                    self.SetCellValue(key, 2, "")
                else: 
                    self.SetCellValue(key, 2, data.vatsymbol + " - " + data.vatrate.name)
                self.SetCellValue(key, 3, data.notes)
            else:
                self.SetCellValue(key, 2, data.notes)


    def addarow(self, evt):
        """This keeps some blank rows at the bottom available for editing
           if the user starts inserting data in the last four rows - 
           add a further row"""

        try:
            # as an item has been edited, enable the apply button
            self.parent.apply_button.Enable(True)
            # and disable the csv and pdf buttons
            self.parent.pdf_button.Enable(False)
            self.parent.csv_button.Enable(False)

            current_number_of_rows = self.GetNumberRows()
            if evt.GetRow()+4 > current_number_of_rows:
                self.AppendRows(numRows=1)
                if self.show_vat:
                    self.SetCellEditor(current_number_of_rows, 2, self.vat_editor)
        except Exception, e:
            error_message("Failure in editing an item row", e)


    def check_fields(self):
        """Read and check the items on the table
           returns status, message"""

        table = []
        field_number = self.GetNumberRows()
        for row in range(0, field_number):
            row_items = []
            for col in range(0,4):
                if not self.show_vat and col == 2:
                    # Get the VAT symbol N, for Not Registered
                    row_items.append('N')
                    continue
                if not self.show_vat and col == 3:
                    # get Notes from column 2
                    item= self.GetCellValue(row, 2)
                    row_items.append(item.strip())
                    continue
                # get VAT from col 2 and notes from col 3
                item= self.GetCellValue(row, col)
                row_items.append(item.strip())
            table.append(row_items)

        # table is now the list of lists of items.

        # Check each item

        for key, data in enumerate(table):            # data[0] = description
            if data[0] == "":                         # data[1] = cost
                continue                              # data[2] = vat symbol
                                                      # data[3] = notes

            if len(data[0]) > 100:
                return False, """Description too long (100 characters max) at line %s
Your changes have not been saved""" % (key+1,)

            status, value = string_to_decimal(data[1])
            if (not status) or (value < D(0)):
                return False, """Cost value incorrect at line %s
Your changes have not been saved""" % (key+1,)

            # substitute the read string for its decimal value
            table[key][1] = value

            if (not data[2]) or (not data[2][0].isalpha()) or (not data[2][0].isupper()):
                return False, """VAT SYMBOL at line %s incorrect
Your changes have not been saved""" % (key+1,)

            # substitute the VAT string for its symbol only
            if self.show_vat:
                table[key][2] = data[2][0]

            if len(data[3]) > 100:
                return False, """Notes too long (100 characters max) at line %s
Your changes have not been saved""" % (key+1,)

        # Check for duplicate descriptions

        for key, data in enumerate(table):            # data[0] = description
            if data[0] == "":
                continue       
            for key2, data2 in enumerate(table):
                if (key != key2) and (data[0] == data2[0]):
                    return False, """Duplicate descriptions at lines %s and %s 
Your changes have not been saved""" % (key+1, key2+1)


        # Update or delete any of the items which need changing

        change = False

        for key, data in enumerate(self.all_items):
            if table[key][0] == "":
                # delete this item
                data.delete()
                change = True
                continue

            if table[key][0] != data.description:
                data.description = table[key][0]
                change = True
            if table[key][1] != data.cost:
                data.cost = table[key][1]
                change = True
            if table[key][2] != data.vatsymbol:
                data.vatsymbol = table[key][2]
                change = True
            if table[key][3] != data.notes:
                data.notes = table[key][3]
                change = True


        # And create new items

        for key in range(len(self.all_items), field_number):
            if table[key][0]:
                # description exists, so create either a PurchaseItem or a SellItem

                if self.item_type == "purchase":
                    create_PurchaseItem(description     = table[key][0],
                                        cost            = table[key][1],
                                        vatrate         = VATRate.from_symbol(table[key][2]),
                                        notes           = table[key][3]) 
                else:
                    create_SellItem(description     = table[key][0],
                                    cost            = table[key][1],
                                    vatrate         = VATRate.from_symbol(table[key][2]),
                                    notes           = table[key][3])
                change = True

        if change:
            info_log("Item changes have been saved")
            return True, "Changes have been saved"
        else:
            return True, "No changes have been implemented"







