####### VATStuff #######
#
# address_panel.py  - A panel containing address fields
#
# Version : 0.7.0
# Date : 20100910
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#


"""Defines AddressPanel - A panel containing address fields
used when inputting and editing company, supplier or customer addresses
Also defines AddressSpin - a panel showing a non-editable address with spin buttons.
"""

import wx

from vatstuff_packages.vs_utilities import *

from vatstuff_packages.vs_gui.gui_functions import *






from vatstuff_packages.vs_verify.form_objects import AddressForm


# ----------------------AddressPanel

class AddressPanel(wx.Panel):
    """A panel containing address fields, called with a name, a list of AddressForm objects,
       A flag mask dictionary; element is 0 if flag is disabled, 1 if enabled,
       A flag defaults dictionary; 0 if unticked, 1 if ticked
       If buttons = True, then next previous, new delete buttons are shown
       otherwise they are not shown and only a single address is editable"""

    def __init__(self, parent, name,
                       address_list = [],
                       flag_mask = None,
                       flag_defaults = None,
                       buttons = True):
        """Create the panel"""
        wx.Panel.__init__(self, parent)

        button_size = (200, vs_cfg["BUTTON_HEIGHT"])

        # Take a note of the variables
        self.name = name
        self.address_list = address_list
        if not flag_mask:
            flag_mask = { 'reg_office'     : 0,
                          'delivery_point' : 0,
                          'invoice_point'  : 0,
                          'order_point'    : 0,
                          'payment_point'  : 0 }

        if not flag_defaults:
            self.flag_defaults = { 'reg_office'     : 0,
                                   'delivery_point' : 1,
                                   'invoice_point'  : 1,
                                   'order_point'    : 1,
                                   'payment_point'  : 1 }
        else:
            self.flag_defaults = flag_defaults

        # Create a list of deleted addresses
        self.deleted_addresses = []

        # First address to be displayed
        if not self.address_list:
            # Create new address as an AddressForm object, with default flags
            self.address = AddressForm(**self.flag_defaults)
            self.address_list = [self.address]
        else:
            self.address = self.address_list[0]

        self.address_index = 0

        # Create some checkboxes

        self.reg_office = wx.CheckBox(self, -1, "Registered Office")
        self.reg_office.SetToolTipString("Tick if this is the companies registered office")
        if not flag_mask['reg_office']:
            self.reg_office.Enable(False)

        self.delivery_point = wx.CheckBox(self, -1, "Deliveries can be sent here")
        self.delivery_point.SetToolTipString("Tick if deliveries can be sent here")
        if not flag_mask['delivery_point']:
            self.delivery_point.Enable(False)

        self.invoice_point = wx.CheckBox(self, -1, "Invoices can be sent here")
        self.invoice_point.SetToolTipString("Tick if invoices can be sent here")
        if not flag_mask['invoice_point']:
            self.invoice_point.Enable(False)

        self.order_point = wx.CheckBox(self, -1, "Orders can be sent here")
        self.order_point.SetToolTipString("Tick if orders can be sent here")
        if not flag_mask['order_point']:
            self.order_point.Enable(False)

        self.payment_point = wx.CheckBox(self, -1, "Payments can be sent here")
        self.payment_point.SetToolTipString("Tick if payments can be sent here")
        if not flag_mask['payment_point']:
            self.payment_point.Enable(False)

        # Put everything in a Vertical sizer
        sizer=wx.BoxSizer(wx.VERTICAL)

        # Create fields and descriptions in a grid sizer
        field_sizer=wx.FlexGridSizer(rows=7, cols=3, hgap=40, vgap=5)

        # name
        self.add_name_field = wx.TextCtrl(self, -1, "", size=(200, vs_cfg["BUTTON_HEIGHT"]))
        self.add_name_field.SetToolTipString("Company name")
        self.add_name_field.SetMaxLength(40)
        field_sizer.Add(self.add_name_field, 0, wx.ALIGN_RIGHT)

        self.add_name_text=wx.StaticText(self, -1, "Company name")
        field_sizer.Add(self.add_name_text, 0, wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        field_sizer.Add(self.reg_office, 0, wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        # line1
        self.add_line1_field = wx.TextCtrl(self, -1, "", size=(200, vs_cfg["BUTTON_HEIGHT"]))
        self.add_line1_field.SetToolTipString("Address 1st line")
        self.add_line1_field.SetMaxLength(40)
        field_sizer.Add(self.add_line1_field, 0, wx.ALIGN_RIGHT)

        self.add_line1_text=wx.StaticText(self, -1, "Address 1st line")
        field_sizer.Add(self.add_line1_text, 0, wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        field_sizer.Add(self.delivery_point, 0, wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        # line2
        self.add_line2_field = wx.TextCtrl(self, -1, "", size=(200, vs_cfg["BUTTON_HEIGHT"]))
        self.add_line2_field.SetToolTipString("Address 2nd line")
        self.add_line2_field.SetMaxLength(40)
        field_sizer.Add(self.add_line2_field, 0, wx.ALIGN_RIGHT)

        self.add_line2_text=wx.StaticText(self, -1, "Address 2nd line")
        field_sizer.Add(self.add_line2_text, 0, wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        field_sizer.Add(self.invoice_point, 0, wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        # City
        self.add_city_field = wx.TextCtrl(self, -1, "", size=(200, vs_cfg["BUTTON_HEIGHT"]))
        self.add_city_field.SetToolTipString("City")
        self.add_city_field.SetMaxLength(40)
        field_sizer.Add(self.add_city_field, 0, wx.ALIGN_RIGHT)

        self.add_city_text=wx.StaticText(self, -1, "City")
        field_sizer.Add(self.add_city_text, 0, wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        field_sizer.Add(self.order_point, 0, wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        # County
        self.add_county_field = wx.TextCtrl(self, -1, "", size=(200, vs_cfg["BUTTON_HEIGHT"]))
        self.add_county_field.SetToolTipString("County")
        self.add_county_field.SetMaxLength(40)
        field_sizer.Add(self.add_county_field, 0, wx.ALIGN_RIGHT)

        self.add_county_text=wx.StaticText(self, -1, "County")
        field_sizer.Add(self.add_county_text, 0, wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        field_sizer.Add(self.payment_point, 0, wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        # Postcode
        self.add_postcode_field = wx.TextCtrl(self, -1, "", size=(200, vs_cfg["BUTTON_HEIGHT"]))
        self.add_postcode_field.SetToolTipString("Postcode")
        self.add_postcode_field.SetMaxLength(40)
        field_sizer.Add(self.add_postcode_field, 0, wx.ALIGN_RIGHT)

        self.add_postcode_text=wx.StaticText(self, -1, "Postcode")
        field_sizer.Add(self.add_postcode_text, 0, wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        field_sizer.Add((0,0),1)

        # Country
        self.add_country_field = wx.TextCtrl(self, -1, "", size=(200, vs_cfg["BUTTON_HEIGHT"]))
        self.add_country_field.SetToolTipString("Country")
        self.add_country_field.SetMaxLength(40)
        field_sizer.Add(self.add_country_field, 0, wx.ALIGN_RIGHT)

        self.add_country_text=wx.StaticText(self, -1, "Country")
        field_sizer.Add(self.add_country_text, 0, wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        field_sizer.Add((0,0),1)

        sizer.Add(field_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # ---------------------- Buttons

        # Create buttons in a grid sizer
        button_sizer=wx.FlexGridSizer(rows=2, cols=2, hgap=20, vgap=20)

        # Next button
        self.next_button = wx.Button(self, label=u"\u2191 Next address", size=button_size)
        self.next_button.SetToolTipString("Click to show next address.")
        button_sizer.Add(self.next_button, 0, wx.RIGHT | wx.ALIGN_CENTER, 40)
        if len(self.address_list)<2:
            self.next_button.Enable(False)

        # New button
        self.new_button = wx.Button(self, label="new address", size=button_size)
        self.new_button.SetToolTipString("Click to add a new address.")
        button_sizer.Add(self.new_button, 0, wx.LEFT | wx.ALIGN_CENTER, 40)

        # Previous button
        self.previous_button = wx.Button(self, label=u"\u2193 Previous address", size=button_size)
        self.previous_button.SetToolTipString("Click to show previous address.")
        button_sizer.Add(self.previous_button, 0, wx.RIGHT | wx.ALIGN_CENTER, 40)
        self.previous_button.Enable(False)

        # Delete button
        self.delete_button = wx.Button(self, label="delete address", size=button_size)
        self.delete_button.SetToolTipString("Delete this address.")
        button_sizer.Add(self.delete_button, 0, wx.LEFT | wx.ALIGN_CENTER, 40)
        if len(self.address_list) < 2:         # only one address available
            self.delete_button.Enable(False)

        sizer.Add(button_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        if not buttons:
            self.new_button.Hide()
            self.delete_button.Hide()
            self.previous_button.Hide()
            self.next_button.Hide()

       
        # Set the size to the panel
        self.SetSizer(sizer)

        self.Fit()

        # Set up the fields
        self.set_address()
        self.set_name_field(self.name)

        self.Bind(wx.EVT_BUTTON, self.new_button_pressed, self.new_button)
        self.Bind(wx.EVT_BUTTON, self.delete_button_pressed, self.delete_button)
        self.Bind(wx.EVT_BUTTON, self.previous_button_pressed, self.previous_button)
        self.Bind(wx.EVT_BUTTON, self.next_button_pressed, self.next_button)

    # Function to set the fields with an address

    def set_address(self):
        "Sets the address fields with address given in self.address"

        self.reg_office.SetValue(False)
        self.delivery_point.SetValue(False)
        self.invoice_point.SetValue(False)
        self.order_point.SetValue(False)
        self.payment_point.SetValue(False)

        if self.address.reg_office:
            self.reg_office.SetValue(True)
        if self.address.delivery_point:
            self.delivery_point.SetValue(True)
        if self.address.invoice_point:
            self.invoice_point.SetValue(True)
        if self.address.order_point:
            self.order_point.SetValue(True)
        if self.address.payment_point:
            self.payment_point.SetValue(True)

        self.add_line1_field.SetValue(self.address.line1)
        self.add_line2_field.SetValue(self.address.line2)
        self.add_city_field.SetValue(self.address.city)
        self.add_county_field.SetValue(self.address.county)
        self.add_postcode_field.SetValue(self.address.postcode)
        self.add_country_field.SetValue(self.address.country)


    def set_name_field(self, name):
        "Sets the address name field"
        if name:
            self.add_name_field.SetValue(name)
        else:
            self.add_name_field.SetValue("")


    # Validator function, checks address fields are ok
    # and returns a list of address lines and a dictionary of flags 

    def address_ok(self):
        """check address fields, returns a status value, and a list
           of address lines and a dictionary of checkbox settings"""

        lines = ["", "", "", "", "", ""]

        # line1
        text = self.add_line1_field.GetValue()
        text=text.strip()
        if len(text) > 40:
            vs_message(self, "The address Line 1 field is incorrect", "ERROR")
            return False, None, None
        lines[0] = text

        # line2
        text = self.add_line2_field.GetValue()
        text=text.strip()
        if len(text) > 40:
            vs_message(self, "The address Line 2 field is incorrect", "ERROR")
            return False, None, None
        lines[1] = text

        # city
        text = self.add_city_field.GetValue()
        text=text.strip()
        if len(text) > 40:
            vs_message(self, "The address City is incorrect", "ERROR")
            return False, None, None
        lines[2] = text

        # County
        text = self.add_county_field.GetValue()
        text=text.strip()
        if len(text) > 40:
            vs_message(self, "The address County is incorrect", "ERROR")
            return False, None, None
        lines[3] = text

        # Postcode
        text = self.add_postcode_field.GetValue()
        text=text.strip()
        if len(text) > 40:
            vs_message(self, "The address Postcode is incorrect", "ERROR")
            return False, None, None
        lines[4] = text

        # Country
        text = self.add_country_field.GetValue()
        text=text.strip()
        if len(text) > 40:
            vs_message(self, "The address Country is incorrect", "ERROR")
            return False, None, None
        lines[5] = text

        # Get checkbox settings
        flags = { 'reg_office'     : 0,
                  'delivery_point' : 0,
                  'invoice_point'  : 0,
                  'order_point'    : 0,
                  'payment_point'  : 0 }

        if self.reg_office.GetValue() :     flags['reg_office']     = 1
        if self.delivery_point.GetValue() : flags['delivery_point'] = 1
        if self.invoice_point.GetValue() :  flags['invoice_point']  = 1
        if self.order_point.GetValue() :    flags['order_point']    = 1
        if self.payment_point.GetValue() :  flags['payment_point']  = 1

        return True, lines, flags


    def get_addresses(self):
        """Gets the company name, and list of addresses,
           returns status, name, address_list and deleted_addresses"""

        # Company name
        name = self.add_name_field.GetValue()
        name = name.strip()
        if len(name) < 2 or len(name) > 40:
            vs_message(self, "The company name field is incorrect", "ERROR")
            return False, None, None, None

        # Currently displayed address
        status, lines, flags = self.address_ok()
        if not status:
            return False, None, None, None

        # Set the current address fields, in self.address_list

        # If this is the registered office, set all other registered office values to zero
        if flags['reg_office']:
            for address in self.address_list:
                address.reg_office = 0

        # update the current address with the field values
        self.address_list[self.address_index].lines = lines
        self.address_list[self.address_index].flags = flags

        # and return with status, name, address_list and deleted_addresses list
        return True, name, self.address_list, self.deleted_addresses


    # ----------------- Handler functions

    @writecheck
    def new_button_pressed(self, event):
        "Insert a new address"
        try:
            # Check current address is ok, before inserting a new address

            # Customer office address
            status, lines, flags = self.address_ok()
            if not status:
                return

            # Set the address fields, both in self.address and self.address_list

            # If this is the registered office, set all other registered office values to zero
            if flags['reg_office']:
                for address in self.address_list:
                    address.reg_office = 0

            # update the current address with the field values
            self.address_list[self.address_index].lines = lines
            self.address_list[self.address_index].flags = flags

            # Can now proceed with inserting a new address

            # Create new address as an AddressForm object, with default flags
            self.address = AddressForm(**self.flag_defaults)

            # self.address_index is the current index position, insert the new item
            # after this position
            self.address_index += 1 

            # insert it into the addresses list
            self.address_list.insert(self.address_index, self.address)

            # Set it into the panel
            self.set_address()

            # Previous button must be enabled as the new item exists after the item
            # at which new was pressed
            self.previous_button.Enable(True)

            if (self.address_index + 1) == len(self.address_list):  # At last address
                self.next_button.Enable(False)
            else:
                self.next_button.Enable(True)

            # New addresses can be deleted
            self.delete_button.Enable(True)

            vs_message(self, "New address inserted, but not saved until the Apply button is pressed.",
                             "INFORMATION")
        except Exception, e:
            error_message("Failure in creating a new address", e)


    @writecheck
    def delete_button_pressed(self, event):
        "Delete the displayed address"
        try:
            # Cannot delete all addresses
            assert len(self.address_list) > 1

            # Record the address in deleted_addresses list if this is a database entry
            if self.address.id > 0:
                self.deleted_addresses.append(self.address)

            if self.address_index == 0:
                new_index = 0
            else:
                new_index = self.address_index -1

            # Remove address
            self.address_list.pop(self.address_index)

            # Display a previous address
            self.address_index = new_index

            self.address = self.address_list[self.address_index]
            self.set_address()

            if len(self.address_list) < 2:         # only one address available
                self.next_button.Enable(False)
                self.previous_button.Enable(False)
                self.delete_button.Enable(False)
                return

            if (self.address_index + 1) == len(self.address_list):  # now at last address
                self.next_button.Enable(False)
            else:
                self.next_button.Enable(True)

            if self.address_index == 0:  # now at first address
                self.previous_button.Enable(False)
            else:
                self.previous_button.Enable(True)
        except Exception, e:
            error_message("Failure in deleting address", e)


    def next_button_pressed(self, event):
        "Checks address fields ok, stores them in the AddressForm object, and goes on to the next"
        try:
            if len(self.address_list)<2:
                self.next_button.Enable(False)
                self.previous_button.Enable(False)
                self.delete_button.Enable(False)
                return

            # Customer office address
            status, lines, flags = self.address_ok()
            if not status:
                return

            # Set the address fields

            # If this is the registered office, set all other registered office values to zero
            if flags['reg_office']:
                for address in self.address_list:
                    address.reg_office = 0

            # update the address with the field values
            self.address_list[self.address_index].lines = lines
            self.address_list[self.address_index].flags = flags

            if (self.address_index +1) == len(self.address_list):    # already at the last address
                self.next_button.Enable(False)
                self.previous_button.Enable(True)
                return

            self.address_index += 1

            # Set the panel to next address
            self.address = self.address_list[self.address_index]
            self.set_address()
            self.previous_button.Enable(True)

            if (self.address_index + 1) == len(self.address_list):  # now at last address
                self.next_button.Enable(False)
            else:
                self.next_button.Enable(True)
        except Exception, e:
            error_message("Failure in moving to next address", e)


    def previous_button_pressed(self, event):
        "Checks address fields ok, stores them in the AddressForm object, and goes on to the previous"
        try:
            if len(self.address_list)<2:
                self.next_button.Enable(False)
                self.previous_button.Enable(False)
                self.delete_button.Enable(False)
                return

            # Customer office address
            status, lines, flags = self.address_ok()
            if not status:
                return

            # Set the address fields

            # If this is the registered office, set all other registered office values to zero
            if flags['reg_office']:
                for address in self.address_list:
                    address.reg_office = 0

            # update the address with the field values
            self.address_list[self.address_index].lines = lines
            self.address_list[self.address_index].flags = flags

            if self.address_index == 0:    # already at the First address
                self.next_button.Enable(True)
                self.previous_button.Enable(False)
                return

            self.address_index -= 1

            # Set the panel to previous address
            self.address = self.address_list[self.address_index]
            self.set_address()
            self.next_button.Enable(True)

            if self.address_index == 0:  # now at first address
                self.previous_button.Enable(False)
            else:
                self.previous_button.Enable(True)
        except Exception, e:
            error_message("Failure in moving to previous address", e)


####### Class showing a non-editable address with spin buttons

class AddressSpin(wx.Panel):
    "Creates a panel showing addresses and a spin button"

    # The value -1 for reg_office, delivery_point, invoice_point, order_point, payment_point
    # is a don't care value, 0 means must be False, 1 means must be True

    def __init__(self, parent,
                       org=None,
                       reg_office=-1,
                       delivery_point=-1,
                       invoice_point=-1,
                       order_point=-1,
                       payment_point=-1):
        """Create the panel"""
        wx.Panel.__init__(self, parent, style= wx.SIMPLE_BORDER)
        
        # Take a note of the variables
        self.parent = parent
        self.org = org

        self.SetBackgroundColour(wx.WHITE)

        max_size = (240, 130)

        # Put this in a statictext
        self.address_text=wx.StaticText(self, -1, "", size=max_size, pos=(10,10))

        self.spinner = wx.SpinButton(self, -1, size = (-1,max_size[1]+10), pos = (max_size[0]+20,0))

        self.set_org(org, reg_office, delivery_point, invoice_point, order_point, payment_point)

        # self.current_address is the address being shown

        self.Fit()
        self.Bind(wx.EVT_SPIN, self.spin_pressed, self.spinner)


    def set_org(self, org=None,
                      reg_office=-1,
                      delivery_point=-1,
                      invoice_point=-1,
                      order_point=-1,
                      payment_point=-1):
        # Set the addresses shown as belonging to org

        self.org = org

        org_addresses = org.get_addresses()


        self.address_list = [None]             # First address is the None - No address to be shown - option
        for address in org_addresses:
            if ((reg_office == -1 or address.reg_office == reg_office) and
                (delivery_point == -1 or address.delivery_point == delivery_point) and
                (invoice_point == -1 or address.invoice_point == invoice_point) and
                (order_point == -1 or address.order_point == order_point) and
                (payment_point == -1 or address.payment_point == payment_point)):
                self.address_list.append(address)

        if self.address_list == [None]:
            self.address_text.SetLabel("No address available!")
            self.spinner.SetRange(0,0)
            self.spinner.SetValue(0)
            self.current_address = None
            return False

        self.current_address = self.address_list[1]
        address_lines_string = self.current_address.lines_string
        self.address_text.SetLabel(address_lines_string)
        self.spinner.SetRange(0,len(self.address_list) -1)
        self.spinner.SetValue(1)
        return True


    def spin_pressed(self, evt):
        try:
            evt.Skip()
            idx = self.spinner.GetValue()
            self.current_address = self.address_list[idx]
            if self.current_address != None:
                address_lines_string = self.current_address.lines_string
                self.address_text.SetLabel(address_lines_string)
            else:
                self.address_text.SetLabel("No Address set")
        except Exception, e:
            error_message("Failure in handling address spinner", e)

