####### VATStuff #######
#
# new_database.py  - creates a new database
#
# Version : 0.6
# Date : 20100831
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#


"""This creates a new database. build does the gui bits, and do_build does the work
   however not a perfect separation - as do_build calls the new_database_wizard"""


import os
import decimal

import wx

from vatstuff_packages.vs_database import make_new

from vatstuff_packages.vs_utilities import *

from vatstuff_packages.vs_gui.gui_functions import *
from vatstuff_packages.vs_gui import vs_wizard

D = decimal.Decimal   # D used to avoid typing


def build(parent):
    "new database requested - asks for the filepath, creates progress bar and calls do_build"
    # Ask for confirmation
    if not vs_yesno(parent, "This creates a new database file, are you sure?", "New database"):
        return
    # close any open frames to ensure no database access is taking place
    close_frames()
    wx.Yield()
    # Get database file
    status, path = browse_for_database(parent, style=wx.SAVE)
    if (not status) or (not path):
        return
    if os.path.exists(path):
        vs_message(parent, "Unable to create new database\nas this file already exists", "ERROR")
        return
    # Create a progress bar
    prog_func = None
    status, message = False, ""
    try:
        # Incrementer produces a progress bar dialog, and returns
        # a callable instance prog_func - which is passed to the working
        # function.  Whenever prog_func() is called it increments the bar
        # To close the dialog prog_func.close() should be called
        # Incrementer is defined in gui_functions
        prog_func = Incrementer(parent,
                                "Creating database",
                                "Please wait as a new database is being created")
        # Create a new database
        status, message = do_build(parent, path, prog_func)
        if status:
            # busy cursor on as prog_func was closed when wizard started
            busy_cursor_on()
            wx.Yield()
            # do big refresh
            big_refresh()
    finally:
        if prog_func:
            prog_func.close()
    if status:
        vs_message(parent, message, "Database Loaded")
    else:
        vs_message(parent, message, "ABORTED")


def do_build(parent, path, prog_func=None):
    """creates a new database
       return (True, message) on success
              (False, message) on failure"""

    info_log("New database requested")

    old_filepath=vs_cfg["DATABASE_FILEPATH"]
    old_write_access = vs_cfg["WRITE_ACCESS"]
    old_lockfile = vs_cfg["LOCK_FILEPATH"]

    if prog_func: prog_func()

    try:
        # A database filepath has been picked, save its path, folder etc to vs_cfg and persistent storage
        set_database_filepath(path)

        # check_write_access() sets vs_cfg["WRITE_ACCESS"], needed to run the
        # wizard, and the assertion confirms no previous active lockfile exists
        # there as there shouldn't be if this is a brand new database
        assert check_write_access()

        # Create new database
        make_new.new_database(prog_func)

        # close progress bar
        if prog_func: prog_func.close()

        # run the new database wizard
        vs_wizard.new_database_wizard(parent, p3=False)

        # New database loaded, check it
        status, message = check()
        if not status:
            raise VS_ERROR, "New database fails check"

        info_log("Created database %s" % vs_cfg["DATABASE_FILEPATH"])
    except Exception,e:
        # If a corrupted database has been created, best remove it
        if os.path.isfile(vs_cfg["DATABASE_FILEPATH"]):
            os.remove(vs_cfg["DATABASE_FILEPATH"])
        # revert to previous database
        set_database_filepath(old_filepath)
        # re-check
        check(prog_func)
        raise

    # New database now loaded and checked

    if old_write_access:
        # remove the old lockfile
        try:
            if os.path.isfile(old_lockfile):
                os.remove(old_lockfile)
        except Exception, e:
            exception_log(e)

    # create the new lockfile
    fp = None
    try:
        fp = open(vs_cfg["LOCK_FILEPATH"], "w")
        fp.write(str(os.getpid()))
    finally:
        if fp:
            fp.close()

    return True, "The new database has been loaded"


