####### VATStuff #######
#
# open_database.py  - opens an existing database
#
# Version : 0.6
# Date : 20100831
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#


"""Opens an existing database which now becomes the current working database.

Three functions are provided; load, do_load and do_upgrade

load does all the gui actions - such as getting the file
name from the user and displaying progress and result messages

do_load does the actual work of loading and checking the
database file - if do_load finds the file needs an upgrade, it passes
back to load - which gets a confirmation from the user, and then calls
do_upgrade which upgrades and loads the file."""



import os
import decimal

import wx

from vatstuff_packages.vs_database import database_versions

from vatstuff_packages.vs_utilities import *

from vatstuff_packages.vs_gui.gui_functions import *


D = decimal.Decimal   # D used to avoid typing


def load(parent):
    """Loads a database - this function does the gui bit, then calls
       do_load, and do_upgrade if necessary, to do the actual loading""" 
    # close any open frames to ensure no database access is taking place
    # this function is under gui_functions
    close_frames()
    wx.Yield()
    # Get database file
    status, path = browse_for_database(parent)
    if (not status) or (not path):
        return
    if not os.path.exists(path):
        vs_message(parent, "Unable to open database\nthis file cannot be read", "ERROR")
        return
    wx.Yield()
    debug_log("Requested open database: %s" % path)
    # Create a progress bar
    prog_func = None
    status, message = False, ""
    try:
        # Incrementer produces a progress bar dialog, and returns
        # a callable instance prog_func - which is passed to the working
        # function.  Whenever prog_func() is called it increments the bar
        # To close the dialog prog_func.close() should be called
        # Incrementer is defined in gui_functions
        prog_func = Incrementer(parent,
                                "Open database",
                                "Please wait as the database is being loaded")
        # load the database
        status, message = do_load(path, prog_func)
        if status:
            # increment the bar and do big refresh
            prog_func()
            big_refresh()
    finally:
        if prog_func:
            prog_func.close()
    if status:
        vs_message(parent, message, "Database Loaded")
        return

    # so status is False
    if message != "can upgrade":
       vs_message(parent, message, "Database load abandonned")
       return

    # The database failed to load, but with message, 'can upgrade'
    # so now ask if ok to upgrade, and try again
    upgrade_text = """\
This database was created by a previous version of %s
To load, I must update it to the latest database format.
Do you wish me to continue?
(Warning - this will make it unreadable by older versions
of %s - so you would be wise to store a copy of the
database file elsewhere first.)""" % (vs_cfg["PROGRAM_ID"], vs_cfg["PROGRAM_ID"])
    if not vs_yesno(parent, upgrade_text, "Old database version"):
        return

    # user wants to try again
    prog_func = None
    status, message = False, ""
    try:
        prog_func = Incrementer(parent,
                                "Upgrade and open",
                                "Please wait as the database is being loaded")
        # upgrade and load the database
        status, message = do_upgrade(path, prog_func)
        if status:
            # increment the bar and do big refresh
            prog_func()
            big_refresh()
    finally:
        if prog_func:
            prog_func.close()
    if status:
        vs_message(parent, message, "Database Loaded")
    else:
        vs_message(parent, message, "Database load abandonned")


def do_load(path, prog_func=None):
    """load the database file defined by path,
       If prog_func exists, it is a callable function to display
       a progress bar, and will be called at intervals 
       returns (True, message) on success,
       (False, message) on failure
       If the failure is due to database version being incompatible,
       but there is the possibility of an upgrade, then the
       message will be 'can upgrade'"""

    info_log("Open a database requested")

    old_filepath=vs_cfg["DATABASE_FILEPATH"]
    old_write_access = vs_cfg["WRITE_ACCESS"]
    old_lockfile = vs_cfg["LOCK_FILEPATH"]

    if prog_func: prog_func()

    try:
        # A database filepath has been picked, save its path, folder, lock folder etc
        # to vs_cfg and persistent storage
        set_database_filepath(path)

        # database loaded, check it
        status, message = check(prog_func)

        if (not status) and message == "Database version is incompatible":
            info_log("Database check returns: %s" % message)
            # if check() fails due to a different database version
            # send message back for conversion
            if vs_cfg["WRITE_ACCESS"]:
                raise VS_CANCEL, "can upgrade"
            else:
                raise VS_CANCEL, "Database version is incompatible,\nno write access so cannot upgrade"

        if not status:
            info_log("Database check returns: %s" % message)
            raise VS_CANCEL, message

        # so database now loaded and checked
        info_log("Loaded database %s" % vs_cfg["DATABASE_FILEPATH"])
    except Exception, e:
        # Database load abandonned, revert to previous database
        set_database_filepath(old_filepath)
        # re-check
        check(prog_func)
        if isinstance(e, VS_CANCEL):
            # Do not raise an exception, just return with the failure
            return False, str(e)
        # if e is not a VS_CANCEL - a genuine error occurred, so re raise it
        raise

    if prog_func: prog_func()

    if old_write_access:
        # remove the old lockfile
        try:
            if os.path.isfile(old_lockfile):
                os.remove(old_lockfile)
        except Exception, e:
            exception_log(e)

    # so database in place and checked,create the lockfile
    if vs_cfg["WRITE_ACCESS"]:
        fp = None
        try:
            fp = open(vs_cfg["LOCK_FILEPATH"], "w")
            fp.write(str(os.getpid()))
        finally:
            if fp:
                fp.close()

    # The database is opened
    if prog_func: prog_func()

    info_log("database loaded : %s" % path)
    if vs_cfg["WRITE_ACCESS"]:
        return True, "The database has been loaded"
    else:
        return True, "Database loaded, read access only\nas another application has access"


def do_upgrade(path, prog_func=None):
    """upgrade and load the database file defined by path,
       If prog_func exists, it is a callable function to display
       a progress bar, and will be called at intervals 
       returns (True, message) on success,
       (False, error_message) on failure"""

    info_log("Upgrade and open a database requested")

    old_filepath=vs_cfg["DATABASE_FILEPATH"]
    old_write_access = vs_cfg["WRITE_ACCESS"]
    old_lockfile = vs_cfg["LOCK_FILEPATH"]

    if prog_func: prog_func()

    try:
        # A database filepath has been picked, save its path, folder etc
        # to vs_cfg and persistent storage
        set_database_filepath(path)

        # database loaded, check it
        status, message = check(prog_func)

        # assert write access ok
        assert check_write_access()

        if (not status) and message == "Database version is incompatible":
            # Try to update the database
            if prog_func: prog_func()
            if not database_versions.versioncheck():
                # check failed, cannot update version
                raise VS_CANCEL, "Error when trying to upgrade"
            # Database upgraded, re-check
            status, message = check(prog_func)

        if not status:
            info_log("Database check returns: %s" % message)
            raise VS_CANCEL, message

        # so database now loaded and checked
        info_log("Loaded database %s" % vs_cfg["DATABASE_FILEPATH"])
    except Exception, e:
        # Database load abandonned, revert to previous database
        set_database_filepath(old_filepath)
        # re-check
        check(prog_func)
        if isinstance(e, VS_CANCEL):
            # Do not raise an exception, just return with the failure
            return False, str(e)
        # if e is not a VS_CANCEL - a genuine error occurred, so re raise it
        raise

    if prog_func: prog_func()

    if old_write_access:
        # remove the old lockfile
        try:
            if os.path.isfile(old_lockfile):
                os.remove(old_lockfile)
        except Exception, e:
            exception_log(e)

    # so database in place and checked,create the lockfile
    if vs_cfg["WRITE_ACCESS"]:
        fp = None
        try:
            fp = open(vs_cfg["LOCK_FILEPATH"], "w")
            fp.write(str(os.getpid()))
        finally:
            if fp:
                fp.close()

    if prog_func: prog_func()
    info_log("database loaded : %s" % path)
    if vs_cfg["WRITE_ACCESS"]:
        return True, "The database has been loaded"
    else:
        return True, "Database loaded, read access only\nas another application has access"


