####### VATStuff #######
#
# restore_backup.py  - loads a previously created backup of the database
#
# Version : 0.6
# Date : 20100831
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#


"""Loads a previously created backup of the database
which now becomes the current working database.

Two functions are provided; restore and do_restore

restore does all the gui actions - such as getting the file
name from the user and displaying progress and result messages

do_restore does the actual work of loading and checking the
backup file"""


import os
import shutil
import tarfile
import decimal
import tempfile

import wx


from vatstuff_packages.vs_database import database_versions

from vatstuff_packages.vs_utilities import *

from vatstuff_packages.vs_gui.gui_functions import *

D = decimal.Decimal   # D used to avoid typing



def restore(parent):
    """Restores a backup file - this function does the gui bit, then calls
       do_restore to do the actual restoration""" 
    # Ask for confirmation
    if not vs_yesno(parent, "This overwrites the current database, are you sure?", "Load Backup"):
        return
    # close any open frames to ensure no database access is taking place
    # this function is under gui_functions
    close_frames()
    wx.Yield()
    # Create the load file dialog.
    # Pre-establish a file filter to show the .bak extension
    wildcard = "Backup file (*.bak)|*.bak|All files (*.*)|*.*"
    # get previous backup_file name
    backup_file = vs_cfg["BACKUP_FILE"]
    if backup_file:
        # Check it exists
        if not os.path.exists(os.path.join(vs_cfg["BACKUP_FOLDER"], backup_file)):
            backup_file = ""
    
    dlg = wx.FileDialog(parent, message="Load file...",
                                defaultDir=vs_cfg["BACKUP_FOLDER"],
                                defaultFile=backup_file,
                                wildcard=wildcard,
                                style=wx.OPEN)
    # Show the dialog and retrieve the user response.
    # If it is the OK response, load the backup file.
    if dlg.ShowModal() == wx.ID_OK:
        backup_file = dlg.GetPath()
        dlg.Destroy()
    else:
        dlg.Destroy()
        return
    wx.Yield()
    debug_log("Requested restore from: %s" % backup_file)
    # Create a progress bar
    prog_func = None
    status, message = False, ""
    try:
        # Incrementer produces a progress bar dialog, and returns
        # a callable instance prog_func - which is passed to the working
        # function.  Whenever prog_func() is called it increments the bar
        # To close the dialog prog_func.close() should be called
        # Incrementer is defined in gui_functions
        prog_func = Incrementer(parent,
                                "Restoring",
                                "Please wait as data is being restored")
        # increment the bar
        prog_func()
        # restore the backup
        status, message = do_restore(backup_file, prog_func)
        if status:
            # increment the bar and do big refresh
            prog_func()
            big_refresh()
    finally:
        if prog_func:
            prog_func.close()
    if status:
        vs_message(parent, message, "Backup Loaded")
    else:
        vs_message(parent, message, "Backup abandonned")
    


def do_restore(backup_file, prog_func=None):
    """load the backup file,
       If prog_func exists, it is a callable function to display
       a progress bar, and will be called at intervals 
       returns (True, "The backup file has been loaded") on success,
       (False, error_message) on failure"""

    info_log("Restore from: %s" % backup_file)
    if not tarfile.is_tarfile(backup_file):
        info_log("backup file format is not recognised")
        return False, "The backup file format is not recognised"

    if prog_func: prog_func()

    # Database filepath
    db_filepath=vs_cfg["DATABASE_FILEPATH"]
    # logo image filepath
    logo_filepath = vs_cfg["LOGO_FILEPATH"]
    logo_file = vs_cfg["LOGO_FILE"]

    # If no logo has been loaded, create a new logo name
    if not os.path.exists(logo_filepath):
        logo_filepath, logo_file = create_logo_name()

    # Create two temporary directories
    temp_dir1 = tempfile.mkdtemp()
    temp_dir2 = tempfile.mkdtemp()
    # temp_dir1 will be used to temporarily store the current database
    # temp_dir2 will be used to temporarily store the tar archive files
    # as they are extracted

    # The current database and logo's will be stored in the temporary
    # directory by renaming them - first get new names
    db_tmp = os.path.join(temp_dir1, "database")
    logo_tmp = os.path.join(temp_dir1, "logo")

    if prog_func: prog_func()

    # vs_cfg["ARCHIVED_FILE"] and vs_cfg["LOGO_IMAGE"] are the file names stored
    # in the tar file.  These are checked, the files extracted, and then renamed
    # to db_filepath and logo_filepath.

    fp = None
    try:
        fp = tarfile.open(backup_file, 'r:bz2')
        archive_list = fp.getnames()
        if not vs_cfg["ARCHIVED_FILE"] in archive_list:
            return False, "The backup file format is not recognised"

        # rename logo and database files to temporary directory    
        # If filepath exists, rename it
        if os.path.exists(db_filepath):
            os.rename(db_filepath, db_tmp)
        # If logo_filepath exists, rename it
        if os.path.exists(logo_filepath):
            os.rename(logo_filepath, logo_tmp)

        # So the existing database and logo are held in temp_dir1
        if prog_func: prog_func()

        # extract backup files into temp_dir2, and then rename them to replace
        # the original database file and logo
        fp.extract(vs_cfg["ARCHIVED_FILE"],temp_dir2)
        # Rename it to db_filepath
        os.rename(os.path.join(temp_dir2, vs_cfg["ARCHIVED_FILE"]), db_filepath)
        # get the logo image if it is there, and do the same
        if vs_cfg["LOGO_IMAGE"] in archive_list:
            fp.extract(vs_cfg["LOGO_IMAGE"],temp_dir2)
            # rename it to logo_filepath
            os.rename(os.path.join(temp_dir2, vs_cfg["LOGO_IMAGE"]), logo_filepath)

        if prog_func: prog_func()

        # So database file extracted, and in place, now check it
        status, message = check(prog_func)
        if (not status) and message == "Database version is incompatible":
            # if check() fails due to a different database version
            # see if it can be converted
            status = database_versions.versioncheck()
            if not status:
                info_log("versioncheck failed to upgrade")
                raise VS_CANCEL, "Incompatible data version, and unable to upgrade"
        if not status:
            info_log("Database check returns: %s" % message)
            raise VS_CANCEL, message

        # Database checked ok
        if prog_func: prog_func()

        vs_cfg["LOGO_FILEPATH"] = logo_filepath
        vs_cfg["LOGO_FILE"] = logo_file

        # and save the logo name in the database
        set_config_item("LOGO_FILE", vs_cfg["LOGO_FILE"])

        # save the backup directory to the per-user persistent store
        set_stored_item("BACKUP_FOLDER", os.path.dirname(backup_file))

        # save the backup filename to the per-user persistent store
        set_stored_item("BACKUP_FILE", os.path.basename(backup_file))

    except Exception, e:
        # Database load abandonned, revert to previous database
        info_log("Reverting to previous database")
        if os.path.exists(db_tmp) and os.path.exists(db_filepath):
            os.remove(db_filepath)
        if os.path.exists(logo_tmp) and os.path.exists(logo_filepath):
            os.remove(logo_filepath)
        if os.path.exists(db_tmp):
            os.rename(db_tmp, db_filepath)
        if os.path.exists(logo_tmp):
            os.rename(logo_tmp, logo_filepath)
        # As original database is back in place, call check() again to
        # ensure all configuration constants are correct
        check(prog_func)
        if isinstance(e, VS_CANCEL):
            # Do not raise an exception, just return with the failure
            return False, str(e)
        # if e is not a VS_CANCEL - a genuine error occurred, so re raise it
        raise
    finally:
        if fp: fp.close()
        # get rid of temporary directories
        shutil.rmtree(temp_dir1, ignore_errors=True)
        shutil.rmtree(temp_dir2, ignore_errors=True)

    if prog_func: prog_func()

    info_log("database %s restored from backup" % db_filepath)
    return True, "The backup file has been loaded"


