####### VATStuff #######
#
# undo_action.py  - confirms the undo action of the admin tab undo button
#
# Version : 0.6
# Date : 20100824
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#


"""Confirms the undo action of the admin tab undo button,
   closes down any open frame, such as an invoice frame,
   which will be altered by the transaction undo,
   and then calls the appropriate function in vatstuff_packages.vs_database.delete_transaction"""


import datetime

import wx

from vatstuff_packages.vs_gui.gui_functions import *

from vatstuff_packages.vs_gui.purchases.view_orders import ShowOrderFrame
from vatstuff_packages.vs_gui.sales.view_invoices import ShowInvoiceFrame
from vatstuff_packages.vs_gui.sales.view_creditnotes import ShowCreditnoteFrame






from vatstuff_packages.vs_database import delete_transaction

from vatstuff_packages.vs_utilities import *



def undo_confirm_and_implement(parent):
    """Finds the last transaction, and if user confirms, deletes it with any associated
        database entries, such as the purchase order etc.,"""

    if not vs_cfg["WRITE_ACCESS"]:
        # Another process has locked the database
        raise VS_ANOTHER_PROCESS, "Deletion cancelled - another process has locked this database"

    last_transaction = Transaction.get_last()

    if not last_transaction:
        vs_message(parent, "Unable to find the last transaction", "ERROR")
        return

    last_transaction_id     = last_transaction.id
    last_transaction_system = last_transaction.system

    # Check the vat scheme has not changed since the last transaction
    # if it has, then offer to delete the VAT scheme
    vatscheme = VATScheme()

    if last_transaction_id <= vatscheme.transaction_id:
        caption = "Delete VAT Scheme change"
        text = """\
Do you wish to delete the change of VAT Scheme?
Choose Yes to delete - or No to cancel"""
        if not vs_yesno(parent, text, caption):
            # Do not implement the vat scheme deletion
            return
        busy_cursor_on()
        wx.Yield()
        delete_transaction.delete_vat_scheme()
        info_log("VAT Scheme change deleted")
        big_refresh(acc_tree=False)
        vs_message(parent, """\
VAT Scheme change deleted. You must re-input the
company VAT registration number if it has changed.
Use the Set VAT scheme button under the VAT tab.""", "Done")
        return

    if last_transaction_system == 3:
        # This was a payment transaction, but it could be a 'quick purchase'
        # investigate if this is the case, and if so, offer to delete the entire
        # purchase
        makepayment = last_transaction.get_object()
        if makepayment.status == 1:
            caption = "Delete quick purchase"
            text = """\
The last 'Quick Purchase' consisted of three combined
transactions - a purchase order, invoice and payment.
Choose Yes to delete all three - or No to cancel"""
            if not vs_yesno(parent, text, caption):
                # Do not implement the quick purchase deletion
                return
            busy_cursor_on()
            wx.Yield()
            # If the associated purchase order frame is open, close it
            close_purchaseorder_frame(makepayment.purchaseorder)
            delete_transaction.delete_quick_purchase(makepayment)
            info_log("Quick Purchase deleted")
            big_refresh()
            vs_message(parent, "Quick Purchase deleted", "Done")
            return


    # Create confirmation text
    caption = "Delete Transaction : %i" % last_transaction_id
    text = """\
Are you sure you wish to delete this transaction:

%s
Transaction Date: %s

""" % (last_transaction.description, last_transaction.datestamp.strftime("%d %b %Y"))

    if last_transaction_system == 0:
        text += "This was a user 'move money' transaction"
        if not vs_yesno(parent, text, caption):
            # Do not implement the transaction deletion
            return
    elif last_transaction_system == 1:
        text += "This was an 'openning balance' transaction"
        if not vs_yesno(parent, text, caption):
            # Do not implement the transaction deletion
            return
    elif last_transaction_system == 2:
        purchaseorder = last_transaction.get_object()
        text += "This deletes purchase order No. %i" % purchaseorder.number
        if not vs_yesno(parent, text, caption):
            # Do not implement the transaction deletion
            return
        busy_cursor_on()
        wx.Yield()
        close_purchaseorder_frame(purchaseorder)
    elif last_transaction_system == 3:
        text += "This deletes your payment"
        if not vs_yesno(parent, text, caption):
            # Do not implement the transaction deletion
            return
        busy_cursor_on()
        wx.Yield()
        # If the associated purchase order frame is open, close it
        makepayment = last_transaction.get_object()
        close_purchaseorder_frame(makepayment.purchaseorder)
    elif last_transaction_system == 4:
        text += "This deletes the received suppliers credit note"
        if not vs_yesno(parent, text, caption):
            # Do not implement the transaction deletion
            return
        suppliercreditnote = last_transaction.get_object()
        if not suppliercreditnote.status:
            # If the associated purchase order frame is open, close it
            close_purchaseorder_frame(suppliercreditnote.purchaseorder)
    elif last_transaction_system == 5:
        invoice = last_transaction.get_object()
        text += "This deletes invoice No. %i" % invoice.number
        if not vs_yesno(parent, text, caption):
            # Do not implement the transaction deletion
            return
        close_invoice_frame(invoice)
    elif last_transaction_system == 6:
        text += "This deletes the last customer payment to you"
        if not vs_yesno(parent, text, caption):
            # Do not implement the transaction deletion
            return
        customer_payment = last_transaction.get_object()
        invoice_list = customer_payment.get_invoices()
        for invoice in invoice_list:
            close_invoice_frame(invoice)
    elif last_transaction_system == 7:
        creditnote = last_transaction.get_object()
        text += "This deletes credit note No. %i" % creditnote.number
        if not vs_yesno(parent, text, caption):
            # Do not implement the transaction deletion
            return
        close_creditnote_frame(creditnote)
        if not creditnote.status:
            # If the associated invoice frame is open, close it
            close_invoice_frame(creditnote.invoice)
    elif last_transaction_system == 8:
        text += "This deletes the 'close order' transaction"
        if not vs_yesno(parent, text, caption):
            # Do not implement the transaction deletion
            return
        # If the associated purchase order frame is open, close it
        closedorder = last_transaction.get_object()
        close_purchaseorder_frame(closedorder.purchaseorder)
    elif last_transaction_system == 9:
        text += "This deletes the supplier invoice received"
        if not vs_yesno(parent, text, caption):
            # Do not implement the transaction deletion
            return
        # If the associated purchase order frame is open, close it
        supplierinvoice = last_transaction.get_object()
        close_purchaseorder_frame(supplierinvoice.purchaseorder)
    elif last_transaction_system == 10:
        text += "This deletes a VAT return transaction"
        if not vs_yesno(parent, text, caption):
            # Do not implement the transaction deletion
            return
    elif last_transaction_system == 11:
        text += "This deletes a VAT correction transaction"
        if not vs_yesno(parent, text, caption):
            # Do not implement the transaction deletion
            return
    elif last_transaction_system == 12:
        # System 12 - delete old data transaction cannot be deleted
        vs_message(parent, "The data deletion cannot be undone, use your backup", "WARNING")
        return
    else:
        # System number not recognised, do not delete
        vs_message(parent, "The last transaction type is not recognised", "ERROR")
        return

    # Delete the last transaction
    busy_cursor_on()
    wx.Yield()
    delete_transaction.delete_last_transaction(last_transaction)
    wx.Yield()
    big_refresh()
    vs_message(parent, "Transaction: %i deleted" % last_transaction_id, "Done")


def close_purchaseorder_frame(purchaseorder):
    "Checks if a frame showing this purchaseorder is open, and if it is, closes it"
    for frm in vs_cfg["TRANSACTION_REFRESH"]:
        if isinstance(frm, ShowOrderFrame):
            if frm.order == purchaseorder:
                frm.Close()
                wx.Yield()
                return


def close_invoice_frame(invoice):
    "Checks if a frame showing this invoice is open, and if it is, closes it"
    for frm in vs_cfg["TRANSACTION_REFRESH"]:
        if isinstance(frm, ShowInvoiceFrame):
            if frm.invoice == invoice:
                frm.Close()
                wx.Yield()
                return


def close_creditnote_frame(creditnote):
    "Checks if a frame showing this creditnote is open, and if it is, closes it"
    for frm in vs_cfg["TRANSACTION_REFRESH"]:
        if isinstance(frm, ShowCreditnoteFrame):
            if frm.creditnote == creditnote:
                frm.Close()
                wx.Yield()
                return

