####### VATStuff #######
#
# edit_addresses.py  - Setup the company edit addresses frame
#
# Version : 0.4
# Date : 20100513
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#
"""Setup the edit addresses frame

It is run as part of the vatstuff gui interface.
"""


import wx





from vatstuff_packages.vs_database.create_objects import create_Address

from vatstuff_packages.vs_gui.address_panel import *
from vatstuff_packages.vs_gui.gui_functions import *

from vatstuff_packages.vs_utilities import *

from vatstuff_packages.vs_verify.form_objects import make_AddressForm



def edit_company_address_dialog(parent):
    "Called when the Company Addresses button is pressed"
    edit_company_address_dialog = EditCompanyAddressDialog(parent)

    # If the cursor is busy, unbusy it - as
    # now waiting for a button press
    busy_cursor_off()

    edit_company_address_dialog.ShowModal()
    edit_company_address_dialog.Destroy()


class EditCompanyAddressDialog(vsDialog):
    "The dialog which edits or adds company addresses"

    def __init__(self, parent):
        "Create the dialog"
        vsDialog.__init__(self, parent, "Edit Addresses")

        main_sizer=wx.BoxSizer(wx.VERTICAL)

        help1_text="""If your company has multiple addresses, you can add or edit them here. Use the Next and
Previous buttons to move between addresses, and for each address set its function (delivery,
invoice or registered office) - only one address can be the registered office."""
        
        self.top_text=wx.StaticText(self, -1, help1_text)
        main_sizer.Add(self.top_text, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # self.address_list is the list of temporary addresses to be edited,

        self.this_company = Company()
        database_addresses = self.this_company.get_addresses()

        # make a list of temporary copies
        self.address_list = [ make_AddressForm(address) for address in database_addresses ]

        # self.address_list is now a list of AddressForm objects (object defined in vatstuff_packages.vs_verify.form_objects)
        # These can be edited directly without affecting the database, but then changes must be
        # sent to the database when Apply is pressed

        # flag mask dictates which of the flag field are enabled or disabled
        # flag defaults set which are ticked or unticked for a new address as default

        # Add the office panel
        flag_mask = { 'reg_office'     : 1,
                      'delivery_point' : 1,
                      'invoice_point'  : 1,
                      'order_point'    : 1,
                      'payment_point'  : 1 }
        flag_defaults = { 'reg_office'     : 0,
                          'delivery_point' : 1,
                          'invoice_point'  : 1,
                          'order_point'    : 1,
                          'payment_point'  : 1 }

        self.company_office_panel = AddressPanel(self,
                                                  self.this_company.name,
                                                  self.address_list,
                                                  flag_mask,
                                                  flag_defaults,
                                                  buttons = True)

        main_sizer.Add(self.company_office_panel, 0, wx.ALL | wx.ALIGN_CENTER, 10)


        # ---------------------- Buttons

        # Create buttons in a horizontal sizer
        button_sizer = wx.BoxSizer(wx.HORIZONTAL)

        button_size = (100, vs_cfg["BUTTON_HEIGHT"])

        # Apply button
        self.apply_button = wx.Button(self, wx.ID_APPLY, size=button_size)
        self.apply_button.SetToolTipString("Click to apply changes.")
        button_sizer.Add(self.apply_button, 0, wx.RIGHT, 40)

        # Cancel button
        self.cancel_button = wx.Button(self, wx.ID_CANCEL, size=button_size)
        self.cancel_button.SetToolTipString("Click to cancel.")
        button_sizer.Add(self.cancel_button, 0)

        # Add button_sizer to the main sizer
        main_sizer.Add(button_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)
        
        # Set the size to the panel
        self.SetSizer(main_sizer)
        main_sizer.SetSizeHints(self)


        # ---------- Bind buttons to handler functions
        
        self.Bind(wx.EVT_BUTTON, self.apply_button_pressed, self.apply_button)
        self.Bind(wx.EVT_BUTTON, self.cancel_button_pressed, self.cancel_button)


    # ----------------- Handler functions

    @writecheck
    @button_wrap
    def apply_button_pressed(self, event):
        "Apply button pressed in the Company edit addresses frame"

        # Company addresses
        status, company_name, self.address_list, deleted_addresses = self.company_office_panel.get_addresses()
        if not status:
            return

        # Check this company name is not already used
        all_suppliers = Supplier.get_all()
        for c in all_suppliers:
            if company_name == c.name:
                vs_message(self, "This company name is a supplier!.", "ERROR")
                return

        all_customers = Customer.get_all()
        for c in all_customers:
            if company_name == c.name:
                vs_message(self, "This company name is a customer!.", "ERROR")
                return

        # Check one address has the registered office set

        reg_office = 0
        for address in self.address_list:
            reg_office += address.reg_office

        if reg_office != 1:
            vs_message(self, "One address must be the registered Office.", "ERROR")
            return

        # save all addresses in self.address_list to the database

        for address in self.address_list:

            if address.id < 0 :
                # This is a new address which has to be created
                database_address = create_Address(system=1)
            else:
                database_address = Address(address.id)

            database_address.lines = address.lines
            database_address.flags = address.flags

            if address.reg_office == 1:
                # Make the default country for addresses, the same country
                # as the registered office
                vs_cfg["COUNTRY"] = database_address.country
                if database_address != self.this_company.registered_address:
                    self.this_company.registered_address = database_address

        # Delete the addresses in the deleted_addresses list
        if deleted_addresses:
            for address in deleted_addresses:
                database_address = Address(address.id)
                database_address.delete()

        # Set company with new company name
        if self.this_company.name != company_name:
            self.this_company.name = company_name
            # Set the title of the top frame
            top_title(company_name)
        
        vs_message(self, "Changes have been saved.", "INFORMATION")
        self.EndModal(wx.ID_OK)


    def cancel_button_pressed(self, event):
        "Close window after an information message"
        vs_message(self, "No changes have been made.", "INFORMATION")
        self.EndModal(wx.ID_CANCEL)




