####### VATStuff #######
#
# gui.py  - starts the gui interface for vatstuff
#
# Version : 0.6
# Date : 20100814
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#

"""The function maingui() within this module runs the gui interface for vatstuff

The gui uses the wxPython library which must be installed. The wx.App is called
which starts StartupFrame

At this point, App is finished, and maingui can enter the main program loop.

StartupFrame is a simple frame, showing only a 'Please Wait. Opening database' message
If a database has been passed in the command line it starts a thread to check the database
and then, on success, hides itself and opens the main application frame 'AppFrame'.

If no database has been passed in the command line, then StartupFrame hides itself and calls
vs_wizard.choose_database which opens ChooseDatabaseFilepathDialog - which gives users the option
to choose a database or to create a new one.

If an existing one is chosen, StartupFrame then shows itself with a progress bar
which is used to indicate database checking and loading

Once a database is loaded, StartupFrame hides, and opens the main application frame 'AppFrame'.

AppFrame creates a tabbed notebook, each tab page within the notebook panel is then created via a
package module, for example the accounts page is in vatstuff_packages.vs_gui.accounts.accounts_tab
and the sales page is in vatstuff_packages.vs_gui.sales.sales_tab."""


import os
import sys
import decimal

import wx

from vatstuff_packages.vs_utilities import *

from vatstuff_packages.vs_gui.gui_functions import *

from vatstuff_packages.vs_gui.accounts import accounts_tab
from vatstuff_packages.vs_gui.sales import sales_tab
from vatstuff_packages.vs_gui.purchases import purchases_tab
from vatstuff_packages.vs_gui.movemoney import movemoney_tab
from vatstuff_packages.vs_gui.vat import vat_tab
from vatstuff_packages.vs_gui.reports import reports_tab
from vatstuff_packages.vs_gui.company import company_tab
from vatstuff_packages.vs_gui.admin import admin_tab
from vatstuff_packages.vs_gui.manual import manual_tab

from vatstuff_packages.vs_database import database_versions
from vatstuff_packages.vs_database import make_new

from vatstuff_packages.vs_gui import vs_wizard

D = decimal.Decimal   # D used to avoid typing




# ------------------ The Application Class

class App(wx.App):
    """vatstuff Gui wx Application class."""

    def __init__(self, database_filepath):
        self.database_filepath = database_filepath
        wx.App.__init__(self)


    def OnInit(self):
        """Create the start up frame, passing it
           the database_filepath if given on the command line"""
        self.startup_frame = StartupFrame(self.database_filepath)
        self.SetTopWindow(self.startup_frame)
        return True



#  ------------------ StartupFrame

class StartupFrame(vsFrame):
    """The starting frame of the gui - normally hidden, and consists of 'Please Wait' text.
       Its main use is to initiate the 'choose a database file' dialog, or the database wizard
       dialog if a new database is chosen.  It starts a new database creation thread if required
       and a database check thread.  On the threads being completed it creates the app frame."""

    def __init__(self, database_filepath):
        """Create the frame"""
        vsFrame.__init__(self, parent=None, title=vs_cfg["PROGRAM_ID"], style=wx.DEFAULT_FRAME_STYLE ^ (wx.CLOSE_BOX))
        
        # The application frame is not immediately created
        # but is eventually create once the database has been successfully checked
        self.app_frame = None

        self.startup_panel = StartupPanel(self, database_filepath)
        if not vs_cfg["BACKGROUND_COLOUR"]:
            vs_cfg["BACKGROUND_COLOUR"] = self.startup_panel.GetBackgroundColour()

        # Get the panel size
        panel_size = self.startup_panel.GetMinSize()

        # Set minimum size equal to the panel
        self.SetMinSize(panel_size)
        self.SetClientSize(panel_size)

    def startup_message(self, message="Please Wait. Opening database . . ."):
        "Calls method of the same name in the panel, to set a message"
        self.startup_panel.startup_message(message)

    def progress_increment(self):
        "Increments the progress bar"
        self.startup_panel.progress_increment()


class StartupPanel(wx.Panel):
    """The startup panel, contains please wait text, and starts the dialogs"""

    def __init__(self, parent, database_filepath):
        wx.Panel.__init__(self, parent)
        self.parent = parent
        self.database_filepath = database_filepath

        self.panel_sizer=wx.BoxSizer(wx.VERTICAL)

        self.text=wx.StaticText(self, -1, "Please Wait. Opening database . . . . . .\n\n")
        self.panel_sizer.Add(self.text, 0, wx.TOP | wx.LEFT | wx.RIGHT | wx.ALIGN_CENTER, 30)

        self.progress_bar = wx.Gauge(self, -1, size = (400, 20))
        self.progress_bar.SetRange(20)
        # ensure the progress bar is set to zero
        self.progress_bar.SetValue(0)

        self.panel_sizer.Add(self.progress_bar, 0, wx.ALL | wx.ALIGN_CENTER, 30)

        # Create a cancel button
        self.cancel_button = wx.Button(self, label="Cancel", size=(100, -1))
        self.panel_sizer.Add(self.cancel_button, 0, wx.ALL | wx.ALIGN_CENTER, 30)
        #self.quit_button.Enable(False)
        self.cancel_flag = False

        # Get a standard button height, slightly larger than the default
        vs_cfg["BUTTON_HEIGHT"] = self.cancel_button.Size[1] + 4

        # Set the size to the panel
        self.SetSizer(self.panel_sizer)
        self.panel_sizer.SetSizeHints(self)

        # set up the cancel button handler
        self.Bind(wx.EVT_BUTTON, self.cancel_button_pressed, self.cancel_button)

        # Store the original database filepath, as currently read from the persistent store
        self.store_database_filepath = vs_cfg["DATABASE_FILEPATH"]

        # set up a timer to start database check once this window has been drawn
        self.timed_start = wx.Timer(self)
        self.Bind(wx.EVT_TIMER, self.on_timer,  self.timed_start)
        self.timed_start.Start(200, oneShot=True)


    def cancel_button_pressed(self, event):
        "Sets self.cancel_flag, which progress_increment() uses to raise a VS_CANCEL"
        # disable button, so cannot be called again
        self.cancel_button.Enable(False)
        self.cancel_flag = True


    def progress_increment(self):
        "Called periodically to increment the progress bar, and test if cancel pressed"
        value = self.progress_bar.GetValue()
        if value < 19:
            self.progress_bar.SetValue(value+1)
        # Check if the cancel button is pressed by calling wx.Yield()
        # which allows cancel_button_pressed() to be run, if the button event is waiting
        wx.Yield()
        if self.cancel_flag:
            # raise error, and set flag to false
            self.cancel_flag = False
            raise VS_CANCEL


    def on_timer(self, event):
        "Pick a database"
        try:
            # Enable the cancel button
            self.cancel_button.Enable(True)
            self.cancel_flag = False
            self.progress_increment()
            status = False
            if self.database_filepath:
                self.parent.Show()
                # a database filepath has been given on the command line
                # set the vs_cfg values, and persistent storage
                set_database_filepath(self.database_filepath)
                # check the database
                status = self.check_database()

            if not status:
                # No database_filepath given on the command line
                # or the database given has failed the check
                # so open a choose database dialog
                self.database_filepath = None
                self.progress_bar.SetValue(0)
                status = self.database_dialog()
                if status == "QUIT":
                    self.parent.Close()
                    return
                if not status:
                    # re-call this function in 200 millisec
                    self.cancel_button.Enable(False)
                    self.cancel_flag = False
                    self.timed_start.Start(200, oneShot=True)
                    return

            # So database checked OK, now start the main application frame
            self.startup_message("Database check ok - starting application\n\n")
            self.progress_increment()

            # In the event of multiple instances of vatstuff being started
            # only one instance should get write access, the rest should get read access only

            # If this database has write access, create the lockfile to stop
            # anyone else writing to it
            if vs_cfg["WRITE_ACCESS"]:
                # Lock the database by creating a lockfile
                fp = None
                try:
                    fp = open(vs_cfg["LOCK_FILEPATH"], "w")
                    fp.write(str(os.getpid()))
                finally:
                    if fp:
                        fp.close()
                # set the program version in the database
                set_config_item("PROGRAM_VERSION", vs_cfg["PROGRAM_VERSION"])

            # increment progress bar, and check for cancel
            self.progress_increment()

            # Log the program and database version, at INFO level
            log_string = "%s version %s, database version %s" % (vs_cfg["PROGRAM_ID"],
                                                                 vs_cfg["PROGRAM_VERSION"],
                                                                 vs_cfg["DATABASE_VERSION"])
            info_log(log_string)
            # Log the database filename
            info_log("opened database : %s" % vs_cfg["DATABASE_FILEPATH"])
            # Create the main application frame
            self.parent.app_frame = AppFrame(self.parent)
            self.parent.Hide()
            self.startup_message() # set default message
            self.parent.app_frame.Show()
            debug_log("AppFrame created and shown")
        except VS_CANCEL:
            # Cancel loading this database
            # re-call this function in 200 millisec
            # to re-show self.database_dialog()
            vs_message(self, "Database load cancelled", "Cancelled")
            self.database_filepath = None
            self.timed_start.Start(200, oneShot=True)
            return
        except Exception, e:
            error_message("Unable to load application", e)
            self.parent.Close()
            return

    def startup_message(self, message="Please Wait. Opening database . . . . . .\n\n"):
        "puts a message in the startup frame panel"
        self.text.SetLabel(message)
        wx.Yield()

    def database_dialog(self):
        """Open a database dialog, returns
           True for database chosen ok
           False for database not checked ok - re-show dialog
           'QUIT' for quit the application"""
        self.parent.Hide()
        self.startup_message() # set startup message back to default
        status, path = vs_wizard.choose_database(self)
        if not status:
            # Quit the program, ensure persistent store is back to the original
            set_database_filepath(self.store_database_filepath)
            return "QUIT"
        # path is the database file chosen, set it up
        set_database_filepath(path)
        # show startup parent again while file is being checked
        self.parent.Show()
        self.progress_increment()

        # 'set_database_filepath' sets the path in vs_cfg["DATABASE_FILEPATH"]
        if os.path.isfile(vs_cfg["DATABASE_FILEPATH"]):
            # Database exists - so check it
            return self.check_database()

        # a database location has been chosen, but no database exists there,
        # so create a new database
        self.startup_message("Please Wait. Creating new database . . .\n\n")
        try:
            # check_write_access() sets vs_cfg["WRITE_ACCESS"], needed to run the
            # wizard, and the assertion confirms no previous active lockfile exists
            # there as there shouldn't be if this is a brand new database
            assert check_write_access()
            make_new.new_database(self.progress_increment)
            info_log("Created database %s" % vs_cfg["DATABASE_FILEPATH"])
        except Exception:
            # If a corrupted database has been created, best remove it
            if os.path.isfile(vs_cfg["DATABASE_FILEPATH"]):
                os.remove(vs_cfg["DATABASE_FILEPATH"])
            # and re-raise error so calling function can handle it
            raise
        self.progress_increment()
        # set message to default
        self.startup_message()
        # New database created - run the wizard
        self.parent.Hide()
        wx.Yield()
        vs_wizard.new_database_wizard(self)
        # show the parent again
        self.parent.Show()
        # Now check the new database created
        return self.check_database()


    def check_database(self):
        """database check, return True if ok False if not
           uses progress_increment() to raise VS_CANCEL if cancel button in startup pressed"""
        self.progress_increment()
        status, message = check(self.progress_increment)
        if status:
            debug_log("Database check returns OK")
            return True
       
        debug_log("Database check returns: %s" % message)
        self.startup_message("Check completed. Problems found . . . .\n\n")
        if message != "Database version is incompatible":
            # check failed - for a reason other than database version
            vs_message(self, message, "ERROR")
            return False

        # if check() fails due to a different database version
        # Do we have write access? If not then cannot convert the database
        if not vs_cfg["WRITE_ACCESS"]:
            vs_message(self,
                        "Database version incompatible, and\nI do not have write access to convert it",
                        "ERROR")
            return False

        message_text = """\
This database was created by a previous version of %s
Do you wish me to update it to the latest database format?
(Warning - this will make it unreadable by older versions
of %s - so you would be wise to store a copy of the
database file elsewhere first.)""" % (vs_cfg["PROGRAM_ID"], vs_cfg["PROGRAM_ID"])
        if not vs_yesno(self, message_text, "Old database version"):
            vs_message(self, "Database load abandonned", "Information")
            return False
        self.startup_message("Converting database . . . .\n\n")
        # See if the database can be converted
        self.progress_increment()
        if not database_versions.versioncheck():
            vs_message(self,
                        "Unable to convert: Database load abandonned",
                        "ERROR")
            return False
        # database has been converted, so now re-check it
        self.progress_increment()
        status, message = check(self.progress_increment)
        if status:
            debug_log("Database check returns OK")
            return True
        else:
            vs_message(self, message, "ERROR")
            return False



#  ------------------ AppFrame

class AppFrame(vsFrame):
    """The main frame of the gui"""

    def __init__(self, parent):
        """Create the frame"""
        vsFrame.__init__(self, parent=parent, title=vs_cfg["PROGRAM_ID"])
        self.parent = parent
        # Create the sizer for the frame
        sizer=wx.BoxSizer(wx.VERTICAL)
        # Create a Tabbed Notebook
        self.tabbed_notebook=vsNotebook(self)
        sizer.Add(self.tabbed_notebook, 1, wx.EXPAND)
        debug_log("The notebook has been completed")
        # If the company id is 1, this means the company has not been set up yet, so show the manual page
        company=Company()
        if company.id == 1:
            debug_log("The company has not been configured")
            self.tabbed_notebook.SetSelection(8)
        else:
            # set the title of the frame as the company name
            company_name = company.name
            if company_name:
                self.SetTitle(company_name)
        # Set the sizer to the frame
        self.SetSizer(sizer)
        # Stop the frame manually reducing below the size set by the notebook
        sizer.SetSizeHints(self)
        # And fit the frame to the sizer
        self.Fit()
        # Bind the Quit Button event from the admin panel to OnCloseMe method
        self.Bind(wx.EVT_BUTTON, self.OnCloseMe, self.tabbed_notebook.admin_panel.quit_button)
        # and on closing, close the parent
        self.Bind(wx.EVT_CLOSE, self.onclose)

        # Create a timer to update the lock file every minute
        self.update_lockfile = wx.Timer(self)
        self.Bind(wx.EVT_TIMER, self.touch_lockfile, self.update_lockfile)
        self.update_lockfile.Start(60000, oneShot=False)

        if not vs_cfg["WRITE_ACCESS"]:
            vs_message(self, "Database locked by another application - read only access allowed", "READ ONLY")


    def touch_lockfile(self, event):
        "Update the time stamp on the lockfile"
        try:
            # only update if we have write access
            if not vs_cfg["WRITE_ACCESS"]: return
            # If there is no lockfile, do not cause an error
            # perhaps a new database is being created
            if os.path.isfile(vs_cfg["LOCK_FILEPATH"]):
                os.utime(vs_cfg["LOCK_FILEPATH"], None)
        except Exception, e:
            exception_log(e)


    def refresh_accounts(self, item=None, current_item=True):
        """Deletes current tree, re-assigns folders and accounts to tree items,
           and select the item,
           if no item given, and current item is False, selects first folder
           if no item given, and current item is True, leaves current selection"""
        self.tabbed_notebook.refresh_accounts(item, current_item)


    def refresh_panels(self):
        """Refresh the panels of the tabbed notebook"""
        self.tabbed_notebook.refresh_panels()


    def get_supplier(self):
        "Get the supplier currently selected on the purchase panel"
        return self.tabbed_notebook.purchases_panel.get_supplier()

    def get_customer(self):
        "Get the customer currently selected on the sales panel"
        return self.tabbed_notebook.sales_panel.get_customer()

    def OnCloseMe(self, event):
        """Quit button has been pressed"""
        message = "This shuts down %s, are you sure?" % vs_cfg["PROGRAM_ID"]
        caption = "Exit %s" % vs_cfg["PROGRAM_ID"]
        if vs_yesno(self, message, caption):
            self.Close()

    def onclose(self, evt):
        # Close the parent, when this closes
        self.parent.Close()



# -------- The main notebook showing the tabs Accounts, Sales, Purchases ...

class vsNotebook(wx.Notebook):
    """The Tabbed Notebook of the gui"""

    def __init__(self, parent):
        """Create the tabbed notebook, and each panel beneath each tab"""
        wx.Notebook.__init__(self, parent, size= (800,600))
        # Log the notebook creation
        debug_log("Creating notebook: %s" % self.__class__.__name__)
        self.SetBackgroundColour(vs_cfg["BACKGROUND_COLOUR"])
        self.parent = parent
       
        # Set the first tab page as accounts
        self.accounts_panel = accounts_tab.AccountsPanel(self)
        # Add accounts_panel to the Notebook
        self.AddPage(self.accounts_panel, " Accounts ")

        # The Sales panel
        self.sales_panel = sales_tab.SalesPanel(self)
        # Add sales_panel to the Notebook
        self.AddPage(self.sales_panel, " Sales ")

        # The Purchases panel
        self.purchases_panel = purchases_tab.PurchasesPanel(self)
        # Add purchases_panel to the Notebook
        self.AddPage(self.purchases_panel, " Purchases ")

        # The Movemoney panel
        self.movemoney_panel = movemoney_tab.MovemoneyPanel(self)
        # Add movemoney_panel to the Notebook
        self.AddPage(self.movemoney_panel, " Move Money ")

        # The VAT panel
        self.vat_panel = vat_tab.VatPanel(self)
        # Add vat_panel to the Notebook
        self.AddPage(self.vat_panel, " VAT ")

        # The Reports panel
        self.reports_panel = reports_tab.ReportsPanel(self)
        # Add reports_panel to the Notebook
        self.AddPage(self.reports_panel, " Reports ")

        # The Company panel
        self.company_panel = company_tab.CompanyPanel(self)
        # Add company_panel to the Notebook
        self.AddPage(self.company_panel, " Company ")

        # The Admin panel
        self.admin_panel = admin_tab.AdminPanel(self)
        # Add admin_panel to the Notebook
        self.AddPage(self.admin_panel, " Admin ")

        # The Manual panel
        self.manual_panel = manual_tab.ManualPanel(self)
        # Add manual_panel to the Notebook
        self.AddPage(self.manual_panel, " Manual ")


    def refresh_accounts(self, item=None, current_item=True):
        """Deletes current tree, re-assigns folders and accounts to tree items,
           and select the item,
           if no item given, and current item is False, selects first folder
           if no item given, and current item is True, leaves current selection"""
        self.accounts_panel.refresh(item, current_item)


    def refresh_panels(self):
        """This refresh ensures the movemoney memory buttons are renewed
           and the purchase, sales and vat panel buttons
           are set up correctly with the suppliers/ customers choice renewed.
           and under the company tab - if the logo is shown or not"""
        self.movemoney_panel.refresh()
        self.purchases_panel.refresh()
        self.sales_panel.refresh()
        self.vat_panel.refresh()
        self.reports_panel.refresh()
        self.company_panel.refresh()
        self.manual_panel.refresh()


# --------------- maingui - called from vatstuff.py and runs this gui

def maingui(database_filepath):
    """Called from vatstuff.py and starts the gui main loop"""
    app = App(database_filepath)

    # Run the main gui     
    app.MainLoop()

    # Unlock the database when the gui closes
    shut_lockfile()

    # Log the program has stopped
    info_log("%s stopped" % vs_cfg["PROGRAM_ID"])



