####### VATStuff #######
#
# gui_functions.py  - functions required by many of the gui modules
#
# Version : 0.7.0
# Date : 20100925
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#

"functions required by many of the gui modules"

import os
import datetime

from functools import wraps

import wx
import wx.grid

from vatstuff_packages.vs_database import change_transaction_date
from vatstuff_packages.vs_utilities import *


_ICONIMAGE = None   # stores the icon image to save it being re-read from disc multiple times
_BUSYCURSOR = False # flags if the cursor is being displayed as 'busy' or not.


################  stops write access if multiple instances of vatstuff are running

def writecheck(f):
    "Creates a decorator to allow a method if database write access is ok"
    # typically used to decorate a button method which is about to write to the database
    @wraps(f)
    def wrapper(self, *args, **kwds):
        if not vs_cfg["WRITE_ACCESS"]:
            # Another process has locked the database
            # 
            return vs_message(self,
                     "Function unavailable - another instance of VATStuff has locked the database",
                     "Database Locked")
        return f(self, *args, **kwds)
    return wrapper


########## button_wrap decorator - wraps a button ########
#
#   The button_wrap:
#      logs the button pressed event
#      sets a busy cursor
#      wraps the wrapped function in a try except clause
#
###########################################################

def button_wrap(f):
    """Creates a decorator to wrap a button

    The button_wrap decorator sets a busy cursor
    logs the button has been pressed
    wraps the wrapped function in a try except clause
    and after the call, unsets the busy cursor"""
    @wraps(f)
    def wrapper(self, *args, **kwds):
        # Turn on busy cursor
        busy_cursor_on()
        wx.Yield()
        # log the function
        debug_log("%s in %s" % (f.__name__, self.__class__.__name__))
        try:
            # Do the function
            result = f(self, *args, **kwds)
        except Exception, e:
            error_message("In method: %s\nin class: %s" % (f.__name__, self.__class__.__name__), e)
            result = None
        wx.Yield()
        # turn off the busy cursor
        busy_cursor_off()
        return result
    return wrapper


def close_frames():
    "Closes any open frame"
    # First close the contacts frame
    if vs_cfg["SEARCH_CONTACTS_FRAME"]:
        vs_cfg["SEARCH_CONTACTS_FRAME"].Close()
    if not vs_cfg["TRANSACTION_REFRESH"]: return
    # get a copy of vs_cfg["TRANSACTION_REFRESH"] since not sure
    # how closing frames that remove themselves from the list will
    # be handled by a loop over the list
    open_frame_list = vs_cfg["TRANSACTION_REFRESH"][:]
    while open_frame_list:
        frm = open_frame_list.pop()
        frm.Close()


def set_icon(frame_object):
    "This defines the image to use to create the frame top left icons"
    global _ICONIMAGE
    try:
        if not _ICONIMAGE:
            _ICONIMAGE = wx.Icon(vs_cfg["ICON_FILEPATH"], wx.BITMAP_TYPE_ANY)
        frame_object.SetIcon(_ICONIMAGE)
    except Exception:
        pass

# --- The busy cursor toggle functions

def busy_cursor_on():
    "Turns the cursor into a busy symbol"
    global _BUSYCURSOR
    if not _BUSYCURSOR:
        wx.BeginBusyCursor()
        _BUSYCURSOR = True


def busy_cursor_off():
    "Turns the cursor busy symbol off"
    global _BUSYCURSOR
    if _BUSYCURSOR:
        wx.EndBusyCursor()
        _BUSYCURSOR = False


# The vsFrame which has icon set and debug log of its creation 

class vsFrame(wx.Frame):
    """A Frame class which inherits from wx.Frame, it sets
       the icon, and logs the frame."""

    def __init__(self, parent, title, style=wx.DEFAULT_FRAME_STYLE):
        """Create the Frame"""
        wx.Frame.__init__(self, parent, title=title, style=style)
        set_icon(self)
        # Log the frame creation
        debug_log("Creating frame: %s" % self.__class__.__name__)


# A Dialog which has icon set and debug log of its creation

class vsDialog(wx.Dialog):
    """A Dialog class which inherits from wx.Dialog, it sets
       the icon, and logs the dialog."""

    def __init__(self, parent, title, style=wx.DEFAULT_DIALOG_STYLE):
        """Create the Dialog"""
        wx.Dialog.__init__(self, parent, title=title, style=style)
        set_icon(self)
        # Log the frame creation
        debug_log("Creating dialog: %s" % self.__class__.__name__)


# This message is used where normally a wx.MessageBox would be used. However defining our own
# messagebox allows this to have a parent - which sets the icon symbol, and also has a simpler
# call than wx.MessageBox

def vs_message(parent, message, caption):
    """Creates a message box, used for information - not yes/no questions
       if the caption is the string ERROR an error symbol is used, otherwise an information symbol"""
    if caption == "ERROR":
        debug_log("User error message: %s" % message)
        style = wx.OK | wx.ICON_ERROR
    else:
        style = wx.OK | wx.ICON_INFORMATION

    if not parent:
        parent = wx.GetApp().GetTopWindow()
    else:
        if not parent.IsTopLevel():
            parent = wx.GetTopLevelParent(parent)

    # If the cursor is busy, unbusy it - as
    # now waiting for a button press
    busy_cursor_off()

    dlg = wx.MessageDialog(parent = parent,
                           message = message,
                           caption=caption,
                           style=style)
    dlg.ShowModal()
    dlg.Destroy()


def vs_yesno(parent, message, caption):
    "Creates a message box, used for yes/no questions, returns True if yes chosen no otherwise"

    if not parent:
        parent = wx.GetApp().GetTopWindow()
    else:
        if not parent.IsTopLevel():
            parent = wx.GetTopLevelParent(parent)

    # If the cursor is busy, unbusy it - as
    # now waiting for a button press
    busy_cursor_off()

    dlg = wx.MessageDialog(parent = parent,
                           message = message,
                           caption=caption,
                           style = wx.YES_NO | wx.ICON_QUESTION)
    retCode = dlg.ShowModal()
    dlg.Destroy()
    if retCode == wx.ID_YES :
        return True
    else:
        return False


def error_message(message, e=None):
    """Displays and logs error message - used for exceptions rather than user input errors
       If the exception e is given, then also logs it"""

    # log the message at level error
    error_log(message)

    if e:
        exception_log(e)

    # Display the message using wx.MessageBox
    message = "ERROR: %s" % message

    if "LOG_FILEPATH" in vs_cfg:
        message += "\nFor error details, view logs at:\n%s" % vs_cfg["LOG_FILEPATH"]

    # If the cursor is busy, unbusy it - as
    # now waiting for a button press
    busy_cursor_off()

    wx.MessageBox(message,
                  caption="ERROR",
                  style=wx.OK | wx.ICON_ERROR)


def save_file_as(parent, file_extension, folder, filename):
    "Create the save file dialog, return (True, filepath), or (False, filepath)"
    # Pre-establish a file filter to show the extension
    wildcard = "%s file (*.%s)|*.%s|All files (*.*)|*.*" % (file_extension, file_extension, file_extension)
    dlg = wx.FileDialog(parent, message="Save file as ...",
                                defaultDir=folder,
                                defaultFile=filename,
                                wildcard=wildcard,
                                style=wx.SAVE|wx.OVERWRITE_PROMPT)

    # If the cursor is busy, unbusy it - as now waiting for a button press
    busy_cursor_off()
    # Show the dialog and retrieve the user response.
    if dlg.ShowModal() == wx.ID_OK:
        filepath = dlg.GetPath()
        dlg.Destroy()
        return True, filepath
    else:
        dlg.Destroy()
        return False, None


def browse_for_database(parent, style=wx.OPEN):
    "Create the browse for database dialog, return (True, filepath), or (False, filepath)"
    # Pre-establish a file filter to show the extension
    wildcard = "%s file (*.%s)|*.%s|All files (*.*)|*.*" % (vs_cfg["DATABASE_EXTENSION"],
                                                            vs_cfg["DATABASE_EXTENSION"],
                                                            vs_cfg["DATABASE_EXTENSION"])
    if style == wx.OPEN:
        message = "Open database file ..."
    elif style == wx.SAVE:
        message = "Save database file ..."
    else:
        message = "Set database file ..."

    dlg = wx.FileDialog(parent, message=message,
                                defaultDir=vs_cfg["DATABASE_FOLDER"],
                                defaultFile=vs_cfg["DATABASE_FILE"],
                                wildcard=wildcard,
                                style=style)
    # If the cursor is busy, unbusy it - as now waiting for a button press
    busy_cursor_off()
    # Show the dialog and retrieve the user response.
    if dlg.ShowModal() == wx.ID_OK:
        filepath = dlg.GetPath()
        dlg.Destroy()
        return True, filepath
    else:
        dlg.Destroy()
        return False, None


def get_logo_bitmap():
    "Returns the logo image as a bitmap, raises exception if image file does not exist"
    if not os.path.exists(vs_cfg["LOGO_FILEPATH"]):
        raise VS_ERROR, "Logo image file does not exist"

    logo_image= wx.Image(vs_cfg["LOGO_FILEPATH"], wx.BITMAP_TYPE_PNG)
    
    # Scale the image to fit inside vs_cfg["LOGO_MAXWIDTH"], vs_cfg["LOGO_MAXHEIGHT"]
    logo_width, logo_height = logo_image.GetSize()
    if logo_width > vs_cfg["LOGO_MAXWIDTH"] or logo_height > vs_cfg["LOGO_MAXHEIGHT"]:
        # scale the image down to the maximum width or height
        mul_w = float(vs_cfg["LOGO_MAXWIDTH"])/logo_width
        mul_h = float(vs_cfg["LOGO_MAXHEIGHT"])/logo_height
        # multiplier is the smallest of the two
        multiplier = mul_w if mul_w < mul_h else mul_h
        logo_image.Rescale(int(logo_width * multiplier),
                           int(logo_height * multiplier),
                           wx.IMAGE_QUALITY_HIGH)
    return wx.BitmapFromImage(logo_image)


def date_to_wxdate(python_date):
    "Given a python date object, returns a wx date object"
    wx_date = wx.DateTimeFromDMY(python_date.day, python_date.month-1, python_date.year)
    return wx_date

def wxdate_to_date(wx_date):
    "Given a wx date object, returns a python date object"
    iso_date = wx_date.FormatISODate().split("-")
    python_date = datetime.date(int(iso_date[0]), int(iso_date[1]), int(iso_date[2]))
    return python_date

def top_disable():
    """Disable panels of the tabbed_notebook apart from the manual"""
    # get top level window
    top_parent = wx.GetApp().startup_frame.app_frame
    top_parent.tabbed_notebook.accounts_panel.Enable(False)
    top_parent.tabbed_notebook.sales_panel.Enable(False)
    top_parent.tabbed_notebook.purchases_panel.Enable(False)
    top_parent.tabbed_notebook.movemoney_panel.Enable(False)
    top_parent.tabbed_notebook.vat_panel.Enable(False)
    top_parent.tabbed_notebook.reports_panel.Enable(False)
    top_parent.tabbed_notebook.company_panel.Enable(False)
    top_parent.tabbed_notebook.admin_panel.Enable(False)

def top_enable():
    "Enable panels of the tabbed_notebook"
    # get top level window
    top_parent = wx.GetApp().startup_frame.app_frame
    top_parent.tabbed_notebook.accounts_panel.Enable(True)
    top_parent.tabbed_notebook.sales_panel.Enable(True)
    top_parent.tabbed_notebook.purchases_panel.Enable(True)
    top_parent.tabbed_notebook.movemoney_panel.Enable(True)
    top_parent.tabbed_notebook.vat_panel.Enable(True)
    top_parent.tabbed_notebook.reports_panel.Enable(True)
    top_parent.tabbed_notebook.company_panel.Enable(True)
    top_parent.tabbed_notebook.admin_panel.Enable(True)

def top_frame():
    "Return the top frame"
    return wx.GetApp().startup_frame.app_frame


def top_refresh_panels():
    """Refresh notebook panels"""
    # get top level window
    top_parent = wx.GetApp().startup_frame.app_frame
    # and refresh panels
    top_parent.refresh_panels()


def top_refresh_accounts(item=None, current_item=True):
    """Deletes current tree, re-assigns folders and accounts to tree items,
       and select the item,
       if no item given, and current item is False, selects first folder
       if no item given, and current item is True, leaves current selection"""
    # get top level window
    top_parent = wx.GetApp().startup_frame.app_frame
    # and refresh account names
    top_parent.refresh_accounts(item, current_item)


def top_title(name=None):
    "Set the title of the top frame"
    # get top level window
    top_parent = wx.GetApp().startup_frame.app_frame
    if not name:
        company=Company()
        company_name = company.name
        if company_name:
            name = company_name
        else:
            name = vs_cfg["PROGRAM_ID"]
    top_parent.SetTitle(name)


def big_refresh(acc_tree=True, item=None, current_item=True, panels=True):
    """Refreshes all open windows that are listed in vs_cfg["TRANSACTION_REFRESH"].
       Refreshes all tab panels, if panels is True
       If acc_tree is True, refreshes the accounts in the tree under the accounts
       tab - and selects item
       if no item given, and current item is False, selects first folder
       if no item given, and current item is True, leaves current selection"""

    # As default; big_refresh() refreshes accounts tree and panels
    # which is what is needed after most transaction changes
    top_parent = wx.GetApp().startup_frame.app_frame

    if acc_tree:
        top_parent.refresh_accounts(item, current_item)
    if panels:
        top_parent.refresh_panels()
        top_title()
    # call the refresh function in every frame listed in vs_cfg["TRANSACTION_REFRESH"]
    for frm in vs_cfg["TRANSACTION_REFRESH"]:
        frm.refresh()


# The ConfirmTransactionDialog is used to confirm a transaction and 
# to allow a user to change the transaction text
# It is normally used by calling the function confirm_transaction

def confirm_transaction(parent, transaction_form):
    """Creates a ConfirmTransactionDialog dialog,
       returns (True, description) on confirmation or (False, description) otherwise"""
    confirm_transaction_dialog = ConfirmTransactionDialog(parent, transaction_form)
    # If the cursor is busy, unbusy it - as
    # now waiting for a button press
    busy_cursor_off()

    if confirm_transaction_dialog.ShowModal() == wx.ID_OK:
        new_description = confirm_transaction_dialog.description
        confirm_transaction_dialog.Destroy()
        debug_log("Transaction confirmed")
        return True, new_description
    else:
        confirm_transaction_dialog.Destroy()
        debug_log("Transaction abandonned")
        return False, transaction_form.description


class ConfirmTransactionDialog(vsDialog):
    """Called when a transaction is to be confirmed"""

    def __init__(self, parent, transaction_form):
        """Create the frame"""
        vsDialog.__init__(self, parent, title="Confirm Transaction")

        if not transaction_form.trans_list:
            raise VS_ERROR, "Missing transaction list"
        
        # Take a note of the variables
        self.description = transaction_form.description  # The memo text to put on the transaction
        self.trans_list = transaction_form.trans_list    # List of lists [account, debit, credit]
        self.datestamp = transaction_form.datestamp      # A Python date object
        
        if not self.description:
            self.description = "Memo to be shown in the account records"

        # Create a vertical sizer to hold everything
        sizer=wx.BoxSizer(wx.VERTICAL)

        # Create memo and date in a grid sizer
        memo_sizer=wx.FlexGridSizer(rows=2, cols=2, hgap=20, vgap=10)

        # memo
        memo_label=wx.StaticText(self, -1, "Transaction Memo:")
        memo_sizer.Add(memo_label, 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.memo_field = wx.TextCtrl(self, -1, self.description, size=(400, vs_cfg["BUTTON_HEIGHT"]))
        self.memo_field.SetToolTipString("The memo to be shown in the account records")
        self.memo_field.SetMaxLength(50)
        memo_sizer.Add(self.memo_field, 0, wx.ALIGN_LEFT)

        # Transaction date
        date_label=wx.StaticText(self, -1, "Transaction Date:")
        memo_sizer.Add(date_label, 0, wx.ALIGN_RIGHT)
        date_text = wx.StaticText(self, -1, self.datestamp.strftime("%d %b %Y"))
        memo_sizer.Add(date_text, 0, wx.ALIGN_LEFT)

        # add the memo_sizer to the main vertical sizer
        sizer.Add(memo_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        help_text=wx.StaticText(self, -1, """These are the accounts, and the amounts, to be debited
and credited, press Apply to implement the transaction.""")

        sizer.Add(help_text, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # -------------------- The transactions

        number = len(self.trans_list)

        # Create transactions in a grid sizer
        t_sizer=wx.FlexGridSizer(rows=number+1, cols=4, hgap=40, vgap=10)
        # number+1 ; the +1 is for the header text titles

        account_text=wx.StaticText(self, -1, "Account")
        t_sizer.Add(account_text, 0, wx.ALIGN_CENTER)

        debit_text=wx.StaticText(self, -1, "Debit")
        t_sizer.Add(debit_text, 0, wx.ALIGN_CENTER)

        credit_text=wx.StaticText(self, -1, "Credit")
        t_sizer.Add(credit_text, 0, wx.ALIGN_CENTER)

        balance_text=wx.StaticText(self, -1, "Balance")
        t_sizer.Add(balance_text, 0, wx.ALIGN_CENTER)

        for index, (account, debit, credit) in enumerate(self.trans_list):
            
            # The Account
            t_sizer.Add(wx.StaticText(self, -1, account.fullname), 0, wx.ALIGN_LEFT)

            # The Debit field
            t_sizer.Add(wx.StaticText(self, -1, to_currency_string(debit)), 0, wx.ALIGN_RIGHT)

            # The Credit field
            t_sizer.Add(wx.StaticText(self, -1, to_currency_string(credit)), 0, wx.ALIGN_RIGHT)

            # The New balance field
            new_balance = account.balance
            # Go through each previouse item in transaction_list in case the account is in multiple times
            for i in range(index+1):
                if self.trans_list[i][0] == account:
                    new_balance = new_balance + self.trans_list[i][1] - self.trans_list[i][2]
            t_sizer.Add(wx.StaticText(self, -1, to_currency_string(new_balance)), 0, wx.ALIGN_RIGHT)

        # add the t_sizer to the main vertical sizer
        sizer.Add(t_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # ---------------------- Apply - Cancel -- buttons

        button_size = (100, vs_cfg["BUTTON_HEIGHT"])

        # Create a horizontal button_sizer to hold the buttons
        button_sizer=wx.BoxSizer(wx.HORIZONTAL)

        # Apply button
        self.apply_button = wx.Button(self, wx.ID_APPLY, size=button_size)
        self.apply_button.SetToolTipString("Click to implement the transaction.")
        button_sizer.Add(self.apply_button, 0, wx.RIGHT, 40)

        # Cancel button
        self.cancel_button = wx.Button(self, wx.ID_CANCEL, size=button_size)
        self.cancel_button.SetToolTipString("Click to cancel.")
        button_sizer.Add(self.cancel_button, 0)

        # Add button_sizer to the main sizer
        sizer.Add(button_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # Set the size to the dialog frame
        self.SetSizer(sizer)
        sizer.SetSizeHints(self)

        # ---------- Bind buttons to handler functions
        
        self.Bind(wx.EVT_BUTTON, self.apply_button_pressed, self.apply_button)
        self.Bind(wx.EVT_BUTTON, self.cancel_button_pressed, self.cancel_button)


    # ----------------- Handler functions

    @writecheck
    @logmethod
    def apply_button_pressed(self, event):
        "Apply button - set the text - to be read by the calling process for transaction implementation"
        # Get new transaction description
        self.description = self.memo_field.GetValue().strip()
        self.EndModal(wx.ID_OK)


    def cancel_button_pressed(self, event):
        "Cancel button - return wx.ID_CANCEL"
        self.EndModal(wx.ID_CANCEL)


# The TwoDatesDialog is used to choose and confirm between two dates 
# It is normally used by calling the function two_dates

def two_dates(parent, title, help_text, from_date=None, to_date=None, from_enable=True, to_enable=True):
    """Creates a TwoDatesDialog dialog, returns (True, from_date, to_date) on confirmation
       or (False, from_date, to_date) otherwise"""
    two_dates_dialog = TwoDatesDialog(parent, title, help_text, from_date, to_date, from_enable, to_enable)

    # If the cursor is busy, unbusy it - as
    # now waiting for a button press
    busy_cursor_off()

    if two_dates_dialog.ShowModal() == wx.ID_OK:
        from_date = two_dates_dialog.from_date
        to_date = two_dates_dialog.to_date
        two_dates_dialog.Destroy()
        return True, from_date, to_date
    else:
        two_dates_dialog.Destroy()
        return False, from_date, to_date


class TwoDatesDialog(vsDialog):
    """Called when an action needing two dates is required"""

    def __init__(self, parent, title, help_text, from_date=None, to_date=None, from_enable=True, to_enable=True):
        """Create the Dialog"""
        vsDialog.__init__(self, parent, title=title)

        # if to_date is None, set self.to_date as today
        if to_date:
            self.to_date = to_date
        else:
            self.to_date = datetime.date.today()

        # if from_date is None, and to_date is greater than 10 days into the month
        # then set self.from_date as the beginning of this month,
        # or set self.from_date as the beginning of last month if the to_date is less than ten days into the month
        if from_date:
            self.from_date = from_date
        else:
            delta = datetime.timedelta(days=10)
            self.from_date = self.to_date - delta
            self.from_date = self.from_date.replace(day=1)

        wx_fromdate = date_to_wxdate(self.from_date)
        wx_todate = date_to_wxdate(self.to_date)

        button_size = (100, vs_cfg["BUTTON_HEIGHT"])

        # Create a vertical sizer to hold everything
        sizer=wx.BoxSizer(wx.VERTICAL)

        # show the help text
        sizer.Add(wx.StaticText(self, -1, help_text), 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # Create choices in a grid sizer
        field_sizer=wx.FlexGridSizer(rows=2, cols=2, hgap=20, vgap=40)

        # The from date
        field_sizer.Add(wx.StaticText(self, -1, "From:"), 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)
        self.from_date_field = wx.DatePickerCtrl(self, -1,
                                                 wx_fromdate, 
                                                 size=(200, vs_cfg["BUTTON_HEIGHT"]),
                                                 style=wx.DP_DROPDOWN | wx.DP_SHOWCENTURY)
        self.from_date_field.SetToolTipString("The from date")
        field_sizer.Add(self.from_date_field, 0, wx.ALIGN_LEFT)
        if not from_enable:
            self.from_date_field.Enable(False)

        # The to date
        field_sizer.Add(wx.StaticText(self, -1, "To:"), 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)
        self.to_date_field = wx.DatePickerCtrl(self, -1,
                                               wx_todate, 
                                               size=(200, vs_cfg["BUTTON_HEIGHT"]),
                                               style=wx.DP_DROPDOWN | wx.DP_SHOWCENTURY)
        self.to_date_field.SetToolTipString("The to date")
        field_sizer.Add(self.to_date_field, 0, wx.ALIGN_LEFT)
        if not to_enable:
            self.to_date_field.Enable(False)

        sizer.Add(field_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 40)

        # ---------------------- Apply - Cancel -- buttons

        # Create a horizontal button_sizer to hold the buttons
        button_sizer=wx.BoxSizer(wx.HORIZONTAL)

        # Apply button
        self.apply_button = wx.Button(self, wx.ID_APPLY, size=button_size)
        self.apply_button.SetToolTipString("Click to set the from and to dates of the report.")
        button_sizer.Add(self.apply_button, 0, wx.RIGHT, 40)

        # Cancel button
        self.cancel_button = wx.Button(self, wx.ID_CANCEL, size=button_size)
        self.cancel_button.SetToolTipString("Click to cancel.")
        button_sizer.Add(self.cancel_button, 0)

        # Add button_sizer to the main sizer
        sizer.Add(button_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # Set the size to the frame
        self.SetSizer(sizer)
        sizer.SetSizeHints(self)

        # ---------- Bind buttons to handler functions
        
        self.Bind(wx.EVT_BUTTON, self.apply_button_pressed, self.apply_button)
        self.Bind(wx.EVT_BUTTON, self.cancel_button_pressed, self.cancel_button)


    # ----------------- Handler functions


    def apply_button_pressed(self, event):
        """Apply button - check and apply all fields"""
        # Convert the dates input to python date objects
        # function wxdate_to_date is in module vatstuff_packages.vs_utilities.wx_utilities
        try:
            from_date = wxdate_to_date(self.from_date_field.GetValue())
            to_date = wxdate_to_date(self.to_date_field.GetValue())
        except Exception, e:
            vs_message(self, "Error in the dates format", "ERROR")
            return

        if to_date < from_date:
            vs_message(self, "The To date is earlier than the From date!", "ERROR")
            return
        self.from_date = from_date
        self.to_date = to_date
        self.EndModal(wx.ID_OK)


    def cancel_button_pressed(self, event):
        "Cancel button - return wx.ID_CANCEL"
        self.EndModal(wx.ID_CANCEL)


# The ShowTransactionDialog is used to view a transaction and edit its description
# and date. If it is a vat correction transaction, a long description is also available
# to be edited.

def show_transaction_dialog(parent, transaction):
    """Creates a ShowTransactionDialog"""
    show_dialog = ShowTransactionDialog(parent, transaction)
    # If the cursor is busy, unbusy it - as now waiting for a button press
    busy_cursor_off()
    show_dialog.ShowModal()
    show_dialog.Destroy()


class ShowTransactionDialog(vsDialog):
    """Called to show the transaction"""

    def __init__(self, parent, transaction):
        """Create the Dialog showing the transaction."""
        vsDialog.__init__(self, parent, title="Transaction : %i" % transaction.id)
        
        # Take a note of the variable
        self.transaction = transaction

        button_size = (100, vs_cfg["BUTTON_HEIGHT"])

        # Create a vertical sizer to hold everything
        self.sizer=wx.BoxSizer(wx.VERTICAL)

        # Find transaction type
        transaction_system = transaction.system

        if transaction_system == 11:
            # This is a vat correction transaction, include the vatcorrection
            # long description as an item of this transaction which can be edited
            self.vatcorrection = transaction.get_object()
            self.long_description = self.vatcorrection.long_description
            # field sizer holds the description and date fields and buttons
            field_sizer=wx.FlexGridSizer(rows=3, cols=2, hgap=20, vgap=10)
        else:
            self.vatcorrection = None
            self.long_description = None
            # field sizer holds the description and date fields and buttons
            field_sizer=wx.FlexGridSizer(rows=2, cols=2, hgap=20, vgap=10)

        # transaction memo
        self.memo_field = wx.TextCtrl(self, -1, self.transaction.description, size=(300, vs_cfg["BUTTON_HEIGHT"]))
        self.memo_field.SetToolTipString("Edit the transaction description if you wish")
        self.memo_field.SetMaxLength(50)
        field_sizer.Add(self.memo_field, 0, wx.ALIGN_RIGHT)

        self.memo_button = wx.Button(self, label="Change description", size=(200, vs_cfg["BUTTON_HEIGHT"]))
        self.memo_button.SetToolTipString("Click to change the description")
        field_sizer.Add(self.memo_button, 0, wx.ALIGN_LEFT)

        # If this is a vat correction, include a long description
        if self.vatcorrection:
            self.long_field = wx.TextCtrl(self, -1, self.long_description, size=(300, 4*vs_cfg["BUTTON_HEIGHT"]), style=wx.TE_MULTILINE)
            self.long_field.SetToolTipString("Edit the reason for the VAT correction if you wish")
            self.long_field.SetMaxLength(200)
            field_sizer.Add(self.long_field, 0, wx.ALIGN_RIGHT)

            self.long_change_button = wx.Button(self, label="Change long description", size=(200, vs_cfg["BUTTON_HEIGHT"]))
            self.long_change_button.SetToolTipString("Click to change the reason for the VAT correction")
            field_sizer.Add(self.long_change_button, 0, wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        # date
        transaction_date = self.transaction.datestamp
        wx_transaction_date = date_to_wxdate(transaction_date)
        self.date_field = wx.DatePickerCtrl(self, -1,
                                            wx_transaction_date, 
                                            size=(200, vs_cfg["BUTTON_HEIGHT"]),
                                            style=wx.DP_DROPDOWN | wx.DP_SHOWCENTURY)
        self.date_field.SetToolTipString("The transaction date")
        field_sizer.Add(self.date_field, 0, wx.ALIGN_RIGHT)

        self.date_button = wx.Button(self, label="Change date", size=(200, vs_cfg["BUTTON_HEIGHT"]))
        self.date_button.SetToolTipString("Click to change the transaction date")
        field_sizer.Add(self.date_button, 0, wx.ALIGN_LEFT)

        if transaction_system == 12:
            # data deletion transaction - date cannot be changed
            self.date_field.Enable(False)
            self.date_button.Enable(False)

        self.sizer.Add(field_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # static text date:
        self.text_date = wx.StaticText(self, -1, transaction_date.strftime("Transaction date : %d %B %Y"))
        self.sizer.Add(self.text_date, 0, wx.LEFT | wx.ALIGN_LEFT, 20)

        # ---------------------- Create the transaction table

        self.transaction_table = TransactionDialogTable(self, transaction)
        self.sizer.Add(self.transaction_table, 1, wx.ALL | wx.ALIGN_CENTER | wx.EXPAND, 20)

        # ------------------- Close button
        self.close_button = wx.Button(self, wx.ID_CLOSE, size=button_size)
        self.close_button.SetToolTipString("Click to close this window")
        self.sizer.Add(self.close_button, 0, wx.BOTTOM | wx.ALIGN_CENTER, 20)

        # Set the sizer
        self.SetSizer(self.sizer)
        self.sizer.SetSizeHints(self)

        self.Bind(wx.EVT_BUTTON, self.memo_button_pressed, self.memo_button)
        self.Bind(wx.EVT_BUTTON, self.date_button_pressed, self.date_button)
        self.Bind(wx.EVT_BUTTON, self.close_button_pressed, self.close_button)
        if self.vatcorrection:
            self.Bind(wx.EVT_BUTTON, self.long_change_button_pressed, self.long_change_button)

    @writecheck
    @button_wrap
    def memo_button_pressed(self, event):
        """change the transaction description"""
        description = self.memo_field.GetValue().strip()
        if self.transaction.description == description:
            # No change in the description - just return
            vs_message(self, "There was no change to the description", "WARNING")
            return
        self.transaction.description = description
        # refresh windows that need it, but not the accounts tree, as the description
        # does not appear there, nor in the panels
        big_refresh(acc_tree=False, panels=False)
        info_log("Transaction %i description changed" % self.transaction.id)
        vs_message(self, "Description changed", "DONE")


    @writecheck
    @button_wrap
    def date_button_pressed(self, event):
        """change the transaction date"""
        try:
            transaction_date = wxdate_to_date(self.date_field.GetValue())
        except Exception, e:
            vs_message(self, "The date is not valid", "ERROR")
            return
        if transaction_date == self.transaction.datestamp:
            # No change in the date - just return
            vs_message(self, "There was no change to the date", "WARNING")
            return
        # the change_transaction_date.check_date checks if the date is ok, returns True, message
        # if it is ok, or False, message if not
        status, message = change_transaction_date.check_date(self.transaction, transaction_date)
        if status:
            if not vs_yesno(self, message, "Confirm date change"):
                return
        else:
            vs_message(self, message, "ERROR")
            return
        # change_transaction_date.change_date does the date change, returns True if ok
        # or False if not
        busy_cursor_on()
        wx.Yield()
        status = change_transaction_date.change_date(self.transaction, transaction_date)
        if not status:
            vs_message(self, "Unable to implement the date change", "ERROR")
            return
        self.text_date.SetLabel(self.transaction.datestamp.strftime("Transaction date : %d %B %Y"))
        self.sizer.Layout()
        wx.Yield()
        # refresh windows that need it, but not the accounts tree, as the date
        # does not appear there, nor in the panels
        big_refresh(acc_tree=False, panels=False)
        vs_message(self, "Date changed", "DONE")


    @writecheck
    @button_wrap
    def long_change_button_pressed(self, event):
        """change the vat correction long description"""
        if not self.vatcorrection:
            return
        self.long_description = self.long_field.GetValue().strip()
        self.vatcorrection.long_description = self.long_description
        info_log("VAT correction long description changed")
        vs_message(self, "The reason for VAT correction changed", "DONE")


    def close_button_pressed(self, event):
        """Close the dialog"""
        self.EndModal(wx.ID_OK)


class TransactionDialogTable(wx.grid.Grid):

    def __init__(self, parent, transaction):
        wx.grid.Grid.__init__(self, parent, style=wx.WANTS_CHARS)

        t_rows = []

        trans_list = transaction.trans_list
        # trans_list being a list of lists - [account, debit, credit]

        for item in trans_list:
            t_rows.append((str(transaction.id),
                           item[0].fullname,
                           to_currency_string(item[1]),
                           to_currency_string(item[2])
                           ))

        num_rows = len(t_rows)
        num_cols = 3

        self.CreateGrid(num_rows, num_cols)
        self.EnableEditing(False)

        # create colum labels
        self.SetColLabelValue(0, "Account")
        self.SetColLabelValue(1, "Debit")
        self.SetColLabelValue(2, "Credit")

        self.SetDefaultRowSize(vs_cfg["BUTTON_HEIGHT"], False)
        self.SetDefaultCellAlignment(wx.ALIGN_LEFT, wx.ALIGN_CENTER)

        # Set the colums showing money as right aligned
        attr_right = wx.grid.GridCellAttr()
        attr_right.SetAlignment(wx.ALIGN_RIGHT, wx.ALIGN_CENTER)
        self.SetColAttr(1, attr_right)
        self.SetColAttr(2, attr_right)

        self.SetColSize(0, 200)
        self.SetColSize(1, 100)
        self.SetColSize(2, 100)

        self.SetColMinimalAcceptableWidth(100)
        self.SetRowLabelSize(100)

        for row, item in enumerate(t_rows):
            # Set row label as the transaction id
            self.SetRowLabelValue(row, item[0])
            # Account
            self.SetCellValue(row, 0, item[1])
            # Debit
            self.SetCellValue(row, 1, item[2])
            # Credit
            self.SetCellValue(row, 2, item[3])


# return html text for a disable report

def no_report():
    "Returns html text to indicate the report is no longer available"
    return u"""\
<html>
<body>
<h3>Report unavailable</h3>
<p>The database has been updated, therefore this report may no longer be valid</p>
<p><br /><br />Please close this window - and re-open, to reload a new report from the database</p>
</body>
</html>"""

# A progress bar dialog, with a callable incrementer instance

class Incrementer:
    "class to create a callable that can increment a progress bar"
    def __init__(self, parent, caption, text):
        self.val = 0
        self.prog_dlg = wx.ProgressDialog(caption,
                                text,
                                20,   # progress bar max is 20
                                parent)

    def __call__(self):
        if self.val < 20:
            self.val +=1
        if self.prog_dlg:
            self.prog_dlg.Update(self.val)

    def close(self):
        "closes the dialog"
        if self.prog_dlg:
            self.prog_dlg.Destroy()
        self.prog_dlg = None



