####### VATStuff #######
#
# items_table.py  - Displays a table of items for purchase order, invoice or credit note
#
# Version : 0.7.0
# Date : 20100923
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#

"""Displays a table of items for purchase order, invoice or credit note"""

import os
import decimal
import datetime

import wx
import wx.grid

from vatstuff_packages.vs_utilities import *

from vatstuff_packages.vs_gui.gui_functions import *


D = decimal.Decimal   # D used to avoid typing


# ------------ The Items Table

class ItemsTable(wx.grid.Grid):
    """Displays a table of items for purchase order, invoice or credit note"""

    def __init__(self, parent, table_type):
        "table_type is either purchaseorder, invoice or creditnote"
        wx.grid.Grid.__init__(self, parent, size=(1000,150), style=wx.WANTS_CHARS)

        self.parent = parent

        num_rows = 5
        num_cols = 6

        # Create a list of decimal zero amounts for the amount column
        self.amount_list = [D("0.00")] * num_rows 

        # Create a list of decimal zero amounts for the VAT column
        self.vat_list = [D("0.000")] * num_rows 

        # Create a list of True / False for the discount check column
        self.discount_list = [False] * num_rows 

        # Create a choice of items
        if table_type == "purchaseorder":
            self.all_items = PurchaseItem.get_all()
        elif table_type == "invoice":
            self.all_items = SellItem.get_all()
        elif table_type == "creditnote":
            # creditnote is a bit redundant as it uses the same items as sellitems
            self.all_items = SellItem.get_all()
        else:
            raise VS_ERROR, "Program error in items_table.py"

        self.item_selection = [ item.description for item in self.all_items ]

        # Create a choice of VAT rates
        all_vat_rates = VATRate.get_all()

        vat_rate_selection = [ str(rate) for rate in all_vat_rates ]

        # pre-pend the vat_rate_selection list with an empty string
        vat_rate_selection.insert(0, "")

        self.CreateGrid(num_rows, num_cols)

        # create colum labels
        self.SetColLabelValue(0, "Qty")
        self.SetColLabelValue(1, "Description")
        self.SetColLabelValue(2, "Unit Cost")
        self.SetColLabelValue(3, "Amount")
        self.SetColLabelValue(4, "Discount")
        self.SetColLabelValue(5, "VAT Rate")

        self.SetDefaultCellAlignment(wx.ALIGN_LEFT, wx.ALIGN_CENTER)
        self.SetDefaultRowSize(vs_cfg["BUTTON_HEIGHT"], False)

        # Set the Quantity and VAT column as centre aligned
        attr_center = wx.grid.GridCellAttr()
        attr_center.SetAlignment(wx.ALIGN_CENTER, wx.ALIGN_CENTER)

        self.SetColAttr(0, attr_center)
        self.SetColAttr(5, attr_center)

        # Set the unit cost as right aligned
        attr_right = wx.grid.GridCellAttr()
        attr_right.SetAlignment(wx.ALIGN_RIGHT, wx.ALIGN_CENTER)
        self.SetColAttr(2, attr_right)

        # Make Amount col 3 right aligned and read only
        attr_3 = wx.grid.GridCellAttr()
        attr_3.SetAlignment(wx.ALIGN_RIGHT, wx.ALIGN_CENTER)
        attr_3.SetReadOnly(True)
        self.SetColAttr(3, attr_3)

        # Make Discount col 4 centre aligned and read only
        attr_4 = wx.grid.GridCellAttr()
        attr_4.SetAlignment(wx.ALIGN_CENTER, wx.ALIGN_CENTER)
        attr_4.SetReadOnly(True)
        self.SetColAttr(4, attr_4)

        # Set the item choice for the description column
        self.item_editor = wx.grid.GridCellChoiceEditor(choices=self.item_selection, allowOthers=True)

        # Set the VAT symbols choice for the VAT column
        self.vat_editor = wx.grid.GridCellChoiceEditor(choices=vat_rate_selection, allowOthers=False)

        for row in range(num_rows):
            self.SetCellEditor(row, 1, self.item_editor)
            self.SetCellRenderer(row, 4, wx.grid.GridCellBoolRenderer())
            self.SetCellEditor(row, 5, self.vat_editor)

        self.SetColSize(0, 100)
        self.SetColSize(1, 400)
        self.SetColSize(2, 100)
        self.SetColSize(3, 100)
        self.SetColSize(4, 100)
        self.SetColSize(5, 120)

        self.SetColMinimalAcceptableWidth(100)
        self.SetRowLabelSize(50)

        # When an item is edited, add a further row to ensure there are always blank rows
        # displayed, and update calculations

        self.Bind(wx.grid.EVT_GRID_CELL_CHANGE, self.update)

        # when a left click is done on the Discount cell, toggle its value
        self.Bind(wx.grid.EVT_GRID_CELL_LEFT_CLICK, self.discount_toggle)


    def discount_toggle(self, evt):
        try:
            col = evt.GetCol()
            if col != 4:
                evt.Skip()
                return
            row = evt.GetRow()
            discount_bool = self.GetCellValue(row, 4)
            if discount_bool == "1":
                self.SetCellValue(row, 4, "")
            else:
                self.SetCellValue(row, 4, "1")
            # and update the totals
            self.update(evt)
            evt.Skip()
        except Exception, e:
            error_message("Failure in toggling the discount flag", e)


    def update(self, evt):
        "Grid cell changed, so update caculations and if necessary append a blank row."

        # Keep some blank rows at the bottom available for editing
        # if the user starts inserting data in the last four rows - 
        # add a further row
        try:
            row=evt.GetRow()
            col=evt.GetCol()

            current_number_of_rows = self.GetNumberRows()
            if row+4 > current_number_of_rows:
                self.AppendRows(numRows=1)
                self.SetCellEditor(current_number_of_rows, 1, self.item_editor)
                self.SetCellRenderer(current_number_of_rows, 4, wx.grid.GridCellBoolRenderer())
                self.SetCellEditor(current_number_of_rows, 5, self.vat_editor)
                self.amount_list += [D("0.00")]
                self.vat_list += [D("0.000")]
                self.discount_list += [False]


            # If the column changed is column 1 - the description,
            # check if the item selected is one of the choice items

            if col == 1:
                # Find which item was selected, and update unit cost and vat columns accordingly
                item_description = self.GetCellValue(row, col)
                try:
                    index = self.item_selection.index(item_description)
                except ValueError:
                    # this description is not in the list
                    index = None
                if index is not None:
                    item = self.all_items[index]
                    self.SetCellValue(row, 2, to_currency_string(item.cost))
                    self.SetCellValue(row, 4, "")
                    if self.parent.show_vat:
                        self.SetCellValue(row, 5, str(item.vatrate))
                    else:
                        self.SetCellValue(row, 5, "")

            # Update totals
            self.calculate_totals()
        except Exception, e:
            error_message("Failure in updating the items cost/VAT values", e)



    def hide_vat_column(self):
        "Called to set the VAT column to empty values and to hide it - used when a zero vat supplier is picked"
        number_of_rows = self.GetNumberRows()
        for row in range(0, number_of_rows):
            self.SetCellValue(row, 5, "")
        self.SetColLabelValue(5, "VAT ignored")
        self.SetColMinimalAcceptableWidth(0)
        self.SetColSize(5, 0)
        self.ForceRefresh()


    def show_vat_column(self):
        "Called to show the vat colum again"
        self.SetColSize(5, 120)
        self.SetColMinimalAcceptableWidth(100)
        self.SetColLabelValue(5, "VAT Rate")
        self.ForceRefresh()


    def calculate_totals(self):
        """Calculates totals, fills in the parent fields, and returns total_cost, total_vat
           if parent.show_vat is False, returns 0.00 for the total_vat value"""
        # Get the discount value from the parent
        status, self.discount = self.parent.get_discount()
        if not status:
            self.discount = D("0.0")

        number_of_rows = self.GetNumberRows()
        total_cost = D("0.00")

        # For each row
        for row in range(0, number_of_rows):
            # Get quantity
            qty = self.GetCellValue(row, 0).strip()
            try:
                qty_value = D(qty.replace(vs_cfg["CP"], u"."))
            except Exception:
                qty_value = D(0)

            # Get amount
            amount = self.GetCellValue(row, 2).strip()

            status, amount_value = string_to_decimal(amount)
            if not status:
                amount_value = D("0.00")

            result = qty_value * amount_value
            self.amount_list[row] = result.quantize(D('.01'), rounding=decimal.ROUND_DOWN)
            if qty == "" or amount == "":
                self.SetCellValue(row, 3, "")
            else:
                self.SetCellValue(row, 3, to_currency_string(self.amount_list[row]))
            # Fill in the discount list
            if self.GetCellValue(row, 4) == "1":
                self.discount_list[row] = True
            else:
                self.discount_list[row] = False
            # Get VAT - calculate the percentage of the discounted amount
            vatcell = self.GetCellValue(row, 5)
            if vatcell == "":
                vatrate = D(0)
            else:
                vatsymbol = vatcell[0]
                vat = VATRate.from_symbol(vatsymbol)
                if not vat:
                    vatrate = D(0)
                else:
                    vatrate = vat.rate

            if self.discount_list[row]:
                amount = self.amount_list[row] - ( self.amount_list[row] * self.discount ) / D("100.000")
                # so after the discount applied, round it to the nearest penny
                amount = amount.quantize(D('.01'), rounding=decimal.ROUND_HALF_EVEN)
            else:
                amount = self.amount_list[row]
            
            # Calculate the vat of this cost

            if vatrate == D(0):
                self.vat_list[row] = D("0.00")
            else:
                result = (amount * vatrate) / D("100.0000")
                self.vat_list[row] = result.quantize(D('.0001'), rounding=decimal.ROUND_HALF_EVEN)

            # Get total cost to show on parent panel
            total_cost += amount


        # and show it on the parent panel
        self.parent.cost_value_field.SetValue(to_currency_string(total_cost))

        # Get total vat
        if self.parent.show_vat:
            total_vat = sum(self.vat_list, D("0.0000"))
            total_vat = total_vat.quantize(D('.01'), rounding=decimal.ROUND_HALF_EVEN)
        else:
            total_vat = D("0.00")

        # and show it on the parent panel
        self.parent.VAT_value_field.SetValue(to_currency_string(total_vat))

        return total_cost, total_vat


    def check_items(self):
        """Check each item and return a (status, item_list, message) - status being True
           if all ok, False otherwise, item_list being a list of items, each element
           being a dictionary of
           {"Qty":Decimal, "Description":string, "Unit Cost":Decimal, "Discount":boolean "VAT":Decimal (The % value)}
           message being None if ok, or an error message if not"""

        current_number_of_rows = self.GetNumberRows()

        item_list = []
        blank = False      # this flag tracks for blank lines

        # Check each row in turn
        for row in range(current_number_of_rows):
            item = {}

            # Get quantity
            qty = self.GetCellValue(row, 0).strip()
            if blank and qty != "":
                message = """An item follows a blank line at line %s.
Do not leave blank lines between items""" % (row+1,)
                return False, item_list, message
            if qty == "":
                blank = True
            else:
                try:
                    decimal_qty = D(qty.replace(vs_cfg["CP"], u"."))
                except Exception:
                    message = "The quantity is not valid at line %s" % (row+1,)
                    return False, item_list, message
                if decimal_qty <= D(0):
                    message = "Invalid quantity at line %s" % (row+1,)
                    return False, item_list, message
                item["Qty"] = decimal_qty

            # Get description
            description = self.GetCellValue(row, 1).strip()
            if blank and description != "":
                message = "The quantity is not valid at line %s" % (row+1,)
                return False, item_list, message
            if not blank:
                if description == "":
                    message = "The description is not valid at line %s" % (row+1,)
                    return False, item_list, message
                item["Description"] = description

            # Get cost
            cost = self.GetCellValue(row, 2).strip()
            if blank and cost != "":
                message = "The quantity is not valid at line %s" % (row+1,)
                return False, item_list, message
            if not blank:
                status, cost_value = string_to_decimal(cost)
                if not status:
                    message = "The unit cost is not valid at line %s" % (row+1,)
                    return False, item_list, message
                if cost_value < D(0):
                    message = "The unit cost is not valid at line %s" % (row+1,)
                    return False, item_list, message
                item["Unit Cost"] = cost_value

            # Fill in the discount list
            if not blank:
                # Get the discount value from the parent, if zero, set all Discounts to False
                status, self.discount = self.parent.get_discount()
                if (not status) or self.discount == D(0):
                    item["Discount"] = False
                else:
                    if self.GetCellValue(row, 4) == "1":
                        item["Discount"] = True
                    else:
                        item["Discount"] = False

            # Get VAT
            vatcell = self.GetCellValue(row, 5).strip()
            if not blank:
                if self.parent.show_vat:
                    if vatcell == "":
                        message = "The VAT is not valid at line %s" % (row+1,)
                        return False, item_list, message
                    vatsymbol = vatcell[0]
                    vat = VATRate.from_symbol(vatsymbol)
                    if not vat:
                        message = "The VAT is not valid at line %s" % (row+1,)
                        return False, item_list, message
                    item["VAT"] = vat.rate
                else:
                    item["VAT"] = D("0.0")

            # Each item for this row ok, add to the item_list
            if not blank:
                item_list.append(item)

        return True, item_list, None


