####### VATStuff #######
#
# more_moves.py  - Creates a frame with selection list allowing multiple move money user definitions
#
# Version : 0.6
# Date : 20100805
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#


"""Creates a frame with selection list allowing multiple move money user definitions"""


import wx

from vatstuff_packages.vs_utilities import *



from vatstuff_packages.vs_database.create_objects import create_Label



from vatstuff_packages.vs_gui.movemoney import complex_move

from vatstuff_packages.vs_gui.gui_functions import *


def more_moves_frame():
    """Show the MoreMovesFrame"""

    # Check the frame is not already open
    for frm in vs_cfg["TRANSACTION_REFRESH"]:
        if isinstance(frm, MoreMovesFrame):
            frm.Raise()
            break
    else:
        # top_frame() is the top frame, which will be the parent
        frm = MoreMovesFrame(top_frame())
        frm.Show()


class MoreMovesFrame(vsFrame):
    "Called when the more.. button is pressed on the movemoney tab panel"

    def __init__(self, parent):
        "Create the frame - holding a MoreMovesPanel"
        vsFrame.__init__(self, parent, title="More move money definitions")

        # Create the MoreMovesPanel
        more_moves_panel = MoreMovesPanel(self)

        # Get the panel size
        panel_size = more_moves_panel.GetMinSize()

        # Set minimum equal to the panel
        self.SetMinSize(panel_size)
        self.SetClientSize(panel_size)

        # update the list of windows which need a refresh if
        # a transaction occurs, by adding self
        vs_cfg["TRANSACTION_REFRESH"].append(self)

        self.Bind(wx.EVT_CLOSE, self.onclose)


    def refresh(self):
        "pass on this"
        # refresh function needed as it is called by big_refresh()
        # but for this frame it does nothing, as there is no need
        # for the big_refresh to refresh the move definitions list
        pass


    def onclose(self, evt):
        "Remove from the open frame list and shut down"
        # Decrement the refresh list
        vs_cfg["TRANSACTION_REFRESH"].remove(self)
        # and destroy the frame
        self.Destroy()



# ----------------------MoreMovesPanel

class MoreMovesPanel(wx.Panel):
    """The panel of the MoreMoves frame"""

    def __init__(self, parent):
        """Create the panel"""
        wx.Panel.__init__(self, parent)
        
        # Take a note of the variables
        self.parent = parent

        button_size = (100, vs_cfg["BUTTON_HEIGHT"])
        field_size  = (250, vs_cfg["BUTTON_HEIGHT"])

        # Create a vertical sizer to hold everything
        self.sizer=wx.BoxSizer(wx.VERTICAL)

        help_text = """\
As well as the money movement definitions set on the buttons, you can create further definitions here
(choose 'New' to create a definition). Select an existing definition from the choice below and choose
'Open' to run or edit it, or 'Delete' to delete it. The 'Close' button shuts down this window."""
        self.sizer.Add(wx.StaticText(self, -1, help_text), 0, wx.ALL | wx.ALIGN_CENTER, 30)

        not_button_labels = get_not_button_labels()
        # a list of label objects, not assigned to buttons
        # ordered by name
      
        
        label_list = [ [label, label.name] for label in not_button_labels ]

        if (not label_list) or label_list ==[[]]:
            self.labels = []
            self.label_names = [" "]
            self.label_selected = None
            description_text = " "
        else:
            labels, label_names = zip(*label_list)
            self.labels = list(labels)
            self.label_names = list(label_names)
            self.label_selected = self.labels[0]
            description_text = self.label_selected.description

        self.label_field = wx.Choice(self, -1, size=field_size, choices=self.label_names)
        self.label_field.SetSelection(0)
        self.label_field.SetToolTipString("Choose a move definition")
        self.sizer.Add(self.label_field, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        self.description_field = wx.StaticText(self, -1, description_text)
        self.sizer.Add(self.description_field, 0, wx.ALL | wx.ALIGN_CENTER, 20)


        # ---------------------- New - Delete - Open - Close -- buttons

        # Create a horizontal button_sizer to hold the buttons
        button_sizer=wx.BoxSizer(wx.HORIZONTAL)

        # New button
        self.new_button = wx.Button(self, label="New", size=button_size)
        self.new_button.SetToolTipString("Click to create a new move definition.")
        button_sizer.Add(self.new_button, 0, wx.RIGHT, 20)

        # Delete button
        self.delete_button = wx.Button(self, label="Delete", size=button_size)
        self.delete_button.SetToolTipString("Click to delete a move definition.")
        button_sizer.Add(self.delete_button, 0, wx.RIGHT, 20)
        if not self.labels:
            self.delete_button.Enable(False)

        # Open button
        self.open_button = wx.Button(self, label="Open", size=button_size)
        self.open_button.SetToolTipString("Click to open a move definition.")
        button_sizer.Add(self.open_button, 0, wx.RIGHT, 20)
        if not self.labels:
            self.open_button.Enable(False)

        # Close button
        self.close_button = wx.Button(self, wx.ID_CLOSE, size=button_size)
        self.close_button.SetToolTipString("Click to close.")
        button_sizer.Add(self.close_button, 0)

        # Add button_sizer to the main sizer
        self.sizer.Add(button_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 30)

        # Set the size to the panel
        self.SetSizer(self.sizer)
        self.sizer.SetSizeHints(self)

        # Bind the fields
        self.Bind(wx.EVT_CHOICE, self.label_changed, self.label_field)
        self.Bind(wx.EVT_BUTTON, self.new_button_pressed, self.new_button)
        self.Bind(wx.EVT_BUTTON, self.delete_button_pressed, self.delete_button)
        self.Bind(wx.EVT_BUTTON, self.open_button_pressed, self.open_button)
        self.Bind(wx.EVT_BUTTON, self.close_button_pressed, self.close_button)


    def label_changed(self, evt):
        "A new label is selected, change the description"
        try:
            # Get label currently shown
            label_selected_index = self.label_field.GetSelection()
            self.label_selected = self.labels[label_selected_index]

            # Place the label description into the self.description_field static text field
            self.description_field.SetLabel(self.label_selected.description)

            self.sizer.Layout()
        except Exception, e:
            error_message("Failure choosing a change definition", e)


    @writecheck
    @button_wrap
    def new_button_pressed(self, evt):
        "Create a new label and show it in the Complexmove Frame"
        # First try to find a unique label name
        not_button_labels = get_not_button_labels()
        label_names = [ label.name for label in not_button_labels ]

        n = vs_cfg["MAX_BUTTONS"]+1
        while n<99:
            temp_name = "Undefined_%02d" % n
            if not temp_name in label_names:
                break
            n += 1
            temp_name = "Undefined"

        new_label = create_Label(name = temp_name, description = "Undefined")
        self.delete_button.Enable(True)
        self.open_button.Enable(True)

        self.label_selected = new_label
        self.refresh()
        complex_move.complex_move_dialog(self, self.label_selected)


    @writecheck
    @button_wrap
    def delete_button_pressed(self, evt):
        "Delete the label selected"
        if not self.labels:
            return

        label_selected_index = self.label_field.GetSelection()
        label_name = self.label_names[label_selected_index]

        # Ask for confirmation
        if not vs_yesno(self, "This deletes <%s>, are you sure?" % label_name, "Delete definition"):
            return

        self.label_selected = self.labels[label_selected_index]
        if not self.label_selected.delete():
            raise VS_ERROR, "Error deleting label"

        # After deletion, refresh the list
        self.refresh()
        if not self.labels:
            self.delete_button.Enable(False)
            self.open_button.Enable(False)


    @button_wrap
    def open_button_pressed(self, evt):
        "Show ComplexMoveDialog for the label currently selected"
        if not self.labels:
            return
        label_selected_index = self.label_field.GetSelection()
        self.label_selected = self.labels[label_selected_index]
        complex_move.complex_move_dialog(self, self.label_selected)


    def close_button_pressed(self, evt):
        """Close button - Close parent"""
        self.parent.Close()
        return


    def refresh(self):
        "Gets labels from database and re-creates the selection list"
        not_button_labels = get_not_button_labels()
        # a list of label objects, not assigned to buttons, ordered by name
        
        label_list = [ [label, label.name] for label in not_button_labels ]

        if (not label_list) or label_list ==[[]]:
            self.labels = []
            self.label_names = [" "]
            self.label_selected = None
            description_text = " "
            selected_index = 0
        else:
            labels, label_names = zip(*label_list)
            self.labels = list(labels)
            self.label_names = list(label_names)
            try:
                selected_index = self.labels.index(self.label_selected)
            except Exception:
                selected_index = 0
            self.label_selected = self.labels[selected_index]
            description_text = self.label_selected.description

        self.label_field.Clear()
        self.label_field.AppendItems(self.label_names)

        # Set list selection
        self.label_field.SetSelection(selected_index)
        # Set description
        self.description_field.SetLabel(description_text)

        self.sizer.Layout()



