####### VATStuff #######
#
# simple_move.py  - defines a dialog to move money between accounts
#
# Version : 0.6
# Date : 20100805
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#


"""Creates a table to move money between accounts

It is run as part of the vatstuff gui interface.
"""


import wx
import datetime
import decimal





from vatstuff_packages.vs_database import transactions

from vatstuff_packages.vs_utilities import *

from vatstuff_packages.vs_gui.gui_functions import *

D = decimal.Decimal   # D used to avoid typing



def simple_move_dialog(parent, transaction_text, transaction_list):
    """Creates a SimpleMoveDialog"""
    simple_move_dialog = SimpleMoveDialog(parent, transaction_text, transaction_list)

    # If the cursor is busy, unbusy it - as
    # now waiting for a button press
    busy_cursor_off()

    simple_move_dialog.ShowModal()
    simple_move_dialog.Destroy()


class SimpleMoveDialog(vsDialog):
    "Creates a transaction set and implements it"

    def __init__(self, parent, transaction_text, transaction_list):
        """Create the Dialog - holding a simple money move between accounts"""
        vsDialog.__init__(self, parent, "Create a transaction")

        # Take a note of the variables
        self.parent = parent
        self.transaction_text = transaction_text   # The memo text to put on the transaction
        self.transaction_list = transaction_list   # List of lists [account, debit, credit]
        
        self.datestamp = wx.DateTime.Now()
        
        if self.transaction_text == None or self.transaction_text == "":
            self.transaction_text = "Memo to be shown in the account records"

        # Create a vertical sizer to hold everything
        sizer=wx.BoxSizer(wx.VERTICAL)

        # Create buttons and descriptions in a grid sizer
        b_sizer=wx.FlexGridSizer(rows=2, cols=2, hgap=20, vgap=10)

        # Transaction text
        transaction_label=wx.StaticText(self, -1, "Transaction Memo")
        b_sizer.Add(transaction_label, 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.transaction_field = wx.TextCtrl(self, -1, self.transaction_text, size=(400, vs_cfg["BUTTON_HEIGHT"]))
        self.transaction_field.SetToolTipString("The memo to be shown in the account records")
        self.transaction_field.SetMaxLength(50)


        b_sizer.Add(self.transaction_field, 0, wx.ALIGN_LEFT)

        # Transaction date
        transaction_date_text=wx.StaticText(self, -1, "Transaction Date")
        b_sizer.Add(transaction_date_text, 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.transaction_date_field = wx.DatePickerCtrl(self, -1,
                                                        self.datestamp, 
                                                        size=(200, vs_cfg["BUTTON_HEIGHT"]),
                                                        style=wx.DP_DROPDOWN | wx.DP_SHOWCENTURY)
        self.transaction_date_field.SetToolTipString("The transaction date")

        b_sizer.Add(self.transaction_date_field, 0, wx.ALIGN_LEFT)

        # add the b_sizer to the main vertical sizer
        sizer.Add(b_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        help_text=wx.StaticText(self, -1, """Set those accounts, and the amounts you wish to debit and
credit, and then press Apply to implement the transaction.""")

        sizer.Add(help_text, 0, wx.ALL | wx.ALIGN_CENTER, 10)


        # -------------------- The transaction panel

        # The transaction_panel
        self.transaction_panel = TransactionPanel(self)
        
        # Fill the transaction_panel with the self.transaction_list
        if self.transaction_list != None:
            self.transaction_panel.setup(self.transaction_list)
        

        # Create a horizontal box sizer to hold the transaction panel
        trans_sizer=wx.BoxSizer(wx.HORIZONTAL)

        # add a bit of space
        trans_sizer.Add((40,40), 1)

        # Add the transaction panel to the sizer
        trans_sizer.Add(self.transaction_panel, 0, wx.ALIGN_CENTER)

        # add a bit of space
        trans_sizer.Add((40,40), 1)

        sizer.Add(trans_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # ---------------------- Apply - Cancel -- buttons

        button_size = (100, vs_cfg["BUTTON_HEIGHT"])

        # Create a horizontal button_sizer to hold the buttons
        button_sizer=wx.BoxSizer(wx.HORIZONTAL)

        # Apply button
        self.apply_button = wx.Button(self, wx.ID_APPLY, size=button_size)
        self.apply_button.SetToolTipString("Click to implement the transaction.")
        button_sizer.Add(self.apply_button, 0, wx.RIGHT, 40)

        # Cancel button
        self.cancel_button = wx.Button(self, wx.ID_CANCEL, size=button_size)
        self.cancel_button.SetToolTipString("Click to cancel.")
        button_sizer.Add(self.cancel_button, 0, wx.LEFT)

        # Add button_sizer to the main sizer
        sizer.Add(button_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # Set the size to the panel
        self.SetSizer(sizer)
        sizer.SetSizeHints(self)

        # ---------- Bind buttons to handler functions
        
        self.Bind(wx.EVT_BUTTON, self.apply_button_pressed, self.apply_button)
        self.Bind(wx.EVT_BUTTON, self.cancel_button_pressed, self.cancel_button)


    # ----------------- Handler functions

    @writecheck
    @button_wrap
    def apply_button_pressed(self, event):
        "Apply button - check and apply all fields"

        # Get top fields
        transaction_text = self.transaction_field.GetValue().strip()

        try:
            transaction_date = wxdate_to_date(self.transaction_date_field.GetValue())
        except Exception:
            vs_message(self, "The date is not valid", "ERROR")
            return

        # check date
        if vs_cfg["DELETE_TO_DATE"]:
            if transaction_date < vs_cfg["DELETE_TO_DATE"]:
                vs_message(self, "A date prior to the last database deletion date cannot be accepted", "ERROR")
                return

        # Check transaction fields,
        status, transaction_list = self.transaction_panel.check_fields()
        if not status:
            return

        wx.Yield()
        # Confirm the transaction (function and dialog in gui_functions)
        status, transaction_text = confirm_transaction(self,
                                                        transaction_text,
                                                        transaction_list,
                                                        transaction_date)
        if not status:
            vs_message(self, "The transaction has not been implemented", "CANCELLED")
            return

        # Implement the transaction
        busy_cursor_on()
        wx.Yield()
        transaction_id = transactions.implement(None, transaction_date, transaction_text, transaction_list)
        if not transaction_id:
            error_message("Failed to implement the transaction")
            return

        # and call big_refresh as a new transaction has been implemented
        big_refresh()
        vs_message(self, "Transaction implemented", "DONE")
        self.EndModal(wx.ID_OK)


    def cancel_button_pressed(self, event):
        """ Cancel button - Close parent"""
        self.EndModal(wx.ID_CANCEL)


# TransactionPanel - this class holds the account, debit and credit fields

class TransactionPanel(wx.Panel):
    """Holds the account, debit and credit fields in a panel"""
    
    max_items = 8    # the maximum number of rows to display

    def __init__(self, parent):
        """Create the panel"""
        wx.Panel.__init__(self, parent)
        
        # Take a note of the parent
        self.parent = parent

        # Create transactions in a grid sizer
        sizer=wx.FlexGridSizer(rows=TransactionPanel.max_items+1, cols=3, hgap=20, vgap=10)
        # TransactionPanel.max_items+1 the +1 is for the header text titles

        account_text=wx.StaticText(self, -1, "Account")
        sizer.Add(account_text, 0, wx.ALIGN_CENTER)

        debit_text=wx.StaticText(self, -1, "Debit")
        sizer.Add(debit_text, 0, wx.ALIGN_CENTER)

        credit_text=wx.StaticText(self, -1, "Credit")
        sizer.Add(credit_text, 0, wx.ALIGN_CENTER)

        # Get a list of accounts - to be used by the account Choice boxes
         
        # self.account_objects_list is a list of account objects
        # self.account_names_list is a list of account names in foldername:accountname format
        # the two lists are in the same order
        self.account_objects_list, self.account_names_list = get_folder_account_lists()

        self.account_names_list.insert(0, "")              # Insert a blank at the beginning of the list
        self.account_objects_list.insert(0, None)          # Keep two lists ordered together

        self.widget_list=[]   # to consist of lists of account, debit, credit wx field widget id's

        # create the transaction fields
        for tran in range(0, TransactionPanel.max_items):
            # Account field
            a_field = wx.Choice(self, -1, size=(200, vs_cfg["BUTTON_HEIGHT"]), choices=self.account_names_list)
            acc_id = a_field.GetId()
            a_field.SetToolTipString("Set Account")
            sizer.Add(a_field, 0, wx.ALIGN_CENTER)

            # Debit field
            d_field = wx.TextCtrl(self, -1, vs_cfg["SYMBOL_ZERO_STRING"], style=wx.ALIGN_RIGHT, size=(100, vs_cfg["BUTTON_HEIGHT"]))
            deb_id = d_field.GetId()
            d_field.SetToolTipString("Set Debit")
            d_field.SetMaxLength(10)
            sizer.Add(d_field, 0, wx.ALIGN_CENTER)

            # Credit field
            c_field = wx.TextCtrl(self, -1, vs_cfg["SYMBOL_ZERO_STRING"], style=wx.ALIGN_RIGHT, size=(100, vs_cfg["BUTTON_HEIGHT"]))
            crd_id = c_field.GetId()
            c_field.SetToolTipString("Set Credit")
            c_field.SetMaxLength(10)
            sizer.Add(c_field, 0, wx.ALIGN_CENTER)

            # record the id's
            self.widget_list.append([acc_id, deb_id, crd_id])

        # Set the size to the panel
        self.SetSizer(sizer)

        self.Fit()


    def check_fields(self):
        """Returns [boolean status, list of results]

        status is True if all fields are ok, False otherwise
        results is a list of lists - [account, debit, credit]"""

        result_list = []
        
        # Check for all lines without account values
        blanks = 0
        for item in range(0,TransactionPanel.max_items):
            # Account field
            acc_field = self.FindWindowById(self.widget_list[item][0])
            account_selected_list_item = acc_field.GetCurrentSelection()
            if account_selected_list_item < 1:         # blank or unselected
                blanks += 1

        if blanks == TransactionPanel.max_items:
            vs_message(self, "No accounts have been set", "ERROR")
            return False, None


        for tran in range(0,TransactionPanel.max_items):
            blank = False
            # Get account into result
            result = []
            # self.widget_list[tran] consists of [acc_id, deb_id, crd_id]

            # Account field
            a_field = self.FindWindowById(self.widget_list[tran][0])
            account_selected_list_item = a_field.GetCurrentSelection()
            if account_selected_list_item < 1:         # blank or unselected
                blank = True
            else:
                account_selected = self.account_objects_list[account_selected_list_item]
                result.append(account_selected)

            # Debit field
            d_field = self.FindWindowById(self.widget_list[tran][1])
            # Get the value
            debit=d_field.GetValue()
            try:
                debit=Currency(debit)
            except CURRENCY_ERROR:
                vs_message(self, "Invalid debit at line %i" % (tran+1), "ERROR")
                return False, None
            if debit.negative:
                vs_message(self, "Invalid Debit at line %i, values must be positive" % (tran+1), "ERROR")
                return False, None
            
            # Credit field
            c_field = self.FindWindowById(self.widget_list[tran][2])
            # Get the value
            credit=c_field.GetValue()
            try:
                credit=Currency(credit)
            except CURRENCY_ERROR:
                vs_message(self, "Invalid credit at line %i" % (tran+1), "ERROR")
                return False, None
            if credit.negative:
                vs_message(self, "Invalid Credit at line %i, values must be positive" % (tran+1), "ERROR")
                return False, None
      
            if blank and (debit or credit):
                vs_message(self, "No account set at line %i" % (tran+1), "ERROR")
                return False, None

            if blank:
                continue    # Goto next iteration

            # Incorrect if both credit and debit are zero
            if not (debit or credit):
                vs_message(self, "Invalid line %i - no credit or debit values" % (tran+1), "ERROR")
                return False, None

            # If both a debit and a credit have been put in - that's not right
            if debit and credit:
                vs_message(self, "Invalid line %i, either credit or debit, not both." % (tran+1), "ERROR")
                return False, None

            # If this account is already in the result, then add these debit, credit to the previous values
            for key, item in enumerate(result_list):
                if result[0] == item[0]:
                    # This account is the same as an account already in the result_list
                    account_debit = item[1] + debit.dec
                    account_credit = item[2] + credit.dec
                    if account_debit == account_credit:
                        # Remove these items from result_list
                        result_list.pop(key)
                    elif account_debit > account_credit:
                        result_list[key][1] = account_debit - account_credit
                        result_list[key][2] = D("0.00")
                    else:
                        result_list[key][1] = D("0.00")
                        result_list[key][2] = account_credit - account_debit
                    # No new line is added to result_list
                    blank = True
                    break

            if blank:
                continue    # Goto next iteration

            # add debit, credit to the result - so it is [account, debit, credit]
            result.append(debit.dec)
            result.append(credit.dec)
            # add result to the result list
            result_list.append(result)

        if not result_list:
            vs_message(self, "This is a zero value transaction!", "ERROR")
            return False, None

        credit_sum = D("0.00")
        debit_sum = D("0.00")
        for item in result_list:
            debit_sum = debit_sum + item[1]
            credit_sum = credit_sum + item[2]

        # Check the transaction balances
        if debit_sum != credit_sum:
            vs_message(self, "The debits do not equal the credits", "ERROR")
            return False, None
            
        # Check the transaction is not zero
        if not debit_sum:
            vs_message(self, "This is a zero value transaction!", "ERROR")
            return False, None
       
        return True, result_list


    def has_system_account(self):
        "Returns True if any of the accounts chosen are a system account, False otherwise"

        for tran in range(0,TransactionPanel.max_items):
            # self.widget_list[tran] consists of [acc_id, deb_id, crd_id]
            # Account field
            a_field = self.FindWindowById(self.widget_list[tran][0])
            account_selected_list_item = a_field.GetCurrentSelection()
            if account_selected_list_item > 0:
                account = self.account_objects_list[account_selected_list_item]
                if account.system > 1:
                    return True
        return False


    def setup(self, transaction_list):
        "Sets up the panel with the values from this transaction list"
        if transaction_list == None: return
        
        for t, t_list in enumerate(transaction_list):        # (t,t_list) = (0, transaction_list[0]) etc....
            account = t_list[0]
            acc_field = self.FindWindowById(self.widget_list[t][0])
            if account == None:
                acc_field.SetSelection(0)
            else:
                acc_field.SetSelection(self.account_objects_list.index(account))
            deb_field = self.FindWindowById(self.widget_list[t][1])
            deb_field.SetValue(unicode(Currency(transaction_list[t][1])))
            crd_field = self.FindWindowById(self.widget_list[t][2])
            crd_field.SetValue(unicode(Currency(transaction_list[t][2])))
        
                


