####### VATStuff #######
#
# organization.py  - create or edit a customer or supplier
#
# Version : 0.6
# Date : 20100731
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#


"""create or edit a customer or supplier

It is run as part of the vatstuff gui interface.
"""

import wx



from vatstuff_packages.vs_database.create_objects import create_Customer, create_Supplier, create_Address

from address_panel import *

from vatstuff_packages.vs_utilities import *

from vatstuff_packages.vs_gui.gui_functions import *

from vatstuff_packages.vs_verify.form_objects import make_AddressForm, CustomerForm, SupplierForm
from vatstuff_packages.vs_verify.common import check_customer, check_supplier


def new_organization_dialog(parent, org):
    "Called when the new_customer_button or new_supplier_button is pressed"
    new_organization_dialog = NewOrganizationDialog(parent, org)

    # If the cursor is busy, unbusy it - as
    # now waiting for a button press
    busy_cursor_off()

    new_organization_dialog.ShowModal()
    new_organization_dialog.Destroy()


class NewOrganizationDialog(vsDialog):
    "The dialog which creates a new customer or supplier"

    def __init__(self, parent, org):
        "Create the dialog, org is a string; either 'customer' or 'supplier'"
        if org == "customer":
            dialog_title = "New Customer"
        else:
            dialog_title = "New Supplier"
        vsDialog.__init__(self, parent, dialog_title)
        
        # Take a note of the variables
        self.org = org
        self.parent = parent
        self.organization = None

        main_sizer=wx.BoxSizer(wx.VERTICAL)

        if self.org == "customer":
            self.organization_form = CustomerForm()
            help1_text="Add new customer details:"
            vat_regnumber_text=wx.StaticText(self, -1, "The customers country code and VAT number")
            discount_text=wx.StaticText(self, -1, "Discount rate applied for this customer")
            org_mask = { 'reg_office'     : 1,
                         'delivery_point' : 1,
                         'invoice_point'  : 1,
                         'order_point'    : 0,
                         'payment_point'  : 1 }
            org_defaults = { 'reg_office'     : 0,
                             'delivery_point' : 1,
                             'invoice_point'  : 1,
                             'order_point'    : 0,
                             'payment_point'  : 1 }
        else:
            self.organization_form = SupplierForm()
            help1_text="Add new supplier details:"
            vat_regnumber_text=wx.StaticText(self, -1, "The suppliers country code and VAT number")
            discount_text=wx.StaticText(self, -1, "Discount rate this supplier gives to you")
            org_mask = { 'reg_office'     : 1,
                         'delivery_point' : 0,
                         'invoice_point'  : 0,
                         'order_point'    : 1,
                         'payment_point'  : 1 }
            org_defaults = { 'reg_office'     : 0,
                             'delivery_point' : 0,
                             'invoice_point'  : 0,
                             'order_point'    : 1,
                             'payment_point'  : 1 }
        
        self.top_text=wx.StaticText(self, -1, help1_text)
        main_sizer.Add(self.top_text, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # Add office panel
        self.office_panel = AddressPanel(self, name = self.organization_form.name,
                                               address_list = [],
                                               flag_mask = org_mask,
                                               flag_defaults = org_defaults,
                                               buttons = False)
        main_sizer.Add(self.office_panel, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # Create vat and discounts in a grid sizer
        a_sizer=wx.FlexGridSizer(rows=4, cols=2, hgap=20, vgap=10)

        # set ec_country flag field
        if self.org == "customer":
            self.ec_country_field = wx.CheckBox(self, -1, "The customer is within the EC")
        else:
            self.ec_country_field = wx.CheckBox(self, -1, "The supplier is within the EC")
        self.ec_country_field.SetValue(self.organization_form.ec_country)

        self.ec_country_field.SetToolTipString("Tick if the organisation is in the European Community")

        a_sizer.Add(self.ec_country_field, 0, wx.ALIGN_LEFT)
        a_sizer.Add(wx.StaticText(self, -1, ""), 0, wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        # set vat_registered flag field
        if self.org == "customer":
            self.vat_registered_field = wx.CheckBox(self, -1, "The customer is VAT Registered")
        else:
            self.vat_registered_field = wx.CheckBox(self, -1, "The supplier is VAT Registered")
        self.vat_registered_field.SetValue(self.organization_form.vat_registered)

        self.vat_registered_field.SetToolTipString("Tick if the organisation is VAT registered")

        a_sizer.Add(self.vat_registered_field, 0, wx.ALIGN_LEFT)
        a_sizer.Add(wx.StaticText(self, -1, ""), 0, wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)


        # Create vat number in a horizontal sizer
        b_sizer=wx.BoxSizer(wx.HORIZONTAL)

        # Set the initial values for VAT country code and registration number

        self.vat_country_code_field = wx.TextCtrl(self, -1, self.organization_form.vat_country_code, size=(40, vs_cfg["BUTTON_HEIGHT"]))
        self.vat_country_code_field.SetToolTipString("Set the companies VAT country code")
        self.vat_country_code_field.SetMaxLength(2)
        b_sizer.Add(self.vat_country_code_field, 0, wx.ALIGN_RIGHT)

        self.vat_regnumber_field = wx.TextCtrl(self, -1, self.organization_form.vat_regnumber, size=(150, vs_cfg["BUTTON_HEIGHT"]))
        self.vat_regnumber_field.SetToolTipString("Set the companies VAT number")
        self.vat_regnumber_field.SetMaxLength(20)
        b_sizer.Add(self.vat_regnumber_field, 0, wx.LEFT | wx.ALIGN_RIGHT, 10)

        a_sizer.Add(b_sizer, 0, wx.ALIGN_RIGHT)

        # set vat_regnumber_text, defined above, in the sizer
        a_sizer.Add(vat_regnumber_text, 0, wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        # Discount Field

        discount_sizer=wx.BoxSizer(wx.HORIZONTAL)
        self.discount_field = wx.TextCtrl(self, -1, self.organization_form.discount_string, style=wx.ALIGN_RIGHT, size=(60, vs_cfg["BUTTON_HEIGHT"]))
        self.discount_field.SetToolTipString("Percentage discount rate")
        self.discount_field.SetMaxLength(4)
        discount_sizer.Add(self.discount_field, 0, wx.ALIGN_RIGHT)
        discount_sizer.Add(wx.StaticText(self, -1, "%"), 0, wx.ALIGN_CENTER_VERTICAL)

        a_sizer.Add(discount_sizer, 0, wx.ALIGN_RIGHT)

        # set discount_text, defined above, in the sizer
        a_sizer.Add(discount_text, 0,  wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        main_sizer.Add(a_sizer, 0, wx.TOP | wx.ALIGN_CENTER, 10) 

        # ---------------------- Apply - Cancel -- buttons

        # Create a horizontal button_sizer to hold the buttons
        button_sizer=wx.BoxSizer(wx.HORIZONTAL)

        # Apply button
        self.apply_button = wx.Button(self, wx.ID_APPLY, size=(100, vs_cfg["BUTTON_HEIGHT"]))
        self.apply_button.SetToolTipString("Click to apply changes.")
        button_sizer.Add(self.apply_button, 0, wx.RIGHT | wx.ALIGN_CENTER, 40)

        # Cancel button
        self.cancel_button = wx.Button(self, wx.ID_CANCEL, size=(100, vs_cfg["BUTTON_HEIGHT"]))
        self.cancel_button.SetToolTipString("Click to cancel.")
        button_sizer.Add(self.cancel_button, 0, wx.LEFT | wx.ALIGN_CENTER, 40)

        # Add button_sizer to the main sizer
        main_sizer.Add(button_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)
        
        # Set the size to the panel
        self.SetSizer(main_sizer)
        main_sizer.SetSizeHints(self)

        # ---------- Bind buttons to handler functions

        self.Bind(wx.EVT_CHECKBOX, self.ec_country_checked, self.ec_country_field)
        self.Bind(wx.EVT_CHECKBOX, self.vat_registered_checked, self.vat_registered_field)
        self.Bind(wx.EVT_BUTTON, self.apply_button_pressed, self.apply_button)
        self.Bind(wx.EVT_BUTTON, self.cancel_button_pressed, self.cancel_button)

    # ----------------- Handler functions

    def ec_country_checked(self, event):
        "The EC country check box ticked or unticked"
        try:
            if self.ec_country_field.GetValue():
                # This is an organization within the eu
                self.organization_form.ec_country = True
                # Set the fields with the stored values
                self.vat_country_code_field.SetValue(self.organization_form.vat_country_code)
                self.vat_country_code_field.Enable(True)
                self.vat_registered_field.SetValue(self.organization_form.vat_registered)
                self.vat_registered_field.Enable(True)
                if self.organization_form.vat_registered:
                    self.vat_regnumber_field.SetValue(self.organization_form.vat_regnumber)
                    self.vat_regnumber_field.Enable(True)
            else:
                # Outside the EU - disable the country code, vat registered and vat reg number
                self.organization_form.ec_country = False
                # Store existing values
                self.organization_form.vat_country_code=self.vat_country_code_field.GetValue()
                self.organization_form.vat_registered=self.vat_registered_field.GetValue()
                if self.organization_form.vat_registered:
                    # Only store the number if self.organization_form.vat_registered is True
                    # otherwise you are only storing a blank field
                    self.organization_form.vat_regnumber=self.vat_regnumber_field.GetValue()
                # Set the fields to blank
                self.vat_country_code_field.SetValue("")
                self.vat_regnumber_field.SetValue("")
                self.vat_registered_field.SetValue(False)
                # Disable the fields
                self.vat_country_code_field.Enable(False)
                self.vat_regnumber_field.Enable(False)
                self.vat_registered_field.Enable(False)
        except Exception, e:
            error_message("Failure handling EC country check box", e)


    def vat_registered_checked(self, event):
        "The VAT registered box ticked or unticked"
        try:
            if self.vat_registered_field.GetValue():
                # This organization is VAT registered
                # Set the registration number field with the stored value and enable it
                self.vat_regnumber_field.SetValue(self.organization_form.vat_regnumber)
                self.vat_regnumber_field.Enable(True)
            else:
                # Not VAT registered - disable the vat reg number field
                # Store existing value
                self.organization_form.vat_regnumber=self.vat_regnumber_field.GetValue()
                self.vat_regnumber_field.SetValue("")
                self.vat_regnumber_field.Enable(False)
        except Exception, e:
            error_message("Failure handling VAT registered check box", e)


    @writecheck
    @button_wrap
    def apply_button_pressed(self, event):
        "Apply button - check and apply all fields"

        # Organization office address
        status, self.organization_form.name, address_list, deleted_addresses = self.office_panel.get_addresses()
        if not status:
            return

        # EC country
        self.organization_form.ec_country = self.ec_country_field.GetValue()

        if self.organization_form.ec_country:
            # VAT Country Code
            self.organization_form.vat_country_code = self.vat_country_code_field.GetValue().strip().upper()

            # VAT Registration number
            self.organization_form.vat_regnumber = self.vat_regnumber_field.GetValue().strip()

            # VAT Registered status
            self.organization_form.vat_registered = self.vat_registered_field.GetValue()
        else:
            self.organization_form.vat_country_code = ""
            self.organization_form.vat_regnumber = ""
            self.organization_form.vat_registered = False

        # Discount
        try:
            self.organization_form.discount = D(self.discount_field.GetValue().replace(vs_cfg["CP"], "."))
        except Exception:
            vs_message(self, "The discount value is incorrect\nNo items have been changed.", "ERROR")
            return

        # Save the organisation to the database
        if self.org == "customer":
            # Check form fields are ok in vatstuff_packages.vs_verify.common.check_customer
            status, message = check_customer(self.organization_form)
            if not status:
                vs_message(self, message, "ERROR")
                return
            # Now create the customer on the database
            self.organization = create_Customer(self.organization_form)
            if not self.organization:
                vs_message(self, "Failed to save customer to the database", "ERROR")
                return
        else:
            # Check form fields are ok in vatstuff_packages.vs_verify.common.check_supplier
            status, message = check_supplier(self.organization_form)
            if not status:
                vs_message(self, message, "ERROR")
                return
            # Now create the supplier on the database
            self.organization = create_Supplier(self.organization_form)
            if not self.organization:
                vs_message(self, "Failed to save supplier to the database", "ERROR")
                return

        # now add the company address

        # refresh everything
        big_refresh()
        # and this sets organization choice
        self.parent.refresh_list(self.organization)

        # Create an address
        address = address_list[0]
        if self.org == "customer":
            database_address = create_Address(customer=self.organization)
        else:
            database_address = create_Address(supplier=self.organization)

        database_address.lines = address.lines
        database_address.flags = address.flags

        if self.org == "customer":
            vs_message(self, """\
A new customer has been created, and a new account:
%s
which is used to record sales to this customer""" % self.organization.account.fullname, "INFORMATION")
        else:
            vs_message(self, """\
A new supplier has been created, and a new account:
%s
which is used to record purchases from this supplier""" % self.organization.account.fullname, "INFORMATION")

        self.EndModal(wx.ID_OK)


    def cancel_button_pressed(self, event):
        "Close window after an information message"
        vs_message(self, "No changes have been made.", "INFORMATION")
        self.EndModal(wx.ID_CANCEL)


# ----------------------EditOrganizationDialog

def edit_organization_dialog(parent, organization):
    "Called when the Edit Customer or Edit Supplier button is pressed"
    edit_organization_dialog = EditOrganizationDialog(parent, organization)

    # If the cursor is busy, unbusy it - as
    # now waiting for a button press
    busy_cursor_off()

    edit_organization_dialog.ShowModal()
    edit_organization_dialog.Destroy()


class EditOrganizationDialog(vsDialog):
    "The dialog which edits a customer or supplier"

    def __init__(self, parent, organization):
        "Create the dialog, organization is either a Customer or Supplier object"
        if isinstance(organization, Customer):
            self.dialog_title = "Edit Customer"
        else:
            self.dialog_title = "Edit Supplier"
        vsDialog.__init__(self, parent, self.dialog_title)

        # Take a note of the variables
        self.parent = parent
        self.organization = organization

        # flag mask dictates which of the address flag fields are enabled or disabled
        # flag defaults set which are ticked or unticked for a new address as default

        if isinstance(self.organization, Customer):
            self.org = "customer"
            self.organization_form = CustomerForm()
            vat_regnumber_text=wx.StaticText(self, -1, "Customer country code and VAT number")
            discount_text=wx.StaticText(self, -1, "Discount rate applied for this customer")
            org_mask = { 'reg_office'     : 1,
                         'delivery_point' : 1,
                         'invoice_point'  : 1,
                         'order_point'    : 0,
                         'payment_point'  : 1 }
            org_defaults = { 'reg_office'     : 0,
                             'delivery_point' : 1,
                             'invoice_point'  : 1,
                             'order_point'    : 0,
                             'payment_point'  : 1 }
        else:
            self.org = "supplier"
            self.organization_form = SupplierForm()
            vat_regnumber_text=wx.StaticText(self, -1, "Supplier country code and VAT number")
            discount_text=wx.StaticText(self, -1, "Discount rate this supplier gives to you")
            org_mask = { 'reg_office'     : 1,
                         'delivery_point' : 0,
                         'invoice_point'  : 0,
                         'order_point'    : 1,
                         'payment_point'  : 1 }
            org_defaults = { 'reg_office'     : 0,
                             'delivery_point' : 0,
                             'invoice_point'  : 0,
                             'order_point'    : 1,
                             'payment_point'  : 1 }

        # copy self.organization attributes into the self.organization_form object
        self.organization_form.copy_from(self.organization)

        main_sizer=wx.BoxSizer(wx.VERTICAL)

        help1_text="""If this company has multiple addresses, you can add or edit them here. Use the Next and
Previous buttons to move between addresses, and for each address set its function."""
        
        main_sizer.Add(wx.StaticText(self, -1, help1_text), 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # self.address_list is the list of temporary addresses to be edited,
        self.address_list = [ make_AddressForm(address) for address in self.organization.get_addresses() ]

        # self.address_list is now a list of AddressForm objects (object defined in vatstuff_packages.vs_verify.form_objects)
        # These can be edited directly without affecting the database, but then changes must be
        # sent to the database when Apply is pressed

        # Add address panel
        self.office_panel = AddressPanel(self,
                                         self.organization_form.name,
                                         self.address_list,
                                         flag_mask = org_mask,
                                         flag_defaults = org_defaults,
                                         buttons = True)

        main_sizer.Add(self.office_panel, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # Create vat and discount in a grid sizer
        a_sizer=wx.FlexGridSizer(rows=4, cols=2, hgap=20, vgap=10)

        # set ec_country flag field
        if self.org == "customer":
            self.ec_country_field = wx.CheckBox(self, -1, "The customer is within the EC")
        else:
            self.ec_country_field = wx.CheckBox(self, -1, "The supplier is within the EC")
        self.ec_country_field.SetValue(self.organization_form.ec_country)

        self.ec_country_field.SetToolTipString("Tick if the organisation is in the European Community")

        a_sizer.Add(self.ec_country_field, 0, wx.ALIGN_LEFT)
        a_sizer.Add(wx.StaticText(self, -1, ""), 0, wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        # set vat_registered flag field
        if self.org == "customer":
            self.vat_registered_field = wx.CheckBox(self, -1, "The customer is VAT Registered")
        else:
            self.vat_registered_field = wx.CheckBox(self, -1, "The supplier is VAT Registered")
        self.vat_registered_field.SetValue(self.organization_form.vat_registered)

        self.vat_registered_field.SetToolTipString("Tick if the organisation is VAT registered")

        a_sizer.Add(self.vat_registered_field, 0, wx.ALIGN_LEFT)
        a_sizer.Add(wx.StaticText(self, -1, ""), 0, wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        # Create vat number in a horizontal sizer
        b_sizer=wx.BoxSizer(wx.HORIZONTAL)

        # Set the initial values for VAT country code and registration number
        if not self.organization_form.ec_country:
            self.organization_form.vat_country_code = ""
            self.organization_form.vat_regnumber = ""

        self.vat_country_code_field = wx.TextCtrl(self, -1, self.organization_form.vat_country_code, size=(40, vs_cfg["BUTTON_HEIGHT"]))
        self.vat_country_code_field.SetToolTipString("Company VAT country code")
        self.vat_country_code_field.SetMaxLength(2)
        if not self.organization_form.ec_country:
            self.vat_country_code_field.Enable(False)

        b_sizer.Add(self.vat_country_code_field, 0, wx.ALIGN_RIGHT)

        self.vat_regnumber_field = wx.TextCtrl(self, -1, self.organization_form.vat_regnumber, size=(150, vs_cfg["BUTTON_HEIGHT"]))
        self.vat_regnumber_field.SetToolTipString("Company VAT number")
        self.vat_regnumber_field.SetMaxLength(20)
        if not self.organization_form.ec_country or not self.organization_form.vat_registered:
            self.vat_regnumber_field.Enable(False)
        b_sizer.Add(self.vat_regnumber_field, 0, wx.LEFT | wx.ALIGN_RIGHT, 10)

        a_sizer.Add(b_sizer, 0, wx.ALIGN_RIGHT)

        # set vat_regnumber_text, defined earlier, in the sizer
        a_sizer.Add(vat_regnumber_text, 0, wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        # Discount Field

        discount_sizer=wx.BoxSizer(wx.HORIZONTAL)
        self.discount_field = wx.TextCtrl(self, -1, self.organization_form.discount_string, style=wx.ALIGN_RIGHT, size=(60, vs_cfg["BUTTON_HEIGHT"]))
        self.discount_field.SetToolTipString("Percentage discount rate")
        self.discount_field.SetMaxLength(4)
        discount_sizer.Add(self.discount_field, 0, wx.ALIGN_RIGHT)
        discount_sizer.Add(wx.StaticText(self, -1, "%"), 0, wx.ALIGN_CENTER_VERTICAL)

        a_sizer.Add(discount_sizer, 0, wx.ALIGN_RIGHT)

        # set discount_text, defined earlier, in the sizer
        a_sizer.Add(discount_text, 0, wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        main_sizer.Add(a_sizer, 0, wx.TOP | wx.ALIGN_CENTER, 10)

        # ---------------------- Buttons

        # Create buttons in a horizontal sizer
        button_sizer = wx.BoxSizer(wx.HORIZONTAL)

        # Apply button
        self.apply_button = wx.Button(self, wx.ID_APPLY, size=(100, vs_cfg["BUTTON_HEIGHT"]))
        self.apply_button.SetToolTipString("Click to apply changes.")
        button_sizer.Add(self.apply_button, 0, wx.RIGHT | wx.ALIGN_CENTER, 40)

        # Cancel button
        self.cancel_button = wx.Button(self, wx.ID_CANCEL, size=(100, vs_cfg["BUTTON_HEIGHT"]))
        self.cancel_button.SetToolTipString("Click to cancel.")
        button_sizer.Add(self.cancel_button, 0, wx.LEFT | wx.ALIGN_CENTER, 40)

        # Add button_sizer to the main sizer
        main_sizer.Add(button_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)
        
        # Set the size to the panel
        self.SetSizer(main_sizer)
        main_sizer.SetSizeHints(self)

        # ---------- Bind buttons to handler functions

        self.Bind(wx.EVT_CHECKBOX, self.ec_country_checked, self.ec_country_field)
        self.Bind(wx.EVT_CHECKBOX, self.vat_registered_checked, self.vat_registered_field)
        self.Bind(wx.EVT_BUTTON, self.apply_button_pressed, self.apply_button)
        self.Bind(wx.EVT_BUTTON, self.cancel_button_pressed, self.cancel_button)


    def ec_country_checked(self, event):
        try:
            self.organization_form.ec_country = self.ec_country_field.GetValue()
            if self.organization_form.ec_country:
                # This is an organization within the eu
                # Set the fields with the stored values
                self.vat_country_code_field.SetValue(self.organization_form.vat_country_code)
                self.vat_registered_field.SetValue(self.organization_form.vat_registered)
                if self.organization_form.vat_registered:
                    self.vat_regnumber_field.SetValue(self.organization_form.vat_regnumber)
                # Enable the fields
                self.vat_country_code_field.Enable(True)
                self.vat_registered_field.Enable(True)
                if self.organization_form.vat_registered:
                    self.vat_regnumber_field.Enable(True)
            else:
                # Outside the EU - disable the country code, vat registered and vat reg number
                # Store existing values
                self.organization_form.vat_country_code=self.vat_country_code_field.GetValue()
                self.organization_form.vat_registered=self.vat_registered_field.GetValue()
                if self.organization_form.vat_registered:
                    # Only store the number if self.vat_registered is True
                    # otherwise you are only storing a blank field
                    self.organization_form.vat_regnumber=self.vat_regnumber_field.GetValue()
                # Set the fields to blank
                self.vat_country_code_field.SetValue("")
                self.vat_regnumber_field.SetValue("")
                self.vat_registered_field.SetValue(False)
                # Disable the fields
                self.vat_country_code_field.Enable(False)
                self.vat_regnumber_field.Enable(False)
                self.vat_registered_field.Enable(False)
        except Exception, e:
            error_message("Failure handling EC country check box", e)


    def vat_registered_checked(self, event):
        try:
            if self.vat_registered_field.GetValue():
                # This organization is VAT registered
                # Set the registration number field with the stored value and enable it
                self.vat_regnumber_field.SetValue(self.organization_form.vat_regnumber)
                self.vat_regnumber_field.Enable(True)
            else:
                # Not VAT registered - disable the vat reg number field
                # Store existing value
                self.organization_form.vat_regnumber=self.vat_regnumber_field.GetValue()
                self.vat_regnumber_field.SetValue("")
                self.vat_regnumber_field.Enable(False)
        except Exception, e:
            error_message("Failure handling VAT registered check box", e)


    @writecheck
    @button_wrap
    def apply_button_pressed(self, event):
        "Apply button - check and apply all fields"
        # Organization office address
        status, self.organization_form.name, self.address_list, deleted_addresses = self.office_panel.get_addresses()
        if not status:
            return

        # EC country
        self.organization_form.ec_country = self.ec_country_field.GetValue()

        if self.organization_form.ec_country:
            # VAT Country Code
            self.organization_form.vat_country_code = self.vat_country_code_field.GetValue().strip().upper()

            # VAT Registration number
            self.organization_form.vat_regnumber = self.vat_regnumber_field.GetValue().strip()

            # VAT Registered status
            self.organization_form.vat_registered = self.vat_registered_field.GetValue()
        else:
            self.organization_form.vat_country_code = ""
            self.organization_form.vat_regnumber = ""
            self.organization_form.vat_registered = False

        # Discount
        try:
            self.organization_form.discount = D(self.discount_field.GetValue().replace(vs_cfg["CP"], "."))
        except Exception:
            vs_message(self, "The discount value is incorrect\nNo items have been changed.", "ERROR")
            return

        # Check the organization_form
        if self.org == "customer":
            # Check form fields are ok in vatstuff_packages.vs_verify.common.check_customer
            status, message = check_customer(self.organization_form, self.organization)
            if not status:
                vs_message(self, message, "ERROR")
                return
        else:
            # Check form fields are ok in vatstuff_packages.vs_verify.common.check_supplier
            status, message = check_supplier(self.organization_form, self.organization)
            if not status:
                vs_message(self, message, "ERROR")
                return

        # save ec_country, vat registration number, vat country code, vat_registered status
        # and discount to the database
        self.organization.ec_country = self.organization_form.ec_country
        self.organization.vat_regnumber = self.organization_form.vat_regnumber
        self.organization.vat_country_code = self.organization_form.vat_country_code
        self.organization.discount = self.organization_form.discount
        self.organization.vat_registered = self.organization_form.vat_registered

        # save all addresses in self.address_list to the database
        for address in self.address_list:
            if address.id < 0 :
                # This is a new address which has to be created
                if self.org == "customer":
                    database_address = create_Address(customer=self.organization)
                else:
                    database_address = create_Address(supplier=self.organization)
            else:
                database_address = Address(address.id)

            database_address.lines = address.lines
            database_address.flags = address.flags

        # Delete the addresses in the deleted_addresses list
        if deleted_addresses:
            for address in deleted_addresses:
                database_address = Address(address.id)
                database_address.delete()

        # Set organization with new company name
        if self.organization.name != self.organization_form.name:
            self.organization.name = self.organization_form.name
            # Set account name with this new name, but check for uniqueness
            organization_folder = self.organization.account.folder
            # Check for name uniqueness within the folder
            accounts_in_folder = organization_folder.get_accounts()
            account_name = self.organization_form.name
            name_unique = False
            index = 1
            while not name_unique:
                for acc in accounts_in_folder:
                    name_unique = True
                    if account_name == acc.name:
                        name_unique = False
                        index += 1
                        account_name = "%s_%i" % (self.organization_form.name, index)
                        break
            self.organization.account.name = account_name
            if self.org == "customer":
                self.organization.account.description = "Customer account for %s" % self.organization_form.name
            else:
                self.organization.account.description = "Supplier account for %s" % self.organization_form.name
            # refresh account lists
            top_refresh_accounts()
            self.parent.refresh_list(self.organization)
            # if a search contacts frame is open, refresh it
            if vs_cfg["SEARCH_CONTACTS_FRAME"]:
                vs_cfg["SEARCH_CONTACTS_FRAME"].refresh(use_current=True)
        vs_message(self, "Changes have been saved.", "INFORMATION")
        self.EndModal(wx.ID_OK)


    def cancel_button_pressed(self, event):
        "Close window after an information message"
        if vs_yesno(self, """\
Closing the edit dialog: are you sure?
Any changes will be lost unless you press Apply""", "Closing - %s" % (self.dialog_title,)):
            self.EndModal(wx.ID_CANCEL)





