####### VATStuff #######
#
# make_order.py  - creates a purchase order
#
# Version : 0.7.0
# Date : 20100913
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#

"""Creates a purchase order

It is run as part of the vatstuff gui interface.
"""


import os
import decimal
import datetime

import wx
import wx.grid

from vatstuff_packages.vs_database import save_purchaseorder

from vatstuff_packages.vs_gui.address_panel import *
from vatstuff_packages.vs_gui.gui_functions import *

from vatstuff_packages.vs_gui.items_table import ItemsTable

from vatstuff_packages.vs_verify import form_objects
from vatstuff_packages.vs_verify.purchases import new_purchaseorder_check

from vatstuff_packages.vs_utilities import *

if vs_cfg["REPORTLABS_AVAILABLE"]:
    import vatstuff_packages.vs_pdf.page_setup
    import vatstuff_packages.vs_pdf.order_form
    import vatstuff_packages.vs_pdf.pdf_functions

D = decimal.Decimal   # D used to avoid typing


def order_dialog(parent, supplier):
    """Creates an OrderDialog"""
    order_dialog = OrderDialog(parent, supplier)

    # If the cursor is busy, unbusy it - as
    # now waiting for a button press
    busy_cursor_off()

    order_dialog.ShowModal()
    order_dialog.Destroy()


class OrderDialog(vsDialog):
    "The order dialog which creates a new purchase order"

    def __init__(self, parent, supplier):
        """Create the Dialog"""
        vsDialog.__init__(self, parent, "Purchase Order")
        
        # Take a note of the variables
        self.supplier = supplier

        # Can this supplier charge you VAT?
        self.show_vat = self.supplier.pay_vat()

        # Store all form contents in an OrderForm object
        self.order = form_objects.OrderForm()
        self.order.show_vat = self.show_vat
        self.order.claim_vat = True

        button_size = (100, vs_cfg["BUTTON_HEIGHT"])

        # Create a vertical sizer to hold everything
        self.sizer=wx.BoxSizer(wx.VERTICAL)

        # Create company and addresses in a grid sizer
        add_sizer=wx.FlexGridSizer(rows=2, cols=3, hgap=20, vgap=10)

        # Create a horizontal supplier_sizer
        supplier_sizer=wx.BoxSizer(wx.HORIZONTAL)

        self.all_suppliers = Supplier.get_all()
        supplier_list = [s.name for s in self.all_suppliers]

        supplier_sizer.Add(wx.StaticText(self, -1, "To:"), 0, wx.RIGHT | wx.ALIGN_RIGHT, 10)

        self.supplier_field = wx.Choice(self, -1, size=(250, vs_cfg["BUTTON_HEIGHT"]), choices=supplier_list)
        self.supplier_field.SetToolTipString("Set Supplier")
        supplier_sizer.Add(self.supplier_field, 0, wx.ALIGN_LEFT)

        key = self.all_suppliers.index(supplier)
        self.supplier_field.SetSelection(key)

        add_sizer.Add(supplier_sizer, 0, wx.ALIGN_CENTER)

        invoice_text=wx.StaticText(self, -1, "Invoice address")
        add_sizer.Add(invoice_text, 0, wx.ALIGN_CENTER)

        delivery_text=wx.StaticText(self, -1, "Delivery address")
        add_sizer.Add(delivery_text, 0, wx.ALIGN_CENTER)

        self.order_address = AddressSpin(self, self.supplier, order_point=1)
        add_sizer.Add(self.order_address, 0, wx.ALIGN_CENTER)

        self.invoice_address = AddressSpin(self, Company(), invoice_point=1)
        add_sizer.Add(self.invoice_address, 0, wx.ALIGN_CENTER)

        self.delivery_address = AddressSpin(self, Company(), delivery_point=1)
        add_sizer.Add(self.delivery_address, wx.ALIGN_CENTER)

        # add the add_sizer to the main vertical sizer
        self.sizer.Add(add_sizer, 0, wx.LEFT | wx.TOP | wx.RIGHT | wx.ALIGN_CENTER, 10)

        # Create references in a grid sizer
        ref_sizer=wx.FlexGridSizer(rows=3, cols=3, hgap=20, vgap=10)

        # First row

        sup_ref_text=wx.StaticText(self, -1, "Suppliers reference (if any):")
        ref_sizer.Add(sup_ref_text, 0, wx.ALIGN_LEFT,)

        order_date_text=wx.StaticText(self, -1, "The date:")
        ref_sizer.Add(order_date_text, 0, wx.ALIGN_LEFT)

        order_number_text=wx.StaticText(self, -1, "Order number (automatically increments):")
        ref_sizer.Add(order_number_text, 0, wx.ALIGN_LEFT)

        # second row

        self.sup_ref_field = wx.TextCtrl(self, -1, "", size=(300, vs_cfg["BUTTON_HEIGHT"]))
        self.sup_ref_field.SetToolTipString("Suppliers quote number or contact name")
        self.sup_ref_field.SetMaxLength(40)
        ref_sizer.Add(self.sup_ref_field, 0, wx.ALIGN_LEFT)

        self.datestamp = wx.DateTime.Now()
        self.order_date_field = wx.DatePickerCtrl(self, -1,
                                                  self.datestamp, 
                                                  size=(150, vs_cfg["BUTTON_HEIGHT"]),
                                                  style=wx.DP_DROPDOWN | wx.DP_SHOWCENTURY)
        self.order_date_field.SetToolTipString("The date of the order")
        ref_sizer.Add(self.order_date_field, 0, wx.ALIGN_LEFT)

        # get_config_item function is in vatstuff_packages.vs_database.odd_functions
        self.last_order_number = get_config_item("LAST_ORDER_NUMBER")

        # Get the width and height of the last order
        self.order.pdf_width, self.order.pdf_height = vs_cfg["LAST_ORDER_PDF_SIZE"]

        if self.last_order_number:
            self.last_order_number = int(self.last_order_number)
            order_number =  self.last_order_number + 1
        else:
            self.last_order_number = 0
            order_number = 1
        
        self.order_number_field = wx.TextCtrl(self, -1, str(order_number), size=(250, vs_cfg["BUTTON_HEIGHT"]))
        self.order_number_field.SetToolTipString("Set your order number")
        self.order_number_field.SetMaxLength(30)
        ref_sizer.Add(self.order_number_field, 0, wx.ALIGN_LEFT)

        # third row

        discount_sizer=wx.BoxSizer(wx.HORIZONTAL)
        discount_sizer.Add(wx.StaticText(self, -1, """Discount this supplier gives to you:
(tick item discount column to apply)"""), 0, wx.RIGHT | wx.ALIGN_RIGHT, 10)
        discount_rate_text = str(self.supplier.discount).replace(u".", vs_cfg["CP"])
        self.discount_rate_field = wx.TextCtrl(self, -1, discount_rate_text, style=wx.ALIGN_RIGHT, size=(60, vs_cfg["BUTTON_HEIGHT"]))
        self.discount_rate_field.SetToolTipString("Set the percentage discount rate")
        self.discount_rate_field.SetMaxLength(4)
        discount_sizer.Add(self.discount_rate_field, 0, wx.ALIGN_CENTER_VERTICAL)
        discount_sizer.Add(wx.StaticText(self, -1, "%"), 0, wx.ALIGN_CENTER_VERTICAL)
        ref_sizer.Add(discount_sizer,0, wx.ALIGN_RIGHT)

        enq_text=wx.StaticText(self, -1, "In case of enquiry please contact:\n(names set under the Company tab)")
        ref_sizer.Add(enq_text, 0, wx.ALIGN_RIGHT,)

        self.form_contacts = Contact.company_list()
        contacts_list = [""]                          # First choice is blank
        for contact in self.form_contacts:
            contacts_list.append(contact.name)
        self.form_contacts.insert(0, None)            # insert None as first item in this list also

        self.enq_field = wx.Choice(self, -1, size=(250, vs_cfg["BUTTON_HEIGHT"]), choices=contacts_list)
        self.enq_field.SetToolTipString("Set contact name")
        ref_sizer.Add(self.enq_field, 0, wx.ALIGN_LEFT)

        # add the ref_sizer to the main vertical sizer
        self.sizer.Add(ref_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # ---------------------- Create the purchaseitems table

        self.purchaseitems_table = ItemsTable(self, "purchaseorder")
        self.sizer.Add(self.purchaseitems_table, 0, wx.LEFT | wx.RIGHT | wx.ALIGN_CENTER, 10)

        if not self.show_vat:
            # Hide the vat column 
            self.purchaseitems_table.hide_vat_column()

        # ---------------------- Make country check, and total cost

        # Create a horizontal results_sizer
        results_sizer=wx.BoxSizer(wx.HORIZONTAL)

        # Put country check in a Vertical sizer
        country_sizer=wx.BoxSizer(wx.VERTICAL)

        print_text=wx.StaticText(self, -1, """Check for the country to be
shown on printed addresses""")
        country_sizer.Add(print_text, 0, wx.ALIGN_CENTER)

        self.print_check = wx.CheckBox(self, -1, "")
        if self.supplier.vat_country_code == vs_cfg["VAT_COUNTRY_CODE"]:
            self.print_check.SetValue(False)
        else:
            self.print_check.SetValue(True)
        country_sizer.Add(self.print_check, 0, wx.TOP | wx.ALIGN_CENTER, 10)

        results_sizer.Add(country_sizer, 0, wx.LEFT | wx.RIGHT | wx.ALIGN_CENTER, 40)

        # page setup button
        self.page_button = wx.Button(self, label="Page setup", size=button_size)
        self.page_button.SetToolTipString("Click to set up the pdf page size and orientation")
        results_sizer.Add(self.page_button, 0, wx.RIGHT | wx.ALIGN_CENTER, 40)

        # pdf button
        self.pdf_button = wx.Button(self, label="Create pdf", size=button_size)
        self.pdf_button.SetToolTipString("Click to create a pdf file of the order form.")
        results_sizer.Add(self.pdf_button, 0, wx.RIGHT | wx.ALIGN_CENTER, 40)

        # Put costs in a grid sizer
        cost_sizer=wx.FlexGridSizer(rows=3, cols=3, hgap=20, vgap=10)

        zero = vs_cfg["SYMBOL_ZERO_STRING"]

        # VAT
        VAT_text=wx.StaticText(self, -1, "VAT")
        cost_sizer.Add(VAT_text, 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.VAT_value_field = wx.TextCtrl(self, -1, zero, style=wx.ALIGN_RIGHT, size=(150, vs_cfg["BUTTON_HEIGHT"]))
        self.VAT_value_field.SetToolTipString("The VAT amount")
        self.VAT_value_field.SetMaxLength(18)
        # If vat not shown disable the VAT field
        if not self.show_vat:
            self.VAT_value_field.Enable(False)
        cost_sizer.Add(self.VAT_value_field, 0, wx.ALIGN_RIGHT)

        self.claim_vat_field = wx.CheckBox(self, -1, "Claim VAT")
        self.claim_vat_field.SetValue(True)
        self.claim_vat_field.SetToolTipString("""\
Untick if VAT cannot be claimed back,
for purchases such as entertainment.""")
        if not self.show_vat:
            self.claim_vat_field.Hide()
        self.vat_scheme_method = VATScheme().method
        if self.vat_scheme_method != "STANDARD" and self.vat_scheme_method != "CASH":
            self.claim_vat_field.Hide()

        cost_sizer.Add(self.claim_vat_field, 0, wx.ALIGN_CENTER_VERTICAL)

        # cost
        cost_text=wx.StaticText(self, -1, "Cost")
        cost_sizer.Add(cost_text, 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.cost_value_field = wx.TextCtrl(self, -1, zero, style=wx.ALIGN_RIGHT, size=(150, vs_cfg["BUTTON_HEIGHT"]))
        self.cost_value_field.SetToolTipString("Cost, exclusive of VAT")
        self.cost_value_field.SetMaxLength(18)
        cost_sizer.Add(self.cost_value_field, 0, wx.ALIGN_RIGHT)

        cost_sizer.Add((20,40), 0)

        # Total
        total_text=wx.StaticText(self, -1, "Total")
        cost_sizer.Add(total_text, 0, wx.ALIGN_RIGHT)
        self.total_value=wx.StaticText(self, -1, zero)
        cost_sizer.Add(self.total_value, 0, wx.ALIGN_RIGHT)

        cost_sizer.Add((20,40), 0)

        results_sizer.Add(cost_sizer, 0, wx.LEFT | wx.ALIGN_RIGHT, 40)


        # Add results_sizer to the main sizer
        self.sizer.Add(results_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # ---------------------- Apply - Cancel -- buttons

        # Create a horizontal button_sizer to hold the buttons
        button_sizer=wx.BoxSizer(wx.HORIZONTAL)

        button_text=wx.StaticText(self, -1, """\
Creating a PDF file does not implement or store the
transaction in the database, press Apply to do that""")
        button_sizer.Add(button_text, 0, wx.RIGHT | wx.ALIGN_CENTER, 40)

        # Apply button
        self.apply_button = wx.Button(self, wx.ID_APPLY, size=button_size)
        self.apply_button.SetToolTipString("Click to apply changes.")
        button_sizer.Add(self.apply_button, 0, wx.RIGHT | wx.ALIGN_CENTER, 40)

        # Cancel button
        self.cancel_button = wx.Button(self, wx.ID_CANCEL, size=button_size)
        self.cancel_button.SetToolTipString("Click to cancel.")
        button_sizer.Add(self.cancel_button, 0, wx.LEFT | wx.ALIGN_CENTER, 40)

        # Add button_sizer to the main sizer
        self.sizer.Add(button_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # Set the size to the panel
        self.SetSizer(self.sizer)
        self.sizer.SetSizeHints(self)

        # ---------- Bind buttons to handler functions

        self.Bind(wx.EVT_TEXT, self.discount_change, self.discount_rate_field)
        self.Bind(wx.EVT_CHOICE, self.supplier_pressed, self.supplier_field)
        self.Bind(wx.EVT_TEXT, self.value_change, self.cost_value_field)
        self.Bind(wx.EVT_TEXT, self.value_change, self.VAT_value_field)
        self.Bind(wx.EVT_BUTTON, self.page_button_pressed, self.page_button)
        self.Bind(wx.EVT_BUTTON, self.pdf_button_pressed, self.pdf_button)        
        self.Bind(wx.EVT_BUTTON, self.apply_button_pressed, self.apply_button)
        self.Bind(wx.EVT_BUTTON, self.cancel_button_pressed, self.cancel_button)
        self.Bind(wx.EVT_CHECKBOX, self.claim_vat_toggle, self.claim_vat_field)


    # ----------------- Handler functions

    def discount_change(self, event):
        "The discount field has changed, update the totals"
        try:
            self.purchaseitems_table.calculate_totals()
        except Exception, e:
            error_message("Failure in calculating discount change", e)


    def value_change(self, event):
        "If the cost or VAT fields change, change the Totals sum"
        try:
            vat = self.VAT_value_field.GetValue().strip()
            cost = self.cost_value_field.GetValue().strip()

            status, vat_value = string_to_decimal(vat)
            if status == False:
                vat_value = D("0.00")
            status, cost_value = string_to_decimal(cost)
            if status == False:
                cost_value = D("0.00")

            total = vat_value + cost_value

            self.total_value.SetLabel(to_currency_string(total))
            self.sizer.Layout()
        except Exception, e:
            error_message("Failure in calculating value change", e)


    def claim_vat_toggle(self, event):
        "The claim VAT check box has been toggled"
        try:
            if not event.Checked():
                vs_message(self, """\
Normally VAT paid on purchases is reclaimed,
however in some cases, such as Entertainment
expenses, it should not be claimed.
Unticking this box ensures the VAT on the
associated supplier invoices is not claimed
when the VAT Return is calculated.""","Untick Claim VAT")
        except Exception, e:
            error_message("Failure in toggling Claim VAT checkbox", e)


    def get_discount(self):
        """Gets discount, returning status, discount, used by self.purchaseitems_table
           when calculating totals"""
        discount_string = self.discount_rate_field.GetValue().strip()
        try:
            discount_value = D(discount_string.replace(vs_cfg["CP"], "."))
        except Exception:
            return False, D("0.0")
        if discount_value < D(0) or discount_value > D("99.9"):
            return False,  D("0.0")
        return True, discount_value


    def supplier_pressed(self, event):
        "Supplier choice pressed - change supplier address"
        try:
            # Get supplier currently shown
            supplier_selected_list_item = self.supplier_field.GetCurrentSelection()
            # Check same supplier has'nt been chosen again
            new_supplier = self.all_suppliers[supplier_selected_list_item]
            if new_supplier == self.supplier:
                return
            self.supplier = new_supplier
            # Pass this to the address field
            self.order_address.set_org(org=self.supplier, order_point=1)
            # And the supplier discount_rate_field
            self.discount_rate_field.SetValue(str(self.supplier.discount).replace(".", vs_cfg["CP"]))
            # Can this supplier charge you VAT?
            self.show_vat = self.supplier.pay_vat()
            self.order.show_vat = self.show_vat
            # Whenever a supplier changes, set the self.claim_vat_field to the default of True
            self.claim_vat_field.SetValue(True)
            if self.show_vat:
                self.VAT_value_field.Enable(True)
                self.purchaseitems_table.show_vat_column()
                if self.vat_scheme_method == "STANDARD" or self.vat_scheme_method == "CASH":
                    self.claim_vat_field.Show()
                else:
                    self.claim_vat_field.Hide()
            else:
                self.VAT_value_field.Enable(False)
                self.purchaseitems_table.hide_vat_column()
                self.claim_vat_field.Hide()
            if self.supplier.vat_country_code == vs_cfg["VAT_COUNTRY_CODE"]:
                self.print_check.SetValue(False)
            else:
                self.print_check.SetValue(True)
            self.purchaseitems_table.calculate_totals()
        except Exception, e:
            error_message("Failure in updating supplier", e)


    @button_wrap
    def page_button_pressed(self, event):
        "page button pressed, setup the pdf page size and orientation"

        if not vs_cfg["REPORTLABS_AVAILABLE"]:
            # no pdf capability
            vs_message(self, "Function unavailable - the reportlab module is not installed",
                             "ERROR")
            return

        page_setup_dialog = vatstuff_packages.vs_pdf.page_setup.PageSetupDialog(self, self.order.pdf_width, self.order.pdf_height)

        if page_setup_dialog.ShowModal() == wx.ID_OK:
            self.order.pdf_width = page_setup_dialog.width
            self.order.pdf_height = page_setup_dialog.height           
        page_setup_dialog.Destroy()



    @button_wrap
    def pdf_button_pressed(self, event):
        "pdf button - check all fields, if ok, create a pdf form"

        if not vs_cfg["REPORTLABS_AVAILABLE"]:
            # no pdf capability
            vs_message(self, "Function unavailable - the reportlab module is not installed",
                             "ERROR")
            return

        status, message = self.check_fields()
        if not status:
            vs_message(self, message, "ERROR")
            return

        filename= "order_%s.pdf" % self.order.number

        # Create the save file dialog.
        status, filepath = save_file_as(self, 'pdf', vs_cfg["ORDERS_PDF_FOLDER"], filename)

        if status:
            busy_cursor_on()
            wx.Yield()
            if vatstuff_packages.vs_pdf.order_form.create_pdf(filepath, self.order):
                busy_cursor_off()
                # And show the file in the default viewer
                if not vatstuff_packages.vs_pdf.pdf_functions.show_pdf_file(filepath):
                    vs_message(self, "pdf file created", "Done")
            else:
                vs_message(self, "Unable to create the pdf file", "ERROR")


    @writecheck
    @button_wrap
    def apply_button_pressed(self, event):
        "Apply button - check and apply all fields"

        status, message = self.check_fields()
        if not status:
            vs_message(self, message, "ERROR")
            return

        # Store the transaction in a TransactionForm object
        transaction = form_objects.TransactionForm()
        # This is a purchase order transaction(system type 2)
        transaction.system = 2

        transaction.description = "Order %s : %s" % (self.order.number, self.order.supplier.name)
        transaction.datestamp = self.order.datestamp

        # Debit the purchase account by the cost plus vat of the order
        transaction.add(get_account_by_system(7), debit= self.order.cost + self.order.vat)

        # Credit the suppliers account by the cost plus vat of the order
        transaction.add(self.order.supplier.account, credit = self.order.cost + self.order.vat)

        status, message = transaction.check()
        if not status:
            vs_message(self, message, "ERROR")
            return

        wx.Yield()
        # Confirm the transaction (function and dialog in gui_functions)
        status, new_description = confirm_transaction(self, transaction)

        if not status:
            vs_message(self, "The Order has not been saved", "CANCELLED")
            return

        transaction.description = new_description

        # Must now implement the transaction and save the purchase order
        busy_cursor_on()
        wx.Yield()
        save_purchaseorder.save(self.order,transaction)

        big_refresh()
        vs_message(self, "Purchase order %s saved" % self.order.number, "DONE")
        self.EndModal(wx.ID_OK)


    def cancel_button_pressed(self, event):
        "Cancel button pressed"
        vs_message(self, "The order has not been saved to the database", "Warning")
        self.EndModal(wx.ID_CANCEL)


    #----------- Check and store all fields button

    def check_fields(self):
        """Checks the fields, and fills in the self.order object
           returns (True, None) if ok, or (False, "warning message") if not"""
        # Check supplier
        supplier_selected_list_item = self.supplier_field.GetCurrentSelection()

        # Set self.order attributes
        self.order.supplier = self.all_suppliers[supplier_selected_list_item]
        self.order.supplier_address = self.order_address.current_address
        self.order.sup_ref_field = self.sup_ref_field.GetValue().strip()
        self.order.invoice_address = self.invoice_address.current_address
        self.order.delivery_address = self.delivery_address.current_address

        # Get date as a python date object
        try:
            self.order.datestamp = wxdate_to_date(self.order_date_field.GetValue())
        except Exception:
            return False, "The date is not valid"

        number_string = self.order_number_field.GetValue().strip()
        try:
            self.order.number = int(number_string)
        except Exception:
            return False, "The order number is not valid"

        # Discount
        discount_string = self.discount_rate_field.GetValue().strip()
        try:
            self.order.discount = D(discount_string.replace(vs_cfg["CP"], "."))
        except Exception:
            return False, "The discount value is incorrect."

        # Contact
        contact_selected_list_item = self.enq_field.GetCurrentSelection()
        if contact_selected_list_item < 0:
            self.order.contact = None
        else:
            self.order.contact = self.form_contacts[contact_selected_list_item]

        # Create an item_list of items being ordered
        # Each list element being itself a dictionary of
        # {"Qty":Decimal, "Description":string, "Unit Cost":Decimal, "Discount":boolean "VAT":Decimal (The % value)}

        status, item_list, message = self.purchaseitems_table.check_items()
        if not status:
            return False, message
        self.order.item_list = item_list

        # print check field
        self.order.print_check = self.print_check.GetValue()

        # Get totals fields

        if self.show_vat:
            vat = self.VAT_value_field.GetValue().strip()
            status, self.order.vat = string_to_decimal(vat)
            if not status:
                return False, "The total VAT value is not valid"
        else:
            self.order.vat = D("0.00")

        cost = self.cost_value_field.GetValue().strip()
        status, self.order.cost = string_to_decimal(cost)
        if not status:
            return False, "The total cost value is not valid"

        # claim vat check field
        self.order.claim_vat = self.claim_vat_field.GetValue()

        # Check these input order details are ok via vatstuff_packages.vs_verify.purchases
        status, message = new_purchaseorder_check(self.order)
        if not status:
            return False, message

        return True, None


