####### VATStuff #######
#
# make_payment.py  - Pay a supplier
#
# Version : 0.7.0
# Date : 20100913
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#

"""Records payment against a purchase order

These functions are called with the 'order' which has been chosen
from the list of open orders created under purchases_tab.py

make_payment_dialog   -  a function that calls MakePaymentDialog
MakePaymentDialog     -  a dialog to create a payment
show_payment_note     -  a function that calls ShowPaymentNoteDialog
ShowPaymentNoteDialog -  a dialog to view a make payment
"""


import datetime
import decimal

import wx

from vatstuff_packages.vs_database.grid_objects import *
from vatstuff_packages.vs_database.transactions import *
from vatstuff_packages.vs_database import save_makepayment

from vatstuff_packages.vs_gui.gui_functions import *
from vatstuff_packages.vs_gui.address_panel import AddressSpin

from vatstuff_packages.vs_verify import form_objects
from vatstuff_packages.vs_verify.purchases import make_payment_check
from vatstuff_packages.vs_utilities import *

from vatstuff_packages.vs_gui.purchases import purchaseorder_status
from vatstuff_packages.vs_gui.purchases import view_orders

if vs_cfg["REPORTLABS_AVAILABLE"]:
    import vatstuff_packages.vs_pdf.page_setup
    import vatstuff_packages.vs_pdf.makepayment_form
    import vatstuff_packages.vs_pdf.pdf_functions

D = decimal.Decimal   # D used to avoid typing


def make_payment_dialog(parent, order):
    """Creates a MakePaymentDialog"""
    make_payment_dialog = MakePaymentDialog(parent, order)

    # If the cursor is busy, unbusy it - as
    # now waiting for a button press
    busy_cursor_off()

    if make_payment_dialog.ShowModal() != wx.ID_OK:
        make_payment_dialog.Destroy()
        return

    wx.Yield()

    # A payment has been made, give the user the opportunity to create a payment note
    payment_made = make_payment_dialog.payment_made
    make_payment_dialog.Destroy()
    if vs_yesno(parent, "Payment saved\nDo you wish to create a payment note?", "Done"):
        busy_cursor_on()
        wx.Yield()
        show_payment_note(parent, payment_made)

    if order.is_open and (order.invoice_cost + order.invoice_vat == order.paid):
        if vs_yesno(parent, """\
Please note : the order is still open. If no further invoices
or payments are expected, you can close the purchase order
by viewing it, and choosing the 'Close Order' button.
Do you wish to view order %s now?""" % (order.number,), "DONE"):
            busy_cursor_on()
            wx.Yield()
            view_orders.show_order_frame(order)


class MakePaymentDialog(vsDialog):
    "Called when a payment to a supplier is to be made"

    def __init__(self, parent, order):
        """Create the Dialog"""
        frame_title = "Payment for Order : %i" % order.number
        vsDialog.__init__(self, parent, frame_title)
        
        # Take a note of the variables
        self.order = order

        # Store all form contents in an MakePaymentForm object
        self.payment = form_objects.MakePaymentForm()

        self.payment.status = 0
        # STATUS
        # 0 if this payment is a normal payment
        # 1 if this is a payment for a 'quick purchase' order

        # order.supplier is the supplier who sent the order, but the supplier may have
        # had details changed since the order was sent, so get the visible supplier
        # with order.supplier.number
        self.payment.supplier = Supplier.from_number(order.supplier.number)

        # The purchase order the payment refers to
        self.payment.purchaseorder = order

        # Initial default payment is total invoice amount less the amount already paid
        # which is also the maximum payment accepted

        self.max_amount = order.invoice_cost + order.invoice_vat - order.paid
        self.payment.amount = self.max_amount

        button_size = (100, vs_cfg["BUTTON_HEIGHT"])

        # Create a vertical sizer to hold everything
        sizer=wx.BoxSizer(wx.VERTICAL)

        # And a horizontal sizer to hold supplier address and purchase order details
        order_sizer = wx.BoxSizer(wx.HORIZONTAL)

        # A vertical sizer holds the supplier name with the address below it
        supplier_sizer=wx.BoxSizer(wx.VERTICAL)

        # Add the supplier name
        supplier_text = "Supplier: %s" % self.payment.supplier.name
        supplier_sizer.Add(wx.StaticText(self, -1, supplier_text), 0, wx.ALIGN_LEFT)

        supplier_sizer.Add(wx.StaticText(self, -1, "Payment address :"), 0, wx.ALIGN_LEFT)

        self.address_field = AddressSpin(self, self.payment.supplier, payment_point=1)
        supplier_sizer.Add(self.address_field, 0, wx.ALIGN_CENTER)


        # Add the supplier_sizer to the order_sizer
        order_sizer.Add(supplier_sizer, 0, wx.LEFT | wx.ALIGN_RIGHT, 20)

        # Show the purchase order status panel
        purchaseorder_status_panel = purchaseorder_status.PurchaseOrderStatusPanel(self, order,
                                                                                         show_address = False,
                                                                                         show_claim_vat = False)
        order_sizer.Add(purchaseorder_status_panel, 0, wx.LEFT | wx.ALIGN_LEFT, 20)

        # add supplier address a and purchase order status panel to the main sizer
        sizer.Add(order_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        supplierinvoice_sizer=wx.BoxSizer(wx.HORIZONTAL)

        inv_text = """\
Select one or more invoices for which this payment is to be made:
(Hold down Ctrl and left mouse click to choose multiple invoices)"""

        supplierinvoice_sizer.Add(wx.StaticText(self, -1, inv_text), 0, wx.RIGHT | wx.ALIGN_RIGHT, 10)

        self.all_openinvoices = order.get_openinvoices()

        choice_list =[]
        for invoice in self.all_openinvoices:
            item = "Invoice: %s  %s" % (invoice.number, invoice.datestamp.strftime("%d %b %Y"))
            choice_list.append(item)


        self.invoice_list_field = wx.ListBox(self, -1, choices = choice_list, style = wx.LB_MULTIPLE)
        self.invoice_list_field.SetToolTipString("Set one or more invoices the payment covers")
        supplierinvoice_sizer.Add(self.invoice_list_field, 0, wx.LEFT | wx.ALIGN_LEFT, 20)

        sizer.Add(supplierinvoice_sizer, 0, wx.ALIGN_CENTER)

        help_text = """Specify the cash or bank account from which this money has been taken."""
        sizer.Add(wx.StaticText(self, -1, help_text), 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # And a grid sizer to hold payment details
        payment_sizer = wx.FlexGridSizer(rows=3, cols=2, hgap=20, vgap=10)

        # Account
        payment_sizer.Add(wx.StaticText(self, -1, "Account:"), 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.all_accounts = Account.get_non_system()

        # self.account_list shows the strings displayed in the choices field
        self.account_list = [ a.fullname for a in self.all_accounts ]

        if len(self.account_list) == 0:
            raise VS_ERROR, "No accounts are available"

        self.payment_account_field = wx.Choice(self, -1, size=(200, vs_cfg["BUTTON_HEIGHT"]), choices=self.account_list)

        # Get the last cash account used - and pre-select it
        last_account = get_last_cash_account()
        if last_account:
            for key, acc in enumerate(self.all_accounts):
                if last_account == acc:
                    self.payment_account_field.SetSelection(key)
                    break
        self.payment_account_field.SetToolTipString("Set a bank or cash account")
        payment_sizer.Add(self.payment_account_field, 0, wx.ALIGN_LEFT)

        # date
        payment_sizer.Add(wx.StaticText(self, -1, "Date:"), 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)
        self.payment_date_field = wx.DatePickerCtrl(self, -1,
                                                    wx.DateTime.Now(), 
                                                    size=(150, vs_cfg["BUTTON_HEIGHT"]),
                                                    style=wx.DP_DROPDOWN | wx.DP_SHOWCENTURY)
        self.payment_date_field.SetToolTipString("The date of the payment")
        payment_sizer.Add(self.payment_date_field, 0, wx.ALIGN_LEFT)

        # pay amount
        payment_sizer.Add(wx.StaticText(self, -1, "Pay:"), 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)
        self.amount_field = wx.TextCtrl(self, -1, vs_cfg["SYMBOL_ZERO_STRING"], style=wx.ALIGN_RIGHT, size=(150, vs_cfg["BUTTON_HEIGHT"]))
        self.amount_field.SetToolTipString("Amount to be paid")
        self.amount_field.SetMaxLength(12)
        payment_sizer.Add(self.amount_field, 0, wx.ALIGN_LEFT)

        # add payment sizer to main sizer
        sizer.Add(payment_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # If there is only one invoice in the choice list, auto select
        # it, and set the amount in the amount field
        if len(choice_list) == 1:
            self.invoice_list_field.SetSelection(0)
            self.amount_field.SetValue(to_currency_string(self.all_openinvoices[0].outstanding))

        # create a horizontal sizer to hold buttons
        button_sizer = wx.BoxSizer(wx.HORIZONTAL)

        # Cancel button
        self.cancel_button = wx.Button(self, wx.ID_CANCEL, size=button_size)
        self.cancel_button.SetToolTipString("Click to cancel.")
        button_sizer.Add(self.cancel_button, 0, wx.RIGHT | wx.ALIGN_CENTER, 40)

        # Apply button
        self.apply_button = wx.Button(self, wx.ID_APPLY, size=button_size)
        self.apply_button.SetToolTipString("Click to apply payment.")
        button_sizer.Add(self.apply_button, 0, wx.LEFT | wx.ALIGN_CENTER, 40)

        sizer.Add(button_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        self.Bind(wx.EVT_BUTTON, self.cancel_button_pressed, self.cancel_button)
        self.Bind(wx.EVT_BUTTON, self.apply_button_pressed, self.apply_button)
        self.Bind(wx.EVT_LISTBOX, self.selection_made, self.invoice_list_field)

        # Set the size
        self.SetSizer(sizer)
        sizer.SetSizeHints(self)


    def cancel_button_pressed(self, event):
        "Cancel button - Close down"
        self.EndModal(wx.ID_CANCEL)


    def selection_made(self, event):
        # Get the list of invoices
        try:
            selections = self.invoice_list_field.GetSelections()
            if not selections:
                self.amount_field.SetValue(vs_cfg["SYMBOL_ZERO_STRING"])
            invoices = []
            for selection in selections:
                invoices.append(self.all_openinvoices[selection])
            unpaid = D("0.00")
            for invoice in invoices:
                unpaid += invoice.outstanding
            self.amount_field.SetValue(to_currency_string(unpaid))
        except Exception, e:
            error_message("Failure in handling the selection", e)


    @writecheck
    @button_wrap
    def apply_button_pressed(self, event):
        "Apply button - check and apply all fields"

        status, message = self.check_fields()
        if not status:
            vs_message(self, message, "ERROR")
            return

        # Create transaction_text, transaction_list, transaction_date
        # to send to the ConfirmTransactionDialog
        self.transaction_text = "Order %s : Payment to %s" % (self.payment.purchaseorder.number,
                                                                self.payment.supplier.name)
        self.transaction_date = self.payment.datestamp

        # self.transaction_list is a list of lists [account, debit, credit]
        # account being an account object, debit and credit being decimal objects

        #     A bank or cash account  Debit by zero  Credit by the payment amount
        t1 = [self.payment.account,   D("0.00"),     self.payment.amount ]

        #     Suppliers account               Debit by the amount     Credit by zero
        t2 = [self.payment.supplier.account,  self.payment.amount,    D("0.00") ]

        self.transaction_list = [t1, t2]

        wx.Yield()
        # Confirm the transaction (function and dialog in gui_functions)
        status, self.transaction_text = confirm_transaction(self,
                                                            self.transaction_text,
                                                            self.transaction_list,
                                                            self.transaction_date)
        if not status:
            vs_message(self, "The payment has not been implemented", "CANCELLED")
            return

        # Must now implement the transaction and save the payment
        busy_cursor_on()
        wx.Yield()
        status, self.payment_made =save_makepayment.save(self.payment,
                                                    self.transaction_date,
                                                    self.transaction_text,
                                                    self.transaction_list)

        if not status:
            vs_message(self, "Failed to save the payment to supplier", "ERROR")
            return 

        # and redisplay transactions and the account tree as account balance will have changed
        big_refresh()
        # all done - so close down
        self.EndModal(wx.ID_OK)


    #----------- Check and store all fields button

    def check_fields(self):
        """Checks the fields, and fills in the self.payment object
           returns (True, None) if ok, or (False, "warning message") if not"""

        # Get the address field
        self.payment.supplier_address = self.address_field.current_address

        # Get the list of invoices
        selections = self.invoice_list_field.GetSelections()
        if not selections:
            return False, "At least one invoice must be selected"

        invoices = [ self.all_openinvoices[selection] for selection in selections ]

        # Get date as a python date object
        try:
            self.payment.datestamp = wxdate_to_date(self.payment_date_field.GetValue())
        except Exception:
            return False, "The date input is not valid"

        # The cash or bank account used to record this purchase
        account_selected_list_item = self.payment_account_field.GetCurrentSelection()
        if account_selected_list_item < 0:         # blank or unselected
            self.payment.account = None
        else:
            self.payment.account = self.all_accounts[account_selected_list_item]

        # Get amount paid field
        amount_string = self.amount_field.GetValue().strip()
        status, amount = string_to_decimal(amount_string)
        if not status:
            return False, "The amount received is not valid"

        self.payment.amount = amount

        # Check these input payment details are ok via vatstuff_packages.vs_verify.purchases
        status, message = make_payment_check(self.payment, invoices)
        if not status:
            return False, message

        return True, None


# The ShowPaymentNoteDialog is used to view a make payment
# It is normally used by calling the function show_payment_note

def show_payment_note(parent, payment):
    """Creates a ShowPaymentNoteDialog dialog"""

    show_payment_note_dialog = ShowPaymentNoteDialog(parent, payment)
    # If the cursor is busy, unbusy it - as
    # now waiting for a button press
    busy_cursor_off()
    show_payment_note_dialog.ShowModal()
    show_payment_note_dialog.Destroy()



class ShowPaymentNoteDialog(vsDialog):
    """Called to show the makepayment"""

    def __init__(self, parent, payment):
        """Create the Dialog"""
        vsDialog.__init__(self, parent, title="Payment Note")
        
        # Take a note of the variables
        self.payment = payment

        button_size = (100, vs_cfg["BUTTON_HEIGHT"])

        # Create a vertical sizer to hold everything
        sizer=wx.BoxSizer(wx.VERTICAL)

        if payment.supplier_address:
            # payment address on the left of a horizontal sizer, payment details on the right
            details_sizer=wx.BoxSizer(wx.HORIZONTAL)
            supplier_text = "Supplier: %s\n%s" % (payment.supplier.name,
                                                      payment.supplier_address.lines_string)
            details_sizer.Add(wx.StaticText(self, -1, supplier_text), 0, wx.RIGHT | wx.ALIGN_TOP, 80)
            # Put payment details in a grid sizer
            pay_sizer=wx.FlexGridSizer(rows=5, cols=2, hgap=20, vgap=10)
        else:
            # details sizer does not exist, just create the pay sizer with the supplier field added
            # Put payment details in a grid sizer
            pay_sizer=wx.FlexGridSizer(rows=6, cols=2, hgap=20, vgap=10)
            # supplier
            pay_sizer.Add(wx.StaticText(self, -1, "Supplier :"), 0, wx.ALIGN_RIGHT)
            supplier_text = payment.supplier.name
            pay_sizer.Add(wx.StaticText(self, -1, supplier_text), 0, wx.ALIGN_LEFT)

        # order number
        pay_sizer.Add(wx.StaticText(self, -1, "Purchase order :"), 0, wx.ALIGN_RIGHT)
        po_number_text = str(payment.purchaseorder.number)
        pay_sizer.Add(wx.StaticText(self, -1, po_number_text), 0, wx.ALIGN_LEFT)

        # date
        pay_sizer.Add(wx.StaticText(self, -1, "Payment date :"), 0, wx.ALIGN_RIGHT)
        date_text = payment.datestamp.strftime("%d %b %Y")
        pay_sizer.Add(wx.StaticText(self, -1, date_text), 0, wx.ALIGN_LEFT)

        # payment amount
        pay_sizer.Add(wx.StaticText(self, -1, "Payment amount :"), 0, wx.ALIGN_RIGHT)
        amount_text = to_currency_string(payment.amount)
        pay_sizer.Add(wx.StaticText(self, -1, amount_text), 0, wx.ALIGN_LEFT)

        # payment account
        pay_sizer.Add(wx.StaticText(self, -1, "Paid from account :"), 0, wx.ALIGN_RIGHT)
        account_text = payment.account.fullname
        pay_sizer.Add(wx.StaticText(self, -1, account_text), 0, wx.ALIGN_LEFT)

        # Transaction ID
        pay_sizer.Add(wx.StaticText(self, -1, "Transaction ID :"), 0, wx.ALIGN_RIGHT)
        transaction_text = str(payment.transaction.id)
        pay_sizer.Add(wx.StaticText(self, -1, transaction_text), 0, wx.ALIGN_LEFT)

        if payment.supplier_address:
            # details sizer contains address on the left, pay on the right
            details_sizer.Add(pay_sizer, 0, wx.LEFT | wx.ALIGN_LEFT, 10)
            sizer.Add(details_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 10)
        else:
            # details sizer does not exist, just add the pay sizer
            sizer.Add(pay_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # buttons

        button_sizer=wx.BoxSizer(wx.HORIZONTAL)

        # page setup button
        self.page_button = wx.Button(self, label="Page setup", size=button_size)
        self.page_button.SetToolTipString("Click to set up the pdf page size and orientation")   
        button_sizer.Add(self.page_button, 0, wx.RIGHT, 20)

        # pdf button
        self.pdf_button = wx.Button(self, label="Create pdf", size=button_size)
        self.pdf_button.SetToolTipString("Click to create a pdf file.")
        button_sizer.Add(self.pdf_button, 0, wx.RIGHT, 20)

        # Close button
        self.close_button = wx.Button(self, wx.ID_CLOSE, size=button_size)
        self.close_button.SetToolTipString("Click to close.")
        button_sizer.Add(self.close_button, 0)

        sizer.Add(button_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # Set the size to the dialog frame
        self.SetSizer(sizer)
        sizer.SetSizeHints(self)

        self.Bind(wx.EVT_BUTTON, self.close_button_pressed, self.close_button)
        self.Bind(wx.EVT_BUTTON, self.page_button_pressed, self.page_button)
        self.Bind(wx.EVT_BUTTON, self.pdf_button_pressed, self.pdf_button)


    def close_button_pressed(self, event):
        """Close the dialog"""
        self.EndModal(wx.ID_OK)


    @button_wrap
    def page_button_pressed(self, event):
        "page button pressed, setup the pdf page size and orientation"
        vatstuff_packages.vs_pdf.page_setup.page_setup_dialog(self)


    @button_wrap
    def pdf_button_pressed(self, event):
        "Create a pdf file"

        if not vs_cfg["REPORTLABS_AVAILABLE"]:
            # no pdf capability
            vs_message(self, "Function unavailable - the reportlab module is not installed",
                             "ERROR")
            return

        # default filename
        filename= "payment_%s_sent_%s.pdf" % (self.payment.transaction.id, self.payment.datestamp.isoformat())

        # Create the save file dialog.
        status, filepath = save_file_as(self, 'pdf', vs_cfg["REPORTS_PDF_FOLDER"], filename)

        if status:
            busy_cursor_on()
            wx.Yield()
            if vatstuff_packages.vs_pdf.makepayment_form.create_pdf(filepath, self.payment):
                # And show the file in the default viewer
                if not vatstuff_packages.vs_pdf.pdf_functions.show_pdf_file(filepath):
                    vs_message(self, "pdf file created", "Done")
            else:
                vs_message(self, "Unable to create the pdf file", "ERROR")


