####### VATStuff #######
#
# purchaseorder_status.py  - create a panel with order status information
#
# Version : 0.7.0
# Date : 20100913
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#

"""Create a panel with order status information
   provide a project button - to display and create projects"""


import decimal
import wx

from vatstuff_packages.vs_utilities import *
from vatstuff_packages.vs_gui.gui_functions import *

from vatstuff_packages.vs_gui.vat.vt_100 import VATReturnFrame


D = decimal.Decimal   # D used to avoid typing


# ----------------------OrderStatusPanel

class PurchaseOrderStatusPanel(wx.Panel):
    "The panel showing order items"

    def __init__(self, parent, order, show_address = True, show_claim_vat = True):
        "Create the panel"
        wx.Panel.__init__(self, parent)
        
        # Take a note of the variables
        self.parent = parent
        self.order = order
        self.show_claim_vat = show_claim_vat

        # The supplier may have changed their name
        supplier = Supplier.from_number(self.order.supplier.number)

        # Create a vertical sizer to hold everything
        self.sizer=wx.BoxSizer(wx.VERTICAL)

        # And a horizontal sizer to hold order details
        order_sizer = wx.BoxSizer(wx.HORIZONTAL)

        # show_address may be False - for example on a MakePayment form where an option to
        # choose a payment address is placed instead
        if show_address:
            if self.order.supplier_address:
                supplier_text = "Supplier: %s\n%s" % (supplier.name,
                                                      self.order.supplier_address.lines_string)
            else:
                supplier_text = "Supplier: %s" % supplier.name

            order_sizer.Add(wx.StaticText(self, -1, supplier_text), 0, wx.RIGHT | wx.ALIGN_TOP, 80)

        # Create vertical sizer to hold order number, date, amount paid, and claim VAT checkbox
        order_number_sizer=wx.BoxSizer(wx.VERTICAL)
        order_number_sizer.Add(wx.StaticText(self, -1, "Order Number: "+str(self.order.number)), 0, wx.BOTTOM, 10)

        self.order_date_field = wx.StaticText(self, -1, "Order Date: "+self.order.datestamp.strftime("%d %b %Y"))
        order_number_sizer.Add(self.order_date_field, 0, wx.BOTTOM, 10)

        self.vat_unclaimable_field = wx.StaticText(self, -1, "VAT: Not to be claimed")
        order_number_sizer.Add(self.vat_unclaimable_field, 0, wx.BOTTOM, 10)
 
        # add checkbox, may be visible under certain conditions
        self.claim_vat_field = wx.CheckBox(self, -1, "Claim VAT")
        if self.order.claim_vat:
            self.vat_unclaimable_field.SetLabel(" ")
            self.claim_vat_field.SetValue(True)
        else:
            self.claim_vat_field.SetValue(False)
        self.claim_vat_field.SetToolTipString("""\
Untick if VAT cannot be claimed back,
for purchases such as entertainment.""")

        # set the claim vat checkbox visibility
        self.claim_vat_visibility()


        order_number_sizer.Add(self.claim_vat_field, 0)

        order_sizer.Add(order_number_sizer, 0, wx.RIGHT | wx.ALIGN_TOP, 80)

        # all_costs_sizer is a FlexGridSizer sizer,
        # left half has the Purchase order, right has Invoiced costs

        all_costs_sizer = wx.FlexGridSizer(rows=2, cols=2, hgap=40, vgap=10)

        all_costs_sizer.Add(wx.StaticText(self, -1, "Purchase Order"), 0, wx.ALIGN_CENTER)
        all_costs_sizer.Add(wx.StaticText(self, -1, "Total Invoiced"), 0, wx.ALIGN_CENTER)


        # Put order costs in a grid sizer
        order_cost_sizer=wx.FlexGridSizer(rows=4, cols=2, hgap=20, vgap=10)

        cost_string = to_currency_string(self.order.cost)
        vat_string = to_currency_string(self.order.vat)
        total_string = to_currency_string(self.order.total)

        order_cost_sizer.Add(wx.StaticText(self, -1, "Net Cost"), 0, wx.ALIGN_RIGHT)
        order_cost_sizer.Add(wx.StaticText(self, -1, cost_string), 0, wx.ALIGN_RIGHT)
        order_cost_sizer.Add(wx.StaticText(self, -1, "VAT"), 0, wx.ALIGN_RIGHT)
        order_cost_sizer.Add(wx.StaticText(self, -1, vat_string), 0, wx.ALIGN_RIGHT)
        order_cost_sizer.Add(wx.StaticText(self, -1, "Order Total"), 0, wx.ALIGN_RIGHT)
        order_cost_sizer.Add(wx.StaticText(self, -1, total_string), 0, wx.ALIGN_RIGHT)
        order_cost_sizer.Add(wx.StaticText(self, -1, "Amount paid"), 0, wx.ALIGN_RIGHT)

        self.amount_paid_field = wx.StaticText(self, -1, to_currency_string(self.order.paid))
        order_cost_sizer.Add(self.amount_paid_field, 0, wx.ALIGN_RIGHT)

        all_costs_sizer.Add(order_cost_sizer, 0, wx.ALIGN_CENTER)

        # Put invoiced costs in a grid sizer
        invoiced_cost_sizer=wx.FlexGridSizer(rows=4, cols=2, hgap=20, vgap=10)

        invoiced_cost_string = to_currency_string(self.order.invoice_cost)
        invoiced_vat_string = to_currency_string(self.order.invoice_vat)
        invoiced_total = self.order.invoice_vat + self.order.invoice_cost
        invoiced_total_string = to_currency_string(invoiced_total)
        invoiced_outstanding_string = to_currency_string(invoiced_total - self.order.paid)

        invoiced_cost_sizer.Add(wx.StaticText(self, -1, "Net Cost"), 0, wx.ALIGN_RIGHT)
        self.invoiced_cost_field = wx.StaticText(self, -1, invoiced_cost_string)
        invoiced_cost_sizer.Add(self.invoiced_cost_field, 0, wx.ALIGN_RIGHT)

        invoiced_cost_sizer.Add(wx.StaticText(self, -1, "VAT"), 0, wx.ALIGN_RIGHT)
        self.invoiced_vat_field = wx.StaticText(self, -1, invoiced_vat_string)
        invoiced_cost_sizer.Add(self.invoiced_vat_field, 0, wx.ALIGN_RIGHT)

        invoiced_cost_sizer.Add(wx.StaticText(self, -1, "Invoiced Total"), 0, wx.ALIGN_RIGHT)
        self.invoiced_total_field = wx.StaticText(self, -1, invoiced_total_string)
        invoiced_cost_sizer.Add(self.invoiced_total_field, 0, wx.ALIGN_RIGHT)

        invoiced_cost_sizer.Add(wx.StaticText(self, -1, "Outstanding"), 0, wx.ALIGN_RIGHT)
        self.invoiced_outstanding_field = wx.StaticText(self, -1, invoiced_outstanding_string)
        invoiced_cost_sizer.Add(self.invoiced_outstanding_field, 0, wx.ALIGN_RIGHT)

        all_costs_sizer.Add(invoiced_cost_sizer, 0, wx.ALIGN_CENTER)

        order_sizer.Add(all_costs_sizer, 0, wx.ALIGN_TOP)

        self.sizer.Add(order_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # Set the size to the panel
        self.SetSizer(self.sizer)
        self.sizer.SetSizeHints(self)

        self.Bind(wx.EVT_CHECKBOX, self.claim_vat_toggle, self.claim_vat_field)


    def refresh(self):
        "Renew any fields which may have changed"
        # order_text_field

        self.order_date_field.SetLabel("Order Date: "+self.order.datestamp.strftime("%d %b %Y"))
        self.amount_paid_field.SetLabel(to_currency_string(self.order.paid))

        # set the claim vat checkbox visibility
        self.claim_vat_visibility()

        # invoice status
        invoiced_cost_string = to_currency_string(self.order.invoice_cost)
        invoiced_vat_string = to_currency_string(self.order.invoice_vat)
        invoiced_total = self.order.invoice_vat + self.order.invoice_cost
        invoiced_total_string = to_currency_string(invoiced_total)
        invoiced_outstanding_string = to_currency_string(invoiced_total - self.order.paid)

        self.invoiced_cost_field.SetLabel(invoiced_cost_string)
        self.invoiced_vat_field.SetLabel(invoiced_vat_string)
        self.invoiced_total_field.SetLabel(invoiced_total_string)
        self.invoiced_outstanding_field.SetLabel(invoiced_outstanding_string)

        self.sizer.Layout()


    def claim_vat_visibility(self):
        "Hides the claim vat checkbox if necessary"

        # STATUS
        # 0 if this Purchase Order is still open, no invoices received
        # 1 order open, and one or more invoices have been received
        # 2 if normally closed and order fully paid, i.e total == paid
        # 3 if closed, with order total less than that paid, i.e. total<paid
        # 4 if closed, with order total greater than that paid, i.e. total>paid
        # 5 if closed by the order being cancelled with no invoice recieved or payment made

        order_status = self.order.status
        last_vatreturn =  VATReturn.get_last()
        vat_scheme_method = VATScheme().method
        self.invoice_list = self.order.get_invoices()
        self.creditnote_list = self.order.get_creditnotes()

        # start off with it being shown
        self.claim_vat_field.Show()
        if (not self.order.show_vat) or (not self.show_claim_vat) or order_status==5:
            # the claim_vat_field should not be visible if there is no vat, or this
            # panel has been requested not to show it via __init__ arguments
            # or the order has been cancelled - in which case this is irrelevent
            self.claim_vat_field.Hide()
        elif (not order_status) or (not last_vatreturn):
            # no invoices received, or no vatreturn made
            # therefore the claim_vat_field depends on the current vat scheme
             if vat_scheme_method != "STANDARD" and vat_scheme_method != "CASH":
                self.claim_vat_field.Hide()
        else:
            # At least one invoice has been received,
            # and a vat return has been made, does the invoice fall before the last date
            # covered by the vat return?
            last_vatreturn_to_date = last_vatreturn.to_date
            for invoice in self.invoice_list:
                if invoice.date_of_taxpoint <= last_vatreturn_to_date:
                    # yes - so hide the field
                    self.claim_vat_field.Hide()
                    break
            else:
                # invoice received, but after the last vatreturn - so still in the
                # current vat period
                if vat_scheme_method != "STANDARD" and vat_scheme_method != "CASH":
                    self.claim_vat_field.Hide()
        # and finally, if no write access - disable it
        if not vs_cfg["WRITE_ACCESS"]:
            # this is to stop the toggle action from changing the tick, even though it will
            # not change on the database
            self.claim_vat_field.Enable(False)


    @writecheck
    @button_wrap
    def claim_vat_toggle(self, event):
        "The claim VAT check box has been toggled"
        if event.Checked():
            vs_message(self, """\
Ticking this box ensures the VAT on the
associated supplier invoices is reclaimed
when the VAT Return is calculated.""","Tick Claim VAT")
            self.order.claim_vat = True
            # This action also sets associated invoices, creditnotes and supplierinvoice
            # payments flags - all changed in the purchaseorder object class
            self.vat_unclaimable_field.SetLabel(" ")
        else:
            vs_message(self, """\
Normally VAT paid on purchases is reclaimed,
however in some cases, such as Entertainment
expenses, it should not be claimed.
Unticking this box ensures the VAT on the
associated supplier invoices is not claimed
when the VAT Return is calculated.""","Untick Claim VAT")
            self.order.claim_vat = False
            # This action also unsets associated invoices, creditnotes and supplierinvoice
            # payments flags - all changed in the purchaseorder object class
            self.vat_unclaimable_field.SetLabel("VAT: Not to be claimed")
        # So the flag is set in the database, however if a VATReturn
        # window is open, it is now incorrect - so refresh it
        # This is the only thing changed by setting this flag,
        # so do this, rather than a big refresh
        for frm in vs_cfg["TRANSACTION_REFRESH"]:
            if isinstance(frm, VATReturnFrame):
                frm.refresh()


