####### VATStuff #######
#
# quick_purchase.py  - Creates a purchase - order, invoice and payment together
#
# Version : 0.7.0
# Date : 20100913
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#


"""Creates a purchase - order, invoice and payment together

It is run as part of the vatstuff gui interface.
"""


import decimal
import datetime

import wx

from vatstuff_packages.vs_database import save_purchaseorder
from vatstuff_packages.vs_database import save_supplierinvoice
from vatstuff_packages.vs_database import save_makepayment

from vatstuff_packages.vs_verify import form_objects
from vatstuff_packages.vs_utilities import *

from vatstuff_packages.vs_gui.gui_functions import *

D = decimal.Decimal   # D used to avoid typing


def quick_purchase_dialog(parent, supplier):
    """Creates a QuickPurchaseDialog"""
    quick_purchase_dialog = QuickPurchaseDialog(parent, supplier)

    # If the cursor is busy, unbusy it - as
    # now waiting for a button press
    busy_cursor_off()

    quick_purchase_dialog.ShowModal()
    quick_purchase_dialog.Destroy()



class QuickPurchaseDialog(vsDialog):
    "Called when the quick purchase button is pressed"

    def __init__(self, parent, supplier):
        """Create the Dialog"""
        vsDialog.__init__(self, parent, "Quick Purchase")

        # Take a note of the variables
        self.supplier = supplier

        # Can this supplier charge you VAT?
        self.show_vat = self.supplier.pay_vat()

        # Store order data in an OrderForm object
        self.order = form_objects.OrderForm()
        self.order.company = Company()
        self.order.show_vat = self.show_vat
        self.order.claim_vat = True

        # Store invoice data in an SupplierInvoiceForm object
        self.invoice = form_objects.SupplierInvoiceForm()
        self.invoice.claim_vat = True

        # Store payment data in a MakePaymentForm object
        self.pay = form_objects.MakePaymentForm()

        button_size = (100, vs_cfg["BUTTON_HEIGHT"])
        field_size  = (250, vs_cfg["BUTTON_HEIGHT"])

        # Create a vertical sizer to hold everything
        self.sizer=wx.BoxSizer(wx.VERTICAL)

        # Create references in a grid bag sizer
        ref_sizer=wx.GridBagSizer(hgap=20, vgap=10)

        # Order Number
        ref_sizer.Add(wx.StaticText(self, -1, "Order number:"), pos=(0,0), flag=wx.ALIGN_RIGHT)

        self.last_order_number = get_config_item("LAST_ORDER_NUMBER")

        if self.last_order_number:
            self.last_order_number = int(self.last_order_number)
            order_number =  self.last_order_number + 1
        else:
            self.last_order_number = 0
            order_number = 1

        self.order.number = order_number

        ref_sizer.Add(wx.StaticText(self, -1, str(order_number)), pos=(0,1), flag=wx.ALIGN_LEFT)

        # Supplier choice
        self.all_suppliers = Supplier.get_all()
        supplier_list = [s.name for s in self.all_suppliers]

        ref_sizer.Add(wx.StaticText(self, -1, "Supplier:"), pos=(1,0), flag=wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.supplier_field = wx.Choice(self, -1, size=field_size, choices=supplier_list)
        self.supplier_field.SetToolTipString("Set Supplier")
        ref_sizer.Add(self.supplier_field, pos=(1,1), flag=wx.ALIGN_LEFT)

        for key,s in enumerate(self.all_suppliers):
            if s == self.supplier:
                self.supplier_field.SetSelection(key)

        # The date
        ref_sizer.Add(wx.StaticText(self, -1, "The date:"), pos=(2,0), flag=wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)
        self.date_field = wx.DatePickerCtrl(self, -1,
                                            wx.DateTime.Now(), 
                                            size=field_size,
                                            style=wx.DP_DROPDOWN | wx.DP_SHOWCENTURY)
        self.date_field.SetToolTipString("The date of the purchase")
        ref_sizer.Add(self.date_field, pos=(2,1), flag=wx.ALIGN_LEFT)

        # Stock / Expense account
        purchase_text=wx.StaticText(self, -1, "Stock or Expense account to be debited:")
        ref_sizer.Add(purchase_text, pos=(3,0), flag=wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        # Get a list of accounts - Need accounts with system numbers 0 and 1
        # get_accounts_by_system_list(system_list) - in odd_functions module         
        self.all_accounts = Account.get_non_system()

        # self.account_list is going to show the strings displayed in the accounts choices field
        self.account_list = [ a.fullname for a in self.all_accounts ]

        if not self.account_list:
            raise VS_ERROR, "No accounts are available"

        self.stock_account_field = wx.Choice(self, -1, size=field_size, choices=self.account_list)
        self.stock_account_field.SetToolTipString("Set a Stock account, if this purchase retains an asset")

        # Get the last account used, and set the account selection the same as previously used:
        last_account = get_last_stock_account()
        if last_account:
            for key, acc in enumerate(self.all_accounts):
                if last_account == acc:
                    self.stock_account_field.SetSelection(key)
                    break
        else:
            # As default, set the account selection as the last in the choice to avoid the bank accounts
            self.stock_account_field.SetSelection(len(self.account_list)-1)

        ref_sizer.Add(self.stock_account_field, pos=(3,1), flag=wx.ALIGN_LEFT)

        # Cash / Bank account

        account_text=wx.StaticText(self, -1, "Cash or Bank account to be credited:")
        ref_sizer.Add(account_text, pos=(4,0), flag=wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.payment_account_field = wx.Choice(self, -1, size=field_size, choices=self.account_list)
        self.payment_account_field.SetToolTipString("Set a bank or cash account")

        # Get the last cash account used - and pre-select it
        last_account = get_last_cash_account()
        if last_account != None:
            for key, acc in enumerate(self.all_accounts):
                if last_account == acc:
                    self.payment_account_field.SetSelection(key)
                    break

        ref_sizer.Add(self.payment_account_field, pos=(4,1), flag=wx.ALIGN_LEFT)

        # Description
        ref_sizer.Add(wx.StaticText(self, -1, "Transaction Memo:"),
                      pos=(5,0), span=(1,2), flag=wx.ALIGN_CENTER)

        description_text = "Order %i:" % order_number
        self.description_field = wx.TextCtrl(self, -1, description_text, size=(400, vs_cfg["BUTTON_HEIGHT"]))
        self.description_field.SetToolTipString("Memo or purchase description")
        self.description_field.SetMaxLength(40)
        ref_sizer.Add(self.description_field, pos=(6,0), span=(1,2), flag=wx.ALIGN_CENTER)

        # Invoice number
        ref_sizer.Add(wx.StaticText(self, -1, "Suppliers invoice number (if any):"),
                      pos=(7,0), span=(1,2), flag=wx.ALIGN_CENTER)

        invoice_number_text = ""
        self.invoice_number_field = wx.TextCtrl(self, -1, invoice_number_text, size=(400, vs_cfg["BUTTON_HEIGHT"]))
        self.invoice_number_field.SetToolTipString("The suppliers invoice number")
        self.invoice_number_field.SetMaxLength(40)
        ref_sizer.Add(self.invoice_number_field, pos=(8,0), span=(1,2), flag=wx.ALIGN_CENTER)


        # Item details
        ref_sizer.Add(wx.StaticText(self, -1, "Description of Item to be purchased:"),
                      pos=(9,0), span=(1,2), flag=wx.ALIGN_CENTER)

        self.all_items = PurchaseItem.get_all()
        self.item_selection = [ item.description for item in self.all_items ]

        self.item_description_field = wx.ComboBox(self, -1, value="", size=(400, vs_cfg["BUTTON_HEIGHT"]),
                                                  choices=self.item_selection,
                                                  style= wx.CB_DROPDOWN | wx.CB_SORT)
        self.item_description_field.SetToolTipString("Choose item, or type new item to be purchased")
        ref_sizer.Add(self.item_description_field, pos=(10,0), span=(1,2), flag=wx.ALIGN_CENTER)

        # VAT Rate - Create a choice of VAT symbols
        ref_sizer.Add(wx.StaticText(self, -1, "VAT Rate:"), pos=(11,0), flag=wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.all_vat_rates = VATRate.get_all()

        vat_rate_selection = [ str(rate) for rate in self.all_vat_rates ]

        # pre-pend the vat_rate_selection list with an empty string
        vat_rate_selection.insert(0, "")

        # pre-pend the self.all_vat_rates list with a None value, to match the "" symbol
        self.all_vat_rates.insert(0, None)

        self.vat_rate_field = wx.Choice(self, -1, size=field_size, choices=vat_rate_selection)
        self.vat_rate_field.SetToolTipString("Choose a VAT Rate")
        ref_sizer.Add(self.vat_rate_field, pos=(11,1), flag=wx.ALIGN_LEFT)

        self.vat_rate = self.all_vat_rates[0]

        # If vat is not shown, then disable the VAT field
        if self.show_vat:
            # As default, choose the Standard VAT Rate
            for key,s in enumerate(vat_rate_selection):
                if "S" in s:
                    self.vat_rate_field.SetSelection(key)
                    self.vat_rate = self.all_vat_rates[key]
                    break
        else:
            self.vat_rate_field.Enable(False)
            self.vat_rate_field.SetSelection(0)


        # Totals description
        ref_sizer.Add(wx.StaticText(self, -1, "Input the Net Cost or Total, whichever is more convenient"),
                      pos=(12,0), span=(1,2), flag=wx.ALIGN_CENTER)

        # Net Cost
        ref_sizer.Add(wx.StaticText(self, -1, "Net Cost:"), pos=(13,0), flag=wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.cost_field = wx.TextCtrl(self, -1, vs_cfg["SYMBOL_ZERO_STRING"], style=wx.ALIGN_RIGHT, size=field_size)
        self.cost_field.SetToolTipString("Cost (not including VAT) of the item purchased")
        self.cost_field.SetMaxLength(12)
        ref_sizer.Add(self.cost_field, pos=(13,1), flag=wx.ALIGN_LEFT)

        # VAT
        ref_sizer.Add(wx.StaticText(self, -1, "VAT:"), pos=(14,0), flag=wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        # Create a Horizontal sizer for the VAT Value, and the checkbox 
        vat_sizer=wx.BoxSizer(wx.HORIZONTAL)

        self.VAT_field = wx.StaticText(self, -1, vs_cfg["SYMBOL_ZERO_STRING"])
        vat_sizer.Add(self.VAT_field, 0, wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        # add some expanding space
        vat_sizer.Add((20,40), 1)

        self.claim_vat_field = wx.CheckBox(self, -1, "Claim VAT")
        self.claim_vat_field.SetValue(True)
        self.claim_vat_field.SetToolTipString("Untick for VAT that cannot be claimed\nsuch as entertainment expenses")
        if not self.show_vat:
            self.claim_vat_field.Hide()
        self.vat_scheme_method = VATScheme().method
        if self.vat_scheme_method != "STANDARD" and self.vat_scheme_method != "CASH":
            self.claim_vat_field.Hide()

        vat_sizer.Add(self.claim_vat_field, 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        # Add the vat sizer to the ref_sizer - allow it to expand to fill this cell
        ref_sizer.Add(vat_sizer, pos=(14,1), flag=wx.EXPAND)


        # Total Cost
        ref_sizer.Add(wx.StaticText(self, -1, "Total:"), pos=(15,0), flag=wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.total_field = wx.TextCtrl(self, -1, vs_cfg["SYMBOL_ZERO_STRING"], style=wx.ALIGN_RIGHT, size=field_size)
        self.total_field.SetToolTipString("Total cost (including VAT) of the item purchased")
        self.total_field.SetMaxLength(12)
        ref_sizer.Add(self.total_field, pos=(15,1), flag=wx.ALIGN_LEFT)

        self.sizer.Add(ref_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)


        # ---------------------- Apply - Cancel -- buttons

        # Create a horizontal button_sizer to hold the buttons
        button_sizer=wx.BoxSizer(wx.HORIZONTAL)

        # Apply button
        self.apply_button = wx.Button(self, wx.ID_APPLY, size=button_size)
        self.apply_button.SetToolTipString("Click to implement the purchase.")
        button_sizer.Add(self.apply_button, 0, wx.RIGHT, 40)

        # Cancel button
        self.cancel_button = wx.Button(self, wx.ID_CANCEL, size=button_size)
        self.cancel_button.SetToolTipString("Click to cancel.")
        button_sizer.Add(self.cancel_button, 0)

        # Add button_sizer to the main sizer
        self.sizer.Add(button_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 30)


        # Set the size to the panel
        self.SetSizer(self.sizer)
        self.sizer.SetSizeHints(self)

        # Bind the fields

        self.Bind(wx.EVT_CHOICE, self.supplier_pressed, self.supplier_field)
        self.Bind(wx.EVT_COMBOBOX, self.item_change, self.item_description_field)
        self.Bind(wx.EVT_CHOICE, self.vat_or_cost_change, self.vat_rate_field)
        self.Bind(wx.EVT_TEXT, self.vat_or_cost_change, self.cost_field)
        self.Bind(wx.EVT_TEXT, self.total_change, self.total_field)
        self.Bind(wx.EVT_BUTTON, self.apply_button_pressed, self.apply_button)
        self.Bind(wx.EVT_BUTTON, self.cancel_button_pressed, self.cancel_button)
        self.Bind(wx.EVT_CHECKBOX, self.claim_vat_toggle, self.claim_vat_field)


    # -------- define handler methods

    def claim_vat_toggle(self, event):
        "The claim VAT check box has been toggled"
        try:
            if not event.Checked():
                vs_message(self, """\
Normally VAT paid on purchases is reclaimed, however
in some cases, such as Entertainment expenses, it
should not be claimed. Unticking this box ensures
the VAT on this purchase is not claimed when the
VAT Return is calculated.""","Untick Claim VAT")
        except Exception, e:
            error_message("Failure in toggling Claim VAT checkbox", e)


    def supplier_pressed(self, event):
        """Supplier choice pressed"""
        try:
            # Get supplier currently shown
            supplier_selected_list_item = self.supplier_field.GetCurrentSelection()
            # Check same supplier has'nt been chosen again
            new_supplier = self.all_suppliers[supplier_selected_list_item]
            if new_supplier == self.supplier:
                return
            self.supplier = new_supplier
            # Can this supplier charge you VAT?
            self.show_vat = self.supplier.pay_vat()
            self.order.show_vat = self.show_vat
            # Whenever a supplier changes, set the self.claim_vat_field to the default of True
            self.claim_vat_field.SetValue(True)
            if self.show_vat:
                self.vat_rate_field.Enable(True)
                if self.vat_scheme_method == "STANDARD" or self.vat_scheme_method == "CASH":
                    self.claim_vat_field.Show()
                else:
                    self.claim_vat_field.Hide()
            else:
                self.vat_rate_field.Enable(False)
                self.vat_rate_field.SetSelection(0)
                self.claim_vat_field.Hide()
                self.vat_or_cost_change(event)
        except Exception, e:
            error_message("Failure in updating supplier", e)


    def item_change(self, event):
        "set the item to be purchased"
        try:
            # Is the item one of the purchase items
            description = self.item_description_field.GetValue().strip()

            selected_item = None
            for key, item in enumerate(self.item_selection):
                if description == item:
                    selected_item = self.all_items[key]
                    break

            if not selected_item: return

            cost = selected_item.cost

            # Set the Cost
            self.cost_field.SetValue(to_currency_string(cost))

            if not self.show_vat:
                return

            # Set the VAT selection
            vatobject = selected_item.vatrate

            for key,v in enumerate(self.all_vat_rates):
                if v == vatobject:
                    self.vat_rate_field.SetSelection(key)
                    # and implement the vat change
                    self.vat_or_cost_change(event)
                    break
        except Exception, e:
            error_message("Failure in calculating value change", e)


    def vat_or_cost_change(self, event):
        "Set the cost or vat value"
        try:
            # First unbind the total_field handler, or will get a loop of events
            if not self.Unbind(wx.EVT_TEXT, source=self.total_field):
                raise VS_ERROR, "Unable to unbind Total Field in quick_purchase.py"

            # Get VAT rate currently shown
            VAT_rate_selected_list_item = self.vat_rate_field.GetSelection()
            if VAT_rate_selected_list_item == 0:
                vatrate = D("0.0")
            else:
                vatrate = self.all_vat_rates[VAT_rate_selected_list_item].rate

            # Get Net cost currently shown
            string_cost = self.cost_field.GetValue().strip()
            status, cost = string_to_decimal(string_cost)
            if not status:
                cost = D("0.00")

            if vatrate == D("0.0"):
                vat_amount = D("0.00")
            else:
                result = (cost * vatrate) / D("100.000")
                vat_amount = result.quantize(D('.01'), rounding=decimal.ROUND_HALF_EVEN)
            # Place this vat amount into the self.VAT_field static text field
            self.VAT_field.SetLabel(to_currency_string(vat_amount))

            # The new total is cost plus vat_amount
            total= cost+vat_amount
            self.total_field.SetValue(to_currency_string(total))

            # Now that the total has changed, rebind it
            self.Bind(wx.EVT_TEXT, self.total_change, self.total_field)
            self.sizer.Layout()
        except Exception, e:
            error_message("Failure in calculating value change", e)


    def total_change(self, event):
        "total value has changed"
        try:
            # First unbind the cost_field handler, or will get a loop of events
            if not self.Unbind(wx.EVT_TEXT, source=self.cost_field):
                raise VS_ERROR, "Unable to unbind Cost Field in quick_purchase.py"

            # Get VAT rate currently shown
            VAT_rate_selected_list_item = self.vat_rate_field.GetSelection()
            if VAT_rate_selected_list_item == 0:
                vatrate = D("0.0")
            else:
                vatrate = self.all_vat_rates[VAT_rate_selected_list_item].rate

            # Get total currently shown
            string_total = self.total_field.GetValue().strip()
            status, total = string_to_decimal(string_total)
            if not status:
                total = D("0.00")

            if vatrate == D("0.0"):
                vat_amount = D("0.00")
            else:
                vat_result = (total * vatrate) / (D("100.000")+vatrate)
                vat_amount = vat_result.quantize(D('.01'), rounding=decimal.ROUND_HALF_EVEN)

            cost = total-vat_amount
            self.cost_field.SetValue(to_currency_string(cost))

            # Place the vat amount into the self.VAT_field static text field
            self.VAT_field.SetLabel(to_currency_string(vat_amount))

            # Now that the cost has changed, rebind it
            self.Bind(wx.EVT_TEXT, self.vat_or_cost_change, self.cost_field)
            self.sizer.Layout()
        except Exception, e:
            error_message("Failure in calculating value change", e)


    def cancel_button_pressed(self, event):
        "Cancel button - Close down"
        self.EndModal(wx.ID_CANCEL)


    @writecheck
    @button_wrap
    def apply_button_pressed(self, event):
        "Apply button - check and apply all fields"

        if not self.check_fields():
            return

        wx.Yield()
        # Ask for confirmation
        message_text = "Purchase item from %s\nAre you sure?" % self.order.supplier.name
        if not vs_yesno(self, message_text, "Confirm transaction"):
            return
        busy_cursor_on()
        wx.Yield()

        ########### Purchase Order ############
        # Create transaction_text, transaction_list, transaction_date

        transaction_text = self.description_field.GetValue().strip()
        transaction_date = self.order.datestamp

        # transaction_list is a list of lists [account, debit, credit]
        # account being an account object, debit and credit being decimal objects

        ## Purchase account: either an asset (stock) to be bought or an expense to be paid

        #      The purchase account      Debit by cost plus vat of the order   Credit by zero
        t1 = [ get_account_by_system(7), self.order.cost + self.order.vat,     D("0.00") ]

        if not t1[0]:
            vs_message(self, "A Purchases account was not found", "ERROR")
            return

        #      Suppliers account            Debit by zero  Credit this by the amount cost plus VAT
        t2 = [ self.order.supplier.account,  D("0.00"),    self.order.cost + self.order.vat  ]
        if not t2[0]:
            vs_message(self, "This suppliers account cannot be found", "ERROR")
            return

        transaction_list = [t1, t2]

        # This is a try - finally cause, that ensures a big_refresh takes place, even on an error
        # however it does not catch exceptions - that is left to dialog_apply
        try:

            po = save_purchaseorder.save(self.order,
                                            transaction_date,
                                            transaction_text,
                                            transaction_list)

            if not po:
                vs_message(self, "An error occurred creating the order", "ERROR")
                return

            ########### Create invoice ############

            self.invoice.purchaseorder = po
            transaction_text = "Invoice received from %s" % self.invoice.supplier.name

            # transaction_list is a list of lists [account, debit, credit]
            # account being an account object, debit and credit being decimal objects

            # Purchase account: either an asset (stock) to be bought or an expense to be paid

            #      The purchase account     Debit by zero   Credit by the cost plus vat of the invoice
            t1 = [ get_account_by_system(7),  D("0.00"),    self.invoice.cost + self.invoice.vat ]

            if not t1[0]:
                vs_message(self, "A Purchases account was not found", "ERROR")
                return

            #     Stock/Expense account  Debit by the cost   Credit by zero
            t2 = [self.invoice.account,  self.invoice.cost,  D("0.00")  ]  
            if not t2[0]:
                vs_message(self, "The stock account cannot be found", "ERROR")
                return

            transaction_list = [t1, t2]

            if self.invoice.vat != D(0):
                # If VAT is being paid, store it in the VAT owed Account

                #     VAT owed account          Debit by the VAT amount  Credit by zero
                t3 = [get_account_by_system(5),  self.invoice.vat,       D("0.00")  ]
                if not t3[0]:
                    vs_message(self, "The VAT owed account cannot be found", "ERROR")
                    return

                transaction_list.append(t3)

            if not save_supplierinvoice.save(self.invoice,
                                                transaction_date,
                                                transaction_text,
                                                transaction_list):

                vs_message(self, "Purchase order created, but an error occurred creating the invoice", "ERROR")
                return

            ########### Create payment ############

            self.pay.purchaseorder = po
            self.pay.status = 1
            # STATUS
            # 0 if this payment is a normal payment
            # 1 if this is a payment for a 'quick purchase' order

            # get the invoice on the purchase order - in this case it is a list of one supplier invoice
            invoices = po.get_invoices()

            # the MakePaymentForm() has a method which assigns the amount to the invoices cost and vat
            # fields, returns True, if successful, False on failure

            if not self.pay.create_invoice_list(invoices):
                vs_message(self, "Order and invoice created, but an error occurred creating the payment", "ERROR")
                return

            # Create transaction_text, transaction_list, transaction_date

            transaction_text = "Payment to %s" % self.pay.supplier.name
            transaction_date = self.pay.datestamp

            # transaction_list is a list of lists [account, debit, credit]
            # account being an account object, debit and credit being decimal objects

            # The amount to be moved
            amount = self.pay.amount

            # Either a bank or cash account  Debit by zero  Credit by the amount to be paid
            t1 = [ self.pay.account,          D("0.00"),    amount ]

            #     Suppliers account        Debit by the amount  Credit by zero
            t2 = [self.pay.supplier.account,  amount,           D("0.00") ]

            if not t2[0]:
                vs_message(self, "This suppliers account cannot be found", "ERROR")
                return

            transaction_list = [t1, t2]

            save_makepayment.save(self.pay,
                                    transaction_date,
                                    transaction_text,
                                    transaction_list)
        finally:
            # and refresh
            wx.Yield()
            big_refresh()

        vs_message(self, "Transaction implemented", "DONE")
        # all done - so close down
        self.EndModal(wx.ID_OK)


    #----------- Check and store all fields button

    def check_fields(self):
        """Checks the fields, and fills in the self.order, self.invoice and self.pay
           objects returns True if ok, or creates a warning message and False if not"""

        # Check supplier
        supplier_selected_list_item = self.supplier_field.GetSelection()
        self.order.supplier = self.all_suppliers[supplier_selected_list_item]
        self.invoice.supplier = self.order.supplier
        self.pay.supplier = self.order.supplier

        # date
        # Get order, invoice and pay dates all as the same date
        try:
            self.order.datestamp = wxdate_to_date(self.date_field.GetValue())
        except Exception, e:
            vs_message(self, "The date is not valid", "ERROR")
            return False

        self.invoice.datestamp = self.order.datestamp
        self.pay.datestamp = self.order.datestamp

        if vs_cfg["DELETE_TO_DATE"]:
            if self.order.datestamp < vs_cfg["DELETE_TO_DATE"]:
                vs_message(self, "A date prior to the last database deletion date cannot be accepted", "ERROR")
                return False

        # If the taxpoint is in a previous tax period, for which a vat return has already been
        # submitted, then flag as error
        vat_scheme = VATScheme()
        vat_period_start_date = vat_scheme.start_date
        if not vat_period_start_date:
            vs_message(self, "The VAT Scheme has not yet been set", "ERROR")
            return False
        if self.order.datestamp < vat_period_start_date:
            vs_message(self, """\
The tax point date is in a VAT period for
which you have already submitted a VAT return!""", "ERROR")
            return False

        # item list
        # {"Qty":Decimal, "Description":string, "Unit Cost":Decimal, "Discount":boolean "VAT":Decimal (The % value)}

        description = self.item_description_field.GetValue().strip()

        self.invoice.number = self.invoice_number_field.GetValue().strip()

        # Get cost field
        string_cost = self.cost_field.GetValue().strip()
        status, cost = string_to_decimal(string_cost)
        if not status:
            cost = D("0.00")
        if cost <= D(0):
            vs_message(self, "The net cost value is not valid", "ERROR")
            return False
        self.order.cost = cost
        self.invoice.cost = cost

        VAT_rate_selected_list_item = self.vat_rate_field.GetSelection()
        if VAT_rate_selected_list_item == 0:
            vatrate = D("0.0")
        else:
            vatrate = self.all_vat_rates[VAT_rate_selected_list_item].rate

        self.order.item_list = [{"Qty":D("1"),
                                 "Description":description,
                                 "Unit Cost":cost,
                                 "Discount":False,
                                 "VAT":vatrate}]

        # The Stock or Expense account used to record this purchase
        account_selected_list_item = self.stock_account_field.GetSelection()
        if account_selected_list_item < 0:         # blank or unselected
            vs_message(self, """A Stock or Expense account is
needed to record the purchase""", "ERROR")
            return False
        else:
            self.invoice.account = self.all_accounts[account_selected_list_item]

        # The cash or bank account used to record this purchase
        account_selected_list_item = self.payment_account_field.GetSelection()
        if account_selected_list_item < 0:         # blank or unselected
            vs_message(self, """A cash or bank account is
needed to record the purchase""", "ERROR")
            return False
        else:
            self.pay.account = self.all_accounts[account_selected_list_item]


        # Get Total field
        string_total = self.total_field.GetValue().strip()
        status, total = string_to_decimal(string_total)
        if not status:
            total = D("0.00")
        if total <= D(0):
            vs_message(self, "The total value is not valid", "ERROR")
            return False
        if total < cost:
            vs_message(self, "The total value is not valid", "ERROR")
            return False
        self.pay.amount = total

        # VAT is total - cost
        self.order.vat = total - cost
        self.invoice.vat = self.order.vat

        # claim vat check field
        self.order.claim_vat = self.claim_vat_field.GetValue()
        self.invoice.claim_vat = self.order.claim_vat

        return True


