####### VATStuff #######
#
# supplier_creditnote.py  - create dialogs to receive a supplier creditnote
#
# Version : 0.7.0
# Date : 20100913
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#

"""Create dialogs to receive a creditnote from a supplier, two frames
are defined - one against an existing order, and one where a payment
is received with the credit note"""


import datetime
import decimal

import wx
import wx.grid

from vatstuff_packages.vs_database.grid_objects import *
from vatstuff_packages.vs_database import save_suppliercreditnote

from vatstuff_packages.vs_verify import form_objects
from vatstuff_packages.vs_verify.purchases import received_creditnote_against_invoice_check
from vatstuff_packages.vs_verify.purchases import received_creditnote_with_payment_check

from vatstuff_packages.vs_utilities import *

from vatstuff_packages.vs_gui.purchases import purchaseorder_status
from vatstuff_packages.vs_gui.purchases import view_orders
from vatstuff_packages.vs_gui.gui_functions import *

D = decimal.Decimal   # D used to avoid typing


def supplier_creditnote_dialog(parent, order):
    """Creates a SupplierCreditnoteDialog"""
    supplier_creditnote_dialog = SupplierCreditnoteDialog(parent, order)

    # If the cursor is busy, unbusy it - as
    # now waiting for a button press
    busy_cursor_off()

    if supplier_creditnote_dialog.ShowModal() != wx.ID_OK:
        supplier_creditnote_dialog.Destroy()
        return

    supplier_creditnote_dialog.Destroy()

    if order.invoice_cost + order.invoice_vat != order.paid:
       vs_message(parent, "Supplier Credit Note created", "DONE")
       return

    # If the whole order invoices have been paid, give the user a chance to
    # view the order, and to possibly close it

    if vs_yesno(parent, """\
Supplier Credit Note created. If no further invoices or
payments are expected, you can close the purchase order
by viewing it, and choosing the 'Close Order' button.
Do you wish to view order %s now?""" % (order.number,), "DONE"):
        busy_cursor_on()
        wx.Yield()
        view_orders.show_order_frame(order)
 
    

class SupplierCreditnoteDialog(vsDialog):
    "The dialog which records a creditnote received"

    def __init__(self, parent, order):
        """Create the Dialog"""
        frame_title = "Credit Note received for Order : %i" % order.number
        vsDialog.__init__(self, parent, frame_title)
        
        # Take a note of the variables
        self.order = order

        # Store all form contents in an SupplierCreditNoteForm object
        self.creditnote = form_objects.SupplierCreditNoteForm()
        self.creditnote.status = 0  # 0 indicates this creditnote is set against an order

        # order.supplier is the supplier who sent the order, but the supplier may have
        # had details changed since the order was sent, so get the visible supplier
        # with order.supplier.number
        self.creditnote.supplier = Supplier.from_number(order.supplier.number)

        self.creditnote.purchaseorder = order             # The purchase order the creditnote refers to
        self.creditnote.claim_vat = order.claim_vat

        button_size = (100, vs_cfg["BUTTON_HEIGHT"])

        # Create a vertical sizer to hold everything
        sizer=wx.BoxSizer(wx.VERTICAL)

        # Show the purchase order status panel
        purchaseorder_status_panel = purchaseorder_status.PurchaseOrderStatusPanel(self, order, show_claim_vat = False)
        sizer.Add(purchaseorder_status_panel, 0, wx.ALIGN_CENTER)

        help_text = """\
Choose the suppliers invoice number which this credit note refers to, and set the
credit note number with the date and the net cost and VAT amounts specified."""
        sizer.Add(wx.StaticText(self, -1, help_text), 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # And a grid sizer to hold credit note details
        note_sizer = wx.FlexGridSizer(rows=5, cols=2, hgap=20, vgap=10)

        # Suppliers Invoice
        note_sizer.Add(wx.StaticText(self, -1, "Suppliers Invoice:"), 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.all_invoices = self.order.get_invoices()
        invoice_list = [inv.number for inv in self.all_invoices]

        self.invoice_field = wx.Choice(self, -1, size=(200, vs_cfg["BUTTON_HEIGHT"]), choices=invoice_list)
        self.invoice_field.SetToolTipString("Choose the received invoice the credit note refers to")

        note_sizer.Add(self.invoice_field, 0, wx.ALIGN_LEFT)

        # Suppliers credit note number
        note_sizer.Add(wx.StaticText(self, -1, "Suppliers credit note Number:"), 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.sup_creditnote_number_field = wx.TextCtrl(self, -1, "", size=(200, vs_cfg["BUTTON_HEIGHT"]))
        self.sup_creditnote_number_field.SetToolTipString("Suppliers credit note number")
        self.sup_creditnote_number_field.SetMaxLength(20)
        note_sizer.Add(self.sup_creditnote_number_field, 0, wx.ALIGN_LEFT)

        # Credit note date
        note_sizer.Add(wx.StaticText(self, -1, "Date:"), 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.datestamp = wx.DateTime.Now()
        self.creditnote_date_field = wx.DatePickerCtrl(self, -1,
                                                    self.datestamp, 
                                                    size=(150, vs_cfg["BUTTON_HEIGHT"]),
                                                    style=wx.DP_DROPDOWN | wx.DP_SHOWCENTURY)
        self.creditnote_date_field.SetToolTipString("The date of the credit note")
        note_sizer.Add(self.creditnote_date_field, 0, wx.ALIGN_LEFT)

        # Cost field
        note_sizer.Add(wx.StaticText(self, -1, "Net cost:"), 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.cost_field = wx.TextCtrl(self, -1, vs_cfg["ZERO_STRING"], style=wx.ALIGN_RIGHT, size=(150, vs_cfg["BUTTON_HEIGHT"]))
        self.cost_field.SetToolTipString("Net cost specified on this credit note")
        self.cost_field.SetMaxLength(12)
        note_sizer.Add(self.cost_field, 0, wx.ALIGN_LEFT)

        # VAT field
        note_sizer.Add(wx.StaticText(self, -1, "VAT:"), 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.VAT_field = wx.TextCtrl(self, -1, vs_cfg["ZERO_STRING"], style=wx.ALIGN_RIGHT, size=(150, vs_cfg["BUTTON_HEIGHT"]))
        self.VAT_field.SetToolTipString("VAT specified on this credit note")
        self.VAT_field.SetMaxLength(12)
        note_sizer.Add(self.VAT_field, 0, wx.ALIGN_LEFT)

        # The VAT field must be disabled if this supplier cannot charge you VAT
        if not self.creditnote.supplier.pay_vat():
            self.VAT_field.Enable(False)

        sizer.Add(note_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 10) 

        # And a horizontal sizer to hold buttons
        button_sizer = wx.BoxSizer(wx.HORIZONTAL)

        # Cancel button
        self.cancel_button = wx.Button(self, wx.ID_CANCEL, size=button_size)
        self.cancel_button.SetToolTipString("Click to cancel.")
        button_sizer.Add(self.cancel_button, 0, wx.RIGHT, 40)

        # Apply button
        self.apply_button = wx.Button(self, wx.ID_APPLY, size=button_size)
        self.apply_button.SetToolTipString("Click to apply the credit note.")
        button_sizer.Add(self.apply_button, 0)

        sizer.Add(button_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        self.Bind(wx.EVT_BUTTON, self.cancel_button_pressed, self.cancel_button)
        self.Bind(wx.EVT_BUTTON, self.apply_button_pressed, self.apply_button)

        # Set the size
        self.SetSizer(sizer)
        sizer.SetSizeHints(self)


    def cancel_button_pressed(self, event):
        "Cancel button - Close parent"
        self.EndModal(wx.ID_CANCEL)


    @writecheck
    @button_wrap
    def apply_button_pressed(self, event):
        "Apply button - check and apply all fields"
        status, message = self.check_fields()
        if not status:
            vs_message(self, message, "ERROR")
            return

        # set the stock/expense account into the configuration
        set_config_item("LAST_STOCK_ACCOUNT_ID", self.creditnote.account.id)

        # All items ok, now create the transaction
        self.transaction_text = "Credit Note received from %s" % self.creditnote.supplier.name
        self.transaction_date = self.creditnote.datestamp

        # self.transaction_list is a list of lists [account, debit, credit]
        # account being an account object, debit and credit being decimal objects

        # The stock/expense account on the invoice to be credited
        #                                        Debit by zero  Credit by the net cost
        t1 = [self.creditnote.account,             D("0.00"),     self.creditnote.cost ]

        #      The purchase account      Debit by the net cost plus VAT               Credit by zero
        t2 = [ get_account_by_system(7), self.creditnote.cost + self.creditnote.vat,  D("0.00") ]
        if not t2[0]:
            vs_message(self, "The purchase account cannot be found", "ERROR")
            return

        #      VAT balance account       Debit by zero  Credit by the VAT amount
        t3 = [ get_account_by_system(5), D("0.00"),     self.creditnote.vat ]
        if not t3[0]:
            vs_message(self, "The VAT balance account cannot be found", "ERROR")
            return

        self.transaction_list = [t1, t2, t3]

        wx.Yield()
        # Confirm the transaction (function and dialog in gui_functions)
        status, self.transaction_text = confirm_transaction(self,
                                                            self.transaction_text,
                                                            self.transaction_list,
                                                            self.transaction_date)
        if not status:
            vs_message(self, "The credit note has not been saved", "CANCELLED")
            return

        # Must now implement the transaction and save the credit note
        busy_cursor_on()
        wx.Yield()
        save_suppliercreditnote.save(self.creditnote,
                                     self.transaction_date,
                                     self.transaction_text,
                                     self.transaction_list)
        big_refresh()
        self.EndModal(wx.ID_OK)


    #----------- Check and store all fields button

    def check_fields(self):
        """Checks the fields, and fills in the self.creditnote object
           returns (True, None) if ok, or (False, "warning message") if not"""

        # The Supplier invoice
        invoice_selected_list_item = self.invoice_field.GetCurrentSelection()
        self.creditnote.supplierinvoice = self.all_invoices[invoice_selected_list_item]

        self.creditnote.account = self.creditnote.supplierinvoice.account

        # Get credit note date
        self.creditnote.datestamp = wxdate_to_date(self.creditnote_date_field.GetValue())

        # Suppliers creditnote number
        self.creditnote.number = self.sup_creditnote_number_field.GetValue().strip()

        # Get cost field
        cost_string = self.cost_field.GetValue().strip()

        status, cost = string_to_decimal(cost_string)
        if not status:
            return False, "The net cost is not valid"

        self.creditnote.cost = cost

        # Get VAT field

        VAT_string = self.VAT_field.GetValue().strip()

        status, VAT = string_to_decimal(VAT_string)
        if not status:
            return False, "The VAT is not valid"

        self.creditnote.vat = VAT

        # Check these input creditnote details are ok via vatstuff_packages.vs_verify.purchases
        status, message = received_creditnote_against_invoice_check(self.creditnote)
        if not status:
            return False, message

        return True, None


######### Create a supplier creditnote, but with an incoming payment from the supplier
#         and not linked with a purchase order or a supplier invoice



def supplier_creditnote_with_payment_dialog(parent, supplier):
    """Creates a SupplierCreditnoteWithPaymentDialog"""
    supplier_creditnote_with_payment_dialog = SupplierCreditnoteWithPaymentDialog(parent, supplier)

    # If the cursor is busy, unbusy it - as
    # now waiting for a button press
    busy_cursor_off()

    supplier_creditnote_with_payment_dialog.ShowModal()
    supplier_creditnote_with_payment_dialog.Destroy()


class SupplierCreditnoteWithPaymentDialog(vsDialog):
    "The dialog which records a creditnote received from a supplier with a payment"

    def __init__(self, parent, supplier):
        """Create the Dialog"""
        vsDialog.__init__(self, parent, "Credit Note received with payment")
       
        # Take a note of the variables
        self.supplier = supplier

        # Store form contents in an SupplierCreditNoteForm object
        self.creditnote = form_objects.SupplierCreditNoteForm()
        self.creditnote.status = 1  # 1 indicates this creditnote comes with a payment

        button_size = (200, vs_cfg["BUTTON_HEIGHT"])
             
        panel_sizer=wx.BoxSizer(wx.VERTICAL)
        sizer=wx.GridBagSizer(hgap=20, vgap=20)

        text1 = """\
You have chosen to input a credit note received, together with a payment from a supplier"""
        sizer.Add(wx.StaticText(self, -1, text1), pos=(0,0), span=(1,2), flag=wx.ALIGN_CENTER)

        # List suppliers
        self.all_suppliers = Supplier.get_all()
        supplier_list = [ s.name for s in self.all_suppliers ]

        self.supplier_field = wx.Choice(self, -1, size=button_size, choices=supplier_list)
        self.supplier_field.SetToolTipString("Set Supplier")

        for key,s in enumerate(self.all_suppliers):
            if s == self.supplier:
                self.supplier_field.SetSelection(key)
        
        sizer.Add(self.supplier_field, pos=(1,0), flag=wx.ALIGN_RIGHT)

        supplier_field_text = "Choose the supplier"
        sizer.Add(wx.StaticText(self, -1, supplier_field_text), pos=(1,1), flag=wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        # The credit note date
        self.creditnote_date_field = wx.DatePickerCtrl(self, -1,
                                                    wx.DateTime.Now(), 
                                                    size=button_size,
                                                    style=wx.DP_DROPDOWN | wx.DP_SHOWCENTURY)
        self.creditnote_date_field.SetToolTipString("Date of the credit note and payment")
        sizer.Add(self.creditnote_date_field, pos=(2,0), flag=wx.ALIGN_RIGHT)

        creditnote_date_field_text = "The date of the credit note and payment"
        sizer.Add(wx.StaticText(self, -1, creditnote_date_field_text), pos=(2,1), flag=wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        # Credit note number
        self.sup_creditnote_number_field = wx.TextCtrl(self, -1, "", size=button_size)
        self.sup_creditnote_number_field.SetToolTipString("Suppliers credit note number (if any)")
        self.sup_creditnote_number_field.SetMaxLength(20)
        sizer.Add(self.sup_creditnote_number_field, pos=(3,0), flag=wx.ALIGN_RIGHT)

        number_field_text = "The suppliers credit note number"
        sizer.Add(wx.StaticText(self, -1, number_field_text), pos=(3,1), flag=wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        text2 = """\
A cash or bank account is needed to put the money in to. As the original purchase
increased a stock or expense account by the net value of the invoice payment, this
account must now be reduced, by the net cost of the credit note. So you will need to
supply both the cash account to be debited, and the stock/expense account to be credited."""
        sizer.Add(wx.StaticText(self, -1, text2), pos=(4,0), span=(1,2), flag=wx.ALIGN_CENTER)

        # Get a list of accounts - Need accounts with system numbers 0 and 1
        # get_accounts_by_system_list(system_list) - in odd_functions module         
        self.all_accounts = Account.get_non_system()

        # self.account_list is going to show the strings displayed in the accounts choices field
        self.account_list = [ a.fullname for a in self.all_accounts ]

        # cash account

        self.bank_account_field = wx.Choice(self, -1, size=button_size, choices=self.account_list)
        self.bank_account_field.SetToolTipString("Set a bank or cash account")
        sizer.Add(self.bank_account_field, pos=(5,0), flag=wx.ALIGN_RIGHT)

        bank_account_text="Cash or Bank account to be debited"
        sizer.Add(wx.StaticText(self, -1, bank_account_text), pos=(5,1), flag=wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        # Get the last cash account used - and pre-select it
        last_account = get_last_cash_account()
        if last_account:
            for key, acc in enumerate(self.all_accounts):
                if last_account == acc:
                    self.bank_account_field.SetSelection(key)
                    break

        # Stock or Expense account

        self.stock_account_field = wx.Choice(self, -1, size=button_size, choices=self.account_list)
        self.stock_account_field.SetToolTipString("Set a Stock or Expense account to be credited")

        # Get the last account used, and set the account selection the same as previously used:
        last_account = get_last_stock_account()
        if last_account:
            for key, acc in enumerate(self.all_accounts):
                if last_account == acc:
                    self.stock_account_field.SetSelection(key)
                    break
        else:
            # As default, set the account selection as the last in the choice to avoid the bank accounts
            self.stock_account_field.SetSelection(len(self.account_list)-1)

        sizer.Add(self.stock_account_field, pos=(6,0), flag=wx.ALIGN_RIGHT)

        stock_account_text = "Stock or Expense account to be credited"
        sizer.Add(wx.StaticText(self, -1, stock_account_text), pos=(6,1), flag=wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        text3 = "Input the net cost and VAT on the credit note"
        sizer.Add(wx.StaticText(self, -1, text3), pos=(7,0), span=(1,2), flag=wx.ALIGN_CENTER)

        # Cost field
        self.cost_field = wx.TextCtrl(self, -1, vs_cfg["ZERO_STRING"], style=wx.ALIGN_RIGHT, size=button_size)
        self.cost_field.SetToolTipString("Net cost specified on this credit note")
        self.cost_field.SetMaxLength(12)
        sizer.Add(self.cost_field, pos=(8,0), flag=wx.ALIGN_RIGHT)

        cost_field_text="Net Cost"
        sizer.Add(wx.StaticText(self, -1, cost_field_text), pos=(8,1), flag=wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        # VAT field
        self.VAT_field = wx.TextCtrl(self, -1, vs_cfg["ZERO_STRING"], style=wx.ALIGN_RIGHT, size=button_size)
        self.VAT_field.SetToolTipString("VAT specified on this credit note")
        self.VAT_field.SetMaxLength(12)
        sizer.Add(self.VAT_field, pos=(9,0), flag=wx.ALIGN_RIGHT)

        # The VAT field must be disabled if this supplier cannot charge you VAT
        if not self.supplier.pay_vat():
            self.VAT_field.Enable(False)

        VAT_field_text="VAT"
        sizer.Add(wx.StaticText(self, -1, VAT_field_text), pos=(9,1), flag=wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        panel_sizer.Add(sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)


        # And a horizontal sizer to hold buttons
        button_sizer = wx.BoxSizer(wx.HORIZONTAL)

        # Cancel button
        self.cancel_button = wx.Button(self, wx.ID_CANCEL, size=button_size)
        self.cancel_button.SetToolTipString("Click to cancel.")
        button_sizer.Add(self.cancel_button, 0, wx.RIGHT | wx.ALIGN_CENTER, 40)

        # Apply button
        self.apply_button = wx.Button(self, wx.ID_APPLY, size=button_size)
        self.apply_button.SetToolTipString("Click to apply the credit note.")
        button_sizer.Add(self.apply_button, 0, wx.LEFT | wx.ALIGN_CENTER, 40)

        panel_sizer.Add(button_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # Set the size to the panel
        self.SetSizer(panel_sizer)
        panel_sizer.SetSizeHints(self)

        self.Bind(wx.EVT_BUTTON, self.cancel_button_pressed, self.cancel_button)
        self.Bind(wx.EVT_BUTTON, self.apply_button_pressed, self.apply_button)
        self.Bind(wx.EVT_CHOICE, self.supplier_pressed, self.supplier_field)


    def supplier_pressed(self, event):
        """Supplier choice pressed - change vat field enabled or disabled
           depending on if this supplier can charge you VAT or not"""
        try:
            # Get supplier currently shown
            supplier_selected_list_item = self.supplier_field.GetSelection()
            self.creditnote.supplier = self.all_suppliers[supplier_selected_list_item]
            # The VAT field must be disabled if this supplier cannot charge you VAT
            if self.creditnote.supplier.pay_vat():
                self.VAT_field.Enable(True)
            else:
                # set value to zero
                self.VAT_field.SetValue(vs_cfg["ZERO_STRING"])
                self.VAT_field.Enable(False)
        except Exception, e:
            error_message("Failure in choosing supplier", e)


    @writecheck
    @button_wrap
    def apply_button_pressed(self, event):
        "Apply button - check and apply all fields"

        status, message = self.check_fields()
        if not status:
            vs_message(self, message, "ERROR")
            return

        # All items ok
        # set the stock/expense account into the configuration
        set_config_item("LAST_STOCK_ACCOUNT_ID", self.creditnote.account.id)

        # now create the transaction
        self.transaction_text = "Credit Note received from %s" % self.creditnote.supplier.name
        self.transaction_date = self.creditnote.datestamp

        VAT_account = get_account_by_system(5)
        if not VAT_account:
            vs_message(self, "The VAT balance account cannot be found", "ERROR")
            return

        # self.transaction_list is a list of lists [account, debit, credit]
        # account being an account object, debit and credit being decimal objects

        #      account                     debit                                         credit
        t1 = [ self.creditnote.account,   D("0.00"),                                    self.creditnote.cost ]
        t2 = [ self.bank_account,         self.creditnote.cost + self.creditnote.vat,   D("0.00")            ]
        t3 = [ VAT_account,               D("0.00"),                                    self.creditnote.vat  ]

        self.transaction_list = [t1, t2, t3]

        wx.Yield()
        # Confirm the transaction (function and dialog in gui_functions)
        status, self.transaction_text = confirm_transaction(self,
                                                            self.transaction_text,
                                                            self.transaction_list,
                                                            self.transaction_date)
        if not status:
            vs_message(self, "The credit note and payment have not been saved", "CANCELLED")
            return

        # Must now implement the transaction and save the credit note
        busy_cursor_on()
        wx.Yield()
        save_suppliercreditnote.save(self.creditnote,
                                    self.transaction_date,
                                    self.transaction_text,
                                    self.transaction_list)
        big_refresh()
        vs_message(self, "Credit Note and payment implemented", "DONE")
        self.EndModal(wx.ID_OK)


    def cancel_button_pressed(self, event):
        """Cancel button pressed - close without saving the credit note"""
        vs_message(self, "The creditnote has not been saved to the database", "Warning")
        self.EndModal(wx.ID_CANCEL)


    #----------- Check and store all fields button

    def check_fields(self):
        """Checks the fields, and fills in the self.creditnote object
           returns (True, None) if ok, or (False, "warning message") if not"""

        # Check supplier
        supplier_selected_list_item = self.supplier_field.GetSelection()
        self.creditnote.supplier = self.all_suppliers[supplier_selected_list_item]

        # Creditnote date
        self.creditnote.datestamp = wxdate_to_date(self.creditnote_date_field.GetValue())

        # The credit note number, used to create the transaction description
        self.creditnote.number = self.sup_creditnote_number_field.GetValue().strip()

        # The cash or bank account used to record this purchase
        account_selected_list_item = self.bank_account_field.GetSelection()
        if account_selected_list_item < 0:         # blank or unselected
            return False, """A cash or bank account is
needed to record the payment"""
        else:
            self.bank_account = self.all_accounts[account_selected_list_item]

        # The Stock or Expense account used to record this payment
        account_selected_list_item = self.stock_account_field.GetSelection()
        if account_selected_list_item < 0:         # blank or unselected
            return False, """A Stock or Expense account is
needed to record the payment"""
        else:
            self.creditnote.account = self.all_accounts[account_selected_list_item]

        # Get cost field
        cost_string = self.cost_field.GetValue().strip()

        status, cost = string_to_decimal(cost_string)
        if not status:
            return False, "The net cost is not valid"

        self.creditnote.cost = cost

        # Get VAT field
        VAT_string = self.VAT_field.GetValue().strip()

        status, VAT = string_to_decimal(VAT_string)
        if not status:
            return False, "The VAT is not valid"

        self.creditnote.vat = VAT

        # Check these input creditnote details are ok via vatstuff_packages.vs_verify.purchases
        status, message = received_creditnote_with_payment_check(self.creditnote)
        if not status:
            return False, message

        return True, None


