####### VATStuff #######
#
# supplier_invoice.py  - receive a supplier invoice
#
# Version : 0.7.0
# Date : 20100913
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#

"""Receive a supplier invoice

Consists of function:
supplier_invoice_dialog

and class:
SupplierInvoiceDialog


When the invoice button is pressed in purchases_tab.py a list of open orders
is presented.  Double clicking on an open order then calls the function
supplier_invoice_dialog(parent, order) with the order chosen
This function then opens the SupplierInvoiceDialog.
This takes the invoice details as presented by the
supplier and creates a transaction, and stores the invoice by calling:
vatstuff_packages.vs_database.save_supplierinvoice.save()
"""

import datetime
import decimal

import wx

from vatstuff_packages.vs_database.grid_objects import *
from vatstuff_packages.vs_database import save_supplierinvoice

from vatstuff_packages.vs_verify import form_objects
from vatstuff_packages.vs_verify.purchases import received_invoice_check

from vatstuff_packages.vs_utilities import *

from vatstuff_packages.vs_gui.purchases import purchaseorder_status
from vatstuff_packages.vs_gui.gui_functions import *

D = decimal.Decimal   # D used to avoid typing



def supplier_invoice_dialog(parent, order):
    """Creates a SupplierInvoiceDialog"""
    supplier_invoice_dialog = SupplierInvoiceDialog(parent, order)

    # If the cursor is busy, unbusy it - as
    # now waiting for a button press
    busy_cursor_off()

    supplier_invoice_dialog.ShowModal()
    supplier_invoice_dialog.Destroy()


class SupplierInvoiceDialog(vsDialog):
    "Called when a supplier invoice is to be created"

    def __init__(self, parent, order):
        """Create the Dialog"""
        frame_title = "Invoice received for Order : %i" % order.number
        vsDialog.__init__(self, parent, frame_title)
        
        # Take a note of the variables
        self.parent = parent
        self.order = order

        # Store all form contents in an SupplierInvoiceForm object
        self.invoice = form_objects.SupplierInvoiceForm()

        # order.supplier is the supplier who sent the order, but the supplier may have
        # had details changed since the order was sent, so get the visible supplier
        # with order.supplier.number
        self.invoice.supplier = Supplier.from_number(order.supplier.number)

        # The purchase order the invoice refers to
        self.invoice.purchaseorder = order
        self.invoice.claim_vat = order.claim_vat

        # The remaining amount on the order still un-invoiced        
        self.invoice.cost = order.cost - order.invoice_cost
        if self.invoice.cost < D("0.00"):
            self.invoice.cost = D("0.00")

        # can this supplier charge you VAT?
        self.charge_VAT = self.invoice.supplier.pay_vat()
        if self.charge_VAT:
            self.invoice.vat = order.vat - order.invoice_vat
            if self.invoice.vat < D("0.00"):
                self.invoice.vat = D("0.00")
        else:
            self.invoice.vat = D("0.00")

        button_size = (100, vs_cfg["BUTTON_HEIGHT"])

        # Create a vertical sizer to hold everything
        sizer=wx.BoxSizer(wx.VERTICAL)

        # Show the purchase order status panel
        purchaseorder_status_panel = purchaseorder_status.PurchaseOrderStatusPanel(self, order, show_claim_vat = False)
        sizer.Add(purchaseorder_status_panel, 0, wx.ALIGN_CENTER)

        help_text = """\
Choose an Asset (stock) account if this purchase is buying goods, or an
Expense account if an expense is being paid. Input the suppliers invoice
number with the tax point date and the amounts requested on the invoice."""
        sizer.Add(wx.StaticText(self, -1, help_text), 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # And a grid sizer to hold invoice details
        invoice_sizer = wx.FlexGridSizer(rows=5, cols=2, hgap=20, vgap=10)

        # Account
        account_text=wx.StaticText(self, -1, "Account:")
        invoice_sizer.Add(account_text, 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        # Set up the stock/expense account choice
        # Get a list of accounts
        # Need accounts with system numbers 0 and 1
        # get_accounts_by_system_list(system_list) - in odd_functions module
         
        self.all_accounts = Account.get_non_system()

        # self.account_list is going to show the strings displayed in the accounts choices field
        self.account_list = [ a.fullname for a in self.all_accounts ]

        if len(self.account_list) == 0:
            raise VS_ERROR, "No accounts are available"

        self.account_field = wx.Choice(self, -1, size=(200, vs_cfg["BUTTON_HEIGHT"]), choices=self.account_list)
        self.account_field.SetToolTipString("""\
Set a Stock account, if this purchase buys an asset
or an Expense account if paying for an expense.""")

        # Get the last account used, and set the account selection the same as previously used:
        last_account = get_last_stock_account()
        # As default, set the account selection as the last in the choice to avoid the bank accounts
        key = len(self.account_list)-1
        
        if last_account:
            try:
                key = self.all_accounts.index(last_account)
            except ValueError:
                pass

        self.account_field.SetSelection(key)

        invoice_sizer.Add(self.account_field, 0, wx.ALIGN_LEFT)

        # Suppliers invoice Number
        sup_invoice_number=wx.StaticText(self, -1, "Invoice Number:")
        invoice_sizer.Add(sup_invoice_number, 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.sup_invoice_number_field = wx.TextCtrl(self, -1, "", size=(200, vs_cfg["BUTTON_HEIGHT"]))
        self.sup_invoice_number_field.SetToolTipString("Set the suppliers invoice number")
        self.sup_invoice_number_field.SetMaxLength(20)
        invoice_sizer.Add(self.sup_invoice_number_field, 0, wx.ALIGN_LEFT)

        # Date
        invoice_sizer.Add(wx.StaticText(self, -1, "Date:"), 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.datestamp = wx.DateTime.Now()
        self.invoice_date_field = wx.DatePickerCtrl(self, -1,
                                                    self.datestamp, 
                                                    size=(150, vs_cfg["BUTTON_HEIGHT"]),
                                                    style=wx.DP_DROPDOWN | wx.DP_SHOWCENTURY)
        self.invoice_date_field.SetToolTipString("The date, to be used as the VAT date (tax point).")
        invoice_sizer.Add(self.invoice_date_field, 0, wx.ALIGN_LEFT)

        # Cost field
        invoice_sizer.Add(wx.StaticText(self, -1, "Net cost:"), 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        cost_text = unicode(Currency(self.invoice.cost))
        self.cost_field = wx.TextCtrl(self, -1, cost_text, style=wx.ALIGN_RIGHT, size=(150, vs_cfg["BUTTON_HEIGHT"]))
        self.cost_field.SetToolTipString("Net cost (cost before VAT) requested on this invoice")
        self.cost_field.SetMaxLength(16)
        invoice_sizer.Add(self.cost_field, 0, wx.ALIGN_LEFT)

        # VAT field
        invoice_sizer.Add(wx.StaticText(self, -1, "VAT:"), 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        VAT_text = unicode(Currency(self.invoice.vat))
        self.VAT_field = wx.TextCtrl(self, -1, VAT_text, style=wx.ALIGN_RIGHT, size=(150, vs_cfg["BUTTON_HEIGHT"]))
        self.VAT_field.SetToolTipString("VAT requested on this invoice")
        self.VAT_field.SetMaxLength(16)
        invoice_sizer.Add(self.VAT_field, 0, wx.ALIGN_LEFT)

        # The VAT field must be disabled if this supplier cannot charge you VAT
        if not self.charge_VAT:
            self.VAT_field.Enable(False)

        sizer.Add(invoice_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 10) 

        # And a horizontal sizer to hold buttons
        button_sizer = wx.BoxSizer(wx.HORIZONTAL)

        # Cancel button
        self.cancel_button = wx.Button(self, wx.ID_CANCEL, size=button_size)
        self.cancel_button.SetToolTipString("Click to cancel.")
        button_sizer.Add(self.cancel_button, 0, wx.RIGHT, 40)

        # Apply button
        self.apply_button = wx.Button(self, wx.ID_APPLY, size=button_size)
        self.apply_button.SetToolTipString("Click to apply the invoice.")
        button_sizer.Add(self.apply_button, 0, wx.LEFT, 40)

        sizer.Add(button_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        self.Bind(wx.EVT_BUTTON, self.cancel_button_pressed, self.cancel_button)
        self.Bind(wx.EVT_BUTTON, self.apply_button_pressed, self.apply_button)

        # Set the size to the panel
        self.SetSizer(sizer)
        sizer.SetSizeHints(self)


    def cancel_button_pressed(self, event):
        "Cancel button - Close parent"
        self.EndModal(wx.ID_CANCEL)


    @writecheck
    @button_wrap
    def apply_button_pressed(self, event):
        "Apply button - check and apply all fields"

        status, message = self.check_fields()
        if not status:
            vs_message(self, message, "ERROR")
            return

        # set the stock/expense account into the configuration
        set_config_item("LAST_STOCK_ACCOUNT_ID", self.invoice.account.id)

        # Store the transaction in a TransactionForm object
        transaction = form_objects.TransactionForm()
        # This is a supplier invoice transaction(system type 9)
        transaction.system = 9

        transaction.description = "Invoice received from %s" % self.invoice.supplier.name
        transaction.datestamp = self.invoice.datestamp

        # Note, the transaction datestamp and invoice datestamp is the same value leading
        # to duplication in the database, however this is left in - in case it is necessary
        # to create a date of issue and taxpoint date - which it is currently thought is
        # not required 

        # Credit the purchase account by the cost plus vat of the invoice
        transaction.add(get_account_by_system(7), credit = self.invoice.cost + self.invoice.vat)

        # Debit the Stock/Expense account by the cost
        if self.invoice.cost:
            transaction.add(self.invoice.account, debit = self.invoice.cost)

        # Debit the VAT Balance account by the VAT amount
        if self.invoice.vat:
            transaction.add(get_account_by_system(5), debit = self.invoice.vat)

        status, message = transaction.check()
        if not status:
            vs_message(self, message, "ERROR")
            return

        wx.Yield()
        # Confirm the transaction (function and dialog in gui_functions)
        status, new_description = confirm_transaction(self, transaction)

        if not status:
            vs_message(self, "The invoice not been saved", "CANCELLED")
            return

        transaction.description = new_description

        # Must now implement the transaction and save the supplier invoice
        busy_cursor_on()
        wx.Yield()
        save_supplierinvoice.save(self.invoice, transaction)

        big_refresh()
        vs_message(self, "Supplier invoice saved", "DONE")
        self.EndModal(wx.ID_OK)


    #----------- Check and store all fields button

    def check_fields(self):
        """Checks the fields, and fills in the self.invoice object
           returns (True, None) if ok, or (False, "warning message") if not"""

        # The Stock or Expense account
        account_selected_list_item = self.account_field.GetCurrentSelection()
        if account_selected_list_item < 0:         # blank or unselected
            return False, "A stock or expense account must be selected"

        # Set self.invoice attributes
        self.invoice.account = self.all_accounts[account_selected_list_item]
        self.invoice.number = self.sup_invoice_number_field.GetValue().strip()

        try:
            self.invoice.datestamp = wxdate_to_date(self.invoice_date_field.GetValue())
        except Exception:
            return False, "The date input is not valid"

        # Get cost field
        cost_string = self.cost_field.GetValue().strip()

        status, cost = string_to_decimal(cost_string)
        if not status:
            return False, "The net cost is not valid"
        self.invoice.cost = cost

        # Get VAT field
        VAT_string = self.VAT_field.GetValue().strip()

        status, VAT = string_to_decimal(VAT_string)
        if not status:
            return False, "The VAT is not valid"
        self.invoice.vat = VAT

        # Check these input invoice details are ok via vatstuff_packages.vs_verify.purchases
        status, message = received_invoice_check(self.invoice)
        if not status:
            return False, message

        return True, None






