####### VATStuff #######
#
# view_orders.py  - lists purchase orders
#
# Version : 0.7.0
# Date : 20100928
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#

""" This is a module in the vatstuff_packages.vs_gui package, and Lists the Purchase Orders

It is run as part of the vatstuff gui interface.
"""

import datetime
import decimal

import wx
import wx.grid

from vatstuff_packages.vs_database.grid_objects import *
from vatstuff_packages.vs_database.transactions import *

from vatstuff_packages.vs_database import save_closeorder
from vatstuff_packages.vs_database import create_objects

from vatstuff_packages.vs_gui.gui_functions import *
from vatstuff_packages.vs_gui.reports import make_report
from vatstuff_packages.vs_gui.purchases import purchaseorder_status

from vatstuff_packages.vs_utilities import *

from vatstuff_packages.vs_verify import form_objects


if vs_cfg["REPORTLABS_AVAILABLE"]:
    import vatstuff_packages.vs_pdf.page_setup
    import vatstuff_packages.vs_pdf.order_form
    import vatstuff_packages.vs_pdf.makepayment_form
    import vatstuff_packages.vs_pdf.pdf_functions

D = decimal.Decimal   # D used to avoid typing


def view_all_orders_frame(order=None):
    """Show the ViewAllOrdersFrame which lists all purchase orders,
       ensuring that the order given as argument is displayed.
       If None is given, display the last entry"""

    # No try:exception statement - allows the caller to catch exceptions

    # Check the frame is not already open
    for frm in vs_cfg["TRANSACTION_REFRESH"]:
        if isinstance(frm, ViewAllOrdersFrame):
            frm.Raise()
            break
    else:
        # top_frame() is the top frame, which will be the parent
        frm = ViewAllOrdersFrame(top_frame())
        frm.Show()

    if order:
        frm.display_row(order.gridrow)
    else:
        frm.display_last()


class ViewAllOrdersFrame(vsFrame):
    """Frame to list all purchase orders"""

    def __init__(self, parent):
        """Create the frame to show a list of orders"""
        vsFrame.__init__(self, parent, title="Purchase Orders")

        # Take a note of the parent
        self.parent = parent

        self.orders_grid = OrdersGrid()   # class defined in vatstuff_packages.vs_database.grid_objects
        # Create a database object of type wx.grid.PyGridTableBase
        # it opens database connections, and gets information for
        # the grid object

        try:
            self.orders_grid_rows = self.orders_grid.GetNumberRows()
            # Create the ViewOrdersPanel
            self.view_all_orders_panel=ViewAllOrdersPanel(self, self.orders_grid)
        except Exception, e:
            # close database connection, and re-raise the exception
            self.orders_grid.CloseConnection()
            raise

        # Get the panel size
        panel_size = self.view_all_orders_panel.GetMinSize()

        # Set minimum size equal to the panel
        self.SetMinSize(panel_size)
        self.SetClientSize(panel_size)

        # update the list of windows which need a refresh if
        # a transaction occurs, by adding self
        vs_cfg["TRANSACTION_REFRESH"].append(self)

        # bind the close event
        self.Bind(wx.EVT_CLOSE, self.onclose)


    def refresh(self):
        "Refreshes the grid"
        new_num_rows = self.orders_grid.GetNumberRows()
        if new_num_rows > self.orders_grid_rows:
            # More rows are in the database, insert new rows on the grid
            msg = wx.grid.GridTableMessage(self.orders_grid,           # The table
                  wx.grid.GRIDTABLE_NOTIFY_ROWS_APPENDED,              # has extra rows
                  new_num_rows - self.orders_grid_rows                 # the number of rows
                  ) 
            self.view_all_orders_panel.grid.ProcessTableMessage(msg)   # The grid associated with the table
            self.orders_grid_rows = new_num_rows                       # record the new number of rows
        elif new_num_rows < self.orders_grid_rows:
            # Less rows are in the database, delete rows on the grid
            msg = wx.grid.GridTableMessage(self.orders_grid,           # The table
                  wx.grid.GRIDTABLE_NOTIFY_ROWS_DELETED,               # has fewer rows
                  new_num_rows,                                        # delete from row
                  self.orders_grid_rows - new_num_rows                 # delete the number of rows
                  ) 
            self.view_all_orders_panel.grid.ProcessTableMessage(msg)   # The grid associated with the table
            self.orders_grid_rows = new_num_rows                       # record the new number of rows

        # grid associated with table can also be obtained with
        # self.orders_grid.GetView().ProcessTableMessage(msg)

        self.view_all_orders_panel.grid.ForceRefresh()


    def display_row(self, row):
        "Calls the display_row function in the view_orders_panel"
        self.view_all_orders_panel.display_row(row)

    def display_last(self):
        "Calls the display_last function in the view_orders_panel"
        self.view_all_orders_panel.display_last()


    def onclose(self, evt):
        "Shutdown the database transaction"
        self.orders_grid.CloseConnection()
        # Decrement the refresh list
        vs_cfg["TRANSACTION_REFRESH"].remove(self)
        self.Destroy()


# ----------------------View all orders panel

class ViewAllOrdersPanel(wx.Panel):
    """Show the grid of orders"""

    def __init__(self, parent, orders_grid):
        """Create the panel"""
        wx.Panel.__init__(self, parent)

        # Take a note of the parent, and the orders_grid
        self.parent = parent
        self.orders_grid = orders_grid
        button_size = (100, vs_cfg["BUTTON_HEIGHT"])
        long_button_size = (150, vs_cfg["BUTTON_HEIGHT"])

        # This is the order double-clicked on and shown, initially None
        self.order = None

        # Create the sizer for the panel
        sizer=wx.BoxSizer(wx.VERTICAL)

        self.description_text=wx.StaticText(self, -1, "Double click on an order to view it")
        sizer.Add(self.description_text, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # Create a grid
        self.grid = wx.grid.Grid(self, size=(900,200))

        self.grid.SetDefaultColSize(100, True)
        self.grid.SetDefaultRowSize(vs_cfg["BUTTON_HEIGHT"], False)
        self.grid.SetColMinimalAcceptableWidth(100)
        self.grid.EnableEditing(False)

        self.grid.SetDefaultCellAlignment(wx.ALIGN_LEFT, wx.ALIGN_CENTER)

        # Assign the orders_grid to the grid
        self.grid.SetTable(self.orders_grid, True)

        # Set the description column size
        self.grid.SetColSize(2, 350)

        # Set the row label size
        self.grid.SetRowLabelSize(100)

        sizer.Add(self.grid, 1, wx.ALL | wx.ALIGN_CENTER | wx.EXPAND, 20)

        # buttons
        button_sizer=wx.BoxSizer(wx.HORIZONTAL)

        # ------------------- Close button
        self.close_button = wx.Button(self, wx.ID_CLOSE, size=button_size)
        self.close_button.SetToolTipString("Click to close this window")
        button_sizer.Add(self.close_button, 0, wx.RIGHT, 20)

        # ------------------- Orders Report button
        self.orders_report_button = wx.Button(self, label="Orders Report", size=long_button_size)
        self.orders_report_button.SetToolTipString("Click to generate a list of orders between two dates")
        button_sizer.Add(self.orders_report_button, 0, wx.RIGHT, 20)

        # ------------------- Invoices Report button
        self.invoices_report_button = wx.Button(self, label="invoices Report", size=long_button_size)
        self.invoices_report_button.SetToolTipString("Click to generate a list of invoices between two dates")
        button_sizer.Add(self.invoices_report_button, 0, wx.RIGHT, 20)

        # ------------------- Creditnotes Report button
        self.creditnotes_report_button = wx.Button(self, label="Creditnotes Report", size=long_button_size)
        self.creditnotes_report_button.SetToolTipString("Click to generate a list of Creditnotes between two dates")
        button_sizer.Add(self.creditnotes_report_button, 0)

        sizer.Add(button_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)


        # Set the size to the panel
        self.SetSizer(sizer)
        sizer.SetSizeHints(self)

        # Bind left mouse double - click
        self.Bind(wx.grid.EVT_GRID_CELL_LEFT_DCLICK, self.show_order, self.grid)

        # Bind the close and report buttons
        self.Bind(wx.EVT_BUTTON, self.close_button_pressed, self.close_button)
        self.Bind(wx.EVT_BUTTON, self.orders_report_button_pressed, self.orders_report_button)
        self.Bind(wx.EVT_BUTTON, self.invoices_report_button_pressed, self.invoices_report_button)
        self.Bind(wx.EVT_BUTTON, self.creditnotes_report_button_pressed, self.creditnotes_report_button)


    def close_button_pressed(self, event):
        """Close parent"""
        self.parent.Close()


    @button_wrap
    def orders_report_button_pressed(self, event):
        """Get the two dates between which the report is generated"""

        help_text = """\
A report will be generated listing
purchase orders between the two dates:"""

        status, from_date, to_date = two_dates(self, "Purchase Orders", help_text)
        if not status:
            return
        busy_cursor_on()
        wx.Yield()

        filename = "purchaseorders%s" % to_date.isoformat()
        make_report.make_report_and_frame('purchaseorders',
                                          "Purchase Orders",
                                          filename,
                                          from_date,
                                          to_date)


    @button_wrap
    def invoices_report_button_pressed(self, event):
        """Get the two dates between which the report is generated and show supplierinvoices"""

        help_text = """\
A report will be generated showing invoices
received from suppliers between the two dates:"""

        status, from_date, to_date = two_dates(self, "Supplier Invoices Report", help_text)
        if not status:
            return
        busy_cursor_on()
        wx.Yield()

        filename = "supplier_invoices%s" % to_date.isoformat()
        make_report.make_report_and_frame("supplierinvoices",
                                          "Invoices received from suppliers",
                                          filename,
                                          from_date,
                                          to_date)


    @button_wrap
    def creditnotes_report_button_pressed(self, event):
        """Get the two dates between which the report is generated and show suppliercredit notes"""

        help_text = """\
A report will be generated showing credit notes
received from suppliers between the two dates:"""

        status, from_date, to_date = two_dates(self, "Supplier Credit Notes Report", help_text)
        if not status:
            return
        busy_cursor_on()
        wx.Yield()

        filename = "supplier_creditnotes%s" % to_date.isoformat()
        make_report.make_report_and_frame("suppliercreditnotes",
                                          "Creditnotes received from suppliers",
                                          filename,
                                          from_date,
                                          to_date)


    def display_row(self, row):
        """Makes row visible"""
        last_row = self.orders_grid.GetNumberRows() - 1
        if row > last_row:
            row = last_row
        if row < 0:
            row = 0
        self.grid.SetGridCursor(row, 0)
        self.grid.MakeCellVisible(row, 0)


    def display_last(self):
        """ Makes the last row visible """
        last_row = self.orders_grid.GetNumberRows() - 1
        self.grid.SetGridCursor(last_row, 0)
        self.grid.MakeCellVisible(last_row, 0)


    @button_wrap
    def show_order(self, event):
        "Display the order in response to a double click"
        row = event.GetRow()
        # get order from its gridrow number
        order = PurchaseOrder.from_gridrow(row)
        show_order_frame(order)


# ------------------------ Show the order frame


def show_order_frame(order):
    "Display the ShowOrderFrame"

    debug_log("showing order id: %s" % order.id)

    # if a ShowOrder frame with this order is already open -show it
    for frm in vs_cfg["TRANSACTION_REFRESH"]:
        if isinstance(frm, ShowOrderFrame):
            if frm.order == order:
                frm.Raise()
                return

    # the frame is not open, so create one showing this order
    show_order_frame= ShowOrderFrame(top_frame(), order)
    show_order_frame.Show()


class ShowOrderFrame(vsFrame):
    "Called when an order is to be viewed"

    def __init__(self, parent, order):
        "Create the frame - showing a purchase order"
        frame_title = "Purchase Order : %i" % order.number
        vsFrame.__init__(self, parent, title=frame_title)

        # used to test which order frame is open
        self.order = order

        # create the panel
        self.show_order_panel = ShowOrderPanel(self, self.order)

        # Get the panel size
        panel_size = self.show_order_panel.GetMinSize()

        # Set minimum size equal to the panel
        self.SetMinSize(panel_size)

        # set the frame size to the panel size
        self.SetClientSize(panel_size)

        # update the list of windows which need a refresh if
        # a transaction occurs, by adding self
        vs_cfg["TRANSACTION_REFRESH"].append(self)

        # bind the close event
        self.Bind(wx.EVT_CLOSE, self.onclose)


    def refresh(self):
        "refresh this frame, by calling the refresh function in its panel"
        self.show_order_panel.refresh()


    def onclose(self, evt):
        # Decrement the refresh list
        vs_cfg["TRANSACTION_REFRESH"].remove(self)
        self.Destroy()


# ----------------------ShowOrderPanel

class ShowOrderPanel(wx.Panel):
    """The panel of the ShowOrder frame"""

    def __init__(self, parent, order):
        """Create the panel"""
        wx.Panel.__init__(self, parent)
        
        # Take a note of the variables
        self.parent = parent
        self.order = order

        button_size = (150, vs_cfg["BUTTON_HEIGHT"])

        # Create a vertical sizer to hold everything
        self.sizer=wx.BoxSizer(wx.VERTICAL)

        # Show the purchase order status panel
        self.purchaseorder_status_panel = purchaseorder_status.PurchaseOrderStatusPanel(self, order)
        self.sizer.Add(self.purchaseorder_status_panel, 0, wx.ALIGN_CENTER)

        if self.order.sup_ref_field:
            sup_ref_text = "Suppliers Reference: %s" % self.order.sup_ref_field
            self.sizer.Add(wx.StaticText(self, -1, sup_ref_text), 0, wx.LEFT| wx.ALIGN_LEFT, 20)


        #################   order status text

        # STATUS
        # 0 if this Purchase Order is still open, no invoices received
        # 1 order open, and one or more invoices have been received
        # 2 if normally closed and order fully paid, i.e total == paid
        # 3 if closed, with order total less than that paid, i.e. total<paid
        # 4 if closed, with order total greater than that paid, i.e. total>paid
        # 5 if closed by the order being cancelled with no invoice recieved or payment made
    
        order_status = self.order.status

        if not order_status:
            status_text = "This order is open, no invoices have been received"
        elif order_status == 1:
            status_text = "This order is open, with invoices received"
        elif order_status == 2:
            status_text = "This order is Closed"
        elif order_status == 3:
            closed_date = self.order.closed_date.strftime("%d %b %Y")
            status_text = "Closed, with payment greater than on the order: %s" % closed_date
        elif order_status == 4:
            closed_date = self.order.closed_date.strftime("%d %b %Y")
            status_text = "Closed, with payment less than on the order: %s" % closed_date
        elif order_status == 5:
            closed_date = self.order.closed_date.strftime("%d %b %Y")
            status_text = "This order was cancelled: %s" % closed_date
        else:
            raise VS_ERROR, "The order status is not recognised"

        self.status_static_text = wx.StaticText(self, -1, status_text)
        self.sizer.Add(self.status_static_text, 0, wx.LEFT| wx.ALIGN_LEFT, 20)

        # sizer for the project button and closed order reason text
        project_sizer = wx.BoxSizer(wx.HORIZONTAL)

        if order_status > 2:
            reason_text = self.order.closed_reason
        else:
            reason_text = ""

        self.reason_static_text = wx.StaticText(self, -1, "Reason:")
        project_sizer.Add(self.reason_static_text, 0, wx.RIGHT, 10)

        self.reason_field = wx.TextCtrl(self, -1, reason_text, size=(300, vs_cfg["BUTTON_HEIGHT"]))
        self.reason_field.SetToolTipString("Reason for order closure")
        self.reason_field.SetMaxLength(80)
        project_sizer.Add(self.reason_field, 0, wx.RIGHT, 10)

        self.reason_button = wx.Button(self, label="Change", size=button_size)
        self.reason_button.SetToolTipString("Click to change the closure reason")
        project_sizer.Add(self.reason_button, 0)

        if order_status < 3:
            # The reason text and buttons should be hidden
            self.reason_static_text.Hide()
            self.reason_field.Hide()
            self.reason_button.Hide()

        # add a space between this and the project information
        project_sizer.Add((20,40), 1, wx.EXPAND)

        if self.order.project:
            project_ref = "Project Ref:\n%s" % self.order.project.number
        else:
            project_ref = " \n "
        self.project_text = wx.StaticText(self, -1, project_ref)
        project_sizer.Add(self.project_text, 0, wx.RIGHT, 10)

        self.project_button = wx.Button(self, label="Project", size=button_size)
        self.project_button.SetToolTipString("Click to assign this purchase to a project")
        project_sizer.Add(self.project_button, 0)

        self.sizer.Add(project_sizer, 0, wx.LEFT | wx.RIGHT | wx.EXPAND, 20)

        # Create a Tabbed Notebook
        self.order_notebook = OrderNotebook(self, self.order)
        self.sizer.Add(self.order_notebook, 1, wx.ALL | wx.ALIGN_CENTER | wx.EXPAND, 10)

        # Create results sizer for the printer button, and the VAT, Cost, Total results
        res_sizer=wx.BoxSizer(wx.HORIZONTAL)

        # close button
        self.close_button = wx.Button(self, wx.ID_CLOSE, size=button_size)
        self.close_button.SetToolTipString("Click to close this window.")
        res_sizer.Add(self.close_button, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # page setup button
        self.page_button = wx.Button(self, label="Page setup", size=button_size)
        self.page_button.SetToolTipString("Click to set up the pdf page size and orientation")
        res_sizer.Add(self.page_button, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # pdf button
        self.pdf_button = wx.Button(self, label="Order form pdf", size=button_size)
        self.pdf_button.SetToolTipString("Click to create a pdf file of the order form.")
        res_sizer.Add(self.pdf_button, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # close_order button
        self.close_order_button = wx.Button(self, label="Cancel Order", size=button_size)
        self.close_order_button.SetToolTipString("Click to cancel this Purchase")
        res_sizer.Add(self.close_order_button, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        if self.order.open_and_invoiced:
            # The order can be closed, but not cancelled, change the button label
            self.close_order_button.SetLabel("Close Order")
            self.close_order_button.SetToolTipString("Click to close this Purchase")

        # The order cannot be closed if there is an amount owing on received invoices
        invoiced_outstanding = self.order.invoice_vat + self.order.invoice_cost - self.order.paid
        if invoiced_outstanding != D(0):
            self.close_order_button.Enable(False)

        # Disable the closed button if the order is already closed or cancelled
        if (not self.order.is_open) or  self.order.is_cancelled:
            self.close_order_button.Enable(False)

        # add the res_sizer to the main vertical sizer
        self.sizer.Add(res_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # Set the size to the panel
        self.SetSizer(self.sizer)

        panel_size = self.sizer.GetMinSize()
        self.SetMinSize(panel_size)

        self.Bind(wx.EVT_BUTTON, self.close_button_pressed, self.close_button)
        self.Bind(wx.EVT_BUTTON, self.page_button_pressed, self.page_button)
        self.Bind(wx.EVT_BUTTON, self.pdf_button_pressed, self.pdf_button)
        self.Bind(wx.EVT_BUTTON, self.close_order_button_pressed, self.close_order_button)
        self.Bind(wx.EVT_BUTTON, self.project_button_pressed, self.project_button)
        self.Bind(wx.EVT_BUTTON, self.reason_button_pressed, self.reason_button)


    @writecheck
    @button_wrap
    def reason_button_pressed(self, event):
        "Change the reason for the order being closed"
        reason = self.reason_field.GetValue().strip()
        if self.order.closed_reason == reason:
            # No change in the reason - just return
            vs_message(self, "There was no change to the closing reason", "WARNING")
            return
        self.order.closed_reason = reason
        info_log("Order id : %i closed reason changed" % self.order.id)
        vs_message(self, "Purchase order closing reason is changed", "DONE")


    @button_wrap
    def project_button_pressed(self, event):
        "List all projects"
        pick_project_dialog(self, self.order)
        # As this order may have changed project
        if self.order.project:
            self.project_text.SetLabel("Project Ref:\n%s" % self.order.project.number)
        else:
            self.project_text.SetLabel(" \n ")
        self.sizer.Layout()


    def close_button_pressed(self, event):
        """close button pressed, close window"""
        self.parent.Close()


    @button_wrap
    def page_button_pressed(self, event):
        "page button pressed, setup the pdf page size and orientation"

        if not vs_cfg["REPORTLABS_AVAILABLE"]:
            # no pdf capability
            vs_message(self, "Function unavailable - the reportlab module is not installed",
                             "ERROR")
            return

        page_setup_dialog = vatstuff_packages.vs_pdf.page_setup.PageSetupDialog(self, self.order.pdf_width, self.order.pdf_height)
        if page_setup_dialog.ShowModal() == wx.ID_OK:
            self.order.pdf_width = page_setup_dialog.width
            self.order.pdf_height = page_setup_dialog.height           
        page_setup_dialog.Destroy()


    @button_wrap
    def pdf_button_pressed(self, event):
        "pdf button create a pdf form"

        if not vs_cfg["REPORTLABS_AVAILABLE"]:
            # no pdf capability
            vs_message(self, "Function unavailable - the reportlab module is not installed",
                             "ERROR")
            return

        filename= "order_%s.pdf" % self.order.number

        # Create the save file dialog.
        status, filepath = save_file_as(self, 'pdf', vs_cfg["ORDERS_PDF_FOLDER"], filename)

        if status:
            busy_cursor_on()
            wx.Yield()
            if vatstuff_packages.vs_pdf.order_form.create_pdf(filepath, self.order):
                busy_cursor_off()
                # And show the file in the default viewer
                if not vatstuff_packages.vs_pdf.pdf_functions.show_pdf_file(filepath):
                    vs_message(self, "pdf file created", "Done")
            else:
                vs_message(self, "Unable to create the pdf file", "ERROR")


    @writecheck
    @button_wrap
    def close_order_button_pressed(self, event):
        """Close or cancel the order"""
        if self.order.is_cancelled: return
        if not self.order.is_open: return

        # initially assume this is a normal closure, ie total == paid
        self.order_status = 2

        if self.order.open_and_invoiced:
            # Close the order
            if self.order.total < self.order.paid:
                self.order_status = 3
            if self.order.total > self.order.paid:
                self.order_status = 4
            self.close_order()
        else:
            # Cancel the order
            self.order_status = 5
            self.cancel_order()


    @logmethod   # note: only log, not a frame_apply
    def close_order(self):
        "Close the order"

        # No try:except as the call is already covered
        # in a try-accept clause

        if self.order_status == 2:
            # Normal close - no transaction involved
            save_closeorder.save(self.order, self.order_status)
            # refresh windows in vs_cfg["TRANSACTION_REFRESH"], but not the
            # account tree, as no changes have been done there
            big_refresh(acc_tree=False)
            vs_message(self, "Order Closed", "DONE")
            return
        
        # The order total and amount paid do not match, so return the
        # sums assigned when the order was created

        # Get the closed reason
        confirm_close=wx.TextEntryDialog(parent=self,
                                       message="""
Are you sure? The amount on the order and the amount
paid do not match. If you wish, record a memo giving
the reason for this discrepency here:""",
                                       caption="Close Order",
                                       defaultValue="",
                                       style=wx.OK | wx.CANCEL | wx.TE_LEFT)
        if confirm_close.ShowModal() != wx.ID_OK:
            confirm_close.Destroy()
            return
        self.closed_reason=confirm_close.GetValue()
        confirm_close.Destroy()

        busy_cursor_on()
        wx.Yield()

        # Store the transaction in a TransactionForm object
        transaction = form_objects.TransactionForm()
        # This is a close order transaction(system type 8)
        transaction.system = 8

        transaction.description = "CLOSED ORDER NUMBER : %s" % self.order.number
        transaction.datestamp = datetime.date.today()

        # the date is set as the later of any invoices, payments or creditnotes
        invoices = self.order.get_invoices()
        if invoices:
            last_invoice_date = invoices[-1].datestamp
            if last_invoice_date > transaction.datestamp:
                transaction.datestamp = last_invoice_date
        payments = self.order.get_payments()
        if payments:
            last_payment_date = payments[-1].datestamp
            if last_payment_date > transaction.datestamp:
                transaction.datestamp = last_payment_date
        creditnotes = self.order.get_creditnotes()
        if creditnotes:
            last_creditnote_date = creditnotes[-1].datestamp
            if last_creditnote_date > transaction.datestamp:
                transaction.datestamp = last_creditnote_date

        if self.order_status == 3:
            # The order over-ran, and more was paid than specified on the purchase order
            amount = self.order.paid - self.order.total

            # Debit the purchase account
            transaction.add(get_account_by_system(7), debit = amount)
            # Credit the suppliers account
            transaction.add(self.order.supplier.account, credit = amount)

        if self.order_status == 4:
            # The order is under paid, presumably due to goods being unavailable
            amount = self.order.total - self.order.paid

            # Credit the purchase account
            transaction.add(get_account_by_system(7), credit = amount)
            # Debit the suppliers account
            transaction.add(self.order.supplier.account, debit = amount)

        status, message = transaction.check()
        if not status:
            vs_message(self, message, "ERROR")
            return

        wx.Yield()
        # Confirm the transaction (function and dialog in gui_functions)
        status, new_description = confirm_transaction(self, transaction)

        if not status:
            vs_message(self, "The order has not been closed", "Transaction not implemented")
            return

        transaction.description = new_description

        # Must now implement the transaction and close the order
        busy_cursor_on()
        wx.Yield()
        save_closeorder.save(self.order,
                                 self.order_status,
                                 self.closed_reason,
                                 transaction)

        # refresh account tree, and windows in vs_cfg["TRANSACTION_REFRESH"]
        big_refresh()
        vs_message(self, "Order Closed", "DONE")


    @logmethod   # note: only log, not a frame_apply
    def cancel_order(self):
        "Cancel the order"

        # No try:except as the call is already covered

        if self.order.paid != D(0): return

        # Get the cancellation reason
        confirm_cancel=wx.TextEntryDialog(parent=self,
                                       message="""
Are you sure? This action cancels the order, reversing the
transactions made when the order was created. Give the
reason for the cancellation here:""",
                                       caption="Cancel Order",
                                       defaultValue="",
                                       style=wx.OK | wx.CANCEL | wx.TE_LEFT)
        result=confirm_cancel.ShowModal()
        self.closed_reason=confirm_cancel.GetValue()
        confirm_cancel.Destroy()
        if result != wx.ID_OK:
            return

        busy_cursor_on()
        wx.Yield()

        # Store the transaction in a TransactionForm object
        transaction = form_objects.TransactionForm()
        # This is a close order transaction(system type 8)
        transaction.system = 8

        transaction.description = "CANCELLED ORDER NUMBER : %s" % self.order.number
        transaction.datestamp = datetime.date.today()

        # Credit the purchase account by the cost plus vat of the order
        transaction.add(get_account_by_system(7), credit = self.order.cost + self.order.vat)  

        # Debit the suppliers account by the cost plus VAT
        transaction.add(self.order.supplier.account, debit = self.order.cost + self.order.vat)

        status, message = transaction.check()
        if not status:
            vs_message(self, message, "ERROR")
            return

        wx.Yield()
        # Confirm the transaction (function and dialog in gui_functions)
        status, new_description = confirm_transaction(self, transaction)

        if not status:
            vs_message(self, "The order has not been cancelled", "Transaction not implemented")
            return

        transaction.description = new_description

        # Must now implement the transaction and close the order
        busy_cursor_on()
        wx.Yield()
        save_closeorder.save(self.order,
                             self.order_status,
                             self.closed_reason,
                             transaction)

        # refresh account tree, and windows in vs_cfg["TRANSACTION_REFRESH"]
        big_refresh()
        vs_message(self, "Order Cancelled", "DONE")


    def refresh(self):
        "refreshes the order status and the tabbed notebook"

        # refresh purchaseorder_status panel as order date may have changed
        self.purchaseorder_status_panel.refresh()

        # status_static_text
        order_status = self.order.status

        if not order_status:
            status_text = "This order is open, no invoices have been received"
        elif order_status == 1:
            status_text = "This order is open, with invoices received"
        elif order_status == 2:
            status_text = "This order is Closed"
        elif order_status == 3:
            closed_date = self.order.closed_date.strftime("%d %b %Y")
            status_text = "Closed, with payment greater than on the order: %s" % closed_date
        elif order_status == 4:
            closed_date = self.order.closed_date.strftime("%d %b %Y")
            status_text = "Closed, with payment less than on the order: %s" % closed_date
        elif order_status == 5:
            closed_date = self.order.closed_date.strftime("%d %b %Y")
            status_text = "This order was cancelled: %s" % closed_date
        else:
            raise VS_ERROR, "The order status is not recognised"

        self.status_static_text.SetLabel(status_text)

        if order_status < 3:
            # The reason text and buttons should be hidden
            self.reason_static_text.Hide()
            self.reason_field.Hide()
            self.reason_button.Hide()
        else:
            # Show the reason
            self.reason_field.SetValue(self.order.closed_reason)
            self.reason_static_text.Show()
            self.reason_field.Show()
            self.reason_button.Show()

        # Refresh the tabbed notebook
        self.order_notebook.refresh()

        if self.order.open_and_invoiced:
            # The order can be closed, but not cancelled, change the button label
            self.close_order_button.SetLabel("Close Order")
            self.close_order_button.SetToolTipString("Click to close this Purchase")
            self.close_order_button.Enable(True)

        # The order cannot be closed if there is an amount owing on received invoices
        invoiced_outstanding = self.order.invoice_vat + self.order.invoice_cost - self.order.paid
        if invoiced_outstanding != D(0):
            self.close_order_button.Enable(False)

        # Disable the closed button if the order is already closed or cancelled
        if (not self.order.is_open) or  self.order.is_cancelled:
            self.close_order_button.Enable(False)

        # Re-do the layout
        self.sizer.Layout()


class OrderNotebook(wx.Notebook):
    """Shows order items, and payments"""

    def __init__(self, parent, order):
        """Create the tabbed notebook, and each panel beneath each tab"""
        wx.Notebook.__init__(self, parent, size = (900,200))

        # Note the notebook is set to a specific size, since the grid size is
        # variable, this forces the grid to have scroll bars if bigger

        self.parent = parent
        self.order = order
       
        # show the items being purchased
        self.show_purchaseitems = ShowPurchaseItemsPanel(self, self.order)
        # Add show_purchaseitems to the Notebook
        self.AddPage(self.show_purchaseitems, " Ordered Items ")

        # Show supplier invoices and credit notes
        self.show_invoices = ShowInvoicesPanel(self, self.order)
        # Add show_invoices to the Notebook
        self.AddPage(self.show_invoices, " Supplier Invoices/Credit notes ")

        # show the payments paid
        self.show_payments = ShowPaymentsPanel(self, self.order)
        # Add show_payments to the Notebook
        self.AddPage(self.show_payments, " Payments made ")

        # show the order transactions
        self.show_transactions = ShowTransactionsPanel(self, self.order)
        # Add show_transactions to the Notebook
        self.AddPage(self.show_transactions, " Transactions ")

    def refresh(self):
        self.show_invoices.refresh()
        self.show_payments.refresh()
        self.show_transactions.refresh()


class ShowPurchaseItemsPanel(wx.Panel):
    """The panel of the ShowPurchaseItemsTable grid"""

    def __init__(self, parent, order):
        """Create the panel"""
        wx.Panel.__init__(self, parent)

        self.parent = parent

        # Create a vertical sizer to hold the grid
        sizer=wx.BoxSizer(wx.VERTICAL)

        # show the itms being purchased
        show_purchaseitems_table = ShowPurchaseItemsTable(self, order)

        sizer.Add(show_purchaseitems_table, 1, wx.ALIGN_CENTER | wx.EXPAND)
        # Set the size to the panel
        self.SetSizer(sizer)


class ShowPurchaseItemsTable(wx.grid.Grid):

    def __init__(self, parent, order):
        wx.grid.Grid.__init__(self, parent, style=wx.WANTS_CHARS)

        self.parent = parent
        self.order = order

        discount = "%s%%" % str(self.order.discount).replace(u".", vs_cfg["CP"])
        no_discount = "0%s0%%" % vs_cfg["CP"]

        num_cols = 6

        amount_list = self.order.get_amount()
        item_list = self.order.item_list
        num_rows = len(item_list)

        self.CreateGrid(num_rows, num_cols)
        self.EnableEditing(False)

        self.SetDefaultRowSize(vs_cfg["BUTTON_HEIGHT"], False)
        self.SetDefaultCellAlignment(wx.ALIGN_LEFT, wx.ALIGN_CENTER)

        # create colum labels
        self.SetColLabelValue(0, "Qty")
        self.SetColLabelValue(1, "Description")
        self.SetColLabelValue(2, "Unit Cost")
        self.SetColLabelValue(3, "Amount")
        self.SetColLabelValue(4, "Discount")
        self.SetColLabelValue(5, "VAT")

        # Set the Quantity, Discount and VAT columns as centre aligned
        attr_center = wx.grid.GridCellAttr()
        attr_center.SetAlignment(wx.ALIGN_CENTER, wx.ALIGN_CENTER)

        self.SetColAttr(0, attr_center)
        self.SetColAttr(4, attr_center)
        self.SetColAttr(5, attr_center)

        # Set the colums showing money as right aligned
        attr_right = wx.grid.GridCellAttr()
        attr_right.SetAlignment(wx.ALIGN_RIGHT, wx.ALIGN_CENTER)
        self.SetColAttr(2, attr_right)
        self.SetColAttr(3, attr_right)

        self.SetColSize(0, 100)
        self.SetColSize(1, 300)
        self.SetColSize(2, 100)
        self.SetColSize(3, 100)
        self.SetColSize(4, 100)
        self.SetColSize(5, 100)


        self.SetColMinimalAcceptableWidth(100)
        self.SetRowLabelSize(50)

        for row, item in enumerate(item_list):
            # Quantity
            self.SetCellValue(row, 0, str(item["Qty"]).replace(u".", vs_cfg["CP"]))
            # Description
            self.SetCellValue(row, 1, item["Description"])
            # Unit Cost
            self.SetCellValue(row, 2, unicode(Currency(item["Unit Cost"])))
            # Amount = Qty * Unit Cost
            self.SetCellValue(row, 3, unicode(Currency(amount_list[row])))
            # Discount
            if item["Discount"]:
                self.SetCellValue(row, 4, discount)
            else:
                self.SetCellValue(row, 4, no_discount)
            # VAT rate
            self.SetCellValue(row, 5, str(item["VAT"]).replace(u".", vs_cfg["CP"])+u"%")


###### Show supplier invoices and creditnotes on the notebook

class ShowInvoicesPanel(wx.Panel):
    """Show supplier invoices and creditnotes on the notebook"""

    def __init__(self, parent, order):
        """Create the panel"""
        wx.Panel.__init__(self, parent)

        self.parent = parent

        # Create a vertical sizer to hold the grid
        sizer=wx.BoxSizer(wx.VERTICAL)

        note_text = wx.StaticText(self, -1, """\
Double click on an item to display the invoice/creditnote details.""")
        sizer.Add(note_text, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # show the invoices
        self.show_invoices_table = ShowInvoicesTable(self, order)

        sizer.Add(self.show_invoices_table, 1, wx.ALIGN_CENTER | wx.EXPAND)
        # Set the size to the panel
        self.SetSizer(sizer)

    def refresh(self):
        self.show_invoices_table.refresh()


class ShowInvoicesTable(wx.grid.Grid):

    def __init__(self, parent, order):
        wx.grid.Grid.__init__(self, parent, style=wx.WANTS_CHARS)

        self.parent = parent
        self.order = order

        num_cols = 4

        # Get a list of all invoices on the order
        self.invoices = self.order.get_invoices()
        # Get a list of all creditnotes on the order
        notes = self.order.get_creditnotes()
        # Add creditnotes to the now mis-named self.invoices list
        self.invoices.extend(notes)

        self.num_rows = len(self.invoices)

        # ORDER BY DATE
        if self.num_rows > 1:
            self.invoices.sort(key=lambda item: item.datestamp)

        self.CreateGrid(self.num_rows, num_cols)
        self.EnableEditing(False)

        self.SetDefaultRowSize(vs_cfg["BUTTON_HEIGHT"], False)
        self.SetDefaultCellAlignment(wx.ALIGN_LEFT, wx.ALIGN_CENTER)

        # create colum labels
        self.SetColLabelValue(0, "Date")
        self.SetColLabelValue(1, "Supplier Invoice/Credit Note")
        self.SetColLabelValue(2, "Net Cost")
        self.SetColLabelValue(3, "VAT")


        # Set the colums showing money as right aligned
        attr_right = wx.grid.GridCellAttr()
        attr_right.SetAlignment(wx.ALIGN_RIGHT, wx.ALIGN_CENTER)
        self.SetColAttr(2, attr_right)
        self.SetColAttr(3, attr_right)

        self.SetColSize(0, 150)
        self.SetColSize(1, 300)
        self.SetColSize(2, 150)
        self.SetColSize(3, 150)

        self.SetColMinimalAcceptableWidth(100)
        self.SetRowLabelSize(100)

        # Fill in the table values

        for row, item in enumerate(self.invoices):
            # Set row label as the transaction id
            self.SetRowLabelValue(row, str(item.transaction.id))
            # Date
            self.SetCellValue(row, 0, item.datestamp.strftime("%d %b %Y"))

            if isinstance(item, SupplierInvoice):
                if not item.number:
                    self.SetCellValue(row, 1, "Invoice")
                else:
                    self.SetCellValue(row, 1, "Invoice : %s" % str(item.number))
                self.SetCellValue(row, 2, to_currency_string(item.cost))
                self.SetCellValue(row, 3, to_currency_string(item.vat))
            else:
                if not item.number:
                    self.SetCellValue(row, 1, "Credit Note")
                else:
                    self.SetCellValue(row, 1, "Credit Note : %s" % str(item.number))

                # Show creditnote cost and vat as negative values (unless they are equal to zero)
                if item.cost == D("0"):
                    self.SetCellValue(row, 2, vs_cfg["SYMBOL_ZERO_STRING"])
                else:
                    self.SetCellValue(row, 2, to_currency_string(D(-1) * item.cost))
                if item.vat == D("0"):
                    self.SetCellValue(row, 3, vs_cfg["SYMBOL_ZERO_STRING"])
                else:
                    self.SetCellValue(row, 3, to_currency_string(D(-1) * item.vat))

        # Bind left mouse double - click
        self.Bind(wx.grid.EVT_GRID_CELL_LEFT_DCLICK, self.show_invoice_creditnote)


    @button_wrap
    def show_invoice_creditnote(self, event):
        """On being double clicked, obtain the supplier invoice or supplier creditnote
           and show its details in a Dialog"""
        row = event.GetRow()
        supplier_object = self.invoices[row]
        if isinstance(supplier_object, SupplierInvoice):
            # show the ShowSupplierInvoiceDialog dialog
            # (defined below in this module)
            show_supplier_invoice_dialog(self, supplier_object)
        elif isinstance(supplier_object, SupplierCreditNote):
            # show the ShowSupplierCreditNoteDialog dialog
            # (defined below in this module)
            show_supplier_creditnote_dialog(self, supplier_object)
        else:
            raise VS_ERROR, "Object associated with the transaction is not recognised"


    def refresh(self):
        "Refresh table, or add further rows to the table if further transactions have been implemented"

        # Get a list of all invoices on the order
        self.invoices = self.order.get_invoices()
        # Get a list of all creditnotes on the order
        notes = self.order.get_creditnotes()
        # Add creditnotes to the now mis-named self.invoices list
        self.invoices.extend(notes)

        num_rows = len(self.invoices)

        # ORDER BY DATE
        if num_rows > 1:
            self.invoices.sort(key=lambda item: item.datestamp)

        if num_rows > self.num_rows:
            self.AppendRows(numRows = num_rows-self.num_rows)
            self.num_rows = num_rows

        for row, item in enumerate(self.invoices):
            # Set row label as the transaction id
            self.SetRowLabelValue(row, str(item.transaction.id))
            # Date
            self.SetCellValue(row, 0, item.datestamp.strftime("%d %b %Y"))

            if isinstance(item, SupplierInvoice):
                if not item.number:
                    self.SetCellValue(row, 1, "Invoice")
                else:
                    self.SetCellValue(row, 1, "Invoice : %s" % str(item.number))
                self.SetCellValue(row, 2, to_currency_string(item.cost))
                self.SetCellValue(row, 3, to_currency_string(item.vat))
            else:
                if not item.number:
                    self.SetCellValue(row, 1, "Credit Note")
                else:
                    self.SetCellValue(row, 1, "Credit Note : %s" % str(item.number))

                # Show creditnote cost and vat as negative values (unless they are equal to zero)
                if item.cost == D("0"):
                    self.SetCellValue(row, 2, vs_cfg["SYMBOL_ZERO_STRING"])
                else:
                    self.SetCellValue(row, 2, to_currency_string(D(-1) * item.cost))
                if item.vat == D("0"):
                    self.SetCellValue(row, 3, vs_cfg["SYMBOL_ZERO_STRING"])
                else:
                    self.SetCellValue(row, 3, to_currency_string(D(-1) * item.vat))


###### Show Payments on the notebook

class ShowPaymentsPanel(wx.Panel):
    """The panel of the ShowPaymentsTable grid"""

    def __init__(self, parent, order):
        """Create the panel"""
        wx.Panel.__init__(self, parent)

        self.parent = parent

        # Create a vertical sizer to hold the grid
        sizer=wx.BoxSizer(wx.VERTICAL)

        note_text = wx.StaticText(self, -1, """\
Double click on a payment for payment details and to create a payment note.""")
        sizer.Add(note_text, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # show the payments made
        self.show_payments_table = ShowPaymentsTable(self, order)

        sizer.Add(self.show_payments_table, 1, wx.ALIGN_CENTER | wx.EXPAND)
        # Set the size to the panel
        self.SetSizer(sizer)

    def refresh(self):
        self.show_payments_table.refresh()


class ShowPaymentsTable(wx.grid.Grid):

    def __init__(self, parent, order):
        wx.grid.Grid.__init__(self, parent, style=wx.WANTS_CHARS)

        self.parent = parent
        self.order = order

        num_cols = 5

        self.payments = self.order.get_payments()

        self.num_rows = len(self.payments)

        self.CreateGrid(self.num_rows, num_cols)
        self.EnableEditing(False)

        self.SetDefaultRowSize(vs_cfg["BUTTON_HEIGHT"], False)
        self.SetDefaultCellAlignment(wx.ALIGN_LEFT, wx.ALIGN_CENTER)

        # create colum labels
        self.SetColLabelValue(0, "Date")
        self.SetColLabelValue(1, "Payment")
        self.SetColLabelValue(2, "Net Paid")
        self.SetColLabelValue(3, "VAT Paid")
        self.SetColLabelValue(4, "Gross")

        # Set the colums showing money as right aligned
        attr_right = wx.grid.GridCellAttr()
        attr_right.SetAlignment(wx.ALIGN_RIGHT, wx.ALIGN_CENTER)
        self.SetColAttr(2, attr_right)
        self.SetColAttr(3, attr_right)
        self.SetColAttr(4, attr_right)

        self.SetColSize(0, 100)
        self.SetColSize(1, 300)
        self.SetColSize(2, 100)
        self.SetColSize(3, 100)
        self.SetColSize(4, 100)

        self.SetColMinimalAcceptableWidth(100)
        self.SetRowLabelSize(100)

        for row, item in enumerate(self.payments):
            # Set row label as the transaction id
            self.SetRowLabelValue(row, str(item.transaction.id))
            # Date
            self.SetCellValue(row, 0, item.datestamp.strftime("%d %b %Y"))
            # Transaction description
            self.SetCellValue(row, 1, item.description)
            # Net cost paid
            self.SetCellValue(row, 2, to_currency_string(item.cost))
            # VAT paid
            self.SetCellValue(row, 3, to_currency_string(item.vat))
            # Gross paid
            self.SetCellValue(row, 4, to_currency_string(item.amount))

        # Bind left mouse double - click
        self.Bind(wx.grid.EVT_GRID_CELL_LEFT_DCLICK, self.show_payment)


    @button_wrap
    def show_payment(self, event):
        "On being double clicked, obtain makepayment and show it in a ShowMakePaymentDialog"
        show_make_payment_dialog(self, self.payments[event.GetRow()])


    def refresh(self):
        "Refresh table, or add further rows to the table if further transactions have been implemented"

        self.payments = self.order.get_payments()

        num_rows = len(self.payments)

        if num_rows > self.num_rows:
            self.AppendRows(numRows = num_rows-self.num_rows)
            self.num_rows = num_rows

        for row, item in enumerate(self.payments):
            # Set row label as the transaction id
            self.SetRowLabelValue(row, str(item.transaction.id))
            # Date
            self.SetCellValue(row, 0, item.datestamp.strftime("%d %b %Y"))
            # Transaction description
            self.SetCellValue(row, 1, item.description)
            # Net cost paid
            self.SetCellValue(row, 2, to_currency_string(item.cost))
            # VAT paid
            self.SetCellValue(row, 3, to_currency_string(item.vat))
            # Gross paid
            self.SetCellValue(row, 4, to_currency_string(item.amount))


###### Show Transactions on the notebook

class ShowTransactionsPanel(wx.Panel):
    """The panel of the ShowTransactionsTable grid"""

    def __init__(self, parent, order):
        """Create the panel"""
        wx.Panel.__init__(self, parent)

        self.parent = parent

        # Create a vertical sizer to hold the grid
        sizer=wx.BoxSizer(wx.VERTICAL)

        note_text = wx.StaticText(self, -1, "Double click on the transaction for details")
        sizer.Add(note_text, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # show the transactions associated with this order
        self.show_transactions_table = ShowTransactionsTable(self, order)

        sizer.Add(self.show_transactions_table, 1, wx.ALIGN_CENTER | wx.EXPAND)
        # Set the size to the panel
        self.SetSizer(sizer)


    def refresh(self):
        self.show_transactions_table.refresh()


class ShowTransactionsTable(wx.grid.Grid):

    def __init__(self, parent, order):
        wx.grid.Grid.__init__(self, parent, style=wx.WANTS_CHARS)

        self.parent = parent
        self.order = order

        num_cols = 2

        # Get all the transactions associated with this order
        self.transactions = self.order.get_all_transactions()

        # So self.transactions contains all the transactions on the order

        self.num_rows = len(self.transactions)
        self.CreateGrid(self.num_rows, num_cols)
        self.EnableEditing(False)

        self.SetDefaultRowSize(vs_cfg["BUTTON_HEIGHT"], False)
        self.SetDefaultCellAlignment(wx.ALIGN_LEFT, wx.ALIGN_CENTER)

        # create colum labels
        self.SetColLabelValue(0, "Date")
        self.SetColLabelValue(1, "Description")

        self.SetColSize(0, 100)
        self.SetColSize(1, 400)

        self.SetColMinimalAcceptableWidth(100)
        self.SetRowLabelSize(100)

        for row, transaction in enumerate(self.transactions):
            # Set row label as the transaction id
            self.SetRowLabelValue(row, str(transaction.id))
            # Date
            self.SetCellValue(row, 0, transaction.datestamp.strftime("%d %b %Y"))
            # Description
            self.SetCellValue(row, 1, transaction.description)

        # Bind left mouse double - click
        self.Bind(wx.grid.EVT_GRID_CELL_LEFT_DCLICK, self.show_transaction)


    @button_wrap
    def show_transaction(self, event):
        "Show the transaction in a dialog"
        show_transaction_dialog(self, self.transactions[event.GetRow()])


    def refresh(self):
        "Renew table and add further rows if further transactions have been implemented"

        # Get all the transactions associated with this order
        self.transactions = self.order.get_all_transactions()
        num_rows = len(self.transactions)

        if num_rows > self.num_rows:
            self.AppendRows(numRows = num_rows-self.num_rows)
            self.num_rows = num_rows

        for row, transaction in enumerate(self.transactions):
            # Set row label as the transaction id
            self.SetRowLabelValue(row, str(transaction.id))
            # Date
            self.SetCellValue(row, 0, transaction.datestamp.strftime("%d %b %Y"))
            # Description
            self.SetCellValue(row, 1, transaction.description)


# The ShowSupplierInvoiceDialog is used to show supplier invoice data, but without a
# button to show the associated order - this is normally called from a purchase order view
# since the order is already on display

def show_supplier_invoice_dialog(parent, supplier_invoice):
    """Creates a ShowSupplierInvoiceDialog"""
    show_dialog = ShowSupplierInvoiceDialog(parent, supplier_invoice)

    # If the cursor is busy, unbusy it - as
    # now waiting for a button press
    busy_cursor_off()

    show_dialog.ShowModal()
    show_dialog.Destroy()


# Optionally, the supplier invoice can also show a 'view associated order' button
# in which case, this function should be called

def show_supplier_invoice_and_order_dialog(parent, supplier_invoice):
    """Creates a ShowSupplierInvoiceDialog with a button to show the associated purchase order"""
    show_dialog = ShowSupplierInvoiceDialog(parent, supplier_invoice, show_order = True)

    # If the cursor is busy, unbusy it - as
    # now waiting for a button press
    busy_cursor_off()

    if show_dialog.ShowModal() != wx.ID_OK:
        show_dialog.Destroy()
        return

    # Get the order to view
    order = show_dialog.order
    show_dialog.Destroy()

    if not order:
        return

    # now show the order associated with this suppliers invoice
    busy_cursor_on()
    wx.Yield()
    debug_log("picked order id: %s" % order.id)
    show_order_frame(order)


class ShowSupplierInvoiceDialog(vsDialog):
    """Called to show a supplier Invoice"""

    def __init__(self, parent, supplier_invoice, show_order = False):
        """Create the Dialog"""
        vsDialog.__init__(self, parent, title="Supplier Invoice")
        
        # Take a note of the variables
        self.parent = parent
        self.supplier_invoice = supplier_invoice
        self.invoice_number = self.supplier_invoice.number

        button_size = (100, vs_cfg["BUTTON_HEIGHT"])

        # Create a vertical sizer to hold everything
        sizer=wx.BoxSizer(wx.VERTICAL)

        # number sizer holds the invoice number and a change button
        number_sizer=wx.BoxSizer(wx.HORIZONTAL)

        number_sizer.Add(wx.StaticText(self, -1, "Invoice number :"), 0, wx.RIGHT | wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL, 10)

        self.number_field = wx.TextCtrl(self, -1, self.invoice_number, size=(200, vs_cfg["BUTTON_HEIGHT"]))
        self.number_field.SetToolTipString("Edit the suppliers invoice number if you wish")
        self.number_field.SetMaxLength(20)
        number_sizer.Add(self.number_field, 0, wx.RIGHT | wx.ALIGN_RIGHT, 20)

        self.change_button = wx.Button(self, label="Change", size=button_size)
        self.change_button.SetToolTipString("Click to change the suppliers invoice number")
        number_sizer.Add(self.change_button, 0)

        sizer.Add(number_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 10)


        # Put details in a grid sizer
        self.order = self.supplier_invoice.purchaseorder
        if self.order:  # As po info could have been deleted
            grid_sizer=wx.FlexGridSizer(rows=8, cols=2, hgap=20, vgap=10)
        else:
            grid_sizer=wx.FlexGridSizer(rows=7, cols=2, hgap=20, vgap=10)
            # cannot show associated order
            show_order = False

        # supplier
        grid_sizer.Add(wx.StaticText(self, -1, "Supplier :"), 0, wx.ALIGN_RIGHT)
        grid_sizer.Add(wx.StaticText(self, -1, self.supplier_invoice.supplier.name), 0, wx.ALIGN_LEFT)

        # order number
        if self.order:
            grid_sizer.Add(wx.StaticText(self, -1, "Purchase order :"), 0, wx.ALIGN_RIGHT)
            po_number_text = str(self.order.number)
            grid_sizer.Add(wx.StaticText(self, -1, po_number_text), 0, wx.ALIGN_LEFT)

        # date
        grid_sizer.Add(wx.StaticText(self, -1, "Invoice date :"), 0, wx.ALIGN_RIGHT)
        date_text = self.supplier_invoice.datestamp.strftime("%d %b %Y")
        grid_sizer.Add(wx.StaticText(self, -1, date_text), 0, wx.ALIGN_LEFT)

        # cost
        grid_sizer.Add(wx.StaticText(self, -1, "Cost :"), 0, wx.ALIGN_RIGHT)
        cost_text = to_currency_string(self.supplier_invoice.cost)
        grid_sizer.Add(wx.StaticText(self, -1, cost_text), 0, wx.ALIGN_LEFT)

        # vat
        grid_sizer.Add(wx.StaticText(self, -1, "VAT :"), 0, wx.ALIGN_RIGHT)
        vat_text = to_currency_string(self.supplier_invoice.vat)
        grid_sizer.Add(wx.StaticText(self, -1, vat_text), 0, wx.ALIGN_LEFT)

        # Account debited
        grid_sizer.Add(wx.StaticText(self, -1, "Account debited :"), 0, wx.ALIGN_RIGHT)
        acc = self.supplier_invoice.account
        if acc:
            acc_text = acc.fullname
        else:
            acc_text = ""
        grid_sizer.Add(wx.StaticText(self, -1, acc_text), 0, wx.ALIGN_LEFT)

        # status
        grid_sizer.Add(wx.StaticText(self, -1, "Status :"), 0, wx.ALIGN_RIGHT)
        if self.supplier_invoice.is_open:
            status_text = "Open"
        else:
            status_text = "Closed"
        grid_sizer.Add(wx.StaticText(self, -1, status_text), 0, wx.ALIGN_LEFT)

        # Transaction ID
        grid_sizer.Add(wx.StaticText(self, -1, "Transaction ID :"), 0, wx.ALIGN_RIGHT)
        transaction_text = str(self.supplier_invoice.transaction.id)
        grid_sizer.Add(wx.StaticText(self, -1, transaction_text), 0, wx.ALIGN_LEFT)


        sizer.Add(grid_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # Close button
        self.close_button = wx.Button(self, wx.ID_CLOSE, size=button_size)
        self.close_button.SetToolTipString("Click to close.")

        if show_order:
            # Create a button sizer
            button_sizer=wx.BoxSizer(wx.HORIZONTAL)
            button_sizer.Add(self.close_button, 0, wx.RIGHT, 40)

            # show order button
            self.show_order_button = wx.Button(self, label="Show purchase order", size=(200, vs_cfg["BUTTON_HEIGHT"]))
            self.show_order_button.SetToolTipString("Click to show the associated purchase order")
            button_sizer.Add(self.show_order_button, 0)
            sizer.Add(button_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)
        else:
            # there is no show order button
            sizer.Add(self.close_button, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # Set the size to the panel
        self.SetSizer(sizer)
        sizer.SetSizeHints(self)

        self.Bind(wx.EVT_BUTTON, self.change_button_pressed, self.change_button)
        self.Bind(wx.EVT_BUTTON, self.close_button_pressed, self.close_button)
        if show_order:
            self.Bind(wx.EVT_BUTTON, self.show_order_button_pressed, self.show_order_button)


    @writecheck
    @button_wrap
    def change_button_pressed(self, event):
        """change the supplier invoice number"""
        number = self.number_field.GetValue().strip()
        if number == self.invoice_number: return
        self.invoice_number = number
        self.supplier_invoice.number = number
        # The only place this number appears is in view orders
        # so if the order associated with this invoice is open - refresh it
        if self.order:
            for frm in vs_cfg["TRANSACTION_REFRESH"]:
                if isinstance(frm, ShowOrderFrame):
                    if frm.order == self.order:
                        frm.refresh()
                        break
        info_log("Supplier invoice id : %s, invoice number changed" % self.supplier_invoice.id)
        vs_message(self, "Invoice number changed", "DONE")


    def close_button_pressed(self, event):
        """Close the dialog with a cancel"""
        self.EndModal(wx.ID_CANCEL)


    def show_order_button_pressed(self, event):
        """Close the dialog with an ok"""
        self.EndModal(wx.ID_OK)



# The ShowSupplierCreditNoteDialog is used to show supplier creditnote data 
# It is normally used by calling the function show_supplier_creditnote_dialog

def show_supplier_creditnote_dialog(parent, supplier_creditnote):
    """Creates a ShowSupplierCreditNoteDialog"""
    show_dialog = ShowSupplierCreditNoteDialog(parent, supplier_creditnote)

    # If the cursor is busy, unbusy it - as
    # now waiting for a button press
    busy_cursor_off()

    show_dialog.ShowModal()
    show_dialog.Destroy()


class ShowSupplierCreditNoteDialog(vsDialog):
    """Called to show a supplier credit note"""

    def __init__(self, parent, supplier_creditnote):
        """Create the Dialog"""
        vsDialog.__init__(self, parent, title="Supplier Credit Note")
        
        # Take a note of the variables
        self.parent = parent
        self.supplier_creditnote = supplier_creditnote
        self.creditnote_number = self.supplier_creditnote.number

        button_size = (100, vs_cfg["BUTTON_HEIGHT"])

        # Create a vertical sizer to hold everything
        sizer=wx.BoxSizer(wx.VERTICAL)

        # number sizer holds the creditnote number and a change button
        number_sizer=wx.BoxSizer(wx.HORIZONTAL)

        number_sizer.Add(wx.StaticText(self, -1, "Credit Note number :"), 0, wx.RIGHT | wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL, 10)

        self.number_field = wx.TextCtrl(self, -1, self.creditnote_number, size=(200, vs_cfg["BUTTON_HEIGHT"]))
        self.number_field.SetToolTipString("Edit the suppliers credit note number if you wish")
        self.number_field.SetMaxLength(20)
        number_sizer.Add(self.number_field, 0, wx.RIGHT | wx.ALIGN_RIGHT, 20)

        self.change_button = wx.Button(self, label="Change", size=button_size)
        self.change_button.SetToolTipString("Click to change the suppliers credit note number")
        number_sizer.Add(self.change_button, 0)

        sizer.Add(number_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 10)


        status = self.supplier_creditnote.status
        if status == 1:
            payment_text = """\
This credit note was received with a payment, it was
not offset against the payment of a supplier invoice"""
            sizer.Add(wx.StaticText(self, -1, payment_text), 0, wx.ALL | wx.ALIGN_CENTER, 10)


        self.order = self.supplier_creditnote.purchaseorder
        if (status == 0) and self.order:  # As po info could have been deleted
            grid_sizer=wx.FlexGridSizer(rows=8, cols=2, hgap=20, vgap=10)
        else:
            grid_sizer=wx.FlexGridSizer(rows=6, cols=2, hgap=20, vgap=10)


        # Put details in a grid sizer

        # supplier
        grid_sizer.Add(wx.StaticText(self, -1, "Supplier :"), 0, wx.ALIGN_RIGHT)
        grid_sizer.Add(wx.StaticText(self, -1, self.supplier_creditnote.supplier.name), 0, wx.ALIGN_LEFT)

        if (status == 0) and self.order:
            # order number
            grid_sizer.Add(wx.StaticText(self, -1, "Purchase order :"), 0, wx.ALIGN_RIGHT)
            po_number_text = str(self.order.number)
            grid_sizer.Add(wx.StaticText(self, -1, po_number_text), 0, wx.ALIGN_LEFT)

            # invoice number
            grid_sizer.Add(wx.StaticText(self, -1, "Supplier Invoice :"), 0, wx.ALIGN_RIGHT)
            inv_number_text = str(self.supplier_creditnote.supplierinvoice.number)
            grid_sizer.Add(wx.StaticText(self, -1, inv_number_text), 0, wx.ALIGN_LEFT)

        # date
        grid_sizer.Add(wx.StaticText(self, -1, "Credit Note date :"), 0, wx.ALIGN_RIGHT)
        date_text = self.supplier_creditnote.datestamp.strftime("%d %b %Y")
        grid_sizer.Add(wx.StaticText(self, -1, date_text), 0, wx.ALIGN_LEFT)

        # cost
        grid_sizer.Add(wx.StaticText(self, -1, "Cost :"), 0, wx.ALIGN_RIGHT)
        cost_text = to_currency_string(self.supplier_creditnote.cost)
        grid_sizer.Add(wx.StaticText(self, -1, cost_text), 0, wx.ALIGN_LEFT)

        # vat
        grid_sizer.Add(wx.StaticText(self, -1, "VAT :"), 0, wx.ALIGN_RIGHT)
        vat_text = to_currency_string(self.supplier_creditnote.vat)
        grid_sizer.Add(wx.StaticText(self, -1, vat_text), 0, wx.ALIGN_LEFT)

        # Account credited
        grid_sizer.Add(wx.StaticText(self, -1, "Account credited :"), 0, wx.ALIGN_RIGHT)
        acc = self.supplier_creditnote.account
        if acc:
            acc_text = acc.fullname
        else:
            acc_text = ""
        grid_sizer.Add(wx.StaticText(self, -1, acc_text), 0, wx.ALIGN_LEFT)

        # Transaction ID
        grid_sizer.Add(wx.StaticText(self, -1, "Transaction ID :"), 0, wx.ALIGN_RIGHT)
        transaction_text = str(self.supplier_creditnote.transaction.id)
        grid_sizer.Add(wx.StaticText(self, -1, transaction_text), 0, wx.ALIGN_LEFT)

        sizer.Add(grid_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # Close button
        self.close_button = wx.Button(self, wx.ID_CLOSE, size=button_size)
        self.close_button.SetToolTipString("Click to close.")

        sizer.Add(self.close_button, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # Set the size to the panel
        self.SetSizer(sizer)
        sizer.SetSizeHints(self)

        self.Bind(wx.EVT_BUTTON, self.change_button_pressed, self.change_button)
        self.Bind(wx.EVT_BUTTON, self.close_button_pressed, self.close_button)


    @writecheck
    @button_wrap
    def change_button_pressed(self, event):
        """change the supplier creditnote number"""
        self.creditnote_number = self.number_field.GetValue().strip()
        self.supplier_creditnote.number = self.creditnote_number
        # The only place this number appears is in view orders
        # so if the order associated with this note is open - refresh it
        if self.order:
            for frm in vs_cfg["TRANSACTION_REFRESH"]:
                if isinstance(frm, ShowOrderFrame):
                    if frm.order == self.order:
                        frm.refresh()
                        break
        info_log("Supplier credit note id : %s, creditnote number changed" % self.supplier_creditnote.id)
        vs_message(self, "Credit Note number changed", "DONE")


    def close_button_pressed(self, event):
        """Close the dialog"""
        self.EndModal(wx.ID_OK)


# The ShowMakePaymentDialog is used to show payment made data 
# It is normally used by calling the function show_make_payment_dialog

def show_make_payment_dialog(parent, payment):
    """Creates a ShowMakePaymentDialog"""
    show_dialog = ShowMakePaymentDialog(parent, payment)

    # If the cursor is busy, unbusy it - as
    # now waiting for a button press
    busy_cursor_off()

    show_dialog.ShowModal()
    show_dialog.Destroy()


class ShowMakePaymentDialog(vsDialog):
    """Called to show the makepayment"""

    def __init__(self, parent, payment):
        """Create the Dialog"""
        vsDialog.__init__(self, parent, title="Payment Sent")
        
        # Take a note of the variables
        self.payment = payment

        button_size = (100, vs_cfg["BUTTON_HEIGHT"])

        # Create a vertical sizer to hold everything
        sizer=wx.BoxSizer(wx.VERTICAL)

        # payment memo

        memo_text = "Payment memo : %s" % self.payment.description

        sizer.Add(wx.StaticText(self, -1, memo_text), 0, wx.ALL | wx.ALIGN_CENTER, 10)

        if payment.supplier_address:
            # payment address on the left of a horizontal sizer, payment details on the right
            details_sizer=wx.BoxSizer(wx.HORIZONTAL)
            supplier_text = "Supplier: %s\n%s" % (payment.supplier.name,
                                                      payment.supplier_address.lines_string)
            details_sizer.Add(wx.StaticText(self, -1, supplier_text), 0, wx.RIGHT | wx.ALIGN_TOP, 80)
            # Put payment details in a grid sizer
            pay_sizer=wx.FlexGridSizer(rows=5, cols=2, hgap=20, vgap=10)
        else:
            # details sizer does not exist, just create the pay sizer with the supplier field added
            # Put payment details in a grid sizer
            pay_sizer=wx.FlexGridSizer(rows=6, cols=2, hgap=20, vgap=10)
            # supplier
            pay_sizer.Add(wx.StaticText(self, -1, "Supplier :"), 0, wx.ALIGN_RIGHT)
            supplier_text = payment.supplier.name
            pay_sizer.Add(wx.StaticText(self, -1, supplier_text), 0, wx.ALIGN_LEFT)

        # order number
        pay_sizer.Add(wx.StaticText(self, -1, "Purchase order :"), 0, wx.ALIGN_RIGHT)
        po_number_text = str(payment.purchaseorder.number)
        pay_sizer.Add(wx.StaticText(self, -1, po_number_text), 0, wx.ALIGN_LEFT)

        # date
        pay_sizer.Add(wx.StaticText(self, -1, "Payment date :"), 0, wx.ALIGN_RIGHT)
        date_text = payment.datestamp.strftime("%d %b %Y")
        pay_sizer.Add(wx.StaticText(self, -1, date_text), 0, wx.ALIGN_LEFT)

        # payment amount
        pay_sizer.Add(wx.StaticText(self, -1, "Payment amount :"), 0, wx.ALIGN_RIGHT)
        amount_text = to_currency_string(payment.amount)
        pay_sizer.Add(wx.StaticText(self, -1, amount_text), 0, wx.ALIGN_LEFT)

        # payment account
        pay_sizer.Add(wx.StaticText(self, -1, "Paid from account :"), 0, wx.ALIGN_RIGHT)
        account_text = payment.account.fullname
        pay_sizer.Add(wx.StaticText(self, -1, account_text), 0, wx.ALIGN_LEFT)

        # Transaction ID
        pay_sizer.Add(wx.StaticText(self, -1, "Transaction ID :"), 0, wx.ALIGN_RIGHT)
        transaction_text = str(payment.transaction.id)
        pay_sizer.Add(wx.StaticText(self, -1, transaction_text), 0, wx.ALIGN_LEFT)

        if payment.supplier_address:
            # details sizer contains address on the left, pay on the right
            details_sizer.Add(pay_sizer, 0, wx.LEFT | wx.ALIGN_LEFT, 10)
            sizer.Add(details_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 10)
        else:
            # details sizer does not exist, just add the pay sizer
            sizer.Add(pay_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        sizer.Add(wx.StaticText(self, -1, "Payment applied to supplier invoices :"), 0, wx.TOP | wx.ALIGN_CENTER, 10)


        # ---------------------- Create the invoices table

        self.invoices_table = PaymentInvoicesTable(self, payment)
        sizer.Add(self.invoices_table, 1, wx.ALL | wx.ALIGN_CENTER | wx.EXPAND, 20)

        # buttons

        button_sizer=wx.BoxSizer(wx.HORIZONTAL)

        # page setup button
        self.page_button = wx.Button(self, label="Page setup", size=button_size)
        self.page_button.SetToolTipString("Click to set up the pdf page size and orientation")   
        button_sizer.Add(self.page_button, 0, wx.RIGHT, 20)

        # pdf button
        self.pdf_button = wx.Button(self, label="Create pdf", size=button_size)
        self.pdf_button.SetToolTipString("Click to create a pdf file.")
        button_sizer.Add(self.pdf_button, 0, wx.RIGHT, 20)

        # Close button
        self.close_button = wx.Button(self, wx.ID_CLOSE, size=button_size)
        self.close_button.SetToolTipString("Click to close.")
        button_sizer.Add(self.close_button, 0)

        sizer.Add(button_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # Set the size to the panel
        self.SetSizer(sizer)
        sizer.SetSizeHints(self)

        self.Bind(wx.EVT_BUTTON, self.close_button_pressed, self.close_button)
        self.Bind(wx.EVT_BUTTON, self.page_button_pressed, self.page_button)
        self.Bind(wx.EVT_BUTTON, self.pdf_button_pressed, self.pdf_button)


    def close_button_pressed(self, event):
        """Close the dialog"""
        self.EndModal(wx.ID_OK)


    @button_wrap
    def page_button_pressed(self, event):
        "page button pressed, setup the pdf page size and orientation"
        vatstuff_packages.vs_pdf.page_setup.page_setup_dialog(self)


    @button_wrap
    def pdf_button_pressed(self, event):
        "Create a pdf file"

        if not vs_cfg["REPORTLABS_AVAILABLE"]:
            # no pdf capability
            vs_message(self, "Function unavailable - the reportlab module is not installed",
                             "ERROR")
            return

        # default filename
        filename= "payment_%s_sent_%s.pdf" % (self.payment.transaction.id, self.payment.datestamp.isoformat())

        # Create the save file dialog.
        status, filepath = save_file_as(self, 'pdf', vs_cfg["REPORTS_PDF_FOLDER"], filename)

        if status:
            if vatstuff_packages.vs_pdf.makepayment_form.create_pdf(filepath, self.payment):
                # And show the file in the default viewer
                if not vatstuff_packages.vs_pdf.pdf_functions.show_pdf_file(filepath):
                    vs_message(self, "pdf file created", "Done")
            else:
                vs_message(self, "Unable to create the pdf file", "ERROR")


class PaymentInvoicesTable(wx.grid.Grid):

    def __init__(self, parent, payment):
        wx.grid.Grid.__init__(self, parent, style=wx.WANTS_CHARS)

        self.parent = parent

        i_rows = []

        self.inv_list = payment.get_invoice_list()
        # inv_list being a list of dictionaries - { "INVOICE":supplierinvoice,
        #                                           "COST":cost,
        #                                           "VAT":vat }
        #         one dictionary for each invoice this payment covers

        num_rows = len(self.inv_list)
        num_cols = 3

        self.CreateGrid(num_rows, num_cols)
        self.EnableEditing(False)

        self.SetDefaultRowSize(vs_cfg["BUTTON_HEIGHT"], False)
        self.SetDefaultCellAlignment(wx.ALIGN_LEFT, wx.ALIGN_CENTER)

        # create colum labels
        self.SetColLabelValue(0, "Supplier invoice number")
        self.SetColLabelValue(1, "Net Paid")
        self.SetColLabelValue(2, "VAT Paid")

        # Set the colums showing money as right aligned
        attr_right = wx.grid.GridCellAttr()
        attr_right.SetAlignment(wx.ALIGN_RIGHT, wx.ALIGN_CENTER)
        self.SetColAttr(1, attr_right)
        self.SetColAttr(2, attr_right)

        self.SetColSize(0, 300)
        self.SetColSize(1, 100)
        self.SetColSize(2, 100)

        self.SetColMinimalAcceptableWidth(100)
        self.SetRowLabelSize(100)

        for row, item in enumerate(self.inv_list):
            # Set row label as the transaction id
            self.SetRowLabelValue(row, str(item["INVOICE"].transaction.id))
            # Invoice number
            self.SetCellValue(row, 0, item["INVOICE"].number)
            # Net
            self.SetCellValue(row, 1, to_currency_string(item["COST"]))
            # VAT
            self.SetCellValue(row, 2, to_currency_string(item["VAT"]))

        # Bind left mouse double - click
        self.Bind(wx.grid.EVT_GRID_CELL_LEFT_DCLICK, self.show_invoice, self)


    @button_wrap
    def show_invoice(self, event):
        "Invoice has been double - clicked"

        invoice = self.inv_list[event.GetRow()]['INVOICE']
        show_supplier_invoice_dialog(self, invoice)
        # After showing the supplier invoice dialog refresh this
        # local table 
        for row, item in enumerate(self.inv_list):
            self.SetCellValue(row, 0, item["INVOICE"].number)


################### PROJECT ###########################

def pick_project_dialog(parent, order):
    """Creates a PickProjectDialog"""
    pick_project_dialog = PickProjectDialog(parent, order)

    # If the cursor is busy, unbusy it - as
    # now waiting for a button press
    busy_cursor_off()

    if pick_project_dialog.ShowModal() != wx.ID_OK:
        pick_project_dialog.Destroy()
        return

    # if ok has been returned, then the view button has been chosen
    # to view a project
    project = pick_project_dialog.project
    pick_project_dialog.Destroy()

    # Call view_project_frame to view the project orders
    busy_cursor_on()
    wx.Yield()
    view_project_frame(project)


class PickProjectDialog(vsDialog):
    "Called when the project button on an order is chosen"

    def __init__(self, parent, order):
        """Create the Dialog"""
        vsDialog.__init__(self, parent, "Projects")

        # Take a note of the order
        self.order = order

        # This dialog can pick a project, set initial value to None
        self.project = None

        button_size = (100, vs_cfg["BUTTON_HEIGHT"])
             
        # Create the sizer for the panel
        sizer=wx.BoxSizer(wx.VERTICAL)

        top_text=wx.StaticText(self, -1, """\
Each Purchase Order can be assigned to a project.
The 'New' button creates a new project.""")
        sizer.Add(top_text, 0, wx.ALL | wx.ALIGN_CENTER, 20)
        
        help_text=wx.StaticText(self, -1, """\
Click a check box to assign (or un-assign) purchase order number %s to the project
Double click a project to list the orders assigned to the selected project.""" % self.order.number)
        sizer.Add(help_text, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        self.project_table = ProjectTable(self, self.order)
        sizer.Add(self.project_table, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # buttons
        button_sizer=wx.BoxSizer(wx.HORIZONTAL)

        # ------------------- Close button
        self.close_button = wx.Button(self, wx.ID_CLOSE, size=button_size)
        self.close_button.SetToolTipString("Click to close this window")
        button_sizer.Add(self.close_button, 0, wx.RIGHT, 20)

        # ------------------- New button
        self.new_button = wx.Button(self, label="New", size=button_size)
        self.new_button.SetToolTipString("Create a new project")
        button_sizer.Add(self.new_button, 0, wx.ALIGN_LEFT)

        sizer.Add(button_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # Set the size to a bit more than the mininimum size
        self.SetSizer(sizer)
        sizer.SetSizeHints(self)

        # Bind the buttons
        self.Bind(wx.EVT_BUTTON, self.close_button_pressed, self.close_button)
        self.Bind(wx.EVT_BUTTON, self.new_button_pressed, self.new_button)

        # Bind left mouse double - click to toggle order to assign/unassign to project
        self.Bind(wx.grid.EVT_GRID_CELL_LEFT_DCLICK, self.view_project_double_click, self.project_table)


    def close_button_pressed(self, event):
        """Close the dialog"""
        self.EndModal(wx.ID_CANCEL)


    @button_wrap
    def view_project_double_click(self, event):
        """Choose a project - so caller can list its orders in a frame"""
        # get the project for this row
        project_ref = self.project_table.GetCellValue(event.GetRow(), 0)
        self.project = Project.from_number(project_ref)
        self.EndModal(wx.ID_OK)


    @writecheck
    @button_wrap
    def new_button_pressed(self, event):
        "Creates an AddProjectDialog dialog"

        add_project_dialog = AddProjectDialog(self)
        if add_project_dialog.ShowModal() == wx.ID_OK:
            new_project = add_project_dialog.new_project
            self.project_table.update(new_project)
        add_project_dialog.Destroy()



class ProjectTable(wx.grid.Grid):

    def __init__(self, parent, order):
        wx.grid.Grid.__init__(self, parent, size=(900,200), style=wx.WANTS_CHARS)

        self.order = order
        all_projects = Project.get_all()

        self.num_rows = len(all_projects)
        num_cols = 4


        self.CreateGrid(self.num_rows, num_cols)
        self.EnableEditing(False)

        self.SetDefaultRowSize(vs_cfg["BUTTON_HEIGHT"], False)
        self.SetDefaultCellAlignment(wx.ALIGN_LEFT, wx.ALIGN_CENTER)

        # create column labels
        self.SetColLabelValue(0, "REF")
        self.SetColLabelValue(1, "Assigned")
        self.SetColLabelValue(2, "Project Description")
        self.SetColLabelValue(3, "Owner")

        for row in range(self.num_rows):
            self.SetCellRenderer(row, 1, wx.grid.GridCellBoolRenderer())

        # Make assignment col 1 centre aligned
        attr_1 = wx.grid.GridCellAttr()
        attr_1.SetAlignment(wx.ALIGN_CENTER, wx.ALIGN_CENTER)
        self.SetColAttr(1, attr_1)

        order_project = None
        if order.project:
            order_project = order.project

        order_row = None
        for row, proj in enumerate(all_projects):
            self.SetCellValue(row, 0, proj.number)
            if order_project == proj:
                self.SetCellValue(row, 1, "1")
                order_row = row
            else:
                self.SetCellValue(row, 1, "")
            self.SetCellValue(row, 2, proj.description)
            if proj.contact:
                self.SetCellValue(row, 3, proj.contact.name)
            else:
                self.SetCellValue(row, 3, "")

        self.SetColSize(0, 100)
        self.SetColSize(1, 100)
        self.SetColSize(2, 400)
        self.SetColSize(3, 200)

        self.SetColMinimalAcceptableWidth(100)
        self.SetRowLabelSize(50)

        if order_row:
            self.SetGridCursor(order_row, 0)
            self.MakeCellVisible(order_row, 0)

        # Bind left mouse click to toggle order to project
        self.Bind(wx.grid.EVT_GRID_CELL_LEFT_CLICK, self.assign_toggle, self)


    @writecheck
    @button_wrap
    def assign_toggle(self, event):
        "On being clicked, assign or unassign purchase order to the project"

        col = event.GetCol()
        if col != 1:
            event.Skip()
            return
        row = event.GetRow()
        if row == None or row < 0: return
        # get the project for this row
        project_ref = self.GetCellValue(row, 0)
        project = Project.from_number(project_ref)
        if self.order.project == project:
            # This is an unassign operation
            message = """\
This removes order number %s
from project %s. Are you sure?""" % (self.order.number, project_ref)
            confirm = vs_yesno(self, message, "Unassign order")
            if confirm:
                self.order.project = None
                self.SetCellValue(row, 1, "")
                return
        else:
            # This assigns the order to a project
            message = """\
Press Yes to assign Purchase Order %s
to project reference %s""" % (self.order.number, project_ref)
            confirm = vs_yesno(self, message, "Assign order to project")
            if confirm:
                self.order.project = project
                # whipe the field value from all rows
                for eachrow in range(self.num_rows):
                    self.SetCellValue(eachrow, 1, "")
                # and assign it to this row
                self.SetCellValue(row, 1, "1")
                self.SetGridCursor(row, 1)
 

    def update(self, new_project, row = None):
        "Update a row with new project details, if row is not given, append a new row"
        if row == None:
            row = self.GetNumberRows()
            self.AppendRows(numRows=1)
            self.num_rows = row+1
            self.SetCellRenderer(row, 1, wx.grid.GridCellBoolRenderer())
        self.SetCellValue(row, 0, new_project.number)
        if self.order.project == new_project:
            self.SetCellValue(row, 1, "1")
        else:
            self.SetCellValue(row, 1, "")
        self.SetCellValue(row, 2, new_project.description)
        if new_project.contact:
            self.SetCellValue(row, 3, new_project.contact.name)
        else:
            self.SetCellValue(row, 3, "")
        self.ForceRefresh()
        self.SetGridCursor(row, 0)
        self.MakeCellVisible(row, 0)


# The AddProjectDialog is used to create a new project

class AddProjectDialog(vsDialog):
    """Called to create a new project"""

    def __init__(self, parent):
        """Create the Dialog"""
        vsDialog.__init__(self, parent, title="Create a new project")

        button_size = (100, vs_cfg["BUTTON_HEIGHT"])

        # Create a vertical sizer to hold everything
        sizer=wx.BoxSizer(wx.VERTICAL)

        # show the help text
        help_text = "Create a new project reference, description and project owner"
        sizer.Add(wx.StaticText(self, -1, help_text), 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # Create fields in a grid sizer
        field_sizer=wx.FlexGridSizer(rows=3, cols=2, hgap=20, vgap=40)

        # The number
        field_sizer.Add(wx.StaticText(self, -1, "Project reference or number:"), 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)
        self.number_field = wx.TextCtrl(self, -1, "", size=(100, vs_cfg["BUTTON_HEIGHT"]))
        self.number_field.SetToolTipString("A unique project reference")
        self.number_field.SetMaxLength(10)
        field_sizer.Add(self.number_field, 0, wx.ALIGN_LEFT)

        # The description
        field_sizer.Add(wx.StaticText(self, -1, "Project description:"), 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)
        self.description_field = wx.TextCtrl(self, -1, "", size=(400, vs_cfg["BUTTON_HEIGHT"]))
        self.description_field.SetToolTipString("An optional project description")
        self.description_field.SetMaxLength(50)
        field_sizer.Add(self.description_field, 0, wx.ALIGN_LEFT)

        # Project owner
        field_sizer.Add(wx.StaticText(self, -1, "Project owner:"), 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)
        self.form_contacts = Contact.company_list()
        contacts_list = [""]                          # First choice is blank
        for contact in self.form_contacts:
            contacts_list.append("%s : %s" % (contact.name, contact.email))
        self.form_contacts.insert(0, None)            # insert None as first item in this list also

        self.contact_field = wx.Choice(self, -1, size=(400, vs_cfg["BUTTON_HEIGHT"]), choices=contacts_list)
        self.contact_field.SetToolTipString("Optional project owner name")
        field_sizer.Add(self.contact_field, 0, wx.ALIGN_LEFT)

        sizer.Add(field_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # show further help text
        help_text2 = "Owner names are created under the Company tab - Company Contacts"
        sizer.Add(wx.StaticText(self, -1, help_text2), 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # ---------------------- Apply - Cancel -- buttons

        # Create a horizontal button_sizer to hold the buttons
        button_sizer=wx.BoxSizer(wx.HORIZONTAL)

        # Apply button
        self.apply_button = wx.Button(self, wx.ID_APPLY, size=button_size)
        self.apply_button.SetToolTipString("Click to create the new project")
        button_sizer.Add(self.apply_button, 0, wx.RIGHT | wx.ALIGN_CENTER, 40)

        # Cancel button
        self.cancel_button = wx.Button(self, wx.ID_CANCEL, size=button_size)
        self.cancel_button.SetToolTipString("Click to cancel.")
        button_sizer.Add(self.cancel_button, 0, wx.LEFT | wx.ALIGN_CENTER, 40)

        # Add button_sizer to the main sizer
        sizer.Add(button_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # Set the size to the frame
        self.SetSizer(sizer)
        sizer.SetSizeHints(self)

        # ---------- Bind buttons to handler functions
        
        self.Bind(wx.EVT_BUTTON, self.apply_button_pressed, self.apply_button)
        self.Bind(wx.EVT_BUTTON, self.cancel_button_pressed, self.cancel_button)


    # ----------------- Handler functions

    @writecheck
    @button_wrap
    def apply_button_pressed(self, event):
        """ Apply button - check and apply all fields"""

        number = self.number_field.GetValue().strip()
        if not number:
            vs_message(self, "A unique project number or reference is required", "ERROR")
            return

        if Project.from_number(number):
            vs_message(self, "A unique project number or reference is required", "ERROR")
            return

        description = self.description_field.GetValue().strip()

        contact_selected_list_item = self.contact_field.GetCurrentSelection()
        if contact_selected_list_item < 0:
            chosen_contact = None
        else:
            chosen_contact = self.form_contacts[contact_selected_list_item]

        self.new_project = create_objects.create_Project(number, description, chosen_contact)
        if not self.new_project:
            vs_message(self, "Failed to create the project", "ERROR")
            return
        # New project obtained and available in self.new_project
        self.EndModal(wx.ID_OK)



    def cancel_button_pressed(self, event):
        "Cancel button - return wx.ID_CANCEL"
        self.EndModal(wx.ID_CANCEL)




##### The ViewProjectFrame shows all orders in a project #####

def view_project_frame(project):
    """Creates a ViewProjectFrame to show all orders in a project"""
    if project:
        # list all orders in this project in a ViewProjectFrame
        # if a ViewProjectFrame frame with this project is already open - show it
        for frm in vs_cfg["TRANSACTION_REFRESH"]:
            if isinstance(frm, ViewProjectFrame):
                if frm.project == project:
                    frm.Raise()
                    return          
        view_project_frame = ViewProjectFrame(top_frame(), project)
        view_project_frame.Show()
    busy_cursor_off()


class ViewProjectFrame(vsFrame):
    "Called to create a frame to view the orders assigned to a project"

    def __init__(self, parent, project):
        "Create the frame - holding a SearchPanel"
        frame_title = "Project %s" % project.number 
        vsFrame.__init__(self, parent, title=frame_title)

        # needed when frames are being tested if they are open or not
        self.project = project

        # Create the ViewProject
        self.view_project_panel = ViewProjectPanel(self, project)

        # Get the panel size
        panel_size = self.view_project_panel.GetMinSize()

        # Set minimum size equal to the panel
        self.SetMinSize(panel_size)
        self.SetClientSize(panel_size)

        # update the list of windows which need a refresh if
        # a transaction occurs, by adding self
        vs_cfg["TRANSACTION_REFRESH"].append(self)

        # bind the close event
        self.Bind(wx.EVT_CLOSE, self.onclose)


    def refresh(self):
        "refresh this frame, by calling the refresh function in its panel"
        self.view_project_panel.refresh()


    def onclose(self, evt):
        # Decrement the refresh list
        vs_cfg["TRANSACTION_REFRESH"].remove(self)
        self.Destroy()



# ----------------------ViewProjectPanel

class ViewProjectPanel(wx.Panel):
    """The panel of the ViewProjectFrame"""

    def __init__(self, parent, project):
        """Create the panel"""
        wx.Panel.__init__(self, parent)

        self.parent = parent
        self.project = project

        # getting local copies speeds things up
        self.project_description = self.project.description
        self.project_contact = self.project.contact
        if self.project_contact:
            self.project_contact_name = self.project_contact.name
            self.project_contact_email = self.project_contact.email
        else:
            self.project_contact_name = ""
            self.project_contact_email = ""

        button_size = (100, vs_cfg["BUTTON_HEIGHT"])

        # Create a vertical sizer to hold everything
        self.sizer=wx.BoxSizer(wx.VERTICAL)


        # Create fields in a grid sizer
        field_sizer=wx.FlexGridSizer(rows=3, cols=3, hgap=20, vgap=20)

        # The description
        field_sizer.Add(wx.StaticText(self, -1, "Project description:"), 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)
        self.description_field = wx.TextCtrl(self, -1, self.project_description, size=(400, vs_cfg["BUTTON_HEIGHT"]))
        self.description_field.SetToolTipString("An optional project description")
        self.description_field.SetMaxLength(50)
        field_sizer.Add(self.description_field, 0, wx.ALIGN_LEFT)

        self.description_button = wx.Button(self, label="Change", size=button_size)
        field_sizer.Add(self.description_button, 0, wx.ALIGN_LEFT)

        # Current owner
        field_sizer.Add(wx.StaticText(self, -1, "Current owner:"), 0, wx.ALIGN_RIGHT)
        if self.project_contact:
            self.current_owner_text = wx.StaticText(self, -1, "%s : %s" % (self.project_contact_name,
                                                                           self.project_contact_email))
        else:
            self.current_owner_text = wx.StaticText(self, -1, "Not set")
        field_sizer.Add(self.current_owner_text, 0, wx.ALIGN_LEFT)
        field_sizer.Add(wx.StaticText(self, -1, ""), 0, wx.ALIGN_LEFT)

        # Project owner
        field_sizer.Add(wx.StaticText(self, -1, "Project owner:"), 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)
        self.form_contacts = Contact.company_list()
        contacts_list = [""]                          # First choice is blank
        for contact in self.form_contacts:
            contacts_list.append("%s : %s" % (contact.name, contact.email))
        self.form_contacts.insert(0, None)            # insert None as first item in this list also

        self.contact_field = wx.Choice(self, -1, size=(400, vs_cfg["BUTTON_HEIGHT"]), choices=contacts_list)
        self.contact_field.SetToolTipString("Optional project owner name")
        field_sizer.Add(self.contact_field, 0, wx.ALIGN_LEFT)

        if self.project.contact:
            try:
                key = self.form_contacts.index(self.project_contact)
            except Exception:
                key = 0
            self.contact_field.SetSelection(key)

        self.contact_button = wx.Button(self, label="Change", size=button_size)
        field_sizer.Add(self.contact_button, 0, wx.ALIGN_LEFT)


        self.sizer.Add(field_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        help_text = "Purchase orders assigned to project %s : double click an order to view it." % self.project.number


        self.sizer.Add(wx.StaticText(self, -1, help_text), 0, wx.ALL | wx.ALIGN_CENTER, 20)
        self.view_project_orders_table = ViewProjectOrdersTable(self, project)

        self.sizer.Add(self.view_project_orders_table, 1, wx.ALL | wx.ALIGN_CENTER | wx.EXPAND, 20)

        # Create a horizontal sizer to hold buttons
        button_sizer=wx.BoxSizer(wx.HORIZONTAL)


        # Close button
        self.close_button = wx.Button(self, wx.ID_CLOSE, size=button_size)
        self.close_button.SetToolTipString("Click to close.")
        button_sizer.Add(self.close_button, 0, wx.RIGHT, 20)

        # Report button
        self.report_button = wx.Button(self, label="Report", size=button_size)
        self.report_button.SetToolTipString("Click to create a list of orders in the project.")
        button_sizer.Add(self.report_button, 0, wx.RIGHT, 20)

        # Delete button
        self.delete_button = wx.Button(self, label="Delete", size=button_size)
        self.delete_button.SetToolTipString("Click to delete this project.")
        button_sizer.Add(self.delete_button, 0)

        self.sizer.Add(button_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # Set the size to the panel
        self.SetSizer(self.sizer)

        # Set panel minimum width to 800
        # Set panel minimum height to 500
        self.SetSizeHints(800, 500)

        # ---------- Bind buttons to handler functions
        self.Bind(wx.EVT_BUTTON, self.close_button_pressed, self.close_button)
        self.Bind(wx.EVT_BUTTON, self.report_button_pressed, self.report_button)
        self.Bind(wx.EVT_BUTTON, self.description_button_pressed, self.description_button)
        self.Bind(wx.EVT_BUTTON, self.contact_button_pressed, self.contact_button)
        self.Bind(wx.EVT_BUTTON, self.delete_button_pressed, self.delete_button)


    @writecheck
    @button_wrap    # note: using dialog_apply, as no need to disable all frames
    def description_button_pressed(self, event):
        "Change the projects description"
        description = self.description_field.GetValue().strip()
        if description == self.project_description: return
        self.project_description = description
        self.project.description = description
        vs_message(self, "Project description changed", "Done")


    @writecheck
    @button_wrap    # note: using dialog_apply, as no need to disable all frames
    def contact_button_pressed(self, event):
        "Change the projects owner"
        contact_selected_list_item = self.contact_field.GetCurrentSelection()
        if contact_selected_list_item < 0:
            chosen_contact = None
        else:
            chosen_contact = self.form_contacts[contact_selected_list_item]

        if chosen_contact == self.project_contact: return

        self.project_contact = chosen_contact
        self.project.contact = chosen_contact

        if self.project_contact:
            self.project_contact_name = self.project_contact.name
            self.project_contact_email = self.project_contact.email
            self.current_owner_text.SetLabel("%s : %s" % (self.project_contact_name,
                                                          self.project_contact_email))
        else:
            self.project_contact_name = ""
            self.project_contact_email = ""
            self.current_owner_text.SetLabel("Not set")

        self.sizer.Layout()
        vs_message(self, "Project owner changed", "Done")


    def close_button_pressed(self, event):
        """Close parent"""
        self.parent.Close()
        return


    @button_wrap
    def report_button_pressed(self, event):
        """Get two dates and generate a report"""
        project_ref = self.project.number
        help_text = """\
A report will be generated listing purchase orders assigned to
project %s which were issued between the two dates:""" % project_ref
        title = "Project %s" % project_ref
        status, from_date, to_date = two_dates(self, title , help_text)
        if not status:
            return
        busy_cursor_on()
        wx.Yield()

        filename = "project_%s" % project_ref
        filename = proper_filename(filename, "project", to_date)
        make_report.make_report_and_frame('project',
                                          "Project Orders",
                                          filename,
                                          from_date,
                                          to_date,
                                          project=self.project)


    @writecheck
    @button_wrap
    def delete_button_pressed(self, event):
        "Delete this project"
        message = """\
This deletes project reference %s
and un-assigns associated Purchase Orders.
Press Yes to confirm.""" % self.project.number
        confirm = vs_yesno(self, message, "Delete project")
        if not confirm:
            return
        self.project.delete()
        # close this window
        self.parent.Close()


    def refresh(self):
        "Refresh the contacts list, and call refresh in the self.view_project_orders_table"

        # Refresh list and set selection to none
        self.contact_field.Clear()
        self.form_contacts = Contact.company_list()
        contacts_list = [""]                          # First choice is blank
        for contact in self.form_contacts:
            contacts_list.append("%s : %s" % (contact.name, contact.email))
        self.form_contacts.insert(0, None)            # insert None as first item in this list also

        self.contact_field.AppendItems(contacts_list)
        self.contact_field.SetSelection(0)

        # now refresh the table
        self.view_project_orders_table.refresh()


############# list orders in the project

class ViewProjectOrdersTable(wx.grid.Grid):

    def __init__(self, parent, project):
        wx.grid.Grid.__init__(self, parent, style=wx.WANTS_CHARS)

        self.parent = parent
        self.project = project
        self.order_list = self.project.order_list

        self.num_rows = len(self.order_list)
        num_cols = 3

        self.CreateGrid(self.num_rows, num_cols)
        self.EnableEditing(False)

        self.SetDefaultRowSize(vs_cfg["BUTTON_HEIGHT"], False)
        self.SetDefaultCellAlignment(wx.ALIGN_LEFT, wx.ALIGN_CENTER)

        # create column labels
        self.SetColLabelValue(0, "Date")
        self.SetColLabelValue(1, "Supplier")
        self.SetColLabelValue(2, "Description")

        for row, data in enumerate(self.order_list):
            self.SetRowLabelValue(row, str(data.number))
            self.SetCellValue(row, 0, data.datestamp.strftime("%d %b %Y"))
            self.SetCellValue(row, 1, data.supplier.name)
            self.SetCellValue(row, 2, data.description)

        self.SetColSize(0, 100)
        self.SetColSize(1, 200)
        self.SetColSize(2, 350)


        self.SetColMinimalAcceptableWidth(100)
        self.SetRowLabelSize(100)

        # Bind left mouse double - click
        self.Bind(wx.grid.EVT_GRID_CELL_LEFT_DCLICK, self.show_order, self)


    def refresh(self):
        "Refresh description and date values"
        self.order_list = self.project.order_list

        num_rows = len(self.order_list)

        if num_rows > self.num_rows:
            self.AppendRows(numRows = num_rows-self.num_rows)
            self.num_rows = num_rows
        elif num_rows < self.num_rows:
            self.DeleteRows(pos = num_rows, numRows = self.num_rows-num_rows)
            self.num_rows = num_rows

        for row, data in enumerate(self.order_list):
            self.SetRowLabelValue(row, str(data.number))
            self.SetCellValue(row, 0, data.datestamp.strftime("%d %b %Y"))
            self.SetCellValue(row, 1, data.supplier.name)
            self.SetCellValue(row, 2, data.description)

    @button_wrap
    def show_order(self, event):
        "Order has been double - clicked"
        order = self.order_list[event.GetRow()]
        show_order_frame(order)





