####### VATStuff #######
#
# make_report.py  - Makes and displays a report
#
# Version : 0.4
# Date : 20100328
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#


"""Create a report between two dates, and show it as html, with the ability
to export as a CSV or PDF file"""


import csv
import datetime
import os.path

import wx

from vatstuff_packages.vs_database import report_functions

from vatstuff_packages.vs_utilities import *

from vatstuff_packages.vs_gui.gui_functions import *


if vs_cfg["REPORTLABS_AVAILABLE"]:
    import vatstuff_packages.vs_pdf.page_setup
    import vatstuff_packages.vs_pdf.create_report



def make_report_and_frame(report_type, report_title, filename, from_date=None, to_date=None, **kwargs):
    """Given a report type, dates and keywords, call report_functions.create_report to create
       a report object, then create a Report Frame using this report, the report title, and
       filename to give to pdf and csv files generated from the frame"""

    # create the report object
    report = report_functions.create_report(report_type, from_date, to_date, **kwargs)
    if not report:
        error_message("Unable to create the Report")
        return

    # now open a ReportFrame, holding this report

    # The frame may already be open - but too bad, there is
    # no easy way of practically differentiating them, and it doesn't
    # really matter if multiple are open
    # This may be something to improve on in future.

    # top_frame() is the top frame, which will be the parent
    frm = ReportFrame(top_frame(), report_title, report, filename)
    frm.Show()


class ReportFrame(vsFrame):
    "Called to show the report"

    def __init__(self, parent, report_title, report, filename):
        """Create the frame"""
        vsFrame.__init__(self, parent, title=report_title)

        # Create the ReportPanel
        self.report_panel = ReportPanel(self, report, filename)

        # Get the panel size
        panel_size = self.report_panel.GetMinSize()

        # Set minimum size equal to the panel
        self.SetMinSize(panel_size)
        self.SetClientSize(panel_size)

        # update the list of windows which need a refresh if
        # a transaction occurs, by adding self
        vs_cfg["TRANSACTION_REFRESH"].append(self)

        # bind the close event
        self.Bind(wx.EVT_CLOSE, self.onclose)


    def refresh(self):
        "Calls the refresh in the panel"
        self.report_panel.refresh()


    def onclose(self, evt):
        "close the frame"
        # Decrement the refresh list
        vs_cfg["TRANSACTION_REFRESH"].remove(self)
        self.Destroy()


class ReportPanel(wx.Panel):
    """The panel of the Report frame"""

    def __init__(self, parent, report, filename):
        """Create the panel"""
        wx.Panel.__init__(self, parent)
        
        # Take a note of the variables
        self.parent = parent
        self.report = report
        self.filename = filename

        button_size = (100, vs_cfg["BUTTON_HEIGHT"])

        sizer=wx.BoxSizer(wx.VERTICAL)

        self.reportwindow = wx.html.HtmlWindow(self, size=(700,500))
        if "gtk2" in wx.PlatformInfo:
            self.reportwindow.SetStandardFonts()

        self.reportwindow.SetPage(self.report.render_to_html())

        # This widget needs to expand in all directions
        sizer.Add(self.reportwindow, 1, wx.ALL | wx.ALIGN_CENTER | wx.EXPAND, 20)

        # buttons

        button_sizer=wx.BoxSizer(wx.HORIZONTAL)

        # page setup button
        self.page_button = wx.Button(self, label="Page setup", size=button_size)
        self.page_button.SetToolTipString("Click to set up the pdf page size and orientation")   
        button_sizer.Add(self.page_button, 0, wx.RIGHT, 20)

        # pdf button
        self.pdf_button = wx.Button(self, label="Create pdf", size=button_size)
        self.pdf_button.SetToolTipString("Click to create a pdf file of the report.")
        button_sizer.Add(self.pdf_button, 0, wx.RIGHT, 20)

        # csv button
        self.csv_button = wx.Button(self, label="CSV File", size=button_size)
        self.csv_button.SetToolTipString("Click to export to a csv file.")
        button_sizer.Add(self.csv_button, 0, wx.RIGHT, 20)

        # Close button
        self.close_button = wx.Button(self, wx.ID_CLOSE, size=button_size)
        self.close_button.SetToolTipString("Click to close.")
        button_sizer.Add(self.close_button, 0)

        sizer.Add(button_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)
      
        # Set the size to the panel
        self.SetSizer(sizer)
        sizer.SetSizeHints(self)

        # ---------- Bind buttons to handler functions
        
        self.Bind(wx.EVT_BUTTON, self.page_button_pressed, self.page_button)
        self.Bind(wx.EVT_BUTTON, self.pdf_button_pressed, self.pdf_button)
        self.Bind(wx.EVT_BUTTON, self.csv_button_pressed, self.csv_button)
        self.Bind(wx.EVT_BUTTON, self.close_button_pressed, self.close_button)


    @button_wrap
    def page_button_pressed(self, event):
        "page button pressed, setup the pdf page size and orientation"
        vatstuff_packages.vs_pdf.page_setup.page_setup_dialog(self)


    @button_wrap
    def pdf_button_pressed(self, event):
        "Create a pdf file from the report"

        if not vs_cfg["REPORTLABS_AVAILABLE"]:
            # no pdf capability
            vs_message(self, "Function unavailable - the reportlab module is not installed",
                             "ERROR")
            return

        # default filename
        filename= "%s.pdf" % self.filename
        if not vatstuff_packages.vs_pdf.create_report.create_pdf(self, filename, self.report):
            vs_message(self, "Unable to create the pdf file", "ERROR")


    @button_wrap
    def csv_button_pressed(self, event):
        """Export the report to a csv file"""

        # default filename
        filename= "%s.csv" % self.filename

        fp=None
        try:
            # Create the save file dialog.
            status, path = save_file_as(self, 'csv', vs_cfg["EXPORT_FOLDER"], filename)
            if not status:
                return

            # open the csv file
            fp=open(path,"wb")
            # save file in csv format
            writer = csv.writer(fp, dialect="excel")
            for row in self.report.render_to_csv():
                writer.writerow(row)
        finally:
            if fp: fp.close()

        # save the directory to the database and to vs_cfg["EXPORT_FOLDER"]
        directory = os.path.dirname(path)
        if directory != vs_cfg["EXPORT_FOLDER"]:
            set_directory("EXPORT_FOLDER", directory)
        info_log("CSV file created: %s" % path)
        vs_message(self, "Your file has been saved", "Saved")


    def close_button_pressed(self, event):
        """Close button - Close parent frame"""
        self.parent.Close()


    def refresh(self):
        """The refresh function disables the buttons - and shows a no_reports
           as this assumes a database change, and hence the report may be incorrect"""
        self.page_button.Enable(False)
        self.pdf_button.Enable(False)
        self.csv_button.Enable(False)
        self.reportwindow.SetPage(no_report())

