####### VATStuff #######
#
# view_transactions.py  - lists all the company transactions
#
# Version : 0.6
# Date : 20100716
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#

""" This is a module in the vatstuff_packages.vs_gui package, and lists the transactions

It is run as part of the vatstuff gui interface.
"""

import datetime
import decimal
import sqlite3
import csv

import wx
import wx.grid
import wx.html

from vatstuff_packages.vs_database import change_transaction_date


from vatstuff_packages.vs_database.grid_objects import *


from vatstuff_packages.vs_database import report_functions
from vatstuff_packages.vs_utilities import *

from vatstuff_packages.vs_gui.gui_functions import *
from vatstuff_packages.vs_gui.sales.view_invoices import show_invoice_frame, show_customer_payment_dialog
from vatstuff_packages.vs_gui.sales.view_creditnotes import show_creditnote_frame
from vatstuff_packages.vs_gui.purchases.view_orders import (show_order_frame,
                                          show_supplier_creditnote_dialog,
                                          show_supplier_invoice_and_order_dialog)


if vs_cfg["REPORTLABS_AVAILABLE"]:
    import vatstuff_packages.vs_pdf.page_setup
    import vatstuff_packages.vs_pdf.create_report

D = decimal.Decimal   # D used to avoid typing


def view_all_transactions_frame(transaction=None):
    """Show the ViewAllTransactionsFrame which lists all transactions,
       ensuring that the transaction given as argument is displayed.
       If None is given, display the last entry"""

    # No try:exception statement - allows the caller to catch exceptions

    # Check the frame is not already open
    for frm in vs_cfg["TRANSACTION_REFRESH"]:
        if isinstance(frm, ViewAllTransactionsFrame):
            frm.Raise()
            break
    else:
        # top_frame() is the top frame, which will be the parent
        frm = ViewAllTransactionsFrame(top_frame())
        frm.Show()

    if transaction:
        frm.display_row(transaction.gridrow)
    else:
        frm.display_last()


class ViewAllTransactionsFrame(vsFrame):
    """Frame to list all transactions"""

    def __init__(self, parent):
        """Create the frame"""
        vsFrame.__init__(self, parent, title="Transactions")

        # Take a note of the variables
        self.parent = parent

        self.transactions_grid = TransactionGrid()
        # class defined in vatstuff_packages.vs_database.grid_objects
        # Create a database object of type wx.grid.PyGridTableBase
        # it opens database connections, and gets information for
        # the grid object

        try:
            self.transactions_grid_rows = self.transactions_grid.GetNumberRows()
            # Create the ViewAllTransactionsPanel
            self.view_all_transactions_panel = ViewAllTransactionsPanel(self, self.transactions_grid)
        except Exception:
            # close database connection, and re-raise the exception
            self.transactions_grid.CloseConnection()
            raise

        # Get the panel size
        panel_size = self.view_all_transactions_panel.GetMinSize()

        # Set minimum size equal to the panel
        self.SetMinSize(panel_size)
        self.SetClientSize(panel_size)

        # update the list of windows which need a refresh if
        # a transaction occurs, by adding self
        vs_cfg["TRANSACTION_REFRESH"].append(self)

        self.Bind(wx.EVT_CLOSE, self.onclose)


    def refresh(self):
        "Refreshes the grid"
        new_num_rows = self.transactions_grid.GetNumberRows()
        if new_num_rows > self.transactions_grid_rows:
            # More rows are in the database, insert new rows on the grid
            msg = wx.grid.GridTableMessage(self.transactions_grid,          # The table
                  wx.grid.GRIDTABLE_NOTIFY_ROWS_APPENDED,                   # has extra rows
                  new_num_rows - self.transactions_grid_rows                # the number of rows
                  ) 
            self.view_all_transactions_panel.grid.ProcessTableMessage(msg)  # The grid associated with the table
            self.transactions_grid_rows = new_num_rows                      # record the new number of rows
        elif new_num_rows < self.transactions_grid_rows:
            # Less rows are in the database, delete rows on the grid
            msg = wx.grid.GridTableMessage(self.transactions_grid,          # The table
                  wx.grid.GRIDTABLE_NOTIFY_ROWS_DELETED,                    # has fewer rows
                  new_num_rows,                                             # delete from row
                  self.transactions_grid_rows - new_num_rows                # delete the number of rows
                  ) 
            self.view_all_transactions_panel.grid.ProcessTableMessage(msg)  # The grid associated with the table
            self.transactions_grid_rows = new_num_rows                      # record the new number of rows

        # grid associated with table can also be obtained with
        # self.transactions_grid.GetView().ProcessTableMessage(msg)

        self.view_all_transactions_panel.grid.ForceRefresh()


    def display_row(self, row):
        "Calls the display_row function in the view_all_transactions_panel"
        self.view_all_transactions_panel.display_row(row)


    def display_last(self):
        "Calls the display_last function in the view_transactions panel"
        self.view_all_transactions_panel.display_last()

    def onclose(self, evt):
        "Shutdown the database transaction"
        self.transactions_grid.CloseConnection()
        # Decrement the refresh list
        vs_cfg["TRANSACTION_REFRESH"].remove(self)
        # and destroy the frame
        self.Destroy()


# ----------------------ViewAllTransactionsPanel


class ViewAllTransactionsPanel(wx.Panel):
    """The panel of the ViewAllTransactionsFrame"""

    def __init__(self, parent, transactions_grid):
        wx.Panel.__init__(self, parent)

        # Take a note of the parent, and the transactions_grid
        self.parent = parent
        self.transactions_grid = transactions_grid

        button_size = (100, vs_cfg["BUTTON_HEIGHT"])
             
        # Create the sizer for the panel
        sizer=wx.BoxSizer(wx.VERTICAL)

        help_text=wx.StaticText(self, -1, "Double click on any line to view the transaction details")
        sizer.Add(help_text, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # Create a grid
        self.grid = wx.grid.Grid(self, size=(750,200))

        self.grid.SetDefaultColSize(100, True)
        self.grid.SetDefaultRowSize(vs_cfg["BUTTON_HEIGHT"], False)
        self.grid.SetColMinimalAcceptableWidth(100)
        self.grid.EnableEditing(False)

        self.grid.SetDefaultCellAlignment(wx.ALIGN_LEFT, wx.ALIGN_CENTER)

        # Assign the transactions_grid to the grid
        self.grid.SetTable(self.transactions_grid, True)

        # Set the description column size
        self.grid.SetColSize(1, 500)

        sizer.Add(self.grid, 1, wx.ALL | wx.ALIGN_CENTER | wx.EXPAND, 20)

        # buttons
        button_sizer=wx.BoxSizer(wx.HORIZONTAL)

        # ------------------- Close button
        self.close_button = wx.Button(self, wx.ID_CLOSE, size=button_size)
        self.close_button.SetToolTipString("Click to close this window")
        button_sizer.Add(self.close_button, 0, wx.RIGHT, 20)

        # ------------------- Report button
        self.report_button = wx.Button(self, label="Report", size=button_size)
        self.report_button.SetToolTipString("Click to generate a report between two dates")
        button_sizer.Add(self.report_button, 0)

        sizer.Add(button_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # Set the size to the panel
        self.SetSizer(sizer)
        sizer.SetSizeHints(self)

        # Bind left mouse double - click
        self.Bind(wx.grid.EVT_GRID_CELL_LEFT_DCLICK, self.show_this_transaction, self.grid)

        # Bind the buttons
        self.Bind(wx.EVT_BUTTON, self.close_button_pressed, self.close_button)
        self.Bind(wx.EVT_BUTTON, self.report_button_pressed, self.report_button)

    def close_button_pressed(self, event):
        """Close parent"""
        self.parent.Close()


    @button_wrap
    def report_button_pressed(self, event):
        """Get the two dates between which the report is generated and show TransactionsReportFrame"""

        help_text = """\
A report will be generated showing
all transactions between the two dates:"""

        status, from_date, to_date = two_dates(self, "Transactions Report", help_text)
        if not status:
            return
        busy_cursor_on()
        wx.Yield()
        reports = report_functions.create_report("transactions", from_date, to_date)
        # Note reports is plural, as it consists of a tuple of two reports - (report, csv_report)
        # a standard report suitable for html and pdf generation, and csv_reports suitable for
        # generating a csv file
        if not reports:
            error_message("Unable to create the Report")
            return
        filename = "transactions%s" % to_date.isoformat()
        # top_frame() is the top frame, which will be the parent
        transactions_report_frame = TransactionsReportFrame(top_frame(), reports, filename)
        transactions_report_frame.Show()


    def display_row(self, row):
        """Makes row visible"""
        last_row = self.transactions_grid.GetNumberRows() - 1
        if row > last_row:
            row = last_row
        if row < 0:
            row = 0
        self.grid.SetGridCursor(row, 0)
        self.grid.MakeCellVisible(row, 0)


    def display_last(self):
        """ Makes the last row visible """
        last_row = self.transactions_grid.GetNumberRows() - 1
        self.grid.SetGridCursor(last_row, 0)
        self.grid.MakeCellVisible(last_row, 0)


    @button_wrap
    def show_this_transaction(self, event):
        "On being double clicked, obtain transaction and show it"
        row = event.GetRow()
        # get the transaction for this row
        transaction = Transaction(int(self.transactions_grid.GetRowLabelValue(row)))
        # Show the transaction in a dialog
        # if this is a vat correction, use the dialog in gui_functions
        if transaction.system == 11:
            show_transaction_dialog(self, transaction)
        else:
            show_transaction_with_view_button(self, transaction)


# -------- TransactionsReportFrame - displays the report, and allows pdf and csv generation
#          make_report.ReportFrame is not used, as there is some differences:
#          This takes two reports, one for display and pdf, and the second for csv generation 

class TransactionsReportFrame(vsFrame):
    """Called when the transactions report button is pressed, to display the frame to
       show the transactions between two dates"""

    def __init__(self, parent, reports, filename):
        """Create the frame"""
        vsFrame.__init__(self, parent, title="Transactions")

        # Create the TransactionsReportPanel
        self.transactions_report_panel = TransactionsReportPanel(self, reports, filename)

        # Get the panel size
        panel_size = self.transactions_report_panel.GetMinSize()

        # Set minimum size equal to the panel
        self.SetMinSize(panel_size)
        self.SetClientSize(panel_size)

        # update the list of windows which need a refresh if
        # a transaction occurs, by adding self
        vs_cfg["TRANSACTION_REFRESH"].append(self)

        # bind the close event
        self.Bind(wx.EVT_CLOSE, self.onclose)


    def refresh(self):
        "Calls the refresh in the panel"
        self.transactions_report_panel.refresh()


    def onclose(self, evt):
        "close the frame"
        # Decrement the refresh list
        vs_cfg["TRANSACTION_REFRESH"].remove(self)
        self.Destroy()


class TransactionsReportPanel(wx.Panel):
    """The panel of the TransactionsReport frame"""

    def __init__(self, parent, reports, filename):
        """Create the panel"""
        wx.Panel.__init__(self, parent, size= (800,600))
        
        # Take a note of the variables
        self.parent = parent
        self.report = reports[0]
        self.csv_report = reports[1]
        self.filename = filename
        button_size = (100, vs_cfg["BUTTON_HEIGHT"])

        self.sizer=wx.BoxSizer(wx.VERTICAL)

        self.reportwindow = wx.html.HtmlWindow(self, size=(700,500))
        if "gtk2" in wx.PlatformInfo:
            self.reportwindow.SetStandardFonts()

        self.reportwindow.SetPage(self.report.render_to_html())

        # This widget needs to expand in all directions
        self.sizer.Add(self.reportwindow, 1, wx.ALL | wx.ALIGN_CENTER | wx.EXPAND, 20)

        # buttons

        button_sizer=wx.BoxSizer(wx.HORIZONTAL)

        # page setup button
        self.page_button = wx.Button(self, label="Page setup", size=button_size)
        self.page_button.SetToolTipString("Click to set up the pdf page size and orientation")   
        button_sizer.Add(self.page_button, 0, wx.RIGHT, 20)

        # pdf button
        self.pdf_button = wx.Button(self, label="Create pdf", size=button_size)
        self.pdf_button.SetToolTipString("Click to create a pdf file of the order form.")
        button_sizer.Add(self.pdf_button, 0, wx.RIGHT, 20)

        # csv button
        self.csv_button = wx.Button(self, label="CSV File", size=button_size)
        self.csv_button.SetToolTipString("Click to export to a csv file.")
        button_sizer.Add(self.csv_button, 0, wx.RIGHT, 20)

        # Close button
        self.close_button = wx.Button(self, wx.ID_CLOSE, size=button_size)
        self.close_button.SetToolTipString("Click to close.")
        button_sizer.Add(self.close_button, 0)


        self.sizer.Add(button_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)
      
        # Set the size to the panel
        self.SetSizer(self.sizer)
        self.sizer.SetSizeHints(self)

        # ---------- Bind buttons to handler functions
        
        self.Bind(wx.EVT_BUTTON, self.page_button_pressed, self.page_button)
        self.Bind(wx.EVT_BUTTON, self.pdf_button_pressed, self.pdf_button)
        self.Bind(wx.EVT_BUTTON, self.csv_button_pressed, self.csv_button)
        self.Bind(wx.EVT_BUTTON, self.close_button_pressed, self.close_button)


    @button_wrap
    def page_button_pressed(self, event):
        "page button pressed, setup the pdf page size and orientation"
        vatstuff_packages.vs_pdf.page_setup.page_setup_dialog(self)


    @button_wrap
    def pdf_button_pressed(self, event):
        "Create a pdf file from the report"
        if not vs_cfg["REPORTLABS_AVAILABLE"]:
            # no pdf capability
            vs_message(self, "Function unavailable - the reportlab module is not installed",
                             "ERROR")
            return

        # default filename
        filename= self.filename + ".pdf"
        if not vatstuff_packages.vs_pdf.create_report.create_pdf(self, filename, self.report):
            vs_message(self, "Unable to create the pdf file", "ERROR")


    @button_wrap
    def csv_button_pressed(self, event):
        """Export the report to a csv file"""

        # default filename
        filename= self.filename + ".csv"

        fp=None
        try:
            # Create the save file dialog.
            status, path = save_file_as(self, 'csv', vs_cfg["EXPORT_FOLDER"], filename)
            if not status:
                return

            # open the csv file
            fp=open(path,"wb")
            # save file in csv format
            writer = csv.writer(fp, dialect="excel")
            for row in self.csv_report.render_to_csv():
                writer.writerow(row)
        finally:
            if fp: fp.close()

        # save the directory to the database and to vs_cfg["EXPORT_FOLDER"]
        directory = os.path.dirname(path)
        if directory != vs_cfg["EXPORT_FOLDER"]:
            set_directory("EXPORT_FOLDER", directory)
        info_log("CSV file created: %s" % path)
        vs_message(self, "Your file has been saved", "Saved")


    def close_button_pressed(self, event):
        """Close button - Close parent frame"""
        self.parent.Close()
        return

    def refresh(self):
        """The refresh function disables the buttons - and shows a no_reports
           as this assumes a database change, and hence the report may be incorrect"""
        self.page_button.Enable(False)
        self.pdf_button.Enable(False)
        self.csv_button.Enable(False)
        self.reportwindow.SetPage(no_report())



####################################################################

# The show_transaction_with_view_button function displays the ShowTransactionViewButtonDialog
# and obtains an associated transaction object if the view button has been pressed - it then
# calls the appropriate frame/dialog to view the requested object

def show_transaction_with_view_button(parent, transaction):
    """Creates a ShowTransactionViewButtonDialog, with a view button to
       view associated invoice, customer payment, order or credit note objects
       (if such ojects are associated with this transaction)"""
    show_dialog = ShowTransactionViewButtonDialog(parent, transaction)

    # If the cursor is busy, unbusy it - as now waiting for a button press
    busy_cursor_off()

    if show_dialog.ShowModal() != wx.ID_OK:
        show_dialog.Destroy()
        return

    # Get the object to view
    object_string, object_to_view = show_dialog.call_frame
    show_dialog.Destroy()

    if (not object_string) or (not object_to_view):
        # if neither are defined, return
        return

    # The view button in the ShowTransactionDialog has been pressed
    # the object_string indicates another window is to be openned and describes
    # the type of object
    # and object_to_view is the object to be viewed in the new window
    busy_cursor_on()
    wx.Yield()

    if object_string == 'suppliercreditnote':
        show_supplier_creditnote_dialog(parent, object_to_view)
        return

    if object_string == 'supplierinvoice':
        show_supplier_invoice_and_order_dialog(parent, object_to_view)
        return

    if object_string == "payment":
        show_customer_payment_dialog(parent, object_to_view)
        return

    if object_string == "order":
        show_order_frame(object_to_view)
        return

    if object_string == "invoice":
        show_invoice_frame(object_to_view)
        return

    if object_string == "creditnote":
        show_creditnote_frame(object_to_view)
        return

    raise VS_ERROR, "object string not recognized"


class ShowTransactionViewButtonDialog(vsDialog):
    """Called to show the transaction"""

    def __init__(self, parent, transaction):
        """Create the Dialog showing the transaction.
           If this transaction is associated with an invoice, customer payment,
           order or credit note, show a button to display the associated object"""
        vsDialog.__init__(self, parent, title="Transaction : %i" % transaction.id)
        
        # Take a note of the variables
        self.transaction = transaction

        # depending on the transaction type a 'view' button will be made available
        # allowing the user to view the invoice, order or creditinote associated
        # with this transaction
        
        # self.call_frame is a list of two elements, The first is a string of either
        # "invoice", "order", "payment" or "creditnote" and the second element is the actual
        # invoice, order, payment or creditnote to be displayed if the view button is pressed
        self.call_frame = [None, None]
        # These are displayed by the button, depending on the item to be displayed
        # initially set them to display an order
        view_display = "View associated Order"
        view_tool = "Press to view the order details associated with this transaction"

        button_size = (100, vs_cfg["BUTTON_HEIGHT"])

        # Create a vertical sizer to hold everything
        self.sizer=wx.BoxSizer(wx.VERTICAL)

        # Find transaction type
        transaction_system = transaction.system

        # If a button cannot be shown, due to data deletion, or no associated
        # thing to show, set this variable to False
        self.show_button = True

        # Set self.call_frame and the button displays
        if transaction_system == 2:       # A purchase order transaction
            self.call_frame = ['order', transaction.get_object()]

        elif transaction_system == 3:     # Pay a supplier invoice transaction
            makepayment = transaction.get_object()
            makepayment_purchaseorder = makepayment.purchaseorder
            if makepayment_purchaseorder:
                self.call_frame = ['order', makepayment_purchaseorder]
            else:
                self.show_button = False

        elif transaction_system == 4:     # Supplier credit note transaction
            suppliercreditnote = transaction.get_object()
            suppliercreditnote_purchaseorder = suppliercreditnote.purchaseorder
            if suppliercreditnote_purchaseorder:
                self.call_frame = ['order', suppliercreditnote_purchaseorder]
            else:
                self.call_frame = ['suppliercreditnote', suppliercreditnote]
                view_display = "View associated Credit Note"
                view_tool = "Press to view the credit note details associated with this transaction"

        elif transaction_system == 8:     # An order close transaction
            closedorder = transaction.get_object()
            closedorder_purchaseorder = closedorder.purchaseorder
            if closedorder_purchaseorder:
                self.call_frame = ['order', closedorder_purchaseorder]
            else:
                self.show_button = False

        elif transaction_system == 9:     # Supplier invoice received transaction
            self.call_frame = ['supplierinvoice', transaction.get_object()]
            view_display = "View associated Invoice"
            view_tool = "Press to view the invoice details associated with this transaction"

        elif transaction_system == 5:     # An invoice transaction
            self.call_frame = ['invoice', transaction.get_object()]
            view_display = "View associated Invoice"
            view_tool = "Press to view the invoice details associated with this transaction"

        elif transaction_system == 6:     # Customer payment transaction
            customer_payment = transaction.get_object()
            if customer_payment.data_deleted():
                # Data associated with this payment has been deleted - therefore cannot show it
                self.show_button = False
            else:
                self.call_frame = ['payment', customer_payment]
                view_display = "View associated payment"
                view_tool = "Press to view the payment details associated with this transaction"

        elif transaction_system == 7:     # A credit note transaction
            creditnote_to_customer = transaction.get_object()
            if creditnote_to_customer.data_deleted():
                # Data associated with this creditnote has been deleted - therefore cannot show it
                self.show_button = False
            else:
                self.call_frame = ['creditnote', creditnote_to_customer]
                view_display = "View associated Credit Note"
                view_tool = "Press to view the credit note details associated with this transaction"

        else:
            # None of the above, so do not show the view button
            self.show_button = False


        # field sizer holds the description and date fields and buttons
        field_sizer=wx.FlexGridSizer(rows=2, cols=2, hgap=20, vgap=10)

        # transaction memo
        self.memo_field = wx.TextCtrl(self, -1, self.transaction.description, size=(300, vs_cfg["BUTTON_HEIGHT"]))
        self.memo_field.SetToolTipString("Edit the transaction description if you wish")
        self.memo_field.SetMaxLength(50)
        field_sizer.Add(self.memo_field, 0, wx.ALIGN_RIGHT)

        self.memo_button = wx.Button(self, label="Change description", size=(200, vs_cfg["BUTTON_HEIGHT"]))
        self.memo_button.SetToolTipString("Click to change the description")
        field_sizer.Add(self.memo_button, 0, wx.ALIGN_LEFT)

        # date
        transaction_date = self.transaction.datestamp
        wx_transaction_date = date_to_wxdate(transaction_date)
        self.date_field = wx.DatePickerCtrl(self, -1,
                                            wx_transaction_date, 
                                            size=(200, vs_cfg["BUTTON_HEIGHT"]),
                                            style=wx.DP_DROPDOWN | wx.DP_SHOWCENTURY)
        self.date_field.SetToolTipString("The transaction date")
        field_sizer.Add(self.date_field, 0, wx.ALIGN_RIGHT)

        self.date_button = wx.Button(self, label="Change date", size=(200, vs_cfg["BUTTON_HEIGHT"]))
        self.date_button.SetToolTipString("Click to change the transaction date")
        field_sizer.Add(self.date_button, 0, wx.ALIGN_LEFT)

        if transaction_system == 12:
            # data deletion transaction - date cannot be changed
            self.date_field.Enable(False)
            self.date_button.Enable(False)

        self.sizer.Add(field_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # static text date:
        self.text_date = wx.StaticText(self, -1, transaction_date.strftime("Transaction date : %d %B %Y"))
        self.sizer.Add(self.text_date, 0, wx.LEFT | wx.ALIGN_LEFT, 20)

        # ---------------------- Create the transaction table  (from gui_functions)

        self.transaction_table = TransactionDialogTable(self, transaction)
        self.sizer.Add(self.transaction_table, 1, wx.ALL | wx.ALIGN_CENTER | wx.EXPAND, 20)

        # Create a horizontal button_sizer to hold the buttons
        self.button_sizer=wx.BoxSizer(wx.HORIZONTAL)

        # ------------------- Close button
        self.close_button = wx.Button(self, wx.ID_CLOSE, size=button_size)
        self.close_button.SetToolTipString("Click to close this window")
        if self.show_button:
            self.button_sizer.Add(self.close_button, 0, wx.RIGHT, 40)
        else:
            self.button_sizer.Add(self.close_button, 0)


        # ------------------- View button, only displayed if self.show_button is True
        if self.show_button:
            self.view_button = wx.Button(self, label=view_display, size=(300, vs_cfg["BUTTON_HEIGHT"]))
            self.view_button.SetToolTipString(view_tool)
            self.button_sizer.Add(self.view_button, 0)

        # Add button_sizer to the main sizer
        self.sizer.Add(self.button_sizer, 0, wx.BOTTOM | wx.ALIGN_CENTER, 20)

        # Set the size to the panel
        self.SetSizer(self.sizer)
        self.sizer.SetSizeHints(self)

        self.Bind(wx.EVT_BUTTON, self.memo_button_pressed, self.memo_button)
        self.Bind(wx.EVT_BUTTON, self.date_button_pressed, self.date_button)
        self.Bind(wx.EVT_BUTTON, self.close_button_pressed, self.close_button)
        if self.show_button:
            self.Bind(wx.EVT_BUTTON, self.view_button_pressed, self.view_button)


    @writecheck
    @button_wrap
    def memo_button_pressed(self, event):
        """change the transaction description"""
        description = self.memo_field.GetValue().strip()
        if self.transaction.description == description:
            # No change in the description - just return
            vs_message(self, "There was no change to the description", "WARNING")
            return
        self.transaction.description = description

        # refresh windows that need it, but not the accounts tree, as the discription
        # does not appear there, nor in the panels
        big_refresh(acc_tree=False, panels=False)
        info_log("Transaction %i description changed" % self.transaction.id)
        vs_message(self, "Description changed", "DONE")


    @writecheck
    @button_wrap
    def date_button_pressed(self, event):
        """change the transaction date"""
        try:
            transaction_date = wxdate_to_date(self.date_field.GetValue())
        except Exception, e:
            vs_message(self, "The date is not valid", "ERROR")
            return
        if transaction_date == self.transaction.datestamp:
            # No change in the date - just return
            vs_message(self, "There was no change to the date", "WARNING")
            return
        # the change_transaction_date.check_date checks if the date is ok, returns True, message
        # if it is ok, or False, message if not
        status, message = change_transaction_date.check_date(self.transaction, transaction_date)
        if status:
            if not vs_yesno(self, message, "Confirm date change"):
                return
        else:
            vs_message(self, message, "ERROR")
            return
        # change_transaction_date.change_date does the date change, returns True if ok
        # or False if not
        busy_cursor_on()
        wx.Yield()
        status = change_transaction_date.change_date(self.transaction, transaction_date)
        if not status:
            vs_message(self, "Unable to implement the date change", "ERROR")
            return
        self.text_date.SetLabel(self.transaction.datestamp.strftime("Transaction date : %d %B %Y"))
        self.sizer.Layout()
        wx.Yield()
        # refresh windows that need it, but not the accounts tree, as the date
        # does not appear there, nor in the panels
        big_refresh(acc_tree=False, panels=False)
        vs_message(self, "Date changed", "DONE")


    def view_button_pressed(self, event):
        """Close the dialog"""
        self.EndModal(wx.ID_OK)


    def close_button_pressed(self, event):
        """Close the dialog"""
        self.EndModal(wx.ID_CANCEL)

