####### VATStuff #######
#
# make_creditnote.py  - creates a credit note
#
# Version : 0.7.0
# Date : 20100925
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#


"""Creates two classes to create a Credit Note

NewCreditNote creates a creditnote associated with a previously created invoice
PaymentCreditNote creates a creditnote accompanied by a payment
"""

import os
import decimal
import datetime

import wx
import wx.grid

from vatstuff_packages.vs_database import save_creditnote

from vatstuff_packages.vs_gui.address_panel import *
from vatstuff_packages.vs_gui.gui_functions import *
from vatstuff_packages.vs_gui.items_table import ItemsTable

from vatstuff_packages.vs_verify import form_objects
from vatstuff_packages.vs_verify.sales import new_creditnote_check

from vatstuff_packages.vs_utilities import *

if "REPORTLABS_AVAILABLE" in vs_cfg and vs_cfg["REPORTLABS_AVAILABLE"]:
    import vatstuff_packages.vs_pdf.page_setup
    import vatstuff_packages.vs_pdf.creditnote_form
    import vatstuff_packages.vs_pdf.pdf_functions

D = decimal.Decimal   # D used to avoid typing


def new_credit_note_dialog(parent, invoice):
    """Creates a creditnote against an invoice by calling NewCreditNoteDialog"""
    new_credit_note_dialog = NewCreditNoteDialog(parent, invoice)

    # If the cursor is busy, unbusy it - as
    # now waiting for a button press
    busy_cursor_off()

    new_credit_note_dialog.ShowModal()
    new_credit_note_dialog.Destroy()


class NewCreditNoteDialog(vsDialog):
    "Create a creditnote associated with a previously created invoice"

    def __init__(self, parent, invoice):
        """Create the Dialog"""
        vsDialog.__init__(self, parent, "Credit Note")
        
        # Take a note of the variable
        self.invoice = invoice

        # Store all form contents in a CreditNoteForm object
        self.creditnote = form_objects.CreditNoteForm()
        self.creditnote.invoice = self.invoice
        self.creditnote.status = 0

        # STATUS - normally 0 when a creditnote is sent against an invoice
        # STATUS - 1 when a creditnote is sent with a payment

        # invoice.customer is the customer who is being invoiced, but the customer may have
        # had details changed since the invoice was sent, so get the visible customer
        # with invoice.customer.number
        self.creditnote.customer = Customer.from_number(self.invoice.customer.number)

        # Can you charge VAT?
        self.show_vat = self.creditnote.customer.charge_vat()
        self.creditnote.show_vat = self.show_vat

        button_size = (100, vs_cfg["BUTTON_HEIGHT"])

        # Create a vertical sizer to hold everything
        self.sizer=wx.BoxSizer(wx.VERTICAL)

        # And a horizontal sizer to hold address and invoice details
        toprow_sizer = wx.BoxSizer(wx.HORIZONTAL)

        # Address sizer
        add_sizer = wx.BoxSizer(wx.VERTICAL)

        customer_text = "Customer: %s" % (self.creditnote.customer.name)
        add_sizer.Add(wx.StaticText(self, -1, customer_text), 0, wx.LEFT | wx.TOP | wx.RIGHT, 10)

        self.customer_address = AddressSpin(self, self.creditnote.customer, invoice_point=1)
        add_sizer.Add(self.customer_address, 0, wx.ALL, 10)

        toprow_sizer.Add(add_sizer, 0, wx.RIGHT | wx.ALIGN_TOP, 40)

        # numbers sizer
        num_sizer = wx.BoxSizer(wx.VERTICAL)

        # Customer order number
        if self.invoice.cus_order_number:
            order_text = "Customer Order Number:\n%s" % self.invoice.cus_order_number
            num_sizer.Add(wx.StaticText(self, -1, order_text), 0, wx.ALL, 10)

        # Credit Note number
        num_sizer.Add(wx.StaticText(self, -1, "Credit Note Number:\n(Automatically increments)"),
                                              0, wx.LEFT | wx.RIGHT | wx.TOP, 10)

        self.last_creditnote_number = get_config_item("LAST_CREDITNOTE_NUMBER")
        if not self.last_creditnote_number:
            self.last_creditnote_number = 0
            creditnote_number = 1
        else:
            self.last_creditnote_number = int(self.last_creditnote_number)
            creditnote_number =  self.last_creditnote_number + 1

        self.creditnote_number_field = wx.TextCtrl(self, -1, str(creditnote_number), size=(250, vs_cfg["BUTTON_HEIGHT"]))
        self.creditnote_number_field.SetToolTipString("Set your credit note number")
        self.creditnote_number_field.SetMaxLength(30)
        num_sizer.Add(self.creditnote_number_field, 0, wx.LEFT | wx.BOTTOM | wx.RIGHT, 10)

        # Create date field in a horizontal sizer
        date_sizer=wx.BoxSizer(wx.HORIZONTAL)

        date_sizer.Add(wx.StaticText(self, -1, "Date:"), 0, wx.RIGHT | wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL, 10)

        self.creditnote_date_field = wx.DatePickerCtrl(self, -1,
                                                  wx.DateTime.Now(), 
                                                  size=(150, vs_cfg["BUTTON_HEIGHT"]),
                                                  style=wx.DP_DROPDOWN | wx.DP_SHOWCENTURY)
        self.creditnote_date_field.SetToolTipString("The date of the credit note")
        date_sizer.Add(self.creditnote_date_field, 0, wx.ALIGN_LEFT)

        num_sizer.Add(date_sizer, 0, wx.ALL, 10)

        toprow_sizer.Add(num_sizer, 0, wx.RIGHT | wx.ALIGN_TOP, 60)

        # invoice_sizer holds invoice data
        invoice_sizer = wx.BoxSizer(wx.VERTICAL)

        # Invoice number and date
        date_text = self.invoice.datestamp.strftime("%d %b %Y")
        invoice_text = "Invoice Number: %i\nInvoice Date: %s" % (self.invoice.number, date_text)
        invoice_sizer.Add(wx.StaticText(self, -1, invoice_text), 0, wx.BOTTOM | wx.ALIGN_LEFT, 10)

        # Put costs in a grid sizer
        cost_sizer=wx.FlexGridSizer(rows=4, cols=2, hgap=20, vgap=10)

        cost_string = unicode(Currency(self.invoice.cost))
        vat_string = unicode(Currency(self.invoice.vat))
        self.total = self.invoice.vat + self.invoice.cost
        total_string = unicode(Currency(self.total))
        outstanding_string = unicode(Currency(self.invoice.outstanding))

        cost_sizer.Add(wx.StaticText(self, -1, "Cost"), 0, wx.ALIGN_RIGHT)
        cost_sizer.Add(wx.StaticText(self, -1, cost_string), 0, wx.ALIGN_RIGHT)
        cost_sizer.Add(wx.StaticText(self, -1, "VAT"), 0, wx.ALIGN_RIGHT)
        cost_sizer.Add(wx.StaticText(self, -1, vat_string), 0, wx.ALIGN_RIGHT)
        cost_sizer.Add(wx.StaticText(self, -1, "Invoice Total"), 0, wx.ALIGN_RIGHT)
        cost_sizer.Add(wx.StaticText(self, -1, total_string), 0, wx.ALIGN_RIGHT)
        cost_sizer.Add(wx.StaticText(self, -1, "Outstanding"), 0, wx.ALIGN_RIGHT)
        cost_sizer.Add(wx.StaticText(self, -1, outstanding_string), 0, wx.ALIGN_RIGHT)

        invoice_sizer.Add(cost_sizer, 0, wx.ALIGN_TOP)

        toprow_sizer.Add(invoice_sizer, 0, wx.ALL | wx.ALIGN_TOP, 10)

        self.sizer.Add(toprow_sizer, 0, wx.TOP | wx.LEFT | wx.RIGHT | wx.ALIGN_CENTER, 10)

        # --- reference details in a grid sizer

        ref_sizer=wx.FlexGridSizer(rows=2, cols=3, hgap=20, vgap=5)

        # First row
        ref_sizer.Add((10,10), 1)
        ref_sizer.Add(wx.StaticText(self, -1, "Reason for credit note (text shown on the pdf):"), 0, wx.ALIGN_LEFT)
        ref_sizer.Add(wx.StaticText(self, -1, "In case of enquiry please contact:"), 0, wx.ALIGN_LEFT)

        # Second row

        # Discount box
        discount_sizer=wx.BoxSizer(wx.HORIZONTAL)
        discount_sizer.Add(wx.StaticText(self, -1, "Discount rate:"), 0, wx.RIGHT | wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL, 5)
        discount_rate_text = str(self.invoice.customer.discount).replace(u".", vs_cfg["CP"])
        self.discount_rate_field = wx.TextCtrl(self, -1, discount_rate_text, style=wx.ALIGN_RIGHT, size=(60, vs_cfg["BUTTON_HEIGHT"]))
        self.discount_rate_field.SetToolTipString("Set the percentage discount rate")
        self.discount_rate_field.SetMaxLength(4)
        discount_sizer.Add(self.discount_rate_field, 0, wx.ALIGN_RIGHT)
        discount_sizer.Add(wx.StaticText(self, -1, "%"), 0, wx.ALIGN_CENTER_VERTICAL)
        ref_sizer.Add(discount_sizer,0, wx.ALIGN_RIGHT)

        # Reason for credit note
        self.creditnote_reason_field = wx.TextCtrl(self, -1, "", size=(350, vs_cfg["BUTTON_HEIGHT"]))
        self.creditnote_reason_field.SetToolTipString("Reason for credit note")
        self.creditnote_reason_field.SetMaxLength(60)
        ref_sizer.Add(self.creditnote_reason_field, 0, wx.ALIGN_LEFT)

        # Enquirey point of contact
        self.form_contacts = Contact.company_list()
        contacts_list = [""]                          # First choice is blank
        for contact in self.form_contacts:
            contacts_list.append(contact.name)
        self.form_contacts.insert(0, None)            # insert None as first item in this list also

        self.enq_field = wx.Choice(self, -1, size=(250, vs_cfg["BUTTON_HEIGHT"]), choices=contacts_list)
        self.enq_field.SetToolTipString("Set contact name")
        ref_sizer.Add(self.enq_field, 0, wx.ALIGN_LEFT)

        self.sizer.Add(ref_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # ---------------------- Create the items table

        self.items_table = ItemsTable(self, "creditnote")
        self.sizer.Add(self.items_table, 0, wx.LEFT | wx.RIGHT | wx.ALIGN_CENTER, 10)

        if not self.show_vat:
            # Hide the vat column
            self.items_table.hide_vat_column()

        # ---------------------- Make country check, and total cost

        # Create a horizontal results_sizer
        results_sizer=wx.BoxSizer(wx.HORIZONTAL)

        # Put country check in a Vertical sizer
        country_sizer=wx.BoxSizer(wx.VERTICAL)

        print_text=wx.StaticText(self, -1, """Check for the country to be
shown on printed addresses""")
        country_sizer.Add(print_text, 0, wx.ALIGN_CENTER)

        self.print_check = wx.CheckBox(self, -1, "")
        if self.creditnote.customer.vat_country_code == vs_cfg["VAT_COUNTRY_CODE"]:
            self.print_check.SetValue(False)
        else:
            self.print_check.SetValue(True)
        country_sizer.Add(self.print_check, 0, wx.TOP | wx.ALIGN_CENTER, 10)

        results_sizer.Add(country_sizer, 0, wx.LEFT | wx.RIGHT | wx.ALIGN_CENTER, 40)

        # page setup button
        self.page_button = wx.Button(self, label="Page setup", size=button_size)
        self.page_button.SetToolTipString("Click to set up the pdf page size and orientation")
        results_sizer.Add(self.page_button, 0, wx.RIGHT | wx.ALIGN_CENTER, 40)

        # pdf button
        self.pdf_button = wx.Button(self, label="Create pdf", size=button_size)
        self.pdf_button.SetToolTipString("Click to create a pdf file of the Credit Note.")
        results_sizer.Add(self.pdf_button, 0, wx.RIGHT | wx.ALIGN_CENTER, 40)

        # Put costs in a grid sizer
        cost_sizer=wx.FlexGridSizer(rows=3, cols=2, hgap=20, vgap=10)

        zero = vs_cfg["SYMBOL_ZERO_STRING"]

        # VAT
        VAT_text=wx.StaticText(self, -1, "VAT")
        cost_sizer.Add(VAT_text, 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.VAT_value_field = wx.TextCtrl(self, -1, zero, style=wx.ALIGN_RIGHT, size=(150, vs_cfg["BUTTON_HEIGHT"]))
        self.VAT_value_field.SetToolTipString("The VAT amount")
        self.VAT_value_field.SetMaxLength(18)
        # If VAT not to be shown disable the VAT field
        if not self.show_vat:
            self.VAT_value_field.Enable(False)
        cost_sizer.Add(self.VAT_value_field, 0, wx.ALIGN_RIGHT)

        # cost
        cost_text=wx.StaticText(self, -1, "Cost")
        cost_sizer.Add(cost_text, 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.cost_value_field = wx.TextCtrl(self, -1, zero, style=wx.ALIGN_RIGHT, size=(150, vs_cfg["BUTTON_HEIGHT"]))
        self.cost_value_field.SetToolTipString("Cost, exclusive of VAT")
        self.cost_value_field.SetMaxLength(18)
        cost_sizer.Add(self.cost_value_field, 0, wx.ALIGN_RIGHT)

        # Total
        total_text=wx.StaticText(self, -1, "Total")
        cost_sizer.Add(total_text, 0, wx.ALIGN_RIGHT)
        self.total_value=wx.StaticText(self, -1, zero)
        cost_sizer.Add(self.total_value, 0, wx.ALIGN_RIGHT)

        results_sizer.Add(cost_sizer, 0, wx.LEFT | wx.ALIGN_RIGHT, 40)

        # Add results_sizer to the main sizer
        self.sizer.Add(results_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # ---------------------- Apply - Cancel -- buttons

        # Create a horizontal button_sizer to hold the buttons
        button_sizer=wx.BoxSizer(wx.HORIZONTAL)

        button_text=wx.StaticText(self, -1, """Creating a PDF file does not implement or store the
transaction in the database, press Apply to do that""")
        button_sizer.Add(button_text, 0, wx.RIGHT | wx.ALIGN_CENTER, 40)

        # Apply button
        self.apply_button = wx.Button(self, wx.ID_APPLY, size=button_size)
        self.apply_button.SetToolTipString("Click to apply changes.")
        button_sizer.Add(self.apply_button, 0, wx.RIGHT | wx.ALIGN_CENTER, 40)

        # Cancel button
        self.cancel_button = wx.Button(self, wx.ID_CANCEL, size=button_size)
        self.cancel_button.SetToolTipString("Click to cancel.")
        button_sizer.Add(self.cancel_button, 0, wx.ALIGN_CENTER)

        # Add button_sizer to the main sizer
        self.sizer.Add(button_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # Set the size to the panel
        self.SetSizer(self.sizer)
        self.sizer.SetSizeHints(self)

        # ---------- Bind buttons to handler functions

        self.Bind(wx.EVT_TEXT, self.discount_change, self.discount_rate_field)

        self.Bind(wx.EVT_TEXT, self.value_change, self.cost_value_field)
        self.Bind(wx.EVT_TEXT, self.value_change, self.VAT_value_field)
        self.Bind(wx.EVT_BUTTON, self.page_button_pressed, self.page_button)
        self.Bind(wx.EVT_BUTTON, self.pdf_button_pressed, self.pdf_button) 
        self.Bind(wx.EVT_BUTTON, self.apply_button_pressed, self.apply_button)
        self.Bind(wx.EVT_BUTTON, self.cancel_button_pressed, self.cancel_button)

    # ----------------- Handler functions

    def discount_change(self, event):
        "The discount field has changed, update the totals"
        try:
            self.items_table.calculate_totals()
        except Exception, e:
            error_message("Failure in calculating discount change", e)


    def value_change(self, event):
        "If the cost or VAT fields change, change the Totals sum"
        try:
            vat = self.VAT_value_field.GetValue().strip()
            cost = self.cost_value_field.GetValue().strip()

            status, vat_value = string_to_decimal(vat)
            if status == False:
                vat_value = D("0.00")
            status, cost_value = string_to_decimal(cost)
            if status == False:
                cost_value = D("0.00")

            total = vat_value + cost_value

            self.total_value.SetLabel(to_currency_string(total))
            self.sizer.Layout()
        except Exception, e:
            error_message("Failure in calculating value change", e)


    def get_discount(self):
        "Gets discount, returning status, discount"
        discount_string = self.discount_rate_field.GetValue().strip()
        try:
            discount_value = D(discount_string.replace(vs_cfg["CP"], "."))
        except Exception:
            return False, D("0.0")
        if discount_value < D(0) or discount_value > D("99.9"):
            return False,  D("0.0")
        return True, discount_value


    @button_wrap
    def page_button_pressed(self, event):
        "page button pressed, setup the pdf page size and orientation"

        if not vs_cfg["REPORTLABS_AVAILABLE"]:
            # no pdf capability
            vs_message(self, "Function unavailable - the reportlab module is not installed",
                             "ERROR")
            return

        page_setup_dialog = vatstuff_packages.vs_pdf.page_setup.PageSetupDialog(self,
                                                              self.creditnote.pdf_width,
                                                              self.creditnote.pdf_height)
        if page_setup_dialog.ShowModal() == wx.ID_OK:
            self.creditnote.pdf_width = page_setup_dialog.width
            self.creditnote.pdf_height = page_setup_dialog.height           
        page_setup_dialog.Destroy()


    @button_wrap
    def pdf_button_pressed(self, event):
        "pdf button - check all fields, if ok, create a pdf form"

        if not vs_cfg["REPORTLABS_AVAILABLE"]:
            # no pdf capability
            vs_message(self, "Function unavailable - the reportlab module is not installed",
                             "ERROR")
            return

        status, message = self.check_fields()
        if not status:
            vs_message(self, message, "ERROR")
            return

        filename= "creditnote_%s.pdf" % self.creditnote.number

        # Create the save file dialog.
        status, filepath = save_file_as(self, 'pdf', vs_cfg["CREDITNOTES_PDF_FOLDER"], filename)

        if status:
            busy_cursor_on()
            wx.Yield()
            if vatstuff_packages.vs_pdf.creditnote_form.create_pdf(filepath, self.creditnote):
                busy_cursor_off()
                # And show the file in the default viewer
                if not vatstuff_packages.vs_pdf.pdf_functions.show_pdf_file(filepath):
                    vs_message(self, "pdf file created", "Done")
            else:
                vs_message(self, "Unable to create the pdf file", "ERROR")


    @writecheck
    @button_wrap
    def apply_button_pressed(self, event):
        """Apply button - check and apply all fields"""

        status, message = self.check_fields()
        if not status:
            vs_message(self, message, "ERROR")
            return

        # the self.check_fields() function has set the self.creditnote object
        # attributes from the form fields

        # Store the transaction in a TransactionForm object
        transaction = form_objects.TransactionForm()
        # This is an credit note transaction(system type 7)
        transaction.system = 7

        transaction.description = "Credit note %s" % self.creditnote.number
        transaction.datestamp = self.creditnote.datestamp
            
        # Credit the customers account by the amount
        transaction.add(self.creditnote.account_credited, credit = self.creditnote.cost + self.creditnote.vat)

        # Debit the sales account by the cost
        if self.creditnote.cost != D(0):
            transaction.add(get_account_by_system(6), debit = self.creditnote.cost)

        # Debit the vat balance account by the vat
        if self.creditnote.vat != D(0):
            transaction.add(get_account_by_system(5), debit = self.creditnote.vat)

        status, message = transaction.check()
        if not status:
            vs_message(self, message, "ERROR")
            return

        wx.Yield()
        # Confirm the transaction (function and dialog in gui_functions)
        status, new_description = confirm_transaction(self, transaction)

        if not status:
            vs_message(self, "The Credit Note has not been saved", "CANCELLED")
            return

        transaction.description = new_description

        # Must now implement the transaction and save the creditnote
        busy_cursor_on()
        wx.Yield()
        save_creditnote.save(self.creditnote, transaction)

        big_refresh()
        vs_message(self, "Credit note %s saved" % self.creditnote.number, "DONE")
        self.EndModal(wx.ID_OK)


    def cancel_button_pressed(self, event):
        """Cancel button - Close parent"""
        vs_message(self, "The creditnote has not been saved to the database", "Warning")
        self.EndModal(wx.ID_CANCEL)


    #----------- Check and store all fields button

    def check_fields(self):
        """Checks the fields, and fills in the self.creditnote object
           returns (True, None) if ok, or (False, "warning message") if not"""

        # Credit customer account by the total amount
        self.creditnote.account_credited = self.creditnote.customer.account
        if not self.creditnote.account_credited:
            return False, "This customers account cannot be found"

        # Get customer address
        self.creditnote.customer_address = self.customer_address.current_address

        # Get creditnote reason
        self.creditnote.reason = self.creditnote_reason_field.GetValue().strip()

        # Datestamp of the issue of this credit note
        try:
            self.creditnote.datestamp = wxdate_to_date(self.creditnote_date_field.GetValue())
        except Exception:
            return False, "The date is not valid"

        # Credit Note Number
        number_string = self.creditnote_number_field.GetValue().strip()
        try:
            self.creditnote.number = int(number_string)
        except Exception:
            return False, "The credit note number is not valid"

        # Discount
        discount_string = self.discount_rate_field.GetValue().strip()
        try:
            self.creditnote.discount = D(discount_string.replace(vs_cfg["CP"], "."))
        except Exception:
            return False, "The discount value is incorrect."

        contact_selected_list_item = self.enq_field.GetCurrentSelection()
        if contact_selected_list_item < 0:
            self.creditnote.contact = None
        else:
            self.creditnote.contact = self.form_contacts[contact_selected_list_item]

        # Create an item_list of items
        # Each list element being itself a dictionary of
        # {"Qty":Decimal, "Description":string, "Unit Cost":Decimal, "Discount":boolean "VAT":Decimal (The % value)}

        status, item_list, message = self.items_table.check_items()
        if not status:
            return False, message
        self.creditnote.item_list = item_list

        self.creditnote.print_check = self.print_check.GetValue()

        # Get totals fields

        if self.show_vat:
            vat = self.VAT_value_field.GetValue().strip()
            status, self.creditnote.vat = string_to_decimal(vat)
            if not status:
                return False, "The total VAT value is not valid"
        else:
            self.creditnote.vat = D("0.00")

        cost = self.cost_value_field.GetValue().strip()
        status, self.creditnote.cost = string_to_decimal(cost)
        if not status:
            return False, "The total cost value is not valid"

        # Check these input creditnote details are ok via vatstuff_packages.vs_verify.sales
        status, message = new_creditnote_check(self.creditnote)
        if not status:
            return False, message

        return True, None


# The CreditnoteWithPaymentDialog class is used if instead of reducing customer payments on
# an invoice already received, the creditnote is passed to them with a cheque for the payment
# as a refund for payments already obtained.  The customer passed to the constructor is the
# customer on the Sales tab - used as the default in a choices field.


def creditnote_with_payment_dialog(parent, customer):
    """Creates a creditnote accompanied by a payment"""
    creditnote_with_payment_dialog = CreditnoteWithPaymentDialog(parent, customer)

    # If the cursor is busy, unbusy it - as
    # now waiting for a button press
    busy_cursor_off()

    creditnote_with_payment_dialog.ShowModal()
    creditnote_with_payment_dialog.Destroy()


class CreditnoteWithPaymentDialog(vsDialog):
    "The dialog which records a creditnote sent to a customer with a payment"

    def __init__(self, parent, customer):
        """Create the Dialog"""
        vsDialog.__init__(self, parent, "Credit Note")
        
        # Take a note of the variable
        self.customer = customer

        # Store all form contents in a CreditNoteForm object
        self.creditnote = form_objects.CreditNoteForm()
        self.creditnote.invoice = None
        self.creditnote.status = 1

        # STATUS - normally 0 when a creditnote is sent against an invoice
        # STATUS - 1 when a creditnote is sent with a payment

        # Can you charge VAT?
        self.show_vat = self.customer.charge_vat()
        self.creditnote.show_vat = self.show_vat

        self.creditnote.company = Company()

        button_size = (100, vs_cfg["BUTTON_HEIGHT"])

        # Create a vertical sizer to hold everything
        self.sizer=wx.BoxSizer(wx.VERTICAL)

        # And a horizontal sizer to hold address and cash account details
        toprow_sizer = wx.BoxSizer(wx.HORIZONTAL)

        # Address sizer
        add_sizer = wx.BoxSizer(wx.VERTICAL)

        # Create a horizontal customer_sizer
        customer_sizer=wx.BoxSizer(wx.HORIZONTAL)

        customer_sizer.Add(wx.StaticText(self, -1, "To:"), 0, wx.RIGHT | wx.ALIGN_RIGHT, 10)

        self.all_customers = Customer.get_all()
        customer_list = [ c.name for c in self.all_customers ]

        self.customer_field = wx.Choice(self, -1, size=(250, vs_cfg["BUTTON_HEIGHT"]), choices=customer_list)
        self.customer_field.SetToolTipString("Set customer")
        customer_sizer.Add(self.customer_field, 0, wx.ALIGN_LEFT)

        key = self.all_customers.index(customer)
        self.customer_field.SetSelection(key)

        add_sizer.Add(customer_sizer, 0, wx.ALIGN_CENTER)

        self.customer_address = AddressSpin(self, customer, invoice_point=1)
        add_sizer.Add(self.customer_address, 0, wx.ALL, 10)

        toprow_sizer.Add(add_sizer, 0, wx.RIGHT | wx.ALIGN_TOP, 40)

        # numbers sizer
        num_sizer = wx.BoxSizer(wx.VERTICAL)

        top_text = "This creditnote is to be accompanied by a payment from you to your customer."
        num_sizer.Add(wx.StaticText(self, -1, top_text), 0, wx.ALL | wx.ALIGN_CENTER, 10)


        # --- input details in a grid sizer

        inp_sizer=wx.FlexGridSizer(rows=3, cols=2, hgap=10, vgap=20)

        # Bank or cash account

        bank_text = """\
Specify the cash or bank account from
which this money is to be taken:"""
        inp_sizer.Add(wx.StaticText(self, -1, bank_text), 0, wx.ALIGN_RIGHT)

        self.all_accounts = Account.get_non_system()

        if not len(self.all_accounts):
            raise VS_ERROR, "No accounts are available"

        self.bank_account_field = wx.Choice(self, -1, size=(200, vs_cfg["BUTTON_HEIGHT"]),
                                            choices=[ a.fullname for a in self.all_accounts ])

        # Get the last cash account used - and pre-select it
        last_account = get_last_cash_account()
        if last_account:
            try:
                key = self.all_accounts.index(last_account)
                self.bank_account_field.SetSelection(key)
            except ValueError:
                pass

        self.bank_account_field.SetToolTipString("Set a bank or cash account")
        inp_sizer.Add(self.bank_account_field, 0, wx.ALIGN_LEFT)

        # Credit Note number
        inp_sizer.Add(wx.StaticText(self, -1, "Credit Note Number:\n(Automatically increments)"),
                                              0, wx.ALIGN_RIGHT)

        self.last_creditnote_number = get_config_item("LAST_CREDITNOTE_NUMBER")
        if not self.last_creditnote_number:
            self.last_creditnote_number = 0
            creditnote_number = 1
        else:
            self.last_creditnote_number = int(self.last_creditnote_number)
            creditnote_number =  self.last_creditnote_number + 1

        self.creditnote_number_field = wx.TextCtrl(self, -1, str(creditnote_number), size=(200, vs_cfg["BUTTON_HEIGHT"]))
        self.creditnote_number_field.SetToolTipString("Set your credit note number")
        self.creditnote_number_field.SetMaxLength(30)
        inp_sizer.Add(self.creditnote_number_field, 0, wx.ALIGN_LEFT)

        inp_sizer.Add(wx.StaticText(self, -1, "Date:"), 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.creditnote_date_field = wx.DatePickerCtrl(self, -1,
                                                  wx.DateTime.Now(), 
                                                  size=(150, vs_cfg["BUTTON_HEIGHT"]),
                                                  style=wx.DP_DROPDOWN | wx.DP_SHOWCENTURY)
        self.creditnote_date_field.SetToolTipString("The date of the credit note")
        inp_sizer.Add(self.creditnote_date_field, 0, wx.ALIGN_LEFT)

        num_sizer.Add(inp_sizer, 0, wx.ALL, 10)

        toprow_sizer.Add(num_sizer, 0, wx.RIGHT | wx.ALIGN_TOP, 40)

        self.sizer.Add(toprow_sizer, 0, wx.TOP | wx.LEFT | wx.RIGHT | wx.ALIGN_CENTER, 20)

        # --- reference details in a grid sizer

        ref_sizer=wx.FlexGridSizer(rows=2, cols=3, hgap=20, vgap=5)

        # First row
        ref_sizer.Add((10,10), 1)
        ref_sizer.Add(wx.StaticText(self, -1, "Reason for credit note (text shown on the pdf):"), 0, wx.ALIGN_LEFT)
        ref_sizer.Add(wx.StaticText(self, -1, "In case of enquiry please contact:"), 0, wx.ALIGN_LEFT)

        # Second row

        # Discount box
        discount_sizer=wx.BoxSizer(wx.HORIZONTAL)
        discount_sizer.Add(wx.StaticText(self, -1, "Discount rate:"), 0, wx.RIGHT | wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL, 5)
        discount_rate_text = str(self.customer.discount).replace(".", vs_cfg["CP"])
        self.discount_rate_field = wx.TextCtrl(self, -1, discount_rate_text, style=wx.ALIGN_RIGHT, size=(60, vs_cfg["BUTTON_HEIGHT"]))
        self.discount_rate_field.SetToolTipString("Set the percentage discount rate")
        self.discount_rate_field.SetMaxLength(4)
        discount_sizer.Add(self.discount_rate_field, 0, wx.ALIGN_RIGHT)
        discount_sizer.Add(wx.StaticText(self, -1, "%"), 0, wx.ALIGN_CENTER_VERTICAL)
        ref_sizer.Add(discount_sizer,0, wx.ALIGN_RIGHT)

        # Reason for credit note
        self.creditnote_reason_field = wx.TextCtrl(self, -1, "", size=(350, vs_cfg["BUTTON_HEIGHT"]))
        self.creditnote_reason_field.SetToolTipString("Reason for credit note")
        self.creditnote_reason_field.SetMaxLength(60)
        ref_sizer.Add(self.creditnote_reason_field, 0, wx.ALIGN_LEFT)

        # Enquirey point of contact
        self.form_contacts = Contact.company_list()
        contacts_list = [""]                          # First choice is blank
        for contact in self.form_contacts:
            contacts_list.append(contact.name)
        self.form_contacts.insert(0, None)            # insert None as first item in this list also

        self.enq_field = wx.Choice(self, -1, size=(250, vs_cfg["BUTTON_HEIGHT"]), choices=contacts_list)
        self.enq_field.SetToolTipString("Set contact name")
        ref_sizer.Add(self.enq_field, 0, wx.ALIGN_LEFT)

        self.sizer.Add(ref_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # ---------------------- Create the items table

        self.items_table = ItemsTable(self, "creditnote")
        self.sizer.Add(self.items_table, 0, wx.LEFT | wx.RIGHT | wx.ALIGN_CENTER, 10)

        if not self.show_vat:
            # Hide the vat column
            self.items_table.hide_vat_column()

        # ---------------------- Make country check, and total cost

        # Create a horizontal results_sizer
        results_sizer=wx.BoxSizer(wx.HORIZONTAL)

        # Put country check in a Vertical sizer
        country_sizer=wx.BoxSizer(wx.VERTICAL)

        print_text=wx.StaticText(self, -1, """Check for the country to be
shown on printed addresses""")
        country_sizer.Add(print_text, 0, wx.ALIGN_CENTER)

        self.print_check = wx.CheckBox(self, -1, "")
        if self.customer.vat_country_code == vs_cfg["VAT_COUNTRY_CODE"]:
            self.print_check.SetValue(False)
        else:
            self.print_check.SetValue(True)
        country_sizer.Add(self.print_check, 0, wx.TOP | wx.ALIGN_CENTER, 10)

        results_sizer.Add(country_sizer, 0, wx.LEFT | wx.RIGHT | wx.ALIGN_CENTER, 40)

        # page setup button
        self.page_button = wx.Button(self, label="Page setup", size=button_size)
        self.page_button.SetToolTipString("Click to set up the pdf page size and orientation")
        results_sizer.Add(self.page_button, 0, wx.RIGHT | wx.ALIGN_CENTER, 40)

        # pdf button
        self.pdf_button = wx.Button(self, label="Create pdf", size=button_size)
        self.pdf_button.SetToolTipString("Click to create a pdf file of the Credit Note.")
        results_sizer.Add(self.pdf_button, 0, wx.RIGHT | wx.ALIGN_CENTER, 40)

        # Put costs in a grid sizer
        cost_sizer=wx.FlexGridSizer(rows=3, cols=2, hgap=20, vgap=10)

        zero = vs_cfg["SYMBOL_ZERO_STRING"]

        # VAT
        VAT_text=wx.StaticText(self, -1, "VAT")
        cost_sizer.Add(VAT_text, 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.VAT_value_field = wx.TextCtrl(self, -1, zero, style=wx.ALIGN_RIGHT, size=(150, vs_cfg["BUTTON_HEIGHT"]))
        self.VAT_value_field.SetToolTipString("The VAT amount")
        self.VAT_value_field.SetMaxLength(18)
        # If the supplier is non-GB or you are not vat registered, then disable the VAT field
        if not self.show_vat:
            self.VAT_value_field.Enable(False)
        cost_sizer.Add(self.VAT_value_field, 0, wx.ALIGN_RIGHT)

        # cost
        cost_text=wx.StaticText(self, -1, "Cost")
        cost_sizer.Add(cost_text, 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.cost_value_field = wx.TextCtrl(self, -1, zero, style=wx.ALIGN_RIGHT, size=(150, vs_cfg["BUTTON_HEIGHT"]))
        self.cost_value_field.SetToolTipString("Cost, exclusive of VAT")
        self.cost_value_field.SetMaxLength(12)
        cost_sizer.Add(self.cost_value_field, 0, wx.ALIGN_RIGHT)

        # Total
        total_text=wx.StaticText(self, -1, "Total")
        cost_sizer.Add(total_text, 0, wx.ALIGN_RIGHT)
        self.total_value=wx.StaticText(self, -1, zero)
        cost_sizer.Add(self.total_value, 0, wx.ALIGN_RIGHT)

        results_sizer.Add(cost_sizer, 0, wx.LEFT | wx.ALIGN_RIGHT, 40)

        # Add results_sizer to the main sizer
        self.sizer.Add(results_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # ---------------------- Apply - Cancel -- buttons

        # Create a horizontal button_sizer to hold the buttons
        button_sizer=wx.BoxSizer(wx.HORIZONTAL)

        button_text=wx.StaticText(self, -1, """Creating a PDF file does not implement or store the
transaction in the database, press Apply to do that""")
        button_sizer.Add(button_text, 0, wx.RIGHT | wx.ALIGN_CENTER, 40)

        # Apply button
        self.apply_button = wx.Button(self, wx.ID_APPLY, size=button_size)
        self.apply_button.SetToolTipString("Click to apply changes.")
        button_sizer.Add(self.apply_button, 0, wx.RIGHT | wx.ALIGN_CENTER, 40)

        # Cancel button
        self.cancel_button = wx.Button(self, wx.ID_CANCEL, size=button_size)
        self.cancel_button.SetToolTipString("Click to cancel.")
        button_sizer.Add(self.cancel_button, 0, wx.ALIGN_CENTER)

        # Add button_sizer to the main sizer
        self.sizer.Add(button_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # Set the size to the panel
        self.SetSizer(self.sizer)
        self.sizer.SetSizeHints(self)

        # ---------- Bind buttons to handler functions

        self.Bind(wx.EVT_TEXT, self.discount_change, self.discount_rate_field)
        self.Bind(wx.EVT_CHOICE, self.customer_pressed, self.customer_field)

        self.Bind(wx.EVT_TEXT, self.value_change, self.cost_value_field)
        self.Bind(wx.EVT_TEXT, self.value_change, self.VAT_value_field)
        self.Bind(wx.EVT_BUTTON, self.page_button_pressed, self.page_button)
        self.Bind(wx.EVT_BUTTON, self.pdf_button_pressed, self.pdf_button) 
        self.Bind(wx.EVT_BUTTON, self.apply_button_pressed, self.apply_button)
        self.Bind(wx.EVT_BUTTON, self.cancel_button_pressed, self.cancel_button)


    # ----------------- Handler functions

    def discount_change(self, event):
        "The discount field has changed, update the totals"
        try:
            self.items_table.calculate_totals()
        except Exception, e:
            error_message("Failure in calculating discount change", e)


    def value_change(self, event):
        "If the cost or VAT fields change, change the Totals sum"
        try:
            vat = self.VAT_value_field.GetValue().strip()
            cost = self.cost_value_field.GetValue().strip()

            status, vat_value = string_to_decimal(vat)
            if not status:
                vat_value = D("0.00")
            status, cost_value = string_to_decimal(cost)
            if not status:
                cost_value = D("0.00")

            total = vat_value + cost_value

            self.total_value.SetLabel(to_currency_string(total))
            self.sizer.Layout()
        except Exception, e:
            error_message("Failure in calculating value change", e)


    def get_discount(self):
        "Gets discount, returning status, discount"
        discount_string = self.discount_rate_field.GetValue().strip()
        try:
            discount_value = D(discount_string.replace(vs_cfg["CP"], "."))
        except Exception:
            return False, D("0.0")
        if discount_value < D(0) or discount_value > D("99.9"):
            return False,  D("0.0")
        return True, discount_value


    def customer_pressed(self, event):
        """Customer choice pressed - change customer address"""
        try:
            # Get customer currently shown
            customer_selected_list_item = self.customer_field.GetCurrentSelection()
            self.customer = self.all_customers[customer_selected_list_item]
            # Pass this to the address field
            self.customer_address.set_org(org=self.customer, invoice_point=1)
            # Discount field
            self.discount_rate_field.SetValue(str(self.customer.discount).replace(".", vs_cfg["CP"]))
            # Can you charge this customer VAT?
            self.show_vat = self.customer.charge_vat()
            self.creditnote.show_vat = self.show_vat
            if self.show_vat:
                self.VAT_value_field.Enable(True)
                self.items_table.show_vat_column()
            else:
                self.VAT_value_field.Enable(False)
                self.items_table.hide_vat_column()
            if self.customer.vat_country_code == vs_cfg["VAT_COUNTRY_CODE"]:
                self.print_check.SetValue(False)
            else:
                self.print_check.SetValue(True)
            self.items_table.calculate_totals()
        except Exception, e:
            error_message("Failure in updating customer", e)


    @button_wrap
    def page_button_pressed(self, event):
        "page button pressed, setup the pdf page size and orientation"

        if not vs_cfg["REPORTLABS_AVAILABLE"]:
            # no pdf capability
            vs_message(self, "Function unavailable - the reportlab module is not installed",
                             "ERROR")
            return

        page_setup_dialog = vatstuff_packages.vs_pdf.page_setup.PageSetupDialog(self, self.creditnote.pdf_width,
                                                                    self.creditnote.pdf_height)
        if page_setup_dialog.ShowModal() == wx.ID_OK:
            self.creditnote.pdf_width = page_setup_dialog.width
            self.creditnote.pdf_height = page_setup_dialog.height           
        page_setup_dialog.Destroy()


    @button_wrap
    def pdf_button_pressed(self, event):
        "pdf button - check all fields, if ok, create a pdf form"

        if not vs_cfg["REPORTLABS_AVAILABLE"]:
            # no pdf capability
            vs_message(self, "Function unavailable - the reportlab module is not installed",
                             "ERROR")
            return

        status, message = self.check_fields()
        if not status:
            vs_message(self, message, "ERROR")
            return

        filename= "creditnote_%s.pdf" % self.creditnote.number

        # Create the save file dialog.
        status, filepath = save_file_as(self, 'pdf', vs_cfg["CREDITNOTES_PDF_FOLDER"], filename)

        if status:
            busy_cursor_on()
            wx.Yield()
            if vatstuff_packages.vs_pdf.creditnote_form.create_pdf(filepath, self.creditnote):
                busy_cursor_off()
                # And show the file in the default viewer
                if not vatstuff_packages.vs_pdf.pdf_functions.show_pdf_file(filepath):
                    vs_message(self, "pdf file created", "Done")
            else:
                vs_message(self, "Unable to create the pdf file", "ERROR")


    @writecheck
    @button_wrap
    def apply_button_pressed(self, event):
        """Apply button - check and apply all fields"""

        status, message = self.check_fields()
        if not status:
            vs_message(self, message, "ERROR")
            return

        # the self.check_fields() function has set the self.creditnote object
        # attributes from the form fields

        # Store the transaction in a TransactionForm object
        transaction = form_objects.TransactionForm()
        # This is an credit note transaction(system type 7)
        transaction.system = 7

        transaction.description = "Credit note %s" % self.creditnote.number
        transaction.datestamp = self.creditnote.datestamp

        # Credit the bank account by the amount
        transaction.add(self.creditnote.account_credited, credit = self.creditnote.cost + self.creditnote.vat)

        # Debit the sales account by the credit note cost
        if self.creditnote.cost != D("0"):
            transaction.add(get_account_by_system(6), debit = self.creditnote.cost)

        # Debit the VAT balance account by the VAT  
        if self.creditnote.vat != D(0):
            transaction.add(get_account_by_system(5), debit = self.creditnote.vat)

        status, message = transaction.check()
        if not status:
            vs_message(self, message, "ERROR")
            return

        wx.Yield()
        # Confirm the transaction (function and dialog in gui_functions)
        status, new_description = confirm_transaction(self, transaction)

        if not status:
            vs_message(self, "The Credit Note has not been saved", "CANCELLED")
            return

        transaction.description = new_description

        # Must now implement the transaction and save the creditnote
        busy_cursor_on()
        wx.Yield()
        save_creditnote.save(self.creditnote, transaction)

        big_refresh()
        vs_message(self, "Credit note %s saved" % self.creditnote.number, "DONE")
        self.EndModal(wx.ID_OK)


    def cancel_button_pressed(self, event):
        """Cancel button pressed - close without saving the credit note"""
        vs_message(self, "The creditnote has not been saved to the database", "Warning")
        self.EndModal(wx.ID_CANCEL)


    #----------- Check and store all fields button

    def check_fields(self):
        """Checks the fields, and fills in the self.creditnote object
           returns True if ok, or creates a warning message and False if not"""

        # Get customer
        customer_selected_list_item = self.customer_field.GetCurrentSelection()
        self.creditnote.customer = self.all_customers[customer_selected_list_item]

        # Get customer address
        self.creditnote.customer_address = self.customer_address.current_address

        # Get creditnote reason
        self.creditnote.reason = self.creditnote_reason_field.GetValue().strip()

        # Datestamp of the issue of this credit note
        self.creditnote.datestamp = wxdate_to_date(self.creditnote_date_field.GetValue())

        # Credit Note Number
        number_string = self.creditnote_number_field.GetValue().strip()
        try:
            self.creditnote.number = int(number_string)
        except Exception:
            return False, "The credit note number is not valid"

        # The cash or bank account used to record this purchase
        account_selected_list_item = self.bank_account_field.GetCurrentSelection()
        if account_selected_list_item < 0:         # blank or unselected
            return False, "The bank account cannot be found"
        else:
            self.creditnote.account_credited = self.all_accounts[account_selected_list_item]

        # Discount
        discount_string = self.discount_rate_field.GetValue().strip()
        try:
            self.creditnote.discount = D(discount_string.replace(vs_cfg["CP"], "."))
        except Exception:
            return False, "The discount value is incorrect."

        # Contact
        contact_selected_list_item = self.enq_field.GetCurrentSelection()
        if contact_selected_list_item < 0:
            self.creditnote.contact = None
        else:
            self.creditnote.contact = self.form_contacts[contact_selected_list_item]

        # Create an item_list of items
        # Each list element being itself a dictionary of
        # {"Qty":Decimal, "Description":string, "Unit Cost":Decimal, "Discount":boolean "VAT":Decimal (The % value)}

        status, item_list, message = self.items_table.check_items()
        if not status:
            return False, message
        self.creditnote.item_list = item_list

        self.creditnote.print_check = self.print_check.GetValue()

        # Get totals fields

        if self.show_vat:
            vat = self.VAT_value_field.GetValue().strip()
            status, self.creditnote.vat = string_to_decimal(vat)
            if not status:
                return False, "The total VAT value is not valid"
        else:
            self.creditnote.vat = D("0.00")

        cost = self.cost_value_field.GetValue().strip()
        status, self.creditnote.cost = string_to_decimal(cost)
        if not status:
            return False, "The total cost value is not valid"

        # Check these input creditnote details are ok via vatstuff_packages.vs_verify.sales
        status, message = new_creditnote_check(self.creditnote)
        if not status:
            return False, message

        return True, None


