####### VATStuff #######
#
# make_invoice.py  - creates an invoice
#
# Version : 0.7.0
# Date : 20100925
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#


"""Creates an invoice

It is run as part of the vatstuff gui interface.
"""

import os
import decimal
import datetime

import wx
import wx.grid

from vatstuff_packages.vs_database import save_invoice

from vatstuff_packages.vs_gui.address_panel import *
from vatstuff_packages.vs_gui.gui_functions import *

from vatstuff_packages.vs_gui.items_table import ItemsTable

from vatstuff_packages.vs_verify import form_objects
from vatstuff_packages.vs_verify.sales import new_invoice_check

from vatstuff_packages.vs_utilities import *

if "REPORTLABS_AVAILABLE" in vs_cfg and vs_cfg["REPORTLABS_AVAILABLE"]:
    import vatstuff_packages.vs_pdf.page_setup
    import vatstuff_packages.vs_pdf.invoice_form
    import vatstuff_packages.vs_pdf.pdf_functions

D = decimal.Decimal   # D used to avoid typing


def invoice_dialog(parent, customer):
    """Creates an InvoiceDialog"""
    invoice_dialog = InvoiceDialog(parent, customer)

    # If the cursor is busy, unbusy it - as
    # now waiting for a button press
    busy_cursor_off()

    invoice_dialog.ShowModal()
    invoice_dialog.Destroy()


class InvoiceDialog(vsDialog):
    "Called when the create invoice button is pressed"

    def __init__(self, parent, customer):
        """Create the Dialog"""
        vsDialog.__init__(self, parent, "Invoice")
        
        # Take a note of the variables
        self.customer = customer
        self.vat_scheme = VATScheme()

        # Can you charge VAT?
        self.show_vat = self.customer.charge_vat()

        # Store all form contents in an InvoiceForm object
        self.invoice = form_objects.InvoiceForm()

        self.invoice.show_vat = self.show_vat

        button_size = (100, vs_cfg["BUTTON_HEIGHT"])

        # Create a vertical sizer to hold everything
        self.sizer=wx.BoxSizer(wx.VERTICAL)

        # Creat a horizontal sizer to hold addresses and dates
        top_sizer=wx.BoxSizer(wx.HORIZONTAL)

        # Create customer and payment addresses in a grid sizer
        add_sizer=wx.FlexGridSizer(rows=2, cols=2, hgap=20, vgap=10)

        # Create a horizontal customer_sizer
        customer_sizer=wx.BoxSizer(wx.HORIZONTAL)

        self.all_customers = Customer.get_all()
        customer_list = [ c.name for c in self.all_customers ]

        customer_sizer.Add(wx.StaticText(self, -1, "To:"), 0, wx.RIGHT | wx.ALIGN_RIGHT, 10)

        self.customer_field = wx.Choice(self, -1, size=(250, vs_cfg["BUTTON_HEIGHT"]), choices=customer_list)
        self.customer_field.SetToolTipString("Set customer")
        customer_sizer.Add(self.customer_field, 0, wx.ALIGN_LEFT)

        key = self.all_customers.index(customer)
        self.customer_field.SetSelection(key)

        add_sizer.Add(customer_sizer, 0, wx.ALIGN_CENTER)

        add_sizer.Add(wx.StaticText(self, -1, "Send payment to:"), 0, wx.ALIGN_LEFT | wx.ALIGN_BOTTOM)

        #  -- Next row of the grid

        self.customer_address = AddressSpin(self, self.customer, invoice_point=1)
        add_sizer.Add(self.customer_address, 0, wx.ALIGN_TOP)

        self.payment_address = AddressSpin(self, self.invoice.company, payment_point=1)
        add_sizer.Add(self.payment_address, wx.ALIGN_TOP)


        # add the add_sizer to the top sizer
        top_sizer.Add(add_sizer, 0, wx.RIGHT, 10)


        # Create invoice number and date fields in a grid sizer - and put in the
        # right of the top_sizer
        date_sizer=wx.FlexGridSizer(rows=3, cols=2, hgap=20, vgap=10)

        # invoice number
        date_sizer.Add(wx.StaticText(self, -1, "Invoice Number:\n(Automatically increments)"),
                                              0, wx.ALIGN_RIGHT)

        self.last_invoice_number = get_config_item(u"LAST_INVOICE_NUMBER")
        if not self.last_invoice_number:
            self.last_invoice_number = 0
            invoice_number = 1
        else:
            self.last_invoice_number = int(self.last_invoice_number)
            invoice_number =  self.last_invoice_number + 1

        self.invoice_number_field = wx.TextCtrl(self, -1, str(invoice_number), size=(200, vs_cfg["BUTTON_HEIGHT"]))
        self.invoice_number_field.SetToolTipString("Set your invoice number")
        self.invoice_number_field.SetMaxLength(30)
        date_sizer.Add(self.invoice_number_field, 0, wx.BOTTOM | wx.ALIGN_LEFT | wx.ALIGN_TOP, 10)

        # supply of goods date
        date_sizer.Add(wx.StaticText(self, -1, "Date of supply of goods:"), 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.date_of_supply_field = wx.DatePickerCtrl(self, -1,
                                                  wx.DateTime.Now(), 
                                                  size=(150, vs_cfg["BUTTON_HEIGHT"]),
                                                  style=wx.DP_DROPDOWN | wx.DP_SHOWCENTURY)
        self.date_of_supply_field.SetToolTipString("The date of the supply of goods")
        date_sizer.Add(self.date_of_supply_field, 0, wx.ALIGN_LEFT)

        # date of issue
        date_sizer.Add(wx.StaticText(self, -1, "Date of issue of invoice:"), 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.date_of_issue_field = wx.DatePickerCtrl(self, -1,
                                                  wx.DateTime.Now(), 
                                                  size=(150, vs_cfg["BUTTON_HEIGHT"]),
                                                  style=wx.DP_DROPDOWN | wx.DP_SHOWCENTURY)
        self.date_of_issue_field.SetToolTipString("The date of the issue of this invoice")
        date_sizer.Add(self.date_of_issue_field, 0, wx.ALIGN_LEFT)

        top_sizer.Add(date_sizer, 0, wx.ALIGN_LEFT | wx.ALIGN_TOP)

        # add the top_sizer to the main vertical sizer
        self.sizer.Add(top_sizer, 0, wx.LEFT | wx.TOP | wx.RIGHT | wx.ALIGN_CENTER, 10)

        # Create reference text in a grid sizer
        ref_sizer=wx.FlexGridSizer(rows=3, cols=3, hgap=20, vgap=10)

        # First row
        ref_sizer.Add(wx.StaticText(self, -1, "Customer order number:"), 0, wx.ALIGN_LEFT)
        ref_sizer.Add(wx.StaticText(self, -1, "Payment instructions:"), 0, wx.ALIGN_LEFT)
        ref_sizer.Add(wx.StaticText(self, -1, "In case of enquiry please contact:"), 0, wx.ALIGN_LEFT)

        # Second row

        self.cus_order_number_field = wx.TextCtrl(self, -1, "", size=(250, vs_cfg["BUTTON_HEIGHT"]))
        self.cus_order_number_field.SetToolTipString("Customers order number (if any)")
        self.cus_order_number_field.SetMaxLength(40)
        ref_sizer.Add(self.cus_order_number_field, 0, wx.ALIGN_LEFT)

        self.payment_instructions_field = wx.TextCtrl(self, -1, "", size=(350, vs_cfg["BUTTON_HEIGHT"]))
        self.payment_instructions_field.SetToolTipString("Payment instructions (if any)")
        self.payment_instructions_field.SetMaxLength(60)
        ref_sizer.Add(self.payment_instructions_field, 0, wx.ALIGN_LEFT)

        self.form_contacts = Contact.company_list()
        contacts_list = [""]                          # First choice is blank
        for contact in self.form_contacts:
            contacts_list.append(contact.name)
        self.form_contacts.insert(0, None)            # insert None as first item in this list also

        self.enq_field = wx.Choice(self, -1, size=(250, vs_cfg["BUTTON_HEIGHT"]), choices=contacts_list)
        self.enq_field.SetToolTipString("Set contact name")
        ref_sizer.Add(self.enq_field, 0, wx.ALIGN_LEFT)

        # Third row

        discount_sizer=wx.BoxSizer(wx.HORIZONTAL)
        discount_sizer.Add(wx.StaticText(self, -1, "Discount rate:"), 0, wx.RIGHT | wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL, 10)
        discount_rate_text = str(self.customer.discount).replace(u".", vs_cfg["CP"])
        self.discount_rate_field = wx.TextCtrl(self, -1, discount_rate_text, style=wx.ALIGN_RIGHT, size=(60, vs_cfg["BUTTON_HEIGHT"]))
        self.discount_rate_field.SetToolTipString("Set the percentage discount rate")
        self.discount_rate_field.SetMaxLength(4)
        discount_sizer.Add(self.discount_rate_field, 0, wx.ALIGN_RIGHT)
        discount_sizer.Add(wx.StaticText(self, -1, "%"), 0, wx.ALIGN_CENTER_VERTICAL)


        ref_sizer.Add(discount_sizer,0, wx.ALIGN_RIGHT)
        ref_sizer.Add(wx.StaticText(self, -1, "Tick each item checkbox to apply the discount"), 0, wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)
        ref_sizer.Add(wx.StaticText(self, -1, "(names set under the Company tab)"), 0, wx.ALIGN_RIGHT)

        # add the ref_sizer to the main vertical sizer
        self.sizer.Add(ref_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # ---------------------- Create the sellitems table

        self.sellitems_table = ItemsTable(self, "invoice")
        self.sizer.Add(self.sellitems_table, 0, wx.LEFT | wx.RIGHT | wx.ALIGN_CENTER, 10)

        if not self.show_vat:
            # Hide the vat column
            self.sellitems_table.hide_vat_column()


        # ---------------------- Make country check, and total cost

        # Create a horizontal results_sizer
        results_sizer=wx.BoxSizer(wx.HORIZONTAL)

        # Put country check in a Vertical sizer
        country_sizer=wx.BoxSizer(wx.VERTICAL)

        print_text=wx.StaticText(self, -1, """Check for the country to be
shown on printed addresses""")
        country_sizer.Add(print_text, 0, wx.ALIGN_CENTER)

        self.print_check = wx.CheckBox(self, -1, "")
        if self.customer.vat_country_code == vs_cfg["VAT_COUNTRY_CODE"]:
            self.print_check.SetValue(False)
        else:
            self.print_check.SetValue(True)
        country_sizer.Add(self.print_check, 0, wx.TOP | wx.ALIGN_CENTER, 10)

        results_sizer.Add(country_sizer, 0, wx.LEFT | wx.RIGHT | wx.ALIGN_CENTER, 40)

        # page setup button
        self.page_button = wx.Button(self, label="Page setup", size=button_size)
        self.page_button.SetToolTipString("Click to set up the pdf page size and orientation")
        results_sizer.Add(self.page_button, 0, wx.RIGHT | wx.ALIGN_CENTER, 40)

        # pdf button
        self.pdf_button = wx.Button(self, label="Create pdf", size=button_size)
        self.pdf_button.SetToolTipString("Click to create a pdf file of the invoice.")
        results_sizer.Add(self.pdf_button, 0, wx.RIGHT | wx.ALIGN_CENTER, 40)

        # Put costs in a grid sizer
        cost_sizer=wx.FlexGridSizer(rows=3, cols=2, hgap=20, vgap=10)

        zero = vs_cfg["SYMBOL_ZERO_STRING"]

        # VAT
        VAT_text=wx.StaticText(self, -1, "VAT")
        cost_sizer.Add(VAT_text, 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.VAT_value_field = wx.TextCtrl(self, -1, zero, style=wx.ALIGN_RIGHT, size=(150, vs_cfg["BUTTON_HEIGHT"]))
        self.VAT_value_field.SetToolTipString("The VAT amount")
        self.VAT_value_field.SetMaxLength(18)
        # If the VAT is not to be shown on the invoice, then disable the VAT field
        if not self.show_vat:
            self.VAT_value_field.Enable(False)
        cost_sizer.Add(self.VAT_value_field, 0, wx.ALIGN_RIGHT)

        # cost
        cost_text=wx.StaticText(self, -1, "Cost")
        cost_sizer.Add(cost_text, 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.cost_value_field = wx.TextCtrl(self, -1, zero, style=wx.ALIGN_RIGHT, size=(150, vs_cfg["BUTTON_HEIGHT"]))
        self.cost_value_field.SetToolTipString("Cost, exclusive of VAT")
        self.cost_value_field.SetMaxLength(18)
        cost_sizer.Add(self.cost_value_field, 0, wx.ALIGN_RIGHT)

        # Total
        total_text=wx.StaticText(self, -1, "Total")
        cost_sizer.Add(total_text, 0, wx.ALIGN_RIGHT)
        self.total_value=wx.StaticText(self, -1, zero)
        cost_sizer.Add(self.total_value, 0, wx.ALIGN_RIGHT)

        results_sizer.Add(cost_sizer, 0, wx.LEFT | wx.ALIGN_RIGHT, 40)


        # Add results_sizer to the main sizer
        self.sizer.Add(results_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 10)


        # ---------------------- Apply - Cancel -- buttons

        # Create a horizontal button_sizer to hold the buttons
        button_sizer=wx.BoxSizer(wx.HORIZONTAL)


        button_text=wx.StaticText(self, -1, """Creating a PDF file does not implement or store the
transaction in the database, press Apply to do that""")
        button_sizer.Add(button_text, 0, wx.RIGHT | wx.ALIGN_CENTER, 40)

        # Apply button
        self.apply_button = wx.Button(self, wx.ID_APPLY, size=button_size)
        self.apply_button.SetToolTipString("Click to apply changes.")
        button_sizer.Add(self.apply_button, 0, wx.RIGHT | wx.ALIGN_CENTER, 40)

        # Cancel button
        self.cancel_button = wx.Button(self, wx.ID_CANCEL, size=button_size)
        self.cancel_button.SetToolTipString("Click to cancel.")
        button_sizer.Add(self.cancel_button, 0, wx.ALIGN_CENTER)

        # Add button_sizer to the main sizer
        self.sizer.Add(button_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # Set the size
        self.SetSizer(self.sizer)
        self.sizer.SetSizeHints(self)

        # ---------- Bind buttons to handler functions

        self.Bind(wx.EVT_TEXT, self.discount_change, self.discount_rate_field)

        self.Bind(wx.EVT_CHOICE, self.customer_pressed, self.customer_field)

        self.Bind(wx.EVT_TEXT, self.value_change, self.cost_value_field)
        self.Bind(wx.EVT_TEXT, self.value_change, self.VAT_value_field)
        self.Bind(wx.EVT_BUTTON, self.page_button_pressed, self.page_button)
        self.Bind(wx.EVT_BUTTON, self.pdf_button_pressed, self.pdf_button) 
        self.Bind(wx.EVT_BUTTON, self.apply_button_pressed, self.apply_button)
        self.Bind(wx.EVT_BUTTON, self.cancel_button_pressed, self.cancel_button)


    # ----------------- Handler functions

    def discount_change(self, event):
        "The discount field has changed, update the totals"
        try:
            self.sellitems_table.calculate_totals()
        except Exception, e:
            error_message("Failure in calculating discount change", e)


    def value_change(self, event):
        "If the cost or VAT fields change, change the Totals sum"
        try:
            vat = self.VAT_value_field.GetValue().strip()
            cost = self.cost_value_field.GetValue().strip()

            status, vat_value = string_to_decimal(vat)
            if not status:
                vat_value = D("0.00")
            status, cost_value = string_to_decimal(cost)
            if not status:
                cost_value = D("0.00")

            total = vat_value + cost_value

            self.total_value.SetLabel(to_currency_string(total))
            self.sizer.Layout()
        except Exception, e:
            error_message("Failure in calculating value change", e)


    def get_discount(self):
        """Gets discount, returning status, discount, used by self.sellitems_table
           when calculating totals"""
        discount_string = self.discount_rate_field.GetValue().strip()
        try:
            discount_value = D(discount_string.replace(vs_cfg["CP"], "."))
        except Exception:
            return False, D("0.0")
        if discount_value < D(0) or discount_value > D("99.9"):
            return False,  D("0.0")
        return True, discount_value


    def customer_pressed(self, event):
        """Customer choice pressed - change customer address"""
        try:
            # Get customer currently shown
            customer_selected_list_item = self.customer_field.GetCurrentSelection()
            self.customer = self.all_customers[customer_selected_list_item]
            # Pass this to the address field
            self.customer_address.set_org(org=self.customer, invoice_point=1)
            # And the customer discount_rate_field
            self.discount_rate_field.SetValue(str(self.customer.discount).replace(".", vs_cfg["CP"]))
            # Can you charge this customer VAT?
            self.show_vat = self.customer.charge_vat()
            self.invoice.show_vat = self.show_vat
            if self.show_vat:
                self.VAT_value_field.Enable(True)
                self.sellitems_table.show_vat_column()
            else:
                self.VAT_value_field.Enable(False)
                self.sellitems_table.hide_vat_column()
            if self.customer.vat_country_code == vs_cfg["VAT_COUNTRY_CODE"]:
                self.print_check.SetValue(False)
            else:
                self.print_check.SetValue(True)
            self.sellitems_table.calculate_totals()
        except Exception, e:
            error_message("Failure in updating customer", e)


    @button_wrap
    def page_button_pressed(self, event):
        "page button pressed, setup the pdf page size and orientation"

        if not vs_cfg["REPORTLABS_AVAILABLE"]:
            # no pdf capability
            vs_message(self, "Function unavailable - the reportlab module is not installed",
                             "ERROR")
            return

        page_setup_dialog = vatstuff_packages.vs_pdf.page_setup.PageSetupDialog(self, self.invoice.pdf_width,
                                                                    self.invoice.pdf_height)
        if page_setup_dialog.ShowModal() == wx.ID_OK:
            self.invoice.pdf_width = page_setup_dialog.width
            self.invoice.pdf_height = page_setup_dialog.height           
        page_setup_dialog.Destroy()


    @button_wrap
    def pdf_button_pressed(self, event):
        "pdf button - check all fields, if ok, create a pdf form"

        if not vs_cfg["REPORTLABS_AVAILABLE"]:
            # no pdf capability
            vs_message(self, "Function unavailable - the reportlab module is not installed",
                             "ERROR")
            return

        status, message = self.check_fields()
        if not status:
            vs_message(self, message, "ERROR")
            return

        filename= "invoice_%s.pdf" % self.invoice.number

        # Create the save file dialog.
        status, filepath = save_file_as(self, 'pdf', vs_cfg["INVOICES_PDF_FOLDER"], filename)

        if status:
            busy_cursor_on()
            wx.Yield()
            if vatstuff_packages.vs_pdf.invoice_form.create_pdf(filepath, self.invoice):
                busy_cursor_off()
                # And show the file in the default viewer
                if not vatstuff_packages.vs_pdf.pdf_functions.show_pdf_file(filepath):
                    vs_message(self, "pdf file created", "Done")
            else:
                vs_message(self, "Unable to create the pdf file", "ERROR")


    @writecheck
    @button_wrap
    def apply_button_pressed(self, event):
        "Apply button - check and apply all fields"

        status, message = self.check_fields()
        if not status:
            vs_message(self, message, "ERROR")
            return

        # the self.check_fields() function has set the self.invoice object
        # attributes from the form fields

        # Store the transaction in a TransactionForm object
        transaction = form_objects.TransactionForm()
        # This is an invoice transaction(system type 5)
        transaction.system = 5

        transaction.description = "Invoice %s : %s" % (self.invoice.number, self.invoice.customer.name)
        transaction.datestamp = self.invoice.date_of_issue

        # Debit the customers account by the cost plus VAT
        transaction.add(self.invoice.customer.account, debit = self.invoice.cost + self.invoice.vat)

        # Credit the sales account by the cost
        if self.invoice.cost:
            transaction.add(get_account_by_system(6), credit = self.invoice.cost)

        # Credit the VAT balance account by the VAT amount
        if self.invoice.vat:
            transaction.add(get_account_by_system(5), credit=self.invoice.vat)

        status, message = transaction.check()
        if not status:
            vs_message(self, message, "ERROR")
            return

        wx.Yield()
        # Confirm the transaction (function and dialog in gui_functions)
        status, new_description = confirm_transaction(self, transaction)

        if not status:
            vs_message(self, "The Invoice has not been saved", "CANCELLED")
            return

        transaction.description = new_description

        # Must now implement the transaction and save the Invoice
        busy_cursor_on()
        wx.Yield()
        save_invoice.save(self.invoice, transaction)

        big_refresh()
        vs_message(self, "Invoice %s saved" % self.invoice.number, "DONE")
        self.EndModal(wx.ID_OK)


    def cancel_button_pressed(self, event):
        """ Cancel button - Close parent"""
        vs_message(self, "The invoice has not been saved to the database", "Warning")
        self.EndModal(wx.ID_CANCEL)


    #----------- Check and store all fields button

    def check_fields(self):
        """Checks the fields, and fills in the self.invoice object
           returns (True, None) if ok, or (False, "warning message") if not"""
        # Check customer
        customer_selected_list_item = self.customer_field.GetCurrentSelection()
        if customer_selected_list_item < 0:         # blank or unselected
            return False, "A customer must be selected"

        # Set self.invoice attributes
        self.invoice.customer = self.all_customers[customer_selected_list_item]
        self.invoice.customer_address = self.customer_address.current_address
        self.invoice.payment_address  = self.payment_address.current_address
        self.invoice.cus_order_number = self.cus_order_number_field.GetValue().strip()
        self.invoice.payment_instructions = self.payment_instructions_field.GetValue().strip()

        # date_of_issue      Datestamp of the issue of this invoice
        try:
            self.invoice.date_of_issue = wxdate_to_date(self.date_of_issue_field.GetValue())
        except Exception:
            return False, "The date of issue is not valid"

        # date_of_supply     Datestamp of the supply of goods
        try:
            self.invoice.date_of_supply = wxdate_to_date(self.date_of_supply_field.GetValue())
        except Exception:
            return False, "The date of supply not valid"

        # date_of_taxpoint   Datestamp of the taxpoint - when VAT is calculated

        # Initially set the taxpoint as the date of issue of the invoice
        self.invoice.date_of_taxpoint = self.invoice.date_of_issue

        # Unless the goods were supplied more than 14 days earlier
        if self.invoice.date_of_supply < self.invoice.date_of_issue:
            timedelta = self.invoice.date_of_issue - self.invoice.date_of_supply
            if timedelta.days > 14:
                self.invoice.date_of_taxpoint = self.invoice.date_of_supply

        # Invoice Number
        number_string = self.invoice_number_field.GetValue().strip()
        try:
            self.invoice.number = int(number_string)
        except Exception:
            return False, "The invoice number is not valid"

        # Discount
        discount_string = self.discount_rate_field.GetValue().strip()
        try:
            self.invoice.discount = D(discount_string.replace(vs_cfg["CP"], "."))
        except Exception:
            return False, "The discount value is incorrect."

        # Contact
        contact_selected_list_item = self.enq_field.GetCurrentSelection()
        if contact_selected_list_item < 0:
            self.invoice.contact = None
        else:
            self.invoice.contact = self.form_contacts[contact_selected_list_item]

        # Create an item_list of items being invoiced
        # Each list element being itself a dictionary of
        # {"Qty":Decimal, "Description":string, "Unit Cost":Decimal, "Discount":boolean "VAT":Decimal (The % value)}

        status, item_list, message = self.sellitems_table.check_items()
        if not status:
            return False, message
        self.invoice.item_list = item_list

        # print check flag
        self.invoice.print_check = self.print_check.GetValue()

        # Get totals fields

        if self.show_vat:
            vat = self.VAT_value_field.GetValue().strip()
            status, self.invoice.vat = string_to_decimal(vat)
            if not status:
                return False, "The total VAT value is not valid"
        else:
            self.invoice.vat = D("0.00")

        cost = self.cost_value_field.GetValue().strip()
        status, self.invoice.cost = string_to_decimal(cost)
        if not status:
            return False, "The total cost value is not valid"

        # Check these input invoice details are ok via vatstuff_packages.vs_verify.sales
        status, message = new_invoice_check(self.invoice)
        if not status:
            return False, message

        return True, None

