####### VATStuff #######
#
# receive_payment.py  - Lists open Invoices and records customer payment against them
#
# Version : 0.7.0
# Date : 20100926
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#

""" This is a module in the vatstuff_packages.vs_gui package,

Lists open Invoices and records customer payment against them
"""

import decimal

import wx
import wx.grid

from vatstuff_packages.vs_database.grid_objects import *
from vatstuff_packages.vs_database.transactions import *
from vatstuff_packages.vs_database import save_payment

from vatstuff_packages.vs_gui.gui_functions import *
from vatstuff_packages.vs_gui.address_panel import AddressSpin

from vatstuff_packages.vs_verify import form_objects
from vatstuff_packages.vs_verify.sales import received_payment_check

from vatstuff_packages.vs_utilities import *

if "REPORTLABS_AVAILABLE" in vs_cfg and vs_cfg["REPORTLABS_AVAILABLE"]:
    import vatstuff_packages.vs_pdf.page_setup
    import vatstuff_packages.vs_pdf.customerpayment_receipt
    import vatstuff_packages.vs_pdf.pdf_functions

D = decimal.Decimal   # D used to avoid typing


def customer_payment_received(parent, customer):
    """Creates a CustomerPaymentDialog dialog, if a payment is received
       then offer to show a payment receipt dialog"""

    customer_payment_received = CustomerPaymentDialog(parent, customer)

    # If the cursor is busy, unbusy it - as
    # now waiting for a button press
    busy_cursor_off()

    if customer_payment_received.ShowModal() != wx.ID_OK:
        customer_payment_received.Destroy()
        return

    # A customer payment has been made
    customer_payment = customer_payment_received.customer_payment
    customer_payment_received.Destroy()

    # offer to make a receipt
    if customer_payment and vs_yesno(parent, "Payment saved\nDo you wish to create a receipt?", "Done"):
        busy_cursor_on()
        wx.Yield()
        show_payment_receipt(parent, customer_payment)


class CustomerPaymentDialog(vsDialog):
    "Called when a customer sends a payment"

    def __init__(self, parent, customer):
        """Create the Dialog"""
        vsDialog.__init__(self, parent, "Payment received")
        
        # Take a note of the variables
        self.parent = parent

        # customer_payment is the attribute created if a customer payment is made
        self.customer_payment = None

        # get those invoices attached to this customer
        self.customer_open_invoices = customer.get_openinvoices()
        if not self.customer_open_invoices:
            raise VS_ERROR, "No open invoices found for this customer"

        # Store all form contents in an PaymentForm object
        self.pay = form_objects.PaymentForm()
        self.pay.customer = customer

        button_size = (100, vs_cfg["BUTTON_HEIGHT"])

        # Create a vertical sizer to hold everything
        sizer=wx.BoxSizer(wx.VERTICAL)

        customer_text = "Payment received from %s" % customer.name
        sizer.Add(wx.StaticText(self, -1, customer_text), 0, wx.TOP | wx.ALIGN_CENTER, 20)

        invoice_sizer=wx.BoxSizer(wx.HORIZONTAL)

        inv_text = """\
Select one or more invoices for
which this payment has been received
(Hold down Ctrl and left mouse click
to choose multiple invoices)"""
        invoice_sizer.Add(wx.StaticText(self, -1, inv_text), 0, wx.ALL | wx.ALIGN_RIGHT, 20)

        choice_list =[]
        for invoice in self.customer_open_invoices:
            item = "Invoice: %s  %s" % (invoice.number, invoice.datestamp.strftime("%d %b %Y"))
            choice_list.append(item)


        self.invoice_list_field = wx.ListBox(self, -1, choices = choice_list, style = wx.LB_MULTIPLE)
        self.invoice_list_field.SetToolTipString("Set one or more invoices the payment covers")
        invoice_sizer.Add(self.invoice_list_field, 0, wx.ALL | wx.ALIGN_LEFT, 20)

        sizer.Add(invoice_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        help_text = """\
Specify the cash or bank account to which this money will be placed.
"""
        sizer.Add(wx.StaticText(self, -1, help_text), 0, wx.TOP | wx.ALIGN_CENTER, 20)

        # And a grid sizer to hold payment details
        payment_sizer = wx.FlexGridSizer(rows=3, cols=2, hgap=20, vgap=10)

        # Account
        payment_sizer.Add(wx.StaticText(self, -1, "Account:"), 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.all_accounts = Account.get_non_system()

        if not self.all_accounts:
            raise VS_ERROR, "No accounts are available"

        self.payment_account_field = wx.Choice(self, -1, size=(200, vs_cfg["BUTTON_HEIGHT"]),
                                               choices=[ a.fullname for a in self.all_accounts ])

        # Get the last cash account used - and pre-select it
        last_account = get_last_cash_account()
        if last_account:
            try:
                key = self.all_accounts.index(last_account)
            except ValueError:
                key = 0
            self.payment_account_field.SetSelection(key)

        self.payment_account_field.SetToolTipString("Set a bank or cash account")
        payment_sizer.Add(self.payment_account_field, 0, wx.ALIGN_LEFT)

        # Customer reference
        payment_sizer.Add(wx.StaticText(self, -1, "Customer reference:"), 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.customer_reference_field = wx.TextCtrl(self, -1, "", size=(200, vs_cfg["BUTTON_HEIGHT"]))
        self.customer_reference_field.SetToolTipString("Customers reference")
        self.customer_reference_field.SetMaxLength(40)
        payment_sizer.Add(self.customer_reference_field, 0, wx.ALIGN_LEFT)

        # Date
        payment_sizer.Add(wx.StaticText(self, -1, "Date:"), 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.datestamp = wx.DateTime.Now()
        self.payment_date_field = wx.DatePickerCtrl(self, -1,
                                                    self.datestamp, 
                                                    size=(150, vs_cfg["BUTTON_HEIGHT"]),
                                                    style=wx.DP_DROPDOWN | wx.DP_SHOWCENTURY)
        self.payment_date_field.SetToolTipString("The date of the payment")
        payment_sizer.Add(self.payment_date_field, 0, wx.ALIGN_LEFT)

        # Amount
        payment_sizer.Add(wx.StaticText(self, -1, "Payment:"), 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.amount_field = wx.TextCtrl(self, -1, vs_cfg["SYMBOL_ZERO_STRING"], style=wx.ALIGN_RIGHT, size=(150, vs_cfg["BUTTON_HEIGHT"]))
        self.amount_field.SetToolTipString("Amount to be paid")
        self.amount_field.SetMaxLength(12)
        payment_sizer.Add(self.amount_field, 0, wx.ALIGN_LEFT)

        sizer.Add(payment_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # If there is only one invoice in the choice list, auto select
        # it, and set the amount in the amount field
        if len(choice_list) == 1:
            self.invoice_list_field.SetSelection(0)
            self.amount_field.SetValue(to_currency_string(self.customer_open_invoices[0].outstanding))

        # And a horizontal sizer to hold buttons
        button_sizer = wx.BoxSizer(wx.HORIZONTAL)

        # Cancel button
        self.cancel_button = wx.Button(self, wx.ID_CANCEL, size=button_size)
        self.cancel_button.SetToolTipString("Click to cancel.")
        button_sizer.Add(self.cancel_button, 0, wx.RIGHT, 40)

        # Apply button
        self.apply_button = wx.Button(self, wx.ID_APPLY, size=button_size)
        self.apply_button.SetToolTipString("Click to apply payment.")
        button_sizer.Add(self.apply_button, 0)

        sizer.Add(button_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        self.Bind(wx.EVT_BUTTON, self.cancel_button_pressed, self.cancel_button)
        self.Bind(wx.EVT_BUTTON, self.apply_button_pressed, self.apply_button)
        self.Bind(wx.EVT_LISTBOX, self.selection_made, self.invoice_list_field)

        # Set the size to the panel
        self.SetSizer(sizer)
        sizer.SetSizeHints(self)


    def cancel_button_pressed(self, event):
        "Cancel button pressed"
        vs_message(self, "No payment has been made", "Warning")
        self.EndModal(wx.ID_CANCEL)


    def selection_made(self, event):
        "Set amount field from the list of invoices selected"
        try:
            selections = self.invoice_list_field.GetSelections()
            if not selections:
                self.amount_field.SetValue(vs_cfg["SYMBOL_ZERO_STRING"])
            invoices = []
            for selection in selections:
                invoices.append(self.customer_open_invoices[selection])
            unpaid = D("0.00")
            for invoice in invoices:
                unpaid += invoice.outstanding
            self.amount_field.SetValue(to_currency_string(unpaid))
        except Exception, e:
            error_message("Failure in selecting invoices", e)


    @writecheck
    @button_wrap
    def apply_button_pressed(self, event):
        "Apply button - check fields and submit the payment transaction"

        status, message = self.check_fields()
        if not status:
            vs_message(self, message, "ERROR")
            return

        # Store the transaction in a TransactionForm object
        transaction = form_objects.TransactionForm()
        # This is an customer payment transaction(system type 6)
        transaction.system = 6

        transaction.description = "Payment from %s" % self.pay.customer.name
        transaction.datestamp = self.pay.datestamp

        # Either a bank or cash account, debit by the amount 
        transaction.add(self.pay.account, debit = self.pay.amount)
    
        # The customers account, credit by the amount
        transaction.add(self.pay.customer.account, credit = self.pay.amount)

        status, message = transaction.check()
        if not status:
            vs_message(self, message, "ERROR")
            return

        wx.Yield()
        # Confirm the transaction (function and dialog in gui_functions)
        status, new_description = confirm_transaction(self, transaction)

        if not status:
            vs_message(self, "The payment has not been saved", "CANCELLED")
            return

        transaction.description = new_description

        # Must now implement the transaction and save the payment
        busy_cursor_on()
        wx.Yield()
        self.customer_payment = save_payment.save(self.pay, transaction)

        # and refresh everything
        big_refresh()
        self.EndModal(wx.ID_OK)


    #----------- Check and store all fields

    def check_fields(self):
        """Checks the fields, and fills in the self.pay object
           returns (True, None) if ok, or (False, "warning message") if not"""

        # Get the list of invoices
        selections = self.invoice_list_field.GetSelections()
        # At least one must be selected
        if not selections:
            return False, "At least one invoice must be selected"

        invoices = [ self.customer_open_invoices[selection] for selection in selections ]

        # Get the payement date
        self.pay.datestamp = wxdate_to_date(self.payment_date_field.GetValue())

        # Get the customer reference
        self.pay.customer_reference = self.customer_reference_field.GetValue().strip()

        # The cash or bank account used to record this purchase
        account_selected_list_item = self.payment_account_field.GetCurrentSelection()
        if account_selected_list_item < 0:         # blank or unselected
            self.pay.account = None
        else:
            self.pay.account = self.all_accounts[account_selected_list_item]

        # Get amount paid field
        amount_string = self.amount_field.GetValue().strip()
        status, amount = string_to_decimal(amount_string)
        if not status:
            return False, "The amount received is not valid"

        self.pay.amount = amount

        # Check these input payment details are ok via vatstuff_packages.vs_verify.sales
        status, message = received_payment_check(self.pay, invoices)
        if not status:
            return False, message

        return True, None


#####  create a receipt dialog

# The PaymentReceiptDialog is used to create a payment receipt - giving the user a chance to
# choose the address where the receipt will go to


def show_payment_receipt(parent, payment):
    """Creates a ShowPaymentReceiptDialog dialog"""

    show_payment_receipt_dialog = ShowPaymentReceiptDialog(parent, payment)

    # If the cursor is busy, unbusy it - as
    # now waiting for a button press
    busy_cursor_off()

    show_payment_receipt_dialog.ShowModal()
    show_payment_receipt_dialog.Destroy()


class ShowPaymentReceiptDialog(vsDialog):
    """Called to create a payment receipt"""

    def __init__(self, parent, payment):
        """Create the Dialog"""
        vsDialog.__init__(self, parent, "Payment Receipt")
        
        # Take a note of the variables
        self.payment = payment

        button_size = (100, vs_cfg["BUTTON_HEIGHT"])

        # Create a vertical sizer to hold everything
        sizer=wx.BoxSizer(wx.VERTICAL)

        # And a horizontal sizer to hold customer address and payment details
        details_sizer = wx.BoxSizer(wx.HORIZONTAL)

        # A vertical sizer holds the customer address
        customer_sizer=wx.BoxSizer(wx.VERTICAL)

        customer_sizer.Add(wx.StaticText(self, -1, "Customer address :"), 0, wx.ALIGN_LEFT)

        self.address_field = AddressSpin(self, payment.customer)
        customer_sizer.Add(self.address_field, 0, wx.ALIGN_CENTER)

        # Add the customer_sizer to the details_sizer
        details_sizer.Add(customer_sizer, 0, wx.LEFT | wx.ALIGN_RIGHT, 20)

        # Put payment details in a grid sizer
        pay_sizer=wx.FlexGridSizer(rows=5, cols=2, hgap=20, vgap=10)

        # customer
        pay_sizer.Add(wx.StaticText(self, -1, "Customer :"), 0, wx.ALIGN_RIGHT)
        customer_text = payment.customer.name
        pay_sizer.Add(wx.StaticText(self, -1, customer_text), 0, wx.ALIGN_LEFT)

        # date
        pay_sizer.Add(wx.StaticText(self, -1, "Payment date :"), 0, wx.ALIGN_RIGHT)
        date_text = payment.datestamp.strftime("%d %b %Y")
        pay_sizer.Add(wx.StaticText(self, -1, date_text), 0, wx.ALIGN_LEFT)

        # payment amount
        pay_sizer.Add(wx.StaticText(self, -1, "Payment amount :"), 0, wx.ALIGN_RIGHT)
        amount_text = to_currency_string(payment.amount)
        pay_sizer.Add(wx.StaticText(self, -1, amount_text), 0, wx.ALIGN_LEFT)

        # payment account
        pay_sizer.Add(wx.StaticText(self, -1, "Paid to account :"), 0, wx.ALIGN_RIGHT)
        account_text = payment.account.fullname
        pay_sizer.Add(wx.StaticText(self, -1, account_text), 0, wx.ALIGN_LEFT)

        # Transaction ID
        pay_sizer.Add(wx.StaticText(self, -1, "Transaction ID :"), 0, wx.ALIGN_RIGHT)
        transaction_text = str(payment.transaction.id)
        pay_sizer.Add(wx.StaticText(self, -1, transaction_text), 0, wx.ALIGN_LEFT)

        details_sizer.Add(pay_sizer, 0, wx.ALL | wx.ALIGN_LEFT, 20)

        sizer.Add(details_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # buttons

        button_sizer=wx.BoxSizer(wx.HORIZONTAL)

        # page setup button
        self.page_button = wx.Button(self, label="Page setup", size=button_size)
        self.page_button.SetToolTipString("Click to set up the pdf page size and orientation")   
        button_sizer.Add(self.page_button, 0, wx.RIGHT, 20)

        # pdf button
        self.pdf_button = wx.Button(self, label="Create pdf", size=button_size)
        self.pdf_button.SetToolTipString("Click to create a pdf file receipt.")
        button_sizer.Add(self.pdf_button, 0, wx.RIGHT, 20)

        # Close button
        self.close_button = wx.Button(self, wx.ID_CLOSE, size=button_size)
        self.close_button.SetToolTipString("Click to close.")
        button_sizer.Add(self.close_button, 0)

        sizer.Add(button_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # Set the size to the dialog frame
        self.SetSizer(sizer)
        sizer.SetSizeHints(self)

        self.Bind(wx.EVT_BUTTON, self.close_button_pressed, self.close_button)
        self.Bind(wx.EVT_BUTTON, self.page_button_pressed, self.page_button)
        self.Bind(wx.EVT_BUTTON, self.pdf_button_pressed, self.pdf_button)


    def close_button_pressed(self, event):
        """Close the dialog"""
        self.EndModal(wx.ID_OK)


    @button_wrap
    def page_button_pressed(self, event):
        "page button pressed, setup the pdf page size and orientation"
        vatstuff_packages.vs_pdf.page_setup.page_setup_dialog(self)


    @writecheck
    @button_wrap
    def pdf_button_pressed(self, event):
        "Create a pdf file"

        # This is one of the few times where the pdf button actually writes a
        # value to the database. Acting more like an Apply button

        # Get the address field
        self.payment.customer_address = self.address_field.current_address

        if not vs_cfg["REPORTLABS_AVAILABLE"]:
            # no pdf capability
            vs_message(self, "Function unavailable - the reportlab module is not installed",
                             "ERROR")
            return


        # default filename
        filename= "payment_receipt_%s_%s.pdf" % (self.payment.datestamp.isoformat(), self.payment.transaction.id)

        # Create the save file dialog.
        status, filepath = save_file_as(self, 'pdf', vs_cfg["REPORTS_PDF_FOLDER"], filename)

        if status:
            if vatstuff_packages.vs_pdf.customerpayment_receipt.create_pdf(filepath, self.payment):
                # And show the file in the default viewer
                if not vatstuff_packages.vs_pdf.pdf_functions.show_pdf_file(filepath):
                    vs_message(self, "pdf file created", "Done")
            else:
                vs_message(self, "Unable to create the pdf file", "ERROR")





