####### VATStuff #######
#
# view_creditnotes.py  - lists credit notes
#
# Version : 0.6
# Date : 20100719
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#


""" This is a module in the vatstuff_packages.vs_gui package, and Lists Credit Notes

It is run as part of the vatstuff gui interface.
"""

import datetime
import decimal

import wx
import wx.grid

from vatstuff_packages.vs_database.grid_objects import *
from vatstuff_packages.vs_database.transactions import *

from vatstuff_packages.vs_gui.sales import receive_payment

from vatstuff_packages.vs_gui.gui_functions import *
from vatstuff_packages.vs_gui.reports import make_report

from vatstuff_packages.vs_database import save_creditnote

from vatstuff_packages.vs_utilities import *


if "REPORTLABS_AVAILABLE" in vs_cfg and vs_cfg["REPORTLABS_AVAILABLE"]:
    import vatstuff_packages.vs_pdf.page_setup
    import vatstuff_packages.vs_pdf.creditnote_form
    import vatstuff_packages.vs_pdf.pdf_functions

D = decimal.Decimal   # D used to avoid typing


def view_all_creditnotes_frame(creditnote=None):
    """Show the ViewAllCreditnotesFrame which lists all creditnotes,
       ensuring that the creditnote given as argument is displayed.
       If None is given, display the last entry"""

    # No try:exception statement - allows the caller to catch exceptions

    # Check the frame is not already open
    for frm in vs_cfg["TRANSACTION_REFRESH"]:
        if isinstance(frm, ViewAllCreditnotesFrame):
            frm.Raise()
            break
    else:
        # top_frame() is the top frame, which will be the parent
        frm = ViewAllCreditnotesFrame(top_frame())
        frm.Show()

    if creditnote:
        frm.display_row(creditnote.gridrow)
    else:
        frm.display_last()


class ViewAllCreditnotesFrame(vsFrame):
    """Frame to list all creditnotes"""

    def __init__(self, parent):
        "Create the frame to show a list of credit notes"
        vsFrame.__init__(self, parent, title="Credit Notes")

        # Take a note of the parent
        self.parent = parent

        self.creditnotes_grid = CreditnotesGrid()   # class defined in vatstuff_packages.vs_database.grid_objects
        # Create a database object of type wx.grid.PyGridTableBase
        # it opens database connections, and gets information for
        # the grid object

        try:
            self.creditnotes_grid_rows = self.creditnotes_grid.GetNumberRows()
            # Create the ViewCreditnotesPanel
            self.view_creditnotes_panel = ViewCreditnotesPanel(self, self.creditnotes_grid)
        except Exception, e:
            # close database connection, and re-raise the exception
            self.creditnotes_grid.CloseConnection()
            raise

        # Get the panel size
        panel_size = self.view_creditnotes_panel.GetMinSize()

        # Set minimum size equal to the panel
        self.SetMinSize(panel_size)
        self.SetClientSize(panel_size)

        # update the list of windows which need a refresh if
        # a transaction occurs, by adding self
        vs_cfg["TRANSACTION_REFRESH"].append(self)

        # bind the close event
        self.Bind(wx.EVT_CLOSE, self.onclose)


    def refresh(self):
        "Refreshes the grid"
        new_num_rows = self.creditnotes_grid.GetNumberRows()
        if new_num_rows > self.creditnotes_grid_rows:
            # More rows are in the database, insert new rows on the grid
            msg = wx.grid.GridTableMessage(self.creditnotes_grid,          # The table
                  wx.grid.GRIDTABLE_NOTIFY_ROWS_APPENDED,                  # has extra rows
                  new_num_rows - self.creditnotes_grid_rows                # the number of rows
                  ) 
            self.view_creditnotes_panel.grid.ProcessTableMessage(msg)      # The grid associated with the table
            self.creditnotes_grid_rows = new_num_rows                      # record the new number of rows
        elif new_num_rows < self.creditnotes_grid_rows:
            # Less rows are in the database, delete rows on the grid
            msg = wx.grid.GridTableMessage(self.creditnotes_grid,          # The table
                  wx.grid.GRIDTABLE_NOTIFY_ROWS_DELETED,                   # has fewer rows
                  new_num_rows,                                            # delete from row
                  self.creditnotes_grid_rows - new_num_rows                # delete the number of rows
                  ) 
            self.view_creditnotes_panel.grid.ProcessTableMessage(msg)      # The grid associated with the table
            self.creditnotes_grid_rows = new_num_rows                      # record the new number of rows

        # grid associated with table can also be obtained with
        # self.creditnotes_grid.GetView().ProcessTableMessage(msg)

        self.view_creditnotes_panel.grid.ForceRefresh()


    def display_row(self, row):
        "Calls the display_row function in the view_creditnotes_panel"
        self.view_creditnotes_panel.display_row(row)

    def display_last(self):
        "Calls the display_last function in the view_creditnotes_panel"
        self.view_creditnotes_panel.display_last()

    def onclose(self, evt):
        "Shutdown the database transaction"
        self.creditnotes_grid.CloseConnection()
        # Decrement the refresh list
        vs_cfg["TRANSACTION_REFRESH"].remove(self)
        self.Destroy()


# ----------------------View creditnotes panel

class ViewCreditnotesPanel(wx.Panel):
    """Show the grid of credit notes"""

    def __init__(self, parent, creditnotes_grid):
        """Create the panel"""
        wx.Panel.__init__(self, parent)

        # Take a note of the parent, and the creditnotes_grid
        self.parent = parent
        self.creditnotes_grid = creditnotes_grid
        button_size = (100, vs_cfg["BUTTON_HEIGHT"])

        # This is the creditnote double-clicked on and shown, initially None
        self.creditnote = None

        # Create the sizer for the panel
        sizer=wx.BoxSizer(wx.VERTICAL)

        self.description_text=wx.StaticText(self, -1, "Double click on a credit note to view it")
        sizer.Add(self.description_text, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # Create a grid
        self.grid = wx.grid.Grid(self, size=(900,200))

        self.grid.SetDefaultColSize(100, True)
        self.grid.SetDefaultRowSize(vs_cfg["BUTTON_HEIGHT"], False)
        self.grid.SetColMinimalAcceptableWidth(100)
        self.grid.EnableEditing(False)

        self.grid.SetDefaultCellAlignment(wx.ALIGN_LEFT, wx.ALIGN_CENTER)

        # Assign the creditnotes_grid to the grid
        self.grid.SetTable(self.creditnotes_grid, True)

        # Set the description column size
        self.grid.SetColSize(2, 350)

        # Set the row label size
        self.grid.SetRowLabelSize(100)

        sizer.Add(self.grid, 1, wx.ALL | wx.ALIGN_CENTER | wx.EXPAND, 20)

        # buttons
        button_sizer=wx.BoxSizer(wx.HORIZONTAL)

        # ------------------- Close button
        self.close_button = wx.Button(self, wx.ID_CLOSE, size=button_size)
        self.close_button.SetToolTipString("Click to close this window")
        button_sizer.Add(self.close_button, 0, wx.RIGHT, 20)


        # ------------------- Report button
        self.report_button = wx.Button(self, label="Report", size=button_size)
        self.report_button.SetToolTipString("Click to generate a report between two dates")
        button_sizer.Add(self.report_button, 0)

        sizer.Add(button_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # Set the sizer to the panel
        self.SetSizer(sizer)
        sizer.SetSizeHints(self)
        
        # Bind left mouse double - click
        self.Bind(wx.grid.EVT_GRID_CELL_LEFT_DCLICK, self.show_creditnote, self.grid)

        # Bind the close and report buttons
        self.Bind(wx.EVT_BUTTON, self.close_button_pressed, self.close_button)
        self.Bind(wx.EVT_BUTTON, self.report_button_pressed, self.report_button)


    def close_button_pressed(self, event):
        "Close parent"
        self.parent.Close()
        return


    @button_wrap
    def report_button_pressed(self, event):
        """Get the two dates between which the report is generated
           create the creditnotes sent report
           and then display it in a report frame"""

        help_text = """\
A report will be generated showing credit notes
issued to customers between the two dates:"""

        status, from_date, to_date = two_dates(self, "Credit Notes Report", help_text)
        if not status:
            return
        busy_cursor_on()
        wx.Yield()

        filename = "creditnotes_to_customers%s" % to_date.isoformat()
        make_report.make_report_and_frame("creditnotes",
                                          "Credit Notes sent to customers",
                                          filename,
                                          from_date,
                                          to_date)


    def display_row(self, row):
        """Makes row visible"""
        last_row = self.creditnotes_grid.GetNumberRows() - 1
        if row > last_row:
            row = last_row
        if row < 0:
            row = 0
        self.grid.SetGridCursor(row, 0)
        self.grid.MakeCellVisible(row, 0)


    def display_last(self):
        "Makes the last row visible"
        last_row = self.creditnotes_grid.GetNumberRows() - 1
        self.grid.SetGridCursor(last_row, 0)
        self.grid.MakeCellVisible(last_row, 0)


    @button_wrap
    def show_creditnote(self, event):
        "Display the credit note in response to a double click"
        row = event.GetRow()
        # get creditnote from its gridrow number
        creditnote = CreditNote.from_gridrow(row)
        show_creditnote_frame(creditnote)


# ------------------------ Show the credit note frame

def show_creditnote_frame(creditnote):
    "Display the ShowCreditnoteFrame"

    if creditnote.data_deleted():
        # Data regarding this creditnote is unavailable
        vs_message(top_frame(), """\
Invoice information relating to this credit
note has been deleted, and therefore further
credit note details cannot be shown.""", "Data Deleted")
        debug_log("data deleted - therefore cannot show creditnote id: %s" % creditnote.id)
        return

    debug_log("showing creditnote id: %s" % creditnote.id)

    # if a ShowCreditnoteFrame with this creditnote is already open -show it
    for frm in vs_cfg["TRANSACTION_REFRESH"]:
        if isinstance(frm, ShowCreditnoteFrame):
            if frm.creditnote == creditnote:
                frm.Raise()
                return

    # the frame is not open, so create one showing this creditnote
    show_creditnote_frame= ShowCreditnoteFrame(top_frame(), creditnote)
    show_creditnote_frame.Show()


class ShowCreditnoteFrame(vsFrame):
    "Called when a creditnote is to be viewed"

    def __init__(self, parent, creditnote):
        "Create the frame - holding a ShowCreditnotePanel"
        frame_title = "Credit Note : %i" % creditnote.number
        vsFrame.__init__(self, parent, title=frame_title)

        # used to test which creditnote frame is open
        self.creditnote = creditnote

        # create the panel
        self.show_creditnote_panel = ShowCreditnotePanel(self, self.creditnote)

        # Get the panel size
        panel_size = self.show_creditnote_panel.GetMinSize()

        # Set minimum size equal to the panel
        self.SetMinSize(panel_size)

        # set the frame size to the panel size
        self.SetClientSize(panel_size)

        # update the list of windows which need a refresh if
        # a transaction occurs, by adding self
        vs_cfg["TRANSACTION_REFRESH"].append(self)

        # bind the close event
        self.Bind(wx.EVT_CLOSE, self.onclose)


    def refresh(self):
        "refresh this frame, by calling the refresh function in its panel"
        self.show_creditnote_panel.refresh()


    def onclose(self, evt):
        # Decrement the refresh list
        vs_cfg["TRANSACTION_REFRESH"].remove(self)
        self.Destroy()


# ----------------------ShowCreditnotePanel

class ShowCreditnotePanel(wx.Panel):
    """The panel of the ShowCreditnote frame"""

    def __init__(self, parent, creditnote):
        """Create the panel"""
        wx.Panel.__init__(self, parent)
        
        # Take a note of the variables
        self.parent = parent
        self.creditnote = creditnote

        button_size = (150, vs_cfg["BUTTON_HEIGHT"])

        # Create a vertical sizer to hold everything
        self.sizer=wx.BoxSizer(wx.VERTICAL)

        # And a horizontal sizer to hold creditnote details
        creditnote_sizer = wx.BoxSizer(wx.HORIZONTAL)

        if self.creditnote.customer_address:
            customer_text = "Customer: %s\n%s" % (self.creditnote.customer.name,
                                                  self.creditnote.customer_address.lines_string)
        else:
            customer_text = "Customer: %s" % self.creditnote.customer.name

        creditnote_sizer.Add(wx.StaticText(self, -1, customer_text), 0, wx.RIGHT | wx.ALIGN_TOP, 80)


        number_date_sizer = wx.BoxSizer(wx.VERTICAL)

        creditnote_number_field = wx.StaticText(self, -1, "Credit Note Number: %i" % self.creditnote.number)
        number_date_sizer.Add(creditnote_number_field, 0, wx.ALIGN_LEFT)

        self.creditnote_date_field = wx.StaticText(self, -1,
                                           self.creditnote.datestamp.strftime("Credit Note Date: %d %b %Y"))
        number_date_sizer.Add(self.creditnote_date_field, 0,  wx.TOP | wx.ALIGN_LEFT, 5)

        if self.creditnote.invoice:
            invoice_number_field = wx.StaticText(self, -1, "Invoice Number: %i" % self.creditnote.invoice.number)
            number_date_sizer.Add(invoice_number_field, 0,  wx.TOP | wx.ALIGN_LEFT, 5)

            invoice_date_field = wx.StaticText(self, -1,
                                  self.creditnote.invoice.date_of_issue.strftime("Invoice Date: %d %b %Y"))
            number_date_sizer.Add(invoice_date_field, 0,  wx.TOP | wx.ALIGN_LEFT, 5)

            if self.creditnote.invoice.cus_order_number:
                customer_order = "Customer order: %s" % self.creditnote.invoice.cus_order_number
                customer_order_field = wx.StaticText(self, -1, customer_order)
                number_date_sizer.Add(customer_order_field, 0,  wx.TOP | wx.ALIGN_LEFT, 5)

        creditnote_sizer.Add(number_date_sizer, 0, wx.RIGHT | wx.ALIGN_TOP, 80)

        # Put costs in a grid sizer
        cost_sizer=wx.FlexGridSizer(rows=4, cols=2, hgap=20, vgap=10)

        cost_string = unicode(Currency(self.creditnote.cost))
        vat_string = unicode(Currency(self.creditnote.vat))
        total = self.creditnote.vat + self.creditnote.cost
        total_string = unicode(Currency(total))

        cost_sizer.Add(wx.StaticText(self, -1, "Credit Note: "), 0, wx.ALIGN_RIGHT)
        cost_sizer.Add(wx.StaticText(self, -1, "amounts"), 0, wx.ALIGN_RIGHT)

        cost_sizer.Add(wx.StaticText(self, -1, "Cost:"), 0, wx.ALIGN_RIGHT)
        cost_sizer.Add(wx.StaticText(self, -1, cost_string), 0, wx.ALIGN_RIGHT)
        cost_sizer.Add(wx.StaticText(self, -1, "VAT:"), 0, wx.ALIGN_RIGHT)
        cost_sizer.Add(wx.StaticText(self, -1, vat_string), 0, wx.ALIGN_RIGHT)
        cost_sizer.Add(wx.StaticText(self, -1, "Total:"), 0, wx.ALIGN_RIGHT)
        cost_sizer.Add(wx.StaticText(self, -1, total_string), 0, wx.ALIGN_RIGHT)

        creditnote_sizer.Add(cost_sizer, 0, wx.ALIGN_TOP)

        self.sizer.Add(creditnote_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # Another horizontal sizer to hold reason for credit note
        status_sizer = wx.BoxSizer(wx.HORIZONTAL)

        status_sizer.Add(wx.StaticText(self, -1, "Reason for Credit Note: "), 0, wx.RIGHT | wx.ALIGN_CENTER_VERTICAL, 20)

        self.reason_field = wx.TextCtrl(self, -1, self.creditnote.reason, size=(400, vs_cfg["BUTTON_HEIGHT"]))
        self.reason_field.SetToolTipString("This text appears on the creditnote pdf")
        self.reason_field.SetMaxLength(60)
        status_sizer.Add(self.reason_field, 0, wx.RIGHT, 20)

        self.reason_button = wx.Button(self, label="Change", size=button_size)
        self.reason_button.SetToolTipString("Click to change the reason for the creditnote")
        status_sizer.Add(self.reason_button, 0)

        self.sizer.Add(status_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # Create a Tabbed Notebook
        self.tabbed_notebook = CreditnoteNotebook(self, self.creditnote)
        self.sizer.Add(self.tabbed_notebook, 1, wx.ALL | wx.ALIGN_CENTER | wx.EXPAND, 10)

        # Create button sizer for the printer and close button
        button_sizer=wx.BoxSizer(wx.HORIZONTAL)

        # close button
        self.close_button = wx.Button(self, wx.ID_CLOSE, size=button_size)
        self.close_button.SetToolTipString("Click to close this window.")
        button_sizer.Add(self.close_button, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # page setup button
        self.page_button = wx.Button(self, label="Page setup", size=button_size)
        self.page_button.SetToolTipString("Click to set up the pdf page size and orientation")
        button_sizer.Add(self.page_button, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # pdf button
        self.pdf_button = wx.Button(self, label="Creditnote pdf", size=button_size)
        self.pdf_button.SetToolTipString("Click to create a pdf file of the credit note.")
        button_sizer.Add(self.pdf_button, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # add the button_sizer to the main vertical sizer
        self.sizer.Add(button_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # Set the size to the panel
        self.SetSizer(self.sizer)

        panel_size = self.sizer.GetMinSize()
        self.SetMinSize(panel_size)

        self.Bind(wx.EVT_BUTTON, self.reason_button_pressed, self.reason_button)
        self.Bind(wx.EVT_BUTTON, self.close_button_pressed, self.close_button)
        self.Bind(wx.EVT_BUTTON, self.page_button_pressed, self.page_button)
        self.Bind(wx.EVT_BUTTON, self.pdf_button_pressed, self.pdf_button)


    @writecheck
    @button_wrap
    def reason_button_pressed(self, event):
        """change the creditnote reason"""
        reason = self.reason_field.GetValue().strip()
        if self.creditnote.reason == reason:
            # No change in the reason - just return
            vs_message(self, "There was no change to the reason text", "WARNING")
            return
        self.creditnote.reason = reason
        # big_refresh(acc_tree=False) - dont think a refresh is necessary here
        info_log("Credit note id : %i reason text changed" % self.creditnote.id)
        vs_message(self, "Reason text changed", "DONE")


    def close_button_pressed(self, event):
        "close button pressed, close window"
        self.parent.Close()


    @button_wrap
    def page_button_pressed(self, event):
        "page button pressed, setup the pdf page size and orientation"

        if not vs_cfg["REPORTLABS_AVAILABLE"]:
            # no pdf capability
            vs_message(self, "Function unavailable - the reportlab module is not installed",
                             "ERROR")
            return

        page_setup_dialog = vatstuff_packages.vs_pdf.page_setup.PageSetupDialog(self, self.creditnote.pdf_width,
                                                                    self.creditnote.pdf_height)
        if page_setup_dialog.ShowModal() == wx.ID_OK:
            self.creditnote.pdf_width = page_setup_dialog.width
            self.creditnote.pdf_height = page_setup_dialog.height           
        page_setup_dialog.Destroy()


    @button_wrap
    def pdf_button_pressed(self, event):
        "pdf button - check all fields, if ok, create a pdf form"

        if not vs_cfg["REPORTLABS_AVAILABLE"]:
            # no pdf capability
            vs_message(self, "Function unavailable - the reportlab module is not installed",
                             "ERROR")
            return

        filename= "creditnote_%s.pdf" % self.creditnote.number

        # Create the save file dialog.
        status, filepath = save_file_as(self, 'pdf', vs_cfg["CREDITNOTES_PDF_FOLDER"], filename)

        if status:
            busy_cursor_on()
            wx.Yield()
            if vatstuff_packages.vs_pdf.creditnote_form.create_pdf(filepath, self.creditnote):
                busy_cursor_off()
                # And show the file in the default viewer
                if not vatstuff_packages.vs_pdf.pdf_functions.show_pdf_file(filepath):
                    vs_message(self, "pdf file created", "Done")
            else:
                vs_message(self, "Unable to create the pdf file", "ERROR")


    def refresh(self):
        "refreshes the credit note date, and notebook"
        # Refresh date
        self.creditnote_date_field.SetLabel(self.creditnote.datestamp.strftime("Credit Note Date: %d %b %Y"))

        # credit note reason field
        self.reason_field.SetValue(self.creditnote.reason)

        # Refresh the tabbed notebook
        self.tabbed_notebook.refresh()

        # Re-do the layout
        self.sizer.Layout()


class CreditnoteNotebook(wx.Notebook):
    "Shows creditnote items, and transaction"

    def __init__(self, parent, creditnote):
        "Create the tabbed notebook, and each panel beneath each tab"
        wx.Notebook.__init__(self, parent, size = (900,200))

        # Note the notebook is set to a specific size, since the grid size is
        # variable, this forces the grid to have scroll bars if bigger

        self.parent = parent
        self.creditnote = creditnote
       
        # show the items being credited
        self.show_items = ShowItemsPanel(self, self.creditnote)
        # Add show_items to the Notebook
        self.AddPage(self.show_items, " Credit Note Items ")

        # show the creditnote transactions
        self.show_transactions = ShowTransactionsPanel(self, self.creditnote)
        # Add show_transactions to the Notebook
        self.AddPage(self.show_transactions, " Transactions ")

    def refresh(self):
        self.show_transactions.refresh()


class ShowItemsPanel(wx.Panel):
    """The panel of the ShowItemsTable grid"""

    def __init__(self, parent, creditnote):
        """Create the panel"""
        wx.Panel.__init__(self, parent)

        # Create a vertical sizer to hold the grid
        sizer=wx.BoxSizer(wx.VERTICAL)

        # show the itms being credited
        show_items_table = ShowItemsTable(self, creditnote)

        sizer.Add(show_items_table, 1, wx.ALIGN_CENTER | wx.EXPAND)
        # Set the size to the panel
        self.SetSizer(sizer)


class ShowItemsTable(wx.grid.Grid):

    def __init__(self, parent, creditnote):
        wx.grid.Grid.__init__(self, parent, style=wx.WANTS_CHARS)

        self.parent = parent
        self.creditnote = creditnote

        discount = "%s%%" % str(self.creditnote.discount).replace(".", vs_cfg["CP"])
        no_discount = "0%s0%%" % vs_cfg["CP"]

        num_cols = 6

        amount_list = self.creditnote.get_amount()
        item_list = self.creditnote.item_list
        num_rows = len(item_list)

        self.CreateGrid(num_rows, num_cols)
        self.EnableEditing(False)

        self.SetDefaultRowSize(vs_cfg["BUTTON_HEIGHT"], False)
        self.SetDefaultCellAlignment(wx.ALIGN_LEFT, wx.ALIGN_CENTER)

        # create colum labels
        self.SetColLabelValue(0, "Qty")
        self.SetColLabelValue(1, "Description")
        self.SetColLabelValue(2, "Unit Cost")
        self.SetColLabelValue(3, "Amount")
        self.SetColLabelValue(4, "Discount")
        self.SetColLabelValue(5, "VAT")

        # Set the Quantity, Discount and VAT columns as centre aligned
        attr_center = wx.grid.GridCellAttr()
        attr_center.SetAlignment(wx.ALIGN_CENTER, wx.ALIGN_CENTER)

        self.SetColAttr(0, attr_center)
        self.SetColAttr(4, attr_center)
        self.SetColAttr(5, attr_center)

        # Set the colums showing money as right aligned
        attr_right = wx.grid.GridCellAttr()
        attr_right.SetAlignment(wx.ALIGN_RIGHT, wx.ALIGN_CENTER)
        self.SetColAttr(2, attr_right)
        self.SetColAttr(3, attr_right)

        self.SetColSize(0, 100)
        self.SetColSize(1, 300)
        self.SetColSize(2, 100)
        self.SetColSize(3, 100)
        self.SetColSize(4, 100)
        self.SetColSize(5, 100)


        self.SetColMinimalAcceptableWidth(100)
        self.SetRowLabelSize(50)

        for row, item in enumerate(item_list):
            # Quantity
            self.SetCellValue(row, 0, str(item["Qty"]).replace(u".", vs_cfg["CP"]))
            # Description
            self.SetCellValue(row, 1, item["Description"])
            # Unit Cost
            self.SetCellValue(row, 2, unicode(Currency(item["Unit Cost"])))
            # Amount = Qty * Unit Cost
            self.SetCellValue(row, 3, unicode(Currency(amount_list[row])))
            # Discount
            if item["Discount"]:
                self.SetCellValue(row, 4, discount)
            else:
                self.SetCellValue(row, 4, no_discount)
            # VAT rate
            self.SetCellValue(row, 5, str(item["VAT"]).replace(u".", vs_cfg["CP"])+u"%")

        return


class ShowTransactionsPanel(wx.Panel):
    """The panel of the ShowTransactionsTable grid"""

    def __init__(self, parent, creditnote):
        """Create the panel"""
        wx.Panel.__init__(self, parent)

        self.parent = parent

        # Create a vertical sizer to hold the grid
        sizer=wx.BoxSizer(wx.VERTICAL)

        note_text = wx.StaticText(self, -1, "Double click on the transaction for details")
        sizer.Add(note_text, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # show the transactions associated with this creditnote
        self.show_transactions_table = ShowTransactionsTable(self, creditnote)

        sizer.Add(self.show_transactions_table, 1, wx.ALIGN_CENTER | wx.EXPAND)
        # Set the size to the panel
        self.SetSizer(sizer)

    def refresh(self):
        self.show_transactions_table.refresh()


class ShowTransactionsTable(wx.grid.Grid):

    def __init__(self, parent, creditnote):
        wx.grid.Grid.__init__(self, parent, style=wx.WANTS_CHARS)

        self.parent = parent
        self.creditnote = creditnote

        # Get the transaction associated with this creditnote
        self.transaction = self.creditnote.transaction

        #self.num_rows = 1
        self.CreateGrid(1, 2)
        self.EnableEditing(False)

        self.SetDefaultRowSize(vs_cfg["BUTTON_HEIGHT"], False)
        self.SetDefaultCellAlignment(wx.ALIGN_LEFT, wx.ALIGN_CENTER)

        # create colum labels
        self.SetColLabelValue(0, "Date")
        self.SetColLabelValue(1, "Description")

        self.SetColSize(0, 100)
        self.SetColSize(1, 300)

        self.SetColMinimalAcceptableWidth(100)
        self.SetRowLabelSize(100)

        self.SetRowLabelValue(0, str(self.transaction.id))
        # Date
        self.SetCellValue(0, 0, self.transaction.datestamp.strftime("%d %b %Y"))
        # Description
        self.SetCellValue(0, 1, self.transaction.description)

        # Bind left mouse double - click
        self.Bind(wx.grid.EVT_GRID_CELL_LEFT_DCLICK, self.show_transaction)


    @button_wrap
    def show_transaction(self, event):
        "Show the transaction in a dialog"
        show_transaction_dialog(self, self.transaction)


    def refresh(self):
        "Refresh the transaction date and description in the table"
        # Date
        self.SetCellValue(0, 0, self.transaction.datestamp.strftime("%d %b %Y"))
        # Description
        self.SetCellValue(0, 1, self.transaction.description)

