####### VATStuff #######
#
# view_invoices.py  - lists invoices
#
# Version : 0.7.0
# Date : 20100926
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#

""" This is a module in the vatstuff_packages.vs_gui package, and Lists Invoices

It is run as part of the vatstuff gui interface.
"""

import datetime
import decimal

import wx
import wx.grid

from vatstuff_packages.vs_database.grid_objects import *
from vatstuff_packages.vs_database import save_creditnote
from vatstuff_packages.vs_database import change_transaction_date

from vatstuff_packages.vs_gui.gui_functions import *
from vatstuff_packages.vs_gui.reports import make_report
from vatstuff_packages.vs_gui.sales import view_creditnotes

from vatstuff_packages.vs_verify import form_objects
from vatstuff_packages.vs_verify.sales import new_creditnote_check

from vatstuff_packages.vs_utilities import *


if "REPORTLABS_AVAILABLE" in vs_cfg and vs_cfg["REPORTLABS_AVAILABLE"]:
    import vatstuff_packages.vs_pdf.page_setup
    import vatstuff_packages.vs_pdf.invoice_form

D = decimal.Decimal   # D used to avoid typing


def view_all_invoices_frame(invoice=None):
    """Show the ViewAllInvoicesFrame which lists all invoices,
       ensuring that the invoice given as argument is displayed.
       If None is given, display the last entry"""

    # No try:exception statement - allows the caller to catch exceptions

    # Check the frame is not already open
    for frm in vs_cfg["TRANSACTION_REFRESH"]:
        if isinstance(frm, ViewAllInvoicesFrame):
            frm.Raise()
            break
    else:
        # top_frame() is the top frame, which will be the parent
        frm = ViewAllInvoicesFrame(top_frame())
        frm.Show()

    if invoice:
        frm.display_row(invoice.gridrow)
    else:
        frm.display_last()


class ViewAllInvoicesFrame(vsFrame):
    """Frame to list all invoices"""

    def __init__(self, parent):
        "Create the frame to show a list of invoices"
        vsFrame.__init__(self, parent, title="Invoices")

        # Take a note of the parent
        self.parent = parent

        self.invoices_grid = InvoicesGrid()
        # class defined in vatstuff_packages.vs_database.grid_objects
        # Create a database object of type wx.grid.PyGridTableBase
        # it opens database connections, and gets information for
        # the grid object

        try:
            self.invoices_grid_rows = self.invoices_grid.GetNumberRows()
            # Create the ViewInvoicesPanel
            self.view_invoices_panel = ViewInvoicesPanel(self, self.invoices_grid)
        except Exception, e:
            # close database connection, and re-raise the exception
            self.invoices_grid.CloseConnection()
            raise

        # Get the panel size
        panel_size = self.view_invoices_panel.GetMinSize()

        # Set minimum size equal to the panel
        self.SetMinSize(panel_size)
        self.SetClientSize(panel_size)

        # update the list of windows which need a refresh if
        # a transaction occurs, by adding self
        vs_cfg["TRANSACTION_REFRESH"].append(self)

        # bind the close event
        self.Bind(wx.EVT_CLOSE, self.onclose)


    def refresh(self):
        "Refreshes the grid"
        new_num_rows = self.invoices_grid.GetNumberRows()
        if new_num_rows > self.invoices_grid_rows:
            # More rows are in the database, insert new rows on the grid
            msg = wx.grid.GridTableMessage(self.invoices_grid,         # The table
                  wx.grid.GRIDTABLE_NOTIFY_ROWS_APPENDED,              # has extra rows
                  new_num_rows - self.invoices_grid_rows               # the number of rows
                  ) 
            self.view_invoices_panel.grid.ProcessTableMessage(msg)     # The grid associated with the table
            self.invoices_grid_rows = new_num_rows                     # record the new number of rows
        elif new_num_rows < self.invoices_grid_rows:
            # Less rows are in the database, delete rows on the grid
            msg = wx.grid.GridTableMessage(self.invoices_grid,         # The table
                  wx.grid.GRIDTABLE_NOTIFY_ROWS_DELETED,               # has fewer rows
                  new_num_rows,                                        # delete from row
                  self.invoices_grid_rows - new_num_rows               # delete the number of rows
                  ) 
            self.view_invoices_panel.grid.ProcessTableMessage(msg)     # The grid associated with the table
            self.invoices_grid_rows = new_num_rows                     # record the new number of rows

        # grid associated with table can also be obtained with
        # self.invoices_grid.GetView().ProcessTableMessage(msg)

        self.view_invoices_panel.grid.ForceRefresh()


    def display_row(self, row):
        "Calls the display_row function in the view_invoices_panel"
        self.view_invoices_panel.display_row(row)

    def display_last(self):
        "Calls the display_last function in the view_invoices_panel"
        self.view_invoices_panel.display_last()

    def onclose(self, evt):
        "Shutdown the database transaction"
        self.invoices_grid.CloseConnection()
        # Decrement the refresh list
        vs_cfg["TRANSACTION_REFRESH"].remove(self)
        self.Destroy()

# ----------------------View invoices panel


class ViewInvoicesPanel(wx.Panel):
    "Show the grid of invoices"

    def __init__(self, parent, invoices_grid):
        "Create the panel"
        wx.Panel.__init__(self, parent)

        # Take a note of the parent, and the invoices_grid
        self.parent = parent
        self.invoices_grid = invoices_grid
        button_size = (100, vs_cfg["BUTTON_HEIGHT"])

        # This is the invoice double-clicked on and shown, initially None
        self.invoice = None

        # Create the sizer for the panel
        sizer=wx.BoxSizer(wx.VERTICAL)

        self.description_text=wx.StaticText(self, -1, "Double click on an invoice to view it")
        sizer.Add(self.description_text, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # Create a grid
        self.grid = wx.grid.Grid(self, size=(900,200))

        self.grid.SetDefaultColSize(100, True)
        self.grid.SetDefaultRowSize(vs_cfg["BUTTON_HEIGHT"], False)
        self.grid.SetColMinimalAcceptableWidth(100)
        self.grid.EnableEditing(False)

        self.grid.SetDefaultCellAlignment(wx.ALIGN_LEFT, wx.ALIGN_CENTER)

        # Assign the invoices_grid to the grid
        self.grid.SetTable(self.invoices_grid, True)

        # Set the description column size
        self.grid.SetColSize(2, 350)

        # Set the row label size
        self.grid.SetRowLabelSize(100)

        sizer.Add(self.grid, 1, wx.ALL | wx.ALIGN_CENTER | wx.EXPAND, 20)

        # buttons
        button_sizer=wx.BoxSizer(wx.HORIZONTAL)

        # ------------------- Close button
        self.close_button = wx.Button(self, wx.ID_CLOSE, size=button_size)
        self.close_button.SetToolTipString("Click to close this window")
        button_sizer.Add(self.close_button, 0, wx.RIGHT, 20)


        # ------------------- Report button
        self.report_button = wx.Button(self, label="Report", size=button_size)
        self.report_button.SetToolTipString("Click to generate a report between two dates")
        button_sizer.Add(self.report_button, 0)

        sizer.Add(button_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)


        # Set the size to the panel
        self.SetSizer(sizer)
        sizer.SetSizeHints(self)

        # Bind left mouse double - click
        self.Bind(wx.grid.EVT_GRID_CELL_LEFT_DCLICK, self.show_invoice, self.grid)

        # Bind the close and report buttons
        self.Bind(wx.EVT_BUTTON, self.close_button_pressed, self.close_button)
        self.Bind(wx.EVT_BUTTON, self.report_button_pressed, self.report_button)


    def close_button_pressed(self, event):
        "Close parent"
        self.parent.Close()


    @button_wrap
    def report_button_pressed(self, event):
        """Get the two dates between which the report is generated
           create the invoices sent report
           and then display it in a report frame"""

        help_text = """\
A report will be generated listing invoices
issued to customers between the two dates:"""

        status, from_date, to_date = two_dates(self, "Invoices Report", help_text)
        if not status:
            return
        busy_cursor_on()
        wx.Yield()

        filename = "invoices_to_customers%s" % to_date.isoformat()
        make_report.make_report_and_frame("invoices",
                                          "Invoices sent to customers",
                                          filename,
                                          from_date,
                                          to_date)

    def display_row(self, row):
        """Makes row visible"""
        last_row = self.invoices_grid.GetNumberRows() - 1
        if row > last_row:
            row = last_row
        if row < 0:
            row = 0
        self.grid.SetGridCursor(row, 0)
        self.grid.MakeCellVisible(row, 0)


    def display_last(self):
        "Makes the last row visible"
        last_row = self.invoices_grid.GetNumberRows() - 1
        self.grid.SetGridCursor(last_row, 0)
        self.grid.MakeCellVisible(last_row, 0)


    @button_wrap
    def show_invoice(self, event):
        "Display the invoice in response to a double click"
        # get invoice from its gridrow number
        invoice = Invoice.from_gridrow(event.GetRow())
        show_invoice_frame(invoice)


# ------------------------ Show the invoice frame

def show_invoice_frame(invoice):
    "Display the ShowInvoiceFrame"

    debug_log("showing invoice id: %s" % invoice.id)

    # if a ShowInvoiceFrame with this invoice is already open -show it
    for frm in vs_cfg["TRANSACTION_REFRESH"]:
        if isinstance(frm, ShowInvoiceFrame):
            if frm.invoice == invoice:
                frm.Raise()
                return

    # the frame is not open, so create one showing this invoice
    show_invoice_frame= ShowInvoiceFrame(top_frame(), invoice)
    show_invoice_frame.Show()


class ShowInvoiceFrame(vsFrame):
    "Called when an invoice is to be viewed"

    def __init__(self, parent, invoice):
        "Create the frame - holding a ShowInvoicePanel"
        frame_title = "Invoice : %i" % invoice.number
        vsFrame.__init__(self, parent, title=frame_title)

        # used to test which invoice frame is open
        self.invoice = invoice

        # create the panel
        self.show_invoice_panel = ShowInvoicePanel(self, self.invoice)

        # Get the panel size
        panel_size = self.show_invoice_panel.GetMinSize()

        # Set minimum size equal to the panel
        self.SetMinSize(panel_size)

        # set the frame size to the panel size
        self.SetClientSize(panel_size)

        # update the list of windows which need a refresh if
        # a transaction occurs, by adding self
        vs_cfg["TRANSACTION_REFRESH"].append(self)

        # bind the close event
        self.Bind(wx.EVT_CLOSE, self.onclose)


    def refresh(self):
        "refresh this frame, by calling the refresh function in its panel"
        self.show_invoice_panel.refresh()

    def onclose(self, evt):
        # Decrement the refresh list
        vs_cfg["TRANSACTION_REFRESH"].remove(self)
        self.Destroy()


# ----------------------ShowInvoicePanel

class ShowInvoicePanel(wx.Panel):
    "The panel of the ShowInvoice frame"

    def __init__(self, parent, invoice):
        """Create the panel"""
        wx.Panel.__init__(self, parent)
        
        # Take a note of the variables
        self.parent = parent
        self.invoice = invoice

        button_size = (150, vs_cfg["BUTTON_HEIGHT"])

        # Create a vertical sizer to hold everything
        self.sizer=wx.BoxSizer(wx.VERTICAL)

        # And a horizontal sizer to hold invoice details
        invoice_sizer = wx.BoxSizer(wx.HORIZONTAL)

        if self.invoice.customer_address:
            customer_text = "Customer: %s\n%s" % (self.invoice.customer.name,
                                                  self.invoice.customer_address.lines_string)
        else:
            customer_text = "Customer: %s" % self.invoice.customer.name

        invoice_sizer.Add(wx.StaticText(self, -1, customer_text), 0, wx.RIGHT | wx.ALIGN_TOP, 80)

        date_sizer = wx.BoxSizer(wx.VERTICAL)

        date_sizer.Add(wx.StaticText(self, -1, "Invoice Number: %i" % self.invoice.number), 0, wx.ALIGN_RIGHT)

        issue_date_text = self.invoice.datestamp.strftime("%d %b %Y")
        supply_date_text = self.invoice.date_of_supply.strftime("%d %b %Y")
        taxpoint_date_text = self.invoice.date_of_taxpoint.strftime("%d %b %Y")

        self.issue_date_field = wx.StaticText(self, -1, "Invoice Date: %s" % issue_date_text)
        self.supply_date_field = wx.StaticText(self, -1, "Supply Date: %s" % supply_date_text)
        self.taxpoint_date_field = wx.StaticText(self, -1, "Taxpoint Date: %s" % taxpoint_date_text)

        date_sizer.Add(self.issue_date_field, 0, wx.ALIGN_RIGHT)
        date_sizer.Add(self.supply_date_field, 0, wx.ALIGN_RIGHT)
        date_sizer.Add(self.taxpoint_date_field, 0, wx.ALIGN_RIGHT)

        # taxpoint button
        self.set_taxpoint_button = wx.Button(self, label="Set taxpoint", size=button_size)
        self.set_taxpoint_button.SetToolTipString("Click to set the taxpoint of the invoice.")
        date_sizer.Add(self.set_taxpoint_button, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        invoice_sizer.Add(date_sizer, 0, wx.RIGHT | wx.ALIGN_TOP, 80)

        if self.invoice.vat_return:
            self.set_taxpoint_button.Enable(False)

        vat_scheme = VATScheme()
        vat_period_start_date = vat_scheme.start_date
        if vat_period_start_date:
            if self.invoice.datestamp < vat_period_start_date:
                self.set_taxpoint_button.Enable(False)

        # Put costs in a grid sizer
        cost_sizer=wx.FlexGridSizer(rows=4, cols=2, hgap=20, vgap=10)

        cost_string = to_currency_string(self.invoice.cost)
        vat_string = to_currency_string(self.invoice.vat)
        total_string = to_currency_string(self.invoice.total)
        outstanding_string = to_currency_string(self.invoice.outstanding)

        # This static text is an instance variable since it is updated by the refresh method
        self.outstanding = wx.StaticText(self, -1, outstanding_string)

        cost_sizer.Add(wx.StaticText(self, -1, "Cost"), 0, wx.ALIGN_RIGHT)
        cost_sizer.Add(wx.StaticText(self, -1, cost_string), 0, wx.ALIGN_RIGHT)
        cost_sizer.Add(wx.StaticText(self, -1, "VAT"), 0, wx.ALIGN_RIGHT)
        cost_sizer.Add(wx.StaticText(self, -1, vat_string), 0, wx.ALIGN_RIGHT)
        cost_sizer.Add(wx.StaticText(self, -1, "Invoice Total"), 0, wx.ALIGN_RIGHT)
        cost_sizer.Add(wx.StaticText(self, -1, total_string), 0, wx.ALIGN_RIGHT)
        cost_sizer.Add(wx.StaticText(self, -1, "Outstanding"), 0, wx.ALIGN_RIGHT)
        cost_sizer.Add(self.outstanding, 0, wx.ALIGN_RIGHT)

        invoice_sizer.Add(cost_sizer, 0, wx.ALIGN_TOP)

        self.sizer.Add(invoice_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # Another horizontal sizer to hold customer reference
        cus_order_sizer = wx.BoxSizer(wx.HORIZONTAL)

        cus_order_sizer.Add(wx.StaticText(self, -1, "Customer's Order Number: "), 0, wx.RIGHT | wx.ALIGN_CENTER_VERTICAL, 20)

        self.cus_order_field = wx.TextCtrl(self, -1, self.invoice.cus_order_number, size=(250, vs_cfg["BUTTON_HEIGHT"]))
        self.cus_order_field.SetToolTipString("This appears on the invoice pdf")
        self.cus_order_field.SetMaxLength(40)
        cus_order_sizer.Add(self.cus_order_field, 0, wx.RIGHT, 20)

        self.cus_order_button = wx.Button(self, label="Change", size=button_size)
        self.cus_order_button.SetToolTipString("Click to change the customers order number")
        cus_order_sizer.Add(self.cus_order_button, 0)

        self.sizer.Add(cus_order_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # Create status text

        status_text = "This invoice is Open"
        if not self.invoice.is_open:
            status_text = "This invoice is Closed"
        if self.invoice.is_cancelled:
            # If the invoice is cancelled, there should only be one credit note
            notes = self.invoice.get_creditnotes()
            assert len(notes) == 1
            note = notes[0]
            cancelled_date = note.datestamp.strftime("%d %b %Y")
            status_text = "This invoice was cancelled : %s" % cancelled_date

        self.status = wx.StaticText(self, -1, status_text)

        self.sizer.Add(self.status, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # Create a Tabbed Notebook
        self.tabbed_notebook = InvoiceNotebook(self, self.invoice)
        self.sizer.Add(self.tabbed_notebook, 1, wx.ALL | wx.ALIGN_CENTER | wx.EXPAND, 10)

        # Create results sizer for the printer button, and the VAT, Cost, Total results
        res_sizer=wx.BoxSizer(wx.HORIZONTAL)

        # close button
        self.close_button = wx.Button(self, wx.ID_CLOSE, size=button_size)
        self.close_button.SetToolTipString("Click to close this window.")
        res_sizer.Add(self.close_button, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # page setup button
        self.page_button = wx.Button(self, label="Page setup", size=button_size)
        self.page_button.SetToolTipString("Click to set up the pdf page size and orientation")
        res_sizer.Add(self.page_button, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # pdf button
        self.pdf_button = wx.Button(self, label="Invoice pdf", size=button_size)
        self.pdf_button.SetToolTipString("Click to create a pdf file of the invoice.")
        res_sizer.Add(self.pdf_button, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # delete_invoice button
        self.delete_invoice_button = wx.Button(self, label="Cancel Invoice", size=button_size)
        self.delete_invoice_button.SetToolTipString("Click to cancel this Sale")
        res_sizer.Add(self.delete_invoice_button, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        if self.invoice.is_open and self.invoice.paid == D(0) and not self.invoice.is_cancelled:
            self.delete_invoice_button.Enable(True)
        else:
            self.delete_invoice_button.Enable(False)

        # add the res_sizer to the main vertical sizer
        self.sizer.Add(res_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # Set the size to the panel
        self.SetSizer(self.sizer)

        panel_size = self.sizer.GetMinSize()
        self.SetMinSize(panel_size)

        self.Bind(wx.EVT_BUTTON, self.set_taxpoint_button_pressed, self.set_taxpoint_button)
        self.Bind(wx.EVT_BUTTON, self.cus_order_button_pressed, self.cus_order_button)
        self.Bind(wx.EVT_BUTTON, self.close_button_pressed, self.close_button)
        self.Bind(wx.EVT_BUTTON, self.page_button_pressed, self.page_button)
        self.Bind(wx.EVT_BUTTON, self.pdf_button_pressed, self.pdf_button)
        self.Bind(wx.EVT_BUTTON, self.delete_invoice_button_pressed, self.delete_invoice_button)


    @writecheck
    @button_wrap
    def set_taxpoint_button_pressed(self, event):
        "Set the invoice dates by calling a Dialog"
        set_dates_dialog(self, self.invoice)


    @writecheck
    @button_wrap
    def cus_order_button_pressed(self, event):
        """change the customer order number"""
        cus_order = self.cus_order_field.GetValue().strip()
        if self.invoice.cus_order_number == cus_order:
            # No change in the order number - just return
            vs_message(self, "There was no change to the customer order number", "WARNING")
            return
        self.invoice.cus_order_number = cus_order
        # big_refresh(acc_tree=False) - dont think a refresh is necessary here
        info_log("Invoice id : %i customer order number changed" % self.invoice.id)
        vs_message(self, "Customer order number changed", "DONE")


    def close_button_pressed(self, event):
        "close button pressed, close window"
        self.parent.Close()


    @button_wrap
    def page_button_pressed(self, event):
        "page button pressed, setup the pdf page size and orientation"

        if not vs_cfg["REPORTLABS_AVAILABLE"]:
            # no pdf capability
            vs_message(self, "Function unavailable - the reportlab module is not installed",
                             "ERROR")
            return

        page_setup_dialog = vatstuff_packages.vs_pdf.page_setup.PageSetupDialog(self,
                                                              self.invoice.pdf_width,
                                                              self.invoice.pdf_height)
        if page_setup_dialog.ShowModal() == wx.ID_OK:
            self.invoice.pdf_width = page_setup_dialog.width
            self.invoice.pdf_height = page_setup_dialog.height           
        page_setup_dialog.Destroy()


    @button_wrap
    def pdf_button_pressed(self, event):
        "pdf button create a pdf form"

        if not vs_cfg["REPORTLABS_AVAILABLE"]:
            # no pdf capability
            vs_message(self, "Function unavailable - the reportlab module is not installed",
                             "ERROR")
            return

        filename= "invoice_%s.pdf" % self.invoice.number

        # Create the save file dialog.
        status, filepath = save_file_as(self, 'pdf', vs_cfg["INVOICES_PDF_FOLDER"], filename)

        if status:
            busy_cursor_on()
            wx.Yield()
            if vatstuff_packages.vs_pdf.invoice_form.create_pdf(filepath, self.invoice):
                busy_cursor_off()
                # And show the file in the default viewer
                if not vatstuff_packages.vs_pdf.pdf_functions.show_pdf_file(filepath):
                    vs_message(self, "pdf file created", "Done")
            else:
                vs_message(self, "Unable to create the pdf file", "ERROR")


    @writecheck
    @button_wrap
    def delete_invoice_button_pressed(self, event):
        "Cancel the invoice, by creating a creditnote"
        if self.invoice.is_cancelled: return
        if not self.invoice.is_open: return
        if self.invoice.paid != D(0): return

        # Get the reason for the cancellation
        busy_cursor_off()
        confirm_cancel=wx.TextEntryDialog(parent=self,
                                          message="""\
Are you sure? This action cancels the Invoice, and creates a Credit Note
which can be viewed and printed via the View Credit Notes button.
Please give the reason for the cancellation, (which will appear on
the credit note), then press OK to Cancel the invoice:""",
                                       caption="Cancel Invoice",
                                       defaultValue="",
                                       style=wx.OK | wx.CANCEL | wx.TE_LEFT)
        result=confirm_cancel.ShowModal()
        reason=confirm_cancel.GetValue()
        confirm_cancel.Destroy()
        if result != wx.ID_OK: return

        busy_cursor_on()
        wx.Yield()

        # Store data in a CreditNoteForm object, this is used to create a credit note

        # Note - this is a form object (not database linked),
        creditnote = form_objects.CreditNoteForm()

        # Store the transaction in a TransactionForm object
        transaction = form_objects.TransactionForm()
        # This is an credit note transaction(system type 7)
        transaction.system = 7

        # invoice.customer is the customer who is being invoiced, but the customer may have
        # had details changed since the invoice was sent, so get the visible customer
        # with invoice.customer.number
        creditnote.customer = Customer.from_number(self.invoice.customer.number)

        creditnote.invoice = self.invoice                   # The invoice the credit note refers to
        creditnote.item_list = self.invoice.item_list       # The items on the invoice
        creditnote.cost = self.invoice.cost                 # net cost on the invoice
        creditnote.vat  = self.invoice.vat                  # VAT on the invoice
        creditnote.reason = reason                          # reason saved on the credit note
        creditnote.show_vat = creditnote.customer.charge_vat()
        creditnote.account_credited = creditnote.customer.account
        creditnote.discount = self.invoice.discount

        company = Company()
        creditnote.company = company

        last_creditnote_number = get_config_item("LAST_CREDITNOTE_NUMBER")
        if not last_creditnote_number:
            creditnote.number = 1
        else:
            creditnote.number = int(last_creditnote_number) + 1

        transaction.description = "CANCELLED INVOICE %s : CREDIT NOTE %s" % (self.invoice.number, creditnote.number)
        transaction.datestamp = datetime.date.today()

        # credit note cannot be issued before the invoice
        if transaction.datestamp < self.invoice.date_of_issue:
            transaction.datestamp = self.invoice.date_of_issue

        # credit note cannot be issued before the delete to date
        if vs_cfg["DELETE_TO_DATE"]:
            if transaction.datestamp < vs_cfg["DELETE_TO_DATE"]:
                transaction.datestamp = vs_cfg["DELETE_TO_DATE"]
        # credit note cannot be issued before the current vat period start date
        vat_scheme = VATScheme()
        vat_period_start_date = vat_scheme.start_date
        if vat_period_start_date:
            if transaction.datestamp < vat_period_start_date:
                transaction.datestamp = vat_period_start_date

        creditnote.datestamp = transaction.datestamp

        # Check these input creditnote details are ok via vatstuff_packages.vs_verify.sales
        status, message = new_creditnote_check(creditnote)
        if not status:
            vs_message(self, message, "ERROR")
            return

        # Credit the customers account by the cost plus VAT
        transaction.add(creditnote.account_credited, credit = creditnote.cost + creditnote.vat)

        # Debit the sales account by the cost
        if creditnote.cost != D(0):
            transaction.add(get_account_by_system(6), debit = creditnote.cost)

        # Debit the VAT balance account by the VAT amount
        if creditnote.vat != D(0):
            transaction.add(get_account_by_system(5), debit = creditnote.vat)

        status, message = transaction.check()
        if not status:
            vs_message(self, message, "ERROR")
            return

        wx.Yield()
        # Confirm the transaction (function and dialog in gui_functions)
        status, new_description = confirm_transaction(self, transaction)

        if not status:
            vs_message(self, "The Invoice has not been cancelled", "Transaction not implemented")
            return

        transaction.description = new_description

        # Must now implement the transaction
        busy_cursor_on()
        wx.Yield()
        save_creditnote.save(creditnote, transaction)

        self.invoice.is_open = False
        self.invoice.is_cancelled = True

        big_refresh()
        info_log("Invoice number: %s cancelled" % self.invoice.number)
        vs_message(self, "Invoice %s cancelled\nCredit Note %s created" % (self.invoice.number, creditnote.number), "DONE")


    def refresh(self):
        "refreshes the invoice status and outstanding amount"
        # Refresh dates
        self.issue_date_field.SetLabel("Invoice Date: %s" % self.invoice.datestamp.strftime("%d %b %Y"))
        self.supply_date_field.SetLabel("Supply Date: %s" % self.invoice.date_of_supply.strftime("%d %b %Y"))
        self.taxpoint_date_field.SetLabel("Taxpoint Date: %s" % self.invoice.date_of_taxpoint.strftime("%d %b %Y"))
        # Refresh status
        status_text = "This invoice is Open"
        if not self.invoice.is_open:
            status_text = "This invoice is Closed"
        if self.invoice.is_cancelled:
            # If the invoice is cancelled, there should only be one credit note
            notes = self.invoice.get_creditnotes()
            assert len(notes) == 1
            note = notes[0]
            cancelled_date = note.datestamp.strftime("%d %b %Y")
            status_text = "This invoice was cancelled : %s" % cancelled_date
        self.status.SetLabel(status_text)
        # customer order number
        self.cus_order_field.SetValue(self.invoice.cus_order_number)

        outstanding_string = unicode(Currency(self.invoice.outstanding))
        self.outstanding.SetLabel(outstanding_string)

        # Refresh the tabbed notebook
        self.tabbed_notebook.refresh()

        if self.invoice.is_open and self.invoice.paid == D(0) and not self.invoice.is_cancelled:
            self.delete_invoice_button.Enable(True)
        else:
            self.delete_invoice_button.Enable(False)

        # Re-do the layout
        self.sizer.Layout()


class InvoiceNotebook(wx.Notebook):
    """Shows invoice items, and payments"""

    def __init__(self, parent, invoice):
        """Create the tabbed notebook, and each panel beneath each tab"""
        wx.Notebook.__init__(self, parent, size = (900,200))

        # Note the notebook is set to a specific size, since the grid size is
        # variable, this forces the grid to have scroll bars if bigger

        self.parent = parent
        self.invoice = invoice
       
        # show the items being sold
        self.show_solditems = ShowSoldItemsPanel(self, self.invoice)
        # Add show_solditems to the Notebook
        self.AddPage(self.show_solditems, " Sold Items ")

        # show the creditnotes created
        self.show_creditnotes = ShowCreditNotesPanel(self, self.invoice)
        # Add show_creditnotes to the Notebook
        self.AddPage(self.show_creditnotes, " Credit Notes ")

        # show the payments paid
        self.show_payments = ShowPaymentsPanel(self, self.invoice)
        # Add show_payments to the Notebook
        self.AddPage(self.show_payments, " Payments received ")

        # show the invoice transactions
        self.show_transactions = ShowTransactionsPanel(self, self.invoice)
        # Add show_transactions to the Notebook
        self.AddPage(self.show_transactions, " Transactions ")

    def refresh(self):
        self.show_payments.refresh()
        self.show_creditnotes.refresh()
        self.show_transactions.refresh()


class ShowSoldItemsPanel(wx.Panel):
    """The panel of the ShowSoldItemsTable grid"""

    def __init__(self, parent, invoice):
        """Create the panel"""
        wx.Panel.__init__(self, parent)

        # Create a vertical sizer to hold the grid
        sizer=wx.BoxSizer(wx.VERTICAL)

        # show the itms being purchased
        show_solditems_table = ShowSoldItemsTable(self, invoice)

        sizer.Add(show_solditems_table, 1, wx.ALIGN_CENTER | wx.EXPAND)
        # Set the size to the panel
        self.SetSizer(sizer)


class ShowSoldItemsTable(wx.grid.Grid):

    def __init__(self, parent, invoice):
        wx.grid.Grid.__init__(self, parent, style=wx.WANTS_CHARS)

        self.parent = parent
        self.invoice = invoice

        discount = "%s%%" % str(self.invoice.discount).replace(u".", vs_cfg["CP"])
        no_discount = "0%s0%%" % vs_cfg["CP"]

        num_cols = 6

        amount_list = self.invoice.get_amount()
        item_list = self.invoice.item_list
        num_rows = len(item_list)

        self.CreateGrid(num_rows, num_cols)
        self.EnableEditing(False)

        self.SetDefaultRowSize(vs_cfg["BUTTON_HEIGHT"], False)
        self.SetDefaultCellAlignment(wx.ALIGN_LEFT, wx.ALIGN_CENTER)

        # create colum labels
        self.SetColLabelValue(0, "Qty")
        self.SetColLabelValue(1, "Description")
        self.SetColLabelValue(2, "Unit Cost")
        self.SetColLabelValue(3, "Amount")
        self.SetColLabelValue(4, "Discount")
        self.SetColLabelValue(5, "VAT")

        # Set the Quantity, Discount and VAT columns as centre aligned
        attr_center = wx.grid.GridCellAttr()
        attr_center.SetAlignment(wx.ALIGN_CENTER, wx.ALIGN_CENTER)

        self.SetColAttr(0, attr_center)
        self.SetColAttr(4, attr_center)
        self.SetColAttr(5, attr_center)

        # Set the colums showing money as right aligned
        attr_right = wx.grid.GridCellAttr()
        attr_right.SetAlignment(wx.ALIGN_RIGHT, wx.ALIGN_CENTER)
        self.SetColAttr(2, attr_right)
        self.SetColAttr(3, attr_right)

        self.SetColSize(0, 100)
        self.SetColSize(1, 300)
        self.SetColSize(2, 100)
        self.SetColSize(3, 100)
        self.SetColSize(4, 100)
        self.SetColSize(5, 100)


        self.SetColMinimalAcceptableWidth(100)
        self.SetRowLabelSize(50)

        for row, item in enumerate(item_list):
            # Quantity
            self.SetCellValue(row, 0, str(item["Qty"]).replace(u".", vs_cfg["CP"]))
            # Description
            self.SetCellValue(row, 1, item["Description"])
            # Unit Cost
            self.SetCellValue(row, 2, unicode(Currency(item["Unit Cost"])))
            # Amount = Qty * Unit Cost
            self.SetCellValue(row, 3, unicode(Currency(amount_list[row])))
            # Discount
            if item["Discount"]:
                self.SetCellValue(row, 4, discount)
            else:
                self.SetCellValue(row, 4, no_discount)
            # VAT rate
            self.SetCellValue(row, 5, str(item["VAT"]).replace(u".", vs_cfg["CP"])+u"%")


class ShowCreditNotesPanel(wx.Panel):
    """The panel of the ShowCreditNotesTable grid"""

    def __init__(self, parent, invoice):
        """Create the panel"""
        wx.Panel.__init__(self, parent)

        self.parent = parent

        # Create a vertical sizer to hold the grid
        sizer=wx.BoxSizer(wx.VERTICAL)

        note_text = wx.StaticText(self, -1, "Double click on the credit note for details")
        sizer.Add(note_text, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # show the credit notes created
        self.show_creditnotes_table = ShowCreditNotesTable(self, invoice)

        sizer.Add(self.show_creditnotes_table, 1, wx.ALIGN_CENTER | wx.EXPAND)
        # Set the size to the panel
        self.SetSizer(sizer)

    def refresh(self):
        self.show_creditnotes_table.refresh()


class ShowCreditNotesTable(wx.grid.Grid):

    def __init__(self, parent, invoice):
        wx.grid.Grid.__init__(self, parent, style=wx.WANTS_CHARS)

        self.parent = parent
        self.invoice = invoice

        num_cols = 4

        self.notes = self.invoice.get_creditnotes()

        self.num_rows = len(self.notes)

        self.CreateGrid(self.num_rows, num_cols)
        self.EnableEditing(False)

        self.SetDefaultRowSize(vs_cfg["BUTTON_HEIGHT"], False)
        self.SetDefaultCellAlignment(wx.ALIGN_LEFT, wx.ALIGN_CENTER)

        # create column labels
        self.SetColLabelValue(0, "Date")
        self.SetColLabelValue(1, "Credit Note Number")
        self.SetColLabelValue(2, "Cost")
        self.SetColLabelValue(3, "VAT")

        # Set the colums showing money as right aligned
        attr_right = wx.grid.GridCellAttr()
        attr_right.SetAlignment(wx.ALIGN_RIGHT, wx.ALIGN_CENTER)
        self.SetColAttr(2, attr_right)
        self.SetColAttr(3, attr_right)

        self.SetColSize(0, 100)
        self.SetColSize(1, 250)
        self.SetColSize(2, 100)
        self.SetColSize(3, 100)

        self.SetColMinimalAcceptableWidth(100)
        self.SetRowLabelSize(100)


        for row, item in enumerate(self.notes):
            # Set row label as the transaction id
            self.SetRowLabelValue(row, str(item.transaction.id))
            self.SetCellValue(row, 0, item.datestamp.strftime("%d %b %Y"))
            self.SetCellValue(row, 1, "Credit Note:%i" % item.number)
            self.SetCellValue(row, 2, to_currency_string(item.cost))
            self.SetCellValue(row, 3, to_currency_string(item.vat))

        # Bind left mouse double - click
        self.Bind(wx.grid.EVT_GRID_CELL_LEFT_DCLICK, self.show_credit_note)


    @button_wrap
    def show_credit_note(self, event):
        "On being double clicked, obtain creditnote and show it in a  ShowCreditnoteFrame"
        row = event.GetRow()
        view_creditnotes.show_creditnote_frame(self.notes[row])


    def refresh(self):
        "Refresh table or add further rows to the table if further creditnotes have been implemented"
        self.notes = self.invoice.get_creditnotes()

        num_rows = len(self.notes)

        if num_rows > self.num_rows:
            self.AppendRows(numRows = num_rows-self.num_rows)
            self.num_rows = num_rows

        # Refresh table
        for row, item in enumerate(self.notes):
            # Set row label as the transaction id
            self.SetRowLabelValue(row, str(item.transaction.id))
            self.SetCellValue(row, 0, item.datestamp.strftime("%d %b %Y"))
            self.SetCellValue(row, 1, "Credit Note:%i" % item.number)
            self.SetCellValue(row, 2, to_currency_string(item.cost))
            self.SetCellValue(row, 3, to_currency_string(item.vat))


class ShowPaymentsPanel(wx.Panel):
    """The panel of the ShowPaymentsTable grid"""

    def __init__(self, parent, invoice):
        """Create the panel"""
        wx.Panel.__init__(self, parent)

        self.parent = parent

        # Create a vertical sizer to hold the grid
        sizer=wx.BoxSizer(wx.VERTICAL)

        note_text = wx.StaticText(self, -1, """\
If a payment received is for more than one invoice, the Net and VAT values shown here
are the amounts applicable to this invoice only - not the full amounts of the payment.
Double click on a payment for full payment details and to create a payment receipt.""")
        sizer.Add(note_text, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # show the payments made
        self.show_payments_table = ShowPaymentsTable(self, invoice)

        sizer.Add(self.show_payments_table, 1, wx.ALIGN_CENTER | wx.EXPAND)
        # Set the size to the panel
        self.SetSizer(sizer)

    def refresh(self):
        self.show_payments_table.refresh()


class ShowPaymentsTable(wx.grid.Grid):

    def __init__(self, parent, invoice):
        wx.grid.Grid.__init__(self, parent, style=wx.WANTS_CHARS)

        self.parent = parent
        self.invoice = invoice

        num_cols = 4

        self.payments = self.invoice.get_payments_with_costvat()

        self.num_rows = len(self.payments)

        self.CreateGrid(self.num_rows, num_cols)
        self.EnableEditing(False)

        self.SetDefaultRowSize(vs_cfg["BUTTON_HEIGHT"], False)
        self.SetDefaultCellAlignment(wx.ALIGN_LEFT, wx.ALIGN_CENTER)

        # create column labels
        self.SetColLabelValue(0, "Date")
        self.SetColLabelValue(1, "Customer reference")
        self.SetColLabelValue(2, "Net Paid")
        self.SetColLabelValue(3, "VAT Paid")

        # Set the colums showing money as right aligned
        attr_right = wx.grid.GridCellAttr()
        attr_right.SetAlignment(wx.ALIGN_RIGHT, wx.ALIGN_CENTER)
        self.SetColAttr(2, attr_right)
        self.SetColAttr(3, attr_right)

        self.SetColSize(0, 100)
        self.SetColSize(1, 300)
        self.SetColSize(2, 100)
        self.SetColSize(3, 100)

        self.SetColMinimalAcceptableWidth(100)
        self.SetRowLabelSize(100)

        for row, item in enumerate(self.payments):
            # Set row label as the transaction id
            self.SetRowLabelValue(row, str(item["PAYMENT"].transaction.id))
            # Date
            self.SetCellValue(row, 0, item["PAYMENT"].datestamp.strftime("%d %b %Y"))
            self.SetCellValue(row, 1, item["PAYMENT"].customer_reference)
            self.SetCellValue(row, 2, to_currency_string(item["COST"]))
            self.SetCellValue(row, 3, to_currency_string(item["VAT"]))

        # Bind left mouse double - click
        self.Bind(wx.grid.EVT_GRID_CELL_LEFT_DCLICK, self.show_payment)


    @button_wrap
    def show_payment(self, event):
        "On being double clicked, obtain payment and show it in a ShowCustomerPaymentDialog"
        row = event.GetRow()
        show_customer_payment_dialog(self, self.payments[row]["PAYMENT"])


    def refresh(self):
        "Refresh table or add further rows to the table if further payments have been implemented"
        self.payments = self.invoice.get_payments_with_costvat()

        num_rows = len(self.payments)

        if num_rows > self.num_rows:
            self.AppendRows(numRows = num_rows-self.num_rows)
            self.num_rows = num_rows

        # Refresh table
        for row, item in enumerate(self.payments):
            # Set row label as the transaction id
            self.SetRowLabelValue(row, str(item["PAYMENT"].transaction.id))
            self.SetCellValue(row, 0, item["PAYMENT"].datestamp.strftime("%d %b %Y"))
            self.SetCellValue(row, 1, item["PAYMENT"].customer_reference)
            self.SetCellValue(row, 2, to_currency_string(item["COST"]))
            self.SetCellValue(row, 3, to_currency_string(item["VAT"]))


class ShowTransactionsPanel(wx.Panel):
    """The panel of the ShowTransactionsTable grid"""

    def __init__(self, parent, invoice):
        """Create the panel"""
        wx.Panel.__init__(self, parent)

        self.parent = parent

        # Create a vertical sizer to hold the grid
        sizer=wx.BoxSizer(wx.VERTICAL)

        note_text = wx.StaticText(self, -1, "Double click on the transaction for details")
        sizer.Add(note_text, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # show the transactions associated with this invoice
        self.show_transactions_table = ShowTransactionsTable(self, invoice)

        sizer.Add(self.show_transactions_table, 1, wx.ALIGN_CENTER | wx.EXPAND)
        # Set the size to the panel
        self.SetSizer(sizer)


    def refresh(self):
        self.show_transactions_table.refresh()


class ShowTransactionsTable(wx.grid.Grid):

    def __init__(self, parent, invoice):
        wx.grid.Grid.__init__(self, parent, style=wx.WANTS_CHARS)

        self.parent = parent
        self.invoice = invoice

        num_cols = 2

        # Get all the transactions associated with this invoice
        self.transactions = self.invoice.get_all_transactions()

        self.num_rows = len(self.transactions)
        self.CreateGrid(self.num_rows, num_cols)
        self.EnableEditing(False)

        self.SetDefaultRowSize(vs_cfg["BUTTON_HEIGHT"], False)
        self.SetDefaultCellAlignment(wx.ALIGN_LEFT, wx.ALIGN_CENTER)

        # create colum labels
        self.SetColLabelValue(0, "Date")
        self.SetColLabelValue(1, "Description")

        self.SetColSize(0, 100)
        self.SetColSize(1, 400)

        self.SetColMinimalAcceptableWidth(100)
        self.SetRowLabelSize(100)


        for row, transaction in enumerate(self.transactions):
            # Set row label as the transaction id
            self.SetRowLabelValue(row, str(transaction.id))
            # Date
            self.SetCellValue(row, 0, transaction.datestamp.strftime("%d %b %Y"))
            # Description
            self.SetCellValue(row, 1, transaction.description)

        # Bind left mouse double - click
        self.Bind(wx.grid.EVT_GRID_CELL_LEFT_DCLICK, self.show_transaction)


    @button_wrap
    def show_transaction(self, event):
        "Show the transaction in a dialog"
        row = event.GetRow()
        show_transaction_dialog(self, self.transactions[row])


    def refresh(self):
        "Refresh table, or add further rows to the table if further transactions have been implemented"

        # Get all the transactions associated with this invoice
        self.transactions = self.invoice.get_all_transactions()

        num_rows = len(self.transactions)

        if num_rows > self.num_rows:
            self.AppendRows(numRows = num_rows-self.num_rows)
            self.num_rows = num_rows

        for row, transaction in enumerate(self.transactions):
            # Set row label as the transaction id
            self.SetRowLabelValue(row, str(transaction.id))
            # Date
            self.SetCellValue(row, 0, transaction.datestamp.strftime("%d %b %Y"))
            # Description
            self.SetCellValue(row, 1, transaction.description)


# The SetDatesDialog is used to set invoice dates
# It is normally used by calling the function set_dates_dialog

def set_dates_dialog(parent, invoice):
    """Creates a SetDatesDialog"""
    show_dialog = SetDatesDialog(parent, invoice)

    # If the cursor is busy, unbusy it - as
    # now waiting for a button press
    busy_cursor_off()

    show_dialog.ShowModal()
    show_dialog.Destroy()


class SetDatesDialog(vsDialog):
    """Alters the invoice dates on confirmation"""

    def __init__(self, parent, invoice):
        """Create the Dialog"""
        vsDialog.__init__(self, parent, "Set Invoice dates")

        self.parent = parent
        self.invoice = invoice
        issue_date = self.invoice.datestamp
        supply_date = self.invoice.date_of_supply
        taxpoint_date = self.invoice.date_of_taxpoint

        wx_issue_date = date_to_wxdate(issue_date)
        wx_supply_date = date_to_wxdate(supply_date)
        wx_taxpoint_date = date_to_wxdate(taxpoint_date)

        button_size = (100, vs_cfg["BUTTON_HEIGHT"])

        # Create a vertical sizer to hold everything
        sizer=wx.BoxSizer(wx.VERTICAL)

        help_text = """\
The taxpoint is the date at which VAT applies, and is
generally the earlier of the invoice date or supply date.
(Unless supply is within 14 days of the invoice.)"""

        # show the help text
        sizer.Add(wx.StaticText(self, -1, help_text), 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # Create choices in a grid sizer
        field_sizer=wx.FlexGridSizer(rows=3, cols=2, hgap=20, vgap=40)

        # The issue date
        field_sizer.Add(wx.StaticText(self, -1, "Issue date:"), 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)
        self.issue_date_field = wx.DatePickerCtrl(self, -1,
                                                 wx_issue_date, 
                                                 size=(200, vs_cfg["BUTTON_HEIGHT"]),
                                                 style=wx.DP_DROPDOWN | wx.DP_SHOWCENTURY)
        self.issue_date_field.SetToolTipString("The date of issue of the invoice")
        field_sizer.Add(self.issue_date_field, 0, wx.ALIGN_LEFT)

        # The supply date
        field_sizer.Add(wx.StaticText(self, -1, "Supply date:"), 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)
        self.supply_date_field = wx.DatePickerCtrl(self, -1,
                                                 wx_supply_date, 
                                                 size=(200, vs_cfg["BUTTON_HEIGHT"]),
                                                 style=wx.DP_DROPDOWN | wx.DP_SHOWCENTURY)
        self.supply_date_field.SetToolTipString("The date of supply of the goods")
        field_sizer.Add(self.supply_date_field, 0, wx.ALIGN_LEFT)

        # The taxpoint date
        field_sizer.Add(wx.StaticText(self, -1, "Taxpoint date:"), 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)
        self.taxpoint_date_field = wx.DatePickerCtrl(self, -1,
                                                 wx_taxpoint_date, 
                                                 size=(200, vs_cfg["BUTTON_HEIGHT"]),
                                                 style=wx.DP_DROPDOWN | wx.DP_SHOWCENTURY)
        self.taxpoint_date_field.SetToolTipString("The taxpoint date")
        field_sizer.Add(self.taxpoint_date_field, 0, wx.ALIGN_LEFT)


        sizer.Add(field_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 40)

        # ---------------------- Apply - Cancel -- buttons

        # Create a horizontal button_sizer to hold the buttons
        button_sizer=wx.BoxSizer(wx.HORIZONTAL)

        # Apply button
        self.apply_button = wx.Button(self, wx.ID_APPLY, size=button_size)
        self.apply_button.SetToolTipString("Click to apply date changes.")
        button_sizer.Add(self.apply_button, 0, wx.RIGHT, 40)

        # Cancel button
        self.cancel_button = wx.Button(self, wx.ID_CANCEL, size=button_size)
        self.cancel_button.SetToolTipString("Click to cancel.")
        button_sizer.Add(self.cancel_button, 0)

        # Add button_sizer to the main sizer
        sizer.Add(button_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # Set the size to the frame
        self.SetSizer(sizer)
        sizer.SetSizeHints(self)

        # ---------- Bind date changes to handler functions
        self.Bind(wx.EVT_DATE_CHANGED, self.issue_date_field_pressed, self.issue_date_field)
        self.Bind(wx.EVT_DATE_CHANGED, self.supply_date_field_pressed, self.supply_date_field)

        # ---------- Bind buttons to handler functions
        
        self.Bind(wx.EVT_BUTTON, self.apply_button_pressed, self.apply_button)
        self.Bind(wx.EVT_BUTTON, self.cancel_button_pressed, self.cancel_button)


    # ----------------- Handler functions

    def issue_date_field_pressed(self, event):
        """When issue date changed, change the taxpoint date"""
        try:
            issue_date = wxdate_to_date(event.GetDate())
            supply_date = wxdate_to_date(self.supply_date_field.GetValue())
            taxpoint_date = issue_date

            # If the goods were supplied more than 14 days earlier
            if supply_date < issue_date:
                timedelta = issue_date - supply_date
                if timedelta.days > 14:
                    taxpoint_date = supply_date

            wx_taxpoint_date = date_to_wxdate(taxpoint_date)
            self.taxpoint_date_field.SetValue(wx_taxpoint_date)
        except Exception, e:
            error_message("Failure in updating issue date", e)


    def supply_date_field_pressed(self, event):
        """When supply date changed, change the taxpoint date"""
        try:
            issue_date = wxdate_to_date(self.issue_date_field.GetValue())
            supply_date = wxdate_to_date(event.GetDate())
            taxpoint_date = issue_date

            # If the goods were supplied more than 14 days earlier
            if supply_date < issue_date:
                timedelta = issue_date - supply_date
                if timedelta.days > 14:
                    taxpoint_date = supply_date

            wx_taxpoint_date = date_to_wxdate(taxpoint_date)
            self.taxpoint_date_field.SetValue(wx_taxpoint_date)
        except Exception, e:
            error_message("Failure in updating supply date", e)


    @writecheck
    @button_wrap
    def apply_button_pressed(self, event):
        """Apply button - check and apply all fields"""
        # Convert the dates input to python date objects
        # function wxdate_to_date is in module vatstuff_packages.vs_utilities.wx_utilities
        issue_date = wxdate_to_date(self.issue_date_field.GetValue())
        supply_date = wxdate_to_date(self.supply_date_field.GetValue())
        taxpoint_date = wxdate_to_date(self.taxpoint_date_field.GetValue())

        if vs_cfg["DELETE_TO_DATE"]:
            if (issue_date < vs_cfg["DELETE_TO_DATE"] or
                supply_date < vs_cfg["DELETE_TO_DATE"] or
                taxpoint_date < vs_cfg["DELETE_TO_DATE"]):
                vs_message(self, """\
Dates prior to the date of the last
data deletion cannot be accepted.""", "ERROR")
                return

        # If the taxpoint date is greater than the issue date, flag an error
        if taxpoint_date > issue_date:
            vs_message(self, """\
The tax point date cannot be
later than the invoice issue date.""", "ERROR")
            return

        # If the taxpoint is in a previous tax period, for which a vat return has already been
        # submitted, then flag as error
        vat_scheme = VATScheme()
        vat_period_start_date = vat_scheme.start_date
        if taxpoint_date < vat_period_start_date:
            vs_message(self, """\
The tax point date cannot be in a VAT period for
which you have already submitted a VAT return!""", "ERROR")
            return

        # Issue date cannot be after payments received
        payments = self.invoice.get_payments()
        if payments:
            if issue_date > payments[0].datestamp:
                vs_message(self, "The Invoice date cannot be\nchanged to after the first payment date", "ERROR")
                return
        # Issue date cannot be after creditnotes sent
        creditnotes = self.invoice.get_creditnotes()
        if creditnotes:
            if issue_date > creditnotes[0].datestamp:
                vs_message(self, "The Invoice date cannot be\nchanged to after sent credit note", "ERROR")
                return

        # Set the dates

        # Change the issue date if necessary
        if issue_date != self.invoice.datestamp:
            if not change_transaction_date.change_date(self.invoice.transaction, issue_date):
                # The issue date could not be changed
                vs_message(self, "Failed to change the invoice issue date", "ERROR")
                return
        self.invoice.date_of_supply = supply_date
        self.invoice.date_of_taxpoint = taxpoint_date

        big_refresh(acc_tree=False)
        info_log("Invoice id : %s, dates changed" % self.invoice.id)
        self.EndModal(wx.ID_OK)


    def cancel_button_pressed(self, event):
        "Cancel button - return wx.ID_CANCEL"
        self.EndModal(wx.ID_CANCEL)


####################### CUSTOMER PAYMENTS ###################

# The ShowCustomerPaymentDialog is used to view a customer payment and edit its customer_reference
# and also show an associated invoice object if the invoice has been double clicked


def show_customer_payment_dialog(parent, payment):
    """Creates a ShowCustomerPaymentDialog, with the capability to
       then view an associated invoice by double clicking it in the list
       of invoices."""

    if payment.data_deleted():
        # Data regarding this payment is unavailable
        vs_message(parent, """\
The full payment was for %s.  However 
Invoice information relating to this
payment has been deleted, and therefore
further payment details cannot be shown.""" % to_currency_string(payment.amount),
"Data Deleted")
        return

    show_dialog = ShowCustomerPaymentDialog(parent, payment)
    busy_cursor_off()

    if show_dialog.ShowModal() != wx.ID_OK:
        show_dialog.Destroy()
        return 

    # Get the invoice to view
    invoice = show_dialog.view_invoice
    show_dialog.Destroy()

    if not invoice:
        return

    # now show the invoice
    busy_cursor_on()
    wx.Yield()
    show_invoice_frame(invoice)



class ShowCustomerPaymentDialog(vsDialog):
    """Called to show the customer payment"""

    def __init__(self, parent, payment):
        """Create the Dialog showing the payment, and associated invoices.
           If an invoice is clicked, then set the self.view_invoice attribute
           that invoice"""

        vsDialog.__init__(self, parent, "Payment Received")
        
        # Take a note of the variables
        self.payment = payment
        self.customer_reference = payment.customer_reference
        self.view_invoice = None

        button_size = (100, vs_cfg["BUTTON_HEIGHT"])

        # Create a vertical sizer to hold everything
        sizer=wx.BoxSizer(wx.VERTICAL)

        # payment reference

        # ref sizer holds the reference and a change button
        ref_sizer=wx.BoxSizer(wx.HORIZONTAL)

        ref_sizer.Add(wx.StaticText(self, -1, "Customer reference :"), 0, wx.RIGHT | wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL, 10)

        self.ref_field = wx.TextCtrl(self, -1, self.customer_reference, size=(400, vs_cfg["BUTTON_HEIGHT"]))
        self.ref_field.SetToolTipString("Edit the customer reference if you wish")
        self.ref_field.SetMaxLength(50)
        ref_sizer.Add(self.ref_field, 0, wx.RIGHT | wx.ALIGN_RIGHT, 10)

        self.change_button = wx.Button(self, label="Change", size=button_size)
        self.change_button.SetToolTipString("Click to change the customer reference")
        ref_sizer.Add(self.change_button, 0, wx.LEFT | wx.ALIGN_LEFT, 10)

        sizer.Add(ref_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        if payment.customer_address:
            # payment address on the left of a horizontal sizer, payment details on the right
            details_sizer=wx.BoxSizer(wx.HORIZONTAL)
            customer_text = "Customer: %s\n%s" % (payment.customer.name,
                                                  payment.customer_address.lines_string)
            details_sizer.Add(wx.StaticText(self, -1, customer_text), 0, wx.RIGHT | wx.ALIGN_TOP, 80)
            # Put payment details in a grid sizer
            pay_sizer=wx.FlexGridSizer(rows=4, cols=2, hgap=20, vgap=10)
        else:
            # details sizer does not exist, just create the pay sizer with the customer field added
            # Put payment details in a grid sizer
            pay_sizer=wx.FlexGridSizer(rows=5, cols=2, hgap=20, vgap=10)
            # supplier
            pay_sizer.Add(wx.StaticText(self, -1, "Customer :"), 0, wx.ALIGN_RIGHT)
            customer_text = payment.customer.name
            pay_sizer.Add(wx.StaticText(self, -1, customer_text), 0, wx.ALIGN_LEFT)

        # date
        pay_sizer.Add(wx.StaticText(self, -1, "Payment date :"), 0, wx.ALIGN_RIGHT)
        date_text = payment.datestamp.strftime("%d %b %Y")
        pay_sizer.Add(wx.StaticText(self, -1, date_text), 0, wx.ALIGN_LEFT)

        # payment amount
        pay_sizer.Add(wx.StaticText(self, -1, "Payment amount :"), 0, wx.ALIGN_RIGHT)
        amount_text = to_currency_string(payment.amount)
        pay_sizer.Add(wx.StaticText(self, -1, amount_text), 0, wx.ALIGN_LEFT)

        # payment account
        pay_sizer.Add(wx.StaticText(self, -1, "Paid to account :"), 0, wx.ALIGN_RIGHT)
        account_text = payment.account.fullname
        pay_sizer.Add(wx.StaticText(self, -1, account_text), 0, wx.ALIGN_LEFT)

        # Transaction ID
        pay_sizer.Add(wx.StaticText(self, -1, "Transaction ID :"), 0, wx.ALIGN_RIGHT)
        transaction_text = str(payment.transaction.id)
        pay_sizer.Add(wx.StaticText(self, -1, transaction_text), 0, wx.ALIGN_LEFT)

        if payment.customer_address:
            # details sizer contains address on the left, pay on the right
            details_sizer.Add(pay_sizer, 0, wx.LEFT | wx.ALIGN_LEFT, 10)
            sizer.Add(details_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 10)
        else:
            # details sizer does not exist, just add the pay sizer
            sizer.Add(pay_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        invoice_table_text = wx.StaticText(self, -1, "Double click on invoice to display invoice details :")

        sizer.Add(invoice_table_text, 0, wx.TOP | wx.ALIGN_CENTER, 10)


        # ---------------------- Create the invoices table

        self.invoices_table = PaymentInvoicesTable(self, payment)
        sizer.Add(self.invoices_table, 1, wx.ALL | wx.ALIGN_CENTER | wx.EXPAND, 20)

        # buttons

        button_sizer=wx.BoxSizer(wx.HORIZONTAL)

        # page setup button
        self.page_button = wx.Button(self, label="Page setup", size=button_size)
        self.page_button.SetToolTipString("Click to set up the pdf page size and orientation")   
        button_sizer.Add(self.page_button, 0, wx.RIGHT, 20)

        # pdf button
        self.pdf_button = wx.Button(self, label="Create pdf", size=button_size)
        self.pdf_button.SetToolTipString("Click to create a pdf file receipt.")
        button_sizer.Add(self.pdf_button, 0, wx.RIGHT, 20)

        # Close button
        self.close_button = wx.Button(self, wx.ID_CLOSE, size=button_size)
        self.close_button.SetToolTipString("Click to close.")
        button_sizer.Add(self.close_button, 0)

        sizer.Add(button_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # Set the size to the panel
        self.SetSizer(sizer)
        sizer.SetSizeHints(self)

        self.Bind(wx.EVT_BUTTON, self.change_button_pressed, self.change_button)
        self.Bind(wx.EVT_BUTTON, self.close_button_pressed, self.close_button)
        self.Bind(wx.EVT_BUTTON, self.page_button_pressed, self.page_button)
        self.Bind(wx.EVT_BUTTON, self.pdf_button_pressed, self.pdf_button)


    @writecheck
    @button_wrap
    def change_button_pressed(self, event):
        """change the payment customer reference"""
        self.customer_reference = self.ref_field.GetValue().strip()
        self.payment.customer_reference = self.customer_reference
        # refresh windows that need it, but not the accounts tree,
        # as this reference does not appear there
        big_refresh(acc_tree=False)
        info_log("Payment id : %s customer reference changed" % self.payment.id)
        vs_message(self, "Customer reference changed", "DONE")


    def close_button_pressed(self, event):
        """Close the dialog"""
        self.EndModal(wx.ID_OK)


    @button_wrap
    def page_button_pressed(self, event):
        "page button pressed, setup the pdf page size and orientation"
        vatstuff_packages.vs_pdf.page_setup.page_setup_dialog(self)


    @button_wrap
    def pdf_button_pressed(self, event):
        "Create a pdf file"

        if not vs_cfg["REPORTLABS_AVAILABLE"]:
            # no pdf capability
            vs_message(self, "Function unavailable - the reportlab module is not installed",
                             "ERROR")
            return

        # default filename
        filename= "payment_receipt_%s_sent_%s.pdf" % (self.payment.transaction.id, self.payment.datestamp.isoformat())

        # Create the save file dialog.
        status, filepath = save_file_as(self, 'pdf', vs_cfg["REPORTS_PDF_FOLDER"], filename)

        if status:
            if vatstuff_packages.vs_pdf.customerpayment_receipt.create_pdf(filepath, self.payment):
                # And show the file in the default viewer
                if not vatstuff_packages.vs_pdf.pdf_functions.show_pdf_file(filepath):
                    vs_message(self, "pdf file created", "Done")
            else:
                vs_message(self, "Unable to create the pdf file", "ERROR")


class PaymentInvoicesTable(wx.grid.Grid):

    def __init__(self, parent, payment):
        wx.grid.Grid.__init__(self, parent, style=wx.WANTS_CHARS)

        self.parent = parent

        self.inv_list = payment.get_invoice_list()
        # inv_list being a list of lists - { "INVOICE":invoice,
        #                                    "COST":cost,
        #                                    "VAT":vat }
        #         one dictionary for each invoice this payment covers

        num_rows = len(self.inv_list)
        num_cols = 3

        self.CreateGrid(num_rows, num_cols)
        self.EnableEditing(False)

        self.SetDefaultRowSize(vs_cfg["BUTTON_HEIGHT"], False)
        self.SetDefaultCellAlignment(wx.ALIGN_LEFT, wx.ALIGN_CENTER)

        # create colum labels
        self.SetColLabelValue(0, "Invoice description")
        self.SetColLabelValue(1, "Net Paid")
        self.SetColLabelValue(2, "VAT Paid")

        # Set the colums showing money as right aligned
        attr_right = wx.grid.GridCellAttr()
        attr_right.SetAlignment(wx.ALIGN_RIGHT, wx.ALIGN_CENTER)
        self.SetColAttr(1, attr_right)
        self.SetColAttr(2, attr_right)

        self.SetColSize(0, 300)
        self.SetColSize(1, 100)
        self.SetColSize(2, 100)

        self.SetColMinimalAcceptableWidth(100)
        self.SetRowLabelSize(100)

        for row, item in enumerate(self.inv_list):
            # Set row label as the invoice number
            self.SetRowLabelValue(row, str(item["INVOICE"].number))
            # Description
            self.SetCellValue(row, 0, item["INVOICE"].description)
            # Net
            self.SetCellValue(row, 1, to_currency_string(item["COST"]))
            # VAT
            self.SetCellValue(row, 2, to_currency_string(item["VAT"]))

        # Bind left mouse double - click
        self.Bind(wx.grid.EVT_GRID_CELL_LEFT_DCLICK, self.choose_invoice)


    def choose_invoice(self, event):
        "On being double clicked, obtain invoice and set parent attribute"
        try:
            row = event.GetRow()
            # get the invoice for this row
            self.parent.view_invoice = self.inv_list[row]["INVOICE"]
            # close the parent
            self.parent.EndModal(wx.ID_OK)
        except Exception, e:
            error_message("Error trying to choose the invoice", e)




