####### VATStuff #######
#
# edit_rates.py  - Creates a table to edit vat_rates
#
# Version : 0.6
# Date : 20100808
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#


"""Creates a table to edit vat_rates

It is run as part of the vatstuff gui interface.
"""

import sys
import decimal

import wx
import wx.grid



from vatstuff_packages.vs_database.create_objects import create_VATRate



from vatstuff_packages.vs_utilities import *

from vatstuff_packages.vs_gui.gui_functions import *

D = decimal.Decimal   # D used to avoid typing


def edit_rates_dialog(parent):
    """Creates an EditRatesDialog"""
    edit_rates_dialog = EditRatesDialog(parent)

    # If the cursor is busy, unbusy it - as
    # now waiting for a button press
    busy_cursor_off()

    edit_rates_dialog.ShowModal()
    edit_rates_dialog.Destroy()


class EditRatesDialog(vsDialog):
    "The dialog to display and edit vat rates"

    def __init__(self, parent):
        """Create the Dialog"""
        vsDialog.__init__(self, parent, "VAT rates")

        button_size = (100, vs_cfg["BUTTON_HEIGHT"])

        # Create a vertical sizer to hold everything
        sizer=wx.BoxSizer(wx.VERTICAL)

        help_text="""\
When purchasing or selling items, you can choose one of the tax code symbols shown
here to apply the appropriate rate of VAT. This table allows you to set the rates,
and you can also insert new rates if required.
Click on any item to edit it, and then click the Apply button to save your changes"""
        
        self.top_text=wx.StaticText(self, -1, help_text)
        sizer.Add(self.top_text, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # ---------------------- Create the rates table

        self.rates_table = RatesTable(self)
        sizer.Add(self.rates_table, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # ---------------------- Apply - Cancel -- buttons

        # Create a horizontal button_sizer to hold the buttons
        button_sizer=wx.BoxSizer(wx.HORIZONTAL)

        # Apply button
        self.apply_button = wx.Button(self, wx.ID_APPLY, size=button_size)
        self.apply_button.SetToolTipString("Click to apply changes.")
        button_sizer.Add(self.apply_button, 0, wx.RIGHT, 40)
        # Start off with apply button disabled - enabled by any changes
        self.apply_button.Enable(False)

        # Close button
        self.close_button = wx.Button(self, wx.ID_CLOSE, size=button_size)
        self.close_button.SetToolTipString("Click to close.")
        button_sizer.Add(self.close_button, 0)

        # Add button_sizer to the main sizer
        sizer.Add(button_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # Set the size to the panel
        self.SetSizer(sizer)
        sizer.SetSizeHints(self)

        # ---------- Bind buttons to handler functions
        
        self.Bind(wx.EVT_BUTTON, self.apply_button_pressed, self.apply_button)
        self.Bind(wx.EVT_BUTTON, self.close_button_pressed, self.close_button)


    # ----------------- Handler functions

    @writecheck
    @button_wrap
    def apply_button_pressed(self, event):
        """Apply button - check and apply all fields"""
        status, message = self.rates_table.check_fields()
        if not status:
            vs_message(self, message, "ERROR")
            return
        self.apply_button.Enable(False)
        self.rates_table.refresh()
        vs_message(self, message, "INFORMATION")


    def close_button_pressed(self, event):
        """Close down"""
        if not self.apply_button.IsEnabled():
            self.EndModal(wx.ID_OK)
            return
        if vs_yesno(self, """\
Closing vat rates list: are you sure?
Any changes will be lost unless you press Apply""", "Closing list"):
            self.EndModal(wx.ID_OK)


class RatesTable(wx.grid.Grid):

    def __init__(self, parent):
        wx.grid.Grid.__init__(self, parent, size=(750,200), style=wx.WANTS_CHARS)

        self.parent = parent
        self.all_vat_rates = VATRate.get_all()

        num_rows = len(self.all_vat_rates) + 5   # Make 5 'blank' rows
        num_cols = 4

        self.CreateGrid(num_rows, num_cols)

        self.SetDefaultRowSize(vs_cfg["BUTTON_HEIGHT"], False)
        self.SetDefaultCellAlignment(wx.ALIGN_LEFT, wx.ALIGN_CENTER)

        # create column labels
        self.SetColLabelValue(0, "Designation")
        self.SetColLabelValue(1, "Symbol")
        self.SetColLabelValue(2, "%")
        self.SetColLabelValue(3, "Description")

        # Set the colums showing VAT symbol and % as centre aligned
        attr = wx.grid.GridCellAttr()
        attr.SetAlignment(wx.ALIGN_CENTER, wx.ALIGN_CENTER)

        self.SetColAttr(1, attr)
        self.SetColAttr(2, attr)

        for key, data in enumerate(self.all_vat_rates):
            self.SetCellValue(key, 0, data.name)
            self.SetCellValue(key, 1, data.symbol)
            self.SetCellValue(key, 2, str(data.rate).replace(".", vs_cfg["CP"]))
            self.SetCellValue(key, 3, data.description)

        self.SetColSize(0, 150)
        self.SetColSize(1, 100)
        self.SetColSize(2, 100)
        self.SetColSize(3, 300)

        self.SetColMinimalAcceptableWidth(100)
        self.SetRowLabelSize(50)

        # When an item is edited, add a further row to ensure there are always blank rows
        # displayed:

        self.Bind(wx.grid.EVT_GRID_CELL_CHANGE, self.addarow)


    def refresh(self):
        "Get all rates again, and re-fill the table"

        self.all_vat_rates = VATRate.get_all()
        number_of_rates = len(self.all_vat_rates)

        # If everything is working ok, this must be true
        assert number_of_rates < self.GetNumberRows()
        
        # clear the grid
        self.ClearGrid()

        for key, data in enumerate(self.all_vat_rates):
            self.SetCellValue(key, 0, data.name)
            self.SetCellValue(key, 1, data.symbol)
            self.SetCellValue(key, 2, str(data.rate).replace(".", vs_cfg["CP"]))
            self.SetCellValue(key, 3, data.description)


    def addarow(self, evt):
        # Keep some blank rows at the bottom available for editing
        # if the user starts inserting data in the last four rows - 
        # add a further row
        try:
            # as an item has been edited, enable the apply button
            self.parent.apply_button.Enable(True)
            current_number_of_rows = self.GetNumberRows()
            if evt.GetRow()+4 > current_number_of_rows:
                self.AppendRows(numRows=1)
        except Exception, e:
            error_message("Failure in editing an item row", e)


    def check_fields(self):
        """Read and check the items on the table"""

        table = []
        field_number = self.GetNumberRows()
        for row in range(0, field_number):
            rate = []
            for col in range(0,4):
                item= self.GetCellValue(row, col)
                rate.append(item.strip())
            table.append(rate)

        # table is now the list of lists of vat rates.

        for key, data in enumerate(table):            # data[0] = name
            if data[1] == "" and data[2] == "":       # data[1] = symbol
                continue                              # data[2] = rate
                                                      # data[3] = description
            # Check designation
            if len(data[0]) > 20:
                return False, """Designation too long (20 characters max) at line %s
Your changes have not been saved""" % (key+1,)

            # Check symbol
            if (data[1] == "") or (len(data[1]) > 1) or (not data[1].isalpha()) or (not data[1].isupper()):
                return False, """The symbol at line %s must be an uppercase letter only.
Your changes have not been saved""" % (key+1,)

            # Check rate
            try:
                number = D(data[2].replace(vs_cfg["CP"], "."))     # Set string as a decimal type 
            except Exception:
                return False, """Error in the VAT rate value at line %s
Your changes have not been saved""" % (key+1,)

            if number < D(0) or number >= D(100):
                return False, """Error in the VAT rate value at line %s
Your changes have not been saved""" % (key+1,)

            # Check description
            if len(data[3]) > 50:
                return False, """Description too long (50 characters max) at line %s
Your changes have not been saved""" % (key+1,)

        # The symbol needs to be unique - both on the table and in the VATrate instances
        # check on tables first:

        for key1, data1 in enumerate(table):
            for key2, data2 in enumerate(table):
                if key1 == key2:
                    continue
                if data1[1] == "" or data2[1] == "":
                    continue
                if data1[1] == data2[1]:
                    return False, """The VAT Rate symbols must be unique,
Your changes have not been saved"""

        # Check the VAT rates in vs_cfg["VAT_SYMBOLS"] all exist
        symbol_list_in_table = [ data[1] for data in table ]
        for symbol in vs_cfg["VAT_SYMBOLS"] :
            if not symbol in symbol_list_in_table:
                return False, """The %s rate with symbol %s must exist,
Your changes have not been saved""" % (vs_cfg["VAT_SYMBOLS"][symbol], symbol)


        # Set S - the standard rate, into vs_cfg["VAT_STANDARD_RATE"]
        standard_rate = ""
        for data in table:
            if data[1] == "S":
                standard_rate = data[2]
        vs_cfg["VAT_STANDARD_RATE"] = D(standard_rate.replace(vs_cfg["CP"],"."))

        # Set R - the reduced rate, into vs_cfg["VAT_REDUCED_RATE"]
        reduced_rate = ""
        for data in table:
            if data[1] == "R":
                reduced_rate = data[2]
        vs_cfg["VAT_REDUCED_RATE"] = D(reduced_rate.replace(vs_cfg["CP"],"."))

        change = False

        # for any symbol that is changing in the vatrates - first set the vatrate
        # to be something temporary and unique - which will later get overidden by
        # the new value

        # The point of this is to cover the situation where a symbol is being moved
        # rather than completely changed - to avoid it getting put into a vatrate 
        # while it still exisits in a later field that hasn't been updated yet 
        for key, data in enumerate(self.all_vat_rates):
            if table[key][1] == "":
                data.delete()
                change = True
                continue
            if table[key][1] != data.symbol:
                data.symbol = str(key)       # using str(key) as a unique value
                change = True

        # Update or delete any of the current rates which need changing

        for key, data in enumerate(self.all_vat_rates):
            if table[key][1] == "":
                # This item already deleted
                continue

            if table[key][0] != data.name:
                data.name = table[key][0]
                change = True
            if table[key][1] != data.symbol:
                data.symbol = table[key][1]
                change = True
            if table[key][2] != str(data.rate).replace(".",vs_cfg["CP"]):
                data.rate = D(table[key][2].replace(vs_cfg["CP"],"."))
                change = True
            if table[key][3] != data.description:
                data.description = table[key][3]
                change = True

        # And create new rates

        for key in range(len(self.all_vat_rates), field_number):
            if table[key][1] == "" or table[key][2] == "":
                continue
            create_VATRate(table[key][0],                               # name
                           table[key][1],                               # symbol
                           D(table[key][2].replace(vs_cfg["CP"],".")),  # rate
                           table[key][3])                               # description
            change = True

        if change:
            info_log("VAT rates changes have been saved")
            return True, "Changes have been saved"
        else:
            return True, "No changes have been implemented"

