####### VATStuff #######
#
# previous_returns.py  - Shows a previously submitted VAT return
#
# Version : 0.6
# Date : 20100816
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#

"""Shows a previously submitted VAT return

Creating an instance of PreviousVATReturnFrame(parent, vat_return)
creates a frame showing the given vat_return
Note this is a subclass of wxFrame and needs the Show() method to
display the frame.
"""


import decimal

import wx
import wx.html





from vatstuff_packages.vs_database.transactions import *

from vatstuff_packages.vs_utilities import *

from vatstuff_packages.vs_gui.gui_functions import *
from vatstuff_packages.vs_verify.form_objects import ReportForm

if vs_cfg["REPORTLABS_AVAILABLE"]:
    import vatstuff_packages.vs_pdf.page_setup
    import vatstuff_packages.vs_pdf.create_report


D = decimal.Decimal   # D used to avoid typing


def previous_vat_return_frame(vatreturn):
    """Show the PreviousVATReturnFrame which lists the VAT Return,
       which has been previously submitted"""

    # No try:exception statement - allows the caller to catch exceptions

    # Check the frame is not already open
    for frm in vs_cfg["TRANSACTION_REFRESH"]:
        if isinstance(frm, PreviousVATReturnFrame):
            if frm.vatreturn == vatreturn:
                frm.Raise()
                break
    else:
        # top_frame() is the top frame, which will be the parent
        frm = PreviousVATReturnFrame(top_frame(), vatreturn)
        frm.Show()


# ------------------- PreviousVATReturnFrame

class PreviousVATReturnFrame(vsFrame):
    "Called to display the frame to show a previously submitted VAT Return"

    def __init__(self, parent, vatreturn):
        """Create the frame"""
        vsFrame.__init__(self, parent, title="VAT Return")

        self.vatreturn = vatreturn

        self.previous_vat_return_panel=PreviousVATReturnPanel(self, vatreturn)

        # Get the panel size
        panel_size = self.previous_vat_return_panel.GetMinSize()

        # Set minimum size equal to the panel
        self.SetMinSize(panel_size)
        self.SetClientSize(panel_size)

        # update the list of windows which need a refresh if
        # a transaction occurs, by adding self
        vs_cfg["TRANSACTION_REFRESH"].append(self)

        # bind the close event
        self.Bind(wx.EVT_CLOSE, self.onclose)


    def refresh(self):
        "Calls the refresh in the panel"
        self.previous_vat_return_panel.refresh()


    def onclose(self, evt):
        "Shutdown the frame"
        # Decrement the refresh list
        vs_cfg["TRANSACTION_REFRESH"].remove(self)
        self.Destroy()



# ----------------------PreviousVATReturnPanel

class PreviousVATReturnPanel(wx.Panel):
    """The panel of the PreviousVATReturn frame"""

    def __init__(self, parent, vatreturn):
        """Create the panel"""
        wx.Panel.__init__(self, parent)
        
        # Take a note of the variables
        self.parent = parent
        self.vatreturn = vatreturn

        button_size = (100, vs_cfg["BUTTON_HEIGHT"])

        self.sizer=wx.BoxSizer(wx.VERTICAL)

        self.reportwindow = wx.html.HtmlWindow(self)
        if "gtk2" in wx.PlatformInfo:
            self.reportwindow.SetStandardFonts()

        self.reportwindow.SetPage(self.create_report())

        # This widget needs to expand in all directions
        self.sizer.Add(self.reportwindow, 1, wx.ALL | wx.ALIGN_CENTER | wx.EXPAND, 20)

        # buttons

        button_sizer=wx.BoxSizer(wx.HORIZONTAL)

        # page setup button
        self.page_button = wx.Button(self, label="Page setup", size=button_size)
        self.page_button.SetToolTipString("Click to set up the pdf page size and orientation")   
        button_sizer.Add(self.page_button, 0, wx.RIGHT, 20)

        # pdf button
        self.pdf_button = wx.Button(self, label="Create pdf", size=button_size)
        self.pdf_button.SetToolTipString("Click to create a pdf file of the order form.")
        button_sizer.Add(self.pdf_button, 0, wx.RIGHT, 20)

        # Close button
        self.close_button = wx.Button(self, wx.ID_CLOSE, size=button_size)
        self.close_button.SetToolTipString("Click to close.")
        button_sizer.Add(self.close_button, 0)

        self.sizer.Add(button_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)
      
        # Set the size to the panel
        self.SetSizer(self.sizer)

        # Set panel minimum size 
        self.SetSizeHints(800, 600)

        # ---------- Bind buttons to handler functions
        
        self.Bind(wx.EVT_BUTTON, self.page_button_pressed, self.page_button)
        self.Bind(wx.EVT_BUTTON, self.pdf_button_pressed, self.pdf_button)
        self.Bind(wx.EVT_BUTTON, self.close_button_pressed, self.close_button)


    @button_wrap
    def page_button_pressed(self, event):
        "page button pressed, setup the pdf page size and orientation"
        vatstuff_packages.vs_pdf.page_setup.page_setup_dialog(self)


    @button_wrap
    def pdf_button_pressed(self, event):
        "Create a pdf file from the report"
        if not vs_cfg["REPORTLABS_AVAILABLE"]:
            # no pdf capability
            vs_message(self, "Function unavailable - the reportlab module is not installed",
                             "ERROR")
            return

        # default filename
        filename= "VAT_Return%s.pdf" % self.vatreturn.to_date.isoformat()
        if not vatstuff_packages.vs_pdf.create_report.create_pdf(self, filename, self.report, self.vatreturn):
            vs_message(self, "Unable to create the pdf file", "ERROR")


    def close_button_pressed(self, event):
        """ Close button - Close parent frame"""
        self.parent.Close()
        return


    def refresh(self):
        """The refresh function disables the report"""
        self.page_button.Enable(False)
        self.pdf_button.Enable(False)
        self.reportwindow.SetPage(no_report())


    # Method to create the report
    def create_report(self):
        "get the vat_return from self.vat_return, create the report in self.report, and returns the html text"

        self.report = ReportForm("Value Added Tax Return")

        self.report.header = ["For the period:",
                              "From: %s" % self.vatreturn.from_date.strftime("%d %b %Y"),
                              "To: %s" % self.vatreturn.to_date.strftime("%d %b %Y"),
                              ""]

        vat_scheme = self.vatreturn.vatscheme

        vatscheme_text = ""
        vat_method = vat_scheme.method
        if vat_method == "STANDARD":
            vatscheme_text = "VAT Scheme : Standard"
        elif vat_method == "CASH":
            vatscheme_text = "VAT Scheme : Cash"
        elif vat_method == "FLAT-BASIC":
            vatscheme_text = """\
VAT Scheme : Flat (Basic) rate at %s%%""" % str(vat_scheme.flat_percent).replace(".", vs_cfg["CP"])
        elif vat_method == "FLAT-CASH":
            vatscheme_text = """\
VAT Scheme : Flat (Cash) rate at %s%%""" % str(vat_scheme.flat_percent).replace(".", vs_cfg["CP"])
        if vatscheme_text:
            self.report.header.append(vatscheme_text)

        # tables are described by a list of list,
        # each list being a row, consisting of cell tuple
        # each cell tuple containing two strings; 
        # The first string is formed from "XXXN",
        # and the second string is the text to be placed in the cell

        # N is the collspan number, as used in a table cell tag attribute
        # such as 'colspan="4"' - in which case N is 4

        # so normally N is '1'

        # XXX is a formatting string :
        # HBL  - Header Bold Left Justified (R for Right instead of L
        # HNL  - Header Normal Left Justified  etc
        # DNL  - Data Cell Normal Left Justified  etc
        # XXC  - Right Justified currency, as pulled from the database

        # BOX 1
        self.report.table = [[ ('DNL1', '1'),
                               ('DNL1', 'VAT due in this period on sales and other outputs'),
                               ('DNC1', str(self.vatreturn.box_one))   ]]

        # BOX 2
        self.report.add_row_to_table( ('DNL1', '2'),
                                      ('DNL1', """\
VAT due in this period on acquisitions from other
EC member States"""),
                                      ('DNC1', str(self.vatreturn.box_two)) )

        # BOX 3
        self.report.add_row_to_table( ('DNL1', '3'),
                                      ('DNL1', "Total VAT due (the sum of boxes 1 and 2)"),
                                      ('DNC1', str(self.vatreturn.box_three)) )

        # BOX 4
        self.report.add_row_to_table( ('DNL1', '4'),
                                      ('DNL1', """\
VAT reclaimed in this period on purchase and other inputs
(including acquisitions from the EC)"""),
                                      ('DNC1', str(self.vatreturn.box_four)) )

        # BOX 5
        self.report.add_row_to_table( ('DNL1', '5'),
                                      ('DNL1', """\
Net VAT to be paid to HMRC or reclaimed by you
(Difference between boxes 3 and 4)"""),
                                      ('DNC1', str(self.vatreturn.box_five)) )

        # BOX 6
        self.report.add_row_to_table( ('DNL1', '6'),
                                      ('DNL1', """\
Total value of sales and all other outputs excluding
any VAT. Include your box 8 figure"""),
                                      ('DNC1', str(self.vatreturn.box_six)) )

        # BOX 7
        self.report.add_row_to_table( ('DNL1', '7'),
                                      ('DNL1', """\
Total value of purchases and all other inputs excluding
any VAT. Include your box 9 figure"""),
                                      ('DNC1', str(self.vatreturn.box_seven)) )

        # BOX 8
        self.report.add_row_to_table( ('DNL1', '8'),
                                      ('DNL1', """\
Total value of all supplies of goods and related costs,
excluding any VAT, to other EC Member States"""),
                                      ('DNC1', str(self.vatreturn.box_eight)) )

        # BOX 9
        self.report.add_row_to_table( ('DNL1', '9'),
                                      ('DNL1', """\
Total value of all acquisitions of goods and related costs,
excluding any VAT, from other EC Member States"""),
                                      ('DNC1', str(self.vatreturn.box_nine)) )

        self.report.footer = []

        # Added text if vat method is flat-basic or flat-cash
        if vat_method == "FLAT-BASIC":
            self.report.footer.extend(
                ["NOTE: Box 1 is calculated as %s%% of total (VAT inclusive) sales" % (
                                 str(vat_scheme.flat_percent).replace(".", vs_cfg["CP"])),
                 ""])
        elif vat_method == "FLAT-CASH":
            self.report.footer.extend(
                ["NOTE: Box 1 is calculated as %s%% of total (VAT inclusive)" % (
                                 str(vat_scheme.flat_percent).replace(".", vs_cfg["CP"])),
                 "payments received from customers",
                 ""])

        # Added text if box 2 has contents
        if self.vatreturn.box_two > D("0.00"):
            vat_standard_rate = str(self.vatreturn.standard_rate).replace(".", vs_cfg["CP"])
            self.report.footer.extend(
                  ["NOTE: Box 2 is calculated at the standard VAT rate : %s%%," % (vat_standard_rate,),
                   "of the cost of your purchases from suppliers within the EC.",
                   "(That is, EC suppliers who do not have %s set as their VAT country code.)" % (vs_cfg["VAT_COUNTRY_CODE"],),
                   ""])

        # Added text if box 8 has contents
        if self.vatreturn.box_eight > D("0.00"):
            self.report.footer.extend(
                   ["NOTE: Box 8 is calculated as the sales to customers in EC member states",
                    "who are VAT registered (their countries equivalent of VAT).",
                    ""])

        self.report.footer.append("Submitted : %s" % self.vatreturn.datestamp.strftime("%d %b %Y"))

        return self.report.render_to_html()

            


