####### VATStuff #######
#
# set_vat_scheme.py  - sets the company VAT scheme and VAT periods
#
# Version : 0.7.0
# Date : 20100913
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#

"""Sets the company VAT scheme and VAT periods

It is run as part of the vatstuff gui interface.
"""


import datetime
import calendar
import decimal

import wx

from vatstuff_packages.vs_database import create_objects

from vatstuff_packages.vs_utilities import *

from vatstuff_packages.vs_gui.gui_functions import *

D = decimal.Decimal   # D used to avoid typing



def vat_scheme_dialog(parent):
    """Creates a VATSchemeDialog"""
    vat_scheme_dialog = VATSchemeDialog(parent)

    # If the cursor is busy, unbusy it - as
    # now waiting for a button press
    busy_cursor_off()

    vat_scheme_dialog.ShowModal()
    vat_scheme_dialog.Destroy()


class VATSchemeDialog(vsDialog):
    "The vat scheme dialog, to display and edit vat schemes"

    def __init__(self, parent):
        """Create the Dialog"""
        vsDialog.__init__(self, parent, "VAT scheme")

        # Get the current VAT Scheme
        self.vat_scheme = VATScheme()
        # Get the current Company
        self.company = Company()

        button_size = (100, vs_cfg["BUTTON_HEIGHT"])

        # Create a vertical sizer to hold everything
        sizer=wx.BoxSizer(wx.VERTICAL)

        top_text=wx.StaticText(self, -1, """\
Changing VAT schemes may require extra payments to HMRC for any VAT owed
on the previous scheme.  You should calculate such corrections manually,
as the VAT return calculations only take account of the current VAT period""")

        sizer.Add(top_text, 0, wx.TOP | wx.ALIGN_CENTER, 20)

        # Create radio fields and text in a grid sizer
        field_sizer=wx.FlexGridSizer(rows=4, cols=2, hgap=40, vgap=40)

        # VAT method

        method_list = ['Standard - calculated from invoices',
                       'Cash - calculated from payments',
                       'Flat-Basic - a percentage of invoices issued',
                       'Flat-Cash  - a percentage of payments received',
                       'Not Registered.'
                       ]

        self.vat_method_field = wx.RadioBox(self, -1, "",
                                            wx.DefaultPosition, wx.DefaultSize,
                                            method_list, 0, wx.NO_BORDER | wx.RA_VERTICAL
                                            )

        current_vat_method = self.vat_scheme.method
        if current_vat_method == "STANDARD":
            self.vat_method_field.SetSelection(0)
        if current_vat_method == "CASH":
            self.vat_method_field.SetSelection(1)
        if current_vat_method == "FLAT-BASIC":
            self.vat_method_field.SetSelection(2)
        if current_vat_method == "FLAT-CASH":
            self.vat_method_field.SetSelection(3)
        if current_vat_method == "NOT_REGISTERED":
            self.vat_method_field.SetSelection(4)
        
        self.vat_method_field.SetItemToolTip(0,"Set if you use the standard VAT calculations.")
        self.vat_method_field.SetItemToolTip(1,"Set if you calculate VAT on payments.")
        self.vat_method_field.SetItemToolTip(2,"Set if you are on the flat rate (basic) scheme.")
        self.vat_method_field.SetItemToolTip(3,"Set if you are on the flat rate (cash) scheme.")
        self.vat_method_field.SetItemToolTip(4,"Set if you are not VAT Registered.")
        field_sizer.Add(self.vat_method_field, 0, wx.ALIGN_LEFT)

        text1=wx.StaticText(self, -1, """
Set the VAT scheme your company uses. You should
take advice on the correct scheme - most have
conditions and require Government agreement.""")
        field_sizer.Add(text1, 0, wx.ALIGN_LEFT)

        # Flat scheme percentage value

        flat_sizer=wx.BoxSizer(wx.HORIZONTAL)
        flat_sizer.Add(wx.StaticText(self, -1, "Flat scheme percentage of sales:"), 0, wx.RIGHT | wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL, 10)
        self.flat_percent = self.vat_scheme.flat_percent
        flat_percent_text = str(self.flat_percent).replace(".", vs_cfg["CP"])
        self.flat_percent_field = wx.TextCtrl(self, -1, flat_percent_text, style=wx.ALIGN_RIGHT, size=(60, vs_cfg["BUTTON_HEIGHT"]))
        self.flat_percent_field.SetToolTipString("Set the percentage of sales to be accounted as VAT")
        self.flat_percent_field.SetMaxLength(4)
        flat_sizer.Add(self.flat_percent_field, 0, wx.ALIGN_CENTER_VERTICAL)
        flat_sizer.Add(wx.StaticText(self, -1, "%"), 0, wx.ALIGN_CENTER_VERTICAL)
        field_sizer.Add(flat_sizer,0, wx.ALIGN_RIGHT)

        text2=wx.StaticText(self, -1, """\
This percentage only applies to the Flat Schemes
(Basic or Cash) and varies by trade sector.""")
        field_sizer.Add(text2, 0, wx.ALIGN_LEFT)

        # If the method is flat, enable the flat scheme percentage field
        if current_vat_method == "FLAT-BASIC" or current_vat_method == "FLAT-CASH":
            self.flat_percent_field.Enable(True)
        else:
            self.flat_percent_field.Enable(False)

        # Have previous vat returns been submitted
        self.vat_return_submitted = False
        if VATReturn.get_last():
            self.vat_return_submitted = True

        # Start of current VAT period
        start_date_datetime_object = self.vat_scheme.start_date
        if start_date_datetime_object:
            start_date = date_to_wxdate(start_date_datetime_object)
        else:
            start_date = wx.DateTime.Now()
            # Set to the first day of the month
            start_date.SetDay(1)

        self.start_date_field = wx.DatePickerCtrl(self, -1,
                                                 start_date, 
                                                 size=(200, vs_cfg["BUTTON_HEIGHT"]),
                                                 style=wx.DP_DROPDOWN | wx.DP_SHOWCENTURY)
        self.start_date_field.SetToolTipString("The start date of the current VAT period")
        field_sizer.Add(self.start_date_field, 0, wx.ALIGN_RIGHT)

        text4=wx.StaticText(self, -1, """\
Set this to the start date of the quarterly VAT period for which
you still have to submit a return. The program will assume the
next payment is due four months after this date.""")
        field_sizer.Add(text4, 0, wx.ALIGN_LEFT)

        if self.vat_return_submitted or current_vat_method == "NOT_REGISTERED":
            # If a previous vat return has been submitted, this VAT scheme
            # must start from the day after that vat return period, and therefore
            # the date cannot be manually set
            self.start_date_field.Enable(False)

        # vat country code and registration number
        vat_sizer=wx.BoxSizer(wx.HORIZONTAL)

        self.vat_country_code_field = wx.TextCtrl(self, -1, self.company.vat_country_code, size=(40, vs_cfg["BUTTON_HEIGHT"]))
        self.vat_country_code_field.SetToolTipString("Your VAT two letter country code")
        self.vat_country_code_field.SetMaxLength(2)
        vat_sizer.Add(self.vat_country_code_field, 0, wx.ALIGN_RIGHT)
        self.vat_country_code_field.Enable(False)

        self.vat_regnumber_field = wx.TextCtrl(self, -1, self.company.vat_regnumber, size=(150, vs_cfg["BUTTON_HEIGHT"]))
        self.vat_regnumber_field.SetToolTipString("Your VAT number")
        self.vat_regnumber_field.SetMaxLength(20)
        vat_sizer.Add(self.vat_regnumber_field, 0, wx.LEFT | wx.ALIGN_RIGHT, 10)

        field_sizer.Add(vat_sizer, 0, wx.ALIGN_RIGHT)

        vat_regnumber_text=wx.StaticText(self, -1, "Set your VAT number")

        field_sizer.Add(vat_regnumber_text, 0, wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        if current_vat_method == "NOT_REGISTERED":
            # If the vat method is NOT REGISTERED, then the vat
            # registration number cannot be set
            self.vat_regnumber_field.Enable(False)

        sizer.Add(field_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 40)

        bottom_text=wx.StaticText(self, -1, """\
Warning - you are advised to save a backup of the database before applying a VAT scheme change.""")
        sizer.Add(bottom_text, 0, wx.BOTTOM | wx.ALIGN_CENTER, 20)


        # ---------------------- Apply - Cancel -- buttons

        # Create a horizontal button_sizer to hold the buttons
        button_sizer=wx.BoxSizer(wx.HORIZONTAL)

        # Apply button
        self.apply_button = wx.Button(self, wx.ID_APPLY, size=button_size)
        self.apply_button.SetToolTipString("Click to apply changes.")
        button_sizer.Add(self.apply_button, 0, wx.RIGHT, 40)

        # Cancel button
        self.cancel_button = wx.Button(self, wx.ID_CANCEL, size=button_size)
        self.cancel_button.SetToolTipString("Click to cancel.")
        button_sizer.Add(self.cancel_button, 0)

        # Add button_sizer to the main sizer
        sizer.Add(button_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # Set the size to the panel
        self.SetSizer(sizer)
        sizer.SetSizeHints(self)

        # ---------- Bind buttons to handler functions
        
        self.Bind(wx.EVT_BUTTON, self.apply_button_pressed, self.apply_button)
        self.Bind(wx.EVT_BUTTON, self.cancel_button_pressed, self.cancel_button)
        self.Bind(wx.EVT_RADIOBOX, self.vat_method_pressed, self.vat_method_field)


    # ----------------- Handler functions

    def vat_method_pressed(self, event):
        "The vat method has been changed"
        try:
            method = self.vat_method_field.GetSelection()
            # If the method is flat, enable the flat scheme percentage field
            if method == 2 or method == 3:
                self.flat_percent_field.Enable(True)
            else:
                self.flat_percent_field.Enable(False)
            # If the method is not registered, disable the vat registration number field
            if method == 4:
                self.vat_regnumber_field.Enable(False)
                self.vat_regnumber_field.SetValue("")
            else:
                self.vat_regnumber_field.Enable(True)
                vrtext=self.vat_regnumber_field.GetValue()
                vrtext=str(vrtext).strip()
                if not vrtext:
                    self.vat_regnumber_field.SetValue(self.company.vat_regnumber)

            if (not self.vat_return_submitted) and method != 4:
                # date can be enabled
                self.start_date_field.Enable(True)
            else:
                self.start_date_field.Enable(False)
        except Exception:
            error_message("Error on setting the vat method", e)


    def get_confirmation_text(self, method, start_date):
        "Returns the confirm text appropriate to the VAT Scheme chosen"

        if method == "STANDARD":
            text = """\
You have chosen the Standard method, in which VAT is
calculated as invoices are issued and received.
"""
        elif method == "CASH":
            text = """\
You have chosen the Cash method, in which VAT is
calculated as payments are made and received.
"""
        elif method == "FLAT-BASIC":
            text = """\
You have chosen the Flat (Basic) method, in which the
VAT you pay is a flat percentage of sales calculated
from the invoices you issue.
"""
        elif method == "FLAT-CASH":
            text = """\
You have chosen the Flat (Cash) method, in which the
VAT you pay is a flat percentage of sales calculated
from the payments you receive.
"""
        elif method == "NOT_REGISTERED":
            text = """\
You have chosen 'Not registered', you will
not be able to charge VAT and the VAT return
calculations will not be available.
"""
            return text
        else:
            text = "Method not recognised"
            return text
        # currently, only quarterly periods are handled

        months_list = calendar.month_name
        month_selected = start_date.month

        # month_list runs from [0] to [12] with [0] being an empty string
        # month_selected is 1 to 12

        end = month_selected+2
        due = month_selected+3
        if end > 12:
            end = end - 12
        if due > 12:
            due = due-12
        text += """\
The current VAT period runs from %s to %s
and payment is due by the end of %s""" % ( months_list[month_selected],
                                                  months_list[end],
                                                  months_list[due])
        return text


    @writecheck
    @button_wrap
    def apply_button_pressed(self, event):
        "Apply button - check and apply all fields"

        # VAT Method
        if self.vat_method_field.GetSelection() == 0:
            self.method="STANDARD"
        elif self.vat_method_field.GetSelection() == 1:
            self.method="CASH"
        elif self.vat_method_field.GetSelection() == 2:
            self.method="FLAT-BASIC"
        elif self.vat_method_field.GetSelection() == 3:
            self.method="FLAT-CASH"
        elif self.vat_method_field.GetSelection() == 4:
            self.method="NOT_REGISTERED"
        else:
            vs_message(self, "The VAT method has not been recognised", "ERROR")
            return

        # Note: The vat registration number and country code are held in the Company object
        # rather than the vatscheme object, this is because invoices etc.. all point to
        # the current company object, and hence to the current vat registration number

        vrtext=self.vat_regnumber_field.GetValue()
        vrtext=str(vrtext).strip()

        vatccode = self.vat_country_code_field.GetValue()
        vatccode=str(vatccode).strip()

        if self.method != "NOT_REGISTERED":
            # VAT Registration number

            if len(vrtext) < 5 or len(vrtext) > 20:
                vs_message(self, "The VAT Registration number is incorrect\nNo items have been changed.", "ERROR")
                return

            # VAT country code
            if len(vatccode) != 2:
                vs_message(self, "The VAT country code is incorrect\nNo items have been changed.", "ERROR")
                return

        percent = "10.0"
        if self.method == "FLAT-BASIC" or self.method == "FLAT-CASH":
            # Get percentage value
            percent = self.flat_percent_field.GetValue()
            percent = str(percent).strip()
            percent = percent.replace(vs_cfg["CP"], ".")
            # test if it can be converted into a floating point number
            try:
                test = float(percent)
            except Exception, e:
                vs_message(self, "Percentage value is incorrect\nNo items have been changed.", "ERROR")
                return
            if test < 0.0 or test > 99.9:
                vs_message(self, "Percentage value is incorrect\nNo items have been changed.", "ERROR")
                return


        # Start Date, with day 1 as this always starts on the 1st day of a month
        self.start_date = wxdate_to_date(self.start_date_field.GetValue()).replace(day=1)

        # Check if there is no change other than the vat registration number - in which case change that separately
        if (self.company.vat_regnumber != vrtext
                and self.start_date == self.vat_scheme.start_date
                and self.method == self.vat_scheme.method
                and D(percent) == self.vat_scheme.flat_percent):
            # So just change the company
            self.company.vat_regnumber = vrtext
            wx.Yield()
            big_refresh()
            vs_message(self, "The VAT registration number has been changed", "DONE")
            self.EndModal(wx.ID_OK)
            return

        if (self.company.vat_regnumber == vrtext
                and self.start_date == self.vat_scheme.start_date
                and self.method == self.vat_scheme.method
                and D(percent) == self.vat_scheme.flat_percent):
            # No changes
            vs_message(self, "There has been no scheme changes", "DONE")
            self.EndModal(wx.ID_OK)
            return

        # Get scheme confirmation
        text = self.get_confirmation_text(self.method, self.start_date)
        if not vs_yesno(self, text, "Confirm VAT Scheme"):
            vs_message(self, "The Scheme has not been changed", "CANCELLED")
            return

        
        # Must now save the scheme to the database
        busy_cursor_on()
        wx.Yield()
        # close any open frames to ensure no database access
        # is taking place
        close_frames()
        create_objects.create_VATScheme( vat_regnumber = vrtext,
                                        vat_country_code = vatccode,
                                            start_date = self.start_date,
                                                method = self.method,
                                                period = "QUARTERLY",
                                                percent = percent )
        wx.Yield()
        big_refresh()
        vs_message(self, "Scheme implemented", "DONE")
        self.EndModal(wx.ID_OK)


    def cancel_button_pressed(self, event):
        """Cancel button - Close parent"""
        vs_message(self, "No changes have been made.", "INFORMATION")
        self.EndModal(wx.ID_CANCEL)






