####### VATStuff #######
#
# vat_correction.py  - Creates or lists vat correction payments
#
# Version : 0.5
# Date : 20100604
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#


"""Creates or lists vat correction payments."""

import sys
import decimal

import wx
import wx.grid







from vatstuff_packages.vs_database import change_transaction_date
from vatstuff_packages.vs_database import save_vatcorrection

from vatstuff_packages.vs_utilities import *

from vatstuff_packages.vs_gui.gui_functions import *

D = decimal.Decimal   # D used to avoid typing



def vat_correction_dialog(parent):
    """Creates a VATCorrectionDialog"""
    vat_correction_dialog = VATCorrectionDialog(parent)

    # If the cursor is busy, unbusy it - as
    # now waiting for a button press
    busy_cursor_off()

    vat_correction_dialog.ShowModal()
    vat_correction_dialog.Destroy()


class VATCorrectionDialog(vsDialog):
    "Called to display VAT correction options"

    def __init__(self, parent):
        """Create the Dialog"""
        vsDialog.__init__(self, parent, "VAT Correction Options")
        
        # Take a note of the variables
        self.parent = parent

        button_size = (200, vs_cfg["BUTTON_HEIGHT"])
        field_size  = (250, vs_cfg["BUTTON_HEIGHT"])

        # This VAT correction will be saved with a long description
        self.long_description = "Type the reason for this VAT correction"

        # Get a list of accounts - Need accounts with system numbers 0 and 1
        # get_accounts_by_system_list(system_list) - in odd_functions module         
        self.all_accounts = Account.get_non_system()

        # self.account_list is going to show the strings displayed in the accounts choices field
        self.account_list = [ a.fullname for a in self.all_accounts ]

        if not self.account_list:
            raise VS_ERROR, "No accounts are available"

        # Create a vertical sizer to hold everything
        sizer=wx.BoxSizer(wx.VERTICAL)

        help_text="Choose an option below to create a VAT correction payment"
        
        sizer.Add(wx.StaticText(self, -1, help_text), 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # Create buttons and descriptions in a grid sizer
        b_sizer=wx.FlexGridSizer(rows=5, cols=2, hgap=20, vgap=20)

        # date
        self.date_field = wx.DatePickerCtrl(self, -1,
                                            wx.DateTime.Now(), 
                                            size=field_size,
                                            style=wx.DP_DROPDOWN | wx.DP_SHOWCENTURY)
        self.date_field.SetToolTipString("The date of the VAT correction payment")
        b_sizer.Add(self.date_field, 0, wx.ALIGN_RIGHT)
        date_text=wx.StaticText(self, -1, "Date of payment")
        b_sizer.Add(date_text, 0, wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        # cash or bank account
        self.payment_account_field = wx.Choice(self, -1, size=field_size, choices=self.account_list)
        self.payment_account_field.SetToolTipString("Set a bank or cash account")
        b_sizer.Add(self.payment_account_field, 0, wx.ALIGN_RIGHT)

        # Get the last cash account used - and pre-select it
        last_account = get_last_cash_account()
        if last_account != None:
            for key, acc in enumerate(self.all_accounts):
                if last_account == acc:
                    self.payment_account_field.SetSelection(key)
                    break

        account_text=wx.StaticText(self, -1, "Cash or Bank account to be credited/debited")
        b_sizer.Add(account_text, 0, wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        # payment amount
        zero = vs_cfg["CS"] + vs_cfg["ZERO_STRING"]
        self.payment_field = wx.TextCtrl(self, -1, zero, style=wx.ALIGN_RIGHT, size=button_size)
        self.payment_field.SetToolTipString("The amount to be sent or received")
        self.payment_field.SetMaxLength(12)
        b_sizer.Add(self.payment_field, 0, wx.ALIGN_RIGHT)

        payment_text=wx.StaticText(self, -1, "VAT correction amount")
        b_sizer.Add(payment_text, 0, wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        # send payment
        self.send_button = wx.Button(self, label="Send Payment", size=button_size)
        self.send_button.SetToolTipString("Press to record a payment made to HMRC")   
        b_sizer.Add(self.send_button, 0, wx.ALIGN_RIGHT)
        send_text=wx.StaticText(self, -1, "Corrective payment sent to HMRC")
        b_sizer.Add(send_text, 0, wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        # receive payment
        self.receive_button = wx.Button(self, label="Receive Payment", size=button_size)
        self.receive_button.SetToolTipString("Press to record a payment received from HMRC")   
        b_sizer.Add(self.receive_button, 0, wx.ALIGN_RIGHT)
        receive_text=wx.StaticText(self, -1, "Corrective payment received from HMRC")
        b_sizer.Add(receive_text, 0, wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        sizer.Add(b_sizer, 0, wx.ALIGN_CENTER)

        help_text2="""To view previous VAT correction payments, double click on any shown below:"""
        
        sizer.Add(wx.StaticText(self, -1, help_text2), 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # ---------------------- Create the table showing previous corrections

        self.corrections_table = CorrectionsTable(self)
        sizer.Add(self.corrections_table, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # ---------------------- Close -- button

        # Close button
        self.close_button = wx.Button(self, wx.ID_CLOSE, size=button_size)
        self.close_button.SetToolTipString("Click to close.")
        sizer.Add(self.close_button, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # Set the size to the panel
        self.SetSizer(sizer)
        sizer.SetSizeHints(self)

        # ---------- Bind buttons to handler functions
        
        self.Bind(wx.EVT_BUTTON, self.send_button_pressed, self.send_button)
        self.Bind(wx.EVT_BUTTON, self.receive_button_pressed, self.receive_button)
        # Bind left mouse double - click
        self.Bind(wx.grid.EVT_GRID_CELL_LEFT_DCLICK, self.show_correction, self.corrections_table)
        self.Bind(wx.EVT_BUTTON, self.close_button_pressed, self.close_button)


    # ----------------- Handler functions

    @writecheck
    @button_wrap
    def send_button_pressed(self, event):
        "Send button pressed"

        # Create transaction_text, transaction_list, transaction_date
        # to send to the ConfirmVATCorrectionDialog

        # Get date as a python date object
        try:
            self.transaction_date = wxdate_to_date(self.date_field.GetValue())
        except Exception:
            vs_message(self, "The date is invalid", "ERROR")
            return

        self.transaction_text = "VAT correction sent to HMRC"

        # self.transaction_list is a list of lists [account, debit, credit]
        # account being an account object, debit and credit being decimal objects

        # The cash or bank account used to record this purchase

        account_selected_list_item = self.payment_account_field.GetCurrentSelection()
        if account_selected_list_item < 0:         # blank or unselected
            vs_message(self, "An account was not found", "ERROR")
            return
        else:
            payment_account = self.all_accounts[account_selected_list_item]

        # Get amount paid field
        amount_string = self.payment_field.GetValue().strip()
        status, amount = string_to_decimal(amount_string)
        if status == False:
            vs_message(self, "The amount to be paid is not valid", "ERROR")
            return
        if amount <= D(0):
            vs_message(self, "The amount to be paid is not valid", "ERROR")
            return

        #      The cash account          Debit by zero          Credit by given amount
        t1 = [ payment_account,          D("0.00"),             amount ]

        #      The vat account           Debit by given amount  Credit by zero
        t2 = [ get_account_by_system(5), amount,                D("0.00") ]

        self.transaction_list = [t1, t2]

        # Confirm the transaction 
        wx.Yield()
        status, self.transaction_text, self.long_description = confirm_VAT_correction(self,
                                                                        self.long_description,
                                                                        self.transaction_text,
                                                                        self.transaction_list,
                                                                        self.transaction_date)
        if not status:
            vs_message(self, "The transaction has not been saved", "CANCELLED")
            return

        # Must now implement the transaction and save the VAT correction
        busy_cursor_on()
        wx.Yield()
        save_vatcorrection.save(self.long_description,
                                amount,
                                self.transaction_date,
                                self.transaction_text,
                                self.transaction_list)

        big_refresh(panels=False)   # panels False as the tab panels do not need refreshing
        self.corrections_table.refresh()
        info_log("VAT Correction saved : %s" % self.transaction_text)
        vs_message(self, "VAT correction saved", "DONE")


    @writecheck
    @button_wrap
    def receive_button_pressed(self, event):
        "Receive button pressed"

        # Create transaction_text, transaction_list, transaction_date
        # to send to the ConfirmVATCorrectionDialog

        # Get date as a python date object
        try:
            self.transaction_date = wxdate_to_date(self.date_field.GetValue())
        except Exception:
            vs_message(self, "The date is invalid", "ERROR")
            return

        self.transaction_text = "VAT correction received from HMRC"

        # self.transaction_list is a list of lists [account, debit, credit]
        # account being an account object, debit and credit being decimal objects

        # The cash or bank account used to record this purchase

        account_selected_list_item = self.payment_account_field.GetCurrentSelection()
        if account_selected_list_item < 0:         # blank or unselected
            vs_message(self, "An account was not found", "ERROR")
            return
        else:
            payment_account = self.all_accounts[account_selected_list_item]

        # Get amount paid field
        amount_string = self.payment_field.GetValue().strip()
        status, amount = string_to_decimal(amount_string)
        if status == False:
            vs_message(self, "The amount received is not valid", "ERROR")
            return
        if amount <= D(0):
            vs_message(self, "The amount received is not valid", "ERROR")
            return

        #      The cash account          Debit by amount     Credit by zero
        t1 = [ payment_account,          amount,             D("0.00") ]

        #      The vat account           Debit by zero       Credit by amount
        t2 = [ get_account_by_system(5), D("0.00"),          amount   ]

        self.transaction_list = [t1, t2]

        wx.Yield()
        # Confirm the transaction 
        status, self.transaction_text, self.long_description = confirm_VAT_correction(self,
                                                                        self.long_description,
                                                                        self.transaction_text,
                                                                        self.transaction_list,
                                                                        self.transaction_date)
        if not status:
            vs_message(self, "The transaction has not been saved", "CANCELLED")
            return

        # Must now implement the transaction and save the VAT correction

        amount = D("-1") * amount

        busy_cursor_on()
        wx.Yield()
        save_vatcorrection.save(self.long_description,
                                amount,
                                self.transaction_date,
                                self.transaction_text,
                                self.transaction_list)

        big_refresh(panels=False)   # panels False as the tab panels do not need refreshing
        self.corrections_table.refresh()
        info_log("VAT Correction saved : %s" % self.transaction_text)
        vs_message(self, "VAT correction saved", "DONE")


    @button_wrap
    def show_correction(self, event):
        "On being double clicked, obtain transaction and show it in a ShowTransactionDialog"
        row = event.GetRow()
        # get the transaction for this row
        transaction = Transaction(int(self.corrections_table.GetRowLabelValue(row)))
        show_transaction_dialog(self, transaction)
        # refresh the table - as description may have changed
        self.corrections_table.refresh()


    def close_button_pressed(self, event):
        """Close the dialog"""
        self.EndModal(wx.ID_OK)


class CorrectionsTable(wx.grid.Grid):

    def __init__(self, parent):
        wx.grid.Grid.__init__(self, parent, size=(700,200), style=wx.WANTS_CHARS)

        all_vat_corrections = VATCorrection.get_all()

        self.num_rows = len(all_vat_corrections)
        num_cols = 3

        self.CreateGrid(self.num_rows, num_cols)
        self.EnableEditing(False)

        self.SetDefaultRowSize(vs_cfg["BUTTON_HEIGHT"], False)
        self.SetDefaultCellAlignment(wx.ALIGN_LEFT, wx.ALIGN_CENTER)

        # create column labels
        self.SetColLabelValue(0, "Date")
        self.SetColLabelValue(1, "Description")
        self.SetColLabelValue(2, "Amount")


        # Set the column showing money as right aligned
        attr = wx.grid.GridCellAttr()
        attr.SetAlignment(wx.ALIGN_RIGHT, wx.ALIGN_CENTER)

        self.SetColAttr(2, attr)

        for row, vat_correction in enumerate(all_vat_corrections):
            transaction = vat_correction.transaction
            # Set row label as the transaction id
            self.SetRowLabelValue(row, str(transaction.id))
            # Date
            self.SetCellValue(row, 0, transaction.datestamp.strftime("%d %b %Y"))
            # Transaction description
            self.SetCellValue(row, 1, transaction.description)
            # correction amount
            self.SetCellValue(row, 2, to_currency_string(vat_correction.amount))

        self.SetColSize(0, 100)
        self.SetColSize(1, 350)
        self.SetColSize(2, 100)

        self.SetColMinimalAcceptableWidth(100)
        self.SetRowLabelSize(100)


    def refresh(self):
        "Refreshes the table"
        all_vat_corrections = VATCorrection.get_all()
        num_rows = len(all_vat_corrections)
        if num_rows > self.num_rows:
            self.AppendRows(numRows = num_rows-self.num_rows)
            self.num_rows = num_rows

        for row, vat_correction in enumerate(all_vat_corrections):
            transaction = vat_correction.transaction
            # Set row label as the transaction id
            self.SetRowLabelValue(row, str(transaction.id))
            # Date
            self.SetCellValue(row, 0, transaction.datestamp.strftime("%d %b %Y"))
            # Transaction description
            self.SetCellValue(row, 1, transaction.description)
            # correction amount
            self.SetCellValue(row, 2, to_currency_string(vat_correction.amount))




# The ConfirmVATCorrectionDialog is used to confirm a VAT correction
# payment and to allow a user to set the descriptive text
# It is normally used by calling the function confirm_VAT_correction

def confirm_VAT_correction(parent, long_description, transaction_text, transaction_list, transaction_date):
    """Creates a ConfirmVATCorrectionDialog, returns (True, new text, new long_description) on confirmation
       or (False, text, new long_description) otherwise"""

    if not transaction_list: return False, transaction_text, long_description

    confirm_VAT_correction_dialog = ConfirmVATCorrectionDialog(parent,
                                                          long_description,
                                                          transaction_text,
                                                          transaction_list,
                                                          transaction_date)
    # If the cursor is busy, unbusy it - as
    # now waiting for a button press
    busy_cursor_off()

    if confirm_VAT_correction_dialog.ShowModal() == wx.ID_OK:
        new_transaction_text = confirm_VAT_correction_dialog.transaction_text
        new_long_description = confirm_VAT_correction_dialog.long_description
        confirm_VAT_correction_dialog.Destroy()
        debug_log("VAT correction confirmed : %s" % new_transaction_text)
        return True, new_transaction_text, new_long_description
    else:
        new_long_description = confirm_VAT_correction_dialog.long_description
        confirm_VAT_correction_dialog.Destroy()
        debug_log("Transaction abandonned")
        return False, transaction_text, new_long_description


class ConfirmVATCorrectionDialog(vsDialog):
    """Called when a VAT correction is to be confirmed, and descriptions input"""

    def __init__(self, parent, long_description, transaction_text, transaction_list, transaction_date):
        """Create the Dialog"""
        vsDialog.__init__(self, parent, "Confirm VAT correction")

        # Take a note of the variables
        self.parent = parent
        self.transaction_text = transaction_text   # The memo text to put on the transaction
        self.transaction_list = transaction_list   # List of lists [account, debit, credit]
        self.transaction_date = transaction_date   # A Python date object
        self.long_description = long_description   # A longer descriptive string
        
        if not self.transaction_text:
            self.transaction_text = "VAT correction"

        if not self.long_description:
            self.long_description = "Type reason for VAT correction"

        # Create a vertical sizer to hold everything
        sizer=wx.BoxSizer(wx.VERTICAL)

        # Create memo and date in a grid sizer
        memo_sizer=wx.FlexGridSizer(rows=3, cols=2, hgap=20, vgap=10)

        # memo
        memo_label=wx.StaticText(self, -1, "Short description")
        memo_sizer.Add(memo_label, 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.memo_field = wx.TextCtrl(self, -1, transaction_text, size=(400, vs_cfg["BUTTON_HEIGHT"]))
        self.memo_field.SetToolTipString("The memo to be shown in the account records")
        self.memo_field.SetMaxLength(50)
        memo_sizer.Add(self.memo_field, 0, wx.ALIGN_LEFT)

        # long description
        long_label=wx.StaticText(self, -1, "Long description")
        memo_sizer.Add(long_label, 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        self.long_field = wx.TextCtrl(self, -1, long_description, size=(400, 4*vs_cfg["BUTTON_HEIGHT"]), style=wx.TE_MULTILINE)
        self.long_field.SetToolTipString("Long description")
        self.long_field.SetMaxLength(200)
        memo_sizer.Add(self.long_field, 0, wx.ALIGN_LEFT)

        # Transaction date
        date_label=wx.StaticText(self, -1, "Transaction Date")
        memo_sizer.Add(date_label, 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)
        date_text = wx.StaticText(self, -1, transaction_date.strftime("%d %b %Y"))
        memo_sizer.Add(date_text, 0, wx.ALIGN_LEFT)

        # add the memo_sizer to the main vertical sizer
        sizer.Add(memo_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        help_text=wx.StaticText(self, -1, """These are the accounts, and the amounts, to be debited
and credited, press Apply to implement the transaction.""")

        sizer.Add(help_text, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # -------------------- The transactions

        number = len(transaction_list)

        # Create transactions in a grid sizer
        t_sizer=wx.FlexGridSizer(rows=number+1, cols=4, hgap=40, vgap=10)
        # number+1 ; the +1 is for the header text titles

        account_text=wx.StaticText(self, -1, "Account")
        t_sizer.Add(account_text, 0, wx.ALIGN_CENTER)

        debit_text=wx.StaticText(self, -1, "Debit")
        t_sizer.Add(debit_text, 0, wx.ALIGN_CENTER)

        credit_text=wx.StaticText(self, -1, "Credit")
        t_sizer.Add(credit_text, 0, wx.ALIGN_CENTER)

        balance_text=wx.StaticText(self, -1, "Balance")
        t_sizer.Add(balance_text, 0, wx.ALIGN_CENTER)

        for index, tran in enumerate(transaction_list):
            
            # The Account
            t_sizer.Add(wx.StaticText(self, -1, tran[0].fullname), 0, wx.ALIGN_LEFT)

            # The Debit field
            t_sizer.Add(wx.StaticText(self, -1, to_currency_string(tran[1])), 0, wx.ALIGN_RIGHT)

            # The Credit field
            t_sizer.Add(wx.StaticText(self, -1, to_currency_string(tran[2])), 0, wx.ALIGN_RIGHT)

            # The New balance field
            new_balance = tran[0].balance
            # Go through each previouse item in transaction_list in case the account is in multiple times
            for i in range(index+1):
                if transaction_list[i][0] == tran[0]:
                    new_balance = new_balance + transaction_list[i][1] - transaction_list[i][2]
            t_sizer.Add(wx.StaticText(self, -1, to_currency_string(new_balance)), 0, wx.ALIGN_RIGHT)

        # add the t_sizer to the main vertical sizer
        sizer.Add(t_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # ---------------------- Apply - Cancel -- buttons

        button_size = (100, vs_cfg["BUTTON_HEIGHT"])

        # Create a horizontal button_sizer to hold the buttons
        button_sizer=wx.BoxSizer(wx.HORIZONTAL)

        # Apply button
        self.apply_button = wx.Button(self, wx.ID_APPLY, size=button_size)
        self.apply_button.SetToolTipString("Click to implement the transaction.")
        button_sizer.Add(self.apply_button, 0, wx.RIGHT, 40)

        # Cancel button
        self.cancel_button = wx.Button(self, wx.ID_CANCEL, size=button_size)
        self.cancel_button.SetToolTipString("Click to cancel.")
        button_sizer.Add(self.cancel_button, 0)

        # Add button_sizer to the main sizer
        sizer.Add(button_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # Set the size to the dialog frame
        self.SetSizer(sizer)
        sizer.SetSizeHints(self)

        # ---------- Bind buttons to handler functions
        
        self.Bind(wx.EVT_BUTTON, self.apply_button_pressed, self.apply_button)
        self.Bind(wx.EVT_BUTTON, self.cancel_button_pressed, self.cancel_button)


    # ----------------- Handler functions

    @writecheck
    @logmethod    # log - not dialog_apply, as don't need busy icon for this
    def apply_button_pressed(self, event):
        "Apply button - set the text and long description"
        # Get new transaction text
        try:
            self.transaction_text = self.memo_field.GetValue().strip()
            self.long_description = self.long_field.GetValue().strip()
            self.EndModal(wx.ID_OK)
        except Exception, e:
            error_message("Failed to read the dialog fields",e)


    def cancel_button_pressed(self, event):
        "Cancel button - return wx.ID_CANCEL"
        self.long_description = self.long_field.GetValue().strip()
        self.EndModal(wx.ID_CANCEL)

