####### VATStuff #######
#
# vt_100.py  - generate a VAT return
#
# Version : 0.7.0
# Date : 20100920
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#

"""Generate a detailed VAT report

It is run as part of the vatstuff gui interface.
"""


import datetime
import calendar
import decimal

import wx
import wx.html





from vatstuff_packages.vs_database.transactions import *
from vatstuff_packages.vs_database import save_vatreturn
from vatstuff_packages.vs_database import vat_standard
from vatstuff_packages.vs_database import vat_cash
from vatstuff_packages.vs_database import vat_flat_basic
from vatstuff_packages.vs_database import vat_flat_cash

from vatstuff_packages.vs_gui.gui_functions import *

from vatstuff_packages.vs_utilities import *
from vatstuff_packages.vs_verify.form_objects import VATReturnForm, ReportForm

if vs_cfg["REPORTLABS_AVAILABLE"]:
    import vatstuff_packages.vs_pdf.page_setup
    import vatstuff_packages.vs_pdf.create_report

D = decimal.Decimal   # D used to avoid typing


def vat_return_frame(from_date, to_date):
    """Show the VATReturnFrame which lists the VAT Return,
       between the two dates"""

    # No try:exception statement - allows the caller to catch exceptions

    # Check the frame is not already open
    for frm in vs_cfg["TRANSACTION_REFRESH"]:
        if isinstance(frm, VATReturnFrame):
            frm.Raise()
            break
    else:
        # top_frame() is the top frame, which will be the parent
        frm = VATReturnFrame(top_frame(), from_date, to_date)
        frm.Show()


class VATReturnFrame(vsFrame):
    "Called to display the frame to show a VAT Return"

    def __init__(self, parent, from_date, to_date):
        """Create the frame"""
        vsFrame.__init__(self, parent, title="VAT Return")

        self.vat_return_panel=VATReturnPanel(self, from_date, to_date)

        # Get the panel size
        panel_size = self.vat_return_panel.GetMinSize()

        # Set minimum size equal to the panel
        self.SetMinSize(panel_size)
        self.SetClientSize(panel_size)

        # update the list of windows which need a refresh if
        # a transaction occurs, by adding self
        vs_cfg["TRANSACTION_REFRESH"].append(self)

        # bind the close event
        self.Bind(wx.EVT_CLOSE, self.onclose)


    def refresh(self):
        "Calls the refresh in the panel"
        self.vat_return_panel.refresh()


    def onclose(self, evt):
        "Shutdown the frame"
        # Decrement the refresh list
        vs_cfg["TRANSACTION_REFRESH"].remove(self)
        self.Destroy()


# ---------------------- The vat return panel

class VATReturnPanel(wx.Panel):
    """The panel of the VATReturn frame"""

    def __init__(self, parent, from_date, to_date):
        """Create the panel"""
        wx.Panel.__init__(self, parent)
        
        # Take a note of the variables
        self.parent = parent

        # This is a flad, set to True while submitting, which informs the refresh
        # function, that it is being called by a vat return submit, rather than by
        # another database change
        self.submitting = False

        button_size = (100, vs_cfg["BUTTON_HEIGHT"])

        self.sizer=wx.BoxSizer(wx.VERTICAL)

        self.reportwindow = wx.html.HtmlWindow(self)
        if "gtk2" in wx.PlatformInfo:
            self.reportwindow.SetStandardFonts()

        # Store all return form contents in a VATReturnForm object
        self.vat_return = VATReturnForm()
        self.vat_return.from_date = from_date
        self.vat_return.to_date = to_date

        # Create the report and fill in the vat return box_one to box_nine attributes
        self.reportwindow.SetPage(self.create_report(from_date, to_date))

        # This widget needs to expand in all directions
        self.sizer.Add(self.reportwindow, 1, wx.ALL | wx.ALIGN_CENTER | wx.EXPAND, 20)

        help_text = "Press Submit to save this return, and create the payment transaction"
        self.sizer.Add(wx.StaticText(self, -1, help_text), 0, wx.ALL | wx.ALIGN_CENTER, 10)


        # buttons

        button_sizer=wx.BoxSizer(wx.HORIZONTAL)

        # page setup button
        self.page_button = wx.Button(self, label="Page setup", size=button_size)
        self.page_button.SetToolTipString("Click to set up the pdf page size and orientation")   
        button_sizer.Add(self.page_button, 0, wx.RIGHT, 20)

        # pdf button
        self.pdf_button = wx.Button(self, label="Create pdf", size=button_size)
        self.pdf_button.SetToolTipString("Click to create a pdf file of the order form.")
        button_sizer.Add(self.pdf_button, 0, wx.RIGHT, 20)

        # Close button
        self.close_button = wx.Button(self, wx.ID_CLOSE, size=button_size)
        self.close_button.SetToolTipString("Click to close.")
        button_sizer.Add(self.close_button, 0, wx.RIGHT, 20)

        # Submit button
        self.submit_button = wx.Button(self, label="Submit", size=button_size)
        self.submit_button.SetToolTipString("Click to Submit the return and pay your VAT")
        button_sizer.Add(self.submit_button, 0, wx.RIGHT, 20)

        self.sizer.Add(button_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)
      
        # Set the size to the panel
        self.SetSizer(self.sizer)

        # Set panel minimum size 
        self.SetSizeHints(800, 600)


        # Only allow submit, if this VAT return start date is equal or after the start of the current period
        vat_scheme = VATScheme()
        if self.vat_return.from_date < vat_scheme.start_date:
            self.submit_button.Enable(False)



        # ---------- Bind buttons to handler functions
        
        self.Bind(wx.EVT_BUTTON, self.page_button_pressed, self.page_button)
        self.Bind(wx.EVT_BUTTON, self.pdf_button_pressed, self.pdf_button)
        self.Bind(wx.EVT_BUTTON, self.close_button_pressed, self.close_button)
        self.Bind(wx.EVT_BUTTON, self.submit_button_pressed, self.submit_button)


    @button_wrap
    def page_button_pressed(self, event):
        "page button pressed, setup the pdf page size and orientation"
        vatstuff_packages.vs_pdf.page_setup.page_setup_dialog(self)


    @button_wrap
    def pdf_button_pressed(self, event):
        "Create a pdf file from the report"
        if not vs_cfg["REPORTLABS_AVAILABLE"]:
            # no pdf capability
            vs_message(self, "Function unavailable - the reportlab module is not installed",
                             "ERROR")
            return

        # default filename
        filename= "VAT_Return%s.pdf" % self.vat_return.to_date.isoformat()
        if not vatstuff_packages.vs_pdf.create_report.create_pdf(self, filename, self.report):
            vs_message(self, "Unable to create the pdf file", "ERROR")


    def close_button_pressed(self, event):
        """ Close button - Close parent frame"""
        self.parent.Close()
        return


    # Method to create the report
    def create_report(self, from_date, to_date):
        "Create the vat_return in self.vat_return, the report in self.report, and returns the html text"

        self.report = ReportForm("Value Added Tax Return")

        self.report.header = ["For the period:",
                              "From: %s" % from_date.strftime("%d %b %Y"),
                              "To: %s" % to_date.strftime("%d %b %Y"),
                              ""]

        vat_scheme = VATScheme()
        vat_method = vat_scheme.method

        vatscheme_text = ""
        if vat_method == "STANDARD":
            vatscheme_text = "VAT Scheme : Standard"
        elif vat_method == "CASH":
            vatscheme_text = "VAT Scheme : Cash"
        elif vat_method == "FLAT-BASIC":
            vatscheme_text = """\
VAT Scheme : Flat (Basic) rate at %s%%""" % str(vat_scheme.flat_percent).replace(".", vs_cfg["CP"])
        elif vat_method == "FLAT-CASH":
            vatscheme_text = """\
VAT Scheme : Flat (Cash) rate at %s%%""" % str(vat_scheme.flat_percent).replace(".", vs_cfg["CP"])
        if vatscheme_text:
            self.report.header.append(vatscheme_text)

        # tables are described by a list of list,
        # each list being a row, consisting of cell tuple
        # each cell tuple containing two strings; 
        # The first string is formed from "XXXN",
        # and the second string is the text to be placed in the cell

        # N is the collspan number, as used in a table cell tag attribute
        # such as 'colspan="4"' - in which case N is 4

        # so normally N is '1'

        # XXX is a formatting string :
        # HBL  - Header Bold Left Justified (R for Right instead of L
        # HNL  - Header Normal Left Justified  etc
        # DNL  - Data Cell Normal Left Justified  etc
        # XXC  - Right Justified currency, as pulled from the database

        # BOX 1
        if vat_method == "STANDARD":
            # Function get_box_1 is in vatstuff_packages.vs_database.vat_standard and returns the
            # VAT due on all sales invoices minus VAT on all creditnotes
            output_VAT = vat_standard.get_box_1(from_date, to_date)
        elif vat_method == "CASH":
            # Function get_box_1 is in vatstuff_packages.vs_database.vat_cash and returns the
            # VAT received on all sales payments received minus VAT on all creditnotes payments sent
            output_VAT = vat_cash.get_box_1(from_date, to_date)
        elif vat_method == "FLAT-BASIC":
            # Function get_box_1 is in vatstuff_packages.vs_database.vat_flat_basic and returns the
            output_VAT = vat_flat_basic.get_box_1(from_date, to_date)
        elif vat_method == "FLAT-CASH":
            # Function get_box_1 is in vatstuff_packages.vs_database.vat_flat_cash and returns the
            output_VAT = vat_flat_cash.get_box_1(from_date, to_date)
        self.vat_return.box_one = output_VAT
        self.report.table = [[ ('DNL1', '1'),
                               ('DNL1', 'VAT due in this period on sales and other outputs'),
                               ('DNC1', str(output_VAT))   ]]

        # BOX 2
        if vat_method == "STANDARD":
            EC_purchases_VAT = vat_standard.get_box_2(from_date, to_date)
        elif vat_method == "CASH":
            EC_purchases_VAT = vat_cash.get_box_2(from_date, to_date)
        elif vat_method == "FLAT-BASIC":
            EC_purchases_VAT = vat_flat_basic.get_box_2(from_date, to_date)
        elif vat_method == "FLAT-CASH":
            EC_purchases_VAT = vat_flat_cash.get_box_2(from_date, to_date)
        self.vat_return.box_two = EC_purchases_VAT
        self.report.add_row_to_table( ('DNL1', '2'),
                                      ('DNL1', """\
VAT due in this period on acquisitions from other
EC member States"""),
                                      ('DNC1', str(EC_purchases_VAT)) )

        # BOX 3
        total_vat = output_VAT + EC_purchases_VAT
        self.vat_return.box_three = total_vat
        self.report.add_row_to_table( ('DNL1', '3'),
                                      ('DNL1', "Total VAT due (the sum of boxes 1 and 2)"),
                                      ('DNC1', str(total_vat)) )

        # BOX 4
        if vat_method == "STANDARD":
            # Function get_box_4 is in vatstuff_packages.vs_database.vat_standard and returns the
            # VAT due on all supplier invoices minus VAT on all supplier creditnotes
            input_VAT = vat_standard.get_box_4(from_date, to_date)
        elif vat_method == "CASH":
            # Function get_box_4 is in vatstuff_packages.vs_database.vat_cash and returns the
            # VAT paid to suppliers minus VAT received with supplier creditnotes
            input_VAT = vat_cash.get_box_4(from_date, to_date)
        elif vat_method == "FLAT-BASIC":
            # Function get_box_4 is in vatstuff_packages.vs_database.vat_flat_basic and returns the
            input_VAT = vat_flat_basic.get_box_4(from_date, to_date)
        elif vat_method == "FLAT-CASH":
            # Function get_box_4 is in vatstuff_packages.vs_database.vat_flat_cash and returns the
            input_VAT = vat_flat_cash.get_box_4(from_date, to_date)
        self.vat_return.box_four = input_VAT
        self.report.add_row_to_table( ('DNL1', '4'),
                                      ('DNL1', """\
VAT reclaimed in this period on purchase and other inputs
(including acquisitions from the EC)"""),
                                      ('DNC1', str(input_VAT)) )

        # BOX 5
        net_VAT = total_vat - input_VAT
        self.vat_return.box_five = net_VAT
        self.report.add_row_to_table( ('DNL1', '5'),
                                      ('DNL1', """\
Net VAT to be paid to HMRC or reclaimed by you
(Difference between boxes 3 and 4)"""),
                                      ('DNC1', str(net_VAT)) )

        # BOX 6
        if vat_method == "STANDARD":
            # Function get_box_6 is in vatstuff_packages.vs_database.vat_standard and returns the
            # cost due on all invoices minus cost on all creditnotes
            output_cost = vat_standard.get_box_6(from_date, to_date)
        elif vat_method == "CASH":
            output_cost = vat_cash.get_box_6(from_date, to_date)
        elif vat_method == "FLAT-BASIC":
            output_cost = vat_flat_basic.get_box_6(from_date, to_date)
        elif vat_method == "FLAT-CASH":
            output_cost = vat_flat_cash.get_box_6(from_date, to_date)
        self.vat_return.box_six = output_cost
        self.report.add_row_to_table( ('DNL1', '6'),
                                      ('DNL1', """\
Total value of sales and all other outputs excluding
any VAT. Include your box 8 figure"""),
                                      ('DNC1', str(output_cost)) )

        # BOX 7
        if vat_method == "STANDARD":
            # Function get_box_7 is in vatstuff_packages.vs_database.vat_standard and returns the
            # cost due on all supplier invoices minus cost on all supplier creditnotes
            input_cost = vat_standard.get_box_7(from_date, to_date)
        elif vat_method == "CASH":
            input_cost = vat_cash.get_box_7(from_date, to_date)
        elif vat_method == "FLAT-BASIC":
            input_cost = vat_flat_basic.get_box_7(from_date, to_date)
        elif vat_method == "FLAT-CASH":
            input_cost = vat_flat_cash.get_box_7(from_date, to_date)
        self.vat_return.box_seven = input_cost
        self.report.add_row_to_table( ('DNL1', '7'),
                                      ('DNL1', """\
Total value of purchases and all other inputs excluding
any VAT. Include your box 9 figure"""),
                                      ('DNC1', str(input_cost)) )

        # BOX 8
        if vat_method == "STANDARD":
            # Function get_box_8 is in vatstuff_packages.vs_database.vat_standard and returns the cost
            # of all invoices to EC member states
            EC_supplies = vat_standard.get_box_8(from_date, to_date)
        elif vat_method == "CASH":
            EC_supplies = vat_cash.get_box_8(from_date, to_date)
        elif vat_method == "FLAT-BASIC":
            EC_supplies = vat_flat_basic.get_box_8(from_date, to_date)
        elif vat_method == "FLAT-CASH":
            EC_supplies = vat_flat_cash.get_box_8(from_date, to_date)
        self.vat_return.box_eight = EC_supplies
        self.report.add_row_to_table( ('DNL1', '8'),
                                      ('DNL1', """\
Total value of all supplies of goods and related costs,
excluding any VAT, to other EC Member States"""),
                                      ('DNC1', str(EC_supplies)) )

        # BOX 9
        if vat_method == "STANDARD":
            # Function get_box_9 is in vatstuff_packages.vs_database.vat_standard and returns the cost
            # of all purchases from EC member states
            EC_acquisitions = vat_standard.get_box_9(from_date, to_date)
        elif vat_method == "CASH":
            EC_acquisitions = vat_cash.get_box_9(from_date, to_date)
        elif vat_method == "FLAT-BASIC":
            EC_acquisitions = vat_flat_basic.get_box_9(from_date, to_date)
        elif vat_method == "FLAT-CASH":
            EC_acquisitions = vat_flat_cash.get_box_9(from_date, to_date)
        self.vat_return.box_nine = EC_acquisitions
        self.report.add_row_to_table( ('DNL1', '9'),
                                      ('DNL1', """\
Total value of all acquisitions of goods and related costs,
excluding any VAT, from other EC Member States"""),
                                      ('DNC1', str(EC_acquisitions)) )

        self.report.footer = []
        # Added text if vat method is flat-basic or flat-cash
        if vat_method == "FLAT-BASIC":
            self.report.footer.extend(
                ["NOTE: Box 1 is calculated as %s%% of total (VAT inclusive) sales" % (
                                 str(vat_scheme.flat_percent).replace(".", vs_cfg["CP"])),
                 ""])
        elif vat_method == "FLAT-CASH":
            self.report.footer.extend(
                ["NOTE: Box 1 is calculated as %s%% of total (VAT inclusive)" % (
                                 str(vat_scheme.flat_percent).replace(".", vs_cfg["CP"])),
                 "payments received from customers",
                 ""])

        # Added text if box 2 has contents
        if EC_purchases_VAT > D("0.00"):
            self.report.footer.extend(
                  ["NOTE: Box 2 is calculated at the standard VAT rate : %s%%," % (
                                   str(vs_cfg["VAT_STANDARD_RATE"]).replace(".", vs_cfg["CP"]),),
                   "of the cost of your purchases from suppliers within the EC.",
                   "(That is, EC suppliers who do not have %s set as their VAT country code.)" % (vs_cfg["VAT_COUNTRY_CODE"],),
                   ""])


        # Added text if box 8 has contents
        if EC_supplies > D("0.00"):
            self.report.footer.extend(
                   ["NOTE: Box 8 is calculated as the sales to customers in EC member states",
                    "who are VAT registered (their countries equivalent of VAT).",
                    ""])

        return self.report.render_to_html()


    @button_wrap
    def submit_button_pressed(self, event):
        "Submit button - save the vat return and do the payment transaction"
        self.submitting = True
        submitted = submit_vat_return_dialog(self, self.vat_return)
        self.submitting = False
        # If the vat return is submitted, disable the submit button
        if submitted:
            self.submit_button.Enable(False)


    def refresh(self):
        """The refresh function disables the submit button - and if the
           refresh is due to any other database change, it disables the report"""
        self.submit_button.Enable(False)
        if self.submitting: return
        # the refresh must be called by some other database access
        self.page_button.Enable(False)
        self.pdf_button.Enable(False)
        self.reportwindow.SetPage(no_report())




# ----------------------SubmitVATReturnDialog

# The SubmitVATReturnDialog is used to get a bank account from which
# the payment is to be made, and a payment date

def submit_vat_return_dialog(parent, vat_return):
    """Creates a SubmitVATReturnDialog"""

    # The submitted flag is set to True if the VAT
    # return has been submitted
    submitted = False
    show_dialog = SubmitVATReturnDialog(parent, vat_return)

    # If the cursor is busy, unbusy it - as
    # now waiting for a button press
    busy_cursor_off()

    if show_dialog.ShowModal() == wx.ID_OK:
        submitted = True

    show_dialog.Destroy()

    return submitted


class SubmitVATReturnDialog(vsDialog):
    """The dialog to get a bank account from which
       the VAT payment is to be made, and a payment date,
       and then to actually create the vat submission transaction"""

    def __init__(self, parent, vat_return):
        """Create the Dialog"""
        vsDialog.__init__(self, parent, "Submit VAT Return")

        # Take a note of the variables
        self.vat_return = vat_return

        button_size = (100, vs_cfg["BUTTON_HEIGHT"])

        # Create a vertical sizer to hold everything
        sizer=wx.BoxSizer(wx.VERTICAL)

        if self.vat_return.box_five >= D("0.00"):
            text = """\
Warning: Once submitted no further sales or
purchases can be made in this VAT period.
Choose the bank or cash account from which
the VAT payment is to be made:"""
        else:
            text = """\
Warning: Once submitted no further sales or
purchases can be made in this VAT period.
You are reclaiming VAT from HMRC.
Choose the bank or cash account into which
the VAT reclaimed is to be deposited:"""

        sizer.Add(wx.StaticText(self, -1, text), 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # get the bank/cash account
        # from which the payment will be made
        self.all_accounts = Account.get_non_system()

        # self.account_list shows the strings displayed in the choices field
        self.account_list = [ a.fullname for a in self.all_accounts ]

        if len(self.account_list) == 0:
            raise VS_ERROR, "No accounts are available"

        self.payment_account_field = wx.Choice(self, -1, size=(200, vs_cfg["BUTTON_HEIGHT"]), choices=self.account_list)

        self.payment_account_field.SetToolTipString("Set a bank or cash account")
        sizer.Add(self.payment_account_field, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # Get the last cash account used - and pre-select it
        last_account = get_last_cash_account()
        if last_account:
            for key, acc in enumerate(self.all_accounts):
                if last_account == acc:
                    self.payment_account_field.SetSelection(key)
                    break

        # date
        sizer.Add(wx.StaticText(self, -1, "And set the date:"), 0, wx.ALL | wx.ALIGN_CENTER, 10)

        self.datestamp = wx.DateTime.Now()
        self.payment_date_field = wx.DatePickerCtrl(self, -1,
                                                    self.datestamp, 
                                                    size=(150, vs_cfg["BUTTON_HEIGHT"]),
                                                    style=wx.DP_DROPDOWN | wx.DP_SHOWCENTURY)
        self.payment_date_field.SetToolTipString("The date of the payment")
        sizer.Add(self.payment_date_field, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # And a horizontal sizer to hold buttons
        button_sizer = wx.BoxSizer(wx.HORIZONTAL)

        # Cancel button
        self.cancel_button = wx.Button(self, wx.ID_CANCEL, size=button_size)
        self.cancel_button.SetToolTipString("Click to cancel.")
        button_sizer.Add(self.cancel_button, 0, wx.RIGHT, 40)

        # Apply button
        self.apply_button = wx.Button(self, wx.ID_APPLY, size=button_size)
        self.apply_button.SetToolTipString("Click to apply payment.")
        button_sizer.Add(self.apply_button, 0)

        sizer.Add(button_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        self.Bind(wx.EVT_BUTTON, self.cancel_button_pressed, self.cancel_button)
        self.Bind(wx.EVT_BUTTON, self.apply_button_pressed, self.apply_button)

        # Set the size
        self.SetSizer(sizer)
        sizer.SetSizeHints(self)


    def cancel_button_pressed(self, event):
        """ Cancel button"""
        self.EndModal(wx.ID_CANCEL)


    @writecheck
    @button_wrap
    def apply_button_pressed(self, event):
        "Apply button - Create the VAT transaction from the input fields"

        # Get the transaction date
        try:
            transaction_date = wxdate_to_date(self.payment_date_field.GetValue())
        except Exception:
            vs_message(self, "The date is invalid", "ERROR")
            return  

        if transaction_date < self.vat_return.to_date:
            vs_message(self, "The date must be after the end of the VAT period", "ERROR")
            return            

        # The cash or bank account used to record this purchase

        account_selected_list_item = self.payment_account_field.GetCurrentSelection()
        if account_selected_list_item < 0:         # blank or unselected
            vs_message(self, "You must select a bank or cash account", "ERROR")
            return

        self.vat_return.account = self.all_accounts[account_selected_list_item]

        # Create transaction_text, transaction_list, transaction_date
        # to send to the ApproveTransaction Frame for transaction approval and implementation

        # The amount to be moved
        amount = self.vat_return.box_five

        # self.transaction_list is a list of lists [account, debit, credit]
        # account being an account object, debit and credit being decimal objects

        # Either a bank or cash account
        t1 = [ self.vat_return.account ]

        # The VAT balance account:
        t2 = [ get_account_by_system(5) ]

        transaction_text = "VAT Return transaction"

        if amount >= D("0.00"):
            # Debit by zero
            t1.append(D("0.00"))
            # Credit this by the amount
            t1.append(amount)
        else:
            # Debit by by the amount
            t1.append(abs(amount))
            # Credit this by zero
            t1.append(D("0.00"))

        vat_scheme = VATScheme()
        vat_method = vat_scheme.method
        from_date = self.vat_return.from_date
        to_date = self.vat_return.to_date
        net_balance = D(0)

        if vat_method == "STANDARD" or vat_method == "FLAT-BASIC":
            # VAT amount on all sales (standard output) calculated from all invoices issued
            # less credit notes issued
            output_VAT = vat_standard.standard_output(from_date, to_date) + self.vat_return.box_two
            # box 2 is the EC purchases VAT, and is the same for Standard and
            # flat schemes - being the standard rate percent of all goods purchased from EC
            # so this is added to make the total standard_output_VAT

            # and the input vat is the vat which would have been claimed under the Standard method
            # This being the vat on purchases, from all supplier invoices received in the period,
            # with the VAT from suppliers credit notes subtracted
            input_VAT = vat_standard.standard_input(from_date, to_date) + self.vat_return.box_two
            # Again the EC purchases VAT is added
        else:
            # VAT method is CASH or FLAT-CASH 
            # VAT amount on all sales (cash output) calculated from all payments received
            # less credit notes with payments issued
            output_VAT = vat_cash.cash_output(from_date, to_date) + self.vat_return.box_two
            # box 2 is the EC purchases VAT, and is the same for Cash and
            # flat schemes - being the standard rate percent of all goods purchased from EC
            # so this is added to make the total cash_output_VAT

            # and the input vat is the vat which would have been claimed under the Cash method
            # This being the vat on purchases, from all supplier payments received in the period,
            # with the VAT from suppliers credit notes received with payments subtracted
            input_VAT = vat_cash.cash_input(from_date, to_date) + self.vat_return.box_two
            # Again the EC purchases VAT is added

        # Remove all the inputs and outputs put into VAT balance during the VAT period
        # This is transaction t2:
        net_balance = output_VAT - input_VAT
        if net_balance >= D(0):
            t2.extend([net_balance, D("0.00")])
        else:
            t2.extend([D("0.00"), abs(net_balance)])
        transaction_list = [t1, t2]

        if vat_method == "STANDARD" or vat_method == "CASH":
            # t3 is the unclaimed VAT, which is the difference between the net_balance
            # and the amount paid - this is transferred to unclaimed VAT
            unclaimed = amount - net_balance
            if unclaimed:
                if unclaimed > D(0):
                    t3 = [  get_account_by_system(9), unclaimed, D("0.00") ]
                else:
                    t3 = [  get_account_by_system(9), D("0.00"), abs(unclaimed) ]
                transaction_list.append(t3)
            # This should all balance     debit        credit
            # cash                         0.00         amount
            # VAT balance                out-in           0.00
            # VAT unclaimed       amount-out+in           0.00


        # if the scheme is FLAT-BASIC or FLAT-CASH, transfer money saved by
        # only having to pay a smaller percentage of sales
        # from the VAT Balance account to the VAT Income account

        # Then transfer the money unclaimed due to not being able to claim
        # vat back from purchases to the 'unclaimed vat' account

        if vat_method == "FLAT-BASIC" or vat_method == "FLAT-CASH":

            # vat_income is the output_VAT minus the amount paid to taxman
            vat_income = output_VAT - amount

            # So this is an income due to not having to pay the full standard rate
            # This is credited into the 'VAT income' account
            if vat_income >= D(0):
                t3 = [ get_account_by_system(8), D("0.00"), vat_income]
            else:
                t3 = [ get_account_by_system(8), abs(vat_income), D("0.00")]
            if vat_income:
                # Only put t3 in the list if it is non zero, otherwise its not doing much
                transaction_list.append(t3)

            # t4 is the input VAT, and is now unclaimed
            # this is debited into the 'Unclaimed VAT' account
            if input_VAT >= D(0):
                t4 = [  get_account_by_system(9), input_VAT, D("0.00") ]
            else:
                t4 = [  get_account_by_system(9), D("0.00"), abs(input_VAT) ]
            if input_VAT:
                # Only put t4 in the list if it is non zero, otherwise its not doing much
                transaction_list.append(t4)

            # This should all balance     debit        credit
            # cash                         0.00         amount
            # VAT balance                out-in           0.00
            # VAT income                   0.00     out-amount
            # VAT unclaimed                  in           0.00


        wx.Yield()
        # Confirm the transaction (function and dialog in gui_functions)
        status, transaction_text = confirm_transaction(self,
                                                        transaction_text,
                                                        transaction_list,
                                                        transaction_date)
        if not status:
            vs_message(self, "The return has not been submitted", "WARNING")
            return

        # Must now implement the transaction and save the vat return
        busy_cursor_on()
        wx.Yield()
        save_vatreturn.save(self.vat_return,
                                transaction_date,
                                transaction_text,
                                transaction_list)

        big_refresh()
        info_log("VAT return submitted")
        vs_message(self, "VAT return submitted", "DONE")
        self.EndModal(wx.ID_OK)


