####### VATStuff #######
#
# view_contacts.py  - Dialog and table to view and edit contacts
#
# Version : 0.6
# Date : 20100802
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#

"""Creates a Dialog and table to view and edit contacts

It is run as part of the vatstuff gui interface.
"""

import sys
import os.path
import csv

import wx
import wx.grid

from vatstuff_packages.vs_utilities import *

from vatstuff_packages.vs_gui.gui_functions import *

from vatstuff_packages.vs_database.create_objects import create_Contact
from vatstuff_packages.vs_database import report_functions

from vatstuff_packages.vs_gui.purchases.view_orders import ViewProjectFrame


def view_contacts_dialog(parent, organization=None, system=0):
    """Creates an ViewContactsDialog when the contacts button is pressed under Sales, Purchases or Company tabs"""
    view_contacts_dialog = ViewContactsDialog(parent, organization, system)

    # If the cursor is busy, unbusy it - as
    # now waiting for a button press
    busy_cursor_off()

    view_contacts_dialog.ShowModal()
    view_contacts_dialog.Destroy()


class ViewContactsDialog(vsDialog):
    "The ViewContactsDialog to view and edit contacts"

    def __init__(self, parent, organization=None, system=0):
        """Create the Dialog, system=1 if the users own organisation is being set"""
        vsDialog.__init__(self, parent, "Contacts")

        # Take a note of the variables
        self.parent = parent
        self.organization = organization
        self.system = system

        button_size = (100, vs_cfg["BUTTON_HEIGHT"])

        # Create a vertical sizer to hold everything
        sizer=wx.BoxSizer(wx.VERTICAL)

        if self.system:
            self.all_contacts = Contact.company_list()
            help_text="""This table holds contact names and details for your company (such as your purchasing department) which
can appear as enquiry points of contact on order forms and invoices, click on any field to edit it."""

        else:
            self.all_contacts = self.organization.get_contacts()
            help_text="This table holds contact details for %s, click on any field to edit it" % self.organization.name

        self.report = report_functions.create_report( "contacts", None, None, contact_list = self.all_contacts)
        
        self.top_text=wx.StaticText(self, -1, help_text)
        sizer.Add(self.top_text, 0, wx.ALL | wx.ALIGN_CENTER, 10)


        # ---------------------- Create the contacts table

        self.contacts_table = ContactsTable(self, self.organization, self.all_contacts)
        sizer.Add(self.contacts_table, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # ---------------------- Apply - Cancel -- buttons

        # Create a horizontal button_sizer to hold the buttons
        button_sizer=wx.BoxSizer(wx.HORIZONTAL)

        # Apply button
        self.apply_button = wx.Button(self, wx.ID_APPLY, size=button_size)
        self.apply_button.SetToolTipString("Click to apply changes.")
        button_sizer.Add(self.apply_button, 0, wx.RIGHT, 40)
        # Start off with apply button disabled - enabled by any changes
        self.apply_button.Enable(False)

        # Close button
        self.close_button = wx.Button(self, wx.ID_CLOSE, size=button_size)
        self.close_button.SetToolTipString("Click to close.")
        button_sizer.Add(self.close_button, 0, wx.RIGHT, 40)

        # csv button
        self.csv_button = wx.Button(self, label="CSV File", size=button_size)
        self.csv_button.SetToolTipString("Click to export the contacts to a csv file.")
        button_sizer.Add(self.csv_button, 0)

        # Add button_sizer to the main sizer
        sizer.Add(button_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # Set the size to the panel
        self.SetSizer(sizer)
        sizer.SetSizeHints(self)

        # ---------- Bind buttons to handler functions
        
        self.Bind(wx.EVT_BUTTON, self.apply_button_pressed, self.apply_button)
        self.Bind(wx.EVT_BUTTON, self.close_button_pressed, self.close_button)
        self.Bind(wx.EVT_BUTTON, self.csv_button_pressed, self.csv_button)


    # ----------------- Handler functions

    @writecheck
    @button_wrap
    def apply_button_pressed(self, event):
        """Apply button - check and apply all fields"""
        status, message = self.contacts_table.check_fields()
        if not status:
            vs_message(self, message, "ERROR")
            return
        if self.system:
            self.all_contacts = Contact.company_list()
        else:
            self.all_contacts = self.organization.get_contacts()
        self.report = report_functions.create_report( "contacts", None, None, contact_list = self.all_contacts)
        self.contacts_table.refresh(self.all_contacts)
        self.csv_button.Enable(True)
        self.apply_button.Enable(False)
        vs_message(self, message, "INFORMATION")


    def close_button_pressed(self, event):
        """Close button"""
        if not self.apply_button.IsEnabled():
            self.EndModal(wx.ID_OK)
            return
        if vs_yesno(self, """\
Closing contacts list: are you sure?
Any changes will be lost unless you press Apply""", "Closing list"):
            self.EndModal(wx.ID_OK)


    @button_wrap
    def csv_button_pressed(self, event):
        """Export the contacts to a csv file"""

        if self.system:
            org_name = Company().name
        else:
            org_name = self.organization.name

        filename = "".join([org_name, "_contacts"])
        filename = proper_filename(filename, "contacts")
        # the function proper_filename removes non alphanumeric characters,
        # if all are removed, it returns the second argument.
        filename = "".join([filename, ".csv"])

        fp=None
        try:
            # Create the save file dialog.
            status, path = save_file_as(self, 'csv', vs_cfg["EXPORT_FOLDER"], filename)
            if not status:
                return

            # open the csv file
            fp=open(path,"wb")
            # save file in csv format
            writer = csv.writer(fp, dialect="excel")
            for row in self.report.render_to_csv():
                writer.writerow(row)
        finally:
            if fp: fp.close()

        # save the directory to the database and to vs_cfg["EXPORT_FOLDER"]
        directory = os.path.dirname(path)
        if directory != vs_cfg["EXPORT_FOLDER"]:
            set_directory("EXPORT_FOLDER", directory)
        info_log("CSV file created: %s" % path)
        vs_message(self, "Your file has been saved", "Saved")


class ContactsTable(wx.grid.Grid):

    def __init__(self, parent, organization=None, all_contacts=[]):
        wx.grid.Grid.__init__(self, parent, size=(900,200), style=wx.WANTS_CHARS)

        self.organization = organization
        self.parent = parent
        self.all_contacts = all_contacts

        num_rows = len(self.all_contacts) + 5   # Make 5 'blank' rows
        self.col_names = ("Name", "Email", "Phone", "Mobile", "Notes")

        num_cols = len(self.col_names)

        self.CreateGrid(num_rows, num_cols)

        # create column labels
        for key, colname in enumerate(self.col_names):
            self.SetColLabelValue(key, colname)

        for key, data in enumerate(self.all_contacts):
            self.SetCellValue(key, 0, data.name)
            self.SetCellValue(key, 1, data.email)
            self.SetCellValue(key, 2, data.phone)
            self.SetCellValue(key, 3, data.mobile)
            self.SetCellValue(key, 4, data.notes)

        self.SetColSize(0, 200)
        self.SetColSize(1, 150)
        self.SetColSize(2, 150)
        self.SetColSize(3, 150)
        self.SetColSize(4, 150)

        self.SetDefaultRowSize(vs_cfg["BUTTON_HEIGHT"], False)
        self.SetDefaultCellAlignment(wx.ALIGN_LEFT, wx.ALIGN_CENTER)

        self.SetColMinimalAcceptableWidth(100)
        self.SetRowLabelSize(50)

        # When a contact is edited, add a further row to ensure there are always blank rows
        # displayed:

        self.Bind(wx.grid.EVT_GRID_CELL_CHANGE, self.addarow)


    def refresh(self, all_contacts):
        "Get all contacts again, and re-fill the table"

        self.all_contacts = all_contacts

        number_of_contacts = len(self.all_contacts)
        current_number_of_rows = self.GetNumberRows()

        # If everything is working ok, this must be true
        assert number_of_contacts < current_number_of_rows
        
        # clear the grid
        self.ClearGrid()

        for key, data in enumerate(self.all_contacts):
            self.SetCellValue(key, 0, data.name)
            self.SetCellValue(key, 1, data.email)
            self.SetCellValue(key, 2, data.phone)
            self.SetCellValue(key, 3, data.mobile)
            self.SetCellValue(key, 4, data.notes)


    def addarow(self, evt):
        """Keep some blank rows at the bottom available for editing
           if the user starts inserting data in the last four rows - 
           add a further row"""
        try:
            # as an item has been edited, enable the apply button
            self.parent.apply_button.Enable(True)
            # and disable the csv as items on table are no longer correct
            self.parent.csv_button.Enable(False)

            current_number_of_rows = self.GetNumberRows()
            if evt.GetRow()+4 > current_number_of_rows:
                self.AppendRows(numRows=1)
        except Exception, e:
            error_message("Failure in editing an item row", e)


    def check_fields(self):
        """Read and check the items on the table
           returns status, message"""

        table = []
        field_number = self.GetNumberRows()
        for row in range(0, field_number):
            contact_items = []
            for col in range(0,5):
                item= self.GetCellValue(row, col)
                contact_items.append(item.strip())
            table.append(contact_items)

        # table is now the list of lists of contact items.

        for key, data in enumerate(table):            # data[0] = name
            if data[0] == "":                         # data[1] = email
                continue                              # data[2] = phone
                                                      # data[3] = mobile
                                                      # data[4] = notes
            # Check each item
            for key, item in enumerate(data):
                if key< 4 and len(item) > 50:
                    return False, """%s too long (50 characters max) at line %s
Your changes have not been saved""" % (self.col_names[key], key+1)
                if len(item) > 100:
                    # The notes item can be up to 100 characters long
                    return False, """Notes too long (100 characters max) at line %s
Your changes have not been saved""" % (key+1,)

        # Update or delete any of the contacts which need changing

        change = False

        for key, data in enumerate(self.all_contacts):
            if table[key][0] == "":
                # delete this item
                data.delete()
                change = True
                continue

            if table[key][0] != data.name:
                data.name = table[key][0]
                change = True
            if table[key][1] != data.email:
                data.email = table[key][1]
                change = True
            if table[key][2] != data.phone:
                data.phone = table[key][2]
                change = True
            if table[key][3] != data.mobile:
                data.mobile = table[key][3]
                change = True
            if table[key][4] != data.notes:
                data.notes = table[key][4]
                change = True

        # And create new contacts

        for key in range(len(self.all_contacts), field_number):
            if table[key][0] != "":
                # Name exists, so create a contact
                customer = None
                supplier = None
                employee = None
                contractor = None
                if self.organization:
                    # customer or supplier
                    system = 0
                    if isinstance(self.organization, Customer):
                        customer = self.organization
                    if isinstance(self.organization, Supplier):
                        supplier = self.organization
                else:
                    # own organisation
                    system = 1

                create_Contact(customer,
                               supplier,
                               employee,
                               contractor,
                               table[key][0],
                               table[key][1],
                               table[key][2],
                               table[key][3],
                               table[key][4],
                               edit_new = False,
                               visible = True,
                               system = system)
                change = True

        if change:
            # if a search contacts frame is open, refresh it
            if vs_cfg["SEARCH_CONTACTS_FRAME"]:
                vs_cfg["SEARCH_CONTACTS_FRAME"].refresh(use_current=True)
            # if a projects frame is open, refresh it
            for frm in vs_cfg["TRANSACTION_REFRESH"]:
                if isinstance(frm, ViewProjectFrame):
                    frm.refresh()
            return True, "Changes have been saved"
        else:
            return True, "No changes have been implemented"

