####### VATStuff #######
#
# vs_wizard.py - creates the wizard startup frames if a new database has been created
#
# Version : 0.6
# Date : 20100814
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#

"""When the program starts - if a new database has been created
   this file creates and runs the wizard frames which setup the
   company name, address, vat registration number, and vat scheme
   used.
"""

import datetime
import calendar
import decimal
import os
import os.path
import time

import wx
import wx.wizard

from vatstuff_packages.vs_database.create_objects import create_Address, create_VATScheme

from vatstuff_packages.vs_gui.address_panel import *
from vatstuff_packages.vs_gui.gui_functions import *

from vatstuff_packages.vs_utilities import *

from vatstuff_packages.vs_verify.form_objects import make_AddressForm

D = decimal.Decimal   # D used to avoid typing
 
def choose_database(parent):
    """"Choose a database file, return (True, path) if a path is chosen
        or (False, None) to quit application"""

    # pick a database location
    choose_database_filepath_dialog = ChooseDatabaseFilepathDialog(parent)

    path = None
    if choose_database_filepath_dialog.ShowModal() == wx.ID_OK:
        path = choose_database_filepath_dialog.path
        choose_database_filepath_dialog.Destroy()
    else:
        # quit button pressed
        choose_database_filepath_dialog.Destroy()
        return False, None
    return True, path


def new_database_wizard(parent, p3=True):
    "Called to create a new database wizard, p3 is True to show page 3, False to not show it"
    # A new database has been created, get company address and vatscheme
    wizard = TheWizard(parent)
    page1 = Page1(wizard)
    page2 = Page2(wizard)
    if p3:
        page3 = Page3(wizard)
    wx.wizard.WizardPageSimple_Chain(page1, page2)
    if p3:
        wx.wizard.WizardPageSimple_Chain(page2, page3)
    wizard.FitToPage(page2)
    wizard.RunWizard(page1)
    wizard.Destroy()


class ChooseDatabaseFilepathDialog(vsDialog):
    """Called when a database file is to be openned at the start of the program
       displayes the splash image, and New, Open, Last and Quit buttons.
       Sets self.path to the database filepath chosen"""

    def __init__(self, parent):
        """Create the frame"""
        vsDialog.__init__(self, parent, vs_cfg["PROGRAM_ID"])
    
        # These are the values being sought
        self.path = vs_cfg["DATABASE_FILEPATH"]
        self.path_directory = vs_cfg["DATABASE_FOLDER"]
        self.path_file = vs_cfg["DATABASE_FILE"]

        button_size = (100, vs_cfg["BUTTON_HEIGHT"])

        # Create a vertical sizer to hold everything
        sizer=wx.BoxSizer(wx.VERTICAL)

        # Get the splash image
        splash_image=os.path.join(vs_cfg["IMAGES_FOLDER"],vs_cfg["SPLASH_IMAGE"])
        if os.path.exists(splash_image):
            bmp = wx.Image(splash_image, wx.BITMAP_TYPE_PNG).ConvertToBitmap()
            splash_bitmap=wx.StaticBitmap(self, -1, bmp)
            sizer.Add(splash_bitmap, 0, wx.ALIGN_CENTER)

        # Create buttons in a grid bag sizer
        b_sizer=wx.GridBagSizer(hgap=20, vgap=20)

        # new button
        self.new_button = wx.Button(self, label="New", size=button_size)
        self.new_button.SetToolTipString("Create a new database file")   
        b_sizer.Add(self.new_button, pos=(0,0), flag=wx.ALIGN_RIGHT)

        new_text=wx.StaticText(self, -1, "Create a new database file")
        b_sizer.Add(new_text, pos=(0,1), flag=wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        # open button
        self.open_button = wx.Button(self, label="Open", size=button_size)
        self.open_button.SetToolTipString("Open an existing database file")   
        b_sizer.Add(self.open_button, pos=(1,0), flag=wx.ALIGN_RIGHT)

        open_text=wx.StaticText(self, -1, "Open an existing database file")
        b_sizer.Add(open_text, pos=(1,1), flag=wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        # last button
        self.last_button = wx.Button(self, label="Last", size=button_size)
        self.last_button.SetToolTipString("Open the most recent database file")   
        b_sizer.Add(self.last_button, pos=(2,0), flag=wx.ALIGN_RIGHT)

        last_text=wx.StaticText(self, -1, "Open the most recent database file")
        b_sizer.Add(last_text, pos=(2,1), flag=wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        if (  (not vs_cfg["STORE_ENABLED"]) or 
              (not vs_cfg["LAST_DATABASE_FILEPATH"]) or 
              (not os.path.isfile(vs_cfg["LAST_DATABASE_FILEPATH"]))  ):
            # Cannot open the last database
            self.last_button.Enable(False)

        # quit button
        self.quit_button = wx.Button(self, wx.ID_EXIT, size=button_size)
        self.quit_button.SetToolTipString("Close the program")   
        b_sizer.Add(self.quit_button, pos=(3,0), flag=wx.ALIGN_RIGHT)

        quit_text=wx.StaticText(self, -1, "Quit the program")
        b_sizer.Add(quit_text, pos=(3,1), flag=wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        # add the b_sizer to the main vertical sizer
        sizer.Add(b_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 30)

        # Set the size to the dialog frame
        self.SetSizer(sizer)
        sizer.SetSizeHints(self)

        # ---------- Bind buttons to handler functions

        self.Bind(wx.EVT_BUTTON, self.new_button_pressed, self.new_button)
        self.Bind(wx.EVT_BUTTON, self.open_button_pressed, self.open_button)
        self.Bind(wx.EVT_BUTTON, self.last_button_pressed, self.last_button)
        self.Bind(wx.EVT_BUTTON, self.quit_button_pressed, self.quit_button)


    # ----------------- Handler functions

    def new_button_pressed(self, event):
        status, path = browse_for_database(self, style=wx.SAVE)
        if status and os.path.exists(path):
            vs_message(self, "Unable to create new database\nas this file already exists", "ERROR")
            return

        if status:
            self.path = path
            self.EndModal(wx.ID_OK)

    def open_button_pressed(self, event):
        status, path = browse_for_database(self, style=wx.OPEN)
        if status:
            self.path = path
            self.EndModal(wx.ID_OK)

    def last_button_pressed(self, event):
        self.path = vs_cfg["LAST_DATABASE_FILEPATH"]
        self.EndModal(wx.ID_OK)

    def quit_button_pressed(self, event):
        "Quit button - return wx.ID_CANCEL"
        self.EndModal(wx.ID_CANCEL)



class TheWizard(wx.wizard.Wizard):
    "The wizard class"
    def __init__(self, parent):
        wx.wizard.Wizard.__init__(self, parent, -1, "%s - Startup wizard" % vs_cfg["PROGRAM_ID"])
        set_icon(self)
        # Log the class creation
        class_log(__name__, self)

        # ---------- Bind the page change
        
        self.Bind(wx.wizard.EVT_WIZARD_PAGE_CHANGING, self.change_page)

    def change_page(self, event):
        page = event.GetPage()
        if isinstance(page, Page1):
            if not page.set_company_configuration():
                event.Veto()
            return
        if isinstance(page, Page2):
            if not page.set_vat_scheme():
                event.Veto()
            return




#  --- page1 - Essentially equal to the CompanyConfigPanel

class Page1(wx.wizard.WizardPageSimple):
    """The panel of the CompanyConfig frame"""

    def __init__(self, parent):
        """Create the panel"""
        wx.wizard.WizardPageSimple.__init__(self, parent)
        
        company = Company()

        main_sizer=wx.BoxSizer(wx.VERTICAL)

        help1_text="Set your companies name and registered office address:"
        
        self.top_text=wx.StaticText(self, -1, help1_text)
        main_sizer.Add(self.top_text, 0, wx.ALIGN_CENTER)

        # Add registered office panel
        self.reg_office_address = company.registered_address

        # make a list of temporary copies
        if self.reg_office_address == None:
            self.address_list = []
        else:
            self.address_list = [make_AddressForm(self.reg_office_address)]

        flag_mask = { 'reg_office'     : 0,
                      'delivery_point' : 1,
                      'invoice_point'  : 1,
                      'order_point'    : 1,
                      'payment_point'  : 1 }
        flag_defaults = { 'reg_office'     : 1,
                          'delivery_point' : 1,
                          'invoice_point'  : 1,
                          'order_point'    : 1,
                          'payment_point'  : 1 }

        self.reg_office_panel = AddressPanel(self,
                                             company.name,
                                             self.address_list,
                                             flag_mask,
                                             flag_defaults,
                                             buttons = False)

        main_sizer.Add(self.reg_office_panel, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        # Create fields and descriptions in a grid sizer
        b_sizer=wx.FlexGridSizer(rows=2, cols=2, hgap=20, vgap=5)

        # Company registration number
        self.company_number_field = wx.TextCtrl(self, -1, company.company_number, size=(200, vs_cfg["BUTTON_HEIGHT"]))
        self.company_number_field.SetToolTipString("Your company registration number")
        self.company_number_field.SetMaxLength(20)
        b_sizer.Add(self.company_number_field, 0, wx.ALIGN_RIGHT)

        company_number_text=wx.StaticText(self, -1, "Set your company registration number")
        b_sizer.Add(company_number_text, 0, wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        # Country of registration
        self.country_field = wx.TextCtrl(self, -1, company.country_registration, size=(200, vs_cfg["BUTTON_HEIGHT"]))
        self.country_field.SetToolTipString("Your country of registration")
        self.country_field.SetMaxLength(20)
        b_sizer.Add(self.country_field, 0, wx.ALIGN_RIGHT)

        country_text=wx.StaticText(self, -1, "Country of registration")
        b_sizer.Add(country_text, 0,  wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        main_sizer.Add(b_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 10)

        help2_text="""\
You can press Cancel to skip this wizard, and fill in
these details within the main application if you wish."""
        
        self.bottom_text=wx.StaticText(self, -1, help2_text)
        main_sizer.Add(self.bottom_text, 0, wx.TOP | wx.ALIGN_CENTER, 10)
        
        # Set the size to the panel
        self.SetSizer(main_sizer)
        main_sizer.SetSizeHints(self)
        self.Fit()


    def set_company_configuration(self):
        debug_log("Company configuration set in the startup wizard") 
        try:
            # Company office address
            status, company_name, address_list, deleted_addresses = self.reg_office_panel.get_addresses()
            if not status:
                return False

            # Company number
            cntext=self.company_number_field.GetValue()
            cntext=str(cntext).strip()
            if len(cntext) > 20:
                vs_message(self, "The company number is incorrect.", "ERROR")
                return False

            # Country of Registration
            crtext=self.country_field.GetValue()
            crtext=str(crtext).strip()
            if len(crtext) < 5 or len(crtext) > 20:
                vs_message(self, "The country of registration is incorrect.", "ERROR")
                return False

            # Set changes
            # get current company again, as this may be different from the company set
            # in the panel as this wizard allows moving back and forth
            company = Company()
            company.country_registration = crtext
            company.company_number = cntext
            company.name = company_name

            # Write registered office

            # Create an address
            address = address_list[0]

            if address.id < 0 :
                # This is a new address which has to be created
                database_address = create_Address(system=1)
            else:
                database_address = Address(address.id)

            database_address.lines = address.lines
            database_address.flags = address.flags

            company.registered_address = database_address

            # Make the default country for addresses, the same country
            # as the registered office
            vs_cfg["COUNTRY"] = database_address.country
        except Exception, e:
            error_message("Error occurred in company configuration changes", e)
            return False
        return True


#  --- page2 - Essentially equal to the VATSchemeDialog

class Page2(wx.wizard.WizardPageSimple):
    """Set the VAT scheme"""

    def __init__(self, parent):
        """Create the page"""
        wx.wizard.WizardPageSimple.__init__(self, parent)
        
        # Get the current VAT Scheme
        self.vat_scheme = VATScheme()

        button_size = (100, vs_cfg["BUTTON_HEIGHT"])

        # Create a vertical sizer to hold everything
        sizer=wx.BoxSizer(wx.VERTICAL)

        top_text=wx.StaticText(self, -1, """This program currently only handles quarterly VAT periods""")
        sizer.Add(top_text, 0, wx.TOP | wx.ALIGN_CENTER, 40)


        # Create radio fields and text in a grid sizer
        field_sizer=wx.FlexGridSizer(rows=4, cols=2, hgap=40, vgap=40)

        # VAT method

        method_list = ['Standard - calculated from invoices',
                       'Cash - calculated from payments',
                       'Flat-Basic - a percentage of invoices issued',
                       'Flat-Cash  - a percentage of payments received',
                       'Not Registered.'
                       ]

        self.vat_method_field = wx.RadioBox(self, -1, "",
                                            wx.DefaultPosition, wx.DefaultSize,
                                            method_list, 0, wx.NO_BORDER | wx.RA_VERTICAL
                                            )

        current_vat_method = self.vat_scheme.method
        if current_vat_method == "STANDARD":
            self.vat_method_field.SetSelection(0)
        if current_vat_method == "CASH":
            self.vat_method_field.SetSelection(1)
        if current_vat_method == "FLAT-BASIC":
            self.vat_method_field.SetSelection(2)
        if current_vat_method == "FLAT-CASH":
            self.vat_method_field.SetSelection(3)
        if current_vat_method == "NOT_REGISTERED":
            self.vat_method_field.SetSelection(4)
        
        self.vat_method_field.SetItemToolTip(0,"Set if you use the standard VAT calculations.")
        self.vat_method_field.SetItemToolTip(1,"Set if you calculate VAT on payments.")
        self.vat_method_field.SetItemToolTip(2,"Set if you are on the flat rate (basic) scheme.")
        self.vat_method_field.SetItemToolTip(3,"Set if you are on the flat rate (cash) scheme.")
        self.vat_method_field.SetItemToolTip(4,"Set if you are not VAT Registered.")
        field_sizer.Add(self.vat_method_field, 0, wx.ALIGN_LEFT)

        text1=wx.StaticText(self, -1, """
Set the VAT scheme your company uses. You should
take advice on the correct scheme - most have
conditions and require Government agreement.""")
        field_sizer.Add(text1, 0, wx.ALIGN_LEFT)

        # Flat scheme percentage value

        flat_sizer=wx.BoxSizer(wx.HORIZONTAL)
        flat_sizer.Add(wx.StaticText(self, -1, "Flat scheme percentage of sales:"), 0, wx.RIGHT | wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL, 10)
        self.flat_percent = self.vat_scheme.flat_percent
        flat_percent_text = str(self.flat_percent).replace(".", vs_cfg["CP"])
        self.flat_percent_field = wx.TextCtrl(self, -1, flat_percent_text, style=wx.ALIGN_RIGHT, size=(60, vs_cfg["BUTTON_HEIGHT"]))
        self.flat_percent_field.SetToolTipString("Set the percentage of sales to be accounted as VAT")
        self.flat_percent_field.SetMaxLength(4)
        flat_sizer.Add(self.flat_percent_field, 0, wx.ALIGN_CENTER_VERTICAL)
        flat_sizer.Add(wx.StaticText(self, -1, "%"), 0, wx.ALIGN_CENTER_VERTICAL)
        field_sizer.Add(flat_sizer,0, wx.ALIGN_RIGHT)

        text2=wx.StaticText(self, -1, """\
This percentage only applies to the Flat Schemes
(Basic or Cash) and varies by trade sector.""")
        field_sizer.Add(text2, 0, wx.ALIGN_LEFT)

        # If the method is flat, enable the flat scheme percentage field
        if current_vat_method == "FLAT-BASIC" or current_vat_method == "FLAT-CASH":
            self.flat_percent_field.Enable(True)
        else:
            self.flat_percent_field.Enable(False)

        # Start of current VAT period
        start_date = wx.DateTime.Now()
        start_date.SetDay(1)

        self.start_date_field = wx.DatePickerCtrl(self, -1,
                                                 start_date, 
                                                 size=(200, vs_cfg["BUTTON_HEIGHT"]),
                                                 style=wx.DP_DROPDOWN | wx.DP_SHOWCENTURY)
        self.start_date_field.SetToolTipString("The start date of the current VAT period")
        field_sizer.Add(self.start_date_field, 0, wx.ALIGN_RIGHT)

        text4=wx.StaticText(self, -1, """\
If not VAT registered, set this to the date you start trading,
otherwise set it to the start date of the current VAT period.""")
        field_sizer.Add(text4, 0, wx.ALIGN_LEFT)

        # vat country code and regestration number
        vat_sizer=wx.BoxSizer(wx.HORIZONTAL)

        self.vat_country_code_field = wx.TextCtrl(self, -1, vs_cfg["VAT_COUNTRY_CODE"], size=(40, vs_cfg["BUTTON_HEIGHT"]))
        self.vat_country_code_field.SetToolTipString("Your VAT two letter country code")
        self.vat_country_code_field.SetMaxLength(2)
        vat_sizer.Add(self.vat_country_code_field, 0, wx.ALIGN_RIGHT)
        self.vat_country_code_field.Enable(False)

        self.vat_regnumber_field = wx.TextCtrl(self, -1, "", size=(150, vs_cfg["BUTTON_HEIGHT"]))
        self.vat_regnumber_field.SetToolTipString("Your VAT number")
        self.vat_regnumber_field.SetMaxLength(20)
        vat_sizer.Add(self.vat_regnumber_field, 0, wx.LEFT | wx.ALIGN_RIGHT, 10)

        field_sizer.Add(vat_sizer, 0, wx.ALIGN_RIGHT)

        vat_regnumber_text=wx.StaticText(self, -1, "Set your VAT number")

        field_sizer.Add(vat_regnumber_text, 0, wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)

        if current_vat_method == "NOT_REGISTERED":
            # If the vat method is NOT REGISTERED, then the vat
            # registration number cannot be set
            self.vat_regnumber_field.Enable(False)

        sizer.Add(field_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 60)

        # Set the size to the panel
        self.SetSizer(sizer)
        sizer.SetSizeHints(self)



        # ---------- Bind handler functions
        
        self.Bind(wx.EVT_RADIOBOX, self.vat_method_pressed, self.vat_method_field)


    # ----------------- Radiobox Handler function

    def vat_method_pressed(self, event):
        "A VAT method has been chosen"
        # Enable the percentage field if the flat method is chosen
        method = self.vat_method_field.GetSelection()
        # If the method is flat, enable the flat scheme percentage field
        if method == 2 or method == 3:
            self.flat_percent_field.Enable(True)
        else:
            self.flat_percent_field.Enable(False)
        # If the method is not registered, disable the vat registration number field
        if method == 4:
            self.vat_regnumber_field.Enable(False)
            self.vat_regnumber_field.SetValue("")
        else:
            self.vat_regnumber_field.Enable(True)
            vrtext=self.vat_regnumber_field.GetValue()
            vrtext=str(vrtext).strip()
            if not vrtext:
                # Get the current Company
                company = Company()
                self.vat_regnumber_field.SetValue(company.vat_regnumber)


    def set_vat_scheme(self):
        "Check and apply vat scheme fields"

        try:
            # VAT Method
            if self.vat_method_field.GetSelection() == 0:
                self.method="STANDARD"
            elif self.vat_method_field.GetSelection() == 1:
                self.method="CASH"
            elif self.vat_method_field.GetSelection() == 2:
                self.method="FLAT-BASIC"
            elif self.vat_method_field.GetSelection() == 3:
                self.method="FLAT-CASH"
            elif self.vat_method_field.GetSelection() == 4:
                self.method="NOT_REGISTERED"
            else:
                vs_message(self, "The VAT method has not been recognised", "ERROR")
                return False

            # Note: The vat registration number and country code are held in the Company object
            # rather than the vatscheme object, this is because invoices etc.. all point to
            # the current company object, and hence to the current vat registration number

            vrtext=self.vat_regnumber_field.GetValue()
            vrtext=str(vrtext).strip()

            vatccode = self.vat_country_code_field.GetValue()
            vatccode=str(vatccode).strip()

            if self.method != "NOT_REGISTERED":
                # VAT Registration number

                if len(vrtext) < 5 or len(vrtext) > 20:
                    vs_message(self, "The VAT Registration number is incorrect\nNo items have been changed.", "ERROR")
                    return False
    
                # VAT country code
                if len(vatccode) != 2:
                    vs_message(self, "The VAT country code is incorrect\nNo items have been changed.", "ERROR")
                    return False

            percent = "10.0"
            if self.method == "FLAT-BASIC" or self.method == "FLAT-CASH":
                # Get percentage value
                percent = self.flat_percent_field.GetValue()
                percent = str(percent).strip()
                percent = percent.replace(vs_cfg["CP"], ".")
                # test if it can be converted into a floating point number
                try:
                    test = float(percent)
                except Exception, e:
                    vs_message(self, "Percentage value is incorrect\nNo items have been changed.", "ERROR")
                    return False
                if test < 0.0 or test > 99.9:
                    vs_message(self, "Percentage value is incorrect\nNo items have been changed.", "ERROR")
                    return False

            # Start Date, with day 1 as this always starts on the 1st day of a month
            self.start_date = wxdate_to_date(self.start_date_field.GetValue()).replace(day=1)

            # Must now save the scheme to the database
            create_VATScheme( vat_regnumber = vrtext,
                           vat_country_code = vatccode,
                                 start_date = self.start_date,
                                     method = self.method,
                                     period = "QUARTERLY",
                                    percent = percent)

        except Exception, e:
            error_message("Error occurred in vat scheme changes", e)
            return False
        return True


#  --- page3 - the final page of the wizard

class Page3(wx.wizard.WizardPageSimple):
    "Displays the final text of the wizard"
    def __init__(self, parent):
        wx.wizard.WizardPageSimple.__init__(self, parent)

        sizer=wx.BoxSizer(wx.VERTICAL)
             
        text = """\
Welcome to %s

On pressing 'Finish' the main window of the application will be shown, which consists
of a number of tabbed functions.

The 'Manual' tab will provide further information.

Under the Accounts tab you will see a list of default accounts, highlight any account and
click the Edit button, and you will be able to set the opening balance of that account.

The Move Money tab allows you to move money between accounts. The Sales and Purchases tabs
allow you add customers and suppliers, then you can create invoices and orders.

Before creating any orders or invoices you should first choose the 'VAT' tab, and then the
'Set VAT Rates' button, to check the percentage rates are correct.  If the Government have
changed any since the program was produced - simply set the correct percentages.

Currently this program is under development, and should not yet be trusted with live data,
however if you can use this in parallel with existing systems any feedback would be greatly
appreciated.

Further details are available at www.vatstuff.co.uk
""" % vs_cfg["PROGRAM_ID"]
        sizer.Add(wx.StaticText(self, -1, text), 0, wx.ALL | wx.ALIGN_CENTER, 20)
        # Set the size to the panel
        self.SetSizer(sizer)
        sizer.SetSizeHints(self)
        self.Fit()



