####### VATStuff #######
#
# create_report.py  - Creates pdf Reports
#
# Version : 0.4
# Date : 20100225
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#


"Creates pdf report files using the reportlab library"


import wx

from vatstuff_packages.vs_utilities import *


from vatstuff_packages.vs_gui.gui_functions import *

import vatstuff_packages.vs_pdf.pdf_functions


from reportlab.platypus import PageTemplate, Frame, NextPageTemplate, FrameBreak
from reportlab.platypus import BaseDocTemplate, Table, Paragraph, Spacer, TableStyle
from reportlab.lib.styles import ParagraphStyle

# global variables needed to pass data into report_page
_report_title = None
_obj = None

def create_pdf(parent, filename, report, obj=None):
    "called by a pdf button on a gui to create and show the provided report"

    global _report_title, _obj
    _report_title = report.title
    _obj = obj

    # Create the save file dialog.
    status, filepath = save_file_as(parent, 'pdf', vs_cfg["REPORTS_PDF_FOLDER"], filename)
    wx.Yield()
    if status:
        # create the actual report
        if do_report(filepath, report):
            # save the directory and report size to the per-user store
            directory = os.path.dirname(filepath)
            set_stored_item("REPORTS_PDF_FOLDER", directory)
            set_stored_item("LAST_REPORT_PDF_SIZE", vs_cfg["LAST_REPORT_PDF_SIZE"])

            # And show the file in the default viewer
            if not vatstuff_packages.vs_pdf.pdf_functions.show_pdf_file(filepath):
                vs_message(parent, "pdf file created", "Done")
        else:
            return False
    return True


def report_page(canvas, doc):
    "Called as page is generated to produce the page background"
    canvas.saveState()
    width, height = vs_cfg["LAST_REPORT_PDF_SIZE"]

    # pos defines:
    #   two points, initially at the top left, and the top right of the canvas
    #   left and right margins
    #   The width of the address column
    pos = {"leftx":0,
           "lefty":height,
           "rightx":width,
           "righty":height,
           "left_margin": 40,
           "right_margin": 40,
           "column_width": 190}

    # Sets the title and company logo and company name
    vatstuff_packages.vs_pdf.pdf_functions.pdf_header(canvas, width, height, pos, _report_title, _obj)

    # Continued from previous page text message just beneath the line

    page_number = canvas.getPageNumber()
    if page_number > 1:
        pos["lefty"] -= 20
        previous_page_number = page_number - 1
        canvas.setFont("Helvetica-Bold", 8)
        canvas.drawString(pos["left_margin"], pos["lefty"],
                          "Continued from page %s" % previous_page_number)

    canvas.setLineWidth(2)
    canvas.line(pos["left_margin"], 58, width-pos["right_margin"], 58)
    canvas.setFont("Helvetica", 8)
    canvas.drawString(pos["left_margin"], 40, "Page %s" % page_number)
    canvas.restoreState()


def do_report(filepath, report):
    "called by create_pdf to generate the pdf file at the given filepath"

    width, height = vs_cfg["LAST_REPORT_PDF_SIZE"]

    pos = {"leftx":0,
           "lefty":height,
           "rightx":width,
           "righty":height,
           "left_margin": 40,
           "right_margin": 40,
           "column_width": 190}

    # top_frame is where the header text goes on the first page only
    # just below the title line
    top_frame  = Frame(pos["left_margin"],
                       height-200,
                       width-pos["left_margin"]-pos["right_margin"]-pos["column_width"],
                       130,
                       showBoundary=0)

    main_frame = Frame(pos["left_margin"],
                       80,
                       width-pos["left_margin"]-pos["right_margin"],
                       height-280,
                       showBoundary=0)

    first_page  =  PageTemplate(id="first", frames=[top_frame, main_frame], onPage=report_page)
    other_pages =  PageTemplate(id="others", frames=[main_frame], onPage=report_page)
    
    doc = BaseDocTemplate(filepath,
                pagesize = vs_cfg["LAST_REPORT_PDF_SIZE"],
                pageTemplates=[first_page, other_pages],
                showBoundary=0,
                leftMargin = pos["left_margin"],
                rightMargin = pos["right_margin"],
                topMargin = 80,
                bottomMargin = 80,
                allowSplitting=1,
                title=report.title,
                _pageBreakQuick=1)

    style = ParagraphStyle('reportstandard')
    style.fontSize = 8
    style.fontName = "Helvetica"

    Story = [NextPageTemplate("others")]
    for item in report.header:
        p = Paragraph(item, style)
        # if item is a "" add a bit of space
        if not item:
            p = Spacer(20, 20)
        Story.append(p)

    # go to next frame
    Story.append(FrameBreak())

    # tables are described by a list of list,
    # each list being a row, consisting of cell tuple
    # each cell tuple containing two strings; 
    # The first string is formed from "XXXN",
    # and the second string is the text to be placed in the cell

    # N is the collspan number, as used in a table cell tag attribute
    # such as 'colspan="4"' - in which case N is 4

    # so normally N is '1'

    # XXX is a formatting string :
    # HBL  - Header Bold Left Justified (R for Right instead of L
    # HNL  - Header Normal Left Justified  etc
    # DNL  - Data Cell Normal Left Justified  etc
    # XXC  - Right Justified currency, as pulled from the database

    # insert the table
    if report.table:

        tablestyle = TableStyle([ ('FONTNAME', (0,0), (-1,-1), "Helvetica"),
                                  ('FONTSIZE', (0,0), (-1,-1), 8),
                                  ('VALIGN',   (0,0), (-1,-1), 'MIDDLE') ])

        # the coords are start (col, row) to end (col, row)

        # Set up any coords which should be bold - do by row, as it is likely only single
        # rows will be bold - not entire nxn blocks
        new_coord = True
        rw = -1
        for row in report.table:
            rw += 1
            cl = -1
            for cell in row:
                col = cl+1
                cl += int(cell[0][3])
                if cell[0][1] == 'B':
                    # This cell is requesting bold text
                    if new_coord:
                        coord = [ (col, rw), (cl, rw) ]
                        new_coord = False
                    else:
                        coord[1] = (cl, rw)
                else:
                    if not new_coord:
                        tablestyle.add('FONTNAME', coord[0], coord[1], "Helvetica-Bold")
                        new_coord = True
            # end of row reached
            if not new_coord:
                tablestyle.add('FONTNAME', coord[0], coord[1], "Helvetica-Bold")
                new_coord = True

        # set up coords which are requesting right alignment

        # First - just do this for header rows - as these will not be nxn blocks
        new_coord = True
        rw = -1
        for row in report.table:
            rw += 1
            cl = -1
            if row[0][0][0] != 'H':
                # Header rows only
                continue
            for cell in row:
                col = cl+1
                cl += int(cell[0][3])
                if cell[0][2] == 'R' or cell[0][2] == 'C':
                    # This cell is requesting right alignment
                    if new_coord:
                        coord = [ (col, rw), (cl, rw) ]
                        new_coord = False
                    else:
                        coord[1] = (cl, rw)
                else:
                    if not new_coord:
                        tablestyle.add('ALIGN', coord[0], coord[1], "RIGHT")
                        new_coord = True
            # end of row reached
            if not new_coord:
                tablestyle.add('ALIGN', coord[0], coord[1], "RIGHT")
                new_coord = True

        # now for the data rows

        coords = []
        # coords is a list of lists of double lists, ie [ [[0,0],[1,1]], [[2,2],[3,3]] ]
        # Find right - alignments on the data rows

        new_coord = True
        rw = -1
        for row in report.table:
            rw += 1
            cl = -1
            if row[0][0][0] == 'H':
                # Data rows only
                continue
            for cell in row:
                col = cl+1
                cl += int(cell[0][3])
                if cell[0][2] == 'R' or cell[0][2] == 'C':
                    # This cell is requesting right alignment
                    if new_coord:
                        coord = [ [col, rw], [cl, rw] ]
                        new_coord = False
                        continue
                    else:
                        coord[1] = [cl, rw]
                else:
                    if not new_coord:
                        coords.append(coord)
                        new_coord = True
            # end of row reached
            if not new_coord:
                coords.append(coord)
                new_coord = True

        # find n x n blocks
        nxn = []
        if coords:
            if len(coords) == 1:
                # Don't bother
                for coord in coords:
                    tablestyle.add('ALIGN', coord[0], coord[1], 'RIGHT')
            else:
                for coord in coords:
                    add_coord = True
                    if nxn:
                        for c in nxn:
                            # if coord start colum is the same as previous c start colum
                            # and coord end column is the same as previous c end column
                            # and coord start row is the same as previous c end row plus one
                            # then alter existing c in nxn by incrementing its end row
                            if (coord[0][0] == c[0][0] and
                                coord[1][0] == c[1][0] and
                                coord[0][1] == c[1][1] +1):
                                # alter existing c in nxn by incrementing its end row
                                c[1][1] += 1
                                add_coord = False
                                break  
                    if add_coord:
                        nxn.append(coord)    
               
        if nxn:
            for coord in nxn:
                tablestyle.add('ALIGN', coord[0], coord[1], 'RIGHT')

        # Find any spans - note this supports column spans on a each row
        rw = -1
        for row in report.table:
            rw += 1
            cl = -1
            for cell in row:
                col = cl+1
                cl += int(cell[0][3])
                if cell[0][3] != '1':
                    # This cell is requesting row span
                    span_from = (col, rw)
                    span_to = (cl, rw)
                    tablestyle.add('SPAN', span_from, span_to)

        table_data = []
        for row in report.table:
            table_data_row = []
            for cell in row:
                if cell[0][2] == 'C':
                    # Currency value
                    table_data_row.append(to_currency_string(cell[1].replace(".", vs_cfg["CP"])))
                else:
                    table_data_row.append(unicode(cell[1]))
                if cell[0][3] != '1':
                    # Insert the extra empty cells
                    for i in range(1, int(cell[0][3])):
                        table_data_row.append('')                
            table_data.append(table_data_row)
        Story.append(Table(table_data, style=tablestyle))

    Story.append(Spacer(1,20))

    # Insert any footer text
    for item in report.footer:
        p = Paragraph(item, style)
        # if item is a "" add a bit of space
        if not item:
            p = Spacer(20, 20)
        Story.append(p)

    doc.build(Story)
    return True




