####### VATStuff #######
#
# invoice_form.py  - Creates an Invoice pdf file
#
# Version : 0.2
# Date : 20090826
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#

"""Creates an Invoice pdf file"""


import os

from reportlab.pdfgen.canvas import Canvas

from vatstuff_packages.vs_utilities import *

from vatstuff_packages.vs_gui.gui_functions import *

from vatstuff_packages.vs_pdf.pdf_functions import *







def create_pdf(filepath, invoice):
    "Create an invoice form pdf file at the given filepath" 

    width = invoice.pdf_width
    height = invoice.pdf_height

    # Is the company charging VAT?
    show_vat = invoice.show_vat

    # The canvas object needs as arguments the pdf filename and the page size as a tuple
    c = Canvas(filepath, (width, height))

    # pos defines:
    #   two points, initially at the top left, and the top right of the canvas
    #   left and right margins
    #   The width of the address column
    pos = {"leftx":0,
           "lefty":height,
           "rightx":width,
           "righty":height,
           "left_margin": 40,
           "right_margin": 40,
           "column_width": 190}

    # Sets the title and company logo and company name
    pdf_header(c, width, height, pos, "Invoice", invoice)

    # save header y values, as they are used to calculate multiple pages
    header_righty = pos["righty"]
    pos["lefty"] -= 30
    header_lefty = pos["lefty"]

    # These header_righty and header_lefty values are the header y values used on subsequent
    # pages - which do not include the to-address or address boxes

    ########## LEFT side of form

    # If the invoice is paid or cancelled, print under the top line of this first page
    c.setFont("Helvetica-Bold", 16)
    if invoice.is_cancelled:
        c.drawString(pos["left_margin"], pos["lefty"], "INVOICE CANCELLED")
    elif not invoice.is_open:
        c.drawString(pos["left_margin"], pos["lefty"], "INVOICE PAID")

    # Print customer address to fall in envelope window on left side of page
    pdf_to_address(c, width, height, pos, invoice.customer.name, invoice.customer_address, invoice)

    # The customers order number, don't show it too high
    if pos["lefty"] > height-300:
        pos["lefty"] = height-300

    c.setFont("Helvetica", 8)
    customer_text = []
    if invoice.cus_order_number:
        customer_text.append("Your order number: %s" % invoice.cus_order_number)
    if invoice.customer.ec_country and invoice.customer.vat_regnumber:
        customer_text.append("Your VAT No. %s %s" % (invoice.customer.vat_country_code, invoice.customer.vat_regnumber))
    if customer_text:
        # temp store y value
        reftexty = pos["lefty"]
        textobject = c.beginText(pos["left_margin"], pos["lefty"])
        textobject.textLines(customer_text)
        c.drawText(textobject)
        pos["leftx"], pos["lefty"] = textobject.getCursor()
        # header_lefty has reduces by this text amount
        header_lefty -= (reftexty - pos["lefty"])

    # The two dates and invoice number on the left
    box_x, box_y = pdf_two_dates_number_box(c, width, height, pos,
                                            "Invoice Number:",
                                            invoice.date_of_issue, invoice.date_of_supply, invoice)
    # introduce a bit of space
    pos["lefty"] -= 20

    # header_lefty should be reduced by the box space, and also by space for the 'continued from' line
    header_lefty -= (box_y+30)

    # header_y is the header space, and is the smallest of header_lefty and header_righty
    if header_lefty > header_righty:
        header_y = header_righty
    else:
        header_y = header_lefty

    ########## RIGHT side of form

    pdf_address_box(c, width, height, pos, "Send payment to:", invoice.payment_address, invoice)

    # introduce a bit of space
    pos["righty"] -= 20

    # put in the contact details on the right
    pdf_contact(c, width, height, pos, invoice)

    ########## Payment instructions

    payment_instructions = invoice.payment_instructions
    if payment_instructions:
        # Put payment instructions in bold above the table
        c.setFont("Helvetica-Bold", 8)
        string_width  = c.stringWidth(payment_instructions, "Helvetica-Bold", 8)
        if string_width > (pos["rightx"] - pos["left_margin"]):
            # Make sure lefty is lower than righty
            if pos["lefty"] >= pos["righty"]:
                pos["lefty"] = pos["righty"] - 20
        c.drawString(pos["left_margin"], pos["lefty"], payment_instructions)
        pos["lefty"] -=20

    ########## Table of items invoiced

    # table is to be lower than the lowest text so far
    if pos["lefty"] > pos["righty"]:
            pos["lefty"]=pos["righty"]

    # get a local copy of the item_list, to avoid multiple database lookups
    item_list = invoice.item_list

    # And the total number of items to print is:
    number_of_items = len( item_list )

    # Are any discounts applied? set discount flag true if there are
    discount = False
    if invoice.discount == D(0) or number_of_items == 0:
        discount = False
    else:
        for item in item_list:
            if item["Discount"] == True:
                discount = True
                break

    # Space required for the table is 20 per item, plus 21 for the header line 
    space = 21 + 20*number_of_items
    # space required at the bottom of the page:
    space += 80

    # space required for the totals box and VAT box
    vat_space = 10
    if discount:
        vat_space += 30   # two lines of text plus space

    if show_vat:
        # Get a list of VAT rates, removing duplicates
        vat_rate_list = []
        for item in item_list:
            rate = item["VAT"]
            if not rate in vat_rate_list:
                vat_rate_list.append(rate)

        # Order the VAT rate list, by descending amounts
        vat_rate_list.sort(reverse=True)

        # vat_space is 21 for the header and 20 per vat line
        if vat_rate_list:
            vat_space += 21 + 20 * len(vat_rate_list)

        # and the space for the totals box is considered to be 70
        totals_space = 70
    else:
        totals_space = 40  # One line of text plus some space 

    # bottom space is largest of vat space or totalspace
    if vat_space > totals_space:
        space += vat_space
    else:
        space += totals_space
        vat_space = totals_space

    if space < pos["lefty"] or number_of_items == 0:
        # can get everything on this page
        total_pages = 1
        # display the grid of items
        pdf_items(c, width, height, pos, 0, number_of_items-1, item_list, invoice, discount, show_vat)
        pos["lefty"] -= 10
        # display the totals box
        pdf_totals(c, width, height, pos, invoice, show_vat)
        c.setFont("Helvetica", 8)
        if discount:
            # display a bit of discount text
            if show_vat:
                lines = ["Costs and VAT are calculated after the",
                         "discount for each item has been applied"]
            else:
                lines = ["Costs are calculated after the",
                         "discount for each item has been applied"]
            textobject = c.beginText(pos["left_margin"], pos["lefty"]-12)
            textobject.textLines(lines)
            c.drawText(textobject)
            pos["leftx"], pos["lefty"] = textobject.getCursor()
        # display the vat box
        if show_vat:
            pdf_vatbox(c, width, height, pos, vat_rate_list, item_list, invoice)
    else:        
        # more than one page is required
        # How many items on this page (80 = page bottom, 21 = table header):
        space_available_on_this_page = pos["lefty"] - 80 - 21
        lines_which_can_fit_on_this_page = int(space_available_on_this_page // 20)
        if lines_which_can_fit_on_this_page >= number_of_items:
            # Ensure at least one line is put on the following page - so it does not contain a totals box only
            items_on_page = number_of_items - 1
        else:
            items_on_page = lines_which_can_fit_on_this_page
        # Draw these items
        pdf_items(c, width, height, pos, 0, items_on_page-1, item_list, invoice, discount, show_vat)

        # max_space_per_page is the space without the totals box
        max_space_per_page = header_y - 80 - 21
        # max_number_of_items is the number that can fit in the max_space_per_page
        max_number_of_items = int(max_space_per_page // 20)

        # min_space_per_page is the space with the totals and vat box
        min_space_per_page = header_y - 80 - 21 - vat_space
        # min_number_of_items is the number that can fit in the min_space_per_page
        min_number_of_items = int(min_space_per_page // 20)

        # How many items require to be printed
        items_left = number_of_items - items_on_page

        # count how many items are to be printed per page
        items_per_page = []

        while items_left>0:
            # next page
            if items_left <= min_number_of_items:
                # All items fit on this page, with a totals box
                items_per_page.append(items_left)
                break
            if items_left <= max_number_of_items:
                # All but one item fits, with no totals box, carry single item to next page
                items_per_page.append(items_left-1)
                items_per_page.append(1)
                break
            # items_left must be greater than will fit on a page    
            items_per_page.append(max_number_of_items)
            items_left -= max_number_of_items

        total_pages =  1 + len(items_per_page)

    ########## Page bottom

    # If the invoice is paid or cancelled, print over the bottom line
    c.setFont("Helvetica-Bold", 16)
    if invoice.is_cancelled:
        c.drawString(pos["left_margin"], 65, "INVOICE CANCELLED")
    elif not invoice.is_open:
        c.drawString(pos["left_margin"], 65, "INVOICE PAID")

    # Sets the bottom line with page numbers
    pdf_bottom(c, width, height, pos, total_pages)

    # End this page and if necessary start a new one
    c.showPage()

    ########## Do the subsequent pages

    if total_pages > 1:
        items_left = number_of_items - items_on_page
        for items in items_per_page:
            # so do this for each page
            pos["leftx"] = 0
            pos["lefty"] = height
            pos["rightx"] = width
            pos["righty"] = height
            # Sets the title and company logo and company name
            pdf_header(c, width, height, pos, "Invoice", invoice)
            # create a bit of space under the line
            pos["lefty"] -= 30
            # put in customer reference text
            c.setFont("Helvetica", 8)
            customer_text = []
            if invoice.cus_order_number:
                customer_text.append("Your order number: %s" % invoice.cus_order_number)
            if invoice.customer.ec_country and invoice.customer.vat_regnumber:
                customer_text.append("Your VAT No. %s %s" % (invoice.customer.vat_country_code,
                                                             invoice.customer.vat_regnumber))
            if customer_text:
                textobject = c.beginText(pos["left_margin"], pos["lefty"])
                textobject.textLines(customer_text)
                c.drawText(textobject)
                pos["leftx"], pos["lefty"] = textobject.getCursor()
            # The two dates and invoice number on the left
            box_x, box_y = pdf_two_dates_number_box(c, width, height, pos,
                                            "Invoice Number:",
                                            invoice.date_of_issue, invoice.date_of_supply, invoice)
            # Continued from previous page text message just beneath
            pos["lefty"] -= 20
            page_number = c.getPageNumber()
            previous_page_number = page_number - 1
            c.setFont("Helvetica-Bold", 8)
            c.drawString(pos["left_margin"], pos["lefty"], "Continued from page %s" % previous_page_number)
            # table is to be lower than the lowest text so far
            if pos["lefty"] > pos["righty"]:
                    pos["lefty"]=pos["righty"]
            # Give a bit of space
            pos["lefty"] -= 6
            # display the grid of items
            pdf_items(c, width, height, pos,
                      number_of_items - items_left,
                      number_of_items - items_left + items -1,
                      item_list, invoice, discount, show_vat)
            items_left -= items
            if items_left < 1:
                # This is the last page
                # display the totals box
                pos["lefty"] -= 10
                pdf_totals(c, width, height, pos, invoice, show_vat)
                c.setFont("Helvetica", 8)
                if discount:
                    # display a bit of discount text
                    if show_vat:
                        lines = ["Costs and VAT are calculated after the",
                                 "discount for each item has been applied"]
                    else:
                        lines = ["Costs are calculated after the",
                                 "discount for each item has been applied"]
                    textobject = c.beginText(pos["left_margin"], pos["lefty"]-12)
                    textobject.textLines(lines)
                    c.drawText(textobject)
                    pos["leftx"], pos["lefty"] = textobject.getCursor()
                # display the vat box
                if show_vat:
                    pdf_vatbox(c, width, height, pos, vat_rate_list, item_list, invoice)

            # If the invoice is closed or cancelled, print over the bottom line
            c.setFont("Helvetica-Bold", 16)
            if invoice.is_cancelled:
                c.drawString(pos["left_margin"], 65, "INVOICE CANCELLED")
            elif not invoice.is_open:
                c.drawString(pos["left_margin"], 65, "INVOICE PAID")

            # Sets the bottom line with page numbers
            pdf_bottom(c, width, height, pos, total_pages)

            # End this page and if necessary start a new one
            c.showPage()

    # Do the full drawing to the canvas, and save the pdf file
    c.save()

    # save the directory to the per-user store
    directory = os.path.dirname(filepath)
    set_stored_item("INVOICES_PDF_FOLDER", directory)

    # save the page size to vs_cfg["LAST_INVOICE_PDF_SIZE"] and the per-user store
    set_stored_item("LAST_INVOICE_PDF_SIZE", (width, height))

    return True





