####### VATStuff #######
#
# makepayment_form.py  - Creates a 'please find payment attached' letter
#
# Version : 0.2
# Date : 20090807
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#

"""Creates a 'please find payment attached' letter with a list of supplier invoices"""


import os

from reportlab.pdfgen.canvas import Canvas

from vatstuff_packages.vs_utilities import *

from vatstuff_packages.vs_gui.gui_functions import *

from vatstuff_packages.vs_pdf.pdf_functions import *







def create_pdf(filepath, payment):
    "Create a payment form pdf file at the given filepath" 

    width, height = vs_cfg["LAST_REPORT_PDF_SIZE"]

    # The canvas object needs as arguments the pdf filename and the page size as a tuple
    c = Canvas(filepath, (width, height))

    # pos defines:
    #   two points, initially at the top left, and the top right of the canvas
    #   left and right margins
    #   The width of the address column
    pos = {"leftx":0,
           "lefty":height,
           "rightx":width,
           "righty":height,
           "left_margin": 40,
           "right_margin": 40,
           "column_width": 190}

    # Sets the title and company logo and company name
    pdf_header(c, width, height, pos, "Payment", payment)

    # save this righty value, as it is used to calculate multiple pages
    header_y = pos["righty"]


    ########## LEFT side of form

    if payment.supplier_address:
        # Print supplier address to fall in envelope window on left side of page
        pdf_to_address(c, width, height, pos, payment.supplier.name, payment.supplier_address, payment)
        # Don't show the next bit too high
        if pos["lefty"] > height-300:
            pos["lefty"] = height-300
    else:
        # Just print the supplier name
        c.setFont("Helvetica-Bold", 10)
        c.drawString(pos["left_margin"], pos["lefty"] - 30, "To : %s" % (payment.supplier.name,))
        pos["lefty"] -= 60

    c.setFont("Helvetica", 8)
    supplier_text = ["Purchase order : %s" % (payment.purchaseorder.number,),
                     "Our payment reference : %s" % (payment.transaction.id,),
                     "Payment date : %s" % (payment.datestamp.strftime("%d %b %Y"))]

    textobject = c.beginText(pos["left_margin"], pos["lefty"])
    textobject.textLines(supplier_text)
    c.drawText(textobject)
    pos["leftx"], pos["lefty"] = textobject.getCursor()

    # introduce a bit of space
    pos["lefty"] -= 30

    # print the payment amount
    c.setFont("Helvetica-Bold", 10)
    c.drawString(pos["left_margin"], pos["lefty"], "Payment amount : %s" % (to_currency_string(payment.amount),))

    # introduce a bit of space
    pos["lefty"] -= 40

    # text message just beneath, and to right a bit as the table is not wide
    c.setFont("Helvetica", 8)
    pos["leftx"] = 100
    c.drawString(pos["leftx"], pos["lefty"], "Please find payment attached, applied to your invoices :")

    # introduce a bit of space
    pos["lefty"] -= 10


    ########## Table of invoices

    # table is to be lower than the lowest text so far
    if pos["lefty"] > pos["righty"]:
            pos["lefty"]=pos["righty"]

    inv_list = payment.get_invoice_list()
    # inv_list being a list of lists - { "INVOICE":supplierinvoice,
    #                                    "COST":cost,
    #                                    "VAT":vat }
    #         one dictionary for each invoice this payment covers

    # And the total number of invoices to print is:
    number_of_items = len( inv_list )

    # Space required for the table is 20 per item, plus 21 for the header line 
    space = 21 + 20*number_of_items
    # space required at the bottom of the page:
    space += 80


    if space < pos["lefty"]:
        # can get everything on this page
        total_pages = 1
        # display the grid of invoices
        invoice_grid(c, width, height, pos, inv_list, number_of_items)
    else:        
        # more than one page is required
        # How many items on this page (80 = page bottom, 21 = table header):
        space_available_on_this_page = pos["lefty"] - 80 - 21
        items_on_page = int(space_available_on_this_page // 20)
        # Draw these items
        invoice_grid(c, width, height, pos, inv_list, items_on_page)

        # space_per_page is
        space_per_page = header_y - 80 - 21
        # items_per_page is the number that can fit in the space_per_page
        items_per_page = int(space_per_page // 20)

        # How many items require to be printed
        items_left = number_of_items - items_on_page

        pages = items_left // items_per_page

        if pages * items_per_page < items_left:
            total_pages =  2 + pages
        else:
            total_pages =  1 + pages

    ########## Page bottom

    # Sets the bottom line with page numbers
    pdf_bottom(c, width, height, pos, total_pages)

    # End this page and if necessary start a new one
    c.showPage()


    ########## Do the subsequent pages

    if total_pages > 1:
        for page in range(1, total_pages):
            # so do this for each page
            pos["leftx"] = 100
            pos["lefty"] = height
            pos["rightx"] = width
            pos["righty"] = height
            # Sets the title and company logo and company name
            pdf_header(c, width, height, pos, "Payment", payment)
            # create a bit of space under the line
            pos["lefty"] -= 20
            page_number = c.getPageNumber()
            previous_page_number = page_number - 1
            c.setFont("Helvetica-Bold", 8)
            c.drawString(pos["left_margin"], pos["lefty"], "Continued from page %s" % previous_page_number)
            # table is to be lower than the lowest text so far
            if pos["lefty"] > pos["righty"]:
                    pos["lefty"]=pos["righty"]
            # Give a bit of space
            pos["lefty"] -= 6
            # display the grid of items
            invoice_grid(c, width, height, pos, inv_list, items_per_page)
            # Sets the bottom line with page numbers
            pdf_bottom(c, width, height, pos, total_pages)
            # End this page and if necessary start a new one
            c.showPage()

    # Do the full drawing to the canvas, and save the pdf file
    c.save()

    # save the directory and pdf size to the per-user store
    directory = os.path.dirname(filepath)
    set_stored_item("REPORTS_PDF_FOLDER", directory)
    set_stored_item("LAST_REPORT_PDF_SIZE", vs_cfg["LAST_REPORT_PDF_SIZE"])

    return True



def invoice_grid(canvas, width, height, pos, inv_list, number):
    "draw a grid of the number of invoices in inv_list"

    # inv_list being a list of lists - { "INVOICE":supplierinvoice,
    #                                    "COST":cost,
    #                                    "VAT":vat }

    if not inv_list:
        # invoice list is empty
        return

    if len(inv_list) < number:
        number = len(inv_list)

    c = canvas

    x = pos["leftx"]
    y = pos["lefty"]-12 # Header text y value

    # Create table headers
    x_inv = c.stringWidth("Supplier Invoice number", "Helvetica-Bold", 8)
    x_inv += 40  # 20 points on each side of the Invoice header string

    x_net = c.stringWidth("Net Paid", "Helvetica-Bold", 8)
    x_net += 40  # 20 points on each side of the net paid header string

    x_vat = c.stringWidth("VAT Paid", "Helvetica-Bold", 8)
    x_vat += 40  # 20 points on each side of the vat paid header string

    c.setFont("Helvetica-Bold", 8)

    c.drawString(x+20, y, "Supplier Invoice number")
    c.drawString(x+x_inv+20, y, "Net Paid")
    c.drawString(x+x_inv+x_net+20, y, "VAT Paid")

    c.setFont("Helvetica", 8)

    for i in range(0, number):
        y -= 20
        inv = inv_list.pop(0)

        invoice_string = inv["INVOICE"].number
        c.drawString(x+20, y, invoice_string)

        net_string = to_currency_string(inv["COST"])
        c.drawRightString(x+x_inv+x_net-20, y, net_string)

        vat_string = to_currency_string(inv["VAT"])
        c.drawRightString(x+x_inv+x_net+x_vat-20, y, vat_string)


    # Create grid
    c.setLineWidth(1)
    y_values = [ pos["lefty"] - 20*Y for Y in range(number + 2) ]
    x_values = [ x, x+x_inv, x+x_inv+x_net, x+x_inv+x_net+x_vat]

    c.grid(x_values, y_values)
    pos["lefty"] = y_values[-1]



