####### VATStuff #######
#
# order_form.py  - Creates a Purchase Order pdf file
#
# Version : 0.2
# Date : 20090826
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#

"""Creates a Purchase Order pdf file"""


import os

from reportlab.pdfgen.canvas import Canvas

from vatstuff_packages.vs_utilities import *

from vatstuff_packages.vs_gui.gui_functions import *

from vatstuff_packages.vs_pdf.pdf_functions import *







def create_pdf(filepath, order):
    "Create an order form pdf file at the given filepath" 

    width = order.pdf_width
    height = order.pdf_height

    # Is the company charging VAT?
    show_vat = order.show_vat

    # The canvas object needs as arguments the pdf filename and the page size as a tuple
    c = Canvas(filepath, (width, height))

    # pos defines:
    #   two points, initially at the top left, and the top right of the canvas
    #   left and right margins
    #   The width of the address column
    pos = {"leftx":0,
           "lefty":height,
           "rightx":width,
           "righty":height,
           "left_margin": 40,
           "right_margin": 40,
           "column_width": 190}

    # Sets the title and company logo and company name
    pdf_header(c, width, height, pos, "Purchase Order", order)

    # save this righty value, as it is used to calculate multiple pages
    header_y = pos["righty"]

    # If the order is closed or cancelled, print under the top line of this first page
    c.setFont("Helvetica-Bold", 16)
    if order.is_cancelled:
        c.drawString(pos["left_margin"], pos["lefty"] - 30, "ORDER CANCELLED")
    elif not order.is_open:
        c.drawString(pos["left_margin"], pos["lefty"] - 30, "ORDER CLOSED")

    ########## LEFT side of form

    # Print supplier address to fall in envelope window on left side of page
    pdf_to_address(c, width, height, pos, order.supplier.name, order.supplier_address, order)

    # The suppliers reference field, don't show it too high
    if pos["lefty"] > height-300:
        pos["lefty"] = height-300
    c.setFont("Helvetica", 8)
    supplier_text = []
    if order.sup_ref_field:
        supplier_text.append("Your reference: %s" % order.sup_ref_field)
    if order.supplier.ec_country and order.supplier.vat_regnumber:
        supplier_text.append("Your VAT No. %s %s" % (order.supplier.vat_country_code, order.supplier.vat_regnumber))
    if supplier_text:
        textobject = c.beginText(pos["left_margin"], pos["lefty"])
        textobject.textLines(supplier_text)
        c.drawText(textobject)
        pos["leftx"], pos["lefty"] = textobject.getCursor()

    # The date and purchase order number on the left
    pdf_date_number_box(c, width, height, pos, "Purchase Order No:", order.datestamp, order)

    # introduce a bit of space
    pos["lefty"] -= 20

    # Please supply text message just beneath
    c.setFont("Helvetica", 8)
    c.drawString(pos["left_margin"], pos["lefty"], "Please supply %s with the following items:" % order.company.name)

    # introduce a bit of space
    pos["lefty"] -= 10

    ########## RIGHT side of form

    # The invoice/delivery addresses on the right
    if order.invoice_address == order.delivery_address:
        # One address for both delivery and invoices
        pdf_address_box(c, width, height, pos, "Send invoices and deliveries to:", order.invoice_address, order)
    else:
        # Invoice-to address below Registered address on the right
        pdf_address_box(c, width, height, pos, "Invoice to:", order.invoice_address, order)
        # Deliver-to address below Invoice address on the right
        pdf_address_box(c, width, height, pos, "Deliver to:", order.delivery_address, order)
        pass

    # introduce a bit of space
    pos["righty"] -= 20

    # put in the contact details on the right
    pdf_contact(c, width, height, pos, order)

    ########## Table of items ordered

    # table is to be lower than the lowest text so far
    if pos["lefty"] > pos["righty"]:
            pos["lefty"]=pos["righty"]

    # get a local copy of the item_list, to avoid multiple database lookups
    item_list = order.item_list

    # And the total number of items to print is:
    number_of_items = len( item_list )

    # Are any discounts applied? set discount flag true if there are
    discount = False
    if order.discount == D(0) or number_of_items == 0:
        discount = False
    else:
        for item in item_list:
            if item["Discount"] == True:
                discount = True
                break

    # Space required for the table is 20 per item, plus 21 for the header line 
    space = 21 + 20*number_of_items
    # space required at the bottom of the page:
    space += 80
    # space required for the totals box
    space += 70

    if space < pos["lefty"] or number_of_items == 0:
        # can get everything on this page
        total_pages = 1
        # display the grid of items
        pdf_items(c, width, height, pos, 0, number_of_items-1, item_list, order, discount, show_vat)
        pos["lefty"] -= 10
        # display the totals box
        pdf_totals(c, width, height, pos, order, show_vat)
        c.setFont("Helvetica", 8)
        if discount:
            if show_vat:
                lines = ["Costs and VAT are calculated after the",
                         "discount for each item has been applied"]
            else:
                lines = ["Costs are calculated after the",
                         "discount for each item has been applied"]
            textobject = c.beginText(pos["left_margin"], pos["lefty"]-22)
            textobject.textLines(lines)
            c.drawText(textobject)
    else:        
        # more than one page is required
        # How many items on this page (80 = page bottom, 21 = table header):
        space_available_on_this_page = pos["lefty"] - 80 - 21
        lines_which_can_fit_on_this_page = int(space_available_on_this_page // 20)
        if lines_which_can_fit_on_this_page >= number_of_items:
            # Ensure at least one line is put on the following page
            # - so it does not contain a totals box only
            items_on_page = number_of_items - 1
        else:
            items_on_page = lines_which_can_fit_on_this_page
        # Draw these items
        pdf_items(c, width, height, pos, 0, items_on_page-1, item_list, order, discount, show_vat)

        # max_space_per_page is the space without the totals box
        max_space_per_page = header_y - 80 - 21
        # max_number_of_items is the number that can fit in the max_space_per_page
        max_number_of_items = int(max_space_per_page // 20)

        # min_space_per_page is the space with the totals box
        min_space_per_page = header_y - 80 - 21 - 70
        # min_number_of_items is the number that can fit in the min_space_per_page
        min_number_of_items = int(min_space_per_page // 20)

        # How many items require to be printed
        items_left = number_of_items - items_on_page

        # count how many items are to be printed per page
        items_per_page = []


        while items_left>0:
            # next page
            if items_left <= min_number_of_items:
                # All items fit on this page, with a totals box
                items_per_page.append(items_left)
                break
            if items_left <= max_number_of_items:
                # All but one item fits, with no totals box, carry single item to next page
                items_per_page.append(items_left-1)
                items_per_page.append(1)
                break
            # items_left must be greater than will fit on a page    
            items_per_page.append(max_number_of_items)
            items_left -= max_number_of_items

        total_pages =  1 + len(items_per_page)

    ########## Page bottom

    # If the order is closed or cancelled, print over the bottom line
    c.setFont("Helvetica-Bold", 16)
    if order.is_cancelled:
        c.drawString(pos["left_margin"], 65, "ORDER CANCELLED")
    elif not order.is_open:
        c.drawString(pos["left_margin"], 65, "ORDER CLOSED")

    # Sets the bottom line with page numbers
    pdf_bottom(c, width, height, pos, total_pages)

    # End this page and if necessary start a new one
    c.showPage()


    ########## Do the subsequent pages

    if total_pages > 1:
        items_left = number_of_items - items_on_page
        for items in items_per_page:
            # so do this for each page
            pos["leftx"] = 0
            pos["lefty"] = height
            pos["rightx"] = width
            pos["righty"] = height
            # Sets the title and company logo and company name
            pdf_header(c, width, height, pos, "Purchase Order", order)
            # create a bit of space under the line
            pos["lefty"] -= 20
            # The date and purchase order number on the left
            pdf_date_number_box(c, width, height,
                                pos, "Purchase Order No:", order.datestamp, order)
            # Continued from previous page text message just beneath
            pos["lefty"] -= 20
            page_number = c.getPageNumber()
            previous_page_number = page_number - 1
            c.setFont("Helvetica-Bold", 8)
            c.drawString(pos["left_margin"], pos["lefty"],
                                "Continued from page %s" % previous_page_number)
            # table is to be lower than the lowest text so far
            if pos["lefty"] > pos["righty"]:
                    pos["lefty"]=pos["righty"]
            # Give a bit of space
            pos["lefty"] -= 6
            # display the grid of items
            pdf_items(c, width, height, pos,
                      number_of_items - items_left,
                      number_of_items - items_left + items -1,
                      item_list, order, discount, show_vat)
            items_left -= items
            if items_left < 1:
                # This is the last page
                # display the totals box
                pos["lefty"] -= 10
                pdf_totals(c, width, height, pos, order, show_vat)
                c.setFont("Helvetica", 8)
                if discount:
                    # display a bit of discount text
                    if show_vat:
                        lines = ["Costs and VAT are calculated after the",
                                 "discount for each item has been applied"]
                    else:
                        lines = ["Costs are calculated after the",
                                 "discount for each item has been applied"]
                    textobject = c.beginText(pos["left_margin"], pos["lefty"]-12)
                    textobject.textLines(lines)
                    c.drawText(textobject)

            # If the order is closed or cancelled, print over the bottom line
            c.setFont("Helvetica-Bold", 16)
            if order.is_cancelled:
                c.drawString(pos["left_margin"], 65, "ORDER CANCELLED")
            elif not order.is_open:
                c.drawString(pos["left_margin"], 65, "ORDER CLOSED")

            # Sets the bottom line with page numbers
            pdf_bottom(c, width, height, pos, total_pages)

            # End this page and if necessary start a new one
            c.showPage()

    # Do the full drawing to the canvas, and save the pdf file
    c.save()

    # save the directory to the per-user store
    directory = os.path.dirname(filepath)
    set_stored_item("ORDERS_PDF_FOLDER", directory)

    # save the page size to vs_cfg["LAST_ORDER_PDF_SIZE"] and the per-user store
    set_stored_item("LAST_ORDER_PDF_SIZE", (width, height))

    return True





