####### VATStuff #######
#
# page_setup.py  - Asks for pdf page size and orientation
#
# Version : 0.6
# Date : 20100806
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#

"""Shows a dialog asking for pdf page size and orientation"""


import wx

from vatstuff_packages.vs_utilities import *
from vatstuff_packages.vs_gui.gui_functions import *


def page_setup_dialog(parent, width=None, height=None):
    "Called when a page setup button is pressed, to create PageSetupDialog"
    
    if not vs_cfg["REPORTLABS_AVAILABLE"]:
        # no pdf capability
        vs_message(parent, "Function unavailable - the reportlab module is not installed",
                             "ERROR")
        return

    # get last report pdf width and height
    if (not width) or (not height):
        width, height = vs_cfg["LAST_REPORT_PDF_SIZE"]
    page_setup_dialog = PageSetupDialog(parent, width, height)
    if page_setup_dialog.ShowModal() == wx.ID_OK:
        vs_cfg["LAST_REPORT_PDF_SIZE"] = (page_setup_dialog.width,
                                          page_setup_dialog.height)          
    page_setup_dialog.Destroy()


class PageSetupDialog(vsDialog):
    """Called when a page setup button is pressed"""

    def __init__(self, parent, width, height):
        """Create the Dialog"""
        vsDialog.__init__(self, parent, "pdf page setup")

        button_size = (100, vs_cfg["BUTTON_HEIGHT"])

        self.width = width
        self.height = height

        # Create a vertical sizer to hold everything
        sizer=wx.BoxSizer(wx.VERTICAL)

        # show the help text
        sizer.Add(wx.StaticText(self, -1, "Set pdf page size and orientation"), 0, wx.ALL | wx.ALIGN_CENTER, 20)


        # Create choices in a grid sizer
        field_sizer=wx.FlexGridSizer(rows=2, cols=2, hgap=20, vgap=40)

        field_sizer.Add(wx.StaticText(self, -1, "Page size:"), 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        # The page size
        self.page_size_field = wx.Choice(self, size=(150, vs_cfg["BUTTON_HEIGHT"]), choices=("A4", "A3", "Letter"))
        self.page_size_field.SetToolTipString("The page size")
        field_sizer.Add(self.page_size_field, 0, wx.ALIGN_LEFT)

        field_sizer.Add(wx.StaticText(self, -1, "Orientation:"), 0, wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL)

        # The orientation
        self.orientation_field = wx.Choice(self, size=(150, vs_cfg["BUTTON_HEIGHT"]), choices=("Portrait", "Landscape"))
        self.orientation_field.SetToolTipString("Set portrait or landscape")
        field_sizer.Add(self.orientation_field, 0, wx.ALIGN_LEFT)

        sizer.Add(field_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 40)

        # pre-set the selections
        if (self.height, self.width) == vs_cfg["A4"]:
            # A4 landscape
            self.page_size_field.SetSelection(0)
            self.orientation_field.SetSelection(1)
        elif (self.width, self.height) == vs_cfg["A3"]:
            # A3 portrait
            self.page_size_field.SetSelection(1)
            self.orientation_field.SetSelection(0)
        elif (self.height, self.width) == vs_cfg["A3"]:
            # A3 landscape
            self.page_size_field.SetSelection(1)
            self.orientation_field.SetSelection(1)
        elif (self.width, self.height) == vs_cfg["LETTER"]:
            # letter portrait
            self.page_size_field.SetSelection(2)
            self.orientation_field.SetSelection(0)
        elif (self.height, self.width) == vs_cfg["LETTER"]:
            # letter landscape
            self.page_size_field.SetSelection(2)
            self.orientation_field.SetSelection(1)
        else:
            # A4 portrait - the default
            self.page_size_field.SetSelection(0)
            self.orientation_field.SetSelection(0)

        # ---------------------- Apply - Cancel -- buttons

        # Create a horizontal button_sizer to hold the buttons
        button_sizer=wx.BoxSizer(wx.HORIZONTAL)

        # Apply button
        self.apply_button = wx.Button(self, wx.ID_APPLY, size=button_size)
        self.apply_button.SetToolTipString("Click to set the page size.")
        button_sizer.Add(self.apply_button, 0, wx.RIGHT, 40)

        # Cancel button
        self.cancel_button = wx.Button(self, wx.ID_CANCEL, size=button_size)
        self.cancel_button.SetToolTipString("Click to cancel.")
        button_sizer.Add(self.cancel_button, 0)

        # Add button_sizer to the main sizer
        sizer.Add(button_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 20)

        # Set the size to the frame
        self.SetSizer(sizer)
        sizer.SetSizeHints(self)

        # ---------- Bind buttons to handler functions
        
        self.Bind(wx.EVT_BUTTON, self.apply_button_pressed, self.apply_button)
        self.Bind(wx.EVT_BUTTON, self.cancel_button_pressed, self.cancel_button)


    # ----------------- Handler functions

    @button_wrap
    def apply_button_pressed(self, event):
        """ Apply button - check and apply all fields"""
        page_type = self.page_size_field.GetCurrentSelection()
        page_orientation = self.orientation_field.GetCurrentSelection()

        if page_type == 0 and page_orientation == 1:
            # A4 landscape
            self.height, self.width = vs_cfg["A4"]
        elif page_type == 1 and page_orientation == 0:
            # A3 portrait
            self.width, self.height = vs_cfg["A3"]
        elif page_type == 1 and page_orientation == 1:
            # A3 landscape
            self.height, self.width = vs_cfg["A3"]
        elif page_type == 2 and page_orientation == 0:
            # Letter portrait
            self.width, self.height = vs_cfg["LETTER"]
        elif page_type == 2 and page_orientation == 1:
            # Letter landscape
            self.height, self.width = vs_cfg["LETTER"]
        else:
            # A4 portrait is the default
            self.width, self.height = vs_cfg["A4"]

        self.EndModal(wx.ID_OK)


    def cancel_button_pressed(self, event):
        "Cancel button - return wx.ID_CANCEL"
        self.EndModal(wx.ID_CANCEL)


