####### VATStuff #######
#
# pdf_functions.py  - Functions to create pdf files using the reportlab library
#
# Version : 0.6
# Date : 20100808
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#


"Functions to create pdf files using the reportlab library"



import os
import subprocess

from vatstuff_packages.vs_utilities import *
from vatstuff_packages.vs_gui.gui_functions import *


def show_pdf_file(filepath):
    "Attempts to open the system default pdf viewer, returns True on success, False on failure"

    # Check filepath ends in .pdf
    if filepath[-4:] != ".pdf":
        return False

    if os.name == "nt":
        try:
            os.startfile(filepath)
        except Exception, e:
            exception_log(e)
            # return False, presumably the system has no pdf viewer
            return False
        return True


    # non-Windows, try calling different methods until one suceeds
    try:
        returncode = subprocess.call(["/usr/bin/xdg-open", filepath])
    except Exception:
        try:
            returncode = subprocess.call(["/usr/bin/gnome-open", filepath])
        except Exception:
            try:
                returncode = subprocess.call(["/usr/bin/kde-open", filepath])               
            except Exception:
                try:
                    returncode = subprocess.call(["/usr/bin/open", filepath])
                except Exception:
                    # return False, presumably the system has no pdf viewer
                    return False
    if returncode:
        return False
    return True


def pdf_header(canvas, width, height, pos, header_text, obj=None):
    "Draws top left header text, line, and top right logo with company registered address"

    # Company details
    if obj:
        if hasattr(obj, 'company'):
            company = obj.company
        else:
            company = Company()
        if hasattr(obj, 'print_check'):
            print_check = obj.print_check
        else:
            print_check = False
    else:
        company = Company()
        print_check = False

    if not ( company and company.registered_address and company.name ):
        # if no company has yet been set - do not print any company name or logo
        pos["righty"] = height-80
        pos["lefty"] = height-80
        pos["leftx"] = pos["left_margin"]
        pos["rightx"] = width-pos["column_width"]-pos["right_margin"]
        return

    company_name         = company.name
    company_address      = company.registered_address.lines
    country_registration = company.country_registration
    company_number       = company.company_number
    vat_regnumber        = company.vat_regnumber
    vat_country_code     = company.vat_country_code

    c = canvas
    c.setAuthor(company_name)
    c.setTitle(header_text)
    if obj and hasattr(obj, 'number'):
        c.setSubject("Number: %s" % obj.number)
    else:
        c.setSubject("%s report" % vs_cfg["PROGRAM_ID"])

    # Put the header text at the top left

    leftx = pos["left_margin"]
    lefty = height-60
    rightx = width-pos["right_margin"]
    righty = height-60

    c.setFont("Helvetica", 16)
    c.drawString(leftx, lefty, header_text)

    logo = False
    if vs_cfg["PIL_AVAILABLE"] and os.path.exists(vs_cfg["LOGO_FILEPATH"]):
        # logo exists, and can be handled by PIL, draw it at the top right
        rightx -= vs_cfg["LOGO_MAXWIDTH"]
        righty = height-20-vs_cfg["LOGO_MAXHEIGHT"]
        # note: str(vs_cfg["LOGO_FILEPATH"]) used here as the drawImage
        # errors if vs_cfg["LOGO_FILEPATH"] is unicode
        c.drawImage(str(vs_cfg["LOGO_FILEPATH"]), rightx, righty)
        logo = True
    else:
        # No logo, put company name, top right above the title line
        c.setFont("Helvetica-Bold", 12)
        string_width  = c.stringWidth(company_name, "Helvetica-Bold", 12)
        if string_width < pos["column_width"]:
            rightx -= pos["column_width"]
        else:
            rightx -= string_width

        c.drawString(rightx, righty, company_name)

    # Draw top line, from left margin to just short of rightx as calculated above
    # and six points below the header text (actually four, as the line is two thick) 
    c.setLineWidth(2)

    lefty -= 6
    c.line(leftx, lefty, rightx-4, lefty)

    # The company address will start at righty
    # set righty to the smaller of righty or lefty, and below them by ten points

    if righty>lefty:
        righty = lefty-10
    else:
        righty -= 10

    # Start the address at the column edge
    rightx = width-pos["column_width"]-pos["right_margin"]

    # Put in company registered address

    if logo:
        address_list=[company_name]
    else:
        address_list=[]

    for index,line in enumerate(company_address):
        if line == "": continue
        if index == 5 and print_check == False: continue
        address_list.append(line)
    address_list.append("")
    if company_number:
        address_list.append("Registered in %s No. %s" % (country_registration, company_number))

    if vat_regnumber :
        address_list.append("VAT No. %s %s" % (vat_country_code, vat_regnumber))

    c.setFont("Helvetica", 8)

    textobject = c.beginText(rightx, righty)
    textobject.textLines(address_list)
    c.drawText(textobject)
    pos["leftx"] = leftx
    pos["lefty"] = lefty
    pos["rightx"], pos["righty"] = textobject.getCursor()
    # The pos["righty"] is one blank line below the textobject
    # that is - at the position of where the next line would start if one were to be drawn


def pdf_to_address(canvas, width, height, pos, name, address, obj):
    """To address 25mm (71 points) from left, 55mm (156 points) from top
       to appear in the window of a DL envelope"""

    c = canvas
    x = 71
    y = height-156-12

    # The -12 is for the first line height

    # Organisation name
    address_list = [name]

    if hasattr(obj, 'print_check'):
        print_check = obj.print_check
    else:
        print_check = False

    # Get address
    if address:
        for index,line in enumerate(address.lines):
            if line == "": continue
            if index == 5 and print_check == False: continue
            address_list.append(line)

    c.setFont("Helvetica", 10)
    textobject = c.beginText(x, y)
    textobject.textLines(address_list)
    c.drawText(textobject)
    x, pos["lefty"] = textobject.getCursor()
    # The pos["lefty"] is one blank line below the textobject
    # that is - at the position of where the next line would start if one were to be drawn


def pdf_date_number_box(canvas, width, height, pos, text, date, obj):
    "draw a box with reference number and date"
    c = canvas
    date_text = date.strftime("%d %b %Y")

    x = pos["left_margin"]
    y = pos["lefty"]

    c.setFont("Helvetica", 8)
    c.setLineWidth(1)

    c.drawString(x+10, y-12, "Date:")
    c.drawString(x+90, y-12, text)
    c.drawString(x+10, y-32, date_text)
    c.drawString(x+90, y-32, str(obj.number))

    # Draw 2x2 grid
    c.grid([x, x+80, x+190], [y, y-20, y-40])
    pos["lefty"] = y-40


def pdf_two_dates_number_box(canvas, width, height, pos, text, date_issue, date_supply, obj):
    "draw a box with reference number and date of issue and date of supply, return (width, height) of box"
    c = canvas
    date_issue  = date_issue.strftime("%d %b %Y")
    date_supply = date_supply.strftime("%d %b %Y")

    x = pos["left_margin"]
    y = pos["lefty"]

    c.setFont("Helvetica", 8)
    c.setLineWidth(1)

    c.drawString(x+10, y-12, text)
    c.drawString(x+10, y-32, "Date of issue:")
    c.drawString(x+10, y-52, "Date of supply:")

    c.drawString(x+110, y-12, str(obj.number))
    c.drawString(x+110, y-32, date_issue)
    c.drawString(x+110, y-52, date_supply)

    # Draw 2x3 grid
    c.grid([x, x+100, x+180], [y, y-20, y-40, y-60])
    pos["lefty"] = y-60
    return (180, 60)


def pdf_address_box(canvas, width, height, pos, text, address, obj):
    "Show a box holding your address"
    if not address:return
    c = canvas

    x = pos["rightx"]
    y = pos["righty"]

    c.setFont("Helvetica", 8)

    c.drawString(x+10, y-12, text)

    # Get company name
    address_list = [obj.company.name]

    # Get address
    for index,line in enumerate(address.lines):
        if line == "": continue
        if index == 5 and obj.print_check == False: continue
        address_list.append(line)

    textobject = c.beginText(x+10, y-32)
    textobject.textLines(address_list)
    c.drawText(textobject)
    endx, endy = textobject.getCursor()

    c.grid([x, x+pos["column_width"]], [pos["righty"], pos["righty"]-20, endy])

    pos["righty"] = endy



def pdf_contact(canvas, width, height, pos, obj):
    "Show contact name and details"

    if not obj.contact: return

    c = canvas
    x = pos["rightx"]
    y = pos["righty"]

    c.setFont("Helvetica", 8)

    lines = ["In case of enquiry, please contact:", obj.contact.name]

    email = obj.contact.email
    phone = obj.contact.phone
    mobile = obj.contact.mobile
    
    if email:
        lines.append("Email : %s" % email)
    if phone:
        lines.append("Phone : %s" % phone)
    if mobile:
        lines.append("Mobile : %s" % mobile)

    textobject = c.beginText(x, y)
    textobject.textLines(lines)
    c.drawText(textobject)

    pos["rightx"], pos["righty"] = textobject.getCursor()


def pdf_items(canvas, width, height, pos, first, last, item_list, obj, discount, show_vat = True):
    "draw the item list from item_list[first] to item_list[last]"
    c = canvas

    x = pos["leftx"]
    y = pos["lefty"]-12 # Header text y value

    rightx = width-pos["right_margin"]-pos["column_width"]

    c.setFont("Helvetica-Bold", 8)

    # Grid values (add 10 for text position)
    qty_x = x
    desc_x = qty_x + 10 + c.stringWidth("Qty", "Helvetica-Bold", 8) + 10
    
    # unit price and remaining x values are calculated from the right of the page
    grid_right = width-pos["right_margin"]
    if show_vat:
        vat_x = grid_right - 60  # pick a vat column width of 60
    else:
        vat_x = grid_right       # no vat column

    if discount:
        dis_x = vat_x - 70       # pick a discount column of width 70
    else:
        dis_x = vat_x            # No discount column

    amnt_x = dis_x - 80          # amount column of width 80

    unitp_x = amnt_x - 80        # unit price column of width 80

    # create a list of x values, used for drawing the grid
    x_values = [ x, qty_x, desc_x, unitp_x, amnt_x ]
    if discount:
        x_values.append(dis_x)
    if show_vat:
        x_values.append(vat_x)
    x_values.append(grid_right)

    c.drawString(qty_x+10, y, "Qty")
    c.drawString(desc_x+10, y, "Description")
    c.drawString(unitp_x+10, y, "Unit price")
    c.drawString(amnt_x+10, y, "Amount")
    if discount:
        c.drawString(dis_x+10, y, "Discount")
    if show_vat:
        c.drawString(vat_x+10, y, "VAT %")

    if len( item_list ) == 0:
        # Create a table grid of header plus one empty line
        # shows the user there has been nothing selected
        c.grid(x_values, [ pos["lefty"], pos["lefty"]-20, pos["lefty"]-40 ])
        pos["lefty"] -= 40
        return

    amount_list = obj.get_amount()
    if discount:
        discount_string = str(obj.discount).replace(".", vs_cfg["CP"])
        discount_string = "%s%%" % discount_string

    c.setFont("Helvetica", 8)

    for index in range(first, last+1):
        y -= 20
        item = item_list[index]

        quantity_string = str(item["Qty"]).replace(".", vs_cfg["CP"])
        c.drawRightString(desc_x-10, y, quantity_string)

        c.drawString(desc_x+10, y, item["Description"])

        unit_cost_string = to_currency_string(item["Unit Cost"])
        c.drawRightString(amnt_x-10, y, unit_cost_string)

        amount_string = to_currency_string(amount_list[index])
        c.drawRightString(dis_x-10, y, amount_string)

        if discount and item["Discount"]:
            c.drawRightString(vat_x-10, y, discount_string)

        if show_vat:
            vat_rate_string = "%s%%" % str(item["VAT"]).replace(".", vs_cfg["CP"])
            c.drawRightString(grid_right-10, y, vat_rate_string)

    # Create grid
    y_values = [ pos["lefty"] - 20*Y for Y in range(last-first+3) ]

    x_values = [ x, qty_x, desc_x, unitp_x, amnt_x ]
    if discount:
        x_values.append(dis_x)
    if show_vat:
        x_values.append(vat_x)
    x_values.append(grid_right)

    c.grid(x_values, y_values)
    pos["lefty"] = y_values[-1]


def pdf_totals(canvas, width, height, pos, obj, show_vat = True):
    "Create Totals box, two cols of 70 and 120, height 60 = three rows of 20"

    c = canvas

    x = pos["rightx"]
    y = pos["lefty"]-12



    if show_vat:
        # Show cost and VAT 
        c.setFont("Helvetica", 8)
        c.drawString(x+10, y, "Cost")
        cost_string = to_currency_string(obj.cost)
        c.drawRightString(x+pos["column_width"]-10, y, cost_string)
        y -= 20
        c.drawString(x+10, y, "VAT")
        vat_string = to_currency_string(obj.vat)
        c.drawRightString(x+pos["column_width"]-10, y, vat_string)
        y -= 20

    # Show total
    c.setFont("Helvetica-Bold", 8)
    c.drawString(x+10, y, "Total")

    total = obj.vat + obj.cost
    total_string = to_currency_string(total)
    c.drawRightString(x+pos["column_width"]-10, y, total_string)

    # Create table grid
    if show_vat:
        c.grid([x, x+70, x+pos["column_width"]],[pos["lefty"], pos["lefty"]-20, pos["lefty"]-40, pos["lefty"]-60])
    else:
        c.grid([x, x+70, x+pos["column_width"]],[pos["lefty"], pos["lefty"]-20])


def pdf_vatbox(canvas, width, height, pos, vat_rate_list, item_list, obj):
    "Draw the box of VAT rates"

    c = canvas

    c.setFont("Helvetica", 8)

    x = pos["leftx"]
    y = pos["lefty"]-12

    c.drawString(x+10, y, "VAT Rates")
    c.drawString(x+80, y, "VAT Amount")

    obj_discount = obj.discount

    for rate in vat_rate_list:
        y -= 20

        rate_string = "%s%%" % str(rate).replace(".", vs_cfg["CP"])
        c.drawString(x+10, y, rate_string)

        # calculate VAT amount for all items using this VAT rate
        vat_amount = D("0.00")
        for item in item_list:
            if rate == item["VAT"]:
                row_cost = item["Qty"] * item["Unit Cost"]
                row_cost = row_cost.quantize(D('.01'), rounding=decimal.ROUND_DOWN)
                # Apply discount
                if item["Discount"]:
                    row_cost = row_cost - ( row_cost * obj_discount ) / D("100.000")
                    # so after the discount applied, round it to the nearest penny
                    row_cost = row_cost.quantize(D('.01'), rounding=decimal.ROUND_HALF_EVEN)
                # Calculate the vat of this cost
                result = (row_cost * item["VAT"]) / D("100.0000")
                vat_amount = vat_amount + result
        # So now have the vat amount for this rate
        if len(vat_rate_list) == 1:
            # If only one rate, show it to penny precision so it is the same
            # as in the VAT totals box
            vat_amount = vat_amount.quantize(D('.01'), rounding=decimal.ROUND_HALF_EVEN)
        else:
            # If multiple rates, show them to one tenth of a penny
            vat_amount = vat_amount.quantize(D('.001'), rounding=decimal.ROUND_HALF_EVEN)
        # Print it
        amount_string = vs_cfg["CS"] + str(vat_amount).replace(".", vs_cfg["CP"])
        c.drawRightString(x+140, y, amount_string)

    x_values = [x, x+70, x+150]
    y_values = [ pos["lefty"]-(y*20) for y in range(len(vat_rate_list)+2) ]

    c.grid(x_values,y_values)


def pdf_bottom(canvas, width, height, pos, total_pages):
    "Draw bottom line with a page number"
    c = canvas
    page_number = c.getPageNumber()

    if page_number < total_pages:
        next_page_number = page_number + 1
        next_page_number_string = "Continued on Page %s" % next_page_number
        c.setFont("Helvetica-Bold", 12)
        string_width  = c.stringWidth(next_page_number_string, "Helvetica-Bold", 12)
        c.drawString(width-pos["right_margin"]-string_width, 66, next_page_number_string)

    c.setLineWidth(2)
    c.line(pos["left_margin"], 58, width-pos["right_margin"], 58)
    c.setFont("Helvetica", 8)
    c.drawString(pos["left_margin"], 40, "Page %s of %s" % (page_number, total_pages))



