# test_accounts.py  - tests creating, deleting and editing accounts
#
# Version : 0.7.0
# Date : 20100927
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#


"""This particular module exercises Accounts creation functions"""


import unittest
import os
import os.path
import datetime
import decimal

from vatstuff_packages.vs_utilities import config
from vatstuff_packages.vs_utilities import objects
from vatstuff_packages.vs_utilities import odd_functions
from vatstuff_packages.vs_database import create_objects
from vatstuff_packages.vs_database import transactions
from vatstuff_packages.vs_verify.accounts import (new_account_check,
                                new_folder_check,
                                edit_folder_check,
                                edit_account_check,
                                delete_folder_check,
                                delete_account_check)
from vatstuff_packages.vs_verify import form_objects


# Create the unittest class

class TestAccounts(unittest.TestCase):
    """A test class for accounts.py"""

    def test_01_new_account(self):
        "new account"

        datestamp = datetime.date.today()
        name = "test account"
        description = "test account description (test_accounts.py)"

        account_type_selected = objects.AccountType.from_name("No_such_type")    
        self.assertFalse(account_type_selected)
        account_type_selected = objects.AccountType.from_name("Asset") 
        self.assertTrue(account_type_selected)
                
        folder_selected = objects.Folder.from_name("No_such_folder")
        self.assertFalse(folder_selected)
        folder_selected = objects.Folder.from_name("Bank")
        self.assertTrue(folder_selected)

        balance = decimal.Decimal("0.00")

        # Check input details are ok
        # Check no account name is caught
        status, message = new_account_check("", folder_selected, balance, datestamp)
        self.assertFalse(status)
        # Check name already exists is caught
        status, message = new_account_check("Current Account", folder_selected, balance, datestamp)
        self.assertFalse(status)
        status, message = new_account_check(name, folder_selected, balance, datestamp)
        self.assertTrue(status)

        account = create_objects.create_Account(folder_selected, name, description, 0, account_type_selected)
        # check the account has been created
        self.assertEqual(account.description, description)
        self.assertEqual(account.folder, folder_selected)
        self.assertEqual(account.name, name)
        self.assertEqual(account.acc_type, account_type_selected)


    def test_02_edit_account(self):
        "account edit"

        datestamp = datetime.date.today()
        old_folder = objects.Folder.from_name("Bank")

        # account to test
        full_name = "%s:%s" % ("Bank", "test account")

        # new details
        account_name = "test expense"
        folder_name = "Expenses"
        description = "test account (test_accounts.py)"

        account_objects_list, account_names_list = odd_functions.get_folder_account_lists()
        self.assertTrue(full_name in account_names_list)
        i = account_names_list.index(full_name)
        account = account_objects_list[i]

        account_type_selected = objects.AccountType.from_name("Expense") 
        self.assertTrue(account_type_selected)
                
        folder_selected = objects.Folder.from_name(folder_name)
        self.assertTrue(folder_selected)

        balance = decimal.Decimal("0.00")

        # Check input details are ok
        # Check no account name is caught
        status, message = edit_account_check(account, "", folder_selected, balance, datestamp)
        self.assertFalse(status)
        # Check name already exists in old folder is caught
        status, message = edit_account_check(account, "Current Account", old_folder, balance, datestamp)
        self.assertFalse(status)
        # Check ok if name is changing to something unique in old folder
        status, message = edit_account_check(account, "test account 123", old_folder, balance, datestamp)
        self.assertTrue(status)
        # Check ok if name is not changing in old folder
        status, message = edit_account_check(account, "test account", old_folder, balance, datestamp)
        self.assertTrue(status)
        # Check name already exists in new folder is caught
        status, message = edit_account_check(account, "Bank charges", folder_selected, balance, datestamp)
        self.assertFalse(status)
        # Check ok for a move to new folder
        status, message = edit_account_check(account, account_name, folder_selected, balance, datestamp)
        self.assertTrue(status)

        # Do the changes
        account.acc_type = account_type_selected
        account.name = account_name
        account.description = description
        account.folder = folder_selected

        # check changes are in place
        self.assertEqual(account.description, description)
        self.assertEqual(account.folder, folder_selected)
        self.assertEqual(account.name, account_name)
        self.assertEqual(account.acc_type, account_type_selected)


    def test_03_delete_account(self):
        "account delete"

        # Note - the test that delete_account_check(account) returns False if
        # the account has a transaction, is done in 'test_transactions.py'
        # after a transaction has been created

        # Note the test that delete_account_check(account) returns False if
        # the account is in a definition, is done in 'xxxxxxx'  (todo)

        # Check a system account cannot be deleted (5 = VAT Balance account)
        account = odd_functions.get_account_by_system(5)
        status, message = delete_account_check(account)
        self.assertFalse(status)

        # account to test
        account_name = "test expense"
        folder_name = "Expenses"
        full_name = "%s:%s" % (folder_name, account_name)
        account_objects_list, account_names_list = odd_functions.get_folder_account_lists()
        self.assertTrue(full_name in account_names_list)
        i = account_names_list.index(full_name)
        account = account_objects_list[i]

        # delete from the database
        status = account.delete()
        self.assertTrue(status)

        account_objects_list, account_names_list = odd_functions.get_folder_account_lists()
        self.assertTrue(full_name not in account_names_list)


    def test_04_new_folder(self):
        "new folder"

        name = "test folder"
        description = "test folder description (test_accounts.py)"

        # Check no folder name is caught
        status, message = new_folder_check("")
        self.assertFalse(status)
        # Check name already exists is caught
        status, message = new_folder_check("Customers")
        self.assertFalse(status)
        status, message = new_folder_check(name)
        self.assertTrue(status)

        folder = create_objects.create_Folder(name, description)
        # check folder has been created
        self.assertEqual(folder.description, description)
        self.assertEqual(folder.name, name)


    def test_05_edit_folder(self):
        "folder edit"

        # select a folder
        folder = objects.Folder.from_name("Bank")
        description = "Bank Folder - test_accounts.py"

        # Check a blank folder name is caught
        status, message = edit_folder_check(folder, "")
        self.assertFalse(status)
        # Check name already exists is caught
        status, message = edit_folder_check(folder, "Customers")
        self.assertFalse(status)
        status, message = edit_folder_check(folder, "Bank2")
        self.assertTrue(status)

        # Change folder details
        folder.name = "Bank2"
        folder.description = description

        # self.assertEqual(folder details have been changed
        self.assertEqual(folder.description, description)
        self.assertEqual(folder.name, "Bank2")

        # Change folder name back
        status, message = edit_folder_check(folder, "Bank")
        self.assertTrue(status)
        # Change folder details
        folder.name = "Bank"
        self.assertEqual(folder.name, "Bank")


    def test_06_delete_folder(self):
        "folder delete"

        # Check folder containing accounts cannot be deleted
        folder = objects.Folder.from_name("Bank")
        status, message = delete_folder_check(folder)
        self.assertFalse(status)

        # Check a system folder cannot be deleted
        folder = objects.Folder.from_name("System")
        status, message = delete_folder_check(folder)
        self.assertFalse(status)

        # Check a new folder can be deleted
        folder = objects.Folder.from_name("test folder")
        status, message = delete_folder_check(folder)
        self.assertTrue(status)

        # delete from the database
        status = folder.delete()
        self.assertTrue(status)

        folder = objects.Folder.from_name("test folder")
        self.assertFalse(folder)


    def test_07_account_types(self):
        "account types"

        # Get all account types, to set the new customer and supplier types

        customer_account_type = objects.AccountType(int(odd_functions.get_config_item("CUSTOMER_ACCOUNT_TYPE_ID")))
        supplier_account_type = objects.AccountType(int(odd_functions.get_config_item("SUPPLIER_ACCOUNT_TYPE_ID")))

        self.assertEqual(customer_account_type.name, "Asset")
        self.assertEqual(supplier_account_type.name, "Liability")

        # Change types
        customer_type_name = "Expense"
        supplier_type_name = "Income"

        customer_type = None
        supplier_type = None

        all_accounttypes = objects.AccountType.get_all()
        for acc_type in all_accounttypes:
            if customer_type_name == acc_type.name:
                customer_type = acc_type
            if supplier_type_name == acc_type.name:
                supplier_type = acc_type

        self.assertTrue(customer_type)
        self.assertTrue(supplier_type)

        odd_functions.set_config_item("CUSTOMER_ACCOUNT_TYPE_ID", customer_type.id)
        odd_functions.set_config_item("SUPPLIER_ACCOUNT_TYPE_ID", supplier_type.id)

        customer_type = objects.AccountType(int(odd_functions.get_config_item("CUSTOMER_ACCOUNT_TYPE_ID")))
        supplier_type = objects.AccountType(int(odd_functions.get_config_item("SUPPLIER_ACCOUNT_TYPE_ID")))

        self.assertEqual(customer_type.name, "Expense")
        self.assertEqual(supplier_type.name, "Income")

        odd_functions.set_config_item("CUSTOMER_ACCOUNT_TYPE_ID", customer_account_type.id)
        odd_functions.set_config_item("SUPPLIER_ACCOUNT_TYPE_ID", supplier_account_type.id)

        customer_type = objects.AccountType(int(odd_functions.get_config_item("CUSTOMER_ACCOUNT_TYPE_ID")))
        supplier_type = objects.AccountType(int(odd_functions.get_config_item("SUPPLIER_ACCOUNT_TYPE_ID")))

        self.assertEqual(customer_type.name, "Asset")
        self.assertEqual(supplier_type.name, "Liability")


    def test_08_transaction(self):
        """transactions.implement""" 

        transaction = form_objects.TransactionForm()
        transaction.system = 1
        transaction.datestamp = datetime.date.today()
        transaction.description = "Set current account opening balance"

        # get a list of accounts with system 0
        account_list = odd_functions.get_accounts_by_system_list([0])

        # pick the bank current account
        current_account = None
        for acc in account_list:
            if "Current Account" in acc.name:
                current_account = acc
                break

        self.assertTrue(current_account)

        # pick the openning balance account
        openning_account = odd_functions.get_account_by_system(2)
        self.assertTrue(openning_account)

        # Test the current account could be deleted (as it has system 0)
        status, message = delete_account_check(current_account)
        self.assertTrue(status)
        # Test the openning balance account can not be deleted (as it has system 2)
        status, message = delete_account_check(openning_account)
        self.assertFalse(status)

        # get the balance of each
        current_account_balance = current_account.balance
        openning_account_balance = openning_account.balance

        self.assertEqual(current_account_balance, decimal.Decimal("0.00"))
        self.assertEqual(openning_account_balance, decimal.Decimal("0.00"))

        # Set up transaction
        transaction.add(current_account, debit = decimal.Decimal("2000.00"))
        transaction.add(openning_account, credit = decimal.Decimal("2000.00"))

        status, message = transaction.check()
        self.assertTrue(status)

        transaction_id = transactions.implement(None, transaction)

        # get new account balances
        new_current_account_balance = current_account.balance
        new_openning_account_balance = openning_account.balance

        self.assertEqual(new_current_account_balance, current_account_balance + decimal.Decimal("2000.00"))
        self.assertEqual(new_openning_account_balance, openning_account_balance - decimal.Decimal("2000.00"))

        database_transaction = objects.Transaction(transaction_id)

        self.assertEqual(transaction.description, database_transaction.description)

        self.assertTrue(database_transaction in current_account)
        self.assertTrue(database_transaction in openning_account)

        # test the current account now has system values of 1
        self.assertEqual(current_account.system, 1)

        # Test the current account cannot be now deleted
        status, message = delete_account_check(current_account)
        self.assertFalse(status)

        # Test the last ledger entry of the current account holds this transaction
        last_gridrow = len(current_account)-1
        self.assertTrue(last_gridrow >= 0)
        self.assertEqual(current_account[last_gridrow][0], database_transaction)

        # Test the last ledger entry of the openning account holds this transaction
        last_gridrow = len(openning_account)-1
        self.assertTrue(last_gridrow >= 0)
        self.assertEqual(openning_account[last_gridrow][0], database_transaction)


    ##### These following tests check the account ledgers add up
    ##### they can be repeated in other test modules if further
    ##### transactions have been implemented

    def test_09_totals(self):
        """accounts balance"""
        # Get a list of all accounts
        all_accounts = objects.Account.get_all()

        # get all the account balances, and as this is double entry, check they add to zero
        total = decimal.Decimal("0.00")
        for account in all_accounts:
            total += account.balance
        self.assertEqual(total, decimal.Decimal("0.00"))


    def test_10_ledgers(self):
        """ledgers balance"""
        # Get a list of all accounts
        all_accounts = objects.Account.get_all()

        for account in all_accounts:
            debit = decimal.Decimal("0.00")
            credit = decimal.Decimal("0.00")
            for statement in account:
                # statement is each line of the account ledger
                # which is a list of [transaction, debit, credit, balance]
                debit += statement[1]
                credit += statement[2]
                self.assertEqual(debit-credit, statement[3])
            # and self.assertEqual(the final ledger balance is equal to that held
            # in the ACCOUNT table
            self.assertEqual(debit-credit, account.balance)


def suite():
    return unittest.TestLoader().loadTestsFromTestCase(TestAccounts)
