# test_objects.py  - tests on the objects.py module
#
# Version : 0.7.0
# Date : 20100927
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#


"""Exercises the functions in objects.py"""


import unittest
import os
import os.path
import sqlite3

from vatstuff_packages.vs_utilities import config
from vatstuff_packages.vs_utilities import objects


# Create the unittest class

class TestObjects(unittest.TestCase):
    """A test class for objects.py"""

    def test_01_select(self):
        "objects.select"
        company = objects.Company()
        sql_string = "SELECT COMPANY_NAME FROM COMPANIES WHERE COMPANY_ID = ?"
        name = objects.select(sql_string, (company.id,))
        self.assertEqual(name,"Test Company Ltd")

        # Check a result which should return None, i.e a non-existing company id
        name = objects.select(sql_string, (-1,))
        self.assertEqual(name, None)

        # Check an exception is raised, for a non-existing field
        sql_string = "SELECT XYZ FROM COMPANIES WHERE COMPANY_ID = ?"""
        self.assertRaises(sqlite3.OperationalError, objects.select, sql_string, (company.id,))


    def test_02_list_select(self):
        "objects.list_select"
        # Get a list of folder names
        sql_string = "SELECT NAME FROM FOLDERS"
        name_list = objects.list_select(sql_string)
        self.assertTrue("System" in name_list)
        self.assertTrue("Customers" in name_list)
        self.assertTrue("Suppliers" in name_list)
        self.assertTrue("XYZ" not in name_list)


    def test_03_update(self):
        "objects.update"
        # Get the folder object with name system
        folder_id  = objects.select("SELECT FOLDER_ID FROM FOLDERS WHERE NAME = ?", ("System",))
        system_folder = objects.Folder(folder_id)
        # Use update to change its description
        sql_string = "UPDATE FOLDERS SET DESCRIPTION = ? WHERE FOLDER_ID = ?"
        objects.update(sql_string, ("Test", folder_id))
        self.assertEqual(system_folder.description, "Test")
        # Change again
        objects.update(sql_string, ("Test123", folder_id))
        self.assertEqual(system_folder.description, "Test123")
        # Prove an exception occurs for a non-existent field
        sql_string = "UPDATE FOLDERS SET XYZ = ? WHERE FOLDER_ID = ?"
        self.assertRaises(sqlite3.OperationalError, objects.update, sql_string, ("Test", folder_id))
        # and check the database is still readable
        self.assertEqual(system_folder.description, "Test123")


    def test_04_insert(self):
        "objects.insert"
        folder_id = objects.insert("""INSERT INTO FOLDERS 
                                     (FOLDER_ID, NAME, DESCRIPTION, SYSTEM, EXPANDED) VALUES (?, ?, ?, ?, ?)""",
                                      (None, "Test Folder", "Test Folder", 0, 0))
        folder = objects.Folder(folder_id)
        self.assertEqual(folder.name, "Test Folder")
        self.assertEqual(folder.description, "Test Folder")
        # Prove an exception occurs for a non existent field
        sql_string = """INSERT INTO FOLDERS 
                        (FOLDER_ID, XYZ, DESCRIPTION, SYSTEM, EXPANDED) VALUES (?, ?, ?, ?, ?)"""
        self.assertRaises(sqlite3.OperationalError, objects.insert, sql_string, (None, "Test Folder", "Test Folder", 0, 0))
        # and check the database is still readable
        self.assertEqual(folder.description, "Test Folder")
        # delete the inserted folder from the database
        status = folder.delete()
        self.assertTrue(status)


    def test_05_get_column(self):
        "objects get_column"
        # Get the folder object with name system
        folder_id  = objects.select("SELECT FOLDER_ID FROM FOLDERS WHERE NAME = ?", ("System",))
        system_folder = objects.Folder(folder_id)
        # Use the get_column method
        system_folder_name = system_folder.get_column("NAME")
        self.assertEqual(system_folder_name, "System")
        # Prove an exception occurs for a non existent field
        self.assertRaises(sqlite3.OperationalError, system_folder.get_column, "NO_FIELD")


    def test_06_set_column(self):
        "objects set_column"
        # Get the folder object with name system
        folder_id  = objects.select("SELECT FOLDER_ID FROM FOLDERS WHERE NAME = ?", ("System",))
        system_folder = objects.Folder(folder_id)
        # Use the set_column method
        system_folder.set_column("DESCRIPTION", "Set column test")
        self.assertEqual(system_folder.get_column("DESCRIPTION"), "Set column test")
        # Prove an exception occurs for a non existent field
        self.assertRaises(sqlite3.OperationalError, system_folder.set_column, "NO_FIELD", "False data")


def suite():
    return unittest.TestLoader().loadTestsFromTestCase(TestObjects)


