# test_purchases.py  - run tests on purchases functions
#
# Version : 0.7.0
# Date : 20100928
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#


"""exercises functions under the Purchases tab"""

import unittest
import os
import os.path
import datetime
import decimal


from vatstuff_packages.vs_utilities import config
from vatstuff_packages.vs_utilities import objects
from vatstuff_packages.vs_utilities import odd_functions
from vatstuff_packages.vs_database import (save_purchaseorder,
                                           save_closeorder,
                                           save_supplierinvoice,
                                           save_makepayment)
from vatstuff_packages.vs_database.create_objects import create_Supplier, create_Address, create_PurchaseItem
from vatstuff_packages.vs_verify.form_objects import TransactionForm, SupplierForm, OrderForm, SupplierInvoiceForm, MakePaymentForm
from vatstuff_packages.vs_verify.common import check_supplier
from vatstuff_packages.vs_verify.purchases import (new_purchaseorder_check,
                                 received_invoice_check,
                                 make_payment_check)

D = decimal.Decimal   # D used to avoid typing


# Create the unittest class

class TestPurchases(unittest.TestCase):
    """A test class for vatstuff purchases"""


    def test_01_new_supplier(self):
        "twenty suppliers"

        # supplier_form is the data normally gathered
        # from a GUI form
        supplier_form = SupplierForm()
        supplier_form.ec_country = True
        supplier_form.vat_country_code = "GB"
        supplier_form.vat_registered = True
        supplier_form.discount = decimal.Decimal("0.0")


        # Create 20 new suppliers
        s = 0
        while s<20:
            supplier_form.name = "Test Supplier " + str(s)
            supplier_form.vat_regnumber = "00000" + str(s)

            # Check supplier_form object
            status, message = check_supplier(supplier_form)
            self.assertTrue(status)

            new_supplier = create_Supplier(supplier_form)
            self.assertTrue(new_supplier)

            line_1 = "Supplier " + str(s) + " address line 1" 

            database_address = create_Address(customer=None,
                                              supplier=new_supplier,
                                              employee=None,
                                              contractor=None,
                                              line1 = line_1,
                                              line2 = "Test line2",
                                              city = "Test City",
                                              county = "Test County",
                                              postcode = "TST1 1TST",
                                              country = "England",
                                              reg_office = 1,
                                              delivery_point = 0,
                                              invoice_point = 0,
                                              order_point = 1,
                                              payment_point = 1,
                                              edit_new = False,
                                              visible = True,
                                              system = 0)

            self.assertTrue(database_address)
            s += 1


    def test_02_edit_supplier(self):
        "changes to supplier"

        all_suppliers = objects.Supplier.get_all()
        this_supplier = all_suppliers[0]
        supplier_form = SupplierForm()

        # copy this_supplier attributes into the supplier_form object
        supplier_form.copy_from(this_supplier)


        # supplier_form is the data normally gathered
        # from a GUI form - change the vat_regnumber
        supplier_form.vat_regnumber = "11112222"

        # Check supplier_form object

        status, message = check_supplier(supplier_form)
        # This should fail, as the check considers this to be a
        # new supplier with the same name as an existing one
        self.assertFalse(status)

        status, message = check_supplier(supplier_form, this_supplier)
        # This should pass as the check now knows this_supplier
        # is being edited
        self.assertTrue(status)

        # save vat registration number
        this_supplier.vat_regnumber = supplier_form.vat_regnumber

        self.assertEqual(this_supplier.vat_regnumber, "11112222")


    def test_03_add_purchase_item(self):
        "add purchase item"
        purchase_item = create_PurchaseItem(description     = "purchase_thing",
                                            cost            = decimal.Decimal("20.00"),
                                            vatrate         = objects.VATRate.from_symbol("S"),
                                            notes           = "A test item to purchase")
        self.assertEqual(purchase_item.description, "purchase_thing")
        self.assertEqual(purchase_item.cost, decimal.Decimal("20.00"))


    def test_04_purchase_order(self):
        "create orders"
        order = OrderForm()
        all_suppliers = objects.Supplier.get_all()

        for supplier in all_suppliers:

            order.supplier = supplier                                # The supplier to whom the order is being sent

            supplier_addresses = order.supplier.get_addresses()
            order.supplier_address = supplier_addresses[0]           # The suppliers address where the order is sent

            order.sup_ref_field = "001"                           # The suppliers reference

            company_addresses = order.company.get_addresses()
            order.invoice_address = company_addresses[0]             # Where invoices should be sent
            order.delivery_address = company_addresses[0]             # Where deliveries should be sent

            order.datestamp = datetime.date.today()              # Datestamp of the issue of this order

            last_order_number = odd_functions.get_config_item("LAST_ORDER_NUMBER")
            if not last_order_number:
                order.number = 1
            else:
                order.number =  int(last_order_number) + 1         # The order number

            all_items = objects.PurchaseItem.get_all()
            item = all_items[0]

            order.item_list = [{"Qty":D(1),
                                "Description": item.description,
                                "Unit Cost":item.cost,
                                "Discount": False,
                                "VAT":item.vatrate.rate}]

            order.cost = item.cost         # Total cost (net - exclusive of VAT)

            result = (order.cost * item.vatrate.rate) / D("100.0000")
            order.vat = result.quantize(D('.01'), rounding=decimal.ROUND_HALF_EVEN)

            # Check these input order details are ok via vatstuff_packages.vs_verify.purchases
            status, message = new_purchaseorder_check(order)
            self.assertTrue(status)

            transaction = TransactionForm()
            transaction.system = 2
            transaction.datestamp = datetime.date.today()
            transaction.description = "Purchase order"

            # Debit the purchase account by the cost plus vat of the order
            transaction.add(odd_functions.get_account_by_system(7), debit = order.cost + order.vat)

            # Credit the suppliers account by the cost plus VAT of the order
            transaction.add(order.supplier.account, credit = order.cost + order.vat)

            status, message = transaction.check()
            self.assertTrue(status)

            # Must now implement the transaction and save the Order
            database_order = save_purchaseorder.save(order, transaction)

            self.assertEqual(database_order.cost, order.cost)
            self.assertEqual(database_order.vat, order.vat)

            # compare the database transaction
            database_transaction = database_order.transaction
            self.assertEqual(transaction.description, database_transaction.description)
            self.assertEqual(database_transaction.system, 2)
            self.assertEqual(len(database_transaction.trans_list), 2)


    def test_05_cancel_order(self):
        "create an order and then cancel it"
        order = OrderForm()
        all_suppliers = objects.Supplier.get_all()
        supplier = all_suppliers[0]  # get first supplier

        order.supplier = supplier                                # The supplier to whom the order is being sent

        supplier_addresses = order.supplier.get_addresses()
        order.supplier_address = supplier_addresses[0]           # The suppliers address where the order is sent

        order.sup_ref_field = "002"                           # The suppliers reference

        company_addresses = order.company.get_addresses()
        order.invoice_address = company_addresses[0]             # Where invoices should be sent
        order.delivery_address = company_addresses[0]             # Where deliveries should be sent

        order.datestamp = datetime.date.today()              # Datestamp of the issue of this order

        last_order_number = odd_functions.get_config_item("LAST_ORDER_NUMBER")
        if not last_order_number:
            order.number = 1
        else:
            order.number =  int(last_order_number) + 1         # The order number

        all_items = objects.PurchaseItem.get_all()
        item = all_items[0]

        order.item_list = [{"Qty":D(1),
                            "Description": item.description,
                            "Unit Cost":item.cost,
                            "Discount": False,
                            "VAT":item.vatrate.rate}]

        order.cost = item.cost         # Total cost (net - exclusive of VAT)

        result = (order.cost * item.vatrate.rate) / D("100.0000")
        order.vat = result.quantize(D('.01'), rounding=decimal.ROUND_HALF_EVEN)

        # Check these input order details are ok via vatstuff_packages.vs_verify.purchases
        status, message = new_purchaseorder_check(order)
        self.assertTrue(status)

        transaction = TransactionForm()
        transaction.system = 2
        transaction.datestamp = datetime.date.today()
        transaction.description = "Purchase order"

        # Debit the purchase account by the cost plus vat of the order
        transaction.add(odd_functions.get_account_by_system(7), debit = order.cost + order.vat)

        # Credit the suppliers account by the cost plus VAT of the order
        transaction.add(order.supplier.account, credit = order.cost + order.vat)

        status, message = transaction.check()
        self.assertTrue(status)

        # Must now implement the transaction and save the Order
        database_order = save_purchaseorder.save(order, transaction)

        self.assertEqual(database_order.cost, order.cost)
        self.assertEqual(database_order.vat, order.vat)

        # Order created, now cancel it
        order_status = 5
        self.assertEqual(database_order.paid, D(0))
        closed_reason = "Test cancellation"

        # Store the transaction in a TransactionForm object
        transaction = TransactionForm()
        # This is a close order transaction(system type 8)
        transaction.system = 8

        transaction.description = "CANCELLED ORDER NUMBER : %s" % database_order.number
        transaction.datestamp = datetime.date.today()

        # Credit the purchase account by the cost plus vat of the order
        transaction.add(odd_functions.get_account_by_system(7), credit = database_order.cost + database_order.vat)  

        # Debit the suppliers account by the cost plus VAT
        transaction.add(database_order.supplier.account, debit = database_order.cost + database_order.vat)

        status, message = transaction.check()
        self.assertTrue(status)

        database_closedorder = save_closeorder.save(database_order,
                                                    order_status,
                                                    closed_reason,
                                                    transaction)

        self.assertTrue(database_order.is_cancelled)

        # compare the database transaction
        database_transaction = database_closedorder.transaction
        self.assertEqual(transaction.description, database_transaction.description)
        self.assertEqual(database_transaction.system, 8)
        self.assertEqual(len(database_transaction.trans_list), 2)


    def test_06_receive_invoices(self):
        "receive invoices"

        invoice = SupplierInvoiceForm()
        all_openorders = odd_functions.get_openorders()

        for order in all_openorders:
            invoice.supplier = objects.Supplier.from_number(order.supplier.number)
            invoice.purchaseorder = order
            invoice.cost = order.cost - order.invoice_cost
            if invoice.cost < D("0.00"):
                invoice.cost = D("0.00")

            invoice.vat = order.vat - order.invoice_vat
            if invoice.vat < D("0.00"):
                invoice.vat = D("0.00")

            all_accounts = objects.Account.get_non_system()
            invoice.account = all_accounts[-1]
            invoice.datestamp = datetime.date.today()
            invoice.number = "Invoice for order %s" % order.number

            status, message = received_invoice_check(invoice)
            self.assertTrue(status)

            # Store the transaction in a TransactionForm object
            transaction = TransactionForm()
            # This is a supplier invoice transaction(system type 9)
            transaction.system = 9

            transaction.description = "Invoice received from %s" % invoice.supplier.name
            transaction.datestamp = invoice.datestamp

            # Credit the purchase account by the cost plus vat of the invoice
            transaction.add(odd_functions.get_account_by_system(7), credit = invoice.cost + invoice.vat)

            # Debit the Stock/Expense account by the cost
            if invoice.cost:
                transaction.add(invoice.account, debit = invoice.cost)

            # Debit the VAT Balance account by the VAT amount
            if invoice.vat:
                transaction.add(odd_functions.get_account_by_system(5), debit = invoice.vat)

            status, message = transaction.check()
            self.assertTrue(status)

            database_supplierinvoice = save_supplierinvoice.save(invoice, transaction)

            self.assertEqual(database_supplierinvoice.cost, invoice.cost)
            self.assertEqual(database_supplierinvoice.vat, invoice.vat)

            # compare the database transaction
            database_transaction = database_supplierinvoice.transaction
            self.assertEqual(transaction.description, database_transaction.description)
            self.assertEqual(database_transaction.system, 9)


    def Xest_07_make_payment(self):
        "make payments"

        # Store all form contents in an PaymentForm object
        payment = MakePaymentForm()

        all_openorders = odd_functions.get_openorders()

        payment.datestamp = datetime.date.today()                         # Datestamp of this payment
        transaction_date = payment.datestamp
        payment.status = 0

        all_accounts = objects.Account.get_non_system()
        payment.account = all_accounts[0]  # Use first available account as the cash account

        for order in all_openorders:
            payment.supplier = objects.Supplier.from_number(order.supplier.number)
            payment.purchaseorder = order
            payment.amount = order.invoice_cost + order.invoice_vat - order.paid

            all_openinvoices = order.get_openinvoices()

            # Check these payment details are ok via vatstuff_packages.vs_verify.purchases
            status, message = make_payment_check(payment, all_openinvoices)
            self.assertTrue(status)

            transaction_text = "Order %s : Payment to %s" % (payment.purchaseorder.number,
                                                             payment.supplier.name)

            #     A bank or cash account  Debit by zero  Credit by the payment amount
            t1 = [payment.account,        D("0.00"),     payment.amount ]

            #     Suppliers account          Debit by the amount  Credit by zero
            t2 = [payment.supplier.account,  payment.amount,      D("0.00") ]

            transaction_list = [t1, t2]

            status, payment_made =save_makepayment.save(payment,
                                                        transaction_date,
                                                        transaction_text,
                                                        transaction_list)

            self.assertTrue(status)
            self.assertEqual(order.outstanding, D("0.00"))
            self.assertEqual(order.paid, payment.amount)
            self.assertFalse(order.is_open)


    ##### These following tests check the account ledgers add up
    ##### they can be repeated in other test modules if further
    ##### transactions have been implemented

    def test_08_totals(self):
        """accounts balance"""
        # Get a list of all accounts
        all_accounts = objects.Account.get_all()

        # get all the account balances, and as this is double entry, check they add to zero
        total = decimal.Decimal("0.00")
        for account in all_accounts:
            total += account.balance
        self.assertEqual(total, decimal.Decimal("0.00"))


    def test_09_ledgers(self):
        """ledgers balance"""
        # Get a list of all accounts
        all_accounts = objects.Account.get_all()

        for account in all_accounts:
            debit = decimal.Decimal("0.00")
            credit = decimal.Decimal("0.00")
            for statement in account:
                # statement is each line of the account ledger
                # which is a list of [transaction, debit, credit, balance]
                debit += statement[1]
                credit += statement[2]
                self.assertEqual(debit-credit, statement[3])
            # and self.assertEqual(the final ledger balance is equal to that held
            # in the ACCOUNT table
            self.assertEqual(debit-credit, account.balance)


def suite():
    return unittest.TestLoader().loadTestsFromTestCase(TestPurchases)



