# test_sales.py  - run tests on sales functions
#
# Version : 0.7.0
# Date : 20100927
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#


"""Exercises functions under the Sales tab"""


import unittest
import os
import os.path
import datetime
import decimal

from vatstuff_packages.vs_utilities import config
from vatstuff_packages.vs_utilities import objects
from vatstuff_packages.vs_utilities import odd_functions
from vatstuff_packages.vs_database import (save_invoice,
                         save_payment,
                         save_creditnote)
from vatstuff_packages.vs_database.create_objects import create_Customer, create_Address, create_SellItem
from vatstuff_packages.vs_verify.form_objects import (TransactionForm,
                                                      CustomerForm,
                                                      InvoiceForm,
                                                      PaymentForm,
                                                      CreditNoteForm)
from vatstuff_packages.vs_verify.common import check_customer
from vatstuff_packages.vs_verify.sales import (new_invoice_check,
                             new_creditnote_check,
                             received_payment_check,
                             new_creditnote_check)

D = decimal.Decimal   # D used to avoid typing


# Create the unittest class

class TestSales(unittest.TestCase):
    """A test class for vatstuff sales"""

    def test_01_new_customer(self):
        "twenty customers"

        # customer_form is the data normally gathered
        # from a GUI form
        customer_form = CustomerForm()
        customer_form.ec_country = True
        customer_form.vat_country_code = "GB"
        customer_form.vat_registered = True
        customer_form.discount = decimal.Decimal("0.0")

        # Create 20 new customers
        c = 0
        while c<20:
            customer_form.name = "Test Customer " + str(c)
            customer_form.vat_regnumber = "00000" + str(c)

            # Check customer_form object
            status, message = check_customer(customer_form)
            self.assertTrue(status)

            new_customer = create_Customer(customer_form)
            self.assertTrue(new_customer)

            line_1 = "Customer " + str(c) + " address line 1" 

            database_address = create_Address(customer=new_customer,
                                              supplier=None,
                                              employee=None,
                                              contractor=None,
                                              line1 = line_1,
                                              line2 = "Test line2",
                                              city = "Test City",
                                              county = "Test County",
                                              postcode = "TST1 1TST",
                                              country = "England",
                                              reg_office = 1,
                                              delivery_point = 1,
                                              invoice_point = 1,
                                              order_point = 0,
                                              payment_point = 1,
                                              edit_new = False,
                                              visible = True,
                                              system = 0)

            self.assertTrue(database_address)
            c += 1


    def test_02_edit_customer(self):
        "changes to customer"

        all_customers = objects.Customer.get_all()
        this_customer = all_customers[0]
        customer_form = CustomerForm()

        # copy this_customer attributes into the customer_form object
        customer_form.copy_from(this_customer)


        # customer_form is the data normally gathered
        # from a GUI form - change the vat_regnumber
        customer_form.vat_regnumber = "1122334455"

        # Check customer_form object

        status, message = check_customer(customer_form)
        # This should fail, as the check considers this to be a
        # new customer with the same name as an existing one
        self.assertFalse(status)

        status, message = check_customer(customer_form, this_customer)
        # This should pass as the check now knows this_customer
        # is being edited
        self.assertTrue(status)

        # save vat registration number
        this_customer.vat_regnumber = customer_form.vat_regnumber

        self.assertEqual(this_customer.vat_regnumber, "1122334455")


    def test_03_add_sell_item(self):
        "add sell item"
        sell_item = create_SellItem(description     = "sell_thing",
                                    cost            = decimal.Decimal("30.00"),
                                    vatrate         = objects.VATRate.from_symbol("S"),
                                    notes           = "A test item to sell")
        self.assertEqual(sell_item.description, "sell_thing")
        self.assertEqual(sell_item.cost, decimal.Decimal("30.00"))


    def test_04_invoice(self):
        "create invoices"
        invoice = InvoiceForm()

        all_customers = objects.Customer.get_all()

        for customer in all_customers:

            invoice.customer = customer                                # The customer to whom the invoice is being sent

            customer_addresses = invoice.customer.get_addresses()
            invoice.customer_address = customer_addresses[0]           # The customers address where the invoice is sent

            invoice.cus_order_number = "001"                           # The customers order number

            company_addresses = invoice.company.get_addresses()
            invoice.payment_address = company_addresses[0]             # Where payment should be sent

            invoice.payment_instructions = "Please pay within 30 days" # Any text payment instruction you wish to add
            invoice.date_of_issue = datetime.date.today()              # Datestamp of the issue of this invoice
            invoice.date_of_supply = datetime.date.today()             # Datestamp of the supply of goods
            invoice.date_of_taxpoint = datetime.date.today()           # Actual date at which VAT is calculated

            last_invoice_number = odd_functions.get_config_item(u"LAST_INVOICE_NUMBER")
            if not last_invoice_number:
                invoice.number = 1
            else:
                invoice.number =  int(last_invoice_number) + 1         # The invoice number

            all_items = objects.SellItem.get_all()
            item = all_items[0]

            invoice.item_list = [{"Qty":D(1),
                                  "Description": item.description,
                                  "Unit Cost":item.cost,
                                  "Discount": False,
                                  "VAT":item.vatrate.rate}]

            invoice.cost = item.cost         # Total cost (net - exclusive of VAT)

            result = (invoice.cost * item.vatrate.rate) / D("100.0000")
            invoice.vat = result.quantize(D('.01'), rounding=decimal.ROUND_HALF_EVEN)

            # Check these input invoice details are ok via vatstuff_packages.vs_verify.sales
            status, message = new_invoice_check(invoice)
            self.assertTrue(status)

            transaction = TransactionForm()
            transaction.system = 5
            transaction.datestamp = datetime.date.today()
            transaction.description = "Sale transaction"

            # Debit the customers account by the cost plus VAT
            transaction.add(invoice.customer.account, debit = invoice.cost + invoice.vat)

            # Credit the sales account by the cost
            transaction.add(odd_functions.get_account_by_system(6), credit = invoice.cost)

            # Credit the VAT balance account by the VAT amount
            transaction.add(odd_functions.get_account_by_system(5), credit = invoice.vat)

            status, message = transaction.check()
            self.assertTrue(status)

            # Must now implement the transaction and save the Invoice
            database_invoice = save_invoice.save(invoice, transaction)

            self.assertEqual(database_invoice.cost, invoice.cost)
            self.assertEqual(database_invoice.vat, invoice.vat)

            # compare the database transaction
            database_transaction = database_invoice.transaction
            self.assertEqual(transaction.description, database_transaction.description)
            self.assertEqual(database_transaction.system, 5)
            self.assertEqual(len(database_transaction.trans_list), 3)


    def test_05_receive_payment(self):
        "receive payments"

        # Store all form contents in an PaymentForm object
        payment = PaymentForm()

        all_openinvoices = odd_functions.get_openinvoices()
        payment.datestamp = datetime.date.today()

        all_accounts = objects.Account.get_non_system()
        payment.account = all_accounts[0]  # Use first available account as the cash account

        for invoice in all_openinvoices:

            payment.customer = invoice.customer                        # The customer to whom the invoice has been sent
            payment.amount   = invoice.outstanding                     # The amount on the invoice
            payment.customer_reference = "Payment from %s" % payment.customer.name

            # Check these payment details are ok via vatstuff_packages.vs_verify.sales
            status, message = received_payment_check(payment, [invoice])
            self.assertTrue(status)

            transaction = TransactionForm()
            transaction.system = 6
            transaction.datestamp = payment.datestamp
            transaction.description = payment.customer_reference

            # Debit account by the amount
            transaction.add(payment.account, debit = payment.amount)

            # Credit the customers account by the amount
            transaction.add(payment.customer.account, credit = payment.amount)

            status, message = transaction.check()
            self.assertTrue(status)

            database_payment = save_payment.save(payment, transaction)

            self.assertEqual(invoice.outstanding, D("0.00"))
            self.assertEqual(invoice.paid, database_payment.amount)
            self.assertFalse(invoice.is_open)

            # compare the database transaction
            database_transaction = database_payment.transaction
            self.assertEqual(transaction.description, database_transaction.description)
            self.assertEqual(database_transaction.system, 6)
            self.assertEqual(len(database_transaction.trans_list), 2)


    def test_06_payment_creditnote(self):
        "creditnote with payment"
        creditnote = CreditNoteForm()

        all_customers = objects.Customer.get_all()
        creditnote.customer = all_customers[0]
        creditnote.invoice = None
        creditnote.status = 1

        # STATUS - normally 0 when a creditnote is sent against an invoice
        # STATUS - 1 when a creditnote is sent with a payment

        # Can you charge VAT?
        creditnote.show_vat = creditnote.customer.charge_vat()
        creditnote.company = objects.Company()

        creditnote.customer_address = None
        creditnote.reason = "Test creditnote"
        creditnote.datestamp = datetime.date.today()

        # Credit Note number
        last_creditnote_number = odd_functions.get_config_item("LAST_CREDITNOTE_NUMBER")
        if not last_creditnote_number:
            creditnote.number = 1
        else:
            creditnote.number =  int(last_creditnote_number) + 1

        all_accounts = objects.Account.get_non_system()
        creditnote.account_credited = all_accounts[0]

        creditnote.discount = D("0.0")
        creditnote.contact = None

        all_items = objects.SellItem.get_all()
        item = all_items[0]

        creditnote.item_list = [{"Qty":D(1),
                                 "Description": item.description,
                                 "Unit Cost":item.cost,
                                 "Discount": False,
                                 "VAT":item.vatrate.rate}]

        creditnote.cost = item.cost         # Total cost (net - exclusive of VAT)

        result = (creditnote.cost * item.vatrate.rate) / D("100.0000")
        creditnote.vat = result.quantize(D('.01'), rounding=decimal.ROUND_HALF_EVEN)

        # Check these input creditnote details are ok via vatstuff_packages.vs_verify.sales
        status, message = new_creditnote_check(creditnote)
        self.assertTrue(status)

        transaction = TransactionForm()
        transaction.system = 7
        transaction.datestamp = creditnote.datestamp
        transaction.description = "Creditnote with payment"

        # Credit the bank account by the amount
        transaction.add(creditnote.account_credited, credit = creditnote.cost + creditnote.vat)

        # Debit the sales account by the credit note cost
        transaction.add(odd_functions.get_account_by_system(6), debit = creditnote.cost)

        # Debit the VAT owed account by the VAT  
        transaction.add(odd_functions.get_account_by_system(5), debit = creditnote.vat)

        status, message = transaction.check()
        self.assertTrue(status)

        database_creditnote = save_creditnote.save(creditnote, transaction)

        # compare the database transaction
        database_transaction = database_creditnote.transaction
        self.assertEqual(transaction.description, database_transaction.description)
        self.assertEqual(database_transaction.system, 7)
        self.assertEqual(len(database_transaction.trans_list), 3)


    ##### These following tests check the account ledgers add up
    ##### they can be repeated in other test modules if further
    ##### transactions have been implemented

    def test_07_totals(self):
        """accounts balance"""
        # Get a list of all accounts
        all_accounts = objects.Account.get_all()

        # get all the account balances, and as this is double entry, check they add to zero
        total = decimal.Decimal("0.00")
        for account in all_accounts:
            total += account.balance
        self.assertEqual(total, decimal.Decimal("0.00"))


    def test_08_ledgers(self):
        """ledgers balance"""
        # Get a list of all accounts
        all_accounts = objects.Account.get_all()

        for account in all_accounts:
            debit = decimal.Decimal("0.00")
            credit = decimal.Decimal("0.00")
            for statement in account:
                # statement is each line of the account ledger
                # which is a list of [transaction, debit, credit, balance]
                debit += statement[1]
                credit += statement[2]
                self.assertEqual(debit-credit, statement[3])
            # and self.assertEqual(the final ledger balance is equal to that held
            # in the ACCOUNT table
            self.assertEqual(debit-credit, account.balance)


def suite():
    return unittest.TestLoader().loadTestsFromTestCase(TestSales)

