# test_sample_databases.py  - creates sample databases for manual tests
#
# Version : 0.6
# Date : 20100902
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#


"""creates sample databases for manual tests"""

import unittest
import os
import os.path
import shutil
import tarfile

from vatstuff_packages.vs_utilities import config
from vatstuff_packages.vs_utilities import objects
from vatstuff_packages.vs_utilities import odd_functions


def print_about():
    "Print a list of these databases"
    print """
This test suit creates the following databases:

1   testdata.vat  - a working sample with the current database version
2   testdata2.vat - the same sample, but with an incorrect ID of VATStuffX
3   testdata3.vat - database with unsupported version 0.1
4   testdata4.vat - database with previous version 0.6
5   testdata5.vat - database with later version 100.0
6   testdata.bak - a backup file containing testdata.vat
7   testdata2.bak - a backup file containing testdata2.vat
8   testdata3.bak - a backup file containing testdata3.vat
9   testdata4.bak - a backup file containing testdata4.va7
10  testdata5.bak - a backup file containing testdata5.vat

These files are available for manual tests - see the document testing.odt
"""


# Create the unittest class

class TestSampleDatabases(unittest.TestCase):
    """A test class for creating sample databases"""

    def test_01_set_config_item(self):
        "create testdata.bak"
        # print out above text
        print_about()
        # create a backup file
        backup_file = os.path.join(config.vs_cfg["DATABASE_FOLDER"],"testdata.bak")
        fp=None
        try:
            fp = tarfile.open(backup_file, 'w:bz2')
            fp.add(config.vs_cfg["DATABASE_FILEPATH"], config.vs_cfg["ARCHIVED_FILE"], False)
            fp.close()
        finally:
            if fp:
                fp.close()
        self.assertTrue(os.path.exists(backup_file))


    def test_02_set_program_id(self):
        "create testdata2 files"
        result = odd_functions.set_config_item("PROGRAM_ID", "VATStuffX")
        self.assertTrue(result)
        # program id changed, take a copy
        copy_file = os.path.join(config.vs_cfg["DATABASE_FOLDER"],"testdata2.vat")
        shutil.copy(config.vs_cfg["DATABASE_FILEPATH"], copy_file)
        self.assertTrue(os.path.exists(copy_file))
        # create a backup file
        backup_file = os.path.join(config.vs_cfg["DATABASE_FOLDER"],"testdata2.bak")
        fp=None
        try:
            fp = tarfile.open(backup_file, 'w:bz2')
            fp.add(copy_file, config.vs_cfg["ARCHIVED_FILE"], False)
            fp.close()
        finally:
            if fp:
                fp.close()
        self.assertTrue(os.path.exists(backup_file))
        # restore database to normal
        result = odd_functions.set_config_item("PROGRAM_ID", "VATStuff")
        self.assertTrue(result)
        result = odd_functions.get_program_id()
        self.assertEqual(result, "VATStuff")


    def test_03_set_database_version_0_1(self):
        "create testdata3 files"
        result = odd_functions.set_config_item("DATABASE_VERSION", "0.1")
        self.assertTrue(result)
        # database version changed, take a copy
        copy_file = os.path.join(config.vs_cfg["DATABASE_FOLDER"],"testdata3.vat")
        shutil.copy(config.vs_cfg["DATABASE_FILEPATH"], copy_file)
        self.assertTrue(os.path.exists(copy_file))
        # create a backup file
        backup_file = os.path.join(config.vs_cfg["DATABASE_FOLDER"],"testdata3.bak")
        fp=None
        try:
            fp = tarfile.open(backup_file, 'w:bz2')
            fp.add(copy_file, config.vs_cfg["ARCHIVED_FILE"], False)
            fp.close()
        finally:
            if fp:
                fp.close()
        self.assertTrue(os.path.exists(backup_file))
        # restore database to normal
        result = odd_functions.set_config_item("DATABASE_VERSION", "0.7")
        self.assertTrue(result)
        result = odd_functions.get_config_item("DATABASE_VERSION")
        self.assertEqual(result, "0.7")


    def test_04_set_database_version_0_6(self):
        "create testdata4 files"
        result = odd_functions.set_config_item("DATABASE_VERSION", "0.6")
        self.assertTrue(result)
        # database version changed, take a copy
        copy_file = os.path.join(config.vs_cfg["DATABASE_FOLDER"],"testdata4.vat")
        shutil.copy(config.vs_cfg["DATABASE_FILEPATH"], copy_file)
        self.assertTrue(os.path.exists(copy_file))
        # create a backup file
        backup_file = os.path.join(config.vs_cfg["DATABASE_FOLDER"],"testdata4.bak")
        fp=None
        try:
            fp = tarfile.open(backup_file, 'w:bz2')
            fp.add(copy_file, config.vs_cfg["ARCHIVED_FILE"], False)
            fp.close()
        finally:
            if fp:
                fp.close()
        self.assertTrue(os.path.exists(backup_file))
        # restore database to normal
        result = odd_functions.set_config_item("DATABASE_VERSION", "0.7")
        self.assertTrue(result)
        result = odd_functions.get_config_item("DATABASE_VERSION")
        self.assertEqual(result, "0.7")


    def test_05_set_database_version_100(self):
        "create testdata5 files"
        result = odd_functions.set_config_item("DATABASE_VERSION", "100.0")
        self.assertTrue(result)
        # database version changed, take a copy
        copy_file = os.path.join(config.vs_cfg["DATABASE_FOLDER"],"testdata5.vat")
        shutil.copy(config.vs_cfg["DATABASE_FILEPATH"], copy_file)
        self.assertTrue(os.path.exists(copy_file))
        # create a backup file
        backup_file = os.path.join(config.vs_cfg["DATABASE_FOLDER"],"testdata5.bak")
        fp=None
        try:
            fp = tarfile.open(backup_file, 'w:bz2')
            fp.add(copy_file, config.vs_cfg["ARCHIVED_FILE"], False)
            fp.close()
        finally:
            if fp:
                fp.close()
        self.assertTrue(os.path.exists(backup_file))
        # test odd_functions.check() on this returns False
        status, message = odd_functions.check()
        self.assertFalse(status)
        # restore database to normal
        result = odd_functions.set_config_item("DATABASE_VERSION", "0.7")
        self.assertTrue(result)
        result = odd_functions.get_config_item("DATABASE_VERSION")
        self.assertEqual(result, "0.7")
        # test odd_functions.check() on this now returns True
        status, message = odd_functions.check()
        self.assertTrue(status)


def suite():
    return unittest.TestLoader().loadTestsFromTestCase(TestSampleDatabases)

