####### VATStuff #######
#
# config.py  - loads config data, and provides utility functions
#
# Version : 0.7.0
# Date : 20100908
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#

"""Loads configuration data and defines some commonly used functions

Application configuration items are available in the global dictionary
vs_cfg with the keys:

PROGRAM_ID                   - program name, and identifier in the database
PROGRAM_VERSION              - program version, a string such as '0.1'
DATABASE_VERSION             - database schema version, a string such as '0.1'
RELEASE_DATE                 - program release date, a string like 'yyyy-mm-dd'
PROGRAM_URL                  - url string, home page for this application
CONFIG_LOCATION              - filepath of configuration file
STORE_VERSION                - per user persistent data is stored in a file, give it a version number
STORE_ENABLED                - per user persistent store status True or False
STORE_FOLDER                 - folder where per-user stored data is kept
STORE_FILE                   - filename holding the store
STORE_FILEPATH               - full filepath of the store 
REPORTLABS_AVAILABLE         - True if the reportlabs module is available, and pdf files can be created
PIL_AVAILABLE                - true if the PIL module is available, also used during pdf creation
SCRIPTDIRECTORY              - The directory of the script, found from sys.argv[0]
DATABASE_FOLDER              - the folder where the database file is sited
ARCHIVED_FILE                - the database filename used inside a tar backup, usually VATStuff.vat
DATABASE_FILE                - the actual database filename
DATABASE_FILEPATH            - the full filepath of the database
DATABASE_EXTENSION           - the default extension of the database filename, without the dot, normally 'vat'
LAST_DATABASE_FILEPATH       - the pathname of the database previously openned
LOCK_FOLDER                  - the folder where the lockfile is kept, usually the same as the database folder
LOCK_FILE                    - the lockfile name, usually the database filename with .vslockfile appended
LOCK_FILEPATH                - the full filepath of the lockfile
WRITE_ACCESS                 - Set to True if this process has write access to the database
CUSTOMER_ACCOUNT_TYPE_NAME   - Default Customer account type, does not change, only used when creating new database 
SUPPLIER_ACCOUNT_TYPE_NAME   - Default Supplier account type, does not change, only used when creating new database
A3
A4
LETTER
LAST_ORDER_PDF_SIZE
LAST_INVOICE_PDF_SIZE
LAST_CREDITNOTE_PDF_SIZE
LAST_REPORT_PDF_SIZE
HTML_FOLDER
HTML_FILES
LOG_FOLDER
LOG_FILE
LOG_FILEPATH
LOGGING         - True turns logging on, False turns it off
DEBUG           - True turns debugging on, False turns it off
IMAGES_FOLDER
EXPORT_FOLDER
BACKUP_FOLDER
BACKUP_FILE
ORDERS_PDF_FOLDER
INVOICES_PDF_FOLDER
CREDITNOTES_PDF_FOLDER
REPORTS_PDF_FOLDER
SPLASH_IMAGE
LOGO_IMAGE
LOGO_FILE
LOGO_FOLDER
LOGO_FILEPATH
LOGO_MAXWIDTH
LOGO_MAXHEIGHT
ICON_IMAGE
ICON_FILEPATH
CURRENCY_SYMBOL
CURRENCY_POINT
CURRENCY_THOUSANDS_SEPARATOR   
CS
CP
CTS
COUNTRY
VAT_COUNTRY_CODE
ZERO_STRING
SYMBOL_ZERO_STRING
MAX_BUTTONS
DEFAULT_VAT_STANDARD_RATE   - (string) Used on database creation, can be set via the config file
DEFAULT_VAT_REDUCED_RATE    - (string) Used on database creation, can be set via the config file
VAT_STANDARD_RATE           - (Decimal) initially set to DEFAULT_VAT_STANDARD_RATE, but can be changed by the user
VAT_REDUCED_RATE            - (Decimal) initially set to DEFAULT_VAT_REDUCED_RATE, but can be changed by the user 
VAT_SYMBOLS
DELETE_TO_DATE              - The date (datetime.date object) to which the last database deletion was done
BUTTON_HEIGHT               - The height of a button
BACKGROUND_COLOUR           - The default window background colour
TRANSACTION_REFRESH         - List of windows that require refresh to be called on them if a transaction
                              change occurs. As such a window opens, it should add itself to this list, as
                              it closes, it should remove itself.  Any function doing a transaction change
                              should call the function vatstuff_packages.vs_gui.gui_functions,big_refresh() which will in turn
                              call the refresh function on every window in this list.
SEARCH_CONTACTS_FRAME       - either None, or holds a reference to the search contacts frame if open

This module contains exception classes, logging functions, currency functions

This module is imported by every other VATStuff application module, via the package import
from vatstuff_packages.vs_utilities import *
so the configuration dictionary and these functions are available everywhere.
To avoid circular imports, this module only imports standard library modules - not any other application module.

It should not import third party libraries such as wx - so it can be used by test
routines and the interactive interpreter.
"""

import os
import os.path
import decimal
import logging
import logging.handlers
import shelve
import random
import ConfigParser
import dbhash
import sys

# dbhash is required since py2exe does not automatically import a dbm module
# for shelve - so importing dbhash makes this available - this is removed in
# python3, so will need to see what happens with py2exe then


from functools import wraps

D = decimal.Decimal   # D used to avoid typing

# Application configuration items are available in the global dictionary vs_cfg

# The PROGRAM_ID is the program name, and is used when openning a database to check
# the database was created by this program.  If this program is forked - then change
# the program id to another name of your choice,  so that the original 'VATStuff'
# program will not load a database created by the forked program and vica versa - since
# money is involved there should be rigorous separation between forks.

# This program ID is also used to create part of the filename of backups, 
# ie VATStuff2009-12-2.bak  and the database filename VATStuff_db.vat - so use
# a string without any spaces or peculiar characters

vs_cfg = {"PROGRAM_ID"      : "VATStuff",
          "PROGRAM_VERSION" : "0.7.0",
          "DATABASE_VERSION": "0.7",
          "RELEASE_DATE"    : "2010-12-01",
          "PROGRAM_URL"     : "http://www.vatstuff.co.uk"}

# Flag to indicate a logger has been created
_vs_logger = None


# The __all__ list defines those names which can be imported using from .. import *

__all__ = [ "vs_cfg",
            "VS_CANCEL",
            "VS_ERROR",
            "VS_ANOTHER_PROCESS",
            "get_stored_item",
            "set_stored_item",
            "error_log",
            "info_log",
            "debug_log",
            "class_log",
            "logmethod",
            "exception_log",
            "vs_unicode_symbol",
            "proper_filename",
            "CURRENCY_ERROR",
            "Currency",
            "to_currency_string",
            "string_to_decimal",
            "create_logo_name" ]

# This list excludes
#   set_defaults
#   read_defaults
#   set_logger

# which require a full import if they are needed

class VS_CANCEL(Exception):
    "Cancel button pressed in startup frame"
    pass


class VS_ERROR(Exception):
    "Error raised in VATStuff."
    pass


class VS_ANOTHER_PROCESS(VS_ERROR):
    "More than one process has accessed the database"
    pass


def set_defaults(scriptdirectory,
                 config_location = None,
                 logging = "DEFAULT",
                 debug_mode = "DEFAULT"):
    """Called from vatstuff.py on startup, to set the programs configuration

       Reads the config file and sets the global configuration dictionary
       vs_cfg values.
       scriptdirectory - the directory the script is run from.
       config_location - filepath to a configuration file, if None it
                         is searched for in a number of standard locations.
       logging is ON, OFF or DEFAULT
       debug_mode is ON, OFF or DEFAULT
       if DEFAULT, the values as set in the config file apply."""

    # Check these third party libraries are available
    try:
        import reportlab
        vs_cfg["REPORTLABS_AVAILABLE"] = True
    except ImportError:
        vs_cfg["REPORTLABS_AVAILABLE"] = False

    try:
        import Image
        vs_cfg["PIL_AVAILABLE"] = True
    except ImportError:
        vs_cfg["PIL_AVAILABLE"] = False

    vs_cfg["SCRIPTDIRECTORY"]=scriptdirectory
    vs_cfg["LAST_DATABASE_FILEPATH"] = None

    # Look for the config file vatstuff.cfg at some possible locations
    possible_config_locations = [ config_location,
                                  os.path.join(scriptdirectory, "vatstuff_files", "vatstuff.cfg"),
                                  os.path.join(sys.prefix, "vatstuff_files", "vatstuff.cfg"),
                                 "/usr/local/vatstuff_files/vatstuff.cfg",
                                 "/usr/local/etc/vatstuff/vatstuff.cfg",
                                 "/usr/local/etc/VATStuff/vatstuff.cfg",
                                 "/etc/vatstuff/vatstuff.cfg",
                                 "/etc/VATStuff/vatstuff.cfg",
                                 "/etc/opt/vatstuff/vatstuff.cfg",
                                 "/etc/opt/VATStuff/vatstuff.cfg",
                                 "/opt/vatstuff_files/vatstuff.cfg",
                                 "/opt/vatstuff/vatstuff.cfg",
                                 "/opt/VATStuff/vatstuff.cfg",
                                 "/opt/vatstuff/vatstuff_files/vatstuff.cfg",
                                 "/opt/VATStuff/vatstuff_files/vatstuff.cfg" ]


    for config_file in possible_config_locations:
        if config_file and os.path.isfile(config_file):
            vs_cfg["CONFIG_LOCATION"] = config_file
            break
    else:
        return False
            

    # read defaults from the config file at vs_cfg["CONFIG_LOCATION"]
    # Defaults read from this file are :
    # vs_cfg["IMAGES_FOLDER"]
    # vs_cfg["HTML_FOLDER"]
    # vs_cfg["STORE_FOLDER"]
    # vs_cfg["DATABASE_FOLDER"]
    # vs_cfg["EXPORT_FOLDER"]
    # vs_cfg["BACKUP_FOLDER"]
    # vs_cfg["LOG_FOLDER"]
    #
    # vs_cfg["DEFAULT_VAT_STANDARD_RATE"]
    # vs_cfg["DEFAULT_VAT_REDUCED_RATE"]
    # vs_cfg["VAT_STANDARD_RATE"]
    # vs_cfg["VAT_REDUCED_RATE"]
    #
    # vs_cfg["CURRENCY_SYMBOL"] = "U+00A3"               the pound stirling unicode character
    # vs_cfg["CURRENCY_POINT"]  = "U+002E"               the decimal point character
    # vs_cfg["CURRENCY_THOUSANDS_SEPARATOR"]  = "U+002C" the comma character

    # vs_cfg["LOGGING"]
    # vs_cfg["DEBUG"]

    if not read_defaults(scriptdirectory, vs_cfg["CONFIG_LOCATION"]):
        return False

    ######### Set up the logger

    vs_cfg["LOG_FILE"]="%s.log" % vs_cfg["PROGRAM_ID"]
    vs_cfg["LOG_FILEPATH"]=os.path.join(vs_cfg["LOG_FOLDER"], vs_cfg["LOG_FILE"])

    # Default logging value is read from the config file, but can be changed if the
    # program starts with the command line argument 'nologs' which sets it to False
    if logging == "ON":
        vs_cfg["LOGGING"] = True
    if logging == "OFF":
        vs_cfg["LOGGING"] = False
    # Any other string - such as "DEFAULT" leaves it as set by the config file

    # Default debug value is read from the config file, but can be changed if the
    # program starts with the command line argument 'debug' which sets it to True
    if not vs_cfg["LOGGING"]:
        # No logging enabled, so debug mode is irrelevant
        vs_cfg["DEBUG"] = False
    else:
        if debug_mode == "ON":
            vs_cfg["DEBUG"] = True
        if debug_mode == "OFF":
            vs_cfg["DEBUG"] = False
        # Any other string - such as "DEFAULT" leaves
        # it as set by the config file

    # now, depending on the above values, start the logger with the correct mode
    set_logger()

    ################  Per-user preferences file  #################
    # The file vs_cfg["STORE_FILEPATH"] holds per-user preferences
    # about the current installation such as where the database is
    # held, so multiple users on a server can be running the program
    # with different databases
    #
    # for Linux systems this would normally be located under
    # hidden user directory ~/.vatstuff

    # home is the users home directory - where data is stored for Linux systems
    home = os.getenv("HOME", os.getenv("HOMEPATH", os.path.join(scriptdirectory, "vatstuff_files")))
    # appdata is where data is stored for windows systems
    appdata = os.getenv("APPDATA", os.path.join(scriptdirectory, "vatstuff_files"))

    if os.name == "nt":
        vs_cfg["STORE_FOLDER"] = os.path.join(appdata, "vatstuff")
    else:
        vs_cfg["STORE_FOLDER"] = os.path.join(home, ".vatstuff")

    vs_cfg["STORE_VERSION"] = "0.1"
    vs_cfg["STORE_ENABLED"] = True

    vs_cfg["STORE_FILE"]="%s.preferences"  % vs_cfg["PROGRAM_ID"]
    vs_cfg["STORE_FILEPATH"]=os.path.join(vs_cfg["STORE_FOLDER"], vs_cfg["STORE_FILE"])

    if not os.path.isdir(vs_cfg["STORE_FOLDER"]):
        try:
            os.makedirs(vs_cfg["STORE_FOLDER"])
        except OSError:
            vs_cfg["STORE_ENABLED"] = False

    # Database file is the actual filename of the running database
    vs_cfg["DATABASE_EXTENSION"] = "vat"
    vs_cfg["DATABASE_FILE"]="%s_db.%s" % (vs_cfg["PROGRAM_ID"], vs_cfg["DATABASE_EXTENSION"])
    vs_cfg["DATABASE_FILEPATH"]=os.path.join(vs_cfg["DATABASE_FOLDER"], vs_cfg["DATABASE_FILE"])

    # ARCHIVED_FILE is the filename as stored in a tar backup file
    vs_cfg["ARCHIVED_FILE"]="%s.%s" % (vs_cfg["PROGRAM_ID"], vs_cfg["DATABASE_EXTENSION"])

    # Default account types, only used when creating new database
    vs_cfg["CUSTOMER_ACCOUNT_TYPE_NAME"] = "Asset" 
    vs_cfg["SUPPLIER_ACCOUNT_TYPE_NAME"] = "Liability"

    # page sizes for creating a pdf in points (1/72 of an inch)
    vs_cfg["A3"]     = (842, 1190)
    vs_cfg["A4"]     = (595, 842)
    vs_cfg["LETTER"] = (612, 792)

    vs_cfg["LAST_ORDER_PDF_SIZE"] = vs_cfg["A4"]
    vs_cfg["LAST_INVOICE_PDF_SIZE"] = vs_cfg["A4"]
    vs_cfg["LAST_CREDITNOTE_PDF_SIZE"] = vs_cfg["A4"]
    vs_cfg["LAST_REPORT_PDF_SIZE"] = vs_cfg["A4"]

    # List of HTML manual files, in the order used by the navigation buttons 
    vs_cfg["HTML_FILES"]=["index.html",
                            "overview.html",
                            "getting_started.html",
                            "accounts.html",
                            "sales.html",
                            "purchases.html",
                            "move_money.html",
                            "vat.html",
                            "reports.html",
                            "company.html",
                            "admin.html",
                            "security.html",
                            "faq.html",
                            "credits.html",
                            "license.html"]


    # The last backup file
    vs_cfg["BACKUP_FILE"] = ""

    # The default folders where pdf files are saved, initially set
    # as the same as the export folder
    vs_cfg["ORDERS_PDF_FOLDER"] = vs_cfg["EXPORT_FOLDER"]
    vs_cfg["INVOICES_PDF_FOLDER"] = vs_cfg["EXPORT_FOLDER"]
    vs_cfg["CREDITNOTES_PDF_FOLDER"] = vs_cfg["EXPORT_FOLDER"]
    vs_cfg["REPORTS_PDF_FOLDER"] = vs_cfg["EXPORT_FOLDER"]


    # IMAGES

    # The startup splash image
    vs_cfg["SPLASH_IMAGE"]="splash.png"

    # For Linux
    vs_cfg["ICON_IMAGE"]="vatstuff32.png"
    # For windows
    if os.name == "nt":
        vs_cfg["ICON_IMAGE"]="vatstuff32.ico"

    # The filepath of the icon file, used in frames
    vs_cfg["ICON_FILEPATH"]=os.path.join(vs_cfg["IMAGES_FOLDER"], vs_cfg["ICON_IMAGE"])

    # This is the file name that is saved to tar backup files
    vs_cfg["LOGO_IMAGE"]="logo.png"

    # This is the filename of the image currently displayed, and associated with the database
    # currently loaded. The name is uniquely created as images are imported, and is saved with the database
    # so start off with a dummy name equal to the logo image
    vs_cfg["LOGO_FILE"]=vs_cfg["LOGO_IMAGE"]

    # The logo image is scaled to fit within a maximum width and height
    vs_cfg["LOGO_MAXWIDTH"]  = 200
    vs_cfg["LOGO_MAXHEIGHT"] = 50

    # defaults are now set, now read from the per-user persistent store
    # and if successful, overwrite the defaults

    # Try reading values from the per-user persistent storage
    if vs_cfg["STORE_ENABLED"] and os.path.exists(vs_cfg["STORE_FILEPATH"]):
        store_version = vs_cfg["STORE_VERSION"]
        get_stored_item("STORE_VERSION")
        if vs_cfg["STORE_VERSION"] != store_version:
            # There's something wrong with the persistent store
            vs_cfg["STORE_ENABLED"] = False
            # this would change if future stores require different versions

        # temporarily store the default database folder and filename,
        # in case getting the ones from the
        # persistent store does not work
        default_database_file   = vs_cfg["DATABASE_FILE"]
        default_database_folder = vs_cfg["DATABASE_FOLDER"]

        # Read values
        get_stored_item("DATABASE_FOLDER")
        get_stored_item("DATABASE_FILE")
        get_stored_item("EXPORT_FOLDER")
        get_stored_item("BACKUP_FOLDER")
        get_stored_item("ORDERS_PDF_FOLDER")
        get_stored_item("INVOICES_PDF_FOLDER")
        get_stored_item("CREDITNOTES_PDF_FOLDER")
        get_stored_item("REPORTS_PDF_FOLDER")
        get_stored_item("BACKUP_FILE")
        get_stored_item("LAST_ORDER_PDF_SIZE")
        get_stored_item("LAST_INVOICE_PDF_SIZE")
        get_stored_item("LAST_CREDITNOTE_PDF_SIZE")
        get_stored_item("LAST_REPORT_PDF_SIZE")

        vs_cfg["DATABASE_FILEPATH"]=os.path.join(vs_cfg["DATABASE_FOLDER"], vs_cfg["DATABASE_FILE"])

        # The last database is the one recovered from the store
        vs_cfg["LAST_DATABASE_FILEPATH"] = vs_cfg["DATABASE_FILEPATH"]

        # but if the database file stored in the persistent store no longer exists
        # use the defaults instead
        if not os.path.isfile(vs_cfg["DATABASE_FILEPATH"]):
            vs_cfg["LAST_DATABASE_FILEPATH"] = None
            vs_cfg["DATABASE_FILEPATH"]=os.path.join(default_database_folder, default_database_file)
    else:
        # If the persistent store does not exist, try to create one
        set_stored_item("STORE_VERSION", vs_cfg["STORE_VERSION"])

    # The database filepath has now been set, either at the default
    # or at the value recovered from the persistent store.

    # The logo_folder is the same folder as the database folder
    vs_cfg["LOGO_FOLDER"]=vs_cfg["DATABASE_FOLDER"]
    vs_cfg["LOGO_FILEPATH"]=os.path.join(vs_cfg["LOGO_FOLDER"], vs_cfg["LOGO_FILE"])

    # Now set the lock file
    # The lock_folder is the same folder as the database folder
    vs_cfg["LOCK_FOLDER"]=vs_cfg["DATABASE_FOLDER"]
    vs_cfg["LOCK_FILE"]=vs_cfg["DATABASE_FILE"]+".vslockfile"
    vs_cfg["LOCK_FILEPATH"]=os.path.join(vs_cfg["LOCK_FOLDER"], vs_cfg["LOCK_FILE"])

    # This is True if this process has write access to the database, False otherwise
    # It is initially set here as false, and the presence of the lockfile is tested as
    # part of the database check, at which point it is set to True or False as the
    # situation requires.
    vs_cfg["WRITE_ACCESS"] = False

    # Currency
    # These default values are read from the config file
    # vs_cfg["CURRENCY_SYMBOL"] = "U+00A3"     # the pound stirling unicode character
    # vs_cfg["CURRENCY_POINT"]  = "U+002E"     # the decimal point character
    # vs_cfg["CURRENCY_THOUSANDS_SEPARATOR"]  = "U+002C"     # the comma character

    # And set the actual Symbols
    vs_cfg["CS"] = vs_unicode_symbol(vs_cfg["CURRENCY_SYMBOL"])
    vs_cfg["CP"] = vs_unicode_symbol(vs_cfg["CURRENCY_POINT"])
    vs_cfg["CTS"] = vs_unicode_symbol(vs_cfg["CURRENCY_THOUSANDS_SEPARATOR"])

    # A zero currency string
    vs_cfg["ZERO_STRING"] = "0%s00" % vs_cfg["CP"]
    vs_cfg["SYMBOL_ZERO_STRING"] = "%s0%s00" % (vs_cfg["CS"], vs_cfg["CP"])

    # Program constants
    vs_cfg["MAX_BUTTONS"] = 6            # the number of user defined buttons to show under 'Move Money'
    vs_cfg["BUTTON_HEIGHT"] = 30         # Set a button height value
    vs_cfg["BACKGROUND_COLOUR"] = None   # set when the gui starts by reading the colour from the startup frame
    vs_cfg["TRANSACTION_REFRESH"]=[]
    # List of windows that require refresh to be called on them if a transaction
    # change occurs, as such a window opens, it should add itself to this list.
    # As it closes, it should remove itself.
    #  Any function doing a transaction change should call the function
    # vatstuff_packages.vs_gui.gui_functions,big_refresh() which will in turn
    # call the refresh function on every window in this list.
    vs_cfg["SEARCH_CONTACTS_FRAME"]=None   # either None, or holds a reference to the search contacts frame if open

    # Country defaults
    vs_cfg["COUNTRY"]          = "England"  # Default country for addresses
    vs_cfg["VAT_COUNTRY_CODE"] = "GB"       # Country code, used as part of the VAT registration number

    # VAT symbols, other symbols can be edited, but these are the 'fixed' symbols which must exist
    vs_cfg["VAT_SYMBOLS"] = {'S':'Standard', 'R':'Reduced', 'N':'Not Registered', 'Z':'Zero', 'E':'Excempt'}

    # The date to which the database was last deleted
    vs_cfg["DELETE_TO_DATE"] = None

    # All configuration items have been set ok
    return True


def read_defaults(scriptdirectory, config_location):
    "Read the defaults from the config file with filepath config_location"

    home = os.getenv("HOME", os.getenv("HOMEPATH", os.path.join(scriptdirectory, "vatstuff_files")))

    # set basic locations, in case the config file has items missing

    vs_cfg["IMAGES_FOLDER"]=os.path.join(scriptdirectory, "vatstuff_files", "images")
    vs_cfg["HTML_FOLDER"]=os.path.join(scriptdirectory, "vatstuff_files", "html")
    vs_cfg["DATABASE_FOLDER"] = home
    vs_cfg["EXPORT_FOLDER"] = home
    vs_cfg["BACKUP_FOLDER"] = home
    vs_cfg["LOG_FOLDER"]=os.path.join(scriptdirectory, "vatstuff_files", "log")

    # set default VAT Standard and Reduced rates

    # These are the default Government set rates, used when a new database is created
    # they are normally overridden by the values read from the config file
    # These values are strings
    vs_cfg["DEFAULT_VAT_STANDARD_RATE"] = "20.0"
    vs_cfg["DEFAULT_VAT_REDUCED_RATE"] = "5.0"

    # Default currency symbols, used when a new database is created
    vs_cfg["CURRENCY_SYMBOL"] = "U+00A3"     # the pound stirling unicode character
    vs_cfg["CURRENCY_POINT"]  = "U+002E"     # the decimal point character
    vs_cfg["CURRENCY_THOUSANDS_SEPARATOR"]  = "U+002C"     # the comma character

    # Default logging and debug mode
    vs_cfg["LOGGING"] = True
    vs_cfg["DEBUG"] = False

    if not os.path.exists(config_location):
        return False
    # Create a ConfigParser object, to read the config file
    cfg=ConfigParser.ConfigParser()
    fp=open(config_location, "rb")
    cfg.readfp(fp)
    fp.close()
    # Read each parameter in turn
    # Folders section values
    if cfg.has_option("folders", "database_folder"):
        vs_cfg["DATABASE_FOLDER"]=cfg.get("folders", "database_folder")
        vs_cfg["DATABASE_FOLDER"]=vs_cfg["DATABASE_FOLDER"].replace("/", os.sep)
        vs_cfg["DATABASE_FOLDER"]=vs_cfg["DATABASE_FOLDER"].replace("\\", os.sep)
    if cfg.has_option("folders", "images_folder"):
        vs_cfg["IMAGES_FOLDER"]=cfg.get("folders", "images_folder")
        vs_cfg["IMAGES_FOLDER"]=vs_cfg["IMAGES_FOLDER"].replace("/", os.sep)
        vs_cfg["IMAGES_FOLDER"]=vs_cfg["IMAGES_FOLDER"].replace("\\", os.sep)
    if cfg.has_option("folders", "html_folder"):
        vs_cfg["HTML_FOLDER"]=cfg.get("folders", "html_folder")
        vs_cfg["HTML_FOLDER"]=vs_cfg["HTML_FOLDER"].replace("/", os.sep)
        vs_cfg["HTML_FOLDER"]=vs_cfg["HTML_FOLDER"].replace("\\", os.sep)
    if cfg.has_option("folders", "export_folder"):
        vs_cfg["EXPORT_FOLDER"]=cfg.get("folders", "export_folder")
        vs_cfg["EXPORT_FOLDER"]=vs_cfg["EXPORT_FOLDER"].replace("/", os.sep)
        vs_cfg["EXPORT_FOLDER"]=vs_cfg["EXPORT_FOLDER"].replace("\\", os.sep)
    if cfg.has_option("folders", "backup_folder"):
        vs_cfg["BACKUP_FOLDER"]=cfg.get("folders", "backup_folder")
        vs_cfg["BACKUP_FOLDER"]=vs_cfg["BACKUP_FOLDER"].replace("/", os.sep)
        vs_cfg["BACKUP_FOLDER"]=vs_cfg["BACKUP_FOLDER"].replace("\\", os.sep)
    if cfg.has_option("folders", "log_folder"):
        vs_cfg["LOG_FOLDER"]=cfg.get("folders", "log_folder")
        vs_cfg["LOG_FOLDER"]=vs_cfg["LOG_FOLDER"].replace("/", os.sep)
        vs_cfg["LOG_FOLDER"]=vs_cfg["LOG_FOLDER"].replace("\\", os.sep)

    # Rates section values
    if cfg.has_option("rates", "vat_standard_rate"):
        vs_cfg["DEFAULT_VAT_STANDARD_RATE"]=cfg.get("rates", "vat_standard_rate")
    if cfg.has_option("rates", "vat_reduced_rate"):
        vs_cfg["DEFAULT_VAT_REDUCED_RATE"]=cfg.get("rates", "vat_reduced_rate")

    # Currency symbols section values
    if cfg.has_option("symbols", "currency_symbol"):
        vs_cfg["CURRENCY_SYMBOL"]=cfg.get("symbols", "currency_symbol")
    if cfg.has_option("symbols", "currency_point"):
        vs_cfg["CURRENCY_POINT"]=cfg.get("symbols", "currency_point")
    if cfg.has_option("symbols", "currency_thousands_separator"):
        vs_cfg["CURRENCY_THOUSANDS_SEPARATOR"]=cfg.get("symbols", "currency_thousands_separator")

    # logging section values
    if cfg.has_option("logging", "enabled"):
        log_enabled=cfg.get("logging", "enabled")
        if log_enabled.lower() == 'no':
            vs_cfg["LOGGING"] = False
    if cfg.has_option("logging", "debug"):
        debug_enabled=cfg.get("logging", "debug")
        if debug_enabled.lower() == 'yes':
            vs_cfg["DEBUG"] = True

    # These are the user-configured rates, initially set as per defaults
    # These values are Decimal objects
    try:
        vs_cfg["VAT_STANDARD_RATE"] = D(vs_cfg["DEFAULT_VAT_STANDARD_RATE"])
        vs_cfg["VAT_REDUCED_RATE"] = D(vs_cfg["DEFAULT_VAT_REDUCED_RATE"])
    except Exception:
        return False

    # Replace %SCRIPTDIRECTORY% for the installation scriptdirectory
    # Replace %HOME% for the users home directory

    vs_cfg["DATABASE_FOLDER"] = os.path.normpath(vs_cfg["DATABASE_FOLDER"].replace("%HOME%", home))
    vs_cfg["DATABASE_FOLDER"] = os.path.normpath(vs_cfg["DATABASE_FOLDER"].replace("%SCRIPTDIRECTORY%",
                                                                                    scriptdirectory))
    vs_cfg["IMAGES_FOLDER"] = os.path.normpath(vs_cfg["IMAGES_FOLDER"].replace("%HOME%", home))
    vs_cfg["IMAGES_FOLDER"] = os.path.normpath(vs_cfg["IMAGES_FOLDER"].replace("%SCRIPTDIRECTORY%", scriptdirectory))
    vs_cfg["HTML_FOLDER"] = os.path.normpath(vs_cfg["HTML_FOLDER"].replace("%HOME%", home))
    vs_cfg["HTML_FOLDER"] = os.path.normpath(vs_cfg["HTML_FOLDER"].replace("%SCRIPTDIRECTORY%", scriptdirectory))
    vs_cfg["EXPORT_FOLDER"] = os.path.normpath(vs_cfg["EXPORT_FOLDER"].replace("%HOME%", home))
    vs_cfg["EXPORT_FOLDER"] = os.path.normpath(vs_cfg["EXPORT_FOLDER"].replace("%SCRIPTDIRECTORY%", scriptdirectory))
    vs_cfg["BACKUP_FOLDER"] = os.path.normpath(vs_cfg["BACKUP_FOLDER"].replace("%HOME%", home))
    vs_cfg["BACKUP_FOLDER"] = os.path.normpath(vs_cfg["BACKUP_FOLDER"].replace("%SCRIPTDIRECTORY%", scriptdirectory))
    vs_cfg["LOG_FOLDER"] = os.path.normpath(vs_cfg["LOG_FOLDER"].replace("%HOME%", home))
    vs_cfg["LOG_FOLDER"] = os.path.normpath(vs_cfg["LOG_FOLDER"].replace("%SCRIPTDIRECTORY%", scriptdirectory))

    # Fail if ".." exists in any directory
    if ".." in vs_cfg["DATABASE_FOLDER"]:
        return False
    if ".." in vs_cfg["IMAGES_FOLDER"]:
        return False
    if ".." in vs_cfg["HTML_FOLDER"]:
        return False
    if ".." in vs_cfg["LOG_FOLDER"]:
        return False
    if ".." in vs_cfg["EXPORT_FOLDER"]:
        return False
    if ".." in vs_cfg["BACKUP_FOLDER"]:
        return False

    # Check the directories exist
    if not os.path.isdir(vs_cfg["DATABASE_FOLDER"]):
        # replace it with home
        vs_cfg["DATABASE_FOLDER"] = home
        # Now check if home exists
        if not os.path.isdir(vs_cfg["DATABASE_FOLDER"]):
            # home does not exist, exit setup
            return False
    # If these do not exist, make them the same as the Database Folder
    if not os.path.isdir(vs_cfg["EXPORT_FOLDER"]):
        vs_cfg["EXPORT_FOLDER"] = vs_cfg["DATABASE_FOLDER"]
    if not os.path.isdir(vs_cfg["BACKUP_FOLDER"]):
        vs_cfg["BACKUP_FOLDER"] = vs_cfg["DATABASE_FOLDER"]
    # The images and html folder must exist for the program to work
    if not os.path.isdir(vs_cfg["IMAGES_FOLDER"]):
        return False
    if not os.path.isdir(vs_cfg["HTML_FOLDER"]):
        return False
    # The log folder is not checked here, if it does not exist
    # then logging is turned off, this is done in set_logger
    return True


##################  Per-User persistent data functions  ######################


def get_stored_item(key):
    """Gets the item from the per-user persistent store given its key,
       and also sets the corresponding vs_cfg value
       if store access is not successfull return the original vs_cfg value"""

    # Check key is in vs_cfg
    assert key in vs_cfg

    if not vs_cfg["STORE_ENABLED"]:
        return vs_cfg[key]

    # get the store filepath
    filepath = vs_cfg["STORE_FILEPATH"]
    db = None
    try:
        db = shelve.open(filepath, protocol=2)
        # recover object from the store
        if db.has_key(key):
            result = db[key]
        else:
            # If the value is not yet in the store, put it in
            db[key] = vs_cfg[key]
            return vs_cfg[key]
    except Exception, e:
        exception_log(e)
        # something wrong with the persistent storage, disable it
        vs_cfg["STORE_ENABLED"] = False
        return vs_cfg[key]
    finally:
        if db:
            db.close()
    # update vs_cfg from the store
    vs_cfg[key] = result
    return vs_cfg[key]


def set_stored_item(key, item):
    """Sets the item in the per-user persistent store given its
       key and the new value and also sets vs_cfg"""

    # Check key is in vs_cfg
    assert key in vs_cfg

    # set the vs_cfg[key] to the item
    vs_cfg[key] = item

    if not vs_cfg["STORE_ENABLED"]:
        return

    # get the per-user persistent store filepath
    filepath = vs_cfg["STORE_FILEPATH"]
    db = None
    try:
        db = shelve.open(filepath, protocol=2)
        # set object into the per-user persistent store
        db[key] = item
    except Exception, e:
        exception_log(e)
        # something wrong with the persistent storage, disable it
        vs_cfg["STORE_ENABLED"] = False
    finally:
        if db:
            db.close()


######################  Logging Functions  ##############################

def set_logger():
    "Set up the logger"

    global _vs_logger
    if not vs_cfg["LOGGING"]:
        # logging has been turned off
        _vs_logger = None
        return


    if not os.path.isdir(vs_cfg["LOG_FOLDER"]):
        try:
            os.makedirs(vs_cfg["LOG_FOLDER"])
        except OSError:
            # if the log folder does not exist,
            # and cannot be made, turn off logging
            vs_cfg["LOGGING"] = False
            _vs_logger = None
            return

    try:
        _vs_logger = logging.getLogger(vs_cfg["PROGRAM_ID"])

        # Set the logger level
        if vs_cfg["DEBUG"]:
            _vs_logger.setLevel(logging.DEBUG)
        else:
            _vs_logger.setLevel(logging.INFO)

        # Set the format
        form = logging.Formatter("%(asctime)s - %(process)d - %(levelname)s - %(message)s")

        # Add the log message handler to the logger
        handler = logging.handlers.RotatingFileHandler(
              vs_cfg["LOG_FILEPATH"], maxBytes=20000, backupCount=5)
              # note: from python 2.6 a delay=True parameter can be added which would be useful
              # add this when the program is converted to python 3

        handler.setFormatter(form)

        _vs_logger.addHandler(handler)
    except Exception:
        # If an error occurs, turn off logging
        vs_cfg["LOGGING"] = False
        _vs_logger = None


def error_log(message):
    "Log message with level error"
    global _vs_logger
    if not vs_cfg["LOGGING"]: return
    try:
        if _vs_logger:
            _vs_logger.error(str(message))
    except Exception:
        # If an error occurs, turn off logging
        vs_cfg["LOGGING"] = False
        _vs_logger = None


def info_log(message):
    "Log message with level info"
    global _vs_logger
    if not vs_cfg["LOGGING"]: return
    try:
        if _vs_logger:
            _vs_logger.info(str(message))
    except Exception, e:
        # If an error occurs, turn off logging
        vs_cfg["LOGGING"] = False
        _vs_logger = None


def debug_log(message):
    "Log message with level debug"
    global _vs_logger
    if not vs_cfg["LOGGING"]: return
    try:
        if _vs_logger and vs_cfg["DEBUG"]:
            _vs_logger.debug(str(message))
    except Exception:
        # If an error occurs, turn off logging
        vs_cfg["LOGGING"] = False
        _vs_logger = None


def class_log(module_name, obj):
    """Logs a module and class name, typically used in the __init__ function
       of a class with a call such as; class_log(__name__, self)"""
    global _vs_logger
    if not vs_cfg["LOGGING"]: return
    if _vs_logger and vs_cfg["DEBUG"]:
        debug_log("Creating an instance of %s.%s" % (module_name, obj.__class__.__name__))


def logmethod(f):
    """Creates a decorator to log a method"""
    @wraps(f)
    def wrapper(self, *args, **kwds):
        debug_log("%s in %s" % (f.__name__, self.__class__.__name__))
        return f(self, *args, **kwds)
    return wrapper


def exception_log(message):
    "Log message with level error plus exception traceback"
    global _vs_logger
    if not vs_cfg["LOGGING"]: return
    try:
        if _vs_logger:
            _vs_logger.exception(str(message))
    except Exception:
        # If an error occurs, turn off logging
        vs_cfg["LOGGING"] = False
        _vs_logger = None


###################### Currency symbol functions #################################

def vs_unicode_symbol(vs_unicode_designation):
    """Returns the unicode_symbol given a unicode designation"""
    vs_hex = vs_unicode_designation[2:]        # Remove the U+, leaving just the hex value
    vs_int = int(vs_hex,16)                    # Get the numeric value of the code
    vs_sym = unichr(vs_int)                    # And that should be it
    return vs_sym


def proper_filename(filename, default_filename, append_date=None):
    """Gets rid of any non alphanumeric characters in filename, replacing them
       with underscore characters. If, after removing non alphanumeric characters,
       and any leading underscores, the result has no length, use default_filename
       instead. Then, if a date is given, append the isoformat version of
       the date and return the result.
       As any dots are also removed - you should append a file extension after
       calling this function, rather than before."""
    new_filename = ""
    for character in filename:
        if character.isalnum():
            new_filename = "".join([new_filename, character])
        else:
            new_filename = "".join([new_filename, "_"])
    new_filename = new_filename.lstrip("_")
    if not new_filename:
        new_filename=default_filename
    if append_date:      
        return "".join([new_filename, "_", append_date.isoformat()])
    else:
        return new_filename


class CURRENCY_ERROR(Exception):
    "Error raised in Currency"
    pass


class Currency(object):
    """An object with attribute 'dec' which is a decimal.Decimal object
       and converts to a string with full currency formatting"""

    def __init__(self, value):
        """Converts a value (string, integer, Currency or decimal.Decimal)
           to a decimal.Decimal object with two decimal places.
           If a decimal.Decimal object is given, it is rounded down to two decimal
           places.
           The string could have the currency symbol and currency point in it.
           Sets self.__decimal_value if the given value format is correct
           or raises a CURRENCY_ERROR if the given value is incorrect"""

        if isinstance(value, decimal.Decimal):
            self.__decimal_value = value.quantize(D('.01'), rounding=decimal.ROUND_DOWN)
            return

        try:
            value = unicode(value)
        except Exception:
            raise CURRENCY_ERROR

        # Take off any leading white space
        value = value.strip()

        # Don't accept a length greater than 20 characters
        # ( 999,999,999,999.99 plus negative and currency symbol)
        if len(value)>20 : raise CURRENCY_ERROR

        # Remove leading currency symbol if it is there
        value = value.strip(vs_cfg["CS"])

        # Take off any leading white space
        value = value.strip()

        # If value is blank, make it equal to zero
        if not value:
            self.__decimal_value = decimal.Decimal("0.00")
            return

        # remove leading -ve sign if it is there
        if value[0] == "-" :
            neg = "-"
        else:
            neg = ""
        value = value.strip("-")

        # Take off any leading white space
        value = value.strip()

        # if currency symbol is after the -, remove it if it is there
        value = value.strip(vs_cfg["CS"])

        # Take off any leading white space
        value = value.strip()

        # Remove any thousand separators
        value = value.replace(vs_cfg["CTS"],"")

        # After these removals, check we have not been left with an empty string
        if not value:
            self.__decimal_value = decimal.Decimal("0.00")
            return

        # So the negative and currency symbols have been taken off, should now be left with digits and
        # a currency point symbol only, if currency point not there - value is whole currency units,
        # without cents/pennies.

        if value.count(vs_cfg["CP"]) == 0:
            # There is no decimal point, the value is an integer number
            if not value.isdigit(): raise CURRENCY_ERROR
            # if value is zero, avoid negative zero
            if decimal.Decimal(value) == decimal.Decimal(0):
                result_string = "0.00"
            else:
                result_string = "%s%s.00" % (neg, value)
            self.__decimal_value = decimal.Decimal(result_string)
            return

        if value.count(vs_cfg["CP"]) > 1:
            # There is more than one currency point in the string!
            raise CURRENCY_ERROR

        # Split the value into two halves
        pounds,pennies=value.split(vs_cfg["CP"])

        if not pennies:
            pennies = "00"
        if not pennies.isdigit(): raise CURRENCY_ERROR
        if len(pennies) > 2: raise CURRENCY_ERROR
        if len(pennies) == 1:
            pennies += "0"

        if not pounds:
            pounds = "0"
        if not pounds.isdigit(): raise CURRENCY_ERROR

        result_string = "%s.%s" % (pounds, pennies)

        # Avoid negative zero
        if decimal.Decimal(result_string) == decimal.Decimal(0):
            result_string = "0.00"
        else:
            result_string = "%s%s" % (neg, result_string)

        self.__decimal_value = decimal.Decimal(result_string)


    @property
    def dec(self):
        "The decimal value of the currency"
        return self.__decimal_value

    @property
    def dec_string(self):
        """The string version of the currency but with only
           the CP changed - no currency symbol or thousands separators"""
        return unicode(self.__decimal_value).replace(".", vs_cfg["CP"])

    @property
    def negative(self):
        "True if the value is negative"
        if self.__decimal_value < decimal.Decimal(0):
            return True
        else:
            return False


    def __str__(self):
        """Returns a unicode string of the currency value, complete
           with currency, thousands separator and currency point symbols"""
        neg = False
        if self.__decimal_value < decimal.Decimal(0):
            neg = True
        # replace '.' with the preferred currency point symbol
        value = unicode(abs(self.__decimal_value)).replace(".", vs_cfg["CP"])
        # Insert a CTS every three digits, with the first
        # at 6 digits from the right (i.e - when greater than 999.99)
        len_test = 6
        while len(value)>len_test:
            value = u"%s%s%s" % (value[:-len_test], vs_cfg["CTS"], value[-len_test:])
            len_test += 4
        # If negative, put a '-' sign before a currency symbol
        if neg:
            value = u"".join(("-",vs_cfg["CS"],value))
        else:
            value = u"".join((vs_cfg["CS"],value))
        return value

    def __bool__(self):
        "Test if this Currency value is non-zero"
        if self.__decimal_value:
            return True
        else:
            return False

    def __nonzero__(self):
        "Test if this Currency value is non-zero"
        if self.__decimal_value:
            return True
        else:
            return False

    def __add__(self,other):
        "Add two Currency values"
        if isinstance(other, self.__class__):
            new_decimal_value = self.__decimal_value + other.dec
            return Currency(new_decimal_value)
        else:
            return NotImplemented

    def __sub__(self,other):
        "Subtract two Currency values"
        if isinstance(other, self.__class__):
            new_decimal_value = self.__decimal_value - other.dec
            return Currency(new_decimal_value)
        else:
            return NotImplemented

    def __eq__(self,other):
        "Test if two Currency values are equal"
        if isinstance(other, self.__class__):
            return self.__decimal_value == other.dec
        else:
            return False

    def __ne__(self,other):
        "Test if two Currency values are not equal"
        if isinstance(other, self.__class__):
            return self.__decimal_value != other.dec
        else:
            return True


def to_currency_string(value):
    """Input variable is given as a string, decimal, integer or Currency,
       converts to string formatted with the currency symbol, thousands separator
       and the currency point.  If the input is a decimal.Decimal value then
       rounds it down to two decimal places. Raises a CURRENCY_ERROR if the
       given value cannot be converted to a currency string."""
    currency_value = Currency(value)
    return unicode(currency_value)


def string_to_decimal(value):
    """Converts a currency string such as 3.76 to decimal.Decimal("3.76")
       The string could have the currency symbol and currency point in it
       returns a status flag, and decimal value if format was correct
       Returns (status, decimal value)
       where status is True if conversion ok, False if not"""
    try:
        currency_value = Currency(value)
    except CURRENCY_ERROR:
        return False, D(0)
    return True, currency_value.dec


#################### Create new unique logo filename ##################################

def create_logo_name():
    "Returns a logo filepath and name that does not currently exist in LOGO_FOLDER"
    # Create a random number between 1 and 1000000
    while True:
        rnumber = random.randint(1, 1000000)
        logo_name = "logo_%s.png" % rnumber
        logo_filepath = os.path.join(vs_cfg["LOGO_FOLDER"], logo_name)
        if not os.path.exists(logo_filepath):
            # This file does not exist, so it is unique in the folder
            return logo_filepath, logo_name



