####### VATStuff #######
#
# objects.py  - Accesses database table rows as objects
#
# Version : 0.7
# Date : 20100910
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#


"""Accesses database table rows as objects.

   The module consists of a number of utility functions for openning
   and accessing the database - select, list_select, update and insert

   These are then followed by _VSDatabase - which is not instantiated
   itself, but provides common attributes and methods for the remaining
   classes which inherit from it

   Following _VSDatabase is a class definition for each table
"""

import decimal
import datetime
import calendar
import sqlite3

from vatstuff_packages.vs_utilities.config import *

D = decimal.Decimal   # D used to avoid typing


##########################################################################
###
### Utility functions that do some of the database
### access work used by many of the classes
###
##########################################################################


#################### This function connects to the database ###########################

def vs_connect():
    """Connects to the database and returns (connection, cursor)
       All calls to the database use this function"""
        
    connection = sqlite3.connect(vs_cfg["DATABASE_FILEPATH"], detect_types=sqlite3.PARSE_DECLTYPES)
    cursor=connection.cursor()

    # the 'detect_types=sqlite3.PARSE_DECLTYPES' is used so sqlite3
    # understands python datetime.date objects.  If another database was used that did not
    # understand them - there would have to be a conversion for every datetime.date object
    # saved and read.

    return connection, cursor


# This assumes a single parameter is being asked, not a list of values
def select(sql_string, args=None):
    """sql_string should be a SELECT statement requesting a single value
       this function returns the value requested"""
    # Note, this function is also called from the odd_functions module
    conn = None
    try:
        conn, c = vs_connect()
        if not args:
            c.execute(sql_string)
        else:
            c.execute(sql_string, args)
        # Get a single value, but as a tuple, with required value at index 0
        result = c.fetchone()
        # Note, there is no except clause, so if an exception occurs, it is propagated upwards
    finally:
        if conn:
            conn.close()
    if not result: return None
    return result[0]


# This assumes a list of single values is being requested
def list_select(sql_string, args=None):
    """sql_string should be a SELECT statementrequesting multiple values of a
       single parameter, this function returns the values in a list"""
    conn = None
    try:
        conn, c = vs_connect()
        if not args:
            c.execute(sql_string)
        else:
            c.execute(sql_string, args)
        # Get multiple values in a tuple, but each value is itself a tuple
        result = c.fetchall()
        # Note, there is no except clause, so if an exception occurs, it is propagated upwards
    finally:
        if conn:
            conn.close()
    if not result: return []
    # return a list of single values (not a tuple of tuples!)
    return [ item[0] for item in result ]


# can also be used for DELETE commands since it simply executes the string
def update(sql_string, args=None):
    """Executes and committs sql_string, used for UPDATE and DELETE"""
    if not vs_cfg["WRITE_ACCESS"]:
        # Another process has locked the database
        raise VS_ANOTHER_PROCESS, "Database change cancelled - another process has locked this database" 
    conn = None
    try:
        conn, c = vs_connect()
        # update the database
        if not args:
            c.execute(sql_string)
        else:
            c.execute(sql_string, args)
        conn.commit()
        # Note, there is no except clause, so if an exception occurs, it is propagated upwards
    finally:
        if conn:
            conn.close()


def insert(sql_string, args=None):
    "Executes an insert statement and returns the row id of the insert"
    if not vs_cfg["WRITE_ACCESS"]:
        # Another process has locked the database
        raise VS_ANOTHER_PROCESS, "Database change cancelled - another process has locked this database"
    conn = None
    try:
        conn, c = vs_connect()
        if not args:
            c.execute(sql_string)
        else:
            c.execute(sql_string, args)
        c.execute("SELECT last_insert_rowid()")
        result = c.fetchone()
        conn.commit()
        # Note, there is no except clause, so if an exception occurs, it is propagated upwards
    finally:
        if conn:
            conn.close()
    if not result: return None
    # returns the rowid of the insert
    return result[0]


################### The parent _VSDatabase object ############################################
##
## All the subsequent classes in this module are children of this parent class, each of these
## children describe rows of a table, and an instance is created with the id of a row
##
##############################################################################################

class _VSDatabase(object):
    """A class describing a row in a table, this parent database object is never
       directly instantiated, but is used by child classes - one for each table"""

    # The table_name and id_name are class attributes
    # as this parent database object is never instantiated
    # the values set here are dummies, and are replaced within each child class

    # The table name this class refers to, these are dummy names
    table_name = "TABLE_NAME"
    id_name = "ID_NAME"

    def __init__(self, row_id):
        """Defines a row from the table with the given id, by setting self.id = row_id."""
        # Make sure the id is an integer
        row_id = int(row_id)
        if row_id < 1:
            raise VS_ERROR, "The id must be a non zero integer"
        # Check it exists in the table
        sql_string = "SELECT %s FROM %s WHERE %s = ?" % (self.id_name, self.table_name, self.id_name)
        result = select(sql_string, (row_id,))
        if not result:
            raise VS_ERROR, "ID %i is not found on table %s" % (row_id, self.table_name)
        # setting self.id, defines that the instance created refers to the row with this id
        self.id = row_id
        # If this is an object which can have a pdf file created, set local copies of
        # the pdf dimensions, initially None, but could be read from database or elsewhere
        self._pdf_height = None
        self._pdf_width = None


    # class method to get a list of all rows of this table
    @classmethod
    def get_all_ordered_by(cls, order_by=None):
        """Get all rows ordered by row id as default, or by other column name if order_by
           is given.  This should not be called for an object which has a potentially large
           number of rows - since a very large list may be produced"""
        if not order_by:
            order_by = cls.id_name
        sql_string = "SELECT %s FROM %s ORDER BY %s" % (cls.id_name, cls.table_name, order_by)
        result = list_select(sql_string)
        if not result: return []
        return [ cls(result_id) for result_id in result ]


    # class method to get a row, given a column value (must be a column of unique values)
    @classmethod
    def object_from_field(cls, col_name, col_value):
        """Returns an object of type cls, given a column name and value
           the column must have unique entries, so that only one row matches
           the column value.
           If no match is found, return None
           Therefore this function finds a row on the table given by a value
           in a given column"""
        if not col_name: raise VS_ERROR, "Function arguments incorrect"
        if col_value is None: raise VS_ERROR, "Search value cannot be None"
        if " " in col_name:
            raise VS_ERROR, "%s does not have column %s" % (self.table_name, col_name)
        sql_string = "SELECT %s FROM %s WHERE %s = ?" % (cls.id_name, cls.table_name, col_name)
        row_id = select(sql_string, (col_value,))
        if not row_id: return None
        return cls(row_id)


    def new_row(self):
        """Given an instance - which refers to a table row, create a copy of the row
           and use it to create a new row returning the new row id"""

        # In some classes, when updating an attribute, the old row needs to be saved
        # and a new row created into which the new attribute will be inserted.
        # This method is used to create such a new row

        if not vs_cfg["WRITE_ACCESS"]:
            # Another process has locked the database
            raise VS_ANOTHER_PROCESS, "Database change cancelled - another process has locked this database" 

        # 1st task is to get a list of the column names, using the cursor.description attribute
        conn = None
        try:
            conn, c = vs_connect()

            c.execute("SELECT * FROM %s" % self.table_name)
            col_name_list = [item[0] for item in c.description]

            # The list has the id column name, which we don't want
            col_name_list.remove(self.id_name)
            # get list as a string
            col_name_list = str(col_name_list)
            # remove starting and ending []'s
            col_names = col_name_list[1:-1]
            # remove all quote marks
            col_names = col_names.replace("\"", " ")
            col_names = col_names.replace("'", " ")

            # This statement creates a copy of a row

            sql_string = "INSERT INTO %s (%s) SELECT %s FROM %s WHERE %s = ?" % (self.table_name,
                                                                                col_names,
                                                                                col_names,
                                                                                self.table_name,
                                                                                self.id_name)
            c.execute(sql_string, (self.id,))
            c.execute("SELECT last_insert_rowid()")
            result = c.fetchone()
            conn.commit()
            # Note, there is no except clause, so if an exception occurs, it is propagated upwards
        finally:
            if conn:
                conn.close()
        if not result: return None
        # returns the rowid of the insert
        return result[0]


    def get_column(self, col_name):
        "Given an instance - which refers to a table row, return the item in column name"
        # col_name must be a column string - check there are no spaces
        if " " in col_name:
            raise VS_ERROR, "%s does not have column %s" % (self.table_name, col_name)

        sql_string = "SELECT %s FROM %s WHERE %s = ?" % (col_name, self.table_name, self.id_name)
        val = select(sql_string, (self.id,))
        return val


    def set_column(self, col_name, val=None):
        "Given an instance - which refers to a table row, update the item in column name to val"
        if val == self.get_column(col_name):
            # If this database item is already equal to val, then no change
            return
        sql_string = "UPDATE %s SET %s = ? WHERE %s = ?" % (self.table_name, col_name, self.id_name)
        update(sql_string, (val, self.id))


    def transaction_description_date(self):
        """Its a fairly common requirement for an object (such as a payment)
           to require the associated transaction description and datestamp - 
           so this method does just that.  If the table does not have a transaction
           id column, this method should not be called as it is not relevant"""

        sql_string = """SELECT DESCRIPTION, DATESTAMP
                        FROM TRANSACTIONS, %s
                        WHERE %s = ?
                        AND %s.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID""" % (self.table_name,
                                                                                  self.id_name,
                                                                                  self.table_name)
        conn = None
        try:
            conn, c = vs_connect()
            c.execute(sql_string, (self.id,))
            # Get the two values as a tuple
            result = c.fetchone()
        finally:
            if conn:
                conn.close()
        if not result: return (None, None)
        return result

    # This property provides an attribute with the associated transaction description and date
    desc_date  = property(transaction_description_date)

    # The pdf width and height are property attributes, with their getter and setters here
    # These are dealt with rather different to other properties, since it is required that
    # the user can set them - even if they have no write access to the database, otherwise
    # a read-only user would not be able to print an invoice with their required dimensions

    # So as default the local values self._pdf_width and self._pdf_height are set to None
    # and the database values are read - but if an error occurs setting database values, such as
    # the user not having write access, or the table not even having a column for these values,
    # then the local attributes are set and used

    def get_pdf_width(self):
        "getter for pdf_width property"
        # return a local copy if one exists
        if self._pdf_width:
            return self._pdf_width
        # try to return one from the database
        try:
            pdf_width = self.get_column("PDF_WIDTH")
        except Exception:
            pdf_width = None           
        return pdf_width

    def get_pdf_height(self):
        "getter for pdf_height property"
        # return a local copy if one exists
        if self._pdf_height:
            return self._pdf_height
        # try to return one from the database
        try:
            pdf_height = self.get_column("PDF_HEIGHT")
        except Exception:
            pdf_height = None           
        return pdf_height

    def set_pdf_width(self, val):
        "setter for pdf_width property"
        # If this fails i.e. someone else is using the database
        # or this table does not have a PDF_WIDTH column, just set a local copy
        try:
            self.set_column("PDF_WIDTH", val)
            self._pdf_width = None
        except Exception:
            self._pdf_width = val

    def set_pdf_height(self, val):
        "setter for pdf_height property"
        # If this fails i.e. someone else is using the database
        # or this table does not have a PDF_HEIGHT column, just set a local copy
        try:
            self.set_column("PDF_HEIGHT", val)
            self._pdf_height = None
        except Exception:
            self._pdf_height = val

    pdf_width  = property(get_pdf_width,    set_pdf_width)
    pdf_height = property(get_pdf_height,   set_pdf_height)

    def __eq__(self,other):
        if isinstance(other, self.__class__):
            return self.id == other.id
        else:
            return False

    def __ne__(self,other):
        if isinstance(other, self.__class__):
            return self.id != other.id
        else:
            return True

    def __repr__(self):
        return "%s(%s)" % (self.__class__.__name__, self.id)

    # Note - very often __str__ will be overidden by subclasses
    # to show a better printed representation
    def __str__(self):
        return "%s(%s)" % (self.__class__.__name__, self.id)

    # An instance should always test to True, as no id found is an
    # error, setting these to return True ensures this, even if the
    # subclass defines a __len__ which may be empty (as in the case
    # of the Account subclass)

    def __bool__(self):
        return True

    def __nonzero__(self):
        return True 


######################################################################
#
# The classes below are now child classes of _VSDatabase
#
# each specifies its own table_name and id_name as class attributes
# and defines property attributes appropriate to the database table
# the class represents
#
# each class is instantiated with a row id of its table - and then
# its properties are attributes of the appropriate columns 
#
######################################################################


####################### Folder #######################################

class Folder(_VSDatabase):
    """An object describing a row in the FOLDERS table - called with Folder(id)
       which gets a folder from the FOLDERS table with the given id.
       To create an entirely new folder on the database, use function
       create_objects.create_Folder"""

    # The table_name and id_name are class attributes
    table_name = "FOLDERS"
    id_name = "FOLDER_ID"

    # Class methods - return one (or more) Folder objects

    # call this with Folder.get_all()
    @classmethod
    def get_all(cls):
        "Get all folders ordered by folder name"
        return cls.get_all_ordered_by("NAME")

    # call this with Folder.get_non_system()
    @classmethod
    def get_non_system(cls):
        "Get all non-system folders ordered by folder name"
        result = list_select("SELECT FOLDER_ID FROM FOLDERS WHERE SYSTEM < 2 ORDER BY NAME")
        # result is a list of folder_id's
        if not result: return []
        return [ cls(folder_id) for folder_id in result ]


    # call this with Folder.from_name(name)
    @classmethod
    def from_name(cls, name):
        "return folder from the given name, or None if the name is not found"
        return cls.object_from_field("NAME", name)

    # call this with Folder.get_first()
    @classmethod
    def get_first(cls):
        "Get first folder, by alphabetical name"
        first_folder_id = select("SELECT FOLDER_ID FROM FOLDERS ORDER BY NAME")
        return cls(first_folder_id) if first_folder_id else None

    # call this with Folder.get_list()
    @classmethod
    def get_list(cls):
        """Returns a list of lists of folder information
           this list being used to create the accounts tree under the accounts tab"""
        conn = None
        try:
            # Make the connection
            conn, c = vs_connect()

            c.execute("""SELECT NAME,
                                DESCRIPTION,
                                EXPANDED,
                                FOLDER_ID
                            FROM FOLDERS
                            ORDER BY NAME""")
            result = c.fetchall()
        finally:
            if conn:
                conn.close()
        return result

    # The following methods are property functions, used to get and set an attribute of the class
        
    def get_name(self):
        "Return the folders name - used by the 'name' property"
        return self.get_column("NAME")

    def get_description(self):
        "Return the folders description - used by the 'description' property"
        return self.get_column("DESCRIPTION")

    def get_system(self):
        "Return the system flag"
        return self.get_column("SYSTEM")

    def get_expanded(self):
        "Return the expanded flag"
        return self.get_column("EXPANDED")
        
    def set_name(self, val):
        "Set the folders name - used by the 'name' property"
        self.set_column("NAME", val)

    def set_description(self, val):
        "Set the folders description - used by the 'description' property"
        self.set_column("DESCRIPTION", val)

    def set_system(self, val):
        "Set the system flag"
        self.set_column("SYSTEM", val)

    def set_expanded(self, val):
        "Set the expanded flag"
        self.set_column("EXPANDED", val)

    name        = property(get_name,        set_name,        doc="The folders name string")
    description = property(get_description, set_description, doc="The folders description string")
    system      = property(get_system,      set_system,      doc="The folders system flag")
    expanded    = property(get_expanded,    set_expanded,    doc="The folder expanded(True) or not(False) flag")

    # The following are methods, not covered by the property function
        
    def get_accounts(self):
        """Returns a list of those accounts within this folder"""
        result = list_select("SELECT ACCOUNT_ID FROM ACCOUNTS WHERE FOLDER_ID = ? ORDER BY NAME", (self.id,))
        # result is a list of account_id's
        if not result: return []
        return [ Account(account_id) for account_id in result ]

    def get_accounts_list(self):
        """Returns a list of lists of account information within this folder
           this list being used to create the accounts tree under the accounts tab"""
        conn = None
        try:
            # Make the connection
            conn, c = vs_connect()
            # Get the particular account from accounts
            c.execute("""SELECT ACCOUNTS.NAME,
                                ACCOUNTS.DESCRIPTION,
                                ACCOUNTS.BALANCE,
                                ACCOUNTTYPES.NAME,
                                ACCOUNTS.ACCOUNT_ID
                         FROM ACCOUNTS, FOLDERS, ACCOUNTTYPES
                         WHERE ACCOUNTS.FOLDER_ID = FOLDERS.FOLDER_ID
                         AND ACCOUNTS.ACCOUNTTYPE_ID = ACCOUNTTYPES.ACCOUNTTYPE_ID
                         AND FOLDERS.FOLDER_ID = ?
                         ORDER BY ACCOUNTS.NAME""", (self.id,))
            result = c.fetchall()
        finally:
            if conn:
                conn.close()
        return result

    def get_account_with_name(self, name):
        """Returns the account from this folder with the given name
           or if none found, returns None"""
        if not name: return None
        result = select("SELECT ACCOUNT_ID FROM ACCOUNTS WHERE FOLDER_ID = ? AND NAME = ?", (self.id, name))
        # result is the account_id of the required account
        return Account(result) if result else None

    def delete(self):
        """Deletes Folder from database"""
        if self.get_system() != 0 : return False
        number = select("SELECT COUNT(*) FROM ACCOUNTS WHERE FOLDER_ID = ?", (self.id,))
        if number:
            # Cannot delete folder
            return False
        # Delete the folder
        update("DELETE FROM FOLDERS WHERE FOLDER_ID = ?", (self.id,))
        return True


####################### AccountType ##############################

class AccountType(_VSDatabase):
    """An object describing a row in the ACCOUNTTYPES table  - called with AccountType(id)
       which gets an account type from the ACCOUNTTYPES table with the given id. To create an
       entirely new account type on the database, use function create_objects.create_AccountType"""

    # The table_name and id_name are class attributes
    table_name = "ACCOUNTTYPES"
    id_name = "ACCOUNTTYPE_ID"

    # call this with AccountType.get_all()
    @classmethod
    def get_all(cls):
        """Gets a list of all accounttypes ordered by accounttypes name"""
        return cls.get_all_ordered_by("NAME")

    # call this with AccountType.from_name(name):
    @classmethod
    def from_name(cls, name):
        """return accounttype from the given name, or None if the name is not found"""
        return cls.object_from_field("NAME", name)

    # The following methods are property functions, used to get and set an attribute of the class
        
    def get_name(self):
        return self.get_column("NAME")

    def get_description(self):
        return self.get_column("DESCRIPTION")

    def set_name(self, val):
        self.set_column("NAME", val)

    def set_description(self, val):
        self.set_column("DESCRIPTION", val)

    name = property(get_name, set_name)
    description = property(get_description, set_description)

    # The following are methods, not covered by the property function
        
    def get_accounts(self):
        """Returns a list of those accounts within this account type"""
        result = list_select("SELECT ACCOUNT_ID FROM ACCOUNTS WHERE ACCOUNTTYPE_ID = ? ORDER BY NAME", (self.id,))
        # result is a list of account_id's
        if not result: return []
        return [ Account(account_id) for account_id in result ]

    def delete(self):
        """Deletes account type from database"""
        number = select("SELECT COUNT(*) FROM ACCOUNTS WHERE ACCOUNTTYPE_ID = ?", (self.id,))
        if number != 0:
            # Cannot delete account type
            return False
        # Delete the account type
        update("DELETE FROM ACCOUNTTYPES WHERE ACCOUNTTYPE_ID = ?", (self.id,))
        return True


####################### Account ##############################

class Account(_VSDatabase):
    """An object describing a row in the ACCOUNTS table - called with Account(id)
       which gets an account from the ACCOUNTS table with the given id. To create an entirely
       new account on the database, use function create_objects.create_Account"""

    # The table_name and id_name are class attributes
    table_name = "ACCOUNTS"
    id_name = "ACCOUNT_ID"

    def __init__(self, id):
        """Calls parent __init__ which tests and sets self.id, then
           sets the self.ledger value"""
        _VSDatabase.__init__(self, id)
        self.ledger = "LEDGER_%i" % self.id

    # call this with Account.get_all()
    @classmethod
    def get_all(cls):
        "Get all accounts ordered by account name"
        return cls.get_all_ordered_by("NAME")

    # call this with Account.get_non_system()
    @classmethod
    def get_non_system(cls):
        "Get all non-system accounts ordered by folder name, account name"
        result = list_select("""SELECT ACCOUNT_ID FROM ACCOUNTS,FOLDERS
                                 WHERE ACCOUNTS.SYSTEM < 2
                                   AND ACCOUNTS.FOLDER_ID = FOLDERS.FOLDER_ID
                                 ORDER BY FOLDERS.NAME, ACCOUNTS.NAME""")
        # result is a list of account_id's
        if not result: return []
        return [ cls(account_id) for account_id in result ]

    # These methods make Account able to access the associated Ledger table as a sequence

    def __len__(self):
        "Return the number of entries in the associated ledger"
        return select("SELECT COUNT(*) FROM %s" % self.ledger)

    def __getitem__(self, gridrow):
        """This method allows a line of the account statement to be returned
           the index is the ledger gridrow.
           If a is an instance of Account, a[0] will return a list representing the first
           ledger entry, a[1] will be the next, if index is out of range then an IndexError
           will be raised"""
        if gridrow < 0: raise IndexError
        max_gridrow = self.__len__() - 1
        if max_gridrow < 0: raise IndexError
        if gridrow > max_gridrow: raise IndexError
        sqlstring = "SELECT TRANSACTION_ID, DEBIT, CREDIT, BALANCE FROM %s WHERE GRIDROW = ?" % self.ledger
        conn = None
        try:
            # Make the connection
            conn, c = vs_connect()
            # Get the particular account from accounts
            c.execute(sqlstring, (gridrow,))
            result = c.fetchone()
        finally:
            if conn:
                conn.close()
        if not result: return []
        result = [ Transaction(result[0]),
                   D(result[1]),
                   D(result[2]),
                   D(result[3]) ]
        return result

    def __contains__(self, transaction):
        "Returns True if the given transaction is within this ledger"
        result = select("SELECT COUNT(*) FROM %s WHERE TRANSACTION_ID = ?" % self.ledger, (transaction.id,))
        return True if result else None

    # The following methods are property functions, used to get and set an attribute of the class

    def get_folder(self):
        folder_id = self.get_column("FOLDER_ID")
        return Folder(folder_id) if folder_id else None

    def get_name(self):
        return self.get_column("NAME")

    def get_description(self):
        return self.get_column("DESCRIPTION")

    def get_balance(self):
        result = self.get_column("BALANCE")
        return D(result)     # returns the Decimal value of the string balance

    def get_currency(self):
        "This converts balance to string with the currency symbol and point, and leading - if negative"
        value = self.get_balance()
        # to_currency_string(value) is in the vatstuff_packages.vs_utilities.config module
        return to_currency_string(value)

    def get_system(self):
        return self.get_column("SYSTEM")

    def get_acc_type(self):
        accounttype_id = self.get_column("ACCOUNTTYPE_ID")
        return AccountType(accounttype_id) if accounttype_id else None

    def get_fullname(self):
        return "%s:%s" % (self.get_folder().name, self.get_name())

    def set_name(self, val):
        self.set_column("NAME", val)

    def set_description(self, val):
        self.set_column("DESCRIPTION", val)

    def set_system(self, val):
        self.set_column("SYSTEM", val)
   
    def set_folder(self, val):
        # update to the new folder value in the account
        self.set_column("FOLDER_ID", val.id)

    def set_balance(self, val):
        # update the balance, val is passed as a Decimal value, make sure it only
        # has two places of decimals, round down to the nearest penny.
        val = val.quantize(D('.01'), rounding=decimal.ROUND_DOWN)
        val = str(val)
        self.set_column("BALANCE", val)
        # Whenever an account balance update occurs, if the system flag is zero,
        # it is automatically updated to 1 via an sql trigger set when the table was created
        # so there is no need to do the system flag update here

    def set_acc_type(self, val):
        # update to the new account type in the account
        self.set_column("ACCOUNTTYPE_ID", val.id)

    name        = property(get_name, set_name)
    description = property(get_description, set_description)
    system      = property(get_system, set_system)
    folder      = property(get_folder, set_folder)
    balance     = property(get_balance, set_balance)
    currency    = property(get_currency)
    acc_type    = property(get_acc_type, set_acc_type)
    fullname    = property(get_fullname)   # No set_fullname


    def definition_count(self):
        """Gets the count of definitions acting on this account,
           used to indicate that the account cannot be deleted"""
        sql_string = "SELECT COUNT(*) FROM DEFINITIONS WHERE ACCOUNT_ID = ?"
        number = select(sql_string, (self.id,))
        return number


    def delete(self):
        """Deletes record from database, the object should be deleted after this"""
        if not vs_cfg["WRITE_ACCESS"]:
            # Another process has locked the database
            raise VS_ANOTHER_PROCESS, "Database change cancelled - another process has locked this database" 

        # Can only delete if system is zero and balance is zero
        if self.get_system() != 0 or self.get_balance() != D(0): return False

        # Check if any definition has this account_id
        if self.definition_count():
            # Cannot delete account
            return False
        
        # get the folder, this was in
        folder = self.get_folder()

        # Delete the account and its ledger as a single transaction       
        # Make the connection
        conn = None
        try:
            conn, c = vs_connect()

            # Delete the table associated with this account
            delete_table_string = "DROP TABLE LEDGER_%i" % self.id
            c.execute(delete_table_string)

            # Delete the account
            c.execute("DELETE FROM ACCOUNTS WHERE ACCOUNT_ID = ?", (self.id,))

            conn.commit()
        finally:
            if conn:
                conn.close()
        return True

    def __str__(self):
        return self.get_fullname()

        
####################### Label ##############################

class Label(_VSDatabase):
    """An object describing a row in the LABELS table - called with Label(id)
       which gets a label from the LABELS table with the given id. To create an entirely
       new label on the database, use function create_objects.create_Label"""
    # The table_name and id_name are class attributes
    table_name = "LABELS"
    id_name = "LABEL_ID"

    # call this with Label.get_all()
    @classmethod
    def get_all(cls):
        "Get all all labels, ordered by button"
        return cls.get_all_ordered_by("BUTTON")

    def get_name(self):
        return self.get_column("NAME")

    def get_description(self):
        return self.get_column("DESCRIPTION")

    def get_button(self):
        return self.get_column("BUTTON")

    def set_name(self, val):
        self.set_column("NAME", val)

    def set_description(self, val):
        self.set_column("DESCRIPTION", val)

    def set_button(self, val):
        self.set_column("BUTTON", val)

    name = property(get_name, set_name)
    description = property(get_description, set_description)
    button = property(get_button, set_button)

        
    def get_definitions(self):
        """Returns a list of those definitions with this label"""
        result=list_select("""SELECT DEFINITION_ID
                              FROM DEFINITIONS
                              WHERE LABEL_ID = ? ORDER BY DEFINITION_ID""", (self.id,))
        
        return [ Definition(definition_id) for definition_id in result ]


    def delete_definitions(self):
        "Delete those definitions with this label"
        update("DELETE FROM DEFINITIONS WHERE LABEL_ID = ?", (self.id,))


    def delete(self):
        """Deletes record from database, the object should be deleted after this"""
        # Can only delete a Label which is not associated with a button
        number = self.get_button()
        if number < vs_cfg["MAX_BUTTONS"]:
            return False
        # Delete all the definitions associated with this label
        update("DELETE FROM DEFINITIONS WHERE LABEL_ID = ?", (self.id,))
        update("DELETE FROM LABELS WHERE LABEL_ID = ?", (self.id,))
        return True


####################### Definition ##############################

class Definition(_VSDatabase):
    """An object describing a row in the DEFINITIONS table - called with Definition(id)
       which gets a definition from the DEFINITIONS table with the given id. To create an entirely
       new definition on the database, use function create_objects.create_Definition"""
    # The table_name and id_name are class attributes
    table_name = "DEFINITIONS"
    id_name = "DEFINITION_ID"

    def get_label(self):
        label_id = self.get_column("LABEL_ID")
        return Label(label_id) if label_id else None

    def get_account(self):
        account_id = self.get_column("ACCOUNT_ID")
        return Account(account_id) if account_id else None

    def get_action(self):
        return self.get_column("ACTION")

    def get_value(self):
        return self.get_column("VALUE")

    def get_operator(self):
        return self.get_column("OPERATOR")

    def get_operand(self):
        return self.get_column("OPERAND")

    def set_label(self, val):
        # update to the new label value in the definition
        self.set_column("LABEL_ID", val.id)      # note val.id not val

    def set_account(self, val):
        if val == None:
            self.set_column("ACCOUNT_ID", None)
        else:
            self.set_column("ACCOUNT_ID", val.id)       # note val.id not val

    def set_action(self, val):
        self.set_column("ACTION", val)

    def set_value(self, val):
        self.set_column("VALUE", val)

    def set_operator(self, val):
        self.set_column("OPERATOR", val)

    def set_operand(self, val):
        self.set_column("OPERAND", val)

    label    = property(get_label,    set_label)
    account  = property(get_account,  set_account)
    action   = property(get_action,   set_action)
    value    = property(get_value,    set_value)
    operator = property(get_operator, set_operator)
    operand  = property(get_operand,  set_operand)
        

    def delete(self):
        """Deletes record from database, the object should be deleted after this"""
        update("DELETE FROM DEFINITIONS WHERE DEFINITION_ID = ?", (self.id,))
        return True


####################### VATRate ##############################

class VATRate(_VSDatabase):
    """An object describing a row in the VAT_RATES table - called with VATRate(id)
       which gets a row from the VAT_RATES table with the given id. To create an entirely
       new row on the database, use function create_objects.create_VATRate"""
    # The table_name and id_name are class attributes
    table_name = "VAT_RATES"
    id_name = "VAT_RATE_ID"

    # call this with VATRate.get_all()
    @classmethod
    def get_all(cls):
        """"Return a list of all vat rates, ordered by vat_rate_id"""
        # no need to set column name, as id is the default
        return cls.get_all_ordered_by()

    # call this with VATRate.from_symbol(symbol)
    @classmethod
    def from_symbol(cls, symbol):
        "Returns a VAT rate from the given symbol"
        return cls.object_from_field("SYMBOL", symbol)

    def get_name(self):
        return self.get_column("NAME")

    def get_description(self):
        return self.get_column("DESCRIPTION")

    def get_symbol(self):
        return self.get_column("SYMBOL")

    def get_rate(self):
        result = self.get_column("RATE")
        return D(result)

    def set_name(self, val):
        self.set_column("NAME", val)

    def set_description(self, val):
        self.set_column("DESCRIPTION", val)

    def set_symbol(self, val):
        self.set_column("SYMBOL", val)

    def set_rate(self, val):
        val = str(val)
        self.set_column("RATE", val)

    name         = property(get_name,         set_name)
    description  = property(get_description,  set_description)
    symbol       = property(get_symbol,       set_symbol)
    rate         = property(get_rate,         set_rate)

    def delete(self):
        "Deletes record from database, the object should be deleted after this"
        # set any sellitem or purchaseitem with this vatrate to have a vatrate of None
        result=list_select("""SELECT SELLITEM_ID
                              FROM SELLITEMS
                              WHERE VAT_RATE_ID = ?""", (self.id,))
        for item_id in result:
            SellItem(item_id).vatrate = None

        result=list_select("""SELECT PURCHASEITEM_ID
                              FROM PURCHASEITEMS
                              WHERE VAT_RATE_ID = ?""", (self.id,))
        for item_id in result:
            PurchaseItem(item_id).vatrate = None
        # And delete the vatrate
        update("DELETE FROM VAT_RATES WHERE VAT_RATE_ID = ?", (self.id,))
        return True


    def __str__(self):
        "Returns a string such as 'S - 17.5%' - as used by dropdown lists on invoices etc"

        conn = None
        result = None
        try:
            conn, c = vs_connect()
            c.execute("SELECT SYMBOL, RATE FROM VAT_RATES WHERE VAT_RATE_ID = ?", (self.id,))
            result = c.fetchone()
            # Note, there is no except clause, so if an exception occurs, it is propagated upwards
        finally:
            if conn:
                conn.close()
        if not result: return ""
        return "%s - %s%%" % (result[0], result[1].replace(".", vs_cfg["CP"]))
    

####################### Customer ##############################

class Customer(_VSDatabase):
    """An object describing a row in the CUSTOMERS table - called with Customer(id)
       which gets a customer from the CUSTOMERS table with the given id. To create an entirely
       new customer on the database, use function create_objects.create_Customer"""

    # The table_name and id_name are class attributes
    table_name = "CUSTOMERS"
    id_name = "CUSTOMER_ID"

    # call this with Customer.get_all()
    @classmethod
    def get_all(cls):
        "Get all visible customers ordered by name"
        result = list_select("SELECT CUSTOMER_ID FROM CUSTOMERS WHERE VISIBLE = 1 ORDER BY NAME")
        if not result: return []
        return [ cls(customer_id) for customer_id in result ]

    # call this with Customer.from_number(number)
    @classmethod
    def from_number(cls, number):
        "Gets the visible customer with the given customer number"
        sql_string = "SELECT CUSTOMER_ID FROM CUSTOMERS WHERE NUMBER = ? AND VISIBLE = 1"
        customer_id = select(sql_string, (number,))
        if not customer_id: return None
        return cls(customer_id)

    # call this with Customer.with_open_invoices()
    @classmethod
    def with_open_invoices(cls):
        """Gets a list of all customers with open invoices, ordered by name.
           Complicated by the fact it must return visible customers only, but the
           invoices may point to historical customers which have been superceded
           by customer edits - so must first get customers by customer number,
           and then the current customers with that number.
           """
        sql_string = """SELECT CUSTOMER_ID FROM CUSTOMERS WHERE CUSTOMERS.NUMBER IN
                           (SELECT DISTINCT CUSTOMERS.NUMBER FROM INVOICES, CUSTOMERS
                            WHERE OPEN = 1
                            AND INVOICES.CUSTOMER_ID = CUSTOMERS.CUSTOMER_ID)
                        AND VISIBLE = 1 ORDER BY NAME"""
        result = list_select(sql_string)
        if not result: return []
        return [ cls(customer_id) for customer_id in result ]

    def new_insert(self):        
        "Insert a new customer - a copy of this one"
        # As customer addresses point to this id, they must now point to the new id
        address_list = self.get_addresses()
        # Also contacts point to this id, they must now point to the new id
        contact_list = self.get_contacts()

        # Cannot just edit, must make a new item
        new_id = self.new_row()
        if not new_id:
            raise VS_ERROR, "Unable to update the customer in the database"

        # Set the addresses to point to the new id
        for address in address_list:  
            update("UPDATE ADDRESSES SET CUSTOMER_ID = ? WHERE ADDRESS_ID = ?", (new_id, address.id))
        # Set the contacts to point to the new id
        for contact in contact_list:  
            update("UPDATE CONTACTS SET CUSTOMER_ID = ? WHERE CONTACT_ID = ?", (new_id, contact.id))
        # Set the old one to be invisible
        update("UPDATE CUSTOMERS SET VISIBLE = ? WHERE CUSTOMER_ID = ?", (0, self.id))
        # Set the new one to be editable
        update("UPDATE CUSTOMERS SET EDIT_NEW = ? WHERE CUSTOMER_ID = ?", (0, new_id))
        # Return the new id
        return new_id

    def get_name(self):
        return self.get_column("NAME")

    def get_ec_country(self):
        return True if self.get_column("EC_COUNTRY") else False

    def get_vat_country_code(self):
        return self.get_column("VAT_COUNTRY_CODE")

    def get_vat_regnumber(self):
        return self.get_column("VAT_REGNUMBER")

    def get_vat_registered(self):
        return True if self.get_column("VAT_REGISTERED") else False

    def get_discount(self):
        discount_string = self.get_column("DISCOUNT")
        discount_decimal = D(discount_string)
        return discount_decimal.quantize(D(".1"), rounding=decimal.ROUND_DOWN)

    def get_account(self):
        account_id = self.get_column("ACCOUNT_ID")
        return Account(account_id) if account_id else None

    def get_number(self):
        return self.get_column("NUMBER")

    def get_sortcode(self):
        return self.get_column("SORTCODE")

    def get_bank_account(self):
        return self.get_column("BANK_ACCOUNT")

    def get_iban(self):
        return self.get_column("IBAN")

    def get_bic(self):
        return self.get_column("BIC")

    def get_edit_new(self):
        return True if self.get_column("EDIT_NEW") else False

    def get_visible(self):
        return True if self.get_column("VISIBLE") else False

    def set_name(self, val):
        if val == self.get_name(): return
        if self.get_edit_new():
            # Cannot just edit, must make a new item
            self.id = self.new_insert()
        self.set_column("NAME", val)

    def set_ec_country(self, val):
        current_val = self.get_ec_country()
        if val and current_val: return
        if (not val) and (not current_val): return
        val = 1 if val else 0
        if self.get_edit_new():
            # Cannot just edit, must make a new item
            self.id = self.new_insert()
        self.set_column("EC_COUNTRY", val)

    def set_vat_country_code(self, val):
        if val == self.get_vat_country_code(): return
        if self.get_edit_new():
            # Cannot just edit, must make a new item
            self.id = self.new_insert()
        self.set_column("VAT_COUNTRY_CODE", val)

    def set_vat_regnumber(self, val):
        if val == self.get_vat_regnumber(): return
        if self.get_edit_new():
            # Cannot just edit, must make a new item
            self.id = self.new_insert()
        self.set_column("VAT_REGNUMBER", val)

    def set_vat_registered(self, val):
        current_val = self.get_vat_registered()
        if val and current_val: return
        if (not val) and (not current_val): return
        val = 1 if val else 0
        if self.get_edit_new():
            # Cannot just edit, must make a new item
            self.id = self.new_insert()
        self.set_column("VAT_REGISTERED", val)

    def set_discount(self, val):
        if val == self.get_discount(): return
        if self.get_edit_new():
            # Cannot just edit, must make a new item
            self.id = self.new_insert()
        val = val.quantize(D(".1"), rounding=decimal.ROUND_DOWN)
        self.set_column("DISCOUNT", str(val))

    def set_sortcode(self, val):
        if val == self.get_sortcode(): return
        if self.get_edit_new():
            # Cannot just edit, must make a new item
            self.id = self.new_insert()
        self.set_column("SORTCODE", val)

    def set_bank_account(self, val):
        if val == self.get_bank_account(): return
        if self.get_edit_new():
            # Cannot just edit, must make a new item
            self.id = self.new_insert()
        self.set_column("BANK_ACCOUNT", val)

    def set_iban(self, val):
        if val == self.get_iban(): return
        if self.get_edit_new():
            # Cannot just edit, must make a new item
            self.id = self.new_insert()
        self.set_column("IBAN", val)

    def set_bic(self, val):
        if val == self.get_bic(): return
        if self.get_edit_new():
            # Cannot just edit, must make a new item
            self.id = self.new_insert()
        self.set_column("BIC", val)

    def set_edit_new(self, val):
        val = 1 if val else 0
        self.set_column("EDIT_NEW", val)

    def set_visible(self, val):
        val = 1 if val else 0
        self.set_column("VISIBLE", val)

    name              = property(get_name,              set_name)
    ec_country        = property(get_ec_country,        set_ec_country)
    vat_country_code  = property(get_vat_country_code,  set_vat_country_code)
    vat_regnumber     = property(get_vat_regnumber,     set_vat_regnumber)
    vat_registered    = property(get_vat_registered,    set_vat_registered)
    discount          = property(get_discount,          set_discount)
    sortcode          = property(get_sortcode,          set_sortcode)
    bank_account      = property(get_bank_account,      set_bank_account)
    iban              = property(get_iban,              set_iban)
    bic               = property(get_bic,               set_bic)
    edit_new          = property(get_edit_new,          set_edit_new)
    visible           = property(get_visible,           set_visible)
    account           = property(get_account)        # no set account
    number            = property(get_number)         # no set number


    def charge_vat(self):
        "Returns True if you should charge this customer VAT, False otherwise"

        # If you are not vat registered, then you do not charge VAT
        # get current VAT Scheme
        vat_scheme = VATScheme()

        if vat_scheme.method == "NOT_REGISTERED":
            return False

        # if the customer is not EC, then you cannot charge him VAT
        if not self.get_ec_country():
            return False

        # if the customer is not VAT Registered, then you charge him VAT
        # regardless of if he is in GB or elswhere in EC
        if not self.get_vat_registered():
            return True

        # If the customer is GB, then you charge VAT
        if self.get_vat_country_code() == vs_cfg["VAT_COUNTRY_CODE"]:
            return True

        # all that remains, is the customer is non-GB and EC, and both he and you are VAT
        # registered - in which case you should not charge VAT
        return False


    def get_addresses(self):
        """Returns a list of those visible adresses belonging to this customer"""
        # Get those addresses with this customer_id
        result = list_select("""SELECT ADDRESS_ID
                                FROM ADDRESSES
                                WHERE CUSTOMER_ID = ? AND VISIBLE = ? 
                                ORDER BY LINE1""", (self.id, 1))
        if not result: return []
        return [ Address(address_id) for address_id in result ]


    def get_contacts(self):
        """Returns a list of those visible contacts belonging to this customer"""
        # Get those contacts with this customer_id
        result = list_select("""SELECT CONTACT_ID
                                FROM CONTACTS
                                WHERE CUSTOMER_ID = ? AND VISIBLE = ? 
                                ORDER BY NAME""", (self.id, 1))
        if not result: return []
        return [ Contact(contact_id) for contact_id in result ]


    def get_openinvoices(self):
        """Gets a list of all open invoices, sent to this customer, ordered by gridrow"""
        # must use customer number rather than id, as the invoices are wanted which are assigned
        # to this customer number (as the customer may have been edited)
        customer_number = self.get_number()
        # Get those open invoices with this customer_number
        result = list_select("""SELECT INVOICE_ID
                                FROM INVOICES, CUSTOMERS
                                WHERE OPEN = 1
                                AND CUSTOMERS.NUMBER = ?
                                AND CUSTOMERS.CUSTOMER_ID = INVOICES.CUSTOMER_ID
                                ORDER BY GRIDROW""", (customer_number,))
        if not result: return []
        return [ Invoice(invoice_id) for invoice_id in result ]


####################### Supplier ##############################

class Supplier(_VSDatabase):
    """An object describing a row in the SUPPLIERS table - called with Supplier(id)
       which gets a supplier from the SUPPLIERS table with the given id. To create an entirely
       new supplier on the database, use function create_objects.create_Supplier"""

    # The table_name and id_name are class attributes
    table_name = "SUPPLIERS"
    id_name = "SUPPLIER_ID"

    # call this with Supplier.get_all()
    @classmethod
    def get_all(cls):
        "Get all visible suppliers ordered by name"
        result = list_select("SELECT SUPPLIER_ID FROM SUPPLIERS WHERE VISIBLE = 1 ORDER BY NAME")
        if not result: return []
        return [ cls(supplier_id) for supplier_id in result ]

    # call this with Supplier.from_number(number)
    @classmethod
    def from_number(cls, number):
        "Gets the visible supplier with the given supplier number"
        sql_string = "SELECT SUPPLIER_ID FROM SUPPLIERS WHERE NUMBER = ? AND VISIBLE = 1"
        supplier_id = select(sql_string, (number,))
        if not supplier_id: return None
        return cls(supplier_id)

    def new_insert(self):        
        "Insert a new supplier - a copy of this one"
        # As customer addresses point to this id, they must now point to the new id
        address_list = self.get_addresses()
        # Also contacts point to this id, they must now point to the new id
        contact_list = self.get_contacts()
        # Cannot just edit, must make a new item
        new_id = self.new_row()
        if not new_id:
            raise VS_ERROR, "Unable to update the supplier in the database"

        # Set the addresses to point to the new id
        for address in address_list:  
            update("UPDATE ADDRESSES SET SUPPLIER_ID = ? WHERE ADDRESS_ID = ?", (new_id, address.id))
        # Set the contacts to point to the new id
        for contact in contact_list:  
            update("UPDATE CONTACTS SET SUPPLIER_ID = ? WHERE CONTACT_ID = ?", (new_id, contact.id))    
        # Set the old one to be invisible
        update("UPDATE SUPPLIERS SET VISIBLE = ? WHERE SUPPLIER_ID = ?", (0, self.id))
        # Set the new one to be editable
        update("UPDATE SUPPLIERS SET EDIT_NEW = ? WHERE SUPPLIER_ID = ?", (0, new_id))
        # Return the new id
        return new_id

    def get_name(self):
        return self.get_column("NAME")

    def get_ec_country(self):
        return True if self.get_column("EC_COUNTRY") else False

    def get_vat_country_code(self):
        return self.get_column("VAT_COUNTRY_CODE")

    def get_vat_regnumber(self):
        return self.get_column("VAT_REGNUMBER")

    def get_vat_registered(self):
        return True if self.get_column("VAT_REGISTERED") else False

    def get_discount(self):
        discount_string = self.get_column("DISCOUNT")
        discount_decimal = D(discount_string)
        return discount_decimal.quantize(D(".1"), rounding=decimal.ROUND_DOWN)

    def get_account(self):
        account_id = self.get_column("ACCOUNT_ID")
        return Account(account_id) if account_id else None

    def get_number(self):
        return self.get_column("NUMBER")

    def get_sortcode(self):
        return self.get_column("SORTCODE")

    def get_bank_account(self):
        return self.get_column("BANK_ACCOUNT")

    def get_iban(self):
        return self.get_column("IBAN")

    def get_bic(self):
        return self.get_column("BIC")

    def get_edit_new(self):
        return True if self.get_column("EDIT_NEW") else False

    def get_visible(self):
        return True if self.get_column("VISIBLE") else False

    def set_name(self, val):
        if val == self.get_name(): return
        if self.get_edit_new():
            # Cannot just edit, must make a new item
            self.id = self.new_insert()
        self.set_column("NAME", val)

    def set_ec_country(self, val):
        current_val = self.get_ec_country()
        if val and current_val: return
        if (not val) and (not current_val): return
        val = 1 if val else 0
        if self.get_edit_new():
            # Cannot just edit, must make a new item
            self.id = self.new_insert()
        self.set_column("EC_COUNTRY", val)

    def set_vat_country_code(self, val):
        if val == self.get_vat_country_code(): return
        if self.get_edit_new():
            # Cannot just edit, must make a new item
            self.id = self.new_insert()
        self.set_column("VAT_COUNTRY_CODE", val)

    def set_vat_regnumber(self, val):
        if val == self.get_vat_regnumber(): return
        if self.get_edit_new():
            # Cannot just edit, must make a new item
            self.id = self.new_insert()
        self.set_column("VAT_REGNUMBER", val)

    def set_vat_registered(self, val):
        current_val = self.get_vat_registered()
        if val and current_val: return
        if (not val) and (not current_val): return
        val = 1 if val else 0
        if self.get_edit_new():
            # Cannot just edit, must make a new item
            self.id = self.new_insert()
        self.set_column("VAT_REGISTERED", val)

    def set_discount(self, val):
        if val == self.get_discount(): return
        if self.get_edit_new():
            # Cannot just edit, must make a new item
            self.id = self.new_insert()
        val = val.quantize(D(".1"), rounding=decimal.ROUND_DOWN)
        self.set_column("DISCOUNT", str(val))

    def set_sortcode(self, val):
        if val == self.get_sortcode(): return
        if self.get_edit_new():
            # Cannot just edit, must make a new item
            self.id = self.new_insert()
        self.set_column("SORTCODE", val)

    def set_bank_account(self, val):
        if val == self.get_bank_account(): return
        if self.get_edit_new():
            # Cannot just edit, must make a new item
            self.id = self.new_insert()
        self.set_column("BANK_ACCOUNT", val)

    def set_iban(self, val):
        if val == self.get_iban(): return
        if self.get_edit_new():
            # Cannot just edit, must make a new item
            self.id = self.new_insert()
        self.set_column("IBAN", val)

    def set_bic(self, val):
        if val == self.get_bic(): return
        if self.get_edit_new():
            # Cannot just edit, must make a new item
            self.id = self.new_insert()
        self.set_column("BIC", val)

    def set_edit_new(self, val):
        val = 1 if val else 0
        self.set_column("EDIT_NEW", val)

    def set_visible(self, val):
        val = 1 if val else 0
        self.set_column("VISIBLE", val)

    name              = property(get_name,              set_name)
    ec_country        = property(get_ec_country,        set_ec_country)
    vat_country_code  = property(get_vat_country_code,  set_vat_country_code)
    vat_regnumber     = property(get_vat_regnumber,     set_vat_regnumber)
    vat_registered    = property(get_vat_registered,    set_vat_registered)
    discount          = property(get_discount,          set_discount)
    sortcode          = property(get_sortcode,          set_sortcode)
    bank_account      = property(get_bank_account,      set_bank_account)
    iban              = property(get_iban,              set_iban)
    bic               = property(get_bic,               set_bic)
    edit_new          = property(get_edit_new,          set_edit_new)
    visible           = property(get_visible,           set_visible)
    account           = property(get_account)        # no set account
    number            = property(get_number )        # no set number


    def pay_vat(self):
        "Returns True if this supplier should charge you VAT, False otherwise"

        # if the supplier is not EC, then he cannot charge you VAT
        if not self.get_ec_country():
            return False

        # if the supplier is not VAT registered, then he cannot charge you VAT
        if not self.get_vat_registered():
            return False

        # If the supplier is GB, then he can charge you VAT
        if self.get_vat_country_code() == vs_cfg["VAT_COUNTRY_CODE"]:
            return True

        # supplier must be in the EC, but not GB

        # If you are not vat registered, then you must pay the VAT the supplier charges
        # get current VAT Scheme
        vat_scheme = VATScheme()

        if vat_scheme.method == "NOT_REGISTERED":
            return True

        # all that remains, is the supplier is non-GB and EC, and both he and you are VAT
        # registered - in which case he should not charge you VAT
        return False

        
    def get_addresses(self):
        """Returns a list of those visible addresses belonging to this supplier"""
        result = list_select("""SELECT ADDRESS_ID
                                FROM ADDRESSES
                                WHERE SUPPLIER_ID = ? AND VISIBLE = ?
                                ORDER BY LINE1""", (self.id, 1))
        
        return [ Address(address_id) for address_id in result ]


    def get_contacts(self):
        """Returns a list of those visible contacts belonging to this supplier"""
        result = list_select("""SELECT CONTACT_ID
                                FROM CONTACTS
                                WHERE SUPPLIER_ID = ? AND VISIBLE = ?
                                ORDER BY NAME""", (self.id, 1))
        
        return [ Contact(contact_id) for contact_id in result ]


###################### Company ##############################

class Company(_VSDatabase):
    """Different from other objects in that id of None is ok and
       refers to 'this company', by getting the COMPANY_ID from
       the CONFIGURATION Table.  Previous company ID's are records of this company
       with previous registered address, VAT numbers etc
       A new company id can be created from the current company instance by simply updating
       attributes such as the company name - the property method then inserts a new row.
       NOTE: A new company row is also created in the create_VATScheme function."""

    # The table_name and id_name are class attributes
    table_name = "COMPANIES"
    id_name = "COMPANY_ID"

    def __init__(self, id = None):
        """If id is given, call the parent __init__, otherwise
           if the id is None, get id from CONFIGURATION table"""
        if id:
            # Call the parent __init__
            _VSDatabase.__init__(self, id)
            return
        # So the id must not have been given - get the current company
        self.id = self.get_current_id()


    def get_current_id(self):
        return int(select("SELECT CFG_ITEM FROM CONFIGURATION WHERE CFG_KEY = ?", ("COMPANY_ID",)))

    def get_name(self):
        return self.get_column("COMPANY_NAME")

    def get_vat_country_code(self):
        return self.get_column("VAT_COUNTRY_CODE")

    def get_vat_regnumber(self):
        return self.get_column("VAT_REGNUMBER")

    def get_company_number(self):
        return self.get_column("COMPANY_NUMBER")

    def get_sortcode(self):
        return self.get_column("SORTCODE")

    def get_bank_account(self):
        return self.get_column("BANK_ACCOUNT")

    def get_iban(self):
        return self.get_column("IBAN")

    def get_bic(self):
        return self.get_column("BIC")

    def get_country_registration(self):
        return self.get_column("COUNTRY_REGISTRATION")

    def get_registered_address(self):
        address_id = self.get_column("ADDRESS_ID")
        return Address(address_id) if address_id else None

    # Apart from last_order_number and last_invoice_number any change to office details
    # does not just update the database, it inserts an entirely new row, so the database retains
    # past company details - needed for past order numbers etc..

    def add_row(self):
        # Create a new row
        self.id = self.new_row()
        if not self.id:
            raise VS_ERROR, "Unable to update the company in the database"      

    def set_name(self, val):
        # Only do the change, if it is necessary
        if val == self.get_name(): return
        # Cannot update previously saved companies, only the current one
        if self.id != self.get_current_id(): return
        # Insert a new row
        self.add_row()
        self.set_column("COMPANY_NAME", val)

    def set_vat_country_code(self, val):
        if val == self.get_vat_country_code(): return
        if self.id != self.get_current_id(): return
        self.add_row()
        self.set_column("VAT_COUNTRY_CODE", val)
        vs_cfg["VAT_COUNTRY_CODE"] = val

    def set_vat_regnumber(self, val):
        if val == self.get_vat_regnumber(): return
        if self.id != self.get_current_id(): return
        self.add_row()
        self.set_column("VAT_REGNUMBER", val)

    def set_company_number(self, val):
        if val == self.get_company_number(): return
        if self.id != self.get_current_id(): return
        self.add_row()
        self.set_column("COMPANY_NUMBER", val)

    def set_sortcode(self, val):
        # Only do the change, if it is necessary
        if val == self.get_sortcode(): return
        # Cannot update previously saved companies, only the current one
        if self.id != self.get_current_id(): return
        # Insert a new row
        self.add_row()
        self.set_column("SORTCODE", val)

    def set_bank_account(self, val):
        # Only do the change, if it is necessary
        if val == self.get_bank_account(): return
        # Cannot update previously saved companies, only the current one
        if self.id != self.get_current_id(): return
        # Insert a new row
        self.add_row()
        self.set_column("BANK_ACCOUNT", val)

    def set_iban(self, val):
        # Only do the change, if it is necessary
        if val == self.get_iban(): return
        # Cannot update previously saved companies, only the current one
        if self.id != self.get_current_id(): return
        # Insert a new row
        self.add_row()
        self.set_column("IBAN", val)

    def set_bic(self, val):
        # Only do the change, if it is necessary
        if val == self.get_bic(): return
        # Cannot update previously saved companies, only the current one
        if self.id != self.get_current_id(): return
        # Insert a new row
        self.add_row()
        self.set_column("BIC", val)

    def set_country_registration(self, val):
        if val == self.get_country_registration(): return
        if self.id != self.get_current_id(): return
        self.add_row()
        self.set_column("COUNTRY_REGISTRATION", val)

    def set_registered_address(self, val):
        if val == self.get_registered_address(): return
        if self.id != self.get_current_id(): return
        self.add_row()
        self.set_column("ADDRESS_ID", val.id)
        # Note val.id, not val

    name                    = property(get_name,                 set_name)
    vat_country_code        = property(get_vat_country_code,     set_vat_country_code)
    vat_regnumber           = property(get_vat_regnumber,        set_vat_regnumber)
    company_number          = property(get_company_number,       set_company_number)
    sortcode                = property(get_sortcode,             set_sortcode)
    bank_account            = property(get_bank_account,         set_bank_account)
    iban                    = property(get_iban,                 set_iban)
    bic                     = property(get_bic,                  set_bic)
    country_registration    = property(get_country_registration, set_country_registration)
    registered_address      = property(get_registered_address,   set_registered_address)
    current_id              = property(get_current_id)

    def get_addresses(self):
        """Returns a list of those visible addresses belonging to this company"""
        result = list_select("""SELECT ADDRESS_ID
                                FROM ADDRESSES
                                WHERE SYSTEM = 1 AND VISIBLE = 1
                                ORDER BY LINE1""")
        return [ Address(address_id) for address_id in result ]


####################### VATScheme ###########################

class VATScheme(_VSDatabase):
    """Different from other objects in that id of None is ok and
       refers to 'the current VAT scheme', by getting the VAT_SCHEME_ID from
       the CONFIGURATION Table.  Previous VAT scheme ID's are records of previous schemes used
       To create a new VATScheme on the database, use function create_objects.create_VATScheme"""

    # The table_name and id_name are class attributes
    table_name = "VAT_SCHEMES"
    id_name = "VAT_SCHEME_ID"

    def __init__(self, id = None):
        """If id is given, call the parent __init__, otherwise
           if the id is None, get id from CONFIGURATION table"""
        if id:
            # Call the parent __init__
            _VSDatabase.__init__(self, id)
            return
        # So the id must not have been given - get the current vat scheme
        self.id = self.get_current_id()

    def get_current_id(self):
        return int(select("SELECT CFG_ITEM FROM CONFIGURATION WHERE CFG_KEY = ?", ("VAT_SCHEME_ID",)))

    def get_transaction_id(self):
        # returns the last transaction id prior to this vat scheme being created
        # used by the undo feature - if a transaction is to be deleted, checks this
        # and if necessary undoes the vat scheme
        # Note it returns an id, not a Transaction object, as the transaction
        # itself may no longer exist
        transaction_id = self.get_column("TRANSACTION_ID")
        if not transaction_id:
            return None
        return transaction_id

    def get_method(self):
        return self.get_column("METHOD")

    def get_period(self):
        return self.get_column("PERIOD")

    def get_start_date(self):
        return self.get_column("STARTDATE")

    def get_quarter_date(self):
        "returns the start date plus a quarter - i.e. to to_date of a vat return"
        from_date = self.get_start_date()

        # need to get the to-date period,  three months on, could go across a new year
        from_month = from_date.month
        from_year = from_date.year

        to_month = from_month + 2
        to_year = from_year
        if to_month > 12:
            to_month = to_month - 12
            to_year = to_year+1

        # Find last day in month and create a to_date object
        day_of_week, to_day = calendar.monthrange(to_year, to_month)
        to_date = datetime.date(to_year, to_month, to_day)
        return to_date

    def get_month(self):
        current_start_date = self.get_start_date()
        if not current_start_date:
            return None
        return current_start_date.month

    def get_flat_percent(self):
        flat_percent = self.get_column("FLAT_PERCENT")
        return D(flat_percent)

    def set_start_date(self, val):
        if val == self.get_start_date(): return
        if self.id != self.get_current_id(): return
        # startdate - don't add a new row - just an update
        self.set_column("STARTDATE", val)

    start_date      = property(get_start_date,   set_start_date)
    transaction_id  = property(get_transaction_id)
    method          = property(get_method)
    period          = property(get_period)
    quarter_date    = property(get_quarter_date)
    month           = property(get_month)
    flat_percent    = property(get_flat_percent)


####################### Address ##############################

class Address(_VSDatabase):
    """An object describing a row in the ADDRESSES table - called with Address(id)
       which gets an address from the ADDRESSES table with the given id. To create an entirely
       new address on the database, use function create_objects.create_Address"""

    # The table_name and id_name are class attributes
    table_name = "ADDRESSES"
    id_name = "ADDRESS_ID"

    def new_insert(self):        
        "Create a new address - a copy of this one"
        # Cannot just edit, must make a new item
        new_id = self.new_row()
        # Set the old one to be invisible
        update("UPDATE ADDRESSES SET VISIBLE = ? WHERE ADDRESS_ID = ?", (0, self.id))
        # Set the new one to be editable
        update("UPDATE ADDRESSES SET EDIT_NEW = ? WHERE ADDRESS_ID = ?", (0, new_id))
        # If the company registered address points to the old one, make sure it now
        # points to the new one
        company = Company()
        if company.registered_address.id == self.id:
            company.registered_address = Address(new_id)
        # Return the new id
        return new_id

    def get_customer(self):
        customer_id = self.get_column("CUSTOMER_ID")
        if not customer_id: return None
        return Customer(customer_id)

    def get_supplier(self):
        supplier_id = self.get_column("SUPPLIER_ID")
        if not supplier_id: return None
        return Supplier(supplier_id)

    def get_employee(self):
        return None                # Employee object not defined

    def get_contractor(self):
        return None                # Contractor object not defined

    def get_name(self):            # Get the customer, supplier or employee name if relevant
        customer = self.get_customer()
        if customer:
            return customer.name
        supplier = self.get_supplier()
        if supplier:
            return supplier.name
        # Similar lines can be inserted for employee and contractor if these objects are defined
        return None

    def get_reg_office(self):
        return self.get_column("REG_OFFICE")

    def get_delivery_point(self):
        return self.get_column("DELIVERY_POINT")

    def get_invoice_point(self):
        return self.get_column("INVOICE_POINT")

    def get_order_point(self):
        return self.get_column("ORDER_POINT")

    def get_payment_point(self):
        return self.get_column("PAYMENT_POINT")

    def get_flags(self):
        return { 'reg_office'     : self.get_column("REG_OFFICE"),
                 'delivery_point' : self.get_column("DELIVERY_POINT"),
                 'invoice_point'  : self.get_column("INVOICE_POINT"),
                 'order_point'    : self.get_column("ORDER_POINT"),
                 'payment_point'  : self.get_column("PAYMENT_POINT") }

    def get_system(self):
        return self.get_column("SYSTEM")

    def get_country(self):
        return self.get_column("COUNTRY")

    def get_lines(self):
        "Returns a list of address lines"
        # Make the connection
        conn, c = vs_connect()
        # Get the particular record from VAT_RATES
        c.execute("""SELECT LINE1,
                            LINE2,
                            CITY,
                            COUNTY,
                            POSTCODE,
                            COUNTRY
                      FROM ADDRESSES WHERE ADDRESS_ID = ?""", (self.id,))
        result = c.fetchone()
        conn.close()
        return result

    def get_lines_string(self):
        "Returns a list of address lines as one string separated by newline characters"
        def join_newline(a, b):
            if b == "": return a
            return a + "\n" + b
        return reduce(join_newline, self.get_lines())

    def get_edit_new(self):
        return True if self.get_column("EDIT_NEW") else False

    def get_visible(self):
        return True if self.get_column("VISIBLE") else False

    def set_name(self, val):
        customer = self.get_customer()
        if customer:
            customer.name = val
            return
        supplier = self.get_supplier()
        if supplier:
            supplier.name = val
            return

    def set_reg_office(self, val):
        self.set_column("REG_OFFICE", val)

    def set_delivery_point(self, val):
        self.set_column("DELIVERY_POINT", val)

    def set_invoice_point(self, val):
        self.set_column("INVOICE_POINT", val)

    def set_order_point(self, val):
        self.set_column("ORDER_POINT", val)

    def set_payment_point(self, val):
        self.set_column("PAYMENT_POINT", val)

    def set_flags(self, val):
        "val is a dictionary of flags"
        self.set_column("REG_OFFICE", val['reg_office'])
        self.set_column("DELIVERY_POINT", val['delivery_point'])
        self.set_column("INVOICE_POINT", val['invoice_point'])
        self.set_column("ORDER_POINT", val['order_point'])
        self.set_column("PAYMENT_POINT", val['payment_point'])

    def set_edit_new(self, val):
        val = 1 if val else 0
        self.set_column("EDIT_NEW", val)

    def set_visible(self, val):
        val = 1 if val else 0
        self.set_column("VISIBLE", val)

    def set_system(self, val):
        self.set_column("SYSTEM", val)

    def set_country(self, val):
        "Sets the country"
        if self.get_edit_new():
            # Cannot just edit, must make a new item
            self.id = self.new_insert()
        self.set_column("COUNTRY", val)

    def set_lines(self, lines):
        "Sets a list of address lines"
        # If a tuple sent, convert to a list
        lines = list(lines)
        if lines == list(self.get_lines()): return
        if self.get_edit_new():
            # Cannot just edit, must make a new item
            self.id = self.new_insert()
        lines.append(self.id)
        update("""UPDATE ADDRESSES SET
                     LINE1 = ?,
                     LINE2 = ?,
                     CITY = ?,
                     COUNTY = ?,
                     POSTCODE = ?,
                     COUNTRY = ?
                     WHERE ADDRESS_ID = ?""", lines)

    name           = property(get_name,           set_name)
    lines          = property(get_lines,          set_lines)
    reg_office     = property(get_reg_office,     set_reg_office)
    delivery_point = property(get_delivery_point, set_delivery_point)
    invoice_point  = property(get_invoice_point,  set_invoice_point)
    order_point    = property(get_order_point,    set_order_point)
    payment_point  = property(get_payment_point,  set_payment_point)
    flags          = property(get_flags,          set_flags)
    edit_new       = property(get_edit_new,       set_edit_new)
    visible        = property(get_visible,        set_visible)
    system         = property(get_system,         set_system)
    country        = property(get_country,        set_country)
    employee       = property(get_employee)
    contractor     = property(get_contractor)
    customer       = property(get_customer)
    supplier       = property(get_supplier)
    lines_string   = property(get_lines_string)


    def delete(self):
        """Deletes record from database"""
        # Cannot delete if this is the companies registered office
        company = Company()
        if company.registered_address.id == self.id:
            return
        if self.get_edit_new():
            self.set_visible(False)
        else:
            update("DELETE FROM ADDRESSES WHERE ADDRESS_ID = ?", (self.id,))
        return


####################### Contact ##############################

class Contact(_VSDatabase):
    """An object describing a row in the CONTACTS table - called with Contact(id)
       which gets a contact from the CONTACTS table with the given id. To create an entirely
       new contact on the database, use function create_objects.create_Contact"""

    # The table_name and id_name are class attributes
    table_name = "CONTACTS"
    id_name = "CONTACT_ID"

    # SYSTEM NUMBER
    # 0 : your customer or suppliers contacts
    # 
    # 1 : your own comapany contacts
    #     these names appear as enquiry contacts on orders and invoices in the drop
    #     down list  when you create an invoice, purchase order or credit note

    # EDIT_NEW - if zero (False) The item name, email, phone, mobile can be edited
    #          - if one (True), if the above are edited, a new item is created, leaving
    #          - the old one untouched

    # VISIBLE - If zero (false) the item is not available for editing
    #         - if one (true) the item can be edited


    # call this with Contact.company_list()
    @classmethod
    def company_list(cls):
        """Gets a list of all contacts in your company, ordered by name, visible only"""
        sql_string = "SELECT CONTACT_ID FROM CONTACTS WHERE SYSTEM = 1 AND VISIBLE = 1 ORDER BY NAME"
        result = list_select(sql_string)
        if not result: return []
        return [ cls(contact_id) for contact_id in result ]


    def new_insert(self):        
        "Create a new contact - a copy of this one"
        # Cannot just edit, must make a new item
        new_id = self.new_row()
        # Set the old one to be invisible
        update("UPDATE CONTACTS SET VISIBLE = ? WHERE CONTACT_ID = ?", (0, self.id))
        # Set the new one to be editable
        update("UPDATE CONTACTS SET EDIT_NEW = ? WHERE CONTACT_ID = ?", (0, new_id))
        # Return the new id
        return new_id

    def get_customer(self):
        customer_id = self.get_column("CUSTOMER_ID")
        if not customer_id: return None
        return Customer(customer_id)

    def get_supplier(self):
        supplier_id = self.get_column("SUPPLIER_ID")
        if not supplier_id: return None
        return Supplier(supplier_id)

    def get_organisation_name(self):
        if self.get_system():
            return Company().name
        org = self.get_customer()
        if org:
            return org.name
        org = self.get_supplier()
        if org:
            return org.name
        return ""

    def get_name(self):
        return self.get_column("NAME")

    def get_email(self):
        return self.get_column("EMAIL")

    def get_phone(self):
        return self.get_column("PHONE")

    def get_mobile(self):
        return self.get_column("MOBILE")

    def get_notes(self):
        return self.get_column("NOTES")

    def get_system(self):
        return self.get_column("SYSTEM")

    def get_edit_new(self):
        return True if self.get_column("EDIT_NEW") else False

    def get_visible(self):
        return True if self.get_column("VISIBLE") else False

    def set_name(self, val):
        if val == self.get_name(): return
        if self.get_edit_new():
            # Cannot just edit, must make a new item
            self.id = self.new_insert()
        self.set_column("NAME", val)

    def set_email(self, val):
        if val == self.get_email(): return
        if self.get_edit_new():
            # Cannot just edit, must make a new item
            self.id = self.new_insert()
        self.set_column("EMAIL", val)

    def set_phone(self, val):
        if val == self.get_phone(): return
        if self.get_edit_new():
            # Cannot just edit, must make a new item
            self.id = self.new_insert()
        self.set_column("PHONE", val)

    def set_mobile(self, val):
        if val == self.get_mobile(): return
        if self.get_edit_new():
            # Cannot just edit, must make a new item
            self.id = self.new_insert()
        self.set_column("MOBILE", val)

    def set_notes(self, val):
        self.set_column("NOTES", val)

    def set_edit_new(self, val):
        val = 1 if val else 0
        self.set_column("EDIT_NEW", val)

    def set_visible(self, val):
        val = 1 if val else 0
        self.set_column("VISIBLE", val)

    def set_system(self, val):
        self.set_column("SYSTEM", val)

    name     = property(get_name,          set_name)
    email    = property(get_email,         set_email)
    phone    = property(get_phone,         set_phone)
    mobile   = property(get_mobile,        set_mobile)
    notes    = property(get_notes,         set_notes)
    edit_new = property(get_edit_new,      set_edit_new)
    visible  = property(get_visible,       set_visible)
    system   = property(get_system,        set_system)
    customer = property(get_customer)
    supplier = property(get_supplier)
    organisation_name = property(get_organisation_name)

    def delete(self):
        """Deletes record from database, If a contact has edit_new False then it is actually
           deleted - as it has never been used, but if it has edit_new True then it
           is given a visible value of False - making it invisible to the user, and effectively deleted"""
        if self.get_edit_new():
            self.set_visible(False)
        else:
            update("DELETE FROM CONTACTS WHERE CONTACT_ID = ?", (self.id,))
        return

    def __str__(self):
        # String is the users name
        return self.get_name()


###################### ItemDescription ##############################

class ItemDescription(_VSDatabase):
    """An object describing a row in the ITEMDESCRIPTIONS table - called with ItemDescription(id)
       which gets a item description from the ITEMDESCRIPTIONS table with the given id. To create a new item
       description on the database, you would normally use the function create_objects.create_ItemDescription
       However you could also create a new row on the database by changing the description property, which
       does not alter the row - rather it either creates a new row or points this object to an existing
       row with that description."""

    # The table_name and id_name are class attributes
    table_name = "ITEMDESCRIPTIONS"
    id_name = "ITEMDESCRIPTION_ID"

    def get_description(self):
        return self.get_column("DESCRIPTION")

    def set_description(self, val):
        "Either creates a new row or points this object to an existing row with that description"
        val = val.strip()
        if val == self.get_description(): return

        # if this description already exists in another row, point to that row instead
        number = select("SELECT COUNT(*) FROM ITEMDESCRIPTIONS WHERE DESCRIPTION = ?", (val,))
        if number > 0:
            self.id = select("SELECT ITEMDESCRIPTION_ID FROM ITEMDESCRIPTIONS WHERE DESCRIPTION = ?", (val,))
            return

        # Cannot just edit, must make a new item description
        self.id=insert("""INSERT INTO ITEMDESCRIPTIONS 
                        (ITEMDESCRIPTION_ID,
                         DESCRIPTION
                         ) VALUES (?, ?)""", (None, val))

    description = property(get_description,   set_description)

    def delete(self):
        """Deletes record from database - Be sure to delete all references to it first!!!"""
        update("DELETE FROM ITEMDESCRIPTIONS WHERE ITEMDESCRIPTION_ID = ?", (self.id,))
        self.id = 0
        return

    def __str__(self):
        return self.get_description()


###################### SellItem ##############################

class SellItem(_VSDatabase):
    """An object describing a row in the SELLITEMS table - these are items the user inputs as commonly
       sold, and are available as a drop down list when creating invoices - called with SellItem(id)
       which gets a sellitem from the SELLITEMS table with the given id. To create an entirely
       new sellitem on the database, use function create_objects.create_SellItem"""

    # The table_name and id_name are class attributes
    table_name = "SELLITEMS"
    id_name = "SELLITEM_ID"

    # call this with SellItem.get_all()
    @classmethod
    def get_all(cls):
        """Gets a list of all sellitems, ordered alphabetically by description"""
        result = list_select("""SELECT SELLITEM_ID FROM SELLITEMS, ITEMDESCRIPTIONS
                                WHERE SELLITEMS.ITEMDESCRIPTION_ID = ITEMDESCRIPTIONS.ITEMDESCRIPTION_ID
                                ORDER BY DESCRIPTION""")
        if not result: return []
        return [ cls(item_id) for item_id in result ]

    def get_itemdescription(self):
        # Gets an ItemDescription object
        itemdescription_id = self.get_column("ITEMDESCRIPTION_ID")
        return ItemDescription(itemdescription_id) if itemdescription_id else None

    def get_description(self):
        # Gets the text of the associated ItemDescription object
        itemdescription_object = self.get_itemdescription()
        return itemdescription_object.description if itemdescription_object else ""

    def get_cost(self):
        result = self.get_column("COST")
        return D(result)

    def get_vatrate(self):
        vatrate_id = self.get_column("VAT_RATE_ID")
        return VATRate(vatrate_id) if vatrate_id else None

    def get_vatsymbol(self):
        vatrate = self.get_vatrate()
        return vatrate.symbol if vatrate else ""

    def get_notes(self):
        return self.get_column("NOTES")

    def set_itemdescription(self, val):
        # val should be an ItemDescription object, not an actual text description
        if val == None:
            self.set_column("ITEMDESCRIPTION_ID", None)
        else:
            self.set_column("ITEMDESCRIPTION_ID", val.id)

    def set_description(self, val):
        "Create an itemdescription object, and set its id into the SELLITEMS table"
        if not val:
            self.set_column("ITEMDESCRIPTION_ID", None)
        # Cannot call create_ItemDescription as that could lead to circular imports
        # so reproduce some of its code here
        number = select("SELECT COUNT(*) FROM ITEMDESCRIPTIONS WHERE DESCRIPTION = ?", (val,))
        if number > 0:
            itemdescription_id = select("SELECT ITEMDESCRIPTION_ID FROM ITEMDESCRIPTIONS WHERE DESCRIPTION = ?",
                                         (val,))
        else:
            # Description does not exist, so insert a new itemdescription
            itemdescription_id = insert("""INSERT INTO ITEMDESCRIPTIONS 
                                   (ITEMDESCRIPTION_ID, DESCRIPTION) VALUES (?, ?)""", (None, val))

        self.set_column("ITEMDESCRIPTION_ID", itemdescription_id)
        return
 
    def set_cost(self, val):
        # update the cost, val is passed as a Decimal value, make sure it only
        # has two places of decimals, round down to the nearest penny.
        val = val.quantize(D('.01'), rounding=decimal.ROUND_DOWN)
        self.set_column("COST", str(val))

    def set_vatrate(self, val):
        if val:
            self.set_column("VAT_RATE_ID", val.id)
        else:
            self.set_column("VAT_RATE_ID", None)

    def set_vatsymbol(self, val):
        vatrate_id = select("SELECT VAT_RATE_ID FROM VAT_RATES WHERE SYMBOL = ?", (val,))
        if not vatrate_id: return
        self.set_column("VAT_RATE_ID", vatrate_id)

    def set_notes(self, val):
        self.set_column("NOTES", val)

    itemdescription = property(get_itemdescription,   set_itemdescription)
    description     = property(get_description,       set_description)
    cost            = property(get_cost,              set_cost)
    vatrate         = property(get_vatrate,           set_vatrate)
    vatsymbol       = property(get_vatsymbol,         set_vatsymbol)
    notes           = property(get_notes,             set_notes)


    def delete(self):
        """Deletes record from database"""
        update("DELETE FROM SELLITEMS WHERE SELLITEM_ID = ?", (self.id,))
        self.id = 0


###################### PurchaseItem ##############################

class PurchaseItem(_VSDatabase):
    """An object describing a row in the PURCHASEITEMS table - these are items the user inputs as commonly
       purchased, and are available as a drop down list when creating orders - called with PurchaseItem(id)
       which gets a purchaseitem from the PURCHASEITEMS table with the given id. To create an entirely
       new purchaseitem on the database, use function create_objects.create_PurchaseItem"""

    # The table_name and id_name are class attributes
    table_name = "PURCHASEITEMS"
    id_name = "PURCHASEITEM_ID"

    # call this with PurchaseItem.get_all()
    @classmethod
    def get_all(cls):
        """Gets a list of all purchaseitems, ordered alphabetically by description"""
        result = list_select("""SELECT PURCHASEITEM_ID FROM PURCHASEITEMS, ITEMDESCRIPTIONS
                                WHERE PURCHASEITEMS.ITEMDESCRIPTION_ID = ITEMDESCRIPTIONS.ITEMDESCRIPTION_ID
                                ORDER BY DESCRIPTION""")
        if not result: return []
        return [ cls(item_id) for item_id in result ]

    def get_itemdescription(self):
        # Gets an ItemDescription object
        itemdescription_id = self.get_column("ITEMDESCRIPTION_ID")
        return ItemDescription(itemdescription_id) if itemdescription_id else None

    def get_description(self):
        # Gets the text of the associated ItemDescription object
        itemdescription_object = self.get_itemdescription()
        return itemdescription_object.description if itemdescription_object else ""

    def get_cost(self):
        result = self.get_column("COST")
        return D(result)

    def get_vatrate(self):
        vatrate_id = self.get_column("VAT_RATE_ID")
        return VATRate(vatrate_id) if vatrate_id else None

    def get_vatsymbol(self):
        vatrate = self.get_vatrate()
        return vatrate.symbol if vatrate else ""

    def get_notes(self):
        return self.get_column("NOTES")

    def set_itemdescription(self, val):
        # val should be an ItemDescription object, not an actual text description
        if val:
            self.set_column("ITEMDESCRIPTION_ID", val.id)
        else:
            self.set_column("ITEMDESCRIPTION_ID", None)

    def set_description(self, val):
        "Create an itemdescription object, and set its id into the PURCHASEITEMS table"
        if not val:
            self.set_column("ITEMDESCRIPTION_ID", None)
        # Cannot call create_ItemDescription as that could lead to circular imports
        # so reproduce some of its code here
        number = select("SELECT COUNT(*) FROM ITEMDESCRIPTIONS WHERE DESCRIPTION = ?", (val,))
        if number:
            itemdescription_id = select("SELECT ITEMDESCRIPTION_ID FROM ITEMDESCRIPTIONS WHERE DESCRIPTION = ?",
                                         (val,))
        else:
            # Description does not exist, so insert a new itemdescription
            itemdescription_id = insert("""INSERT INTO ITEMDESCRIPTIONS 
                                   (ITEMDESCRIPTION_ID, DESCRIPTION) VALUES (?, ?)""", (None, val))
        self.set_column("ITEMDESCRIPTION_ID", itemdescription_id)

    def set_cost(self, val):
        # update the cost, val is passed as a Decimal value, make sure it only
        # has two places of decimals, round down to the nearest penny.
        val = val.quantize(D('.01'), rounding=decimal.ROUND_DOWN)
        self.set_column("COST", str(val))

    def set_vatrate(self, val):
        if val:
            self.set_column("VAT_RATE_ID", val.id)
        else:
            self.set_column("VAT_RATE_ID", None)

    def set_vatsymbol(self, val):
        vatrate_id = select("SELECT VAT_RATE_ID FROM VAT_RATES WHERE SYMBOL = ?", (val,))
        if vatrate_id:
            self.set_column("VAT_RATE_ID", vatrate_id)

    def set_notes(self, val):
        self.set_column("NOTES", val)

    itemdescription = property(get_itemdescription,   set_itemdescription)
    description     = property(get_description,       set_description)
    cost            = property(get_cost,              set_cost)
    vatrate         = property(get_vatrate,           set_vatrate)
    vatsymbol       = property(get_vatsymbol,         set_vatsymbol)
    notes           = property(get_notes,             set_notes)

    def delete(self):
        """Deletes record from database"""
        update("DELETE FROM PURCHASEITEMS WHERE PURCHASEITEM_ID = ?", (self.id,))
        self.id = 0


###################### TRANSACTION ###########################

class Transaction(_VSDatabase):
    """An object describing a row in the TRANSACTIONS table - called with Transaction(id)
       which gets a transaction from the TRANSACTIONS table with the given id. To create an entirely
       new transaction on the database, use the transactions.implement function"""

    # The table_name and id_name are class attributes
    table_name = "TRANSACTIONS"
    id_name = "TRANSACTION_ID"

    # call this with Transaction.get_last()
    @classmethod
    def get_last(cls):
        "Get the last transaction"
        transaction_id = select("SELECT MAX(TRANSACTION_ID) FROM TRANSACTIONS")
        return cls(transaction_id) if transaction_id else None

    def get_description(self):
        return self.get_column("DESCRIPTION")

    def set_description(self, val):
        self.set_column("DESCRIPTION", val)

    def get_datestamp(self):
        return self.get_column("DATESTAMP")

    def get_gridrow(self):
        return self.get_column("GRIDROW")

    def get_system(self):
        return self.get_column("SYSTEM")

    def get_trans_list(self):
        # trans_list, being a list of lists of [account, debit, credit]
        list_of_account_ids = list_select("SELECT DISTINCT ACCOUNT_ID FROM TRANSACTIONACCOUNTS WHERE TRANSACTION_ID = ?",
                                        (self.id,))
        if not list_of_account_ids: return []
        trans_list = []
        conn = None
        try:
            conn, c = vs_connect()
            for account_id in list_of_account_ids:
                ledger = "LEDGER_%i" % account_id
                sql_string = "SELECT DEBIT, CREDIT FROM %s WHERE TRANSACTION_ID = ? ORDER BY LEDGER_ID" % ledger
                c.execute(sql_string, (self.id,))
                for result in c:
                    trans_list.append( [ account_id, result[0], result[1] ] )
        finally:
            if conn:
                conn.close()
        return [ [Account(account_id), D(debit), D(credit)] for account_id, debit, credit in trans_list ]

    description = property(get_description,   set_description)
    datestamp   = property(get_datestamp)
    gridrow     = property(get_gridrow)
    system      = property(get_system)
    trans_list  = property(get_trans_list)

    def get_object(self):
        """Method returning the object of the associated transaction
           For transactions that have no related object - such as a move money - return None"""
        transaction_system = self.get_system()
        if transaction_system == 2:
            # This is a purchase order transaction
            purchaseorder_id = select("""SELECT PURCHASEORDER_ID FROM PURCHASEORDERS
                                         WHERE TRANSACTION_ID = ?""", (self.id,))
            if not purchaseorder_id: return None
            return PurchaseOrder(purchaseorder_id)
        elif transaction_system == 3:
            # This is a payment to a supplier
            makepayment_id = select("""SELECT MAKEPAYMENT_ID FROM MAKEPAYMENTS
                                       WHERE TRANSACTION_ID = ?""", (self.id,))
            if not makepayment_id: return None
            return MakePayment(makepayment_id)
        elif transaction_system == 4:
            # This is a suppliers credit note
            suppliercreditnote_id = select("""SELECT SUPPLIERCREDITNOTE_ID FROM SUPPLIERCREDITNOTES
                                              WHERE TRANSACTION_ID = ?""", (self.id,))
            if not suppliercreditnote_id: return None
            return SupplierCreditNote(suppliercreditnote_id)
        elif transaction_system == 5:
            # This is an invoice
            invoice_id = select("""SELECT INVOICE_ID FROM INVOICES
                                              WHERE TRANSACTION_ID = ?""", (self.id,))
            if not invoice_id: return None
            return Invoice(invoice_id)
        elif transaction_system == 6:
            # This is a payment from a customer
            payment_id = select("""SELECT CUSTOMERPAYMENT_ID FROM CUSTOMERPAYMENTS
                                   WHERE TRANSACTION_ID = ?""", (self.id,))
            if not payment_id: return None
            return CustomerPayment(payment_id)
        elif transaction_system == 7:
            # This is an issued creditnote
            creditnote_id = select("""SELECT CREDITNOTE_ID FROM CREDITNOTES
                                              WHERE TRANSACTION_ID = ?""", (self.id,))
            if not creditnote_id: return None
            return CreditNote(creditnote_id)
        elif transaction_system == 8:
            # This is a close order transaction
            closedorder_id = select("""SELECT CLOSEDORDER_ID FROM CLOSEDORDERS
                                           WHERE TRANSACTION_ID = ?""", (self.id,))
            if not closedorder_id: return None
            return ClosedOrder(closedorder_id)
        elif transaction_system == 9:
            # This is a suppliers invoice received transaction
            supplierinvoice_id = select("""SELECT SUPPLIERINVOICE_ID FROM SUPPLIERINVOICES
                                           WHERE TRANSACTION_ID = ?""", (self.id,))
            if not supplierinvoice_id: return None
            return SupplierInvoice(supplierinvoice_id)
        elif transaction_system == 10:
            # This is a VAT return submission transaction
            vatreturn_id = select("""SELECT VAT_RETURN_ID FROM VAT_RETURNS
                                           WHERE TRANSACTION_ID = ?""", (self.id,))
            if not vatreturn_id: return None
            return VATReturn(vatreturn_id)
        elif transaction_system == 11:
            # This is a VAT correction transaction
            vatcorrection_id = select("""SELECT VATCORRECTION_ID FROM VATCORRECTIONS
                                           WHERE TRANSACTION_ID = ?""", (self.id,))
            if not vatcorrection_id: return None
            return VATCorrection(vatcorrection_id)
        else:
            return None


#################### VATCorrection ###################################

class VATCorrection(_VSDatabase):
    """Used to read VATCorrection details - called with VATCorrection(id) which gets
       a vat correction from the VATCORRECTIONS table with the given id. To create an entirely
       new vat correction on the database, use save function in the save_vatcorrection.py module"""

    # The table_name and id_name are class attributes
    table_name = "VATCORRECTIONS"
    id_name = "VATCORRECTION_ID"

    # call this with VATCorrection.get_all()
    @classmethod
    def get_all(cls):
        "Return a list of all vat corrections, ordered by transaction gridrow (i.e. by date)"
        result = list_select("""SELECT VATCORRECTION_ID FROM VATCORRECTIONS, TRANSACTIONS
                                WHERE VATCORRECTIONS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                                ORDER BY GRIDROW""")
        if not result: return []
        all_vat_corrections = [ cls(vat_correction_id) for vat_correction_id in result ]
        return all_vat_corrections

    def get_transaction(self):
        transaction_id = self.get_column("TRANSACTION_ID")
        return Transaction(transaction_id) if transaction_id else None

    def get_long_description(self):
        return self.get_column("LONGDESCRIPTION")

    def get_amount(self):
        result = self.get_column("AMOUNT")
        return D(result)

    def set_long_description(self, val):
        self.set_column("LONGDESCRIPTION", val)

    long_description = property(get_long_description,   set_long_description)
    transaction      = property(get_transaction)
    amount           = property(get_amount)


###################### PROJECT ###########################

class Project(_VSDatabase):
    """An object describing a row in the PROJECTS table - called with Project(id) which
       gets a project from the PROJECTS table with the given id. To create an entirely
       new project on the database, use the create_objects.create_Project"""

    # The table_name and id_name are class attributes
    table_name = "PROJECTS"
    id_name = "PROJECT_ID"

    # call this with Project.get_all()
    @classmethod
    def get_all(cls):
        "Get all projects ordered by project_number - which is a text value"
        return cls.get_all_ordered_by("PROJECT_NUMBER")

    # call this with Project.from_number(number)
    @classmethod
    def from_number(cls, number):
        "Get the project with this number field (the project reference number)"
        return cls.object_from_field("PROJECT_NUMBER", number)

    def get_number(self):
        return self.get_column("PROJECT_NUMBER")

    def set_number(self, val):
        self.set_column("PROJECT_NUMBER", val)

    def get_contact(self):
        contact_id = self.get_column("CONTACT_ID")
        if not contact_id: return None
        return Contact(contact_id)

    def set_contact(self, val):
        if val:
            contact_id = val.id
            self.set_column("CONTACT_ID", contact_id)
            update("UPDATE CONTACTS SET EDIT_NEW = ? WHERE CONTACT_ID = ?", (1, contact_id))
        else:
            self.set_column("CONTACT_ID", None)
    
    def get_description(self):
        return self.get_column("DESCRIPTION")

    def get_order_list(self):
        "Get the list of orders assigned to this project"
        result = list_select("""SELECT PURCHASEORDER_ID FROM PURCHASEORDERS WHERE PROJECT_ID = ? ORDER BY GRIDROW""", (self.id,))
        # result is a list of purchaseorder_id's
        if not result: return []
        return [ PurchaseOrder(purchaseorder_id) for purchaseorder_id in result ]

    def set_description(self, val):
        self.set_column("DESCRIPTION", val)

    number      = property(get_number,      set_number)
    contact     = property(get_contact,     set_contact)
    description = property(get_description, set_description)
    order_list  = property(get_order_list)

    def delete(self):
        """Deletes project"""
        if not vs_cfg["WRITE_ACCESS"]:
            # Another process has locked the database
            raise VS_ANOTHER_PROCESS, "Database change cancelled - another process has locked this database"
        conn = None
        try:
            conn, cursor = vs_connect()
            cursor.execute("UPDATE PURCHASEORDERS SET PROJECT_ID = ? WHERE PROJECT_ID = ?", (None, self.id))
            cursor.execute("DELETE FROM PROJECTS WHERE PROJECT_ID = ?", (self.id,))
            conn.commit()
        finally:
            if conn:
                conn.close()


################## Purchaseorder ##################################

class PurchaseOrder(_VSDatabase):
    """Used to read PurchaseOrder details - called with PurchaseOrder(id) which
       gets an order from the PURCHASEORDERS table with the given id. To create an entirely
       new order on the database, use save function in the save_purchaseorder.py module"""

    # STATUS
    # 0 if this Purchase Order is still open, no invoices received
    # 1 order open, and one or more invoices have been received
    # 2 if normally closed and order fully paid
    # 3 if closed, with still an amount to pay
    # 4 if closed, with extra to pay
    # 5 if closed by the order being cancelled with no invoice recieved or payment made

    # The table_name and id_name are class attributes
    table_name = "PURCHASEORDERS"
    id_name = "PURCHASEORDER_ID"

    # call this with PurchaseOrder.from_number(order_number)
    @classmethod
    def from_number(cls, order_number):
        "Returns a PurchaseOrder instance given the order_number"
        return cls.object_from_field("ORDER_NUMBER", order_number)

    # call this with PurchaseOrder.from_gridrow(row)
    @classmethod
    def from_gridrow(cls, row):
        "Returns a PurchaseOrder instance given the gridrow of the order"
        return cls.object_from_field("GRIDROW", row)

    ### property getters and setters

    def get_company(self):
        company_id = self.get_column("COMPANY_ID")
        if not company_id: return None
        return Company(company_id)

    def get_supplier(self):
        supplier_id = self.get_column("SUPPLIER_ID")
        if not supplier_id: return None
        return Supplier(supplier_id)

    def get_print_check(self):
        print_check = self.get_column("PRINT_CHECK")
        return True if print_check else False

    def get_supplier_address(self):
        supplier_address_id = self.get_column("SUPPLIER_ADDRESS_ID")
        if not supplier_address_id: return None
        return Address(supplier_address_id)

    def get_sup_ref_field(self):
        return self.get_column("SUPPLIER_REFERENCE")

    def get_invoice_address(self):
        invoice_address_id = self.get_column("INVOICE_ADDRESS_ID")
        if not invoice_address_id: return None
        return Address(invoice_address_id)

    def get_delivery_address(self):
        delivery_address_id = self.get_column("DELIVERY_ADDRESS_ID")
        if not delivery_address_id: return None
        return Address(delivery_address_id)

    def get_project(self):
        project_id = self.get_column("PROJECT_ID")
        if not project_id: return None
        return Project(project_id)

    def get_datestamp(self):
        desc,date = self.transaction_description_date()
        return date

    def get_description(self):
        desc,date = self.transaction_description_date()
        return desc

    def get_number(self):
        return self.get_column("ORDER_NUMBER")

    def get_contact(self):
        contact_id = self.get_column("CONTACT_ID")
        if not contact_id: return None
        return Contact(contact_id)

    def get_transaction(self):
        transaction_id = self.get_column("TRANSACTION_ID")
        if not transaction_id: return None
        return Transaction(transaction_id)

    def get_discount(self):
        discount = self.get_column("DISCOUNT")
        return D(discount)

    def get_vat(self):
        vat = self.get_column("VAT")
        return D(vat)

    def get_cost(self):
        cost = self.get_column("COST")
        return D(cost)

    def get_total(self):
        return self.get_vat()+self.get_cost()

    def get_invoice_vat(self):
        invoice_vat = self.get_column("INVOICE_VAT")
        return D(invoice_vat)

    def get_invoice_cost(self):
        invoice_cost = self.get_column("INVOICE_COST")
        return D(invoice_cost)

    def get_paid(self):
        paid = self.get_column("PAID")
        return D(paid)

    def get_outstanding(self):
        if not self.get_is_open():
            return D("0.00")
        return self.get_vat()+self.get_cost()-self.get_paid()

    def get_status(self):
        return self.get_column("STATUS")

    def get_is_open(self):
        order_status = self.get_status()
        if order_status == 0 or order_status == 1:
            return True
        else:
            return False

    def get_open_and_invoiced(self):
        order_status = self.get_status()
        return True if order_status == 1 else False

    def get_is_cancelled(self):
        order_status = self.get_status()
        return True if order_status == 5 else False

    def get_closedorder(self):
        closedorder_id = self.get_column("CLOSEDORDER_ID")
        if not closedorder_id: return None
        return ClosedOrder(closedorder_id)

    def get_closed_reason(self):
        closedorder_id = self.get_column("CLOSEDORDER_ID")
        if not closedorder_id:
            return ""
        return select("SELECT REASON FROM CLOSEDORDERS WHERE CLOSEDORDER_ID = ?",
                                    (closedorder_id,))

    def get_closed_date(self):
        closedorder_id = self.get_column("CLOSEDORDER_ID")
        if not closedorder_id:
            return None
        return select("""SELECT DATESTAMP
                         FROM TRANSACTIONS, CLOSEDORDERS
                         WHERE CLOSEDORDER_ID = ?
                         AND CLOSEDORDERS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                         """, (closedorder_id,))

    def get_closed_transaction(self):
        closedorder_id = self.get_column("CLOSEDORDER_ID")
        if not closedorder_id:
            return None
        transaction_id = select("SELECT TRANSACTION_ID FROM CLOSEDORDERS WHERE CLOSEDORDER_ID = ?",
                                 (closedorder_id,))
        if not transaction_id:
            return None
        return Transaction(transaction_id)

    def get_gridrow(self):
        return self.get_column("GRIDROW")

    def get_show_vat(self):
        show_vat = self.get_column("SHOW_VAT")
        return True if show_vat else False

    def get_claim_vat(self):
        claim_vat = self.get_column("CLAIM_VAT")
        return True if claim_vat else False

    def get_item_list(self):
        # Each item_list element being a dictionary of
        # {"Qty":Decimal, "Description":string, "Unit Cost":Decimal, "Discount":boolean "VAT":Decimal (The % value)}

        # Qty is a Decimal value
        # Description is a string
        # Unit Cost is a Decimal value
        # Discount is True if this item has a discount applied, false otherwise
        # VAT is a Decimal value - being the percentage
        ordered_item_ids = list_select("""SELECT ORDEREDITEM_ID
                                          FROM ORDEREDITEMS WHERE PURCHASEORDER_ID = ?
                                          ORDER BY ORDEREDITEM_ID""", (self.id,))
        item_list = []
        for item_id in ordered_item_ids:
            item = {}
            # Get Qty
            qty = select("SELECT QUANTITY FROM ORDEREDITEMS WHERE ORDEREDITEM_ID = ?", (item_id,))
            item["Qty"] = D(qty)
            # Get Description
            idesc_id = select("SELECT ITEMDESCRIPTION_ID FROM ORDEREDITEMS WHERE ORDEREDITEM_ID = ?", (item_id,))
            itemdescription = ItemDescription(idesc_id)
            item["Description"] = itemdescription.description
            # Get Unit Cost
            unit_cost = select("SELECT UNIT_COST FROM ORDEREDITEMS WHERE ORDEREDITEM_ID = ?", (item_id,))
            item["Unit Cost"] = D(unit_cost)
            # Get Discount
            discount = select("SELECT DISCOUNT FROM ORDEREDITEMS WHERE ORDEREDITEM_ID = ?", (item_id,))
            if discount == 1:
                item["Discount"] = True
            else:
                item["Discount"] = False
            # Get VAT
            vat = select("SELECT VAT_PERCENT FROM ORDEREDITEMS WHERE ORDEREDITEM_ID = ?", (item_id,))
            item["VAT"] = D(vat)
            # And add this item to the list
            item_list.append(item)
        return item_list

    def set_project(self, val):
        project_id = None
        if val:
            project_id = val.id
        self.set_column("PROJECT_ID", project_id)

    def set_invoice_vat(self, val):
        self.set_column("INVOICE_VAT", str(val))

    def set_invoice_cost(self, val):
        self.set_column("INVOICE_COST", str(val))

    def set_paid(self, val):
        self.set_column("PAID", str(val))

    def set_status(self, val):
        self.set_column("STATUS", val)

    def set_closed_reason(self, val):
        closedorder = self.get_closedorder()
        if not closedorder:
            raise VS_ERROR, "No closed order object exists for this order"
        closedorder.reason = val

    def set_claim_vat(self, val):
        val = 1 if val else 0
        # Set the claim_vat in the order
        # Set the claim vat on associated sepplier invoices
        if not vs_cfg["WRITE_ACCESS"]:
            # Another process has locked the database
            raise VS_ANOTHER_PROCESS, "Database change cancelled - another process has locked this database" 
        conn = None
        try:
            conn, c = vs_connect()
            # update the database
            c.executescript("""
                  UPDATE PURCHASEORDERS SET CLAIM_VAT = %s WHERE PURCHASEORDER_ID = %s;
                  UPDATE SUPPLIERINVOICES SET CLAIM_VAT = %s WHERE PURCHASEORDER_ID = %s;
                  UPDATE SUPPLIERCREDITNOTES SET CLAIM_VAT = %s WHERE PURCHASEORDER_ID = %s;
                  UPDATE SUPPLIERINVOICEPAYMENTS SET CLAIM_VAT = %s WHERE SUPPLIERINVOICE_ID IN
                      (SELECT SUPPLIERINVOICE_ID FROM SUPPLIERINVOICES WHERE PURCHASEORDER_ID = %s);
               """ % (val, self.id, val, self.id, val, self.id, val, self.id ))
        finally:
            if conn:
                conn.close()

    company               = property(get_company)
    supplier              = property(get_supplier)
    print_check           = property(get_print_check)
    supplier_address      = property(get_supplier_address)
    sup_ref_field         = property(get_sup_ref_field)
    invoice_address       = property(get_invoice_address)
    delivery_address      = property(get_delivery_address)
    project               = property(get_project,  set_project)
    datestamp             = property(get_datestamp)
    description           = property(get_description)
    number                = property(get_number)
    contact               = property(get_contact)
    transaction           = property(get_transaction)
    discount              = property(get_discount)
    vat                   = property(get_vat)
    cost                  = property(get_cost)
    total                 = property(get_total)
    outstanding           = property(get_outstanding)
    item_list             = property(get_item_list)
    invoice_vat           = property(get_invoice_vat,  set_invoice_vat)
    invoice_cost          = property(get_invoice_cost, set_invoice_cost)
    paid                  = property(get_paid,         set_paid)
    status                = property(get_status,       set_status)
    is_open               = property(get_is_open)
    open_and_invoiced     = property(get_open_and_invoiced)
    is_cancelled          = property(get_is_cancelled)
    closedorder           = property(get_closedorder)
    closed_reason         = property(get_closed_reason, set_closed_reason)
    closed_date           = property(get_closed_date)
    closed_transaction    = property(get_closed_transaction)
    gridrow               = property(get_gridrow)
    show_vat              = property(get_show_vat)
    claim_vat              = property(get_claim_vat, set_claim_vat)

    # Methods - not set as attributes

    def get_amount(self):
        "Return a list of Qty * Unit Cost - as Decimal values"
        amount_list = []
        item_list = self.get_item_list()
        for item in item_list:
            result = item["Qty"] * item["Unit Cost"]
            amount_list.append(result.quantize(D('.01'), rounding=decimal.ROUND_DOWN))
        return amount_list

    def get_payments(self):
        "Return a list of payments associated with this purchase order"
        id_list = list_select("""SELECT MAKEPAYMENT_ID
                                 FROM MAKEPAYMENTS, TRANSACTIONS
                                 WHERE PURCHASEORDER_ID = ? AND
                                 MAKEPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                                 ORDER BY DATESTAMP""", (self.id,))
        return [ MakePayment(payment_id) for payment_id in id_list ]

    def get_invoices(self):
        "Return a list of supplier invoices associated with this purchase order"
        id_list = list_select("""SELECT SUPPLIERINVOICE_ID
                                 FROM SUPPLIERINVOICES, TRANSACTIONS
                                 WHERE PURCHASEORDER_ID = ?
                                 AND SUPPLIERINVOICES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                                 ORDER BY TRANSACTIONS.DATESTAMP""", (self.id,))
        return [ SupplierInvoice(invoice_id) for invoice_id in id_list ]

    def get_openinvoices(self):
        "Return a list of open supplier invoices associated with this purchase order"
        id_list = list_select("""SELECT SUPPLIERINVOICE_ID
                                 FROM SUPPLIERINVOICES, TRANSACTIONS
                                 WHERE PURCHASEORDER_ID = ?
                                       AND OPEN = 1
                                 AND SUPPLIERINVOICES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                                 ORDER BY TRANSACTIONS.DATESTAMP""", (self.id,))
        return [ SupplierInvoice(invoice_id) for invoice_id in id_list ]

    def get_creditnotes(self):
        "Return a list of supplier creditnotes associated with this purchase order"
        id_list = list_select("""SELECT SUPPLIERCREDITNOTE_ID
                                 FROM SUPPLIERCREDITNOTES, TRANSACTIONS
                                 WHERE SUPPLIERCREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                                 AND PURCHASEORDER_ID = ? ORDER BY TRANSACTIONS.DATESTAMP""", (self.id,))
        return [ SupplierCreditNote(note_id) for note_id in id_list ]

    def get_all_transactions(self):
        "Returns a list of all transactions associated with this order"
        # The first is the order creation transaction
        transactions = [self.get_transaction()]

        if not self.get_is_cancelled():
            # Get a list of all invoices on the order
            invoices = self.get_invoices()
            for inv in invoices:
                transactions.append(inv.transaction)

            # Get a list of all creditnotes on the order
            creditnotes = self.get_creditnotes()
            for cre in creditnotes:
                transactions.append(cre.transaction)

            # Any payment transactions
            payments = self.get_payments()
            for pay in payments:
                transactions.append(pay.transaction)

        # Has order been closed with an extra transaction (cancelled or over/under spend)
        if self.get_status() > 2:
            transactions.append(self.get_closed_transaction())

        # order by transaction gridrow
        transactions.sort(key = lambda x: x.gridrow)

        return transactions


###################### CLOSEDORDER ###########################

class ClosedOrder(_VSDatabase):
    """An object describing a row in the CLOSEDORDERS table - called with ClosedOrder(id) which
       gets a closed order from the CLOSEDORDERS table with the given id. To create an entirely
       new closedorder on the database, use the vatstuff_packages.vs_database.save_closeorder.save() function"""

    # The table_name and id_name are class attributes
    table_name = "CLOSEDORDERS"
    id_name = "CLOSEDORDER_ID"

    def get_transaction(self):
        transaction_id = self.get_column("TRANSACTION_ID")
        if not transaction_id: return None
        return Transaction(transaction_id)

    def get_datestamp(self):
        desc,date = self.transaction_description_date()
        return date

    def get_purchaseorder(self):
        purchaseorder_id = select("""SELECT PURCHASEORDER_ID
                                     FROM PURCHASEORDERS
                                     WHERE CLOSEDORDER_ID = ?""", (self.id,))
        if not purchaseorder_id: return None
        return PurchaseOrder(purchaseorder_id)

    def get_reason(self):
        return self.get_column("REASON")

    def set_reason(self, val):
        self.set_column("REASON", val)

    transaction   = property(get_transaction)
    datestamp     = property(get_datestamp)
    reason        = property(get_reason, set_reason)
    purchaseorder = property(get_purchaseorder)


# -------------- MakePayment object

class MakePayment(_VSDatabase):
    """Used to retrieve payments made to suppliers data from the database - called with MakePayment(id)
       which gets a payment from the MAKEPAYMENTS table with the given id. To create an entirely
       new payment on the database, use save function in the save_makepayment.py module"""

    # The table_name and id_name are class attributes
    table_name = "MAKEPAYMENTS"
    id_name = "MAKEPAYMENT_ID" 

    def get_supplier(self):
        supplier_id = self.get_column("SUPPLIER_ID")
        if not supplier_id: return None
        return Supplier(supplier_id)

    def get_supplier_address(self):
        supplier_address_id = self.get_column("SUPPLIER_ADDRESS_ID")
        if not supplier_address_id: return None
        return Address(supplier_address_id)

    def get_purchaseorder(self):
        purchaseorder_id = self.get_column("PURCHASEORDER_ID")
        if not purchaseorder_id: return None
        return PurchaseOrder(purchaseorder_id)

    def get_company(self):
        company_id = self.get_column("COMPANY_ID")
        if not company_id: return None
        return Company(company_id)

    def get_print_check(self):
        "return the print_check of the associated purchase order"
        purchaseorder = self.get_purchaseorder()
        if not purchaseorder:
            return False
        return purchaseorder.print_check

    def get_datestamp(self):
        desc,date = self.transaction_description_date()
        return date

    def get_description(self):
        desc,date = self.transaction_description_date()
        return desc

    def get_account(self):
        account_id = self.get_column("ACCOUNT_ID")
        if not account_id: return None
        return Account(account_id)

    def get_transaction(self):
        transaction_id = self.get_column("TRANSACTION_ID")
        if not transaction_id: return None
        return Transaction(transaction_id)

    def get_cost(self):
        cost_list = list_select("""SELECT COST
                                       FROM SUPPLIERINVOICEPAYMENTS
                                       WHERE MAKEPAYMENT_ID = ?""", (self.id,))
        cost = D("0.00")
        for c in cost_list:
            cost += D(c)
        return cost 


    def get_vat(self):
        vat_list = list_select("""SELECT VAT
                                       FROM SUPPLIERINVOICEPAYMENTS
                                       WHERE MAKEPAYMENT_ID = ?""", (self.id,))
        vat = D("0.00")
        for v in vat_list:
            vat += D(v)
        return vat 

    def get_amount(self):
        amount = self.get_column("AMOUNT")
        return D(amount)

    def get_status(self):
        return self.get_column("STATUS")

    def set_status(self, val):
        self.set_column("STATUS", val)

    # STATUS
    # 0 if this payment is a normal payment
    # 1 if this is a payment for a 'quick purchase' order


    supplier            = property(get_supplier)
    supplier_address    = property(get_supplier_address)
    purchaseorder       = property(get_purchaseorder)
    company             = property(get_company)
    print_check         = property(get_print_check)
    datestamp           = property(get_datestamp)
    description         = property(get_description)
    account             = property(get_account)
    transaction         = property(get_transaction)
    vat                 = property(get_vat)
    cost                = property(get_cost)
    amount              = property(get_amount)
    status              = property(get_status,       set_status)


    def get_invoice_list(self):
        """Returns a list of dictionaries, each dictionary being:
            { "INVOICE":supplierinvoice,
                 "COST":cost,
                  "VAT":vat }         one dictionary for each supplierinvoice this makepayment covers"""
        conn = None
        try:
            conn, c = vs_connect()

            c.execute("""SELECT SUPPLIERINVOICEPAYMENTS.SUPPLIERINVOICE_ID,
                                SUPPLIERINVOICEPAYMENTS.COST,
                                SUPPLIERINVOICEPAYMENTS.VAT
                          FROM SUPPLIERINVOICEPAYMENTS, SUPPLIERINVOICES
                          WHERE SUPPLIERINVOICEPAYMENTS.SUPPLIERINVOICE_ID = SUPPLIERINVOICES.SUPPLIERINVOICE_ID
                                   AND MAKEPAYMENT_ID = ?
                                   ORDER BY TRANSACTION_ID""", (self.id,))
            result = c.fetchall()
        finally:
            if conn:
                conn.close()

        if not result:
            return []

        return [ {"INVOICE":SupplierInvoice(invoice_id),
                  "COST":D(cost),
                  "VAT":D(vat)} for invoice_id, cost, vat in result ]


    def get_invoices(self):
        "Returns a list of supplierinvoices this makepayment covers, ordered by supplierinvoice date"
        result = list_select("""SELECT SUPPLIERINVOICEPAYMENTS.SUPPLIERINVOICE_ID
                             FROM SUPPLIERINVOICEPAYMENTS, SUPPLIERINVOICES, TRANSACTIONS
                             WHERE SUPPLIERINVOICEPAYMENTS.SUPPLIERINVOICE_ID = SUPPLIERINVOICES.SUPPLIERINVOICE_ID
                             AND SUPPLIERINVOICES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                             AND MAKEPAYMENT_ID = ?
                             ORDER BY DATESTAMP""", (self.id,))
        return [ SupplierInvoice(invoice_id) for invoice_id in result ]


# -------------- SupplierInvoice object

class SupplierInvoice(_VSDatabase):
    """Used to retrieve supplier invoice data from the database - called with SupplierInvoice(id)
       which gets a supplier invoice from the SUPPLIERINVOICES table with the given id. To create an entirely
       new invoice on the database, use save function in the save_supplierinvoice.py module"""

    # The table_name and id_name are class attributes
    table_name = "SUPPLIERINVOICES"
    id_name = "SUPPLIERINVOICE_ID" 

    def get_supplier(self):
        supplier_id = self.get_column("SUPPLIER_ID")
        if not supplier_id: return None
        return Supplier(supplier_id)

    def get_purchaseorder(self):
        purchaseorder_id = self.get_column("PURCHASEORDER_ID")
        if not purchaseorder_id: return None
        return PurchaseOrder(purchaseorder_id)

    def get_account(self):
        "The stock or expense account debited by this order"
        # Three accounts are associated with this order transaction, two are system
        # accounts, so get the account with SYSTEM value less than 2
        account_id = select("""SELECT ACCOUNTS.ACCOUNT_ID
                           FROM SUPPLIERINVOICES, TRANSACTIONS, TRANSACTIONACCOUNTS, ACCOUNTS
                           WHERE SUPPLIERINVOICES.SUPPLIERINVOICE_ID = ?
                           AND SUPPLIERINVOICES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                           AND TRANSACTIONACCOUNTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                           AND TRANSACTIONACCOUNTS.ACCOUNT_ID = ACCOUNTS.ACCOUNT_ID
                           AND ACCOUNTS.SYSTEM < 2""", (self.id,))
        if not account_id: return None
        return Account(account_id)

    def get_transaction(self):
        transaction_id = self.get_column("TRANSACTION_ID")
        if not transaction_id: return None
        return Transaction(transaction_id)

    def get_datestamp(self):
        desc,date = self.transaction_description_date()
        return date

    def get_description(self):
        desc,date = self.transaction_description_date()
        return desc

    def get_date_of_taxpoint(self):
        return self.get_datestamp()

    def get_number(self):
        return self.get_column("NUMBER")

    def get_cost(self):
        cost = self.get_column("COST")
        return D(cost)

    def get_vat(self):
        vat = self.get_column("VAT")
        return D(vat)

    def get_total(self):
        return self.get_vat() + self.get_cost()

    def get_outstanding(self):
        return self.get_vat()+self.get_cost()-self.get_paid()

    def get_is_open(self):
        is_open = self.get_column("OPEN")
        if is_open == 1:
            return True
        else:
            return False

    def get_paid(self):
        paid = self.get_column("PAID")
        return D(paid)

    def get_vat_return(self):
        vat_return_id = self.get_column("VAT_RETURN_ID")
        if not vat_return_id: return None
        return VATReturn(vat_return_id)

    def get_claim_vat(self):
        claim_vat = self.get_column("CLAIM_VAT")
        return True if claim_vat else False

    def set_number(self, val):
        self.set_column("NUMBER", val)

    def set_is_open(self, val):
        if val:
            self.set_column("OPEN", 1)
        else:
            self.set_column("OPEN", 0)

    def set_paid(self, val):
        self.set_column("PAID", str(val))

    def set_claim_vat(self, val):
        val = 1 if val else 0
        # If claim vat is changed on this invoice, it has also to be
        # changed on all the associated payments and creditnotes
        if not vs_cfg["WRITE_ACCESS"]:
            # Another process has locked the database
            raise VS_ANOTHER_PROCESS, "Database change cancelled - another process has locked this database" 
        conn = None
        try:
            conn, c = vs_connect()
            # update the database
            c.executescript("""
                  UPDATE SUPPLIERINVOICES SET CLAIM_VAT = %s  WHERE SUPPLIERINVOICE_ID = %s;
                  UPDATE SUPPLIERINVOICEPAYMENTS SET CLAIM_VAT = %s  WHERE SUPPLIERINVOICE_ID = %s;
                  UPDATE SUPPLIERCREDITNOTES SET CLAIM_VAT = %s WHERE SUPPLIERINVOICE_ID = %s;
               """ % (val, self.id, val, self.id, val, self.id))
        finally:
            if conn:
                conn.close()

    supplier            = property(get_supplier)
    purchaseorder       = property(get_purchaseorder)
    account             = property(get_account)
    transaction         = property(get_transaction)
    datestamp           = property(get_datestamp)
    date_of_taxpoint    = property(get_date_of_taxpoint)
    description         = property(get_description)
    number              = property(get_number, set_number)
    cost                = property(get_cost)
    vat                 = property(get_vat)
    total               = property(get_total)
    outstanding         = property(get_outstanding)
    is_open             = property(get_is_open, set_is_open)
    paid                = property(get_paid, set_paid)
    vat_return          = property(get_vat_return)
    claim_vat           = property(get_claim_vat, set_claim_vat)

    def get_creditnotes(self):
        """Gets a list of all suppliercreditnotes, created against the supplierinvoice"""
        id_list = list_select("""SELECT SUPPLIERCREDITNOTE_ID
                                 FROM SUPPLIERCREDITNOTES, TRANSACTIONS
                                 WHERE SUPPLIERCREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                                 AND SUPPLIERINVOICE_ID = ? ORDER BY TRANSACTIONS.DATESTAMP""", (self.id,))
        return [ SupplierCreditNote(note_id) for note_id in id_list ]

    def get_payments_with_costvat(self):
        """Gets a list of all Payments made against the invoice, as a list of dictionaries
           each dictionary being of the format:
           {"PAYMENT":MakePayment, "COST":cost, "VAT":vat, "VAT_RETURN":VatReturn}"""
        conn = None
        try:
            conn, c = vs_connect()

            c.execute("""SELECT SUPPLIERINVOICEPAYMENTS.MAKEPAYMENT_ID, 
                                SUPPLIERINVOICEPAYMENTS.COST,
                                SUPPLIERINVOICEPAYMENTS.VAT,
                                SUPPLIERINVOICEPAYMENTS.VAT_RETURN_ID
                                 FROM SUPPLIERINVOICEPAYMENTS, MAKEPAYMENTS, TRANSACTIONS
                                 WHERE SUPPLIERINVOICEPAYMENTS.SUPPLIERINVOICE_ID = ?
                                 AND SUPPLIERINVOICEPAYMENTS.MAKEPAYMENT_ID = MAKEPAYMENTS.MAKEPAYMENT_ID
                                 AND TRANSACTIONS.TRANSACTION_ID = MAKEPAYMENTS.TRANSACTION_ID
                                 ORDER BY DATESTAMP""", (self.id,))

            result = c.fetchall()
        finally:
            if conn:
                conn.close()
        if not result:
            return []
        return [ {"PAYMENT":MakePayment(makepayment_id),
                  "COST":D(cost),
                  "VAT":D(vat),
                  "VAT_RETURN":VATReturn(vat_return_id) if vat_return_id else None}
                  for makepayment_id, cost, vat, vat_return_id in result ]


# -------------- SupplierCreditNote object

class SupplierCreditNote(_VSDatabase):
    """Used to retrieve Supplier Credit Note data from the database - called with SupplierCreditNote(id)
       which gets a creditnote from the SUPPLIERCREDITNOTES table with the given id. To create an entirely
       new creditnote on the database, use save function in the save_suppliercreditnote.py module"""

    # The table_name and id_name are class attributes
    table_name = "SUPPLIERCREDITNOTES"
    id_name = "SUPPLIERCREDITNOTE_ID" 

    def get_supplier(self):
        supplier_id = self.get_column("SUPPLIER_ID")
        if not supplier_id: return None
        return Supplier(supplier_id)

    def get_supplierinvoice(self):
        supplierinvoice_id = self.get_column("SUPPLIERINVOICE_ID")
        if not supplierinvoice_id: return None
        return SupplierInvoice(supplierinvoice_id)

    def get_purchaseorder(self):
        purchaseorder_id = self.get_column("PURCHASEORDER_ID")
        if not purchaseorder_id: return None
        return PurchaseOrder(purchaseorder_id)

    def get_account(self):
        account_id = self.get_column("ACCOUNT_ID")
        if not account_id: return None
        return Account(account_id)

    def get_transaction(self):
        transaction_id = self.get_column("TRANSACTION_ID")
        if not transaction_id: return None
        return Transaction(transaction_id)

    def get_datestamp(self):
        desc,date = self.transaction_description_date()
        return date

    def get_description(self):
        desc,date = self.transaction_description_date()
        return desc

    def get_number(self):
        return self.get_column("NUMBER")

    def get_vat(self):
        vat = self.get_column("VAT")
        return D(vat)

    def get_cost(self):
        cost = self.get_column("COST")
        return D(cost)

    def get_amount(self):
        return self.get_vat() + self.get_cost()

    def get_status(self):
        return self.get_column("STATUS")

    def get_vat_return(self):
        vat_return_id = self.get_column("VAT_RETURN_ID")
        if not vat_return_id: return None
        return VATReturn(vat_return_id)

    def get_claim_vat(self):
        claim_vat = self.get_column("CLAIM_VAT")
        return True if claim_vat else False

    def set_number(self, val):
        self.set_column("NUMBER", val)

    def set_claim_vat(self, val):
        val = 1 if val else 0
        self.set_column("CLAIM_VAT", val)

    supplier            = property(get_supplier)
    supplierinvoice     = property(get_supplierinvoice)
    purchaseorder       = property(get_purchaseorder)
    account             = property(get_account)
    transaction         = property(get_transaction)
    datestamp           = property(get_datestamp)
    description         = property(get_description)
    number              = property(get_number, set_number)
    vat                 = property(get_vat)
    cost                = property(get_cost)
    amount              = property(get_amount)
    status              = property(get_status)
    vat_return          = property(get_vat_return)
    claim_vat           = property(get_claim_vat, set_claim_vat)

    # STATUS - normally 0 when a creditnote is received against an invoice
    # STATUS - 1 when a creditnote is received with a payment


################# Invoice ##########################

class Invoice(_VSDatabase):
    """Used to read Invoice details - called with Invoice(id) which
       gets an invoice from the INVOICES table with the given id.
       To create an entirely new invoice on the database, use the save function
       in the save_invoice.py module"""

    # The table_name and id_name are class attributes
    table_name = "INVOICES"
    id_name = "INVOICE_ID"

    # call this with Invoice.from_number(invoice_number)
    @classmethod
    def from_number(cls, invoice_number):
        "Returns an Invoice instance given the invoice_number"
        return cls.object_from_field("INVOICE_NUMBER", invoice_number)

    # call this with Invoice.from_gridrow(row)
    @classmethod
    def from_gridrow(cls, row):
        "Returns an Invoice instance given the gridrow of the invoice"
        return cls.object_from_field("GRIDROW", row)

    def get_company(self):
        company_id = self.get_column("COMPANY_ID")
        if not company_id: return None
        return Company(company_id)

    def get_customer(self):
        customer_id = self.get_column("CUSTOMER_ID")
        if not customer_id: return None
        return Customer(customer_id)

    def get_print_check(self):
        print_check = self.get_column("PRINT_CHECK")
        return True if print_check else False

    def get_customer_address(self):
        customer_address_id = self.get_column("CUSTOMER_ADDRESS_ID")
        if not customer_address_id: return None
        return Address(customer_address_id)

    def get_cus_order_number(self):
        return self.get_column("CUSTOMER_ORDER_NUMBER")

    def get_payment_instructions(self):
        return self.get_column("PAYMENT_INSTRUCTIONS")

    def get_payment_address(self):
        payment_address_id = self.get_column("PAYMENT_ADDRESS_ID")
        if not payment_address_id: return None
        return Address(payment_address_id)

    def get_datestamp(self):
        desc,date = self.transaction_description_date()
        return date

    def get_date_of_issue(self):
        return self.get_datestamp()

    def get_date_of_supply(self):
        return self.get_column("DATE_OF_SUPPLY")

    def get_date_of_taxpoint(self):
        return self.get_column("DATE_OF_TAXPOINT")

    def get_description(self):
        desc,date = self.transaction_description_date()
        return desc

    def get_number(self):
        return self.get_column("INVOICE_NUMBER")

    def get_contact(self):
        contact_id = self.get_column("CONTACT_ID")
        if not contact_id: return None
        return Contact(contact_id)

    def get_transaction(self):
        transaction_id = self.get_column("TRANSACTION_ID")
        if not transaction_id: return None
        return Transaction(transaction_id)

    def get_discount(self):
        discount = self.get_column("DISCOUNT")
        return D(discount)

    def get_vat(self):
        vat = self.get_column("VAT")
        return D(vat)

    def get_cost(self):
        cost = self.get_column("COST")
        return D(cost)

    def get_total(self):
        return self.get_vat()+self.get_cost()

    def get_paid(self):
        paid = self.get_column("PAID")
        return D(paid)

    def get_outstanding(self):
        return self.get_vat()+self.get_cost()-self.get_paid()

    def get_is_open(self):
        return True if self.get_column("OPEN") else False

    def get_is_cancelled(self):
        return True if self.get_column("CANCELLED") else False

    def get_vat_return(self):
        vat_return_id = self.get_column("VAT_RETURN_ID")
        if not vat_return_id: return None
        return VATReturn(vat_return_id)

    def get_show_vat(self):
        return True if self.get_column("SHOW_VAT") else False

    def get_gridrow(self):
        return self.get_column("GRIDROW")

    def get_item_list(self):
        # Each item_list element being a dictionary of
        # {"Qty":Decimal, "Description":string, "Unit Cost":Decimal, "Discount":boolean "VAT":Decimal (The % value)}

        # Qty is a Decimal value
        # Description is a string
        # Unit Cost is a Decimal value
        # Discount is True if this item has a discount applied, false otherwise
        # VAT is a Decimal value - being the percentage
        sold_item_ids = list_select("""SELECT SOLDITEM_ID
                                       FROM SOLDITEMS WHERE INVOICE_ID = ?
                                       ORDER BY SOLDITEM_ID""", (self.id,))
        item_list = []
        for item_id in sold_item_ids:
            item = {}
            # Get Qty
            qty = select("SELECT QUANTITY FROM SOLDITEMS WHERE SOLDITEM_ID = ?", (item_id,))
            item["Qty"] = D(qty)
            # Get Description
            idesc_id = select("SELECT ITEMDESCRIPTION_ID FROM SOLDITEMS WHERE SOLDITEM_ID = ?", (item_id,))
            itemdescription = ItemDescription(idesc_id)
            item["Description"] = itemdescription.description
            # Get Unit Cost
            unit_cost = select("SELECT UNIT_COST FROM SOLDITEMS WHERE SOLDITEM_ID = ?", (item_id,))
            item["Unit Cost"] = D(unit_cost)
            # Get Discount
            discount = select("SELECT DISCOUNT FROM SOLDITEMS WHERE SOLDITEM_ID = ?", (item_id,))
            if discount == 1:
                item["Discount"] = True
            else:
                item["Discount"] = False
            # Get VAT
            vat = select("SELECT VAT_PERCENT FROM SOLDITEMS WHERE SOLDITEM_ID = ?", (item_id,))
            item["VAT"] = D(vat)
            # And add this item to the list
            item_list.append(item)
 
        return item_list


    def set_paid(self, val):
        self.set_column("PAID", str(val))

    def set_date_of_supply(self, val):
        self.set_column("DATE_OF_SUPPLY", val)

    def set_date_of_taxpoint(self, val):
        self.set_column("DATE_OF_TAXPOINT", val)

    def set_is_open(self, val):
        if val:
            self.set_column("OPEN", 1)
        else:
            self.set_column("OPEN", 0)

    def set_is_cancelled(self, val):
        if val:
            self.set_column("CANCELLED", 1)
        else:
            self.set_column("CANCELLED", 0)

    def set_cus_order_number(self, val):
        return self.set_column("CUSTOMER_ORDER_NUMBER", val)

    company              = property(get_company)
    customer             = property(get_customer)
    print_check          = property(get_print_check)
    customer_address     = property(get_customer_address)
    cus_order_number     = property(get_cus_order_number, set_cus_order_number)
    payment_instructions = property(get_payment_instructions)
    payment_address      = property(get_payment_address)
    datestamp            = property(get_datestamp)
    date_of_issue        = property(get_date_of_issue)
    date_of_supply       = property(get_date_of_supply,   set_date_of_supply)
    date_of_taxpoint     = property(get_date_of_taxpoint, set_date_of_taxpoint)
    description          = property(get_description)
    number               = property(get_number)
    contact              = property(get_contact)
    transaction          = property(get_transaction)
    discount             = property(get_discount)
    vat                  = property(get_vat)
    cost                 = property(get_cost)
    total                = property(get_total)
    outstanding          = property(get_outstanding)
    item_list            = property(get_item_list)
    paid                 = property(get_paid,         set_paid)
    is_open              = property(get_is_open,      set_is_open)
    is_cancelled         = property(get_is_cancelled, set_is_cancelled)
    gridrow              = property(get_gridrow)
    show_vat             = property(get_show_vat)
    vat_return           = property(get_vat_return)

    def get_amount(self):
        "Return a list of Qty * Unit Cost - as Decimal values"
        amount_list = []
        item_list = self.get_item_list()
        for item in item_list:
            result = item["Qty"] * item["Unit Cost"]
            amount_list.append(result.quantize(D('.01'), rounding=decimal.ROUND_DOWN))
        return amount_list

    def get_creditnotes(self):
        """Gets a list of all CreditNotes, created against the invoice"""
        id_list = list_select("""SELECT CREDITNOTE_ID
                                 FROM CREDITNOTES
                                 WHERE INVOICE_ID = ? ORDER BY GRIDROW""", (self.id,))
        return [ CreditNote(note_id) for note_id in id_list ]

    def get_payments(self):
        """Gets a list of all Customer Payments, against the invoice"""
        id_list = list_select("""SELECT INVOICEPAYMENTS.CUSTOMERPAYMENT_ID
                                 FROM INVOICEPAYMENTS, CUSTOMERPAYMENTS, TRANSACTIONS
                                 WHERE INVOICEPAYMENTS.INVOICE_ID = ?
                                 AND INVOICEPAYMENTS.CUSTOMERPAYMENT_ID = CUSTOMERPAYMENTS.CUSTOMERPAYMENT_ID
                                 AND TRANSACTIONS.TRANSACTION_ID = CUSTOMERPAYMENTS.TRANSACTION_ID
                                 ORDER BY DATESTAMP""", (self.id,))
        return [ CustomerPayment(customerpayment_id) for customerpayment_id in id_list ]

    def get_payments_with_costvat(self):
        """Gets a list of all Customer Payments, against the invoice, as a list of dictionaries
           each dictionary being of the format
           {"PAYMENT":CustomerPayment, "COST":cost, "VAT":vat, "VAT_RETURN":VatReturn}"""
        conn = None
        try:
            conn, c = vs_connect()
            c.execute("""SELECT INVOICEPAYMENTS.CUSTOMERPAYMENT_ID,
                                INVOICEPAYMENTS.COST,
                                INVOICEPAYMENTS.VAT,
                                INVOICEPAYMENTS.VAT_RETURN_ID
                                 FROM INVOICEPAYMENTS, CUSTOMERPAYMENTS, TRANSACTIONS
                                 WHERE INVOICEPAYMENTS.INVOICE_ID = ?
                                 AND INVOICEPAYMENTS.CUSTOMERPAYMENT_ID = CUSTOMERPAYMENTS.CUSTOMERPAYMENT_ID
                                 AND TRANSACTIONS.TRANSACTION_ID = CUSTOMERPAYMENTS.TRANSACTION_ID
                                 ORDER BY DATESTAMP""", (self.id,))
            result = c.fetchall()
        finally:
            if conn:
                conn.close()
        if not result:
            return []
        return [ {"PAYMENT":CustomerPayment(customerpayment_id),
                  "COST":D(cost),
                  "VAT":D(vat),
                  "VAT_RETURN":VATReturn(vat_return_id) if vat_return_id else None}
                  for customerpayment_id, cost, vat, vat_return_id in result ]

    def get_all_transactions(self):
        "Return a list of all transactions associated with this invoice"
        # The first is the invoice creation transaction
        transactions = [self.get_transaction()]

        # Then any credit note transactions
        credit_notes = self.get_creditnotes()
        for note in credit_notes:
            transactions.append(note.transaction)

        # Then any payment transactions
        payments = self.get_payments()
        for pay in payments:
            transactions.append(pay.transaction)

        # order by transaction datestamp
        transactions.sort(key = lambda x: x.datestamp)

        return transactions


# -------------- CustomerPayment object

class CustomerPayment(_VSDatabase):
    """Used to retrieve customer payment data from the database - called with CustomerPayment(id)
       which gets a payment from the CUSTOMERPAYMENTS table with the given id. To create an entirely
       new payment on the database, use save function in the save_payment.py module"""

    # The table_name and id_name are class attributes
    table_name = "CUSTOMERPAYMENTS"
    id_name = "CUSTOMERPAYMENT_ID"

    def get_customer(self):
        customer_id = self.get_column("CUSTOMER_ID")
        if not customer_id: return None
        return Customer(customer_id)

    def get_customer_address(self):
        customer_address_id = self.get_column("CUSTOMER_ADDRESS_ID")
        if not customer_address_id: return None
        return Address(customer_address_id)

    def get_company(self):
        company_id = self.get_column("COMPANY_ID")
        if not company_id: return None
        return Company(company_id)

    def get_print_check(self):
        "return the print_check of the associated first invoice"
        invoice_list = self.get_invoices()
        if not invoice_list:
            return False
        return invoice_list[0].print_check

    def get_datestamp(self):
        desc,date = self.transaction_description_date()
        return date

    def get_description(self):
        desc,date = self.transaction_description_date()
        return desc

    def get_customer_reference(self):
        return self.get_column("CUSTOMER_REFERENCE")

    def get_account(self):
        account_id = self.get_column("ACCOUNT_ID")
        if not account_id: return None
        return Account(account_id)

    def get_transaction(self):
        transaction_id = self.get_column("TRANSACTION_ID")
        if not transaction_id: return None
        return Transaction(transaction_id)

    def get_cost(self):
        cost_list = list_select("""SELECT COST
                                       FROM INVOICEPAYMENTS
                                       WHERE CUSTOMERPAYMENT_ID = ?""", (self.id,))
        cost = D("0.00")
        for c in cost_list:
            cost += D(c)
        return cost 


    def get_vat(self):
        vat_list = list_select("""SELECT VAT
                                       FROM INVOICEPAYMENTS
                                       WHERE CUSTOMERPAYMENT_ID = ?""", (self.id,))
        vat = D("0.00")
        for v in vat_list:
            vat += D(v)
        return vat 


    def get_amount(self):
        amount = self.get_column("AMOUNT")
        return D(amount)

    def set_customer_address(self, val):
        """val is a customer address - this is settable as the receipt can be created
           after the payment is created, so the customer address on the receipt needs
           to be set here"""
        if val:
            self.set_column("CUSTOMER_ADDRESS_ID", val.id)
            # also set the customer address edit new flag
            val.edit_new = True
        else:
            self.set_column("CUSTOMER_ADDRESS_ID", None)

    def set_customer_reference(self, val):
        self.set_column("CUSTOMER_REFERENCE", val)

    customer            = property(get_customer)
    customer_address    = property(get_customer_address,   set_customer_address)
    company             = property(get_company)
    print_check         = property(get_print_check)
    datestamp           = property(get_datestamp)
    description         = property(get_description)
    customer_reference  = property(get_customer_reference, set_customer_reference)
    account             = property(get_account)
    transaction         = property(get_transaction)
    cost                = property(get_cost)
    vat                 = property(get_vat)
    amount              = property(get_amount)


    def data_deleted(self):
        """Returns True if some of the data associated with this payment i.e the invoice
           has been deleted - in which case the payment details cannot be displayed as they 
           are no longer available"""
        blank_invoice_references = select("""SELECT COUNT(*) FROM INVOICEPAYMENTS
                                             WHERE INVOICE_ID IS NULL
                                             AND CUSTOMERPAYMENT_ID = ?""", (self.id,))
        if blank_invoice_references:
            # There are blank invoice references, therefore data has been deleted
            return True
        else:
            return False


    def get_invoices(self):
        "Returns a list of invoices associated with this payment"
        invoice_ids = list_select("""SELECT INVOICEPAYMENTS.INVOICE_ID
                                       FROM INVOICEPAYMENTS, INVOICES
                                       WHERE INVOICEPAYMENTS.INVOICE_ID = INVOICES.INVOICE_ID
                                       AND CUSTOMERPAYMENT_ID = ?
                                       ORDER BY GRIDROW""", (self.id,))
        return [ Invoice(invoice_id) for invoice_id in invoice_ids ]

    def get_invoice_list(self):
        """Returns a list of dictionaries, each dictionary being:
            { "INVOICE":invoice,
                 "COST":cost,
                  "VAT":vat }         one dictionary for each invoice this payment covers"""
        conn = None
        try:
            conn,c = vs_connect()

            c.execute("""SELECT INVOICEPAYMENTS.INVOICE_ID, INVOICEPAYMENTS.COST, INVOICEPAYMENTS.VAT
                                       FROM INVOICEPAYMENTS, INVOICES
                                       WHERE INVOICEPAYMENTS.INVOICE_ID = INVOICES.INVOICE_ID
                                       AND CUSTOMERPAYMENT_ID = ?
                                       ORDER BY GRIDROW""", (self.id,))
            result = c.fetchall()
        finally:
            if conn:
                conn.close()
        if not result:
            return []
        return [ {"INVOICE":Invoice(invoice_id),
                  "COST":D(cost),
                  "VAT":D(vat)} for invoice_id, cost, vat in result ]

    def get_invoice_cost(self, invoice):
        "returns the cost on this payment associated with this invoice"
        cost = select("""SELECT COST FROM INVOICEPAYMENTS
                                       WHERE CUSTOMERPAYMENT_ID = ?
                                       AND INVOICE_ID = ?""", (self.id, invoice.id))
        return D(cost)

    def get_invoice_vat(self, invoice):
        "returns the vat on this payment associated with this invoice"
        vat = select("""SELECT VAT FROM INVOICEPAYMENTS
                                       WHERE CUSTOMERPAYMENT_ID = ?
                                       AND INVOICE_ID = ?""", (self.id, invoice.id))
        return D(vat)


# -------------- CreditNote object

class CreditNote(_VSDatabase):
    """Used to retrieve Credit Note data from the database - called with CreditNote(id) which
       gets a creditnote from the CREDITNOTES table with the given id. To create an entirely
       new creditnote on the database, use save function in the save_creditnote.py module"""

    # The table_name and id_name are class attributes
    table_name = "CREDITNOTES"
    id_name = "CREDITNOTE_ID"

    # call this with CreditNote.from_number(creditnote_number)
    @classmethod
    def from_number(cls, creditnote_number):
        "Returns a CreditNote instance given the creditnote_number"
        return cls.object_from_field("CREDITNOTE_NUMBER", creditnote_number)

    # call this with CreditNote.from_gridrow(row)
    @classmethod
    def from_gridrow(cls, row):
        "Returns a CreditNote instance given the gridrow of the note"
        return cls.object_from_field("GRIDROW", row)

    def get_company(self):
        company_id = self.get_column("COMPANY_ID")
        if not company_id: return None
        return Company(company_id)

    def get_customer(self):
        customer_id = self.get_column("CUSTOMER_ID")
        if not customer_id: return None
        return Customer(customer_id)

    def get_print_check(self):
        return True if self.get_column("PRINT_CHECK") else False

    def get_customer_address(self):
        customer_address_id = self.get_column("CUSTOMER_ADDRESS_ID")
        if not customer_address_id: return None
        return Address(customer_address_id)

    def get_invoice(self):
        invoice_id = self.get_column("INVOICE_ID")
        if not invoice_id: return None
        return Invoice(invoice_id)

    def get_datestamp(self):
        desc,date = self.transaction_description_date()
        return date

    def get_description(self):
        desc,date = self.transaction_description_date()
        return desc

    def get_reason(self):
        return self.get_column("REASON")

    def set_reason(self, val):
        self.set_column("REASON", val)

    def get_number(self):
        return self.get_column("CREDITNOTE_NUMBER")

    def get_transaction(self):
        transaction_id = self.get_column("TRANSACTION_ID")
        if not transaction_id: return None
        return Transaction(transaction_id)

    def get_contact(self):
        contact_id = self.get_column("CONTACT_ID")
        if not contact_id: return None
        return Contact(contact_id)

    def get_discount(self):
        discount = self.get_column("DISCOUNT")
        return D(discount)

    def get_show_vat(self):
        return True if self.get_column("SHOW_VAT") else False

    def get_item_list(self):
        # Each item_list element being a dictionary of
        # {"Qty":Decimal, "Description":string, "Unit Cost":Decimal, "Discount":boolean "VAT":Decimal (The % value)}

        # Qty is a Decimal value
        # Description is a string
        # Unit Cost is a Decimal value
        # Discount is True if this item has a discount applied, false otherwise
        # VAT is a Decimal value - being the percentage
        creditenote_item_ids = list_select("""SELECT CREDITNOTEITEM_ID
                                       FROM CREDITNOTEITEMS WHERE CREDITNOTE_ID = ?
                                       ORDER BY CREDITNOTEITEM_ID""", (self.id,))
        item_list = []
        for item_id in creditenote_item_ids:
            item = {}
            # Get Qty
            qty = select("SELECT QUANTITY FROM CREDITNOTEITEMS WHERE CREDITNOTEITEM_ID = ?", (item_id,))
            item["Qty"] = D(qty)
            # Get Description
            idesc_id = select("SELECT ITEMDESCRIPTION_ID FROM CREDITNOTEITEMS WHERE CREDITNOTEITEM_ID= ?", (item_id,))
            itemdescription = ItemDescription(idesc_id)
            item["Description"] = itemdescription.description
            # Get Unit Cost
            unit_cost = select("SELECT UNIT_COST FROM CREDITNOTEITEMS WHERE CREDITNOTEITEM_ID = ?", (item_id,))
            item["Unit Cost"] = D(unit_cost)
            # Get Discount
            discount = select("SELECT DISCOUNT FROM CREDITNOTEITEMS WHERE CREDITNOTEITEM_ID = ?", (item_id,))
            if discount == 1:
                item["Discount"] = True
            else:
                item["Discount"] = False
            # Get VAT
            vat = select("SELECT VAT_PERCENT FROM CREDITNOTEITEMS WHERE CREDITNOTEITEM_ID = ?", (item_id,))
            item["VAT"] = D(vat)
            # And add this item to the list
            item_list.append(item)
        return item_list


    def get_vat(self):
        vat = self.get_column("VAT")
        return D(vat)

    def get_cost(self):
        cost = self.get_column("COST")
        return D(cost)

    def get_gridrow(self):
        return self.get_column("GRIDROW")

    def get_status(self):
        return self.get_column("STATUS")

    def get_vat_return(self):
        vat_return_id = self.get_column("VAT_RETURN_ID")
        if not vat_return_id: return None
        return VATReturn(vat_return_id)

    def set_status(self, val):
        self.set_column("STATUS", val)

    company             = property(get_company)
    customer            = property(get_customer)
    print_check         = property(get_print_check)
    customer_address    = property(get_customer_address)
    invoice             = property(get_invoice)
    datestamp           = property(get_datestamp)
    description         = property(get_description)
    reason              = property(get_reason,       set_reason)
    number              = property(get_number)
    contact             = property(get_contact)
    transaction         = property(get_transaction)
    discount            = property(get_discount)
    item_list           = property(get_item_list)
    vat                 = property(get_vat)
    cost                = property(get_cost)
    gridrow             = property(get_gridrow)
    status              = property(get_status,       set_status)
    show_vat            = property(get_show_vat)
    vat_return          = property(get_vat_return)

    def data_deleted(self):
        """Returns True if some of the data associated with this creditnote i.e the invoice
           has been deleted - in which case the creditnote details cannot be displayed as they 
           are no longer available"""
        blank_invoice_references = select("""SELECT COUNT(*) FROM CREDITNOTES
                                             WHERE INVOICE_ID IS NULL
                                             AND CREDITNOTE_ID = ?
                                             AND STATUS = ?""", (self.id, 0))
        if blank_invoice_references:
            # There are blank invoice references, and creditnote status is zero, indicating
            # there should be an invoice reference. Therefore data has been deleted
            return True
        else:
            return False

    def get_amount(self):
        "Return a list of Qty * Unit Cost - as Decimal values"
        amount_list = []
        item_list = self.get_item_list()
        for item in item_list:
            result = item["Qty"] * item["Unit Cost"]
            amount_list.append(result.quantize(D('.01'), rounding=decimal.ROUND_DOWN))
        return amount_list


# -------------- VATReturn object

class VATReturn(_VSDatabase):
    """Used to retrieve VATReturn data from the database - called with VATReturn(id) which
       gets a vat return from the VAT_RETURNS table with the given id. To create an entirely
       new vat return on the database, use save function in the save_vatreturn.py module"""

    # The table_name and id_name are class attributes
    table_name = "VAT_RETURNS"
    id_name = "VAT_RETURN_ID"

    # call this with VATReturn.get_last()
    @classmethod
    def get_last(cls):
        "Returns the last VATReturn - could be None if there was not a last"
        last_vat_return_id = select("""SELECT CFG_ITEM
                                         FROM CONFIGURATION
                                        WHERE CFG_KEY = ?""", ("LAST_VAT_RETURN_ID",))
        if not last_vat_return_id: return None
        last_vat_return_id = int(last_vat_return_id)
        # This could be a zero
        if not last_vat_return_id: return None
        return cls(last_vat_return_id)

    # call this with VATReturn.get_all()
    @classmethod
    def get_all(cls):
        """"Return a list of all vat returns, ordered by from_date"""
        return cls.get_all_ordered_by("FROM_DATE")

    def get_company(self):
        company_id = self.get_column("COMPANY_ID")
        if not company_id: return None
        return Company(company_id)

    def get_datestamp(self):
        desc,date = self.transaction_description_date()
        return date

    def get_description(self):
        desc,date = self.transaction_description_date()
        return desc

    def get_vatscheme(self):
        vatscheme_id = self.get_column("VAT_SCHEME_ID")
        if not vatscheme_id: return None
        return VATScheme(vatscheme_id)

    def get_method(self):
        vatscheme = self.get_vatscheme()
        if not vatscheme:
            return None
        return vatscheme.method

    def get_previous_vatreturn(self):
        previous_vatreturn_id = self.get_column("PREVIOUS_VAT_RETURN_ID")
        if not previous_vatreturn_id: return None
        return VATReturn(previous_vatreturn_id)

    def get_standard_rate(self):
        "Returns the standard vat rate when this return was submitted"
        standard_rate = self.get_column("STANDARD_RATE")
        if not standard_rate: return vs_cfg["VAT_STANDARD_RATE"]
        return D(standard_rate)

    def get_transaction(self):
        transaction_id = self.get_column("TRANSACTION_ID")
        if not transaction_id: return None
        return Transaction(transaction_id)

    def get_from_date(self):
        return self.get_column("FROM_DATE")

    def get_to_date(self):
        return self.get_column("TO_DATE")

    def get_account(self):
        account_id = self.get_column("ACCOUNT_ID")
        if not account_id: return None
        return Account(account_id)

    def get_box_one(self):
        text =  self.get_column("BOX_ONE")
        if not text: return D("0.00")
        return D(text)

    def get_box_two(self):
        text =  self.get_column("BOX_TWO")
        if not text: return D("0.00")
        return D(text)

    def get_box_three(self):
        text =  self.get_column("BOX_THREE")
        if not text: return D("0.00")
        return D(text)

    def get_box_four(self):
        text =  self.get_column("BOX_FOUR")
        if not text: return D("0.00")
        return D(text)

    def get_box_five(self):
        text =  self.get_column("BOX_FIVE")
        if not text: return D("0.00")
        return D(text)

    def get_box_six(self):
        text =  self.get_column("BOX_SIX")
        if not text: return D("0.00")
        return D(text)

    def get_box_seven(self):
        text =  self.get_column("BOX_SEVEN")
        if not text: return D("0.00")
        return D(text)

    def get_box_eight(self):
        text =  self.get_column("BOX_EIGHT")
        if not text: return D("0.00")
        return D(text)

    def get_box_nine(self):
        text =  self.get_column("BOX_NINE")
        if not text: return D("0.00")
        return D(text)

    company             = property(get_company)
    datestamp           = property(get_datestamp)
    description         = property(get_description)
    vatscheme           = property(get_vatscheme)
    method              = property(get_method)
    previous_vatreturn  = property(get_previous_vatreturn)
    standard_rate       = property(get_standard_rate)
    transaction         = property(get_transaction)
    from_date           = property(get_from_date)
    to_date             = property(get_to_date)
    account             = property(get_account)
    box_one             = property(get_box_one)
    box_two             = property(get_box_two)
    box_three           = property(get_box_three)
    box_four            = property(get_box_four)
    box_five            = property(get_box_five)
    box_six             = property(get_box_six)
    box_seven           = property(get_box_seven)
    box_eight           = property(get_box_eight)
    box_nine            = property(get_box_nine)

    def list_invoice_ids(self):
        "Return a list of invoice id's with this vat return set"
        return list_select("SELECT INVOICE_ID FROM INVOICES WHERE VAT_RETURN_ID = ?", (self.id,))

    def list_creditnote_ids(self):
        "Return a list of creditnote id's with this vat return set"
        return list_select("SELECT CREDITNOTE_ID FROM CREDITNOTES WHERE VAT_RETURN_ID = ?", (self.id,))

    def list_supplierinvoice_ids(self):
        "Return a list of supplierinvoice id's with this vat return set"
        return list_select("SELECT SUPPLIERINVOICE_ID FROM SUPPLIERINVOICES WHERE VAT_RETURN_ID = ?", (self.id,))

    def list_suppliercreditnote_ids(self):
        "Return a list of suppliercreditnote id's with this vat return set"
        return list_select("SELECT SUPPLIERCREDITNOTE_ID FROM SUPPLIERCREDITNOTES WHERE VAT_RETURN_ID = ?", (self.id,))

    def list_customerpayment_ids(self):
        "Return a list of customerpayment id's with this vat return set"
        return list_select("SELECT CUSTOMERPAYMENT_ID FROM CUSTOMERPAYMENTS WHERE VAT_RETURN_ID = ?", (self.id,))

    def list_makepayment_ids(self):
        "Return a list of makepayment id's with this vat return set"
        return list_select("SELECT MAKEPAYMENT_ID FROM MAKEPAYMENTS WHERE VAT_RETURN_ID = ?", (self.id,))


