####### VATStuff #######
#
# odd_functions.py  - miscellaneous functions
#
# Version : 0.6
# Date : 20100817
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#


"""miscellaneous functions, required thoughout the vatstuff program"""

# Do not import wx - so these functions can be used by testing
# processes without the graphical wx being present


import decimal
import datetime
import time
import os

from vatstuff_packages.vs_utilities.config import *
from vatstuff_packages.vs_utilities.objects import *


D = decimal.Decimal   # D used to avoid typing

# The CONFIGURATION table has the following keys
_KEY_LIST     = ["PROGRAM_ID",
                "PROGRAM_VERSION",
                "DATABASE_VERSION",                       
                "CURRENCY_SYMBOL",
                "CURRENCY_POINT",
                "CURRENCY_THOUSANDS_SEPARATOR",
                "COMPANY_ID",
                "VAT_SCHEME_ID",
                "LAST_VAT_RETURN_ID",
                "LAST_ORDER_NUMBER",
                "LAST_INVOICE_NUMBER",
                "LAST_CREDITNOTE_NUMBER",
                "LAST_CASH_ACCOUNT_ID",
                "LAST_STOCK_ACCOUNT_ID",
                "CUSTOMER_ACCOUNT_TYPE_ID",
                "SUPPLIER_ACCOUNT_TYPE_ID",
                "LOGO_FILE",
                "DELETE_TO_DATE"
                ]

# ------------ Set the database to be used by VATStuff

def set_database_filepath(database_filepath, logo_file = None):
    """Sets the database folder and filepath to that given in database_filepath
       Note, logo_file (filename - not full path), is stored and read from the
       database, but as this function could be called when creating a new database
       then this is the value that will be input into the new database.
       Normally it is None, so the function uses vs_cfg["LOGO_IMAGE"] as
       the default"""

    database_directory = os.path.dirname(database_filepath)
    database_file = os.path.basename(database_filepath)

    vs_cfg["DATABASE_FILEPATH"] = database_filepath
    if not logo_file:
        vs_cfg["LOGO_FILE"] = vs_cfg["LOGO_IMAGE"]
    else:
        vs_cfg["LOGO_FILE"] = logo_file

    # Set the database folder and file in the per-user store and in vs_cfg
    set_stored_item("DATABASE_FOLDER", database_directory)
    set_stored_item("DATABASE_FILE", database_file)

    # The lock_folder and logo_folder are currently the same folder as the database folder

    vs_cfg["LOCK_FOLDER"]=vs_cfg["DATABASE_FOLDER"]
    vs_cfg["LOCK_FILE"]=vs_cfg["DATABASE_FILE"]+".vslockfile"
    vs_cfg["LOCK_FILEPATH"]=os.path.join(vs_cfg["LOCK_FOLDER"], vs_cfg["LOCK_FILE"])

    vs_cfg["LOGO_FOLDER"]=vs_cfg["DATABASE_FOLDER"]
    vs_cfg["LOGO_FILEPATH"]=os.path.join(vs_cfg["LOGO_FOLDER"], vs_cfg["LOGO_FILE"])


# -------------- Check the database

def check(prog_func=None):
    """Check the new database which has been loaded, returns True, None if ok
       or False, message if not
       prog_func is an optional function which, if given, is called at intervals during the check
       this function can be used to display progress - by incrementing a progress bar"""
    
    debug_log("Checking the database")
    if prog_func: prog_func()
    try:
        # Check write access, and set vs_cfg["WRITE_ACCESS"] accordingly
        check_write_access()
        # Note: this sets vs_cfg["WRITE_ACCESS"] but it does not create a lockfile
        if prog_func: prog_func()
        # Check this database was created by this program
        program_id = get_program_id()
        if program_id != vs_cfg["PROGRAM_ID"]:
            info_log("Database incompatible, has id %s" % program_id)
            return False, "Database program id is incompatible"
        debug_log("program id %s" % program_id)

        # Check database file version,
        read_database_version = get_database_version()
        if read_database_version != vs_cfg["DATABASE_VERSION"]:
            info_log("Database version is incompatible. Has version %s" % read_database_version)
            if float(read_database_version) < float(vs_cfg["DATABASE_VERSION"]):
                return False, "Database version is incompatible"
            else:
                return False, "Database created by a later version of %s" % program_id
        debug_log("database version %s" % read_database_version)

        # Get a list of all accounts
        all_accounts = Account.get_all()
        if prog_func: prog_func()

        # get all the account balances, and as this is double entry, check they add to zero
        total = D(0)
        for account in all_accounts:
            total += account.balance
        if total:
            return False, "Database faulty, total of account balances does not equal zero"
        if prog_func: prog_func()

        # Database version checked, now try to read items from it
        currency_symbol = get_config_item("CURRENCY_SYMBOL")
        currency_point = get_config_item("CURRENCY_POINT")
        currency_thousands_separator = get_config_item("CURRENCY_THOUSANDS_SEPARATOR")
        delete_to_date = get_config_item("DELETE_TO_DATE")

        # read the LOGO_FILE
        logo_file = get_config_item("LOGO_FILE")

        # read the registered office country, and company vat country code from the database
        default_country, default_vat_country_code = read_country()

        # read the standard vat rate
        standard_rate = D(0)
        standard_vat_rate = VATRate.from_symbol("S")
        if standard_vat_rate:
            standard_rate = standard_vat_rate.rate
        else:
            standard_rate = D(0)

        # read the reduced vat rate
        reduced_rate = D(0)
        reduced_vat_rate = VATRate.from_symbol("R")
        if reduced_vat_rate:
            reduced_rate = reduced_vat_rate.rate
        else:
            reduced_rate = D(0)

        if prog_func: prog_func()

    except VS_CANCEL:
        # Do not attempt to catch a VS_CANCEL - let it propogate upwards
        raise

    except Exception, e:
        exception_log(e)
        return False, "Error during database check"

    # All seems ok, set the config dictionary values
    if logo_file:
        vs_cfg["LOGO_FILE"] = logo_file
        vs_cfg["LOGO_FILEPATH"]=os.path.join(vs_cfg["LOGO_FOLDER"], vs_cfg["LOGO_FILE"])
    if currency_symbol:
        vs_cfg["CURRENCY_SYMBOL"] = currency_symbol
        vs_cfg["CS"]= vs_unicode_symbol(currency_symbol)
    if currency_point:
        vs_cfg["CURRENCY_POINT"] = currency_point
        vs_cfg["CP"]= vs_unicode_symbol(currency_point)
    vs_cfg["ZERO_STRING"] = "0%s00" % vs_cfg["CP"]
    vs_cfg["SYMBOL_ZERO_STRING"] = "%s0%s00" % (vs_cfg["CS"], vs_cfg["CP"])
    if currency_thousands_separator:
        vs_cfg["CURRENCY_THOUSANDS_SEPARATOR"] = currency_thousands_separator
        vs_cfg["CTS"]= vs_unicode_symbol(currency_thousands_separator)
    if default_country:
        vs_cfg["COUNTRY"] = default_country
    if default_vat_country_code:
        vs_cfg["VAT_COUNTRY_CODE"] = default_vat_country_code
    if standard_rate > D(0):
        vs_cfg["VAT_STANDARD_RATE"] = standard_rate
    if reduced_rate > D(0):
        vs_cfg["VAT_REDUCED_RATE"] = reduced_rate
    vs_cfg["DELETE_TO_DATE"] = delete_to_date

    if prog_func: prog_func()
    debug_log("Database check OK")

    return True, None


def check_write_access():
    """Does the lockfile exist? If not, set vs_cfg['WRITE_ACCESS'] = True
       then return the vs_cfg['WRITE_ACCESS'] value.
       If it does exist, check its time stamp and the pid it contains"""

    vs_cfg["WRITE_ACCESS"] = False
    if not os.path.exists(vs_cfg["LOCK_FILEPATH"]):
        vs_cfg["WRITE_ACCESS"] = True
        return True

    # if it is older than two minutes, it could be left over from
    # a previous crash, so assume it is not valid, and give the user write access
    # Note: the app frame has a wxTimer which updates the lockfile every minute
    time_since_lockfile_updated = time.time() - os.stat(vs_cfg["LOCK_FILEPATH"]).st_atime
    if time_since_lockfile_updated > 120:
        vs_cfg["WRITE_ACCESS"] = True
        return True
    # If this value is negative, the system time could have changed
    # so give the user write access
    if time_since_lockfile_updated < 0:
        vs_cfg["WRITE_ACCESS"] = True
        return True

    # Read the lockfile, if it has the same pid as this process, then this process
    # owns it, and write_access should be true
    my_pid = str(os.getpid())
    fp = None
    try:
        fp = open(vs_cfg["LOCK_FILEPATH"], "r")
        contents = fp.read()
        if my_pid == contents:
            vs_cfg["WRITE_ACCESS"] = True
            return True
    finally:
        if fp:
            fp.close()

    # So a lockfile exists, and contains a different pid
    return False


def shut_lockfile():
    "Deletes the lockfile"
    if not os.path.isfile(vs_cfg["LOCK_FILEPATH"]): return
    if vs_cfg["WRITE_ACCESS"]:
        try:
            os.remove(vs_cfg["LOCK_FILEPATH"])
        except Exception, e:
            exception_log(e)
        return
    # Its possible the lockfile was created by this program
    # but vs_cfg["WRITE_ACCESS"] was not set correctly, so read the
    # lockfile pid, and if it belongs to this program, remove
    # the lockfile anyway
    my_pid = str(os.getpid())
    fp = None
    delete_lockfile = False
    try:
        fp = open(vs_cfg["LOCK_FILEPATH"], "r")
        contents = fp.read()
        if my_pid == contents:
            delete_lockfile = True
    except Exception, e:
        exception_log(e)
        return
    finally:
        if fp:
            fp.close()
    # So, if the file is ours, try again
    if delete_lockfile:
        try:
            os.remove(vs_cfg["LOCK_FILEPATH"])
        except Exception, e:
            exception_log(e)
    # so the lockfile should now be deleted, if not, we have
    # done our best, either the lockfile doesn't belong to us
    # or an error occurred which has left it in place - in which
    # case the timeout function should obsolete it


# --------------- CONFIGURATION table items

def get_config_item(key):
    """Gets the configuration item from the database given its key"""

    global _KEY_LIST
    if not key in _KEY_LIST:
        return None

    sql_string = "SELECT CFG_ITEM FROM CONFIGURATION WHERE CFG_KEY = ?"
    result = select(sql_string, (key,))

    if key == "DELETE_TO_DATE" and result:
        # return a datetime object rather than a string
        year, month, day = result.split("-")
        result = datetime.date(int(year), int(month), int(day))

    return result


def set_config_item(key, item):
    """Sets the configuration item in the database given its key and the new value
       Return True if successfull, False otherwise"""

    global _KEY_LIST
    if not key in _KEY_LIST:
        return False

    if key == "DELETE_TO_DATE" and item:
        # insert a string object rather than a date object
        item = item.isoformat()

    # item must be a string
    item = str(item)

    number = select("SELECT COUNT(*) FROM CONFIGURATION WHERE CFG_KEY = ?", (key,))
    if number > 0:
        # An item with this key exists, so update it
        sql_string = "UPDATE CONFIGURATION SET CFG_ITEM = ? WHERE CFG_KEY = ?"
        update(sql_string, (item, key))
    else:
        # No item with this key exists, so insert it
        insert("""INSERT INTO CONFIGURATION 
                                   (CFG_ITEM, CFG_KEY) VALUES (?, ?)""", (item, key))
    return True


def get_program_id():
    """Get the program id which created this database"""
    program_id = get_config_item("PROGRAM_ID")
    if not program_id:
        raise VS_ERROR, "Error trying to read the Database program ID"
    return program_id


def get_database_version():
    """Get the database version from the database"""
    # Note the database version is different from the program version
    # since the program may change versions without changing the database format.
    # The database version only changes when the database is incompatable with previous
    # databases
    database_version = get_config_item("DATABASE_VERSION")
    if not database_version:
        raise VS_ERROR, "Error trying to read the Database Version number"
    return database_version


def read_country():
    "Gets default country and vat country code from the database"

    company_id = get_config_item("COMPANY_ID")
    if not company_id:
        raise VS_ERROR, "Cannot read company information from the database"

    default_country = ""
    default_vat_country_code = ""

    conn = None
    try:
        # Make the connection
        conn, c = vs_connect()

        c.execute("SELECT ADDRESS_ID FROM COMPANIES WHERE COMPANY_ID = ?", (company_id,))
        result = c.fetchone()

        if result:
            if result[0]:
                # A registered office has been set, get it's country
                office_id = result[0]
                c.execute("SELECT COUNTRY FROM ADDRESSES WHERE ADDRESS_ID = ?", (office_id,))
                result = c.fetchone()
                if result:
                    if result[0]:
                        default_country = result[0]

        # Get vat country code
        c.execute("SELECT VAT_COUNTRY_CODE FROM COMPANIES WHERE COMPANY_ID = ?", (company_id,))
        result = c.fetchone()
        if result:
            if result[0]:
                default_vat_country_code = result[0]
    finally:
        if conn:
            conn.close()
    return default_country, default_vat_country_code


# --------------- Read Account Information

def get_last_cash_account():
    """Gets the last cash/bank account used"""
    last_account_from_config = get_config_item("LAST_CASH_ACCOUNT_ID")
    if not last_account_from_config:
        return None
    account_id = int(last_account_from_config)
    if account_id == 0:
        return None
    return Account(account_id)


def get_last_stock_account():
    """Gets the last stock/expense account used"""
    last_account_from_config = get_config_item("LAST_STOCK_ACCOUNT_ID")
    if not last_account_from_config:
        return None
    account_id = int(last_account_from_config)
    if account_id == 0:
        return None
    return Account(account_id)


def get_account_by_system(system):
    """Gets the account with the given system number"""
    if system < 2 or system == 3 or system == 4:
        # These system numbers have multiple accounts, this function only applies
        # to accounts with a dedicated system number
        return None

    sql_string = "SELECT ACCOUNT_ID FROM ACCOUNTS WHERE SYSTEM = ?"
    account_id = select(sql_string, (system,))

    if not account_id:
        return None

    return Account(account_id)


def get_accounts_by_system_list(system_list):
    "Returns a list of all accounts with system numbers in the system_list"

    # This is aimed for Choice menu options, to reduce the Accounts choices shown
    # and is therefore ordered alphabetically by folder name
         
    all_folders = Folder.get_all()   # List of folders in alphabetical order
    all_accounts = []

    for folder in all_folders:
        # Get the list of accounts in this folder, sorted in alphabetical order
        all_accounts.extend(folder.get_accounts())

    chosen_accounts = [ acc for acc in all_accounts if acc.system in system_list ]
    return chosen_accounts


def get_folder_account_lists():
    """Returns two lists, a list of all account objects in foldername:accountname alphabetical order
       plus a list of foldername:accountname's, in the same order - for use in drop down account list
       choices"""

    conn = None
    try:
        # Make the connection
        conn, c = vs_connect()

        # Get the particular account from accounts
        c.execute("""SELECT ACCOUNT_ID, FOLDERS.NAME, ACCOUNTS.NAME
                     FROM ACCOUNTS, FOLDERS
                     WHERE ACCOUNTS.FOLDER_ID = FOLDERS.FOLDER_ID
                     ORDER BY FOLDERS.NAME, ACCOUNTS.NAME""")
        result = c.fetchall()
    finally:
        if conn:
            conn.close()

    account_objects_list = [ Account(item[0]) for item in result ]
    account_names_list = [ "%s:%s" % (item[1],item[2]) for item in result ]

    return account_objects_list, account_names_list


# --------------- Read Label Information

def get_button_labels():
    """Gets a list of button labels, ordered by button"""

    sql_string = "SELECT LABEL_ID FROM LABELS WHERE BUTTON < ? ORDER BY BUTTON"
    result = list_select(sql_string, (vs_cfg["MAX_BUTTONS"],))

    if not result:
        raise VS_ERROR, "Error reading Move Money labels from the database"

    button_labels = [ Label(label_id) for label_id in result ]
    return button_labels


def get_not_button_labels():
    """Gets a list of all labels, ordered by name, and not assigned to buttons"""

    sql_string = "SELECT LABEL_ID FROM LABELS WHERE BUTTON >= ? ORDER BY NAME"
    result = list_select(sql_string, (vs_cfg["MAX_BUTTONS"],))

    if not result:
        return []

    labels = [ Label(label_id) for label_id in result ]

    return labels


# --------------- Read VAT Information

def get_previous_vat_return_scheme(vat_return=None):
    """given a vat return, get the previous vat return, scheme, and method
       if the vat_return given is None - assume current period so
       return the last vat return and scheme"""
    previous_vatreturn = None
    if vat_return:
        previous_vatreturn = vat_return.previous_vatreturn
    else:
        previous_vatreturn = VATReturn.get_last()

    if previous_vatreturn:
        previous_vatscheme = previous_vatreturn.vatscheme
        previous_method = previous_vatscheme.method
    else:
        previous_vatscheme = None
        previous_method = None

    return previous_vatreturn, previous_vatscheme, previous_method


def get_vat_return_dates(vat_return=None):
    """Returns the dates covering the vat_return, if vat_return is None, gets the dates
       of the current vat_period. returns (from_date, to_date) datetime.date objects"""
    if not vat_return:
        vat_scheme = VATScheme()
        from_date = vat_scheme.start_date
        to_date = vat_scheme.quarter_date
    else:
        from_date = vat_return.from_date
        to_date = vat_return.to_date
    return from_date, to_date


def get_latest_delete_date(initial_date = None):
    """Get the latest date to which data can be deleted. The 'delete old data' button on the
       admin tab uses this - any deletion must be prior to the date returned here.
       initial_date is normally None, and this function starts with the current date as the
       latest - and then pushes it back as it checks for open invoices, etc
       However initial_date could be set to some other date for test purposes."""
    if initial_date:
        to_date = initial_date
    else:
        to_date = datetime.date.today()

    # get the date of the earliest open purchase order, if
    # it is earlier than to_date, give to_date this value
    earliest_openorder = get_earliest_openorder()
    if earliest_openorder:
        datestamp = earliest_openorder.datestamp
        if datestamp < to_date:
            to_date = datestamp

    # get the date of the earliest open invoice, if
    # it is earlier than to_date, give to_date this value
    earliest_openinvoice = get_earliest_openinvoice()
    if earliest_openinvoice:
        datestamp = earliest_openinvoice.datestamp
        if datestamp < to_date:
            to_date = datestamp

    vat_scheme = VATScheme()
    if vat_scheme.method == "NOT_REGISTERED":
        # If not vat registered, then open invoices/orders is the only criteria needed
        return to_date

    # Get the start date of the current VAT period
    vat_period_start = vat_scheme.start_date
    if vat_period_start < to_date:
        to_date = vat_period_start

    # Set to_date to the from_date
    # of the last vat return
    last_return = VATReturn.get_last()
    if last_return:
        datestamp = last_return.from_date
        if datestamp < to_date:
            to_date = datestamp

    # If an invoice, payment or creditnote is in the current vat
    # period, then cannot delete the associated invoice/order

    # The date of start of current VAT period is vat_period_start

    ## CHECK INVOICES TO CUSTOMERS RELATED TO CUSTOMER PAYMENTS
    # Find any customer payments received with a date
    # after the start of the current VAT period
    sql_string1 = """SELECT CUSTOMERPAYMENT_ID FROM CUSTOMERPAYMENTS, TRANSACTIONS
                     WHERE DATESTAMP >= ?
                     AND CUSTOMERPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID"""

    # Find the invoices these refer to
    sql_string2 = """SELECT INVOICE_ID FROM INVOICEPAYMENTS WHERE CUSTOMERPAYMENT_ID IN (%s)""" % sql_string1

    # Get minimum datestamp for these selected invoices
    sql_string3 = """SELECT MIN(DATESTAMP) FROM TRANSACTIONS, INVOICES
                     WHERE TRANSACTIONS.TRANSACTION_ID = INVOICES.TRANSACTION_ID
                     AND INVOICE_ID IN (%s)""" % sql_string2

    datestamp = select(sql_string3, (vat_period_start,))
    if datestamp:
        # It looks as if the MIN function returns a unicode value of the datestamp, rather
        # than a datetime.date object
        if not isinstance(datestamp, datetime.date):
            int_dates = map(int, datestamp.split("-"))
            datestamp = datetime.date(*int_dates)
        if datestamp < to_date:
            to_date = datestamp

    ## CHECK INVOICES RELATED TO CREDITNOTES SENT TO CUSTOMERS
    # Get minimum datestamp for the invoices associated with any creditnotes
    # sent with a date after the start of the current VAT period
    sql_string = """SELECT MIN(DATESTAMP) FROM TRANSACTIONS, INVOICES
                     WHERE TRANSACTIONS.TRANSACTION_ID = INVOICES.TRANSACTION_ID
                     AND INVOICE_ID IN
                      (SELECT INVOICE_ID FROM CREDITNOTES, TRANSACTIONS
                       WHERE DATESTAMP >= ?
                       AND CREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID)"""

    datestamp = select(sql_string, (vat_period_start,))
    if datestamp:
        # It looks as if the MIN function returns a unicode value of the datestamp, rather
        # than a datetime.date object
        if not isinstance(datestamp, datetime.date):
            int_dates = map(int, datestamp.split("-"))
            datestamp = datetime.date(*int_dates)
        if datestamp < to_date:
            to_date = datestamp

    ## CHECK PURCHASEORDERS SENT TO SUPPLIERS RELATED TO PAYMENTS MADE
    # Get minimum datestamp for the purchaseorders associated with any payments
    # made with a date after the start of the current VAT period
    sql_string = """SELECT MIN(DATESTAMP) FROM TRANSACTIONS, PURCHASEORDERS
                     WHERE TRANSACTIONS.TRANSACTION_ID = PURCHASEORDERS.TRANSACTION_ID
                     AND PURCHASEORDER_ID IN
                      (SELECT PURCHASEORDER_ID FROM MAKEPAYMENTS, TRANSACTIONS
                       WHERE DATESTAMP >= ?
                       AND MAKEPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID)"""

    datestamp = select(sql_string, (vat_period_start,))
    if datestamp:
        # It looks as if the MIN function returns a unicode value of the datestamp, rather
        # than a datetime.date object
        if not isinstance(datestamp, datetime.date):
            int_dates = map(int, datestamp.split("-"))
            datestamp = datetime.date(*int_dates)
        if datestamp < to_date:
            to_date = datestamp

    ## CHECK PURCHASEORDERS SENT TO SUPPLIERS RELATED TO SUPPLIER CREDITNOTES RECEIVED
    # Get minimum datestamp for the purchaseorders associated with any supplier creditnote
    # received with a date after the start of the current VAT period
    sql_string = """SELECT MIN(DATESTAMP) FROM TRANSACTIONS, PURCHASEORDERS
                     WHERE TRANSACTIONS.TRANSACTION_ID = PURCHASEORDERS.TRANSACTION_ID
                     AND PURCHASEORDER_ID IN
                      (SELECT PURCHASEORDER_ID FROM SUPPLIERCREDITNOTES, TRANSACTIONS
                       WHERE DATESTAMP >= ?
                       AND SUPPLIERCREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID)"""

    datestamp = select(sql_string, (vat_period_start,))
    if datestamp:
        # It looks as if the MIN function returns a unicode value of the datestamp, rather
        # than a datetime.date object
        if not isinstance(datestamp, datetime.date):
            int_dates = map(int, datestamp.split("-"))
            datestamp = datetime.date(*int_dates)
        if datestamp < to_date:
            to_date = datestamp

    ## CHECK PURCHASEORDERS SENT TO SUPPLIERS RELATED TO SUPPLIER INVOICES RECEIVED
    # Get minimum datestamp for the purchaseorders associated with any supplier invoices
    # received with a date after the start of the current VAT period
    sql_string = """SELECT MIN(DATESTAMP) FROM TRANSACTIONS, PURCHASEORDERS
                     WHERE TRANSACTIONS.TRANSACTION_ID = PURCHASEORDERS.TRANSACTION_ID
                     AND PURCHASEORDER_ID IN
                      (SELECT PURCHASEORDER_ID FROM SUPPLIERINVOICES, TRANSACTIONS
                       WHERE DATESTAMP >= ?
                       AND SUPPLIERINVOICES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID)"""

    datestamp = select(sql_string, (vat_period_start,))
    if datestamp:
        # It looks as if the MIN function returns a unicode value of the datestamp, rather
        # than a datetime.date object
        if not isinstance(datestamp, datetime.date):
            int_dates = map(int, datestamp.split("-"))
            datestamp = datetime.date(*int_dates)
        if datestamp < to_date:
            to_date = datestamp

    return to_date


# --------------------Read Contacts Information

def search_for_contacts(organization_text="", name_text=""):
    """Gets a list of all visible contacts defined by organization_text and name_text,
       returns a list of lists - to match the search contacts grid, not a list of Contact objects"""

    # Set sql wildcards
    name_text = "%" + name_text + "%"
    organization_text = "%" + organization_text + "%"
    this_company = Company()
    this_company_id = this_company.id

    conn=None
    try:
        # Make database connection
        conn, c = vs_connect()
        c.execute("""SELECT COMPANIES.COMPANY_NAME,
                            CONTACTS.NAME,
                            CONTACTS.EMAIL,
                            CONTACTS.PHONE,
                            CONTACTS.MOBILE,
                            CONTACTS.NOTES
                       FROM CONTACTS, COMPANIES
                      WHERE CONTACTS.NAME LIKE ?
                        AND CONTACTS.VISIBLE = 1
                        AND CONTACTS.SYSTEM = 1
                        AND COMPANIES.COMPANY_ID = ?
                        AND COMPANIES.COMPANY_NAME LIKE ?

                    UNION

                     SELECT CUSTOMERS.NAME,
                            CONTACTS.NAME,
                            CONTACTS.EMAIL,
                            CONTACTS.PHONE,
                            CONTACTS.MOBILE,
                            CONTACTS.NOTES
                       FROM CONTACTS, CUSTOMERS
                      WHERE CONTACTS.NAME LIKE ?
                        AND CONTACTS.VISIBLE = 1
                        AND CUSTOMERS.CUSTOMER_ID = CONTACTS.CUSTOMER_ID
                        AND CUSTOMERS.NAME LIKE ?

                    UNION

                     SELECT SUPPLIERS.NAME,
                            CONTACTS.NAME,
                            CONTACTS.EMAIL,
                            CONTACTS.PHONE,
                            CONTACTS.MOBILE,
                            CONTACTS.NOTES
                       FROM CONTACTS, SUPPLIERS
                      WHERE CONTACTS.NAME LIKE ?
                        AND CONTACTS.VISIBLE = 1
                        AND SUPPLIERS.SUPPLIER_ID = CONTACTS.SUPPLIER_ID
                        AND SUPPLIERS.NAME LIKE ?

                   ORDER BY CONTACTS.NAME""", (name_text, this_company_id, organization_text,
                                               name_text, organization_text,
                                               name_text, organization_text ))

        result = c.fetchall()
    finally:
        if conn:
            conn.close()
    # returns a list of lists - ready for displaying, not a list of contacts objects
    return result
  

# -------------------- Read Purchase Order Information

def get_orders_from_supplier_invoice_number(supplier_invoice_number):
    """Get a list of purchase orders to which a supplier invoice has been assigned
       Note - there could be multiple orders, as different suppliers may use the
       same invoice number"""
    sql_string = "SELECT PURCHASEORDER_ID FROM SUPPLIERINVOICES WHERE NUMBER = ?"
    result = list_select(sql_string, (supplier_invoice_number,))
    if not result: return []
    return [ PurchaseOrder(purchaseorder_id) for purchaseorder_id in result ]


def get_orders_from_supplier_reference(supplier_reference):
    """Get a list of purchase orders to which a supplier reference has been assigned
       Note - there could be multiple orders, as different suppliers may use the
       same reference"""
    sql_string = "SELECT PURCHASEORDER_ID FROM PURCHASEORDERS WHERE SUPPLIER_REFERENCE = ?"
    result = list_select(sql_string, (supplier_reference,))
    if not result: return []
    return [ PurchaseOrder(purchaseorder_id) for purchaseorder_id in result ]


def get_openorders():
    """Gets a list of all open purchaseorders, ordered by gridrow"""
    sql_string = "SELECT PURCHASEORDER_ID FROM PURCHASEORDERS WHERE STATUS = 0 OR STATUS = 1 ORDER BY GRIDROW"
    result = list_select(sql_string)
    if not result: return []
    all_purchaseorders = [ PurchaseOrder(purchaseorder_id) for purchaseorder_id in result ]
    return all_purchaseorders


def get_open_and_invoiced_orders():
    """Gets a list of open purchaseorders with a received invoice, ordered by gridrow"""
    sql_string = "SELECT PURCHASEORDER_ID FROM PURCHASEORDERS WHERE STATUS = 1 ORDER BY GRIDROW"
    result = list_select(sql_string)
    if not result: return []
    all_purchaseorders = [ PurchaseOrder(purchaseorder_id) for purchaseorder_id in result ]
    return all_purchaseorders


def get_earliest_openorder():
    "Get the earliest purchase order which is still open"
    list_of_openorders = get_openorders()
    if list_of_openorders:
        return list_of_openorders[0]
    return None


def count_openorders():
    """Gets a count of all open purchaseorders"""
    sql_string = "SELECT COUNT(*) FROM PURCHASEORDERS WHERE STATUS = 0 OR STATUS = 1"
    result = select(sql_string)
    return result


def get_max_order_number():
    "Gets the maximum purchaseorder number"
    result = select("SELECT MAX(ORDER_NUMBER) FROM PURCHASEORDERS")
    if not result:
        result = 0
    return result


# -------------------- Read Invoice Information

def get_invoices_from_customer_ordernumber(cus_order_number):
    """Gets a list of all invoices with the given customer order number,
       ordered by gridrow"""
    sql_string = "SELECT INVOICE_ID FROM INVOICES WHERE CUSTOMER_ORDER_NUMBER = ? ORDER BY GRIDROW"
    result = list_select(sql_string, (cus_order_number,))
    if not result: return []
    return [ Invoice(invoice_id) for invoice_id in result ]


def get_openinvoices():
    """Gets a list of all open invoices, ordered by gridrow"""
    sql_string = "SELECT INVOICE_ID FROM INVOICES WHERE OPEN = 1 ORDER BY GRIDROW"
    result = list_select(sql_string)
    if not result: return []
    all_invoices = [ Invoice(invoice_id) for invoice_id in result ]
    return all_invoices


def get_earliest_openinvoice():
    "Get the earliest invoice which is still open"
    list_of_openinvoices = get_openinvoices()
    if list_of_openinvoices:
        return list_of_openinvoices[0]
    return None


def count_openinvoices():
    """Gets a count of all open invoices"""

    sql_string = "SELECT COUNT(*) FROM INVOICES WHERE OPEN = 1"
    result = select(sql_string)

    return result


# -------------------- Read received customer payments Information

def get_payments_from_customer_reference(cus_reference):
    """Gets a list of all received payments with the given customer reference,
       ordered by datestamp"""

    sql_string = """SELECT CUSTOMERPAYMENT_ID FROM CUSTOMERPAYMENTS, TRANSACTIONS
                     WHERE CUSTOMER_REFERENCE = ?
                     AND CUSTOMERPAYMENTS.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                     ORDER BY DATESTAMP"""
    result = list_select(sql_string, (cus_reference,))

    if not result: return []
    return [ CustomerPayment(payment_id) for payment_id in result ]


# -------------------- Read supplier invoices received Information

def get_supplierinvoices_from_supplierinvoice_number(invoice_number):
    """Gets a list of all received supplier invoices with the given
       invoice number ordered by datestamp"""

    sql_string = """SELECT SUPPLIERINVOICE_ID FROM SUPPLIERINVOICES, TRANSACTIONS
                     WHERE NUMBER = ?
                     AND SUPPLIERINVOICES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                     ORDER BY DATESTAMP"""
    result = list_select(sql_string, (invoice_number,))

    if not result: return []
    return [ SupplierInvoice(supplier_invoice_id) for supplier_invoice_id in result ]

# -------------------- Read supplier credit notes received Information

def get_suppliercreditnotes_from_suppliercreditnote_number(suppliercreditnote_number):
    """Gets a list of all received supplier creditnotes with the given
       creditnote number ordered by datestamp"""

    sql_string = """SELECT SUPPLIERCREDITNOTE_ID FROM SUPPLIERCREDITNOTES, TRANSACTIONS
                     WHERE NUMBER = ?
                     AND SUPPLIERCREDITNOTES.TRANSACTION_ID = TRANSACTIONS.TRANSACTION_ID
                     ORDER BY DATESTAMP"""
    result = list_select(sql_string, (suppliercreditnote_number,))

    if not result: return []
    return [ SupplierCreditNote(suppliercreditnote_id) for suppliercreditnote_id in result ]



