####### VATStuff #######
#
# vatstuff_packages.vs_verify.accounts.py  - functions for verifying account input
#
# Version : 0.6
# Date : 20100716
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009, 2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#

"""
Provides functions for verifying user input when
creating or editing an account or folder
"""

from vatstuff_packages.vs_utilities import *


def new_folder_check(folder_name):
    """Check folder details input by the user when creating a new folder,
       return status, message
       If OK, status is True, message is None
       If not, status is False, message is a reason string"""

    # Check if an empty name has been chosen
    if not folder_name:
        return False, "A folder name must be input"

    # Check for folder name uniqueness
    if folder_name in [ folder.name for folder in Folder.get_all() ]:
        return False, "This folder name is already being used"

    return True, None


def edit_folder_check(folder, folder_name):
    """Check folder name input by the user when editing a folder
       return status, message
       If OK, status is True, message is None
       If not, status is False, message is a reason string"""

    # Check if an empty name has been chosen
    if not folder_name:
        return False, "A folder name must be input"

    # Is name being changed?, if so test for uniqueness
    if folder_name != folder.name:
        # Check for name uniqueness
        if folder_name in [ folder.name for folder in Folder.get_all() ]:
            return False, "This folder name is already being used"

    return True, None


def delete_folder_check(folder):
    """Check folder details input by the user when deleting a folder,
       return status, message
       If OK, status is True, message is None
       If not, status is False, message is a reason string"""

    # Checks if the system flag is 2
    if folder.system == 2:
        return False, "This folder holds accounts used for\nsystem calculations and cannot be deleted"

    # Checks if the system flag is 3
    if folder.system == 3:
        return False, "This folder holds customer accounts\nand cannot be deleted"

    # Checks if the system flag is 4
    if folder.system == 4:
        return False, "This folder holds supplier accounts\nand cannot be deleted"

    # Checks if the system flag is zero
    if not folder.system == 0:
        return False, "Only a folder with no accounts beneath it can be deleted"

    return True, None


def new_account_check(account_name, folder_selected, balance, datestamp):
    """Check account details input by the user, return status, message
       If OK, status is True, message is None
       If not, status is False, message is a reason string"""

    # Check a non-empty name is chosen
    if not account_name:
        return False, "An account name must be input"

    # Check a folder is selected
    if not folder_selected:
        return False, "Selected folder not recognised"

    # Check it is not a system folder
    if folder_selected.system > 1:
        return False, "You cannot create an account in a system folder"

    # Check for name uniqueness within the folder
    if folder_selected.get_account_with_name(account_name):
        return False, "This name is already used in this folder"

    if balance:
        # A balance transaction is to be implemented, check the datestamp
        if vs_cfg["DELETE_TO_DATE"]:
            if datestamp < vs_cfg["DELETE_TO_DATE"]:
                return False, "A date prior to the last database\ndeletion date cannot be accepted"

    return True, None


def edit_account_check(account, account_name, folder_selected, balance, datestamp):
    """Check account details input by the user, return status, message
       If OK, status is True, message is None
       If not, status is False, message is a reason string"""

    # Check a non-empty name is chosen
    if not account_name:
        return False, "An account name must be input"

    # Check a folder is selected
    if not folder_selected:
        return False, "Selected folder not recognised"

    # Check it is not a system folder
    if folder_selected.system > 1:
        return False, "You cannot create an account in a system folder"

    # Is name being changed?, if so test for uniqueness in the target folder
    if folder_selected != account.folder or account_name != account.name:
        # Check for name uniqueness within the folder
        if folder_selected.get_account_with_name(account_name):
            return False, "This name is already used in this folder"

    if balance:
        if account.system:
            # A balance can only be input if account.system is zero
            return False, "A balance cannot be set on this account"
        # A balance transaction is to be implemented, check the datestamp
        if vs_cfg["DELETE_TO_DATE"]:
            if datestamp < vs_cfg["DELETE_TO_DATE"]:
                return False, "A date prior to the last database\ndeletion date cannot be accepted"

    return True, None


def delete_account_check(account):
    """Check account details input by the user when deleting an account,
       return status, message
       If OK, status is True, message is None
       If not, status is False, message is a reason string"""

    # Checks this is not a system account
    if account.system > 1:
        return False, "This account is used for system calculations\nand cannot be deleted"

    # Checks if the system flag is zero, or the balance is not zero
    if account.system != 0 or account.balance != 0:
        return False, "Only an account with no transactions can be deleted"

    # Checks if the account is used in any of the move money definitions
    if account.definition_count():
        return False, """\
This account is used by one of the transaction
definitions under the 'Move Money' tab - it must be
removed from there before it can be deleted"""
    return True, None


def account_type_table_check(customer_type_name, supplier_type_name, types_list, all_accounttypes):
    """Check account type details when the table of account types is edited by the user
       return status, message
       If OK, status is True, message is None
       If not, status is False, message is a reason string"""

    if (not customer_type_name) or (not supplier_type_name):
        return False, """\
The default customer and supplier account types
should both be set to valid types"""

    # types_list are values from the types_table - a list of lists (name, description)
            
    for index, acc_type in enumerate(types_list):
        if len(acc_type[0])>50:
            return False, "Type name too long at line %s (50 characters max)" % (index+1,)
        if len(acc_type[1])>100:
            return False, "Description too long at line %s (100 characters max)" % (index+1,)

    names_list = [ acc_type[0] for acc_type in types_list ]
    if not customer_type_name in names_list:
        return False, "The default customer type must exist"
    if not supplier_type_name in names_list:
        return False, "The default supplier type must exist"

    # Are any non-blank names duplicated
    for name in names_list:
        if name and names_list.count(name) > 1:
            return False, "The type '%s' appears more than once" % name

    # check if any are to be deleted, that they are not assigned to an account
    for index, acc_type in enumerate(all_accounttypes):
        if not names_list[index]:
            # This is to be deleted, are there any accounts in this?
            if acc_type.get_accounts():
                return False, "Type at line %s cannot be deleted,\nit is assigned to an account" % (index+1,)

    return True, None
