####### VATStuff #######
#
# common.py  - common verification functions
#
# Version : 0.3
# Date : 20100122
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#


"""common verification functions"""


import decimal
import datetime

from vatstuff_packages.vs_utilities import *



D = decimal.Decimal   # D used to avoid typing


def check_customer(customer, current_customer = None):
    """Given a CustomerForm object, check attributes and return
       (False, message) on failure
       (True, None) on success
       current_customer is a Customer object that is being edited, and
       is set to None if a new customer is being created"""

    if not customer.name:
        return False, "A customer name must be supplied"

    # Get own company details
    company = Company()

    if customer.name == company.name:
        return False, "This customer name is your own company!"

    # Check this customers name is not already used
    for c in Customer.get_all():
        if c == current_customer: continue
        if customer.name == c.name:
            return False, "This customer name has already been used."

    # EC country
    if customer.ec_country:
        # VAT Country Code
        if len(customer.vat_country_code) != 2:
            return False, "The VAT country code is incorrect\nNo items have been changed."
        # VAT registration number
        if len(customer.vat_regnumber) > 20:
            return False, "The VAT Registration number is incorrect\nNo items have been changed."
        # VAT Registered status
        # If vat_registered and VAT Country Code is not GB and VAT registration number is blank, then flag an error
        if customer.vat_registered and customer.vat_country_code != vs_cfg["VAT_COUNTRY_CODE"] and not customer.vat_regnumber:
            return False, """\
The VAT Registration number
(this countries equivalent to VAT)
must be input."""
    else:
        if customer.vat_country_code or customer.vat_regnumber or customer.vat_registered:
            return False, "Non-EC company should not have VAT details set"

    # Discount
    if customer.discount < D(0) or customer.discount > D("99.9"):
        return False, "The discount value is incorrect\nNo items have been changed."

    return True, None


def check_supplier(supplier, current_supplier = None):
    """Given a SupplierForm object, check attributes and return
       (False, message) on failure
       (True, None) on success
       current_supplier is a Supplier object that is being edited, and
       is set to None if a new supplier is being created"""

    if not supplier.name:
        return False, "A supplier name must be supplied"

    # Get own company details
    company = Company()

    if supplier.name == company.name:
        return False, "This supplier name is your own company!"

    # Check this suppliers name is not already used
    for s in Supplier.get_all():
        if s == current_supplier: continue
        if supplier.name == s.name:
            return False, "This supplier name has already been used."

    # EC country
    if supplier.ec_country:
        # VAT Country Code
        if len(supplier.vat_country_code) != 2:
            return False, "The VAT country code is incorrect\nNo items have been changed."
        # VAT registration number
        if len(supplier.vat_regnumber) > 20:
            return False, "The VAT Registration number is incorrect\nNo items have been changed."
        # VAT Registered status
        # If vat_registered and VAT Country Code is not GB and VAT registration number is blank, then flag an error
        if supplier.vat_registered and supplier.vat_country_code != vs_cfg["VAT_COUNTRY_CODE"] and not supplier.vat_regnumber:
            return False, """\
The VAT Registration number
(this countries equivalent to VAT)
must be input."""
    else:
        if supplier.vat_country_code or supplier.vat_regnumber or supplier.vat_registered:
            return False, "Non-EC company should not have VAT details set"

    # Discount
    if supplier.discount < D(0) or supplier.discount > D("99.9"):
        return False, "The discount value is incorrect\nNo items have been changed."

    return True, None

