####### VATStuff #######
#
# form_objects.py  - defines objects used to pass form information
#
# Version : 0.7.0
# Date : 20100925
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#


"""Defines objects used to pass form information from the gui to the
   functions which save the object to the database.
   Also defines
   link_payment_to_invoices(payment_amount, invoices)
   which proportions a money payment amongst a list of invoices"""


import decimal
import datetime

from vatstuff_packages.vs_utilities import *

D = decimal.Decimal   # D used to avoid typing


class TransactionForm(object):
    """Holds transaction form data"""

    def __init__(self):
        """Sets transaction system, datestamp, description and
           trans_list, which is a list of the accounts and the amounts
           to debit and credit

           System flags
    
        0 - User defined transaction - using the move buttons (The default)
        1 - An opening balance transaction
        2 - A purchase order transaction
        3 - Pay a supplier invoice transaction
        4 - Supplier credit note transaction
        5 - An invoice transaction
        6 - Customer payment transaction
        7 - Create a credit note transaction
        8 - An order close transaction
        9 - Supplier invoice received transaction
       10 - A VAT return transaction
       11 - A VAT correction transaction
       12 - A 'Carry Forward' transaction after a delete old data action"""

        self.system=0
        self.datestamp = None
        self.description = ""
        self.trans_list = []

    def add(self, account, debit=D("0.00"), credit=D("0.00")):
        "Add an account, and the debit, credit value to trans_list"
        self.trans_list.append([account, debit, credit])

    def clear(self):
        "Clears the trans_list"
        self.trans_list = []

    def check(self):
        "Returns (True, None) if ok, (False, message) if not"
        if not self.datestamp:
            return False, "No date has been provided"
        if vs_cfg["DELETE_TO_DATE"]:
            if self.datestamp < vs_cfg["DELETE_TO_DATE"]:
                return False, "A date prior to the last database\ndeletion date cannot be accepted"
        self.description = self.description.strip()
        if not self.description:
            return False, "No description has been provided"
        if not self.trans_list:
            return False, "No list of accounts to debit and credit have been provided"
        debit_sum = D("0.00")
        credit_sum = D("0.00")
        for account, debit, credit in self.trans_list:
            if not account:
                return False, "An account is missing"
            if debit < D(0):
                return False, "A negative debit cannot be accepted"
            if credit < D(0):
                return False, "A negative credit cannot be accepted"
            if debit and credit:
                return False, "Either debit or credit must be zero"
            debit_sum += debit
            credit_sum += credit
        if debit_sum != credit_sum:
            return False, "The debits do not equal the credits"
        # All seems ok
        return True, None


class CustomerForm(object):
    """Holds customer form data"""

    def __init__(self):
        self.name=""
        self.ec_country = True
        self.vat_country_code = vs_cfg["VAT_COUNTRY_CODE"]
        self.vat_regnumber = ""
        self.vat_registered = True
        self.discount = D("0.0")

    def get_discount_string(self):
        return str(self.discount).replace(".", vs_cfg["CP"])

    discount_string = property(get_discount_string)

    def copy_from(self, customer):
        "Copies attributes from database customer object"
        self.name = customer.name
        self.ec_country = customer.ec_country
        self.vat_country_code = customer.vat_country_code
        self.vat_regnumber = customer.vat_regnumber
        self.vat_registered = customer.vat_registered
        self.discount = customer.discount


class SupplierForm(object):
    """Holds supplier form data"""

    def __init__(self):
        self.name=""
        self.ec_country = True
        self.vat_country_code = vs_cfg["VAT_COUNTRY_CODE"]
        self.vat_regnumber = ""
        self.vat_registered = True
        self.discount = D("0.0")

    def get_discount_string(self):
        return str(self.discount).replace(".", vs_cfg["CP"])

    discount_string = property(get_discount_string)

    def copy_from(self, supplier):
        "Copies attributes from database supplier object"
        self.name = supplier.name
        self.ec_country = supplier.ec_country
        self.vat_country_code = supplier.vat_country_code
        self.vat_regnumber = supplier.vat_regnumber
        self.vat_registered = supplier.vat_registered
        self.discount = supplier.discount


class OrderForm(object):
    """Holds order form data. This is completed when the user presses apply on the purchaseorder
       form, and is passed to the save_purchaseorder.save function - and carries all the purchaseorder
       information which is then saved to the database."""

    def __init__(self):
        self.company = Company()                # The company
        self.supplier = None                    # The supplier to whom the order is being sent
        self.print_check = False                # True if country to be show in addresses
        self.pdf_width   = vs_cfg["LAST_ORDER_PDF_SIZE"][0]
        self.pdf_height  = vs_cfg["LAST_ORDER_PDF_SIZE"][1]
        self.supplier_address = None            # The suppliers address where the order is sent
        self.sup_ref_field = ""                 # The suppliers reference text
        self.invoice_address = None             # Where invoices should be sent
        self.delivery_address = None            # where deliveries should be sent
        self.datestamp = None                   # Datestamp on this order form
        self.number = 0                         # The purchase order number (a 'long' value)
        self.contact = None                     # The contact for enquiries
        self.discount = D("0.0")                # The discount rate
        self.item_list = []
        # Each item_list element being a dictionary of
        # {"Qty":Decimal, "Description":string, "Unit Cost":Decimal, "Discount":boolean "VAT":Decimal (The % value)}
        # Qty is a Decimal value
        # Description is a string
        # Unit Cost is a Decimal value
        # VAT is a Decimal value - being the percentage
        self.vat = D("0.00")          # Total VAT
        self.cost = D("0.00")         # Total cost (net - exclusive of VAT)
        # As this is a temporary object used when a new order is being created, the following are always at default
        self.paid = D("0.00")
        self.is_open = True
        self.is_cancelled = False
        self.show_vat = True             # Show the VAT column and VAT totals on the order
        self.claim_vat = True            # True if VAT is to be claimed, False otherwise

    def get_amount(self):
        "Return a list of Qty * Unit Cost - as Decimal values"
        amount_list = []
        for item in self.item_list:
            result = item["Qty"] * item["Unit Cost"]
            amount_list.append(result.quantize(D('.01'), rounding=decimal.ROUND_DOWN))
        return amount_list


class InvoiceForm(object):
    """Holds invoice form data"""

    def __init__(self):
        self.company = Company()               # The company
        self.customer = None                   # The customer to whom the invoice is being sent
        self.print_check = False               # True if country to be shown in addresses
        self.pdf_width   = vs_cfg["LAST_INVOICE_PDF_SIZE"][0]
        self.pdf_height  = vs_cfg["LAST_INVOICE_PDF_SIZE"][1]
        self.customer_address = None           # he customers address where the invoice is sent
        self.cus_order_number = ""             # The customers order number
        self.payment_address = None            # Where payment should be sent
        self.payment_instructions = ""         # Any text payment instruction you wish to add
        self.date_of_issue = None              # Datestamp of the issue of this invoice
        self.date_of_supply = None             # Datestamp of the supply of goods
        self.date_of_taxpoint = None           # Actual date at which VAT is calculated
        self.number = 0                        # The invoice number
        self.contact = None                    # The contact for enquiries
        self.discount = D("0.0")               # The discount rate
        self.item_list = []
        # Each item_list element being a dictionary of:
        # {"Qty":Decimal, "Description":string, "Unit Cost":Decimal, "Discount":boolean "VAT":Decimal (The % value)}
        # Qty is a Decimal value
        # Description is a string
        # Unit Cost is a Decimal value
        # VAT is a Decimal value - being the percentage
        self.vat = D("0.00")          # Total VAT
        self.cost = D("0.00")         # Total cost (net - exclusive of VAT)
        self.is_open = True
        self.is_cancelled = False
        self.show_vat = True             # Show the VAT column and VAT totals on the invoice

    def get_amount(self):
        "Return a list of Qty * Unit Cost - as Decimal values"
        amount_list = []
        for item in self.item_list:
            result = item["Qty"] * item["Unit Cost"]
            amount_list.append(result.quantize(D('.01'), rounding=decimal.ROUND_DOWN))
        return amount_list


class SupplierInvoiceForm(object):
    """Holds supplier invoice form data"""

    def __init__(self):
        self.supplier = None            # The supplier who sent this invoice
        self.purchaseorder = None       # The purchase order the invoice refers to
        self.account = None             # The stock or expense account to debit
        self.datestamp = None           # Datestamp on this invoice
        self.number = None              # The suppliers invoice number,
                                        # a 'text' value since it could include letters
        self.cost = D("0.00")           # Net cost
        self.vat = D("0.00")            # VAT
        self.claim_vat = True           # True if VAT is to be claimed, False otherwise

# -------------- SupplierCreditNoteForm object used to temporarily hold suppliers credit notes

class SupplierCreditNoteForm(object):
    """Holds supplier creditnote form data"""

    def __init__(self):
        self.supplier = None            # The supplier who sent this creditnote
        self.supplierinvoice = None     # The supplier invoice the creditnote refers to
        self.purchaseorder = None       # The purchase order the creditnote refers to
        self.account = None             # The stock or expense account to credit
        self.datestamp = None           # Datestamp on this creditnote
        self.number = None              # The suppliers creditnote number
        self.cost = D("0.00")           # Net cost
        self.vat = D("0.00")            # VAT
        self.status = 0                 # 0 for normal, 1 if received with a payment
        self.claim_vat = True           # True if VAT is to be claimed, False otherwise        


# -------------- MakePaymentForm object used to temporarily hold payment data

class MakePaymentForm(object):
    """Holds payment form data - payment to be made to a supplier"""

    def __init__(self):
        self.company = Company()                # The company
        self.supplier = None                    # The supplier to be paid
        self.supplier_address = None            # The suppliers address where the payment is sent
        self.purchaseorder = None               # The purchase order the payment is for
        self.datestamp = None                   # Date of the payment
        self.account = None                     # The bank or cash account to credit
        self.amount = D("0.00")                 # Total amount
        self.status = 0                         # STATUS
                                                #   0 if this payment is a normal payment
                                                #   1 if this is a payment for a 'quick purchase' order

        self.supplierinvoice_list = []  # list of dictionaries - { "INVOICE":supplierinvoice,
                                        #                           "COST":cost,
                                        #                           "VAT":vat }
                                        # one dictionary for each supplierinvoice this payment covers

    def create_invoice_list(self, supplierinvoices):
        "creates the self.invoice_list given a list of supplierinvoices"
        # The self.amount value should be set before calling this function
        # the link_payment_to_invoices function is defined further below in this module
        status, self.supplierinvoice_list = link_payment_to_invoices(self.amount, supplierinvoices)
        # returns True on success, False on Failure
        return status



# -------------- PaymentForm object used to temporarily hold customers payment form


class PaymentForm(object):
    """Holds payment form data - payment received from a customer"""

    def __init__(self):
        self.company = Company()                # The company
        self.customer = None                    # The customer who is paying
        self.customer_address = None            # The customer address where a receipt could be sent
        self.datestamp = None                   # Datestamp on this payment
        self.customer_reference = None          # The customers reference or payment number
        self.account = None                     # The bank or cash account to debit
        self.amount = D("0.00")                 # Total amount
        self.invoice_list = []          # list of dictionaries - { "INVOICE":invoice,
                                        #                           "COST":cost,
                                        #                           "VAT":vat }
                                        # one dictionary for each invoice this payment covers

    def create_invoice_list(self, invoices):
        "creates the self.invoice_list given a list of invoices"
        # The self.amount value should be set before calling this function
        # the link_payment_to_invoices function is defined further below in this module
        status, self.invoice_list = link_payment_to_invoices(self.amount, invoices)
        # returns True on success, False on Failure
        return status


class CreditNoteForm(object):
    """Holds Credit Note form data."""

    def __init__(self):
        self.company = Company()                        # The company
        self.customer = None                            # The customer who this note is sent to
        self.print_check = False                        # True if country to be shown in addresses
        self.pdf_width   = vs_cfg["LAST_CREDITNOTE_PDF_SIZE"][0]
        self.pdf_height  = vs_cfg["LAST_CREDITNOTE_PDF_SIZE"][1]
        self.customer_address = None                    # The customers address where the note is to be sent
        self.invoice = None                             # The invoice this is with respect to
        self.datestamp = datetime.date.today()          # Datestamp on this credit note
        self.reason = None                              # Reason for the credit note
        self.number = None                              # Credit Note number
        self.contact = None                             # The contact for enquiries
        self.discount = D("0.0")                        # The discount rate
        self.item_list = []
        # Each item_list element being a dictionary of:
        # {"Qty":Decimal, "Description":string, "Unit Cost":Decimal, "Discount":boolean "VAT":Decimal (The % value)}
        # Qty is a Decimal value
        # Description is a string
        # Unit Cost is a Decimal value
        # VAT is a Decimal value - being the percentage
        self.cost = D("0.00")                           # Total cost
        self.vat = D("0.00")                            # Total VAT
        self.status = 0                                 # status value
        # STATUS - normally 0 when a creditnote is sent against an invoice
        # STATUS - 1 when a creditnote is sent with a payment
        self.account_credited = None     # Customer account when a creditnote is sent against an invoice
                                         # or a bank account when money is sent
        self.show_vat = True             # Show the VAT column and VAT totals on the credit note


    def get_amount(self):
        "Return a list of Qty * Unit Cost - as Decimal values"
        amount_list = []
        for item in self.item_list:
            result = item["Qty"] * item["Unit Cost"]
            amount_list.append(result.quantize(D('.01'), rounding=decimal.ROUND_DOWN))
        return amount_list

        
class VATReturnForm(object):
    """Holds VAT Return data."""

    def __init__(self):
        self.from_date = None
        self.to_date = None
        self.company = Company()
        self.account = None                    # The bank or cash account to credit
        self.box_one = D("0.00")
        self.box_two = D("0.00")
        self.box_three = D("0.00")
        self.box_four = D("0.00")
        self.box_five = D("0.00")
        self.box_six = D("0.00")
        self.box_seven = D("0.00")
        self.box_eight = D("0.00")
        self.box_nine = D("0.00")


####################### AddressForm object ##############################

# Holds an address object, but not stored in the database, for
# temporary editing

class AddressForm(object):

    def __init__(self,
                 address_id = -1,
                 line1 = "",
                 line2 = "",
                 city = "",
                 county = "",
                 postcode = "",
                 country = "",
                 reg_office = 0,
                 delivery_point = 0,
                 invoice_point = 0,
                 order_point = 0,
                 payment_point = 0):
        "Create a temporary AddressForm object - not held in the database"
        if country == "":
            country = vs_cfg["COUNTRY"]

        self.id             = address_id
        self.line1          = line1
        self.line2          = line2
        self.city           = city
        self.county         = county
        self.postcode       = postcode
        self.country        = country
        self.reg_office     = reg_office
        self.delivery_point = delivery_point
        self.invoice_point  = invoice_point
        self.order_point    = order_point
        self.payment_point  = payment_point

    def get_lines(self):
        return (self.line1, self.line2, self.city, self.county, self.postcode, self.country)

    def set_lines(self, lines_list):
        "Takes a list containing address lines and sets them into attributes"
        self.line1    = lines_list[0]
        self.line2    = lines_list[1]
        self.city     = lines_list[2]
        self.county   = lines_list[3]
        self.postcode = lines_list[4]
        self.country  = lines_list[5]

    def get_flags(self):
        return { 'reg_office'     : self.reg_office,
                 'delivery_point' : self.delivery_point,
                 'invoice_point'  : self.invoice_point,
                 'order_point'    : self.order_point,
                 'payment_point'  : self.payment_point  }

    def set_flags(self, flags_dict):
        "Takes a dictionary containing flags and sets them into attributes"
        self.reg_office     = flags_dict['reg_office']
        self.delivery_point = flags_dict['delivery_point']
        self.invoice_point  = flags_dict['invoice_point']
        self.order_point    = flags_dict['order_point']
        self.payment_point  = flags_dict['payment_point']

    lines = property(get_lines, set_lines)
    flags = property(get_flags, set_flags)



# Function to make a temporary AddressForm from an Address object

def make_AddressForm(address=None):
    "Returns an AddressForm, given an Address"

    if address == None: return AddressForm()

    # Create new blank AddressForm, with same id as the database Address object
    temp_address = AddressForm(address.id)
    # Set its lines and flags equal to the database Address object
    temp_address.lines = address.lines
    temp_address.flags = address.flags

    return temp_address


# ----------------- ReportForm, holds a report

class ReportForm(object):
    """Holds a report

    a report consists of a title, a list of header lines, a table
    and a list of footer lines, these can be rendered as html text"""

    def __init__(self, title = None):
        "Set the report title"
        self.title = title

        # header and footer are lists of text lines
        self.header = []
        self.footer = []

        # tables are described by a list of list,
        # each list being a row, consisting of cell tuples
        # each cell tuple containing two strings 
        # The first string is formed from "XXXN",
        # and the second string is the text to be placed in the cell

        # If the c

        # N is the collspan number, as used in a table cell tag attribute
        # such as 'colspan="4"' - in which case N is 4

        # so normally N is '1'

        # XXX is a formatting string :
        # HBL  - Header Bold Left Justified (R for Right instead of L
        # HNL  - Header Normal Left Justified  etc
        # DNL  - Data Cell Normal Left Justified  etc
        # XXC  - Currency cell, to be shown as right justified currency

        self.table = []
        # Totals is a True/False flag - True if the last line of the table is a Totals line
        self.totals = False
        
    def get_table_rows(self):
        return len(self.table)

    def get_table_cols(self):
        if not self.table:
            return 0
        number_of_cols = 0
        for item in self.table[0]:
            number_of_cols += int(item[0][3])
        return number_of_cols

    table_rows = property(get_table_rows)
    table_cols = property(get_table_cols)

    def add_row_to_table(self, *row):
        "add a row to the table list"
        if self.totals:
            # raise error, cannot add a row after totals have been calculated
            raise VS_ERROR, "Cannot add a row after totals have been calculated"
        if not row:
            raise VS_ERROR, "Row content missing"
        if not self.table:
            # This is the first row, no need to check previous rows - just add it
            self.table.append(row)
            return
        # previous rows exist, check additional row has same number of columns
        cols_in_row = 0
        for item in row:
            cols_in_row += int(item[0][3])
        if cols_in_row != self.get_table_cols():
            raise VS_ERROR, "Number of columns in row does not match the table"
        self.table.append(row)

    def create_totals_row(self):
        """Creates a totals row by adding up the currency columns and adds it to the table
           must only be called when all the table rows have been appended"""
        if len(self.table)<3:
            # If only one or two rows, ie, heading line and one line of data
            # don't bother creating a totals
            return
        # Assume first line of table is a header line
        # Count number of currency items on the second line
        currency_items = 0
        for item in self.table[1]:
             # item[0] is the format string, item[1] is the contents
            if item[0][2] == 'C':
                currency_items +=1
        if not currency_items:
            return
        # Create a list of sums
        sum_list = [D("0.00")] * currency_items
        # Add up the table currency columns
        for row in self.table:
            sum_list_index = 0
            for cell in row:
                if cell[0][2] == 'C':
                    sum_list[sum_list_index] += D(cell[1])
                    sum_list_index += 1

        # sum_list should now hold sum of columns, create the final row
        row = []
        sum_list_index = 0
        for item in self.table[1]:
            if item[0][2] == 'C':
                row.append( [ "DBC%s" % item[0][3], str(sum_list[sum_list_index]) ] )
                sum_list_index += 1
            else:
                row.append( [ item[0], "" ] )
        self.table.append(row)
        self.totals = True


    def add_to_header(self, line):
        "add a line to the header list"
        self.header.append(line)

    def add_to_footer(self, line):
        "add a line to the footer list"
        self.footer.append(line)

    def render_to_html(self):
        "Returns the report as a html report"
        report_html = u"""\
<html>
<body>
<h3>%s</h3>
<p>""" % self.title

        for line in self.header:
           report_html += "%s<br />" % line
        report_html += "</p>"

        if self.table:
            report_html += '<center><table cellspacing="10">'
            for row in self.table:
                report_html += "<tr>"
                for cell in row:
                    if cell[0][0] == 'H':
                        # This is a header cell
                        report_html += '<th'
                    else:
                        # This is a data cell
                        report_html += '<td'
                    if cell[0][2] == 'R' or cell[0][2] == 'C':
                        # right justified
                        report_html += ' align="right"'
                    if cell[0][2] == 'L':
                        # left justified
                        report_html += ' align="left"'
                    if cell[0][3] != '1':
                        # has a colspan value
                        report_html += ' colspan="%s"' % cell[0][3]
                    report_html += '>'
                    if cell[0][1] == 'B':
                        # put in a bold tag
                        report_html += '<b>'
                    # Insert the cell contents, inserting breaks where
                    # the contents have a new line
                    if cell[0][2] == 'C':
                        # Currency value
                        contents = to_currency_string(cell[1].replace(".", vs_cfg["CP"]))
                    else:
                        contents = unicode(cell[1]).replace('\n', '<br />\n')
                    report_html += contents
                    if cell[0][1] == 'B':
                        report_html += '</b>'
                    if cell[0][0] == 'H':
                        report_html += '</th>'
                    else:
                        report_html += '</td>'
                report_html += "</tr>"
            report_html += "</center></table>"

        report_html += "<p>"
        for line in self.footer:
           report_html += "%s<br />" % line
        report_html += """\
</p>
</body>
</html>"""

        return report_html


    def render_to_csv(self):
        """Yields each row of the table as a list of items, ready for the
           csv.writer writerow method"""
        if not self.table:
            return

        last_key = len(self.table) - 1

        for key,row in enumerate(self.table):
            if self.totals:
                # miss out the last totals line
                if key == last_key: return
                
            csv_row = []
            for cell in row:
                # Insert the cell contents
                contents = unicode(cell[1])
                if cell[0][2] == 'C':
                    # Currency value
                    contents = contents.replace(".", vs_cfg["CP"])
                contents = contents.encode('utf-8')
                csv_row.append(contents)
                if cell[0][3] != '1':
                    # This text spans several columns
                    for i in range(1, int(cell[0][3])):
                        contents = "".encode('utf-8')
                        csv_row.append(contents)
            yield csv_row



# function used by PaymentForm and MakePaymentForm objects

def link_payment_to_invoices(payment_amount, invoices):
    """Proportions the payment_amount into vat and net costs applied to each invoice

       In other words - a payment amount is given, which is to pay one or more invoices - so
       how is this payment amount split across the invoices in net cost and vat per invoice,
       given that it may be a part payment, and also the invoices may have already been part
       paid for or may have credit notes against them.

       payment_amount is a Decimal value.

       invoices is a list of either Invoice or SupplierInvoice objects.

       This function returns status, invoice_payments
       status is True if successful (False if the payment submitted is zero, less than zero 
       or greater than that owed on the invoices)
       invoice_payments is a list of dictionaries, each dictionary being of the form:
       { "INVOICE":invoiceobject, "COST":cost, "VAT":vat }
       one dictionary for each invoiceobject this payment covers - the 'invoiceobject' being
       either an Invoice or SupplierInvoice"""

    invoice_payments = []

    if payment_amount <= D("0.00"):
        return False, invoice_payments

    if not invoices:
        return False, invoice_payments

    # sort invoices into date_of_taxpoint order
    invoices.sort(key = lambda x: x.date_of_taxpoint)

    for invoice in invoices:
        # if the invoice is closed, further payment cannot be added
        if not invoice.is_open:
            continue

        invoice_dict = {"INVOICE":invoice, "COST":D("0.00"), "VAT":D("0.00")}

        # find  unpaid values on the invoice, start with unpaid values as the full value
        # and then subtract any creditnotes and previous payments made
        unpaid_cost  = invoice.cost
        unpaid_vat   = invoice.vat

        # must get any previous part payments or creditnotes that reduce these values

        # Create lists from the invoice, with cost and vat values
        creditnote_list = [[creditnote.cost, creditnote.vat]
                                   for creditnote in invoice.get_creditnotes() ]

        # for any creditnotes received, subtract the credit note value from the invoice
        if creditnote_list:
            for note in creditnote_list:
                unpaid_cost -= note[0]
                unpaid_vat  -= note[1]


        payments_list = invoice.get_payments_with_costvat()
        # Gets a list of all payments previosly made against the invoice, as a list of dictionaries
        # each dictionary being of the format {"PAYMENT":paymentobject, "COST":cost, "VAT":vat}"""

        # get remaining, unpaid values by subtracting the payments
        if payments_list:
            for pay in payments_list:
                unpaid_cost -= pay["COST"]
                unpaid_vat -= pay["VAT"]

        unpaid_total = unpaid_cost + unpaid_vat

        # if the payment amount covers this unpaid total, then the current payment cost
        # and vat are the cost and vat for this amount
        if payment_amount == unpaid_total:
            invoice_dict["COST"] = unpaid_cost
            invoice_dict["VAT"] =  unpaid_vat
            invoice_payments.append(invoice_dict)
            # payment_amount is fully used up
            payment_amount = D("0.00")
            # Don't continue to the next invoice, as the payment is fully assigned
            break

        # if the remaining payment amount is less than this unpaid total
        # pay as much of invoice as possible
        if payment_amount < unpaid_total:
            cost_proportion = unpaid_cost / unpaid_total
            invoice_dict["COST"] = (payment_amount * cost_proportion).quantize(D('.01'),
                                                                      rounding=decimal.ROUND_DOWN)
            invoice_dict["VAT"] = payment_amount - invoice_dict["COST"]
            invoice_payments.append(invoice_dict)
            # payment_amount is fully used up
            payment_amount = D("0.00")
            # Don't continue to the next invoice, as the payment is fully assigned
            break

        # The remaining payment is greater than this unpaid total, then pay off this
        # invoice and continue to the next invoice
        invoice_dict["COST"] = unpaid_cost
        invoice_dict["VAT"] =  unpaid_vat
        invoice_payments.append(invoice_dict)
        payment_amount -= (unpaid_cost + unpaid_vat)

    # And after all the payment_amount is assigned to the invoices, if there is any payment
    # amount left over - then this is an error
    if payment_amount:
        return False, []

    return True, invoice_payments


