####### VATStuff #######
#
# vatstuff_packages.vs_verify.purchases.py  - functions for verifying input
#
# Version : 0.6
# Date : 20100716
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#

"""
Provides functions for verifying user input under the purchases tab
"""

from vatstuff_packages.vs_utilities import *


def new_purchaseorder_check(order):
    """order is an instance of form_objects.OrderForm object, this
       function checks attributes are ok, input via vatstuff_packages.vs_gui.purchases.make_order
       returns (True, None) if ok, or (False, "warning message") if not"""

    # check date
    if vs_cfg["DELETE_TO_DATE"]:
        if order.datestamp < vs_cfg["DELETE_TO_DATE"]:
            return False, """\
A date prior to the last database
deletion date cannot be accepted"""

    if not order.item_list:
        return False, "No items have been specified"

    last_order_number = get_config_item("LAST_ORDER_NUMBER")
    if not last_order_number:
        last_order_number = 0
    else:
        last_order_number = int(last_order_number)

    if order.number <= last_order_number :
        return False, "The order number is not valid"

    if order.discount < D(0) or order.discount > D("99.9"):
        return False,  "The discount value is incorrect."

    if order.vat < D("0.00"):
        return False, "The total VAT value is not valid"

    if order.cost <= D(0):
        return False, "The total cost value is not valid"

    if (not order.show_vat) and order.vat > D(0):
        # Something wrong, if show_vat is false, the vat must be zero
        return False, "This supplier cannot charge VAT"

    if (not order.show_vat) and (not order.claim_vat):
        # Something wrong, if show_vat is false, then claim_vat must be true
        return False, "The claim VAT field is incorrectly unset"

    if (not order.claim_vat):
        # The request is not to claim vat
        vat_scheme_method = VATScheme().method
        if vat_scheme_method != "STANDARD" and vat_scheme_method != "CASH":
            # Something wrong, claim_vat should always be true, except when set by the
            # user to False - and this should only happen with STANDARD and CASH schemes
            return False, "The claim VAT field is incorrectly unset"

    return True, None


def received_invoice_check(invoice):
    """invoice is an instance of form_objects.SupplierInvoiceForm() object.
       This function checks attributes are ok,
       input via vatstuff_packages.vs_gui.purchases.supplier_invoice
       returns (True, None) if ok, or (False, "warning message") if not"""

    if not invoice.account:
        return False, "The stock account cannot be found"

    order = invoice.purchaseorder
    if not order.is_open:
        return False, "The purchase order has been closed"

    # The invoice date cannot be earlier than the order date
    if invoice.datestamp < order.datestamp:
        return False, """\
The invoice date cannot be 
before the order date."""

    # This invoice date cannot be earlier than the current VAT period start date
    vat_scheme = VATScheme()
    vat_period_start_date = vat_scheme.start_date
    if not vat_period_start_date:
        return False, "The VAT Scheme has not yet been set"

    if invoice.datestamp < vat_period_start_date:
        return False, """\
The tax point date is in a VAT period for
which you have already submitted a VAT return!"""

    if vs_cfg["DELETE_TO_DATE"]:
        if invoice.datestamp < vs_cfg["DELETE_TO_DATE"]:
            return False, "A date prior to the last database deletion date cannot be accepted"

    # The suppliers invoice number must not be blank
    # and must be unique amongst those invoices already received for this purchase order
    if not invoice.number:
        return False, "A suppliers invoice number must be input"

    # compare this with other invoices on this order
    if invoice.number in [inv.number for inv in order.get_invoices()]:
        return False, "This suppliers invoice number already exists on this order"

    if invoice.cost < D(0):
        return False, "The net cost is not valid"

    if invoice.vat < D(0):
        return False, "The VAT is not valid"

    if (invoice.cost+invoice.vat) == D(0):
        return False, "The total cost is not valid"

    if invoice.vat != D(0):
        if not invoice.supplier.pay_vat():
            # Returns True if this supplier should charge you VAT, False otherwise
            return False, "This supplier should not charge you VAT"

    return True, None


def make_payment_check(payment, invoices):
    """payment is an instance of form_objects.MakePaymentForm() object, invoices is a list of
       invoices this payment covers. This function checks attributes are ok,
       input via vatstuff_packages.vs_gui.purchases.make_payment
       returns (True, None) if ok, or (False, "warning message") if not"""

    if not payment.account:
        return False, "A cash or bank account is\nneeded to record the payment"

    if not payment.supplier.account:
        return False, "This suppliers account cannot be found"

    # All the invoices, and hence the payment must refer to a single order
    for invoice in invoices:
        if payment.purchaseorder != invoice.purchaseorder:
            return False, "All the selected invoices must be for the same order"

    # The payment cannot be made before the order
    if payment.datestamp < payment.purchaseorder.datestamp:
        return False, """\
The payment cannot be made before
the purchase order is issued!"""

    # The payment cannot be made before any of the supplier invoices received
    for invoice in invoices:
        if payment.datestamp < invoice.datestamp:
            return False, "Cannot create a payment before\nthe date of Invoice %s" % invoice.number

    # This payment date cannot be earlier than the current VAT period start date
    vat_scheme = VATScheme()
    vat_period_start_date = vat_scheme.start_date
    if not vat_period_start_date:
        return False, "The VAT Scheme has not yet been set"

    if payment.datestamp < vat_period_start_date:
        return False, """\
The payment date is in a VAT period for
which you have already submitted a VAT return!"""

    if payment.amount <= D(0):
        return False, "The amount to be paid is not valid"

    # the MakePaymentForm() has a method which assigns the amount to the invoices cost and vat
    # fields, returns True, if successful, False on failure

    if not payment.create_invoice_list(invoices):
        return False, "The amount to be paid does not match the invoices selected"

    return True, None


def received_creditnote_against_invoice_check(creditnote):
    """creditnote is an instance of form_objects.SupplierCreditNoteForm() object.
       This function checks attributes are ok,
       input via vatstuff_packages.vs_gui.purchases.supplier_creditnote
       returns (True, None) if ok, or (False, "warning message") if not"""

    if creditnote.status != 0:
        # status should be 0 when a creditnote is sent against an invoice
        return False, "The creditnote status is incorrect"

    if not creditnote.account:
        return False, "The stock/expense account cannot be found"

    order = creditnote.purchaseorder
    if not order.is_open:
        return False, "The purchase order has been closed"

    # If the datestamp is in a previous tax period, for which a vat return has already been
    # submitted, then flag as error
    vat_scheme = VATScheme()
    vat_period_start_date = vat_scheme.start_date
    if not vat_period_start_date:
        return False, "The VAT Scheme has not yet been set"

    if creditnote.datestamp < vat_period_start_date:
        return False, """\
The credit note date is in a VAT period for
which you have already submitted a VAT return!"""

    if vs_cfg["DELETE_TO_DATE"]:
        if creditnote.datestamp < vs_cfg["DELETE_TO_DATE"]:
            return False, "A date prior to the last database deletion date cannot be accepted"

    # The suppliers creditnote number must not be blank
    # and must be unique amongst those creditnotes already received for this purchase order
    if not creditnote.number:
        return False, "A suppliers credit note number must be input"

    # compare this with other creditnotes on this order
    if creditnote.number in [supcreditnote.number for supcreditnote in order.get_creditnotes()]:
        return False, "This suppliers credit note number already exists on this order"

    if creditnote.cost < D(0):
        return False, "The net cost is not valid"

    if creditnote.cost > creditnote.supplierinvoice.cost:
        return False, "The net cost is greater than that invoiced"

    if creditnote.vat < D(0):
        return False, "The VAT is not valid"

    if creditnote.vat > creditnote.supplierinvoice.vat:
        return False, "The VAT is greater than that invoiced"

    # max amount on credit note is total invoiced cost and vat, but minus that already paid
    max_amount = order.invoice_cost + order.invoice_vat - order.paid

    if creditnote.cost + creditnote.vat > max_amount:
        return False, """\
The amount on the credit note is greater than that
which remains to be paid on the received invoices."""

    # A credit note of zero value cannot be accepted
    if creditnote.cost == D(0) and creditnote.vat == D(0):
        return False, "A credit note of zero value cannot be accepted."

    return True, None


def received_creditnote_with_payment_check(creditnote):
    """creditnote is an instance of form_objects.SupplierCreditNoteForm() object.
       This function checks attributes are ok,
       input via vatstuff_packages.vs_gui.purchases.supplier_creditnote
       returns (True, None) if ok, or (False, "warning message") if not"""

    if creditnote.status != 1:
        # status should be 1 when a creditnote is sent with a payment
        return False, "The creditnote status is incorrect"

    if not creditnote.account:
        return False, "The stock/expense account cannot be found"

    # If the datestamp is in a previous tax period, for which a vat return has already been
    # submitted, then flag as error
    vat_scheme = VATScheme()
    vat_period_start_date = vat_scheme.start_date
    if not vat_period_start_date:
        return False, "The VAT Scheme has not yet been set"

    if creditnote.datestamp < vat_period_start_date:
        return False, """\
The credit note date is in a VAT period for
which you have already submitted a VAT return!"""

    if vs_cfg["DELETE_TO_DATE"]:
        if creditnote.datestamp < vs_cfg["DELETE_TO_DATE"]:
            return False, "A date prior to the last database deletion date cannot be accepted"

    # The suppliers creditnote number must not be blank
    if not creditnote.number:
        return False, "A suppliers credit note number must be input"

    if creditnote.cost < D(0):
        return False, "The net cost is not valid"

    if creditnote.vat < D(0):
        return False, "The VAT is not valid"

    # A credit note of zero value cannot be accepted
    if creditnote.cost == D(0) and creditnote.vat == D(0):
        return False, "A credit note of zero value cannot be accepted."

    return True, None
