####### VATStuff #######
#
# vatstuff_packages.vs_verify.sales.py  - functions for verifying input
#
# Version : 0.6
# Date : 20100716
#
# Author : Bernard Czenkusz
# Email  : bernie@skipole.co.uk
#
#
# Copyright (c) 2009,2010 Bernard Czenkusz
#
# This file is part of VATStuff.
#
#    VATStuff is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    VATStuff is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with VATStuff.  If not, see <http://www.gnu.org/licenses/>.
#

"""
Provides functions for verifying user input under the sales tab
"""

from vatstuff_packages.vs_utilities import *


def new_invoice_check(invoice):
    """invoice is an instance of form_objects.InvoiceForm object, this
       function checks attributes are ok, input via vatstuff_packages.vs_gui.sales.make_invoice
       returns (True, None) if ok, or (False, "warning message") if not"""

    if vs_cfg["DELETE_TO_DATE"]:
        if invoice.date_of_issue < vs_cfg["DELETE_TO_DATE"]:
            return False, """\
An invoice date prior to the last database
deletion date cannot be accepted"""

    if vs_cfg["DELETE_TO_DATE"]:
        if invoice.date_of_supply < vs_cfg["DELETE_TO_DATE"]:
            return False, """\
A supply date prior to the last database
deletion date cannot be accepted"""

    # If the taxpoint is in a previous tax period, for which a vat return has already been
    # submitted, then flag as error
    vat_scheme = VATScheme()
    vat_period_start_date = vat_scheme.start_date
    if not vat_period_start_date:
        return False, "The VAT Scheme has not yet been set"
    if invoice.date_of_taxpoint < vat_period_start_date:
        return False, """\
The tax point date is in a VAT period for
which you have already submitted a VAT return!"""

    if not invoice.item_list:
        return False, "No items have been specified"

    last_invoice_number = get_config_item(u"LAST_INVOICE_NUMBER")
    if not last_invoice_number:
        last_invoice_number = 0
    else:
        last_invoice_number = int(last_invoice_number)

    if invoice.number <= last_invoice_number :
        return False, "The invoice number is not valid"

    if invoice.discount < D(0) or invoice.discount > D("99.9"):
        return False,  "The discount value is incorrect."

    if invoice.vat < D(0):
        return False, "The total VAT value is not valid"

    if invoice.vat != D(0):
        if not invoice.customer.charge_vat():
            # Returns True if you can charge this customer VAT, False otherwise
            return False, "You cannot charge this customer VAT"

    if invoice.cost <= D(0):
        return False, "The total cost value is not valid"

    if not invoice.customer.account:
        return False, "This customers account cannot be found"

    return True, None


def new_creditnote_check(creditnote):
    """creditnote is an instance of form_objects.CreditNoteForm object, this
       function checks attributes are ok, input via vatstuff_packages.vs_gui.sales.make_creditnote
       returns (True, None) if ok, or (False, "warning message") if not"""

    if vs_cfg["DELETE_TO_DATE"]:
        if creditnote.datestamp < vs_cfg["DELETE_TO_DATE"]:
            return False, """\
A creditnote date prior to the last database
deletion date cannot be accepted"""

    # If the datestamp is in a previous tax period, for which a vat return has already been
    # submitted, then flag as error
    vat_scheme = VATScheme()
    vat_period_start_date = vat_scheme.start_date
    if not vat_period_start_date:
        return False, "The VAT Scheme has not yet been set"
    if creditnote.datestamp < vat_period_start_date:
        return False, """\
The credit note date is in a VAT period for
which you have already submitted a VAT return!"""

    if not creditnote.item_list:
        return False, "No items have been specified"

    # Credit Note Number
    last_creditnote_number = get_config_item("LAST_CREDITNOTE_NUMBER")
    if not last_creditnote_number:
        last_creditnote_number = 0
    else:
        last_creditnote_number = int(last_creditnote_number)

    if creditnote.number <= last_creditnote_number:
        return False, "The credit note number is not valid"

    if not creditnote.account_credited:
        return False, "The account to be credited cannot be found"
    
    if creditnote.discount < D(0) or creditnote.discount > D("99.9"):
        return False, "The discount value is incorrect."

    if creditnote.vat < D(0):
        return False, "The total VAT value is not valid"

    if creditnote.cost < D(0):
        return False, "The total cost value is not valid"

    if creditnote.cost == D(0) and creditnote.vat == D(0):
        return False, "No amount to implement!"

    if creditnote.status == 0:
        # status is 0 when a creditnote is sent against an invoice
        total_on_note = creditnote.cost+creditnote.vat
        if total_on_note > creditnote.invoice.outstanding:
            return False, "Total is greater than the sum outstanding on the invoice!"
        if creditnote.datestamp < creditnote.invoice.date_of_issue:
            return False, "The credit note cannot be issued before the invoice!"

    return True, None


def received_payment_check(payment, invoices):
    """payment is an instance of form_objects.PaymentForm() object, invoices is a list of
       invoices this payment covers. This function checks attributes are ok,
       input via vatstuff_packages.vs_gui.sales.receive_payment
       returns (True, None) if ok, or (False, "warning message") if not"""

    if not payment.account:
        return False, "A cash or bank account is\nneeded to record the payment"

    if not payment.customer.account:
        return False, "This customers account cannot be found"

    if not invoices:
        return False, "At least one invoice must be selected"

    for invoice in invoices:
        if payment.customer != Customer.from_number(invoice.customer.number):
            return False, "All the selected invoices must be to the same customer"

    # Cannot accept a payment before the invoice tax point
    for invoice in invoices:
        if payment.datestamp < invoice.date_of_taxpoint:
            return False, "Cannot accept a payment before\nthe taxpoint date of Invoice %s" % invoice.number

    # If the datestamp is in a previous tax period, for which a vat return has already been
    # submitted, then flag as error
    vat_scheme = VATScheme()
    vat_period_start_date = vat_scheme.start_date
    if not vat_period_start_date:
        return False, "The VAT Scheme has not yet been set"
    if payment.datestamp < vat_period_start_date:
        return False, """\
The payment date is in a VAT period for
which you have already submitted a VAT return!"""


    if not payment.customer_reference:
        return False, """\
You must insert some text in the customer reference
field, to help should you or the customer need to
reference the payment in future"""

    if payment.amount <= D(0):
        return False, "The amount received is not valid"

    # the PaymentForm() has a method which assigns the amount to the invoices cost and vat
    # fields, returns True, if successful, False on failure

    if not payment.create_invoice_list(invoices):
        return False, "The amount on the payment does not match the invoices selected"

    return True, None
